
//EventGenerator.C: The implementaton file for 
//printerComponent. 
//--Madhu

#include "EventGenerator.h"
#include <iostream>
#include "../../../../framework/ccacore/DefaultPortInfo.h"
#include "../../../../framework/idl/eventPort/UsesEventPort.h"

// the cca specs require the component
// to have an empty constructor
EventGenerator::EventGenerator()  {
}



// The setServices() function is called by the framework
// after the component has been instantiated. The components
// behavior during the setServices invocation is very prescribed.
// The component    saves the passed reference to the  services
// object as a private instance variable.  Prior to returning from 
// the setServices() call, the component must instantiate and add
// all its initial Provides ports and register all its initial uses
// ports  using this instance variable. Components may dynamically
// add new ports later, but this will cause a newPortEvent to be
// generated by the framework. Components cannot delete ports. 
// The way the framwork notifies a component to shut down is to
// call the setServices() with a null value. When such an invocation
// of setServices completes, it is safe for the Framework to garbage
// collect the component.
// Note that the component itself implements the provides
// port and hence it passes "this"
void EventGenerator::setServices(Services *cc){
  core = cc;
  core->registerUsesPort(new DefaultPortInfo("publish", "EventPort_idl")); 
  core->addProvidesPort(this, new DefaultPortInfo("listen", "EventPort_idl"));
  if (cc == NULL) {
    if (core != NULL) {
      delete core;
    }
  }
}


// component implements the provides port interface
// returns an int as this will be invoked as a remote
// method call by HPCxx. Do not change this return void.
int EventGenerator::notify(Event *ev) {
  // I changed this line temporarily because of a compile error. -kec
  std::cout << "EventGenerator:: An event has been received " << *ev << std::endl;
  cout.flush();
  return 0;
}

void EventGenerator::execute() {
  std::cout << "EventGenerator:: execute called ! " << std::endl << std::endl;
  std::cout.flush();
  UsesEventPort *output;

  output = dynamic_cast<UsesEventPort *>(core->getPort("publish")); 
  for (int i=0; i<9; i++) {
    Event *ev = new Event("Ken&Madhu", "debugger", "MasterKey", i, "DetailsArePrivate");
    int result = output->notify(ev);
  }
  core->releasePort("publish");
  std::cout << "EventGenerator:: execute done ! " << std::endl << std::endl;
  std::cout.flush();
}
