// EventGeneratorServer.C: This is the implementation
// class provided by the component developer.
// --Madhu

#include <iostream.h>
#include "../../../../framework/ccacore/Port.h"
#include "../../../../framework/ccacore/CCATServicesProxy.h"
#include "EventGenerator.h"
#include "EventGeneratorHome.h"
#include "../../../../framework/idl/eventPort/EventPort_idl.h"
#include "../../../../framework/idl/eventPort/ProvidesEventPortSkel.h"
#include "../../../../framework/idl/eventPort/UsesEventPortSkel.h"
#include "../../../../framework/ccacore/RegisterType.h"
#include "../../../../framework/ccacore/FRAMEWORK_REGISTRATION_ID.h"
#include <hpcxx.h>

HPCxx_Mutex myLock;

EventGenerator *eventGen = NULL;
CCATServicesSkel *eventGenCore = NULL;

// Flag used to make sure that the core is initialized 
// before the function  returnComponentID tries to access it.
// We cannot use sync variables here, as hpcxx_initAgent
// may not have gone  through (i.e. Nexus may not have been
// initialized) when this function will get called.
static bool coreDone = false;

// This global method is invoked by the creation
// service (gram) to get a handle to the component.
// It is okay to busy wait (and hence waste cpu cycles)
// as the component should not be allowed to attempt to 
// do anything useful before coreDone gets set to true
CCATServicesProxy returnComponentID(){
  while (!coreDone) ;
  ((CCATServicesProxy *)(eventGenCore->getComponentID()))->setName("EVENT_GENERATOR_COMP_ID");
  return *((CCATServicesProxy *)(eventGenCore->getComponentID()));
}

int main(int argc, char **argv) {
  hpcxx_initAgent(argc, argv);

  // returnComponentID(...) is specific to 
  // a user component. As a result it needs to
  // be registered here
  hpcxx_register(returnComponentID, RETURN_COMPONENT_ID);
  cout.flush();

  // register all the remote methods
  // that will be remotely invoked
  // once the component is initialized
  // Needs to be done only once per 
  // instantiation, hence it is a static
  // method call
  CCATServicesSkel::registerAll();

  // Avoid race condtions
  // Let only one thread be able to 
  // setServices(...) on the component
  myLock.lock();

  eventGen = EventGeneratorHome::instantiate();
  
  // Instantiate a CCAT specific implementation
  // of the services object
  eventGenCore = new CCATServicesSkel();

  // This call is required as part of Component
  // Interface. The component now has a handle
  // to the framework.
  eventGen->setServices(eventGenCore);

  myLock.unlock();

  // It is now okay to return the componentID
  // to the creationService
  coreDone = true;
  
  cout << "EventGeneratorServer:Up and waiting for requests\n" << endl;
  cout.flush();
  
  // The component needs to get beyond this 
  // blocking call to call the component's execute()
  // method. This call will unblock when 
  // he composition engine calls run() on the
  // the componentID of this component.
  eventGenCore->startRunning();
  cout << "PrinterServer: Main:: Received signal to execute \n" << endl;
  cout.flush();
  
  // Must have recieved a run() signal on 
  // the componentID. 
  eventGen->execute();
  
  // Will fix this later
  // This needs to be used, but am not
  // sure if this is the righ place for it
  // FIXME FIXME 
  eventGenCore->waitForKill();
  cout << "Component has been killed! \n ";
  cout << "EventGeneratorServer exiting ... \n";
  cout.flush();

  // This sleep() call is useful if the
  // above kill statement is used. Just ensures
  // that the component dies before HPCxx 
  // starts its cleanup
  sleep(5);
  hpcxx_exit();
}




