
//PrinterComponent.C: The implementaton file for 
//printerComponent. 
//--Madhu

#include "PrinterComponent.h"
#include <iostream>
#include "../../../../framework/ccacore/DefaultPortInfo.h"
#include "../../../../framework/idl/string/UsesString_idlPort.h"
#include "../../../../framework/idl/string/CCAT_string_idl.h"
#include "../../../../framework/creation/ComponentFactory.h"

// the cca specs require the component
// to have an empty constructor
PrinterComponent::PrinterComponent()  {
}



// The setServices() function is called by the framework
// after the component has been instantiated. The components
// behavior during the setServices invocation is very prescribed.
// The component    saves the passed reference to the  services
// object as a private instance variable.  Prior to returning from 
// the setServices() call, the component must instantiate and add
// all its initial Provides ports and register all its initial uses
// ports  using this instance variable. Components may dynamically
// add new ports later, but this will cause a newPortEvent to be
// generated by the framework. Components cannot delete ports. 
// The way the framwork notifies a component to shut down is to
// call the setServices() with a null value. When such an invocation
// of setServices completes, it is safe for the Framework to garbage
// collect the component.
// Note that the component itself implements the provides
// port and hence it passes "this"
void PrinterComponent::setServices(Services *cc){
  core = cc;
  core->registerUsesPort(new DefaultPortInfo("stringOutPortName", "String_idl")); 
  core->addProvidesPort(this, new DefaultPortInfo("stringInPortName", "String_idl"));
  // should this be done before the earlier three statements, and shouldn't we
  // skip the register statements in this case?
  if (cc == NULL) {
    if (core != NULL) {
      delete core;
    }
  }
}


// component implements the provides port interface
// returns an int as this will be invoked as a remote
// method call by HPCxx. Do not change this return void.
int PrinterComponent::setString(char * s) {
  cout << "PrinterComponent:: The string received is: " << s << endl;
  cout.flush();
  return 0;
}

void PrinterComponent::execute() {
  cout << "PrinterComponent:: execute called ! " << endl << endl;
  cout.flush();
  UsesString_idlPort *output;
  output = dynamic_cast<UsesString_idlPort *>(core->getPort("stringOutPortName"));
  char *s = "Hello World from PrinterComponent ! ";
  int result = output->setString(s);
  core->releasePort("string_out");
  cout << "PrinterComponent:: execute done ! " << endl << endl;
  cout.flush();
}

