// PrinterServer.C: This is the implementation
// class provided by the component developer.
// Note that since the component itself implements
// the provides port, the implementation of 
// setString(...) can be found in this file
// --Madhu

#include <iostream.h>
#include "../../../../framework/ccacore/Port.h"
#include "../../../../framework/ccacore/CCATServicesProxy.h"
#include "PrinterComponent.h"
#include "PrinterComponentHome.h"
#include "../../../../framework/idl/string/CCAT_string_idl.h"
#include "../../../../framework/ccacore/RegisterType.h"
#include "../../../../framework/ccacore/FRAMEWORK_REGISTRATION_ID.h"
#include "../../../system/event-channel/EventChannelComponent.h"
#include "../../../system/event-channel/EventChannelComponentHome.h"
#include "../../../../framework/idl/eventPort/ProvidesEventPortSkel.h"
#include "../../../../framework/idl/eventChannel/ProvidesFilteringEventChannelSkel.h" 
#include <hpcxx.h>

HPCxx_Mutex myLock;

PrinterComponent *printer = NULL;
CCATServicesSkel *printerCore = NULL;

// Flag used to make sure that the core is initialized 
// before the function  returnComponentID tries to access it.
// We cannot use sync variables here, as hpcxx_initAgent
// may not have gone  through (i.e. Nexus may not have been
// initialized) when this function will get called.
static bool coreDone = false;

// This global method is invoked by the creation
// service (gram) to get a handle to the component.
// It is okay to busy wait (and hence waste cpu cycles)
// as the component should not be allowed to attempt to 
// do anything useful before coreDone gets set to true
CCATServicesProxy returnComponentID(){
  while (!coreDone) ;
  ((CCATServicesProxy *)(printerCore->getComponentID()))->setName("Printer CompID");
  return *(dynamic_cast<CCATServicesProxy *>(printerCore->getComponentID()));
}

int main(int argc, char **argv) {
  hpcxx_initAgent(argc, argv);

  // returnComponentID(...) is specific to 
  // a user component. As a result it needs to
  // be registered here
  hpcxx_register(returnComponentID, RETURN_COMPONENT_ID);

  // register all the remote methods
  // that will be remotely invoked
  // once the component is initialized
  // Needs to be done only once per 
  // instantiation, hence it is a static
  // method call
  CCATServicesSkel::registerAll();

  // Avoid race condtions
  // Let only one thread be able to 
  // setServices(...) on the component
  myLock.lock();

  // Event channel creation and setup

  // Create a standard event channel component to be added to each application
  // component. This will take care of all the required initialization,
  // including ports and component registration.
  EventChannelComponent *channel = EventChannelComponentHome::instantiate();

  // create a core for the event channel. This must be different from the core
  // for the application component, although they must share the pseudo
  // components. The core creation will make calls on the
  // FrameworkServicesfactory and create the pseudo components, but will not
  // connect them to the event channel ports. When you instantiate a core with
  // an event channel as a parameter, it will complete this "framework
  // bootstrapping" process by making connections between the event channel and
  // the pseudo components.
  CCATServicesSkel *eventCore = new CCATServicesSkel();
  channel->setServices (eventCore);
  CCATServicesProxy* eventCompID = (CCATServicesProxy *)eventCore->getComponentID();

  // Instantiate a CCAT specific implementation of the services object.
  // This will get the pseudo components from the 
  printerCore = new CCATServicesSkel(eventCompID);

  // Instantiate a printer component. Will also do whatever initialization
  // necessary for this component to run properly, which includes port and
  // component registration.
  printer = PrinterComponentHome::instantiate();

  // This call is required as part of Component Interface. The component now has
  // a handle to the framework.
  printer->setServices(printerCore);

  // export the event channel component's "register" provides port as the
  // application component's "register" provides port.
  ConnectionService *connectionService = 
    (ConnectionService *) (printerCore->getPort("ConnectionService"));
  CCATServicesProxy* printerCompID = 
    (CCATServicesProxy *) printerCore->getComponentID();  
  connectionService->exportAs(printerCompID, eventCompID, "register", "register");  

  myLock.unlock();

  // It is now okay to return the componentID
  // to the creationService
  coreDone = true;
  
  cout << "PrinterServer:Up and waiting for requests\n" << endl;
  cout.flush();
  
  // The component needs to get beyond this 
  // blocking call to call the component's execute()
  // method. This call will unblock when 
  // he composition engine calls run() on the
  // the componentID of this component.
  printerCore->startRunning();
  cout << "PrinterServer: Main:: Received signal to execute \n" << endl;
  cout.flush();
  
  // Must have recieved a run() signal on 
  // the componentID. 
  printer->execute();
  
  // Will fix this later
  // This needs to be used, but am not
  // sure if this is the righ place for it
  // FIXME FIXME 
  printerCore->waitForKill();
  cout << "Component has been killed! \n ";
  cout << "PrinterServer exiting ... \n";
  cout.flush();

  // This sleep() call is useful if the
  // above kill statement is used. Just ensures
  // that the component dies before HPCxx 
  // starts its cleanup
  sleep(5);
  hpcxx_exit();
}


