#ifdef SGI64
#undef _BSD_COMPAT
#undef _POSIX_C_SOURCE
#endif
#include <stdlib.h>
#ifdef SGI64
#define _BSD_COMPAT
#define _POSIX_C_SOURCE 199506L
#endif
#include "AssembleComponent.h"
#include "Assembler.h"
#include "SolutionVector_impl.h"
#include "Geometry_impl.h"
#include "../Common/SolutionVector.h"
#include "../Common/Geometry.h"
#include "AssembleParms_impl.h"

#include "../../../../framework/ccacore/Port.h"
#include "../../../../framework/ccacore/CCATServicesProxy.h"
#include "../../../../framework/ccacore/RegisterType.h"
#include "../../../../framework/ccacore/FRAMEWORK_REGISTRATION_ID.h"

#include "../../../../framework/idl/runItAgain/UsesRunItAgain.h"

AssembleComponent::AssembleComponent() {

    int ec = -1;

    m_core = NULL;
    m_solutionVectorPort = new SolutionVector_impl(this);
    m_geometryPort = new Geometry_impl(this);
    m_assembleParmsPort = new AssembleParms_impl(this);
    m_computing = false;
    m_stopped = false;

    m_sv_save = NULL;
    m_geom_save = NULL;

    /*
     * Init mutex
     */

    pthread_mutexattr_t attr;

    pthread_mutexattr_init(&attr);

    #if defined(DEBUG) && !defined(__linux__)
        ec = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
        assert(ec == 0);
    #endif

    pthread_mutex_init(&m_lock, &attr);

    pthread_mutexattr_destroy(&attr);

    /*
     * Init condition variable.
     */

    pthread_cond_init(&m_cv, NULL);

    /*
     * Init pthread attr
     */

    ec = pthread_attr_init(&m_threadAttr);
    assert(ec == 0);
    ec = pthread_attr_setdetachstate(&m_threadAttr, PTHREAD_CREATE_DETACHED);
    assert(ec == 0);
    //ec = pthread_attr_setscope(&m_threadAttr, PTHREAD_SCOPE_SYSTEM);
    //assert(ec == 0);
}

AssembleComponent::~AssembleComponent() {

    int ec = -1;

    if (m_core != NULL) {
        delete m_core;
    }

    delete m_solutionVectorPort;
    delete m_geometryPort;

    ec = pthread_attr_destroy(&m_threadAttr);
    assert(ec == 0);

    ec = pthread_cond_destroy(&m_cv);
    assert(ec == 0);

    ec = pthread_mutex_destroy(&m_lock);
    assert(ec == 0);
}

void
AssembleComponent::setServices(Services *cc) {

    if (cc == NULL) {

        assert(m_core != NULL);
        delete m_core;
        m_core = NULL;
        return;

    } else {

        m_core = cc;

        m_core->addProvidesPort(m_solutionVectorPort,
         new DefaultPortInfo("inputSV", "SolutionVector_idl"));
        m_core->addProvidesPort(m_geometryPort,
         new DefaultPortInfo("inputGeometry", "Geometry_idl"));
        m_core->addProvidesPort(m_assembleParmsPort,
         new DefaultPortInfo("inputAssembleParms", "AssembleParms_idl"));
	
       m_core->registerUsesPort(new DefaultPortInfo("outputRunItAgain",
         "RunItAgain_idl"));
    }
}

struct Info {
    Info(AssembleComponent *cop, SolutionVector *s, Geometry *g)
     : component(cop), sv(s), geom(g) {}
    AssembleComponent *const component;
    SolutionVector *const sv;
    Geometry *const geom;
};

void *
AssembleComponent::compute_wrapper(void *vp) {

    Info *ip = static_cast<Info *>(vp);
    Info info = *ip;
    delete ip;

    info.component->compute(info.sv, info.geom);

    return NULL;
}

void
AssembleComponent::startComputation(SolutionVector *sv, Geometry *g) {

    pthread_t tid;

    Info *info = new Info(this, sv, g);
    assert(info != NULL);

    pthread_create(&tid, &m_threadAttr, compute_wrapper, info);
}

void
AssembleComponent::compute(SolutionVector *sv, Geometry *g) {
const char *xfn = "AssembleComponent::compute";

    int ec = -1;

    cout << xfn << " called." << endl;
  
    assert(sv != NULL);
    assert(g != NULL);

    /*
    cout << xfn << " solution Vector is " << endl;
    for (int i =0; i<sls->n; i++) {
        cout << (sls->x)[i]  << endl;
    }
    */

	// This doesn't work on gromit (irix
    ec = putenv("PATH=/work/matlab/bin:/bin:/usr/bin:/usr/local/bin");
    assert(ec == 0);
    // not required, now that matlab outputs a jpeg
    //ec = putenv("DISPLAY=baldy.extreme.indiana.edu:0");
    ec = putenv("DISPLAY=exodus.extreme.indiana.edu:0");
    assert(ec == 0);

    double *z = Assembler(g, sv->solutionDoubleArray);
    assert(z != NULL);
    delete [] z;
    cout << xfn << " solution displayed." << endl;

    UsesRunItAgain *up = dynamic_cast<UsesRunItAgain *>(m_core
      ->getPort("outputRunItAgain"));
    up->sendRunItAgain();
    m_core->releasePort("outputRunItAgain");
    
    cout << xfn << " sendRunItAgain called." << endl;

    pthread_mutex_lock(&m_lock);
    m_computing = false;
    pthread_cond_broadcast(&m_cv);
    pthread_mutex_unlock(&m_lock);

    /*
    delete sv;
    delete geom;
    */
}
