
// Turns out only one small change needed in Assembler.C.
// Run xdiff on it with yours to see what I added.

// Note: I specify a JPG with 30% quality.  If size is a
// real problem, we can drop that to 5%.  I doubt that
// will be necessary.  Also note that the image is much
// larger in terms of screen real estate than the Matlab
// graphics window normally is.  This is a Good Thing.

// Finally, the image will be written out to PDEsnapshot.jpg
// I can easily convert the code to write out to a sequence
// of files, e.g. PDE001.jpg, PDE002.jpg, etc. 

// -RB

// --------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include <strings.h>
#include <engine.h>  // Matlab Engine include
#include <Assembler.h> 
#include <Geometry.h>

// --------------------------------------------------
// Function that assembles a geometry object and
// a solution vector, to create a plottable (X,Y,Z)
// thingy for Matlab.  The equivalent Matlab code is:
//
//  Load geometry;
//  M = length(x) - 1;
//  N = length(y) - 1;
//  Z = zeros(N+1,M+1);
//  Z(1,:)   = B(:)' ;
//  Z(N+1,:) = T(:)';
//  Z(2:N,1) = L;
//  Z(2:N,M+1) = R;
//  Load solution;
//  for k = 2:N
//    Z(k,2:M) = sol((k-2)*(M-1)+1 : (k-1)*(M-1))';
//  end
//  surf(x,y,Z);
//
//     Randall Bramley
//     Department of Computer Science
//     Indiana University
//     Phone: (812) 855-7790
//     email: bramley@cs.indiana.edu
//     Fri Nov 12 13:18:45 EST 1999
//
//     Modified to write out Matlab graphics window
//     to local file PDEsnapshop.jpg, on
//     Tue Feb 29 15:58:38 EST 2000
// --------------------------------------------------

#define  BUFSIZE 256

// ------------------ 
// Matlab structures. 
// ------------------ 
bool engineInitialized = false;
Engine *ep;
mxArray *T = NULL; 
mxArray *X = NULL; 
mxArray *Y = NULL; 

double *Assembler( Geometry* geom, double* sol){

	int i, j, k;
	char buffer[BUFSIZE];

	// --------------------
	// Unpack the arguments 
	// --------------------
	
	int M = geom->M;
	int N = geom->N;
	double* x   = geom->x;
	double* y   = geom->y;
	double* left   = geom->left;
	double* right  = geom->right;
	double* top    = geom->top;
	double* bottom = geom->bottom;

	double* Z = new double[(M+1)*(N+1)];
	if (Z == NULL) {
		cout << "BARF outta memory in Assembler\n";
		return NULL;
	}
	// cout << "In Assembler, M = " << M << endl;
	// cout << "In Assembler, N = " << N << endl;

	// --------------------------------------
	// Load Z with bottom boundary conditions
	// --------------------------------------
	for (k = 0; k < M+1; k++) {
		// cout << "Bottom index: " << k << endl;
		Z[k] = bottom[k];
	}

	// -----------------------------------
	// Load Z with top boundary conditions
	// -----------------------------------
	for (k = 0; k < M+1; k++) {
		// cout << "Top index: " << N*(M+1)+k << endl;
		Z[N*(M+1)+k] = top[k];
	}

	// ------------------------------------
	// Load Z with left boundary conditions
	// ------------------------------------
	for (k = 0; k < N-1; k++) {
		// cout << "Left index: " << (M+1)*(k+1) << endl;
		Z[(M+1)*(k+1)] = left[k];
	}

	// -------------------------------------
	// Load Z with right boundary conditions
	// -------------------------------------
	for (k = 0; k < N-1; k++) {
		// cout << "Right index: " << (M+1)*(k+2)-1 << endl;
		Z[(M+1)*(k+2)-1] = right[k];
	}

	// -------------------------------
	// Now load in the solution vector
	// -------------------------------
	k = 0;
	for (i = 1; i < N; i++) {
		for (j = 1; j < M; j++) {
		  Z[i*(M+1)+j] = sol[k++];
		}
	}

//  	if (!engineInitialized) {
//  	  // ------------------------------------------------------------
//  	  // Start the MATLAB engine locally by executing the string
//  	  // "matlab"
//  	  //
//  	  // To start the session on a remote host, use the name of
//  	  // the host as the string rather than \0
//  	  //
//  	  // For more complicated cases, use any string with whitespace,
//  	  // and that string will be executed literally to start MATLAB
//  	  //
//  	  // ------------------------------------------------------------
//  	  if (!(ep = engOpen("\0"))) {
//  	    fprintf(stderr, "\nCan't start MATLAB engine\n");
//  	    return NULL;
//  	  }
//  	  engineInitialized = true;
//  	}

//  printf("matlab running.\n");
//  fflush(stdout);

//  	// ------------------------------------
//  	// Create Matlab variables for the data
//  	// ------------------------------------

//  	T = mxCreateDoubleMatrix(N+1, M+1, mxREAL);
//  	X = mxCreateDoubleMatrix(M+1, 1, mxREAL);
//  	Y = mxCreateDoubleMatrix(N+1, 1, mxREAL);

//  	mxSetName(T, "T");
//  	mxSetName(X, "X");
//  	mxSetName(Y, "Y");

//  	memcpy((void *)mxGetPr(T),
//                 (void *)Z,
//                 sizeof(double)*(M+1)*(N+1));

//  	memcpy((void *)mxGetPr(X),
//                 (void *)x,
//                 sizeof(double)*(M+1));

//  	memcpy((void *)mxGetPr(Y),
//                 (void *)y,
//                 sizeof(double)*(N+1));

//  	// ---------------------------------------------
//  	// Place the variables into the MATLAB workspace
//  	// ---------------------------------------------

//  	engPutArray(ep, T);
//  	engPutArray(ep, X);
//  	engPutArray(ep, Y);

//  	// ---------------------------------
//  	// Do a surface plot of the solution
//  	// ---------------------------------
//  	engEvalString(ep, "surf(X,Y,T);");
//  	engEvalString(ep, "title('Solution for PDE');");
//  	engEvalString(ep, "xlabel('x-coord');");
//  	engEvalString(ep, "ylabel('y-coord');");

//  	//---------------------------------------------
//  	// Now write out the graphics window to a file;
//  	// the file is called PDEsnapshot.jpg.  Note 
//  	// number after djpeg; it gives the percent of
//  	// compression.  Use 95 for high quality pics,
//  	// and 25 for low quality but smaller ones.
//  	//---------------------------------------------
//  	engEvalString(ep, "print -djpeg30 assemblerresults/PDEsnapshot.jpg ;");

//  	//---------------------------------------------
//  	// Insert a "pause" command to make sure Matlab
//  	// draws the plot.  This really sucks.
//  	//---------------------------------------------
//  	engEvalString(ep, "pause(10);");

//   	// ------------------------------------------------
//   	// Debugging stuff: capture Matlab's output of whos
//   	// ------------------------------------------------
//  	engOutputBuffer(ep, buffer, BUFSIZE);
//  	engEvalString(ep, "whos");
//  	printf("%s", buffer+2);  // First two are prompt >>
//  	engEvalString(ep, "size(X)");
//  	printf("%s", buffer+2);  

//  	// ---------------------------------------
//  	// Done. Free memory, close MATLAB engine.
//  	// ---------------------------------------
//  	printf("Cleaning up x,y,T\n");

//  	mxDestroyArray(T);
//  	mxDestroyArray(X);
//  	mxDestroyArray(Y);

	//	engEvalString(ep, "close;");
	//	engClose(ep);

	return Z;
}

