#include "BasicInfoComponent.h"
#include "BasicInfo.h"
#include "SparseLinearSystem_impl.h"
#include "BasicInfoParms_impl.h"
#include "../../../../framework/ccacore/DefaultPortInfo.h"
#include "../../../../framework/idl/sparseLinearSystem/UsesSparseLinearSystem.h"

#include "../../../../framework/ccacore/Port.h"
#include "../../../../framework/ccacore/CCATServicesProxy.h"
#include "../../../../framework/ccacore/RegisterType.h"
#include "../../../../framework/ccacore/FRAMEWORK_REGISTRATION_ID.h"

static int counter;


BasicInfoComponent::BasicInfoComponent() {

  int ec = -1;


  cout << "CREATION!!!!!!!!!!!!!!" << counter << endl;
  whichone = counter++;

  m_core = NULL;
  m_sparseLinearSystemPort = new SparseLinearSystem_impl(this);
  m_basicInfoParmsPort = new BasicInfoParms_impl(this);
  m_computing = false;
  m_stopped = false;
  m_parms_cache = new BasicInfoParms;

  /*
   * Init mutex
   */

  pthread_mutexattr_t attr;

  pthread_mutexattr_init(&attr);

#if defined(DEBUG) && !defined(__linux__)
  ec = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
  assert(ec == 0);
#endif

  pthread_mutex_init(&m_lock, &attr);

  pthread_mutexattr_destroy(&attr);

  /*
   * Init condition variable.
   */

  pthread_cond_init(&m_cv, NULL);

  /*
   * Init pthread attr
   */

  ec = pthread_attr_init(&m_threadAttr);
  assert(ec == 0);
  ec = pthread_attr_setdetachstate(&m_threadAttr, PTHREAD_CREATE_DETACHED);
  assert(ec == 0);
  //ec = pthread_attr_setscope(&m_threadAttr, PTHREAD_SCOPE_SYSTEM);
  //assert(ec == 0);

  /*
    printf("%d, %lld\n", sizeof(m_cv.__pthread_cond_data),
    m_cv.__pthread_cond_data);
  */
}

BasicInfoComponent::~BasicInfoComponent() {

  int ec = -1;

  if (m_core != NULL) {
    delete m_core;
    m_core = NULL;
  }

  delete m_sparseLinearSystemPort;
  delete m_basicInfoParmsPort;

  ec = pthread_attr_destroy(&m_threadAttr);
  assert(ec == 0);

  ec = pthread_cond_destroy(&m_cv);
  assert(ec == 0);

  ec = pthread_mutex_destroy(&m_lock);
  assert(ec == 0);
}

void
BasicInfoComponent::setServices(Services *cc) {

  if (cc == NULL) {

    assert(m_core != NULL);
    delete m_core;
    return;

  } else {

    m_core = cc;

    m_core->addProvidesPort(m_sparseLinearSystemPort,
			    new DefaultPortInfo("inputSLS", "SparseLinearSystem_idl"));
    m_core->addProvidesPort(m_basicInfoParmsPort,
			    new DefaultPortInfo("inputBasicInfoParms", "BasicInfoParms_idl"));
    m_core->registerUsesPort(new DefaultPortInfo("outputSLS",
						 "SparseLinearSystem_idl"));
  }
}

struct BIInfo {

  BIInfo(BasicInfoComponent *com, SparseLinearSystem *sls_,
	 BasicInfoParms *parms_) : component(com), sls(sls_), parms(parms_) {}
  BasicInfoComponent *const component;
  SparseLinearSystem *const sls;
  BasicInfoParms *const parms;
};

void *
BasicInfoComponent::compute_wrapper(void *vp) {

  BIInfo *ip = static_cast<BIInfo *>(vp);
  BIInfo info = *ip;
  delete ip;

  info.component->compute(info.sls, info.parms);

  return NULL;
}


void
BasicInfoComponent::startComputation(SparseLinearSystem *sls,
				     BasicInfoParms *p) {
  const char *xfn = "BasicInfoComponent::startComputation";

  cout << xfn << " called." << endl;

  pthread_t tid;

  BIInfo *info = new BIInfo(this, sls, p);
  assert(info != NULL);

  cout << xfn << "creating thread." << endl; cout.flush();
    
  pthread_create(&tid, &m_threadAttr, compute_wrapper, info);
    
  cout << xfn << "done creating thread." << endl;
}

void
BasicInfoComponent::compute(SparseLinearSystem *sls, BasicInfoParms *parms) {
  const char *xfn = "BasicInfoComponent::compute";

  int ec = -1;
  
  cout << xfn << " called." << endl;

  assert(sls != NULL);
  assert(parms != NULL);


  /*
    ec = BasicInfo(*parms, sls);
    assert(ec == 0);
  */
  sleep(5);
  cout << xfn << " BasicInfo done." << endl;

  UsesSparseLinearSystem *up = dynamic_cast<UsesSparseLinearSystem *>(m_core
								      ->getPort("outputSLS"));
  assert( up != NULL);
  cout << xfn << " Received outputSLS port" << endl;
  up->sendSparseLinearSystem(sls);
  m_core->releasePort("outputSLS");
  cout << xfn << " sent out SLS." << endl;
  
  /*
    delete sls;
    delete parms;
  */
 
  ec = pthread_mutex_lock(&m_lock);
  assert(ec == 0);
  m_computing = false;
  cout << xfn << " broadcasting to ." << &m_cv << endl;
  ec = pthread_cond_broadcast(&m_cv);
  assert(ec == 0);

  ec = pthread_mutex_unlock(&m_lock);
  assert(ec == 0);
}



