#include "SparseLinearSystem_impl.h"
#include "BasicInfoComponent.h"
#include "BasicInfoParms.h"

#include <signal.h>



SparseLinearSystem_impl::SparseLinearSystem_impl(BasicInfoComponent *c)
 : m_component(c) {}

SparseLinearSystem_impl::~SparseLinearSystem_impl() {}


int
SparseLinearSystem_impl::sendSparseLinearSystem(SparseLinearSystem *sls) {
const char *xfn = "BasicInfo: SparseLinearSystem_impl::sendSparseLinearSystem";
    
    cout << xfn << " called" << endl << flush;

    pthread_mutex_lock(&m_component->m_lock);

    while (m_component->m_computing || m_component->m_stopped) {
        pthread_cond_wait(&m_component->m_cv, &m_component->m_lock);
    }
    assert(!m_component->m_computing);
    assert(!m_component->m_stopped);

    // If we made it past here, then must be ready to compute.
    m_component->m_computing = true;
    m_component->startComputation(sls,
     new BasicInfoParms(*m_component->m_parms_cache));

    pthread_mutex_unlock(&m_component->m_lock);

    return 0;
}


// Implementation when BasicInfoParms port is not involved
//  int
//  SparseLinearSystem_impl::sendSparseLinearSystem(SparseLinearSystem *sls) {
//  const char *xfn = "SparseLinearSystem_impl::sendSparseLinearSystem";
    
//      int ec = -1;

//      cout << xfn << " called" << endl;

//      ec = pthread_mutex_lock(&m_component->m_lock);
//      assert(ec == 0);

//      while (m_component->m_computing) {
//  cout << xfn << " waiting on  " << &m_component->m_cv << endl;
//          pthread_cond_wait(&m_component->m_cv, &m_component->m_lock);
//  cout << xfn << " woken" << endl;
//      }
//      assert(!m_component->m_computing);
//      assert(m_component->m_parms_cache != NULL);

//      m_component->m_computing = true;
//  cout << xfn << " starting computation" << endl;
//      m_component->startComputation(sls,
//       new BasicInfoParms(*m_component->m_parms_cache));
//  cout << xfn << " done starting " << endl;

//      ec = pthread_mutex_unlock(&m_component->m_lock);
//      assert(ec == 0);

//      cout << xfn << " exiting." << endl;

//      return 0;
//  }

