        subroutine getinfo(m,n,iout,a,ja,ia,ao,jao,iao,info)
        implicit none
        real*8 a(*),ao(*)
        integer n, iout
        integer ja(*),ia(n+1),jao(*),iao(n+1),nzdiag 
        character title*72,key*8
        logical valued , square
        real*8 info(0:100)
        integer zcol(10), zrow(10)
c ----------------------------------------------------------------------c
c Provide information on a sparse matrix and return them 	       c
c to the calling program.  This routine assumes the matrix is stored   c
c in CSR format, and it will be necessary to convert if the matrix     c
c starts out in any other format.  Most the routines are from the      c
c Sparskit package of Yousef Saad and Edmond Chow, with only minor     c
c modifications and wrappers.  For the most part, changes consist of   c
c being careful about square versus nonsquare matrices.  When a        c
c quantity does not make sense (such as the number of entries nsky     c
c required for skyline storage), a nonsense value such as -1 is        c
c returned.							       c
c								       c
c     Randall Bramley						       c
c     Department of Computer Science 				       c
c     Indiana University					       c
c     Phone: (812) 855-7790					       c
c     email: bramley@cs.indiana.edu				       c
c     Fri May 30 11:43:14 EST 1997				       c
c								       c
c----------------------------------------------------------------------c
c
c On entry :
c-----------
c
c m	= integer. row dimension of matrix	
c
c n	= integer. column dimension of matrix	
c
c iout  = integer. unit number where the information it to be output.	
c         If iout <= 0, then no output is performed.
c
c a	= real array containing the nonzero elements of the matrix
c	  the elements are stored by columns in order 
c	  (i.e. column i comes before column i+1, but the elements
c         within each column can be disordered).
c
c ja	= integer array containing the row indices of elements in a
c
c ia	= integer array containing of length n+1 containing the 
c         pointers to the beginning of the columns in arrays a and ja.
c	  It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c 
c valued= logical equal to .true. if values are provided and .false.
c         if only the pattern of the matrix is provided. (in that
c         case a(*) and ao(*) are dummy arrays.
c
c title = a 72-character title describing the matrix
c         NOTE: The first character in title is ignored (it is often
c         a one, for carriage control).
c
c key   = an 8-character key for the matrix
c
c On return
c---------- 
c   The entries of a, ja, ia are sorted.
c
c---------- 
c 
c ao	= real*8 array of length nnz used as work array.
c
c jao	= integer work array of length max(2*n+1,nnz) 
c
c iao   = integer work array of length m+1
c
c Note  : title and key are the same parameters as those
c         used for Harwell-Bowing matrices.
c 
c-----------------------------------------------------------------------
c Output description:
c--------------------
c
c    A vector called "info" is returned, with fields having the
c      following interpretations.  Note that info is indexed from
c      zero, not one, and info(0) is an error flag.  If info(0) is
c      not equal to zero, the results are possibly flawed.
c
c info(0)  : Error flag.  Should be zero for successful return
c            Note that in some cases it will be nonzero, but
c            most of the information fields are still correct.
c info(1)  : Order n of matrix (actually input as well)
c info(2)  : Number of nonzeros nnz.
c info(3)  : Average number of nonzero elements per row
c info(4)  : Standard deviation for that average.
c info(5)  : Nonzeros in strict upper triangular part.
c info(6)  : Nonzeros in strict lower triangular part.
c info(7)  : Number of zeros in main diagonal.
c info(8)  : Max nonzeros in a row.
c info(9)  : Number of nonzeros in smallest row.
c info(10) : Number of nonzeros in largest column.
c info(11) : Number of nonzeros in smallest column.
c info(12) : Matching elements in A and transp(A).  Count of the
c            number of positions (i,j) such that if a(i,j) .ne. 0
c            then a(j,i) .ne. 0.  If this is equal to nnz, then
c            the matrix is structurally symmetric.
c info(13) : Relative symmetry match: ratio of info(12) to nnz.
c            Is equal to 1.0 for a structurally symmetric matrix.
c info(14) : Average distance of nonzeros from main diagonal
c info(15) : Standard deviation of distance of nonzeros from diagonal
c info(16) : Frobenius norm of A
c info(17) : Frobenius norm of the symmetric part 1/2 (A + A')
c info(18) : Frobenius norm of the nonsymmetric part 1/2 (A - A')
c            Note that if this is zero, then the matrix is
c            symmetric in value as well as structure.
c info(19) : Maximum element in A
c info(20) : Relative number of weakly diagonally dominant columns
c info(21) : Relative number of weakly diagonally dominant rows 
c info(22) : Upper bandwidth: max(i-j), for A(i,j) .ne. 0
c info(23) : Lower bandwidth: max(j-i), for A(i,j) .ne. 0
c info(24) : Maximum bandwidth
c info(25) : Average bandwidth
c info(26) : Number of nonzeros for skyline storage
c info(27) : Bandwidth containing 90% of matrix nonzeros
c info(28) : Bandwidth containing 80% of matrix nonzeros
c info(29) : Total number of nonzero diagonals
c
c info(30-39) : Ten most important diagonals, given as offsets from
c               main diagonal.  Important is in terms of number of 
c               nonzeros, and does nto account for the values of those
c               nonzeros.
c info(40-49) : Accumulated percentage of nonzeros from ten most
c               important diagonals
c info(50) : Block size if matrix has a block structure.
c
c info(51) : Number of rows which are all zero.  INDICATES SINGULARITY.
c info(52) : Number of cols which are all zero.  INDICATES SINGULARITY.
c info(53) : Number of explicitly zero entries.
c info(60-69) : First ten row numbers which are zero rows
c info(70-79) : First ten col numbers which are zero rows
c
c-----------------------------------------------------------------------
	real*8 dcount(20),amx
	integer ioff(20)
	character*61 tmpst
	logical sym

        integer ipar1, nnz, job, ipos, nsky, nlower, nupper
        integer ndiag, nzmaxc, nzminc, nzmaxr, nzminr
        integer nzcol, nzrow, imatch, n2
        integer ml, mu, iband, nrow, ncol
        integer jb2, jb1, nblk
        integer i, k, k1, k2, j, m
        integer unzero
        real*8  av, st, ddomc, ddomr, fnorm, fas, fan
        real*8  std, bndav, dist

c-----------------------------------------------------------------------
	data ipar1 /1/

	if (iout .gt. 0) then
           write (iout,99) 
	end if 
c---------------------------------------------------------------------

	square = (m .eq. n)
	nnz = ia(m+1)-ia(1)

        info(0) = 0
        info(1) = n
        info(2) = nnz

c       --------------------------------------------
c       This code assumes CSR format, and that the
c       matrix, even if symmetric, has all entries
c       stored - including symmetrically equal ones.
c       --------------------------------------------
	sym = .false.

	job = 1
	ipos = 1

c       ----------------------------------------------------
c       Do a transpose twice to clean up the matrix, and to
c       get a copy of the transpose matrix.
c       ----------------------------------------------------
        call csrcsc(m, n, job, ipos, a, ja, ia, ao, jao, iao)
        call csrcsc(n, m, job, ipos, ao, jao, iao, a, ja, ia)

c-------------------------------------------------------------------
c computing max bandwith, max number of nonzero elements per column
c min nonzero elements per column/row, row/column diagonal dominance 
c occurences, average distance of an element from diagonal, number of 
c elemnts in lower and upper parts, ...
c------------------------------------------------------------------

c         ----------------------------------------------
c         The skyline data structure only makes sense if
c         the matrix is square.
c         ----------------------------------------------
          if (square) then
             call skyline(n,sym,ja,ia,jao,iao,nsky)
          else 
             nsky = -1
             info(0) = 1
          end if

c         -----------------------------------------------
c         Get counts of nonzeros in upper/lower/diag part
c         -----------------------------------------------
          call nonz_lud(m, n,a,ja,ia,nlower, nupper, ndiag,unzero) 

          call avnz_row(m, n,ja,ia,iao, ndiag, av, st)

	  if (sym)  nupper = nlower 
          info(3) = av
          info(4) = st
          info(5) = nlower
          info(6) = nupper
          info(7) = m - ndiag
          info(53) = unzero
          
c         --------------------------------------------
c         Max/min numbers of nonzeros per row; number
c         of zero rows or columns (bad news for linear
c         systems which purport to be nonsingular).
c         NOTE: roles of row/col are reversed since we
c         use CSR instead of CSC
c         --------------------------------------------
          call nonz(m, n,sym, ja, ia, iao, nzmaxc, nzminc,
     *                  nzmaxr, nzminr, nzcol, nzrow,
     *                  zcol, zrow)

          info(8)  = nzmaxc
          info(9)  = nzminc
          info(10) = nzmaxr
          info(11) = nzminr
          info(51) = nzrow
          info(52) = nzcol
          do k = 0, 9
            info(70+k) = zrow(k+1)
            info(60+k) = zcol(k+1)
          end do 


c         ----------------------------
c         Check for diagonal dominance
c         ----------------------------
          valued = .true.
          if (square) then
              call diag_domi(m, n,sym,valued,a, ja,ia,ao,iao,
     *                           ddomc, ddomr)
          else
              ddomc = -1.0d0
              ddomr = -1.0d0
          end if

c----------------------------------------------------------------------- 
c symmetry and near symmetry - Frobenius  norms
c-----------------------------------------------------------------------
          call frobnorm(m, n,sym,a,ja,ia,Fnorm)   

          if (square) then
            call ansym(m,n,sym,a,ja,ia,ao,jao,iao,imatch,av,fas,fan)
          else 
            imatch = -1
            fan  = -1.0d0
            fas  = -1.0d0
            av   = -1.0d0
          end if

          if (square) then
             call distaij(m, n,nnz,sym,ja,ia,dist, std)
          else
             dist = -1.0d0
             std  = -1.0d0
          end if

          amx = 0.0d0
          do 40 k=1, nnz
            amx = max(amx, abs(a(k)) )
 40       continue

          info(12) = imatch
          info(13) = av
          info(14) = dist
          info(15) = std
          info(16) = Fnorm
          info(17) = fas
          info(18) = fan
          info(19) = amx
          info(20) = ddomr
          info(21) = ddomc


c-----------------------------------------------------------------------
c--------------------bandedness- main diagonals ----------------------- -
c-----------------------------------------------------------------------
        if (square) then
	  n2 = m+n-1
	  do i = 1, n2
               jao(i) = 0
          end do  

          do i = 1, m
               do k = ia(i), ia(i+1) -1
                  j = ja(k)
                  jao(n+i-j) = jao(n+i-j) +1
               end do
          end do 
 
          call bandwidth(m, n,ja, ia, ml, mu, iband, bndav)
c
c     Write bandwidth information 
c     
          info(22) = ml
          info(23) = mu
          info(24) = iband
          info(25) = bndav
          info(26) = nsky

          nrow = m
          ncol = n

          call distdiag(nrow,ncol,ja,ia,jao)
          call bandpart(m, n,ja,ia,jao,90,jb2)
          call bandpart(m, n,ja,ia,jao,80,jb1)

          info(27) = 2*jb2+1
          info(28) = 2*jb1+1
c-----------------------------------------------------------------
          nzdiag = 0
          n2 = m+n-1
          do i = 1, n2
             if (jao(i) .ne. 0) nzdiag = nzdiag+1
          end do

          call n_imp_diag(m, n,nnz,jao,ipar1, ndiag,ioff,dcount)

          info(29) = nzdiag
          do k = 0, 9
             info(30 + k) = ioff(k+1)
             info(40 + k) = dcount(k+1)
          end do

	  if (iout .gt. 0) then
c            -----------
c            Basic Info:
c            -----------
             write (iout, 99)
             write (iout, 290)
             write(iout,96)
             write(iout, 100) n, nnz
             write(iout, 1020) int(info(8)), int(info(9))
	     if (.not. sym) write(iout, 1021) int(info(10)), 
     1             int(info(11)), int(info(53))
             write(iout, 101) info(3), info(4)
             write(iout,1045) info(19)
             write(iout,96)

c            --------------
c            Symmetry Info:
c            --------------
             write (iout, 291)
             write(iout,96)
             write(iout, 107) int(info(5))
             write(iout, 108) int(info(6))
             write (iout,1035) int(info(12)), info(13)
             if (info(5) .eq. 0 ) write(iout, 105)
             if (info(6) .eq. 0) write(iout, 106)
             write(iout,104) info(16), info(17), info(18)
             write(iout,96)

c            -----------------------
c            Diagonal dominance info
c            -----------------------
             write (iout, 292)
             write(iout,96)
             write(iout,1047) info(20), info(21)
             write(iout,96)

c            ----------------------------------
c            Potential singularity information:
c            ----------------------------------
             write (iout, 293)
             write(iout,96)
             write(iout, 109) int(info(7))

	     write(iout,115) int(info(52))
             if (info(52) .gt. 0) then
	        write(iout,1155) 
     1            (int(info(60+k)),k = 0,min(int(9),int(info(52)-1)))
             end if

	     write(iout,116) int(info(51))
             if (info(51) .gt. 0) then
	        write(iout,1165) 
     1            (int(info(70+k)),k = 0,min(int(9),int(info(51)-1)))
             end if
             write(iout,96)

c            -----------------------------
c            Syline structure information:
c            -----------------------------
             write (iout, 294)
             write(iout,96)
             write(iout,1175) int(info(26))
             write(iout,96)

c            -----------------------------
c            Banded structure information:
c            -----------------------------
             write (iout, 295)
             write(iout,96)
             write (iout,103) info(14), info(15)
             write(iout,117)  int(info(22)), int(info(23)), 
     1                       int(info(24)), info(25)
             write (iout,112) int(info(27)), int(info(28))
             write (iout,118) int(info(29))
             write(iout,96) 

c            -----------------------------
c            Dense structure information:
c            -----------------------------
             write (iout, 300)
             write (iout, 310) info(1)*info(1)
             write(iout,96) 
c            -----------------------------------------
c            Note: following info is not returned yet.
c            -----------------------------------------
             write (tmpst,'(10i6)') (ioff(j),j=1,ndiag)
             write (iout,110) ndiag,tmpst
             write (tmpst,'(10f6.1)')(dcount(j), j=1,ndiag)
             write (iout,111) tmpst
             write (iout, 96)
	  end if
        else
          info(22) = -1
          info(23) = -1
          info(24) = -1
          info(25) = -1.0d0
          info(26) = -1
          info(27) = -1
          info(28) = -1
          info(29) = -1
          do k = 0, 9
             info(30 + k) = -1
             info(40 + k) = -1.0d0
          end do
        end if
c-----------------------------------------------------------------------
c     determine block size if matrix is a block matrix..
c-----------------------------------------------------------------------
          call blkfnd(m, n, ja, ia, nblk)
          info(50) = nblk
	  if (iout .gt. 0) then
             write(iout,114) int(info(50))
	  end if
c      write (iout,96)

c======================================================================
c---------- Format statements -------------------------- 
c======================================================================
 99    format (2x,38(2h *))
 96    format (6x,' *',65(1h-),'*')
c-----------------------------------------------------------------------
 100   format(
     * 6x,' *  Dimension N                                      = ',
     * i10,'  *'/
     * 6x,' *  Number of nonzero elements                       = ',
     * i10,'  *')
 101   format(
     * 6x,' *  Average number of nonzero elements/Column        = ',
     * f10.4,'  *'/
     * 6x,' *  Standard deviation for above average             = ',
     * f10.4,'  *')
c-----------------------------------------------------------------------
 1020       format(
     * 6x,' *  Max nonzeros in a column                         = ',
     * i10,'  *'/
     * 6x,' *  Min nonzeros in a column                         = ',
     * i10,'  *')
 1021       format(
     * 6x,' *  Max nonzeros in a row                            = ',
     * i10,'  *'/
     * 6x,' *  Min nonzeros in a row                            = ',
     * i10,'  *'/
     * 6x,' *  Number of explicitly stored zeros                = ',
     * i10,'  *')
 117        format(
     * 6x,' *  Lower bandwidth  (max: i-j, a(i,j) .ne. 0)       = ',
     * i10,'  *'/
     * 6x,' *  Upper bandwidth  (max: j-i, a(i,j) .ne. 0)       = ',
     * i10,'  *'/
     * 6x,' *  Maximum Bandwidth                                = ',
     * i10,'  *'/
     * 6x,' *  Average Bandwidth                                = ',
     * 1pe10.2,'  *')
 1175       format(
     * 6x,' *  Number of nonzeros in skyline storage            = ',
     * i10,'  *')
 1035  format(
     * 6x,' *  Number of entries matched in structural symmetry = ',
     * i10,'  *'/
     * 6x,' *  Structural symmetry coefficient (symmetry=1)     = ',
     * f10.4,'  *')
 103   format(
     * 6x,' *  Average distance of a(i,j)  from diag.           = ',
     * 1pe10.3,'  *'/
     * 6x,' *  Standard deviation for above average             = ',
     * 1pe10.3,'  *') 
 1045  format(
     * 6x,' *  Maximum element in A (abs. value)                = ',
     * 1pe10.3,'  *')

 1047  format(
     * 6x,' *  Fraction of weakly diagonally dominant rows    =   ',
     * 1pe10.3,'  *'/
     * 6x,' *  Fraction of weakly diagonally dominant columns =   ',
     * 1pe10.3,'  *')

 104   format(
     * 6x,' *  Frobenius norm of A                              = ',
     * 1pe10.3,'  *'/
     * 6x,' *  Frobenius norm of symmetric part                 = ',
     * 1pe10.3,'  *'/
     * 6x,' *  Frobenius norm of nonsymmetric part              = ',
     * 1pe10.3,'  *')
 105        format(
     * 6x,' *  The matrix is lower triangular ...       ',21x,' *')
 106        format(
     * 6x,' *  The matrix is upper triangular ...       ',21x,' *')
 107        format(
     * 6x,' *  Nonzero elements in strict lower part            = ',
     * i10,'  *')
 108       format(
     * 6x,' *  Nonzero elements in strict upper part            = ',
     * i10,'  *')
 109       format(
     * 6x,' *  Number of zeros on main diagonal                 = ',
     * i10,'  *')
 110   format(6x,' *  The ', i2, ' most important',
     *	   ' diagonals are (offsets)    : ',10x,'  *',/,  
     * 6x,' *',a61,3x,' *')

 111   format(6x,' *  The accumulated percentages they represent are ',
     * '  : ', 10x,'  *',/,
     * 6x,' *',a61,3x,' *')
c 111	format( 
c     * 6x,' *  They constitute the following % of A             = ',
c     * f8.1,' %  *') 
 112	format( 
     * 6x,' *  90% of matrix is in the band of width            = ',
     * i10,'  *',/,
     * 6x,' *  80% of matrix is in the band of width            = ',
     * i10,'  *')
 113 	format( 
     * 6x,' *  The matrix does not have a block structure ',19x,
     *    ' *')
 114 	format( 
     * 6x,' *  Block structure found with block size            = ',
     * i10,'  *')
 115 	format( 
     * 6x,' *  Number of rows consisting of all zeros           = ', 
     * i10,'  *')

1155 	format( 
     * 6x,' *  First rows which are exactly zero  :               ', 
     * 10x,'  *',/,
     * 6x,' *',5(i6,4x),/,
     * 6x,' *',5(i6,4x))

 116 	format( 
     * 6x,' *  Number of columns consisting of all zeros        = ', 
     * i10,'  *')

1165 	format( 
     * 6x,' *  First cols which are exactly zero  :               ', 
     * 10x,'  *',/,
     * 6x,' *',5(i6,4x),/,
     * 6x,' *',5(i6,4x))

 118 	format( 
     * 6x,' *  The total number of nonvoid diagonals is         = ', 
     * i10,'  *')
 290 	format( 
     * 6x,' *  BASIC INFORMATION                                  ',
     * 10x,'  *')
 291 	format( 
     * 6x,' *  SYMMETRY INFORMATION                               ',
     * 10x,'  *')
 292 	format( 
     * 6x,' *  DIAGONAL DOMINANCE INFORMATION                     ',
     * 10x,'  *')
 293 	format( 
     * 6x,' *  POTENTIAL SINGULARITY INFORMATION                  ',
     * 10x,'  *')
 294 	format( 
     * 6x,' *  SKYLINE DATA STRUCTURE INFORMATION                 ',
     * 10x,'  *')
 295 	format( 
     * 6x,' *  BANDED STRUCTURE INFORMATION                       ',
     * 10x,'  *')
 300 	format( 
     * 6x,' *  DENSE DATA STRUCTURE INFORMATION                   ',
     * 10x,'  *')
 310 	format( 
     * 6x,' *  Number of doubles to store matrix as dense is      ', 
     * 1pe10.4,'  *')
c-------------------------- end of dinfo --------------------------
        return
        end
c----------------------------------------------------------------------c
c                          S P A R S K I T                             c
c----------------------------------------------------------------------c
c                  INFORMATION ROUTINES. INFO MODULE                   c
c----------------------------------------------------------------------c
c bandwidth :  Computes  ml     = lower_bandwidth(A)                   c
c                        mu     = upper_bandwidth(A)                   c
c                        iband  = max_bandwidth(A)                     c
c                        bndav  = average_bandwidth(A)                 c
c nonz      :  Computes  nzmaxc = max_column_length(A)                 c
c                        nzminc = min_column_length(A)                 c
c                        nzmaxr = max_row_length(A)                    c
c                        nzminr = min_row_length(A)                    c
c                        nzcol  = zero_column_number(A)                c
c                        nzrow  = zero_row_number(A)                   c
c diag_domi :  Computes  ddomc  = diag_domi_column_percentage(A)       c
c                        ddomr  = diag_domi_row_percentage(A)          c
c frobnorm  :  Computes  Fnorm  = Frobenius_norm(A)                    c
c ansym     :  Computes  fas    = sym_part_Frobenius_norm(A)           c
c                        fan    = nonsym_part_Frobenius_norm(A)        c
c                        imatch = matching_elements_number(A)          c
c                        av     = relative_sym_match(A)                c
c distaij   :  Computes  dist   = average_dist_of_a(i,j)(A)            c
c                        std    = standard_deviation(A)                c
c skyline   :  Computes  nsky   = nonzero_number_in_skyline(A)         c
c distdiag  :  Computes  dist   = element_number_in_eachdiag(A)        c
c bandpart  :  Computes  band   = bandwidth_width(A)                   c
c n_imp_diag:  Computes  ndiag  = important_diag_number(A)             c
c nonz_lud  :  Computes  nlower = nonzero_number_of_lower_part(A)      c
c                        nupper = nonzero_number_of_upper_part(A)      c
c                        ndiag  = nonzero_number_of_maindiag(A)        c
c avnz_col  :  Computes  av     = average_nonzero_number_in_column(A)  c
c                        st     = standard_deviation(A)                c
c vbrinfo   :  Print info on matrix in variable block row format       c
c----------------------------------------------------------------------c

	subroutine bandwidth(m, n,ja, ia, ml, mu, iband, bndav)
        implicit real*8 (a-h, o-z)
        real*8 bndav
        integer m, n, ja(*), ia(n+1)
        integer ml, mu, iband
c-----------------------------------------------------------------------
c this routine computes the lower, upper, maximum, and average 
c bandwidths. 
c-----------------------------------------------------------------------
c On Entry:
c----------
c m     = integer. row dimension of matrix
c n     = integer. column dimension of matrix
c a     = real array containing the nonzero elements of the matrix
c         the elements are stored by columns in order
c         (i.e. column i comes before column i+1, but the elements
c         within each column can be disordered).
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length n+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c
c on return
c----------
c ml    = lower bandwidth as defined by
c        ml = max(i-j | all  a(i,j).ne. 0)
c mu    = upper bandwidth as defined by
c        mu = max ( j-i | all  a(i,j).ne. 0 )
c iband =  maximum bandwidth as defined by
c         iband = Max (  Max [ j | a(i,j) .ne. 0 ] - 
c                        Min [ j | a(i,j) .ne. 0 ] )
c bndav = Average Bandwidth          
c-----------------------------------------------------------------------

        iband = 0
        ml = -m
        mu = -n
        bndav = 0.0d0

        do 10 i = 1, m
                j0 = ia(i)
                j1 = ia(i+1) - 1

                jminc = ja(j0)
                jmaxc = ja(j1)

                ml = max0(ml,i-jminc)
                mu = max0(mu,jmaxc-i)
                iband = max0(iband,jmaxc-jminc+1)
                bndav = bndav+real( jmaxc-jminc+1)

 10     continue

        bndav = bndav/real(m)

        return
        end


	subroutine nonz(m, n,sym, ja, ia, iao, nzmaxc, nzminc, 
     *                  nzmaxr, nzminr, nzcol, nzrow,
     *                  zcol, zrow)
        implicit real*8 (a-h, o-z)
        integer m, ja(*), ia(m+1), iao(m+1)
	integer nzmaxc, nzminc, nzmaxr, nzminr, nzcol, nzrow
	integer zcol(10), zrow(10)
        logical sym
c----------------------------------------------------------------------
c this routine computes maximum numbers of nonzero elements 
c per column/row, minimum numbers of nonzero elements per column/row, 
c and  numbers of zero columns/rows.
c----------------------------------------------------------------------
c On Entry:
c----------
c m     = integer row dimension of matrix
c n     = integer column dimension of matrix
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length m+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(m+1) = nnz+1.
c iao   = integer work array of length m+1
c sym   = logical variable indicating whether or not the matrix is 
c         stored in symmetric mode.
c on return
c----------
c nzmaxc = max length of columns
c nzminc = min length of columns 
c nzmaxr = max length of rows
c nzminr = min length of rows
c nzcol  = number of zero columns
c nzrow = number of zero rows
c

        nzmaxc = 0
        nzminc = n
        nzmaxr = 0
        nzminr = m
        nzcol  = 0
        nzrow  = 0

	do 10 i = 1, m
           j0 = ia(i)
           j1 = ia(i+1) - 1
	   lenc = j1+1-j0
           if (lenc .le. 0) then
              nzcol = nzcol +1
              if (nzcol .le. 10) then
                 zcol(nzcol) = i
              end if 
           end if 
	   nzmaxc = max0(nzmaxc,lenc)
	   nzminc = min0(nzminc,lenc)
 10 	continue

	do 110 i = 1, n
           j0r = iao(i)
           j1r = iao(i+1) - 1
	   lenr = j1r+1-j0r
	   if (lenr .le. 0) then
              nzrow = nzrow+1
              if (nzrow .le. 10) then
                 zrow(nzrow) = i
              end if 
	   end if 
	   nzmaxr = max0(nzmaxr,lenr)
	   nzminr = min0(nzminr,lenr)
 110 	continue
        return
	end

        subroutine diag_domi(m, n,sym,valued,a, ja,ia,ao,iao, 
     *                           ddomc, ddomr)
        implicit real*8 (a-h, o-z)
        real*8 a(*), ao(*), ddomc, ddomr
        integer m, n, ja(*), ia(m+1), iao(n+1)
        logical sym, valued
        
c-----------------------------------------------------------------
c this routine computes the percentage of weakly diagonally 
c dominant rows/columns
c-----------------------------------------------------------------
c on entry:
c ---------
c m     = integer row dimension of matrix
c n     = integer column dimension of matrix
c a     = real array containing the nonzero elements of the matrix
c         the elements are stored by columns in order
c         (i.e. column i comes before column i+1, but the elements
c         within each column can be disordered).
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length n+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c ao    = real array containing the nonzero elements of the matrix
c         the elements are stored by rows in order
c         (i.e. row i comes before row i+1, but the elements
c         within each row can be disordered).
c iao   = integer array containing of length n+1 containing the
c         pointers to the beginning of the rows in arrays ao and jao.
c         It is assumed that iao(*) = 1 and iao(n+1) = nzz+1.
c sym   = logical variable indicating whether or not the matrix is
c         symmetric.
c valued= logical equal to .true. if values are provided and .false.
c         if only the pattern of the matrix is provided. (in that
c         case a(*) and ao(*) are dummy arrays.
c
c on return
c----------
c ddomc = percentage of weakly diagonally dominant columns
c ddomr = percentage of weakly diagonally dominant rows
c-------------------------------------------------------------------

c number of diagonally dominant columns
        nddomc = 0
c number of diagonally dominant rows
        nddomr = 0

	if (valued) then
        do i = 1, m
		aii = 0.0d0
       		dsumc = 0.0d0
          	do k = ia(i), ia(i+1)-1
		  if (ja(k) .eq. i) aii = abs(a(k))
       		  dsumc = dsumc + abs(a(k))
          	end do 

          	dsumr = 0.0d0
          	do k = iao(i), iao(i+1)-1
 		  dsumr = dsumr+abs(ao(k))
 	        end do

          	if (sym) then
             	  if (dsumr+dsumc .le. 3.0d0*aii) 
     *                       nddomc = nddomc + 1
             	else
             	  if (dsumc .le. 2.0d0*aii) nddomc = nddomc + 1
             	  if (dsumr .le. 2.0d0*aii) nddomr = nddomr + 1
          	endif
        end do
 	end if
 	if (sym) nddomr = nddomc

        ddomr = real(nddomr)/ real(m)
        ddomc = real(nddomc)/ real(n)

        return
	end


         subroutine frobnorm(m, n,sym,a,ja,ia,Fnorm)

         implicit real*8 (a-h, o-z)
         real*8 a(*),Fnorm
         integer m, n, ja(*),ia(m+1)
         logical sym

c--------------------------------------------------------------------------
c this routine computes the Frobenius norm of A.
c--------------------------------------------------------------------------
c on entry:
c-----------
c m      = integer row dimension of matrix
c n      = integer colum dimension of matrix
c a      = real array containing the nonzero elements of the matrix
c          the elements are stored by columns in order
c          (i.e. column i comes before column i+1, but the elements
c          within each column can be disordered).
c ja     = integer array containing the row indices of elements in a.
c ia     = integer array containing of length n+1 containing the
c          pointers to the beginning of the columns in arrays a and 
c          ja. It is assumed that ia(*)= 1 and ia(n+1)  = nnz +1.
c sym    = logical variable indicating whether or not the matrix is
c          symmetric.
c
c on return
c-----------
c Fnorm  = Frobenius norm of A.
c--------------------------------------------------------------------------

         Fdiag = 0.0
         Fnorm = 0.0

         do i = 1,m
            do k = ia(i), ia(i+1)-1
               j = ja(k)
               if (j .eq. i) then
                  Fdiag = Fdiag + a(k)**2
               else
                  Fnorm = Fnorm + a(k)**2
               endif
            enddo 
         enddo 

         if (sym) then
            Fnorm = 2*Fnorm +Fdiag
         else
            Fnorm = Fnorm + Fdiag
         endif

         Fnorm = sqrt(Fnorm)

         return
         end
c----------------------------------------------------------------------      
         subroutine ansym(m,n,sym,a,ja,ia,ao,jao,iao,imatch,
     *                   av,fas,fan)
c---------------------------------------------------------------------
c this routine computes the Frobenius norm of the symmetric and
c non-symmetric parts of A, computes number of matching elements
c in symmetry and relative symmetry match. 
c---------------------------------------------------------------------
c on entry:
c----------
c n   = integer column dimension of matrix
c a   = real array containing the nonzero elements of the matrix
c       the elements are stored by columns in order
c       (i.e. column i comes before column i+1, but the elements
c       within each column can be disordered).
c ja  = integer array containing the row indices of elements in a
c ia  = integer array containing of length n+1 containing the
c       pointers to the beginning of the columns in arrays a and ja.
c       It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c sym = logical variable indicating whether or not the matrix is
c       symmetric.
c on return
c----------
c fas   = Frobenius norm of symmetric part
c fan   = Frobenius norm of non-symmetric part
c imatch = number of matching elements in symmetry
c av     = relative symmetry match (symmetry = 1)
c ao,jao,iao = transpose of A just as a, ja, ia contains 
c              information of A.
c-----------------------------------------------------------------------
       implicit real*8 (a-h, o-z)
       real*8 a(*),ao(*),fas,fan,av, Fnorm, st, diff
       integer m, n, ja(*), ia(n+1), jao(*), iao(n+1), imatch, i
       logical sym

       if (sym) then
          fnorm = 0.0D0
          st = 0.0D0
          do i = 1, n
             do j1 = ia(i), ia(i+1)-1
                if (ja(j1).ne.i) then
                   fnorm = fnorm + a(j1)*a(j1)
                else
                   st = st + a(j1)*a(j1)
                endif
             enddo
          enddo
          fnorm = fnorm+st+st
          imatch = nnz
          fas = sqrt(fnorm)
          fan = 0.0d0
          goto 7
       endif
c
       st     = 0.0d0
       fas    = 0.0d0
       fan    = 0.0d0
       diff   = 0.0d0
       imatch = 0
       nnz    = ia(n+1)-ia(1)
c
c     need to transpose the matrix (two times to guarantee the rows
c     are ordered)
c
c       call csrcsc(m,n,1,1,a,ja,ia,ao,jao,iao)
c       call csrcsc(n,m,1,1,ao,jao,iao,a,ja,ia)
c
       do 6 i=1,n
          k1 = ia(i)
          k2 = iao(i)
          k1max = ia(i+1) - 1
          k2max = iao(i+1) - 1
c
 5        if (k1.le.k1max .and. k2.le.k2max) then
c
c     compare row i and column i as two ordered lists
c
             j1 = ja(k1)
             j2 = jao(k2)
             if (j1 .lt. j2) then
                diff = diff + a(k1)*a(k1)
                k1 = k1 + 1
             else if (j1 .gt. j2) then
                k2 = k2 + 1
             else
                fas = fas + (a(k1)+ao(k2))**2
                fan = fan + (a(k1)-ao(k2))**2
                st  = st  + a(k1)**2
                imatch = imatch + 1
                k1 = k1 + 1
                k2 = k2 + 1
             endif
             goto 5
          endif
c
          if (k1.le.k1max) then
c
c     clean in the extra elements in row i
c
             do j1 = k1, k1max
                diff = diff + a(j1)*a(j1)
             enddo
          endif
 6     continue
c
       fas = 0.25D0 * fas
       fan = 0.25D0 * fan
c
       if (imatch.eq.nnz) then
          st = 0.0D0
       else
          st = 0.5D0 * diff
          if (st.lt.0.0D0) st = 0.0D0
       endif
       fas = sqrt(fas + st)
       fan = sqrt(fan + st)
c
 7     av = dble(imatch)/dble(nnz)
       return
       end
c------end-of-ansym-----------------------------------------------------
c-----------------------------------------------------------------------
        subroutine distaij(m, n,nnz,sym,ja,ia,dist, std)
 
        implicit real*8 (a-h, o-z)
        real*8  dist, std
        integer m, n, ja(*), ia(m+1)
c-----------------------------------------------------------------------
c this routine computes the average distance of a(i,j) from diag and
c standard deviation  for this average.
c-----------------------------------------------------------------------
c On entry :
c-----------
c m     = integer. row dimension of matrix
c n     = integer. column dimension of matrix
c nnz   = number of nonzero elements of matrix
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length n+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c sym   = logical variable indicating whether or not the matrix is
c         symmetric.
c on return
c----------
c dist  = average distance of a(i,j) from diag.
c std   = standard deviation for above average.
c-----------------------------------------------------------------------
c
c distance of an element from diagonal.
c
       dist   = 0.0
       std = 0.0

        do 3 i=1,m
                j0 = ia(i)
                j1 = ia(i+1) - 1

          	do 31 k=j0, j1
                	 j=ja(k)
                	 dist = dist + real(iabs(j-i) )
 31             continue
 3      continue

          dist = dist/real(nnz)

          do 6 i = 1, n 
             do 61 k=ia(i), ia(i+1) - 1
                std=std+(dist-real(iabs(ja(k)-i)))**2
 61          continue
 6        continue

          std = sqrt(std/ real(nnz))

          return
	  end

	
	  subroutine skyline(n,sym,ja,ia,jao,iao,nsky)
	  implicit real*8 (a-h, o-z)
	  integer n, ja(*), ia(n+1), jao(*), iao(n+1)
	  integer nskyl, nskyu, nsky
	  logical sym
c-------------------------------------------------------------------
c Compute the number of nonzeros in the skyline storage.  Note that
c this only makes sense if the matrix is square.
c-------------------------------------------------------------------
c
c On entry :
c-----------
c n     = order of matrix
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length n+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c iao   = integer array containing of length n+1 containing the
c         pointers to the beginning of the rows in arrays ao and jao.
c         It is assumed that iao(*) = 1 and iao(n+1) = nzz+1.
c jao   = integer array containing the column indices of elements in ao.
c sym   = logical variable indicating whether or not the matrix is
c         symmetric.
c on return
c----------
c nsky  = number of nonzeros in skyline storage
c-------------------------------------------------------------------
c
c nskyu = skyline storage for upper part
          nskyu = 0
c nskyl = skyline storage for lower part
          nskyl = 0

          do 10 i =1,n
                j0 = ia(i)
                j0r = iao(i)
 
                jminc = ja(j0)
                jminr = jao(j0r)
                if (sym) jminc = jminr
 
                nskyl = nskyl + i-jminr + 1
                nskyu = nskyu + i-jminc + 1
 
 10   	  continue

          nsky = nskyl+nskyu-n
	  if (sym)  nsky = nskyl

          return
          end

            subroutine distdiag(nrow,ncol,ja,ia,dist)
            implicit real*8 (a-h, o-z)
            integer nrow,ncol,ja(*), ia(nrow+1),dist(*)

c----------------------------------------------------------------------
c this routine computes the numbers of elements in each diagonal. 
c----------------------------------------------------------------------
c On entry :
c-----------
c nrow  = integer. row dimension of matrix
c ncol  = integer. column dimension of matrix
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length n+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c on return
c----------
c dist  = integer array containing the numbers of elements in each of 
c         the nrow+ncol-1 diagonals of A. dist(k) contains the 
c         number of elements in diagonal '-nrow+k'.  k ranges from 
c         1 to (nrow+ncol-1).
c----------------------------------------------------------------------

            nnz  = ia(nrow+1)-ia(1)
            n2   = nrow+ncol-1
            do 8 i=1, n2
             dist(i) = 0
   8        continue
            do 9 i=1, nrow
                k1 = ia(i)
                k2 = ia(i+1) -1
                do 91 k=k1, k2
                   j = ja(k)
                   dist(nrow+j-i) = dist(nrow+j-i) +1
  91            continue
   9        continue
 
            return
            end


            subroutine bandpart(m,n,ja,ia,dist,nper,band)
            implicit real*8 (a-h, o-z)
            integer n,ja(*), ia(n+1),dist(*)
            integer nper,band
 
c-------------------------------------------------------------------------
c this routine computes the bandwidth of the banded matrix, which contains
c 'nper' percent of the original matrix.
c-------------------------------------------------------------------------
c On entry :
c-----------
c n     = integer. column dimension of matrix
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length n+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c dist  = integer array containing the numbers of elements in the 
c         matrix with different distance of row indices and column 
c         indices.
c nper  = percentage of matrix  within the bandwidth
c on return
c----------
c band  = the width of the bandwidth
c----------------------------------------------------------------------

            nnz  = ia(m+1)-ia(1)
            iacc  = dist(n)
            band  = 0
            j     = 0

  10        j     = j+1
            iacc  = iacc + dist(n+j) +dist(n-j) 
            if (iacc*100 .le. nnz*nper) then 
                band = band +1 
                goto 10         
            endif 
          
            return 
            end 
 
         subroutine  n_imp_diag(m,n,nnz,dist, ipar1,ndiag,ioff,dcount)
         implicit real*8 (a-h, o-z)
         real*8  dcount(*)
         integer m, n,nnz, dist(*), ndiag, ioff(*), ipar1

c-----------------------------------------------------------------------
c this routine computes the most important diagonals.
c-----------------------------------------------------------------------
c
c On entry :
c-----------
c m     = integer. row dimension of matrix
c n     = integer. column dimension of matrix
c nnz   = number of nonzero elements of matrix
c dist  = integer array containing the numbers of elements in the 
c         matrix with different distance of row indices and column 
c         indices. ipar1 = percentage of nonzero  elements of A that 
c         a diagonal should have in order to be an important diagonal 
c on return
c----------
c ndiag = number of the most important diagonals
c ioff  = the offsets with respect to the main diagonal
c dcount= the accumulated percentages
c-----------------------------------------------------------------------
          n2 = m+n-1
          ndiag = 10
          ndiag = min0(n2,ndiag)
          itot  = 0
          ii = 0
          idiag = 0
c     sort diagonals by decreasing order of weights.
 40       jmax = 0
          i    = 1
          do 41 k=1, n2
             j = dist(k)
             if (j .lt. jmax) goto 41
             i = k
             jmax = j
 41       continue
c     permute ----
c     save offsets and accumulated count if diagonal is acceptable
c     (if it has at least ipar1*nnz/100 nonzero elements)
c     quite if no more acceptable diagonals --
c
c          if (jmax*100 .lt. ipar1*nnz) goto 4
          ii = ii+1
          ioff(ii) = i - m
          dist(i)   = - jmax
          itot = itot + jmax
          dcount(ii) = real(100*itot)/real(nnz)
          if (ii .lt. ndiag) goto 40
 4        continue
          ndiag = ii
c-------
          t = real (icount) / real (nnz)

          return
          end

	subroutine nonz_lud(m, n,a,ja,ia,nlower, nupper, ndiag, unzero)
        implicit real*8 (a-h, o-z)
        integer m, n,  ja(*), ia(m+1)
        integer nlower, nupper, ndiag, unzero
        real*8 a(*)

c---------------------------------------------------------------------
c this routine computes the number of nonzero elements in strict lower
c part, strict upper part, and main diagonal.
c---------------------------------------------------------------------   
c
c On entry :
c-----------
c m     = integer. row dimension of matrix
c n     = integer. column dimension of matrix
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length n+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c on return
c----------
c nlower= number of nonzero elements in strict lower part
c nupper= number of nonzero elements in strict upper part
c ndiag = number of nonzero elements in main diagonal
c------------------------------------------------------------------- 
	
        nupper = 0
        ndiag = 0
        unzero = 0
       
        do i = 1, m
          do k = ia(i), ia(i+1)-1
            j = ja(k)
            if (j .lt. i) nupper = nupper+1
            if (j .eq. i .and. a(k).ne. 0.0d0) ndiag = ndiag + 1 
            if (a(k).eq. 0.0d0) unzero = unzero + 1
          end do
        end do

          nlower = ia(m+1)-1-nupper-ndiag

        return
	end

        subroutine avnz_row(m, n,ja,ia,iao, ndiag, av, st)
        implicit real*8 (a-h, o-z)
        real*8 av, st
        integer m, n,  ja(*), ia(m+1), iao(m+1)

c---------------------------------------------------------------------
c this routine computes average number of nonzero elements/row and
c standard deviation for this average
c---------------------------------------------------------------------
c
c On entry :
c-----------
c m     = integer. row dimension of matrix
c n     = integer. column dimension of matrix
c ja    = integer array containing the row indices of elements in a
c ia    = integer array containing of length n+1 containing the
c         pointers to the beginning of the columns in arrays a and ja.
c         It is assumed that ia(*) = 1 and ia(n+1) = nzz+1.
c ndiag = number of the most important diagonals
c On return
c----------
c av    = average number of nonzero elements/column
c st    = standard deviation for this average
c Notes
c---------
c standard deviation will not be correct for symmetric storage. 
c----------------------------------------------------------------------


c standard deviatioan for the average
        st = 0.0d0

c average and standard deviation
c
        av = real(ia(m+1)-1)/real(m)
c
c will be corrected later.
c

        do i = 1,m
          j0 = ia(i)
          j1 = ia(i+1) - 1

          do k = j0, j1
                j = ja(k)
          end do
          lenc = j1+1-j0
          st = st + (real(lenc) - av)**2
        end do
        st = sqrt( st / real(m) )

        return
        end
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
       subroutine vbrinfo(nr, nc, kvstr, kvstc, ia, ja, ka, iwk, iout)
c-----------------------------------------------------------------------
      integer nr, nc, kvstr(nr+1), kvstc(nc+1), ia(nr+1), ja(*), ka(*)
      integer iwk(nr+nc+2+nr), iout
c-----------------------------------------------------------------------
c     Print info on matrix in variable block row format.
c-----------------------------------------------------------------------
c     On entry:
c--------------
c     nr,nc   = matrix block row and block column dimension
c     kvstr   = first row number for each block row
c     kvstc   = first column number for each block column
c     ia,ja,ka,a = input matrix in VBR format
c     iout    = unit number for printed output
c
c     On return:
c---------------
c     Printed output to unit number specified in iout.  If a non-square
c     matrix is provided, the analysis will be performed on the block
c     rows, otherwise a row/column conformal partitioning will be used.
c
c     Work space:
c----------------
c     iwk(1:nb+1) = conformal block partitioning
c        (nb is unknown at start but is no more than nr+nc)
c     iwk(nb+2:nb+2+nr) = frequency of each blocksize
c     The workspace is not assumed to be initialized to zero, nor is it
c     left that way.
c
c-----------------------------------------------------------------------
c-----local variables
      integer n, nb, nnz, nnzb, i, j, neq, max, num
      character*101 tmpst
      integer bsiz(10), freq(10)
c-----------------------------------------------------------------------

      n = kvstr(nr+1)-1
      nnz = ka(ia(nr+1)) - ka(1)
      nnzb = ia(nr+1) - ia(1)
      write (iout, 96)
      write (iout, 100) n, nnz, real(nnz)/real(n)
      write (iout, 101) nr, nnzb, real(nnzb)/real(nr)

c-----if non-square matrix, do analysis on block rows,
c     else do analysis on conformal partitioning
      if (kvstr(nr+1) .ne. kvstc(nc+1)) then
         write (iout, 103)
         do i = 1, nr+1
            iwk(i) = kvstr(i)
         enddo
         nb = nr
      else
         call kvstmerge(nr, kvstr, nc, kvstc, nb, iwk)
         if ((nr .ne. nc) .or. (nc .ne. nb)) write (iout, 104) nb
      endif

c-----accumulate frequencies of each blocksize
      max = 1
      iwk(1+nb+2) = 0
      do i = 1, nb
         neq = iwk(i+1) - iwk(i)
         if (neq .gt. max) then
            do j = max+1, neq
               iwk(j+nb+2) = 0
            enddo
            max = neq
         endif
         iwk(neq+nb+2) = iwk(neq+nb+2) + 1
      enddo

c-----store largest 10 of these blocksizes
      num = 0
      do i = max, 1, -1
         if ((iwk(i+nb+2) .ne. 0) .and. (num .lt. 10)) then
            num = num + 1
            bsiz(num) = i
            freq(num) = iwk(i+nb+2)
         endif
      enddo

c-----print information about blocksizes
      write (iout, 109) num
      write (tmpst,'(10i6)') (bsiz(j),j=1,num)
      write (iout,110) num,tmpst
      write (tmpst,'(10i6)') (freq(j),j=1,num)
      write (iout,111) tmpst
      write (iout, 96)

c-----------------------------------------------------------------------
 99    format (2x,38(2h *))
 96    format (6x,' *',65(1h-),'*')

 100   format(
     * 6x,' *  Number of rows                                   = ',
     * i10,'  *'/
     * 6x,' *  Number of nonzero elements                       = ',
     * i10,'  *'/
     * 6x,' *  Average number of nonzero elements/Row           = ',
     * f10.4,'  *')

 101   format(
     * 6x,' *  Number of block rows                             = ',
     * i10,'  *'/
     * 6x,' *  Number of nonzero blocks                         = ',
     * i10,'  *'/
     * 6x,' *  Average number of nonzero blocks/Block row       = ',
     * f10.4,'  *')

 103   format(
     * 6x,' *  Non-square matrix.                                 ',
     *     '            *'/
     * 6x,' *  Performing analysis on block rows.                 ',
     *     '            *')

 104   format(
     * 6x,' *  Non row-column conformal partitioning supplied.    ',
     *     '            *'/
     * 6x,' *  Using conformal partitioning.  Number of bl rows = ',
     * i10,'  *')

 109   format(
     * 6x,' *  Number of different blocksizes                   = ',
     * i10,'  *')

 110   format(6x,' *  The ', i2, ' largest dimension nodes',
     *     ' have dimension    : ',10x,'  *',/,
     * 6x,' *',a61,3x,' *')

 111   format(6x,' *  The frequency of nodes these ',
     *     'dimensions are      : ',10x,'  *',/,
     * 6x,' *',a61,3x,' *')
c---------------------------------
      return
      end
c-----------------------------------------------------------------------
c-----------------------end-of-vbrinfo----------------------------------
      subroutine blkfnd (nrow,ncol,ja,ia,nblk)
c-----------------------------------------------------------------------
c This routine attemptps to determine whether or not  the input
c matrix has a block structure and finds the blocks size
c if it does. A block matrix is one which is
c comprised of small square dense blocks. If there are zero
c elements within the square blocks and the original data structure
c takes these zeros into account then blkchk may fail to find the
c correct block size. 
c----------------------------------------------------------------------- 
c on entry
c---------
c nrow	= integer equal to the row dimension of the matrix.  
c ncol  = integer equal to the col dimension of the matrix.  
c ja    = integer array containing the column indices of the entries 
c         nonzero entries of the matrix stored by row.
c ia    = integer array of length nrow + 1 containing the pointers 
c         beginning of each row in array ja.
c		
c nblk  = integer containing the assumed value of nblk if job = 0
c         
c on return
c---------- 
c nblk  = integer containing the value found for nblk when job = 1.
c         if imsg .ne. 0 this value is meaningless however.
c
c----------------------------------------------------------------------c
c           Y. Saad, Sep. 21 1989                                      c
c----------------------------------------------------------------------c
      integer ia(nrow+1),ja(*) 
c----------------------------------------------------------------------- 
c first part of code will find candidate block sizes.
c criterion used here is a simple one: scan rows and  determine groups 
c of rows that have the same length and such that the first column 
c number and the last column number are identical. 
c----------------------------------------------------------------------- 
      minlen = ia(2)-ia(1)
      irow   = 1
      do 1 i = 2,nrow
         len = ia(i+1)-ia(i)
         if (len .lt. minlen) then
            minlen = len 
            irow = i
         endif
 1    continue
c     
c     ---- candidates are all dividers of minlen
c
      nblk = 1
      if (minlen .le. 1) return
c     
      do 99 iblk = minlen, 1, -1
         if (mod(minlen,iblk) .ne. 0) goto 99
         len = ia(2) - ia(1)
         len0 = len
         jfirst = ja(1) 
         jlast = ja(ia(2)-1)
         do 10 jrow = irow+1,irow+nblk-1
            i1 = ia(jrow)
            i2 = ia(jrow+1)-1
            len = i2+1-i1
            jf = ja(i1)
            jl = ja(i2) 
            if (len .ne. len0 .or. jf .ne. jfirst .or. 
     *           jl .ne. jlast) goto 99
 10      continue
c     
c     check for this candidate ----
c     
         call blkchk (nrow,ncol,ja,ia,iblk,imsg)	       
         if (imsg .eq. 0) then 
c     
c     block size found
c     
            nblk = iblk
            return
         endif 
 99   continue	       
c--------end-of-blkfnd ------------------------------------------------- 
c-----------------------------------------------------------------------
      end
c-----------------------------------------------------------------------
      subroutine blkchk (nrow,ncol,ja,ia,nblk,imsg)
c----------------------------------------------------------------------- 
c This routine checks whether the input matrix is a block
c matrix with block size of nblk. A block matrix is one which is
c comprised of small square dense blocks. If there are zero
c elements within the square blocks and the data structure
c takes them into account then blkchk may fail to find the
c correct block size. 
c----------------------------------------------------------------------- 
c on entry
c---------
c nrow	= integer equal to the row dimension of the matrix.  
c ncol  = integer equal to the col dimension of the matrix.  
c ja    = integer array containing the column indices of the entries 
c         nonzero entries of the matrix stored by row.
c ia    = integer array of length nrow + 1 containing the pointers 
c         beginning of each row in array ja.
c
c nblk  = integer containing the value of nblk to be checked. 
c         
c on return
c---------- 
c
c imsg  = integer containing a message  with the following meaning.
c          imsg = 0 means that the output value of nblk is a correct
c                   block size. nblk .lt. 0 means nblk not correct 
c                   block size.
c          imsg = -1 : nblk does not divide nrow 
c          imsg = -2 : a starting element in a row is at wrong position
c             (j .ne. mult*nblk +1 )
c          imsg = -3 : nblk does divide a row length - 
c          imsg = -4 : an element is isolated outside a block or
c             two rows in same group have different lengths
c----------------------------------------------------------------------c
c           Y. Saad, Sep. 21 1989                                      c
c----------------------------------------------------------------------c
      integer ia(nrow+1),ja(*) 
c---------------------------------------------------------------------- 
c first part of code will find candidate block sizes.
c this is not guaranteed to work . so a check is done at the end
c the criterion used here is a simple one:
c scan rows and determine groups of rows that have the same length
c and such that the first column number and the last column number 
c are identical. 
c---------------------------------------------------------------------- 
      imsg = 0
      if (nblk .le. 1) return
      nr = nrow/nblk
      if (nr*nblk .ne. nrow) goto 101
c--   main loop --------------------------------------------------------- 
      irow = 1
      do 20 ii=1, nr
c     i1= starting position for group of nblk rows in original matrix
         i1 = ia(irow)
         j2 = i1
c     lena = length of each row in that group  in the original matrix
         lena = ia(irow+1)-i1
c     len = length of each block-row in that group in the output matrix
         len = lena/nblk
         if (len* nblk .ne. lena) goto 103
c     
c     for each row
c     
         do 6 i = 1, nblk
            irow = irow + 1
            if (ia(irow)-ia(irow-1) .ne. lena ) goto 104
c     
c     for each block
c     
            do 7 k=0, len-1
               jstart = ja(i1+nblk*k)-1
               if ( (jstart/nblk)*nblk .ne. jstart) goto 102
c     
c     for each column
c     
               do 5 j=1, nblk
                  if (jstart+j .ne. ja(j2) )  goto 104
                  j2 = j2+1 
 5             continue
 7          continue
 6       continue
 20   continue
c     went through all loops successfully:
      return
 101  imsg = -1
      return
 102  imsg = -2
      return
 103  imsg = -3
      return
 104  imsg = -4
c----------------end of chkblk ----------------------------------------- 
c-----------------------------------------------------------------------
      return
      end
c-----------------------------------------------------------------------
      subroutine kvstmerge(nr, kvstr, nc, kvstc, n, kvst)
c-----------------------------------------------------------------------
      integer nr, kvstr(nr+1), nc, kvstc(nc+1), n, kvst(*)
c-----------------------------------------------------------------------
c     Merges block partitionings, for conformal row/col pattern.
c-----------------------------------------------------------------------
c     On entry:
c--------------
c     nr,nc   = matrix block row and block column dimension
c     kvstr   = first row number for each block row
c     kvstc   = first column number for each block column
c
c     On return:
c---------------
c     n       = conformal row/col matrix block dimension
c     kvst    = conformal row/col block partitioning
c
c     Notes:
c-----------
c     If matrix is not square, this routine returns without warning.
c
c-----------------------------------------------------------------------
c-----local variables
      integer i,j
c---------------------------------

      if (kvstr(nr+1) .ne. kvstc(nc+1)) return

      i = 1
      j = 1
      n = 1
  200 if (i .gt. nr+1) then
         kvst(n) = kvstc(j)
         j = j + 1
      elseif (j .gt. nc+1) then
         kvst(n) = kvstr(i)
         i = i + 1
      elseif (kvstc(j) .eq. kvstr(i)) then
         kvst(n) = kvstc(j)
         j = j + 1
         i = i + 1
      elseif (kvstc(j) .lt. kvstr(i)) then
         kvst(n) = kvstc(j)
         j = j + 1
      else
         kvst(n) = kvstr(i)
         i = i + 1
      endif
      n = n + 1
      if (i.le.nr+1 .or. j.le.nc+1) goto 200
      n = n - 2
c---------------------------------
      return
      end
c-----------------------------------------------------------------------
c------------------------end-of-kvstmerge-------------------------------
      subroutine csrcsc (m, n,job,ipos,a,ja,ia,ao,jao,iao)
      integer m, n, job, ipos
      integer ia(n+1),iao(n+1),ja(*),jao(*)
      real*8  a(*),ao(*)
c-----------------------------------------------------------------------
c Compressed Sparse Row     to      Compressed Sparse Column
c
c (transposition operation)   Not in place. 
c----------------------------------------------------------------------- 
c -- not in place --
c this subroutine transposes a matrix stored in a, ja, ia format.
c ---------------
c on entry:
c----------
c n	= dimension of A.
c job	= integer to indicate whether to fill the values (job.eq.1) of the
c         matrix ao or only the pattern., i.e.,ia, and ja (job .ne.1)
c
c ipos  = starting position in ao, jao of the transposed matrix.
c         the iao array takes this into account (thus iao(1) is set to ipos.)
c         Note: this may be useful if one needs to append the data structure
c         of the transpose to that of A. In this case use for example
c                call csrcsc (n,1,n+2,a,ja,ia,a,ja,ia(n+2)) 
c	  for any other normal usage, enter ipos=1.
c a	= real array of length nnz (nnz=number of nonzero elements in input 
c         matrix) containing the nonzero elements.
c ja	= integer array of length nnz containing the column positions
c 	  of the corresponding elements in a.
c ia	= integer of size n+1. ia(k) contains the position in a, ja of
c	  the beginning of the k-th row.
c
c on return:
c ---------- 
c output arguments:
c ao	= real array of size nzz containing the "a" part of the transpose
c jao	= integer array of size nnz containing the column indices.
c iao	= integer array of size n+1 containing the "ia" index array of
c	  the transpose. 
c
c----------------------------------------------------------------------- 
c----------------- compute lengths of rows of transp(A) ----------------
      do i = 1,n+1
         iao(i) = 0
      end do

      do i = 1, m
         do k = ia(i), ia(i+1)-1 
            j = ja(k)+1
            iao(j) = iao(j)+1
         end do
      end do
c---------- compute pointers from lengths ------------------------------
      iao(1) = ipos 
      do 4 i = 1, n
         iao(i+1) = iao(i) + iao(i+1)
 4    continue
c--------------- now do the actual copying ----------------------------- 
      do i = 1,m
         do k = ia(i), ia(i+1)-1 
            j = ja(k) 
            next = iao(j)
            if (job .eq. 1)  ao(next) = a(k)
            jao(next) = i
            iao(j) = next+1
         end do
      end do
c-------------------------- reshift iao and leave ---------------------- 
      do i = n,1,-1
         iao(i+1) = iao(i)
      end do
      iao(1) = ipos
c--------------- end of csrcsc ----------------------------------------- 
c-----------------------------------------------------------------------
      end
