/*
// --------------------------------------------------------------------------
// The domain is the unit square, with M+1 x N+1 mesh lines.
// The mesh lines corresponding to
// 	x = 0
// 	x = M+1
// 	y = 0
// 	y = N+1
// are Dirichlet boundary conditions, so
// the total number of unknowns is (M-1)*(N-1).
//
// The mesh is assumed to be M x N, and the lines corresponding to
// x = 0, x = M, y = 0, and y = N are boundary conditions from
// the function BC().  So for M = 5 and N = 4, the mesh looks like
//
//             B   B   B   B   B   B
//             B   8   9   10  11  B
//             B   4   5   6   7   B
//             B   0   1   2   3   B
//             B   B   B   B   B   B
//
// where B = boundary condition node and the numbered nodes are the
// indexing of the unknowns.  Furthermore, the boundary condition vector
// boundary[] returns the values in the boundary nodes, with the indexing
//
//             6    7    8    9    10   11
//             14   X    X    X    X    17
//             13   X    X    X    X    16
//             12   X    X    X    X    15
//             0    1    2    3    4    5
// --------------------------------------------------------------------------
*/

#ifndef GEOMETRY_H
#define GEOMETRY_H

class HPCxx_Buffer;

class Geometry {

	friend void hpcxx_pack(HPCxx_Buffer& buffer, Geometry *g, int count);
	friend void hpcxx_unpack(HPCxx_Buffer& buffer, Geometry *g, int count);

 public:

    int M;         // Number of points in x-direction
    int N;         // Number of points in y-direction
    int size;      // Number of points in the geometry
    double *x;     // Vector of mesh x coordinates, length M+1
    double *y;     // Vector of mesh y coordinates, length N+1

    double *left;   // Boundary vector of length N-1
    double *right;  // Boundary vector of length N-1
    double *top;    // Boundary vector of length M+1
    double *bottom; // Boundary vector of length M+1

 public:

	Geometry();
	Geometry(const Geometry &g);
	void operator=(const Geometry &g);
	~Geometry();

 private:

	void free();
	void copy(const Geometry &g);
};

#endif
