/*
//  --------------------------------------------------------------------------------
//  PutSparseSystem is to be replaced by the uses port code for output of
//  a sparse linear system.  Right now, it writes to a file by a
//  call to the Fortran routine prtmt.  A more readable output format for
//  small systems, called dumpmt_(), also available.
//
//  The stuff here is the same as it is for GetSparseSystem, to wit:
//  
//    titlelen: length of matrix title including the null terminator
//              (so, titlelen = ::strlen(title) + 1).
//    title: pointer to the title string ( null terminated ).
//  
//    keylen: length of matrix key including the null terminator.
//    key:  pointer to the key string ( null terminated ).
//  
//    n: order of matrix. Later we may need to change this to allow nonsquare
//       matrices, but for now the numbers of rows and columns are always the
//  	 same.
//  
//    nnz: number of non-zeros in the sparse matrix data structure.  More
//  	   precisely, the number of stored entries - since some zeros may be
//  	   explicitly stored in some applications.  We use standard terminology,
//  	   however, and call the stored entries "nonzeros".
//  
//    colinds: pointer to the column indices of the matrix nonzeros.  This
//             array must be at least of length nnz, and uses 1-based indexing.
//  
//    rowptrs: pointer to the row pointers. This array
//             must be of length at least n+1, and uses 1-based indexing.
//  
//    values:  pointer to the matrix values, of length at least nnz.
//  
//    rhs: pointer to the values of the right-hand side vector. This array
//         must be of at least length n.
//  
//    nparts:  for matrices partitioned for shared memory machines, this
//             indicates how many "block rows" are present.  
//    bstarts: array of pointers to the first row in each block row.  So,
//             for example, rows bstarts[k] to bstarts[k+1]-1 comprise the
//             k-th block row of the matrix A, where 0 < k < nparts+1.
//             This array must be of length at least nparts+1.
//             
//    Mungings: a class with information needed to retrieve the solution
//              to a sparse system that has undergone various reorderings and
//              scalings.  This should be empty for reading from a file, but
//              in general will be filled on getting a SLS from another component
//
//  nparts, bstarts, and Mungings are not output at this time, however.
//  
//  
//  Randall Bramley
//  Department of Computer Science
//  Indiana University, Bloomington
//  bramley@cs.indiana.edu                             
//  Wed Oct 27 16:35:18 EST 1999
//  --------------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <errno.h>
#include <iostream.h>
#include <assert.h>
#include <mmio.h>
#include <PutSparseSystem.h>
#include <SparseLinearSystem.h>

#ifdef REMOVE_
#define prtmt_ prtmt
#define dumpmt_ dumpmt
#endif

// ------------------
// Fortran functions:
// ------------------
extern "C" {
void prtmt_(int *nrow,
	    int *ncol,
	    double *A,
	    int *colinds,
	    int *rowptrs,
	    double *b,
	    char guesol[4],
	    char title[80],
	    char key[16],
	    char type[8],
	    int *ifmt,
	    int *job,
	    int *iounit);

};


int PutSparseSystem(SparseLinearSystem* sls){

  int iounit = 9;
  // ifmt indicates the number of significant digits to write
  // out.  Reduce this for smaller files; increase to 17 for full
  // double precision accuracy.
  int ifmt = 6;
  int job = 3; // Write out the right hand side vector as well as A.
  char guesol[4];
  char type[8];

  strncpy(type,"rua",4);
  guesol[3] = '\0';
  
  strncpy(guesol," ",2);
  guesol[2] = '\0';

  cout << "About to call prtmt ...";

  prtmt_(&(sls->n),
	 &(sls->n),
	 sls->values,
	 sls->colinds,
	 sls->rowptrs,
	 sls->rhs,
	 guesol,
	 sls->title,
	 sls->key,
	 type,
	 &ifmt,
	 &job,
	 &iounit);
  cout << " ... prtmt called" << endl;

  return 0;

}
