

/*-----------------------*/
/* Function prototypes:  */
/*-----------------------*/

    /* --------------------------------- */
    /* Dependent on problem PDE to solve */
    /* --------------------------------- */
    double afun(double x, double y, double time);    /* u_xx coefficient */
    double bfun(double x, double y, double time);    /* u_yy coefficient */
    double cfun(double x, double y, double time);    /* u_x  coefficient */
    double dfun(double x, double y, double time);    /* u_y  coefficient */
    double efun(double x, double y, double time);    /* u    coefficient */
    double ffun(double x, double y, double time);    /* forcing function */

/* ------------------------------------------------------- */
/* Function that computes and returns the stencil operator */
/* for five-pt centered differences. Should only be called */
/* for points in the interior of the rectangular domain    */
/* ------------------------------------------------------- */
void getsten(int i, int j, int M, int N, double time, double stencil[6])
{

 double x, y, ratio, h, rsq;

 double a, b, c, d;

 x = (double) i / (double) M;
 y = (double) j / (double) N;

 h = (double) 1 / (double) M;
 ratio = (double) N / (double) M;
 rsq = ratio*ratio;

 a = afun(x,y,time);
 b = bfun(x,y,time);
 c = cfun(x,y,time);
 d = dfun(x,y,time);

 stencil[0] = 2*a + 2*rsq*b- efun(x,y,time)*h*h;
 stencil[1] = c*h/2 - a;
 stencil[2] = d*h*ratio/2 - rsq*b;
 stencil[3] = -(c*h/2 + a);
 stencil[4] = -(d*h*ratio/2 + rsq*b);
 stencil[5] = -h*h*ffun(x,y,time);

}
