c======================================================================== 
      subroutine rowinc(num,q,adata)
      implicit none
c======================================================================== 
c
c  Implement shell sort, with hardwired increments.  The algorithm for
c  sorting entries in q(0:n-1) is as follows:
c----------------------------------------------------------------
c  inc = initialinc(n)
c  while inc >= 1
c     for i = inc to n-1
c         j = i
c         x = q(i)
c         while j >= inc and q(j-inc) > x
c            q(j) = q(j-inc)
c            j    = j-inc
c         end while 
c         q(j) = x
c     end for
c     inc = nextinc(inc,n)
c  end while
c----------------------------------------------------------------
c
c  However, the code implemented here is for q(1:n), in keeping with
c  the Fortran convention of indexing from 1.
c  The increments used are 1, 4, 13, 40, 121, ..., (3**i - 1)/2, ...
c  In this case, nextinc(inc,n) = (inc-1)/3.  Usually shellsort
c  would have the largest increment the largest integer of the form
c  (3**i - 1)/2 that is less than n, but here it is fixed at 121
c  because most sparse matrices have 121 or fewer nonzero entries
c  per row.  If this routine is expanded for a complete sparse
c  factorization routine, or if a large number of levels of fill is
c  allowed, then possibly it should be replaced with more efficient
c  sorting.  This routine differs from srtr in symbfac because it
c  also sorts the data field given by adata(*).
c
c     Randall Bramley
c     Department of Computer Science
c     Indiana University
c     email: bramley@cs.indiana.edu
c     Tue May 17 11:37:38 EST 1994
c
c======================================================================== 
c
      integer num, q(num)
      real*8 adata(num), t
c
      integer iinc(5), key, icn, ih, ii, i, j, jj
      data iinc/1,4,13,40,121/
c
      if   (num .eq. 0) then
            icn   =  0
      elseif   (num .lt. 14) then
                  icn   =  1
            elseif   (num .lt. 41) then
                  icn   =  2
                  elseif   (num .lt. 122) then
                              icn   =  3
                        elseif   (num .lt. 365) then
                                    icn   =  4
                              else
                                    icn   =  5
      end if
      do 40 ii = 1, icn
            ih = iinc(icn + 1 - ii)
            do 30  j = ih+1, num
                  i = j-ih
                  key = q(j)
                  t   = adata(j)
                  do 10 jj = 1, j-ih, ih
                        if (key .ge. q(i)) then
                              go to 20
                        else
                              q(i + ih)     = q(i)
                              adata(i + ih) = adata(i)
                              i             = i - ih
                        end if
 10               continue
 20               continue
                  q(i + ih) = key
                  adata(i + ih) = t
 30         continue
 40   continue
c
      return
      end
