/*
//  --------------------------------------------------------------------------------
//  GetSparseSystem here differs from that in most components, in that it
//  takes an argument giving a H/B or Matrix Market file name.  For the MM
//  files, no right hand side file name is taken - instead, one is generated
//  artificially.  That can be added later as another argument to the main
//  function below.  Otherwise, the variables mean the same as in the
//  GetSparseSystem() with no args.
//  
//    titlelen: length of matrix title including the null terminator
//              (so, titlelen = ::strlen(title) + 1).
//    title: pointer to the title string ( null terminated ).
//  
//    keylen: length of matrix key including the null terminator.
//    key:  pointer to the key string ( null terminated ).
//  
//    n: order of matrix. Later we may need to change this to allow nonsquare
//       matrices, but for now the numbers of rows and columns are always the
//  	 same.
//  
//    nnz: number of non-zeros in the sparse matrix data structure.  More
//  	   precisely, the number of stored entries - since some zeros may be
//  	   explicitly stored in some applications.  We use standard terminology,
//  	   however, and call the stored entries "nonzeros".
//  
//    colinds: pointer to the column indices of the matrix nonzeros.  This
//             array must be at least of length nnz, and uses 1-based indexing.
//  
//    rowptrs: pointer to the row pointers. This array
//             must be of length at least n+1, and uses 1-based indexing.
//  
//    values:  pointer to the matrix values, of length at least nnz.
//  
//    rhs: pointer to the values of the right-hand side vector. This array
//         must be of at least length n.
//  
//    nparts:  for matrices partitioned for shared memory machines, this
//             indicates how many "block rows" are present.  
//    bstarts: array of pointers to the first row in each block row.  So,
//             for example, rows bstarts[k] to bstarts[k+1]-1 comprise the
//             k-th block row of the matrix A, where 0 < k < nparts+1.
//             This array must be of length at least nparts+1.
//             
//    Mungings: a class with information needed to retrieve the solution
//              to a sparse system that has undergone various reorderings and
//              scalings.  This should be empty for reading from a file, but
//              in general will be filled on getting a SLS from another component
//  
//  
//  Randall Bramley
//  Department of Computer Science
//  Indiana University, Bloomington
//  bramley@cs.indiana.edu                             
//  Thu Oct 28 08:21:35 EST 1999
//  --------------------------------------------------------------------------------
*/

#include <SparseLinearSystem.h>

SparseLinearSystem* GetSparseSystem(char *MatFileName);

