/*  
//  ---------------------------------------------------------------------------
//
//  Test driver for a GetSystemFromFile component.
//  NewSystem differs from all other components in that
//  it reads in a sparse linear system from a Harwell/Boeing
//  or Matrix Market file, rather than getting it from
//  an input port.
//  
//
//  Randall Bramley
//  Department of Computer Science
//  Indiana University, Bloomington
//  bramley@cs.indiana.edu                             
//  Wed Oct 27 14:30:56 EST 1999
//  ---------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <errno.h>
#include <iostream.h>
#include <assert.h>
#include <SparseLinearSystem.h>
#include <GetSparseSystem.h>
#include <PutSparseSystem.h>
#include <FreeStorage.h>
#include <KeepOnTruckin.h>
#include <GetSystemFromFileParms.h>

int main(int argc, char *argv[]) {

  if (argc < 2) {
	cout << "Need to provide a H/B matrix file name to "
             << argv[0] << endl;
	return (-1);
  }

  int k = 0;

  while (KeepOnTruckin(k)){

      GetSystemFromFileParms p = getParameters(argv[1]);

      SparseLinearSystem *sls;

      cout << "Inside of " << argv[0] << " gonna call GetSparseSystem" 
	   << " to read matrix file " << argv[1] << endl;

      // --------------------------------------------------
      // Replace the following with a provides port access.
      // Now it just calls readmt_(), a Fortran H/B reader.
      // --------------------------------------------------

      sls = GetSparseSystem(p.HBFileName);

      // -------------------------------------------------------
      // Now call PutSparseSystem to write out the matrix again.
      // -------------------------------------------------------

      int errflag = PutSparseSystem(sls);

      FreeStorage(sls);

      k++;

  }


  return 0;
}
