#include "PutSolutionToFileComponent.h"
#include "../../../../framework/idl/solutionVector/SolutionVector_idl.h"
#include "../../../../framework/ccacore/DefaultPortInfo.h"
#include "../../../../framework/util/macros.h"
#include "SolutionVector_impl.h"
#include "PutSolutionToFileParms_impl.h"
#include "PutSolutionToFileParms.h"



PutSolutionToFileComponent::PutSolutionToFileComponent() {

    int ec = -1;

    m_core = NULL;
    m_solutionVectorPort = new SolutionVector_impl(this);
    m_putSolutionToFileParmsPort = new PutSolutionToFileParms_impl(this);
    m_computing = false;

    // XXX
    m_sv_cache = NULL;
    m_parms_cache = new PutSolutionToFileParms;

    /*
     * Init mutex
     */

    pthread_mutexattr_t attr;

    pthread_mutexattr_init(&attr);

    #if defined(DEBUG) && !defined(__linux__)
        ec = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
        assert(ec == 0);
    #endif

    pthread_mutex_init(&m_lock, &attr);

    pthread_mutexattr_destroy(&attr);

    /*
     * Init condition variable.
     */

    pthread_cond_init(&m_cv, NULL);

    /*
     * Init pthread attr
     */

    ec = pthread_attr_init(&m_threadAttr);
    assert(ec == 0);
    ec = pthread_attr_setdetachstate(&m_threadAttr, PTHREAD_CREATE_DETACHED);
    assert(ec == 0);
    //ec = pthread_attr_setscope(&m_threadAttr, PTHREAD_SCOPE_SYSTEM);
    //assert(ec == 0);
}

PutSolutionToFileComponent::~PutSolutionToFileComponent() {

    int ec = -1;

    if (m_core != NULL) {
        delete m_core;
    }

    DELETE(m_solutionVectorPort);
    DELETE(m_putSolutionToFileParmsPort);

    ec = pthread_attr_destroy(&m_threadAttr);
    assert(ec == 0);

    ec = pthread_cond_destroy(&m_cv);
    assert(ec == 0);

    ec = pthread_mutex_destroy(&m_lock);
    assert(ec == 0);
}

void
PutSolutionToFileComponent::setServices(Services *cc) {

    if (cc == NULL) {

        DELETE(m_core);
        return;

    } else {

        m_core = cc;

        m_core->addProvidesPort(m_solutionVectorPort,
         new DefaultPortInfo("inputSV", "SolutionVector_idl"));
        m_core->addProvidesPort(m_putSolutionToFileParmsPort,
         new DefaultPortInfo("inputParms", "PutSolutionToFileParms_idl"));
    }
}

struct Info {
    Info(PutSolutionToFileComponent *p_comp, SolutionVector *p_sv,
     PutSolutionToFileParms *p_parms)
     : component(p_comp), sv(p_sv), parms(p_parms) {}
    PutSolutionToFileComponent *const component;
    SolutionVector *const sv;
    PutSolutionToFileParms *const parms;
};

void *
PutSolutionToFileComponent::compute_wrapper(void *vp) {

    Info *ip = static_cast<Info *>(vp);
    Info info = *ip;
    delete ip;

    info.component->compute(info.sv, info.parms);

    return NULL;
}

void
PutSolutionToFileComponent::startComputation(SolutionVector *sv,
 PutSolutionToFileParms *parms) {

    pthread_t tid;

    Info *info = new Info(this, sv, parms);
    assert(info != NULL);

    pthread_create(&tid, &m_threadAttr, compute_wrapper, info);
}

void
PutSolutionToFileComponent::compute(SolutionVector *sv,
 PutSolutionToFileParms *parms) {
const char *xfn = "PutSolutionToFileComponent::compute";

printf("Writing solution of length %d to file %s.\n",
 sv->lengthOfDoubleArray, parms->fileName);

    assert(sv != NULL);
    assert(parms != NULL);
    assert(parms->fileName != NULL);

    int ec = -1;

    FILE *fp = fopen(parms->fileName, "w");
    assert(fp != NULL);

    for (int i = 0; i < sv->lengthOfDoubleArray; i++) {
        fprintf(fp, "%lf\n", sv->solutionDoubleArray[i]);
    }

    fclose(fp);

cout << "putSolutionToFile done" << endl;

    pthread_mutex_lock(&m_lock);

    m_computing = false;

    pthread_cond_broadcast(&m_cv);

    pthread_mutex_unlock(&m_lock);

    delete sv;
    delete parms;

}
