#include "ReorderComponent.h"
#include "Reorder.h"
#include "ReorderParms.h"
#include "SparseLinearSystem_impl.h"
#include "ReorderParms_impl.h"
#include "../../../../framework/ccacore/DefaultPortInfo.h"
#include "../../../../framework/idl/sparseLinearSystem/UsesSparseLinearSystem.h"



ReorderComponent::ReorderComponent() {

    int ec = -1;

    m_core = NULL;
    m_sparseLinearSystemPort = new SparseLinearSystem_impl(this);
    m_reorderParmsPort = new ReorderParms_impl(this);
    m_computing = false;

    // XXX
    m_sls_cache = NULL;
    m_parms_cache = new ReorderParms;

    /*
     * Init mutex
     */

    pthread_mutexattr_t attr;

    pthread_mutexattr_init(&attr);

    #if defined(DEBUG) && !defined(__linux__)
        ec = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
        assert(ec == 0);
    #endif

    pthread_mutex_init(&m_lock, &attr);

    pthread_mutexattr_destroy(&attr);

    /*
     * Init condition variable.
     */

    pthread_cond_init(&m_cv, NULL);

    /*
     * Init pthread attr
     */

    ec = pthread_attr_init(&m_threadAttr);
    assert(ec == 0);
    ec = pthread_attr_setdetachstate(&m_threadAttr, PTHREAD_CREATE_DETACHED);
    assert(ec == 0);
    //ec = pthread_attr_setscope(&m_threadAttr, PTHREAD_SCOPE_SYSTEM);
    //assert(ec == 0);
}

ReorderComponent::~ReorderComponent() {

    int ec = -1;

    if (m_core != NULL) {
        delete m_core;
    }

    delete m_sparseLinearSystemPort;
    delete m_reorderParmsPort;

    ec = pthread_attr_destroy(&m_threadAttr);
    assert(ec == 0);

    ec = pthread_cond_destroy(&m_cv);
    assert(ec == 0);

    ec = pthread_mutex_destroy(&m_lock);
    assert(ec == 0);
}

void
ReorderComponent::setServices(Services *cc) {

    if (cc == NULL) {

        assert(m_core != NULL);
        delete m_core;
		m_core = NULL;
        return;

    } else {

        m_core = cc;

        m_core->addProvidesPort(m_sparseLinearSystemPort,
         new DefaultPortInfo("inputSLS", "SparseLinearSystem_idl"));
        m_core->addProvidesPort(m_reorderParmsPort,
         new DefaultPortInfo("inputParms", "ReorderParms_idl"));
		m_core->registerUsesPort(new DefaultPortInfo("outputSLS",
		 "SparseLinearSystem_idl"));
    }
}

struct Info {
    Info(ReorderComponent *cop, SparseLinearSystem *sls_, ReorderParms *ps)
	 : component(cop), sls(sls_), parms(ps) {}
    ReorderComponent *const component;
	SparseLinearSystem *const sls;
	ReorderParms *const parms;
};

void *
ReorderComponent::compute_wrapper(void *vp) {

    Info *ip = static_cast<Info *>(vp);
    Info info = *ip;
    delete ip;

    info.component->compute(info.sls, info.parms);

    return NULL;
}

void
ReorderComponent::startComputation() {

    pthread_t tid;

    Info *info = new Info(this, m_sls_cache, m_parms_cache);
    assert(info != NULL);

    pthread_create(&tid, &m_threadAttr, compute_wrapper, info);

	m_sls_cache = NULL;
	m_parms_cache = NULL;
}

void
ReorderComponent::compute(SparseLinearSystem *sls, ReorderParms *parms) {
  
	cout << "ReorderComponent :: starting computation .... !!!! " << endl;

	assert(sls != NULL);
	assert(parms != NULL);


	int ec = -1;
	sls = Reorder(parms, sls, &ec);
	assert(ec == 0);

cout << "reorder done" << endl;

    UsesSparseLinearSystem *up = dynamic_cast<UsesSparseLinearSystem *>(m_core
	 ->getPort("outputSLS"));
    up->sendSparseLinearSystem(sls);
    m_core->releasePort("outputSLS");
 
    pthread_mutex_lock(&m_lock);

    m_computing = false;

    pthread_cond_broadcast(&m_cv);

	delete sls;
	delete parms;

    pthread_mutex_unlock(&m_lock);
}
