C----- SUBROUTINE GENND
C***************************************************************           1.
C***************************************************************           2.
C********     GENND ..... GENERAL NESTED DISSECTION     ********           3.
C***************************************************************           4.
C***************************************************************           5.
C                                                                          6.
C     PURPOSE - SUBROUTINE GENND FINDS A NESTED DISSECTION                 7.
C        ORDERING FOR A GENERAL GRAPH.                                     8.
C                                                                          9.
C                                                                         10.
C     INPUT PARAMETERS -                                                  11.
C        NEQNS - NUMBER OF EQUATIONS.                                     12.
C        (XADJ, ADJNCY) - ADJACENCY STRUCTURE PAIR.                       13.
C                                                                         14.
C     OUTPUT PARAMETERS -                                                 15.
C        PERM - THE NESTED DISSECTION ORDERING.                           16.
C                                                                         17.
C     WORKING PARAMETERS -                                                18.
C        MASK - IS USED TO MASK OFF VARIABLES THAT HAVE                   19.
C               BEEN NUMBERED DURING THE ORDERNG PROCESS.                 20.
C        (XLS, LS) - THIS LEVEL STRUCTURE PAIR IS USED AS                 21.
C               TEMPORARY STORAGE BY FNROOT.                              22.
C                                                                         23.
C     PROGRAM SUBROUTINES -                                               24.
C        FNDSEP, REVRSE.                                                  25.
C                                                                         26.
C***************************************************************          27.
C                                                                         28.
       SUBROUTINE  GENND ( NEQNS, XADJ, ADJNCY, MASK,                     29.
     1                     PERM, XLS, LS )                                30.
C                                                                         31.
C***************************************************************          32.
C                                                                         33.
          INTEGER ADJNCY(1), MASK(1), LS(1), PERM(1),                     34.
     1            XLS(1)                                                  35.
          INTEGER XADJ(1), I, NEQNS, NSEP, NUM, ROOT                      36.
C                                                                         37.
C***************************************************************          38.
C                                                                         39.
          DO 100 I = 1, NEQNS                                             40.
             MASK(I) = 1                                                  41.
  100     CONTINUE                                                        42.
          NUM   = 0                                                       43.
          DO 300 I = 1, NEQNS                                             44.
C            -----------------------------                                45.
C            FOR EACH MASKED COMPONENT ...                                46.
C            -----------------------------                                47.
  200        IF ( MASK(I) .EQ. 0 )  GO TO 300                             48.
                ROOT = I                                                  49.
C               -------------------------------------------               50.
C               FIND A SEPARATOR AND NUMBER THE NODES NEXT.               51.
C               -------------------------------------------               52.
                CALL  FNDSEP ( ROOT, XADJ, ADJNCY, MASK,                  53.
     1                         NSEP, PERM(NUM+1), XLS, LS )               54.
                NUM  = NUM + NSEP                                         55.
                IF ( NUM .GE. NEQNS )  GO TO 400                          56.
                GO TO 200                                                 57.
  300     CONTINUE                                                        58.
C         ----------------------------------------------                  59.
C         SINCE SEPARATORS FOUND FIRST SHOULD BE ORDERED                  60.
C         LAST, ROUTINE REVRSE IS CALLED TO ADJUST THE                    61.
C         ORDERING VECTOR.                                                62.
C         ----------------------------------------------                  63.
  400     CALL  REVRSE ( NEQNS, PERM )                                    64.
          RETURN                                                          65.
       END                                                                66.


C----- SUBROUTINE FNDSEP
C***************************************************************           1.
C***************************************************************           2.
C************     FNDSEP ..... FIND SEPARATOR       ************           3.
C***************************************************************           4.
C***************************************************************           5.
C                                                                          6.
C     PURPOSE - THIS ROUTINE IS USED TO FIND A SMALL                       7.
C               SEPARATOR FOR A CONNECTED COMPONENT SPECIFIED              8.
C               BY MASK IN THE GIVEN GRAPH.                                9.
C                                                                         10.
C     INPUT PARAMETERS -                                                  11.
C        ROOT - IS THE NODE THAT DETERMINES THE MASKED                    12.
C               COMPONENT.                                                13.
C        (XADJ, ADJNCY) - THE ADJACENCY STRUCTURE PAIR.                   14.
C                                                                         15.
C     OUTPUT PARAMETERS -                                                 16.
C        NSEP - NUMBER OF VARIABLES IN THE SEPARATOR.                     17.
C        SEP - VECTOR CONTAINING THE SEPARATOR NODES.                     18.
C                                                                         19.
C     UPDATED PARAMETER -                                                 20.
C        MASK - NODES IN THE SEPARATOR HAVE THEIR MASK                    21.
C               VALUES SET TO ZERO.                                       22.
C                                                                         23.
C     WORKING PARAMETERS -                                                24.
C        (XLS, LS) - LEVEL STRUCTURE PAIR FOR LEVEL STRUCTURE             25.
C               FOUND BY FNROOT.                                          26.
C                                                                         27.
C     PROGRAM SUBROUTINES -                                               28.
C        FNROOT.                                                          29.
C                                                                         30.
C***************************************************************          31.
C                                                                         32.
      SUBROUTINE  FNDSEP ( ROOT, XADJ, ADJNCY, MASK,                      33.
     1                     NSEP, SEP, XLS , LS )                          34.
C                                                                         35.
C***************************************************************          36.
C                                                                         37.
         INTEGER ADJNCY(1), LS(1), MASK(1), SEP(1), XLS(1)                38.
         INTEGER XADJ(1), I, J, JSTOP, JSTRT, MIDBEG,                     39.
     1           MIDEND, MIDLVL, MP1BEG, MP1END,                          40.
     1           NBR, NLVL, NODE, NSEP, ROOT                              41.
C                                                                         42.
C***************************************************************          43.
C                                                                         44.
         CALL  FNROOT ( ROOT, XADJ, ADJNCY, MASK,                         45.
     1                  NLVL, XLS, LS )                                   46.
C        ----------------------------------------------                   47.
C        IF THE NUMBER OF LEVELS IS LESS THAN 3, RETURN                   48.
C        THE WHOLE COMPONENT AS THE SEPARATOR.                            49.
C        ----------------------------------------------                   50.
         IF ( NLVL .GE. 3 )  GO TO 200                                    51.
            NSEP = XLS(NLVL+1) - 1                                        52.
            DO 100 I = 1, NSEP                                            53.
               NODE = LS(I)                                               54.
               SEP(I) = NODE                                              55.
               MASK(NODE) = 0                                             56.
  100       CONTINUE                                                      57.
            RETURN                                                        58.
C        ----------------------------------------------------             59.
C        FIND THE MIDDLE LEVEL OF THE ROOTED LEVEL STRUCTURE.             60.
C        ----------------------------------------------------             61.
  200    MIDLVL = (NLVL + 2)/2                                            62.
         MIDBEG = XLS(MIDLVL)                                             63.
         MP1BEG = XLS(MIDLVL + 1)                                         64.
         MIDEND = MP1BEG - 1                                              65.
         MP1END = XLS(MIDLVL+2) - 1                                       66.
C        -------------------------------------------------                67.
C        THE SEPARATOR IS OBTAINED BY INCLUDING ONLY THOSE                68.
C        MIDDLE-LEVEL NODES WITH NEIGHBORS IN THE MIDDLE+1                69.
C        LEVEL. XADJ IS USED TEMPORARILY TO MARK THOSE                    70.
C        NODES IN THE MIDDLE+1 LEVEL.                                     71.
C        -------------------------------------------------                72.
         DO 300 I = MP1BEG, MP1END                                        73.
            NODE = LS(I)                                                  74.
            XADJ(NODE) = - XADJ(NODE)                                     75.
  300    CONTINUE                                                         76.
         NSEP  = 0                                                        77.
         DO 500 I = MIDBEG, MIDEND                                        78.
            NODE = LS(I)                                                  79.
            JSTRT = XADJ(NODE)                                            80.
            JSTOP = IABS(XADJ(NODE+1)) - 1                                81.
            DO 400 J = JSTRT, JSTOP                                       82.
               NBR = ADJNCY(J)                                            83.
               IF ( XADJ(NBR) .GT. 0 )  GO TO 400                         84.
                  NSEP = NSEP + 1                                         85.
                  SEP(NSEP) = NODE                                        86.
                  MASK(NODE) = 0                                          87.
                  GO TO 500                                               88.
  400       CONTINUE                                                      89.
  500    CONTINUE                                                         90.
C        -------------------------------                                  91.
C        RESET XADJ TO ITS CORRECT SIGN.                                  92.
C        -------------------------------                                  93.
         DO 600 I = MP1BEG, MP1END                                        94.
            NODE = LS(I)                                                  95.
            XADJ(NODE) = - XADJ(NODE)                                     96.
  600    CONTINUE                                                         97.
         RETURN                                                           98.
      END                                                                 99.

C***************************************************************            3SPK
C***************************************************************            4SPK
C*************     REVRSE ..... REVERSE VECTOR     *************            5SPK
C***************************************************************            6SPK
C***************************************************************            7SPK
C                                                                           8SPK
C     PURPOSE - THIS ROUTINE REVERSES THE ENTRIES IN AN                     9SPK
C        INPUT (SHORT) INTEGER VECTOR.                                     10SPK
C                                                                          11SPK
C     INPUT PARAMETER -                                                    12SPK
C        N     - THE SIZE OF THE INTEGER VECTOR.                           13SPK
C                                                                          14SPK
C     UPDATED PARAMETER -                                                  15SPK
C        IVECTR - THE INTEGER VECTOR WHICH ON OUTPUT WILL BE               16SPK
C                 REVERSED.  (IVECTR IS AN ARRAY OF SHORT                  17SPK
C                 INTEGERS.)                                               18SPK
C                                                                          19SPK
C***************************************************************           20SPK
C                                                                          21SPK
      SUBROUTINE  REVRSE ( N, IVECTR )                                     22SPK
C                                                                          23SPK
C***************************************************************           24SPK
C                                                                          25SPK
         INTEGER    IVECTR(1)                                              26SPK
         INTEGER    I     , ISAVE , J     , MID   , N                      27SPK
C                                                                          28SPK
C***************************************************************           29SPK
C                                                                          30SPK
         IF  ( N .LE. 0 )  RETURN                                          31SPK
         MID = N/2                                                         32SPK
         IF  ( MID .LE. 0 )  RETURN                                        33SPK
         J = N                                                             34SPK
         DO  100  I = 1, MID                                               35SPK
             ISAVE = IVECTR(I)                                             36SPK
             IVECTR(I) = IVECTR(J)                                         37SPK
             IVECTR(J) = ISAVE                                             38SPK
             J = J - 1                                                     39SPK
  100    CONTINUE                                                          40SPK
         RETURN                                                            41SPK
C                                                                          42SPK
      END                                                                  43SPK
