C--- SPARSPAK-A (ANSI FORTRAN) RELEASE III --- NAME = GENRCM                1SPK
C  (C)  UNIVERSITY OF WATERLOO   JANUARY 1984                               2SPK
C***************************************************************            3SPK
C***************************************************************            4SPK
C******     GENRCM ..... GENERAL REVERSE CUTHILL MCKEE     *****            5SPK
C***************************************************************            6SPK
C***************************************************************            7SPK
C                                                                           8SPK
C     PURPOSE - GENRCM FINDS THE REVERSE CUTHILL-MCKEE ORDERING             9SPK
C        FOR A GENERAL GRAPH.  FOR EACH CONNECTED COMPONENT IN             10SPK
C        THE GRAPH, GENRCM OBTAINS THE ORDERING BY CALLING THE             11SPK
C        SUBROUTINE RCM.                                                   12SPK
C                                                                          13SPK
C     INPUT PARAMETERS -                                                   14SPK
C        NEQNS  - NUMBER OF EQUATIONS                                      15SPK
C        (XADJ,ADJNCY) - ARRAY PAIR CONTAINING THE ADJACENCY               16SPK
C                 STRUCTURE OF THE GRAPH OF THE MATRIX.                    17SPK
C                                                                          18SPK
C     OUTPUT PARAMETER -                                                   19SPK
C        PERM   - VECTOR THAT CONTAINS THE RCM ORDERING.                   20SPK
C                                                                          21SPK
C     WORKING PARAMETERS -                                                 22SPK
C        MASK   - IS USED TO MARK VARIABLES THAT HAVE BEEN                 23SPK
C                 NUMBERED DURING THE ORDERING PROCESS.  IT                24SPK
C                 IS INITIALIZED TO 1, AND SET TO ZERO AS                  25SPK
C                 EACH NODE IS NUMBERED.                                   26SPK
C        XLS    - THE INDEX VECTOR FOR A LEVEL STRUCTURE.  THE             27SPK
C                 LEVEL STRUCTURE IS STORED IN THE CURRENTLY               28SPK
C                 UNUSED SPACES IN THE PERMUTATION VECTOR PERM.            29SPK
C                                                                          30SPK
C     PROGRAM SUBROUTINES -                                                31SPK
C        FNROOT, RCM   .                                                   32SPK
C                                                                          33SPK
C***************************************************************           34SPK
C                                                                          35SPK
      SUBROUTINE  GENRCM ( NEQNS, XADJ, ADJNCY, PERM, MASK,                36SPK
     1                     XLS )                                           37SPK
C                                                                          38SPK
C***************************************************************           39SPK
C                                                                          40SPK
         INTEGER    ADJNCY(1), MASK(1)  , PERM(1)  , XLS(1)                41SPK
         INTEGER    XADJ(1)                                                42SPK
         INTEGER    CCSIZE, I     , NEQNS , NLVL  , NUM   ,                43SPK
     1              ROOT                                                   44SPK
C                                                                          45SPK
C***************************************************************           46SPK
C                                                                          47SPK
         IF  ( NEQNS .LE. 0 )  then
            Write(*,*) 'No equations in genrcm!'
            RETURN
         end IF 
C                                                                          49SPK
         DO  100  I = 1, NEQNS                                             50SPK
             MASK(I) = 1                                                   51SPK
  100    CONTINUE                                                          52SPK
         NUM = 1                                                           53SPK
C                                                                          54SPK
         DO  200  I = 1, NEQNS                                             55SPK
C            ---------------------------------------                       56SPK
C            FOR EACH MASKED CONNECTED COMPONENT ...                       57SPK
C            ---------------------------------------                       58SPK
             IF  ( MASK(I) .EQ. 0 )  GO TO 200                             59SPK
                 ROOT = I                                                  60SPK
C                -----------------------------------------                 61SPK
C                FIRST FIND A PSEUDO-PERIPHERAL NODE ROOT.                 62SPK
C                NOTE THAT THE LEVEL STRUCTURE FOUND BY                    63SPK
C                FNROOT IS STORED STARTING AT PERM(NUM).                   64SPK
C                THEN RCM IS CALLED TO ORDER THE COMPONENT                 65SPK
C                USING ROOT AS THE STARTING NODE.                          66SPK
C                -----------------------------------------                 67SPK
                 CALL  FNROOT ( ROOT, XADJ, ADJNCY, MASK,                  68SPK
     1                          NLVL, XLS, PERM(NUM) )                     69SPK
                 CALL  RCM ( ROOT, XADJ, ADJNCY, MASK,                     70SPK
     1                       PERM(NUM), CCSIZE, XLS )                      71SPK
                 NUM = NUM + CCSIZE                                        72SPK
                 IF  ( NUM .GT. NEQNS )  RETURN                            73SPK
  200    CONTINUE                                                          74SPK
         RETURN                                                            75SPK
C                                                                          76SPK
      END                                                                  77SPK
C--- SPARSPAK-A (ANSI FORTRAN) RELEASE III --- NAME = FNROOT                1SPK
C  (C)  UNIVERSITY OF WATERLOO   JANUARY 1984                               2SPK
C***************************************************************            3SPK
C***************************************************************            4SPK
C******     FNROOT ..... FIND PSEUDO-PERIPHERAL NODE     *******            5SPK
C***************************************************************            6SPK
C***************************************************************            7SPK
C                                                                           8SPK
C     PURPOSE - FNROOT IMPLEMENTS A MODIFIED VERSION OF THE                 9SPK
C        SCHEME BY GIBBS, POOLE, AND STOCKMEYER TO FIND PSEUDO-            10SPK
C        PERIPHERAL NODES.  IT DETERMINES SUCH A NODE FOR THE              11SPK
C        SECTION SUBGRAPH SPECIFIED BY MASK AND ROOT.                      12SPK
C                                                                          13SPK
C     INPUT PARAMETERS -                                                   14SPK
C        (XADJ,ADJNCY) - ADJACENCY STRUCTURE PAIR FOR THE GRAPH.           15SPK
C        MASK   - SPECIFIES A SECTION SUBGRAPH. NODES FOR WHICH            16SPK
C                 MASK IS ZERO ARE IGNORED BY FNROOT.                      17SPK
C                                                                          18SPK
C     UPDATED PARAMETER -                                                  19SPK
C        ROOT   - ON INPUT, IT (ALONG WITH MASK) DEFINES THE               20SPK
C                 COMPONENT FOR WHICH A PSEUDO-PERIPHERAL NODE             21SPK
C                 IS TO BE FOUND.  ON OUTPUT, IT IS THE NODE               22SPK
C                 OBTAINED.                                                23SPK
C                                                                          24SPK
C     OUTPUT PARAMETERS -                                                  25SPK
C        NLVL   - IS THE NUMBER OF LEVELS IN THE LEVEL                     26SPK
C                 STRUCTURE ROOTED AT THE NODE ROOT.                       27SPK
C        (XLS,LS) - THE LEVEL STRUCTURE ARRAY PAIR CONTAINING              28SPK
C                 THE LEVEL STRUCTURE FOUND.                               29SPK
C                                                                          30SPK
C    PROGRAM SUBROUTINE -                                                  31SPK
C       ROOTLS.                                                            32SPK
C                                                                          33SPK
C***************************************************************           34SPK
C                                                                          35SPK
      SUBROUTINE  FNROOT ( ROOT, XADJ, ADJNCY, MASK, NLVL,                 36SPK
     1                     XLS, LS )                                       37SPK
C                                                                          38SPK
C***************************************************************           39SPK
C                                                                          40SPK
         INTEGER    ADJNCY(1), LS(1)    , MASK(1)  , XLS(1)                41SPK
         INTEGER    XADJ(1)                                                42SPK
         INTEGER    CCSIZE, J     , JSTRT , K     , KSTOP ,                43SPK
     1              KSTRT , MINDEG, NABOR , NDEG  , NLVL  ,                44SPK
     1              NODE  , NUNLVL, ROOT                                   45SPK
C                                                                          46SPK
C***************************************************************           47SPK
C                                                                          48SPK
C        ---------------------------------------------                     49SPK
C        DETERMINE THE LEVEL STRUCTURE ROOTED AT ROOT.                     50SPK
C        ---------------------------------------------                     51SPK
         CALL  ROOTLS ( ROOT, XADJ, ADJNCY, MASK, NLVL, XLS, LS )          52SPK
         CCSIZE = XLS(NLVL+1) - 1                                          53SPK
         IF  ( NLVL .EQ. 1  .OR.  NLVL .EQ. CCSIZE )  RETURN               54SPK
C        ----------------------------------------------------              55SPK
C        PICK A NODE WITH MINIMUM DEGREE FROM THE LAST LEVEL.              56SPK
C        ----------------------------------------------------              57SPK
  100    CONTINUE                                                          58SPK
             JSTRT = XLS(NLVL)                                             59SPK
             MINDEG = CCSIZE                                               60SPK
             ROOT = LS(JSTRT)                                              61SPK
             IF  ( CCSIZE .EQ. JSTRT )  GO TO 400                          62SPK
                 DO  300  J = JSTRT, CCSIZE                                63SPK
                     NODE = LS(J)                                          64SPK
                     NDEG = 0                                              65SPK
                     KSTRT = XADJ(NODE)                                    66SPK
                     KSTOP = XADJ(NODE+1) - 1                              67SPK
                     DO  200  K = KSTRT, KSTOP                             68SPK
                         NABOR = ADJNCY(K)                                 69SPK
                         IF  ( MASK(NABOR) .GT. 0 )  NDEG = NDEG + 1       70SPK
  200                CONTINUE                                              71SPK
                     IF  ( NDEG .GE. MINDEG )  GO TO 300                   72SPK
                         ROOT = NODE                                       73SPK
                         MINDEG = NDEG                                     74SPK
  300            CONTINUE                                                  75SPK
  400        CONTINUE                                                      76SPK
C            ----------------------------------------                      77SPK
C            AND GENERATE ITS ROOTED LEVEL STRUCTURE.                      78SPK
C            ----------------------------------------                      79SPK
             CALL  ROOTLS ( ROOT, XADJ, ADJNCY, MASK,                      80SPK
     1                      NUNLVL, XLS, LS )                              81SPK
             IF  ( NUNLVL .LE. NLVL )  RETURN                              82SPK
             NLVL = NUNLVL                                                 83SPK
             IF  ( NLVL .LT. CCSIZE )  GO TO 100                           84SPK
                 RETURN                                                    85SPK
C                                                                          86SPK
      END                                                                  87SPK
C--- SPARSPAK-A (ANSI FORTRAN) RELEASE III --- NAME = RCM                   1SPK
C  (C)  UNIVERSITY OF WATERLOO   JANUARY 1984                               2SPK
C***************************************************************            3SPK
C***************************************************************            4SPK
C*******     RCM ..... REVERSE CUTHILL-MCKEE ORDERING     ******            5SPK
C***************************************************************            6SPK
C***************************************************************            7SPK
C                                                                           8SPK
C     PURPOSE - RCM NUMBERS A CONNECTED COMPONENT SPECIFIED BY              9SPK
C        MASK AND ROOT, USING THE RCM ALGORITHM.  THE NUMBERING            10SPK
C        IS TO BE STARTED AT THE NODE ROOT.                                11SPK
C                                                                          12SPK
C     INPUT PARAMETERS -                                                   13SPK
C        ROOT   - IS THE NODE THAT DEFINES THE CONNECTED                   14SPK
C                 COMPONENT AND IT IS USED AS THE STARTING                 15SPK
C                 NODE FOR THE RCM ORDERING.                               16SPK
C        (XADJ,ADJNCY) - ADJACENCY STRUCTURE PAIR FOR THE GRAPH.           17SPK
C                                                                          18SPK
C     UPDATED PARAMETER -                                                  19SPK
C        MASK   - ONLY THOSE NODES WITH NONZERO INPUT MASK                 20SPK
C                 VALUES ARE CONSIDERED BY THE ROUTINE.  THE               21SPK
C                 NODES NUMBERED BY RCM WILL HAVE THEIR MASK               22SPK
C                 VALUES SET TO ZERO.                                      23SPK
C                                                                          24SPK
C     OUTPUT PARAMETERS -                                                  25SPK
C        PERM   - WILL CONTAIN THE RCM ORDERING.                           26SPK
C        CCSIZE - IS THE SIZE OF THE CONNECTED COMPONENT THAT              27SPK
C                 HAS BEEN NUMBERED BY RCM.                                28SPK
C                                                                          29SPK
C     WORKING PARAMETER -                                                  30SPK
C        DEG    - IS A TEMPORARY VECTOR USED TO HOLD THE DEGREE            31SPK
C                 OF THE NODES IN THE SECTION GRAPH SPECIFIED BY           32SPK
C                 MASK AND ROOT.                                           33SPK
C                                                                          34SPK
C     PROGRAM SUBROUTINE -                                                 35SPK
C        DEGREE.                                                           36SPK
C                                                                          37SPK
C***************************************************************           38SPK
C                                                                          39SPK
      SUBROUTINE  RCM ( ROOT, XADJ, ADJNCY, MASK, PERM, CCSIZE,            40SPK
     1                  DEG )                                              41SPK
C                                                                          42SPK
C***************************************************************           43SPK
C                                                                          44SPK
         INTEGER    ADJNCY(1), DEG(1)   , MASK(1)  , PERM(1)               45SPK
         INTEGER    XADJ(1)                                                46SPK
         INTEGER    CCSIZE, FNBR  , I     , J     , JSTOP ,                47SPK
     1              JSTRT , K     , L     , LBEGIN, LNBR  ,                48SPK
     1              LPERM , LVLEND, NBR   , NODE  , ROOT                   49SPK
C                                                                          50SPK
C***************************************************************           51SPK
C                                                                          52SPK
C        -------------------------------------                             53SPK
C        FIND THE DEGREES OF THE NODES IN THE                              54SPK
C        COMPONENT SPECIFIED BY MASK AND ROOT.                             55SPK
C        -------------------------------------                             56SPK
         CALL  DEGREE ( ROOT, XADJ, ADJNCY, MASK, DEG, CCSIZE,             57SPK
     1                  PERM )                                             58SPK
         MASK(ROOT) = 0                                                    59SPK
         IF  ( CCSIZE .LE. 1 )  RETURN                                     60SPK
         LVLEND = 0                                                        61SPK
         LNBR = 1                                                          62SPK
  100    CONTINUE                                                          63SPK
C            --------------------------------------------                  64SPK
C            LBEGIN AND LVLEND POINT TO THE BEGINNING AND                  65SPK
C            THE END OF THE CURRENT LEVEL RESPECTIVELY.                    66SPK
C            --------------------------------------------                  67SPK
             LBEGIN = LVLEND + 1                                           68SPK
             LVLEND = LNBR                                                 69SPK
             DO  600  I = LBEGIN, LVLEND                                   70SPK
C                ----------------------------------                        71SPK
C                FOR EACH NODE IN CURRENT LEVEL ...                        72SPK
C                ----------------------------------                        73SPK
                 NODE = PERM(I)                                            74SPK
                 JSTRT = XADJ(NODE)                                        75SPK
                 JSTOP = XADJ(NODE+1) - 1                                  76SPK
C                -----------------------------------------                 77SPK
C                FIND THE UNNUMBERED NEIGHBORS OF NODE.                    78SPK
C                FNBR AND LNBR POINT TO THE FIRST AND LAST                 79SPK
C                UNNUMBERED NEIGHBORS RESPECTIVELY OF THE                  80SPK
C                CURRENT NODE IN PERM.                                     81SPK
C                -----------------------------------------                 82SPK
                 FNBR = LNBR + 1                                           83SPK
                 DO  200  J = JSTRT, JSTOP                                 84SPK
                     NBR = ADJNCY(J)                                       85SPK
                     IF  ( MASK(NBR) .EQ. 0 )  GO TO 200                   86SPK
                         LNBR = LNBR + 1                                   87SPK
                         MASK(NBR) = 0                                     88SPK
                         PERM(LNBR) = NBR                                  89SPK
  200            CONTINUE                                                  90SPK
                 IF  ( FNBR .GE. LNBR )  GO TO 600                         91SPK
C                    ------------------------------------------            92SPK
C                    SORT THE NEIGHBORS OF NODE IN INCREASING              93SPK
C                    ORDER BY DEGREE. LINEAR INSERTION IS USED.            94SPK
C                    ------------------------------------------            95SPK
                     K = FNBR                                              96SPK
  300                CONTINUE                                              97SPK
                         L = K                                             98SPK
                         K = K + 1                                         99SPK
                         NBR = PERM(K)                                    100SPK
  400                    CONTINUE                                         101SPK
                             IF  ( L .LT. FNBR )  GO TO 500               102SPK
                             LPERM = PERM(L)                              103SPK
                             IF  ( DEG(LPERM) .LE. DEG(NBR) )  GO TO 500  104SPK
                             PERM(L+1) = LPERM                            105SPK
                             L = L - 1                                    106SPK
                             GO TO 400                                    107SPK
  500                    CONTINUE                                         108SPK
                         PERM(L+1) = NBR                                  109SPK
                         IF  ( K .LT. LNBR )  GO TO 300                   110SPK
  600        CONTINUE                                                     111SPK
             IF  ( LNBR .GT. LVLEND )  GO TO 100                          112SPK
C                                                                         113SPK
C        ---------------------------------------                          114SPK
C        WE NOW HAVE THE CUTHILL MCKEE ORDERING.                          115SPK
C        REVERSE IT BELOW ...                                             116SPK
C        ---------------------------------------                          117SPK
         K = CCSIZE/2                                                     118SPK
         L = CCSIZE                                                       119SPK
         DO  700  I = 1, K                                                120SPK
             LPERM = PERM(L)                                              121SPK
             PERM(L) = PERM(I)                                            122SPK
             PERM(I) = LPERM                                              123SPK
             L = L - 1                                                    124SPK
  700    CONTINUE                                                         125SPK
         RETURN                                                           126SPK
C                                                                         127SPK
      END                                                                 128SPK
C--- SPARSPAK-A (ANSI FORTRAN) RELEASE III --- NAME = ROOTLS                1SPK
C  (C)  UNIVERSITY OF WATERLOO   JANUARY 1984                               2SPK
C***************************************************************            3SPK
C***************************************************************            4SPK
C*********     ROOTLS ..... ROOTED LEVEL STRUCTURE     *********            5SPK
C***************************************************************            6SPK
C***************************************************************            7SPK
C                                                                           8SPK
C     PURPOSE - ROOTLS GENERATES THE LEVEL STRUCTURE ROOTED                 9SPK
C        AT THE INPUT NODE CALLED ROOT.  ONLY THOSE NODES FOR              10SPK
C        WHICH MASK IS NONZERO WILL BE CONSIDERED.                         11SPK
C                                                                          12SPK
C     INPUT PARAMETERS -                                                   13SPK
C        ROOT   - THE NODE AT WHICH THE LEVEL STRUCTURE IS TO              14SPK
C                 BE ROOTED.                                               15SPK
C        (XADJ,ADJNCY) - ADJACENCY STRUCTURE PAIR FOR THE                  16SPK
C                 GIVEN GRAPH.                                             17SPK
C        MASK   - IS USED TO SPECIFY A SECTION SUBGRAPH.                   18SPK
C                 NODES WITH MASK(I)=0 ARE IGNORED.                        19SPK
C                                                                          20SPK
C     OUTPUT PARAMETERS -                                                  21SPK
C        NLVL   - IS THE NUMBER OF LEVELS IN THE LEVEL                     22SPK
C                 STRUCTURE.                                               23SPK
C        (XLS,LS) - ARRAY PAIR FOR THE ROOTED LEVEL STRUCTURE.             24SPK
C                                                                          25SPK
C***************************************************************           26SPK
C                                                                          27SPK
      SUBROUTINE  ROOTLS ( ROOT, XADJ, ADJNCY, MASK, NLVL,                 28SPK
     1                     XLS, LS )                                       29SPK
C                                                                          30SPK
C***************************************************************           31SPK
C                                                                          32SPK
         INTEGER    ADJNCY(1), LS(1)    , MASK(1)  , XLS(1)                33SPK
         INTEGER    XADJ(1)                                                34SPK
         INTEGER    I     , J     , JSTOP , JSTRT , LBEGIN,                35SPK
     1              CCSIZE, LVLEND, LVSIZE, NBR   , NLVL  ,                36SPK
     1              NODE  , ROOT                                           37SPK
C                                                                          38SPK
C***************************************************************           39SPK
C                                                                          40SPK
C        ------------------                                                41SPK
C        INITIALIZATION ...                                                42SPK
C        ------------------                                                43SPK
         MASK(ROOT) = 0                                                    44SPK
         LS(1) = ROOT                                                      45SPK
         NLVL = 0                                                          46SPK
         LVLEND = 0                                                        47SPK
         CCSIZE = 1                                                        48SPK
C                                                                          49SPK
C        -----------------------------------------------------             50SPK
C        LBEGIN IS THE POINTER TO THE BEGINNING OF THE CURRENT             51SPK
C        LEVEL, AND LVLEND POINTS TO THE END OF THIS LEVEL.                52SPK
C        -----------------------------------------------------             53SPK
  100    CONTINUE                                                          54SPK
             LBEGIN = LVLEND + 1                                           55SPK
             LVLEND = CCSIZE                                               56SPK
             NLVL = NLVL + 1                                               57SPK
             XLS(NLVL) = LBEGIN                                            58SPK
C            -------------------------------------------------             59SPK
C            GENERATE THE NEXT LEVEL BY FINDING ALL THE MASKED             60SPK
C            NEIGHBORS OF NODES IN THE CURRENT LEVEL.                      61SPK
C            -------------------------------------------------             62SPK
             DO  300  I = LBEGIN, LVLEND                                   63SPK
                 NODE = LS(I)                                              64SPK
                 JSTRT = XADJ(NODE)                                        65SPK
                 JSTOP = XADJ(NODE+1) - 1                                  66SPK
                 IF  ( JSTOP .LT. JSTRT )  GO TO 300                       67SPK
                     DO  200  J = JSTRT, JSTOP                             68SPK
                         NBR = ADJNCY(J)                                   69SPK
                         IF  ( MASK(NBR) .EQ. 0 )  GO TO 200               70SPK
                             CCSIZE = CCSIZE + 1                           71SPK
                             LS(CCSIZE) = NBR                              72SPK
                             MASK(NBR) = 0                                 73SPK
  200                CONTINUE                                              74SPK
  300        CONTINUE                                                      75SPK
C            ------------------------------------------                    76SPK
C            COMPUTE THE CURRENT LEVEL WIDTH.                              77SPK
C            IF IT IS NONZERO, GENERATE THE NEXT LEVEL.                    78SPK
C            ------------------------------------------                    79SPK
             LVSIZE = CCSIZE - LVLEND                                      80SPK
             IF  ( LVSIZE .GT. 0 )  GO TO 100                              81SPK
C                                                                          82SPK
C        -------------------------------------------------------           83SPK
C        RESET MASK TO ONE FOR THE NODES IN THE LEVEL STRUCTURE.           84SPK
C        -------------------------------------------------------           85SPK
         XLS(NLVL+1) = LVLEND + 1                                          86SPK
         DO  400  I = 1, CCSIZE                                            87SPK
             NODE = LS(I)                                                  88SPK
             MASK(NODE) = 1                                                89SPK
  400    CONTINUE                                                          90SPK
         RETURN                                                            91SPK
C                                                                          92SPK
      END                                                                  93SPK
C--- SPARSPAK-A (ANSI FORTRAN) RELEASE III --- NAME = DEGREE                1SPK
C  (C)  UNIVERSITY OF WATERLOO   JANUARY 1984                               2SPK
C***************************************************************            3SPK
C***************************************************************            4SPK
C*******     DEGREE ..... DEGREE IN MASKED COMPONENT     *******            5SPK
C***************************************************************            6SPK
C***************************************************************            7SPK
C                                                                           8SPK
C     PURPOSE - THIS ROUTINE COMPUTES THE DEGREES OF THE NODES              9SPK
C        IN THE CONNECTED COMPONENT SPECIFIED BY MASK AND ROOT.            10SPK
C        NODES FOR WHICH MASK IS ZERO ARE IGNORED.                         11SPK
C                                                                          12SPK
C     INPUT PARAMETERS -                                                   13SPK
C        ROOT   - IS THE INPUT NODE THAT DEFINES THE COMPONENT.            14SPK
C        (XADJ,ADJNCY) - ADJACENCY STRUCTURE PAIR.                         15SPK
C        MASK   - SPECIFIES A SECTION SUBGRAPH.                            16SPK
C                                                                          17SPK
C     OUTPUT PARAMETERS -                                                  18SPK
C        DEG    - ARRAY CONTAINING THE DEGREES OF THE NODES IN             19SPK
C                 THE COMPONENT.                                           20SPK
C        CCSIZE - SIZE OF THE COMPONENT SPECIFED BY MASK AND               21SPK
C                 ROOT.                                                    22SPK
C                                                                          23SPK
C     WORKING PARAMETER -                                                  24SPK
C        LS     - A TEMPORARY VECTOR USED TO STORE THE NODES OF            25SPK
C                 THE COMPONENT LEVEL BY LEVEL.                            26SPK
C                                                                          27SPK
C***************************************************************           28SPK
C                                                                          29SPK
      SUBROUTINE  DEGREE ( ROOT, XADJ, ADJNCY, MASK, DEG,                  30SPK
     1                     CCSIZE, LS )                                    31SPK
C                                                                          32SPK
C***************************************************************           33SPK
C                                                                          34SPK
         INTEGER    ADJNCY(1), DEG(1)   , LS(1)    , MASK(1)               35SPK
         INTEGER    XADJ(1)                                                36SPK
         INTEGER    CCSIZE, I     , IDEG  , J     , JSTOP ,                37SPK
     1              JSTRT , LBEGIN, LVLEND, LVSIZE, NBR   ,                38SPK
     1              NODE  , ROOT                                           39SPK
C                                                                          40SPK
C***************************************************************           41SPK
C                                                                          42SPK
C        -------------------------------------------------                 43SPK
C        INITIALIZATION ...                                                44SPK
C        THE ARRAY XADJ IS USED AS A TEMPORARY MARKER TO                   45SPK
C        INDICATE WHICH NODES HAVE BEEN CONSIDERED SO FAR.                 46SPK
C        -------------------------------------------------                 47SPK
         LS(1) = ROOT                                                      48SPK
         XADJ(ROOT) = - XADJ(ROOT)                                         49SPK
         LVLEND = 0                                                        50SPK
         CCSIZE = 1                                                        51SPK
C                                                                          52SPK
  100    CONTINUE                                                          53SPK
C            -----------------------------------------------------         54SPK
C            LBEGIN IS THE POINTER TO THE BEGINNING OF THE CURRENT         55SPK
C            LEVEL, AND LVLEND POINTS TO THE END OF THIS LEVEL.            56SPK
C            -----------------------------------------------------         57SPK
             LBEGIN = LVLEND + 1                                           58SPK
             LVLEND = CCSIZE                                               59SPK
C            -----------------------------------------------               60SPK
C            FIND THE DEGREES OF NODES IN THE CURRENT LEVEL,               61SPK
C            AND AT THE SAME TIME, GENERATE THE NEXT LEVEL.                62SPK
C            -----------------------------------------------               63SPK
             DO  400  I = LBEGIN, LVLEND                                   64SPK
                 NODE = LS(I)                                              65SPK
                 JSTRT = - XADJ(NODE)                                      66SPK
                 JSTOP = IABS(XADJ(NODE+1)) - 1                            67SPK
                 IDEG = 0                                                  68SPK
                 IF  ( JSTOP .LT. JSTRT )  GO TO 300                       69SPK
cvd$l noconcur
cvd$l novector
                     DO  200  J = JSTRT, JSTOP                             70SPK
                         NBR = ADJNCY(J)                                   71SPK
                         IF  ( MASK(NBR) .EQ. 0 )  GO TO 200               72SPK
                             IDEG = IDEG + 1                               73SPK
                             IF  ( XADJ(NBR) .LT. 0 )  GO TO 200           74SPK
                                 XADJ(NBR) = - XADJ(NBR)                   75SPK
                                 CCSIZE = CCSIZE + 1                       76SPK
                                 LS(CCSIZE) = NBR                          77SPK
  200                CONTINUE                                              78SPK
  300            CONTINUE                                                  79SPK
                 DEG(NODE) = IDEG                                          80SPK
  400        CONTINUE                                                      81SPK
C            -----------------------------------------                     82SPK
C            COMPUTE THE CURRENT LEVEL WIDTH.                              83SPK
C            IF IT IS NONZERO, GENERATE ANOTHER LEVEL.                     84SPK
C            -----------------------------------------                     85SPK
             LVSIZE = CCSIZE - LVLEND                                      86SPK
             IF  ( LVSIZE .GT. 0 )  GO TO 100                              87SPK
C                                                                          88SPK
C        ------------------------------------------                        89SPK
C        RESET XADJ TO ITS CORRECT SIGN AND RETURN.                        90SPK
C        ------------------------------------------                        91SPK
         DO  500  I = 1, CCSIZE                                            92SPK
             NODE = LS(I)                                                  93SPK
             XADJ(NODE) = - XADJ(NODE)                                     94SPK
  500    CONTINUE                                                          95SPK
         RETURN                                                            96SPK
C                                                                          97SPK
      END                                                                  98SPK
