#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include "forts.h"

int getapat(double *A , int *colinds , int *rowptrs, 
            double *A2, int *colinds2, int *rowptrs2, 
            int n, int nnz) {

/*

Compute the structure of A + A', and put it into the
adjacency structure for A2.  Note that although A and
A2 are passed in, this function currently does not
compute the values of A + A'.

Returned value is the number of nonzeros in A + A'; if
run out of memory here, instead a negative number is
returned in nnzc.  If error occurs in apmbt.f, the value
zero is returned in nnzc.

     Randall Bramley
     Department of Computer Science
     Indiana University
     Phone: (812) 855-7790
     email: bramley@cs.indiana.edu
     Mon May 26 14:48:14 EST 1997

*/


     /* ----------------------------------------- */
     /* Working storage to compute A+A' structure */
     /* ----------------------------------------- */
     double *C = NULL;
     int *jc = NULL;
     int *ic = NULL;
     int *iw = NULL;
     int nnzc = 0;

     int job = 0;
     int ierr = 0;
     size_t length = 0;

     nnzc = 2*nnz;

     iw = (int *) malloc((1+n)*sizeof(int));
     if (iw == NULL) return(-1);

     jc = (int *) malloc(nnzc*sizeof(int));
     if (jc == NULL) return(-1); 

     ic = (int *) malloc(nnzc*sizeof(int));
     if (ic == NULL) return(-1); 

     apmbt_(&n, &n, &job,
            A, colinds, rowptrs,
            A, colinds, rowptrs,
            C, jc, ic,
            &nnzc, iw, &ierr);

     if (ierr != 0) return(0);
     nnzc = ic[n]-1;

/*
     length = (size_t) nnz*sizeof(double);
     bcopy(A, A2, length);
*/

     length = (size_t) nnzc*sizeof(int);
     bcopy(jc, colinds2, length);

     length = (size_t) (n+1)*sizeof(int);
     bcopy(ic, rowptrs2, length);

     free((void *)ic);
     free((void *)iw);
     free((void *)jc);
     return(nnzc);
}
