/*  
//
//  Test driver for Reorder.  Algorithm is:
//  
//  k = 0;
//  while (keep_going(k))
//	GetSparseSystem;
//	GetReorderParms;
//	call Reorder()
//	k = k + 1;
//      PutSparseSystem;
//	FreeStorage;
//  end while
//
//  Randall Bramley
//  Department of Computer Science
//  Indiana University, Bloomington
//  bramley@cs.indiana.edu                             
//  Sun Nov  7 08:24:17 EST 1999
//  --------------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <errno.h>
#include <iostream.h>
#include <assert.h>
#include <mmio.h>
#include <SparseLinearSystem.h>
#include <GetSparseSystem.h>
#include <PutSparseSystem.h>
#include <FreeStorage.h>
#include <KeepOnTruckin.h>
#include <ReorderParms.h>
#include <Reorder.h>

int main(int argc, char *argv[]) {

  int ErrorCode;
  int k = 0;  // Number of times through

  while (KeepOnTruckin(k)) {

      ReorderParms p = getParameters();

      SparseLinearSystem *OriginalSystem;
      SparseLinearSystem *ReorderedSystem;


      // --------------------------------------------------
      // Replace the following with a provides port access.
      // Now it just calls readmt_(), a Fortran H/B reader.
      // --------------------------------------------------

      OriginalSystem = GetSparseSystem();

      // ------------------------------------------
      // Call reorder to rearrange the system.
      // Note that a new sparse system is returned,
      // along with row/column permutation vectors.
      // ------------------------------------------

      ReorderedSystem = Reorder(&p, OriginalSystem, 
                               &ErrorCode);
      cout << "ErrorCode from Reorder is: " << ErrorCode << endl;

      // --------------------------------------------------
      // Now call PutSparseSystem to output to next module
      // --------------------------------------------------

      int errflag = PutSparseSystem(ReorderedSystem);

      FreeStorage(OriginalSystem);

      k++;

  }

  return 0;
}
