      subroutine scala(job, A, rwptr, colind, n, b, D)
      implicit none

c----------------------------------------------------------------
c     Scale a CSR matrix to have certain properties, 
c     depending on the job parameter.
c       job = 1  ---> set max(max(A)) = 1.0
c       job = 2  ---> set max of each row to be 1.0
c       job = 3  ---> scale each column so diagonal entry is 1.0
c
c     If job > 1, a vector is returned in D that gives
c     the scaling vector used, so that
c          Anew = D*A   (job = 2)  or
c          Anew = A*D   (job = 3)
c
c     In case job < 3, the right hand side vector is also scaled.
c
c     Warning: this code assumes that A is not the zero matrix!
c       Also, if the max-norm of a row is zero when job = 2, 
c       or if the diagonal element is zero when job = 3, the
c       code sets the corresponding D element to be zero and
c       does not scale that row or column.
c----------------------------------------------------------------

      integer rwptr(*), n, colind(*), k, i, nnz
      integer s, job

      real*8 A(*), t
      real*8 b(n), D(n)

      integer idamax
      external idamax

c---------- Scaling one: max(max(A)) = 1.0 -------------
      if (job .eq. 1) then
         nnz = rwptr(n+1)-1
         k = idamax(nnz, A, 1)
         t = 1.0d0/A(k)
         call dscal(nnz,t,A,1)
         return
      end if 

c---------- Scaling two: max(A') = 1.0 -------------
      if (job .eq. 2) then
         do i = 1, n
             s = rwptr(i)
             nnz = rwptr(i+1) - s
             t = abs(A(s))
             do k = s+1, rwptr(i+1)-1
               if (abs(A(k)) .gt. t) t = abs(A(k))
             end do 
             if ( t .ne. 0.0d0) then
                D(i) = 1.0d0/t
                call dscal(nnz,D(i),A(s),1)
                b(i) = D(i)*b(i)
             else
                D(i) = 0.0d0
             end if
         end do 
         return
      end if 
c---------- Scaling three: A*D has diag(A) = I --------
      if (job .eq. 3) then
         do i = 1, n
            D(i) = 1.0d0
            do k = rwptr(i), rwptr(i+1)-1
               if (colind(k) .eq. i .and. A(k).ne. 0.0d0) then
                  D(i) = 1.0d0/A(k)
                  go to 100
               end if
            end do
 100     continue
         end do 
         nnz = rwptr(n+1)-1
         do i = 1, nnz
            A(i) = A(i)*D(colind(i))
         end do 
         return
      end if 

      return
      end 
c
c==============================================================
c
      subroutine unscalx(n, x, Dr)
      implicit none
      integer n, k
      real*8 x(n), Dr(n)
      do k = 1, n
         x(k) = x(k)*Dr(k)
      end do
      return
      end

c-----------------------------------------------------------------------
c
      subroutine unscala(job,A, rwptr, colind, n, b, D)
      implicit none

c----------------------------------------------------------------
c     Unscale a CSR matrix to reverse effects of scala.
c----------------------------------------------------------------

      integer rwptr(*), n, colind(*), k, i, nnz
      integer s, job

      real*8 A(*), t
      real*8 b(n), D(n)

      integer idamax
      external idamax

c---------- Scaling one: max(max(A)) = 1.0 -------------
      if (job .eq. 1) then
c         nnz = rwptr(n+1)-1
c         k = idamax(nnz, A, 1)
c         t = 1.0d0/A(k)
c         call dscal(nnz,t,A,1)
c         return
      end if 

c---------- Scaling two: max(A') = 1.0 -------------
      if (job .eq. 2) then
         do i = 1, n
             s = rwptr(i)
             nnz = rwptr(i+1) - s
             t = 1.0d0/D(i)
             call dscal(nnz,t,A(s),1)
             b(i) = t*b(i)
         end do 
         return
      end if 
c---------- Scaling three: A*D has diag(A) = I --------
      if (job .eq. 3) then
         nnz = rwptr(n+1)-1
         do i = 1, nnz
            A(i) = A(i)/D(colind(i))
         end do 
         return
      end if 

      return
      end 
c
c-----------------------------------------------------------------------
