#include <iomanip.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <SplibParms.h>
#include "../../../../framework/util/hpcxx_byte.h"
#include "../../../../framework/util/HPCxx_String.h"

SplibParms::SplibParms(){
  int k;
  for (k = 0; k < 20; k++) iunit[k] = k+16;
  UnitsOpen         = 0;
  relative          = 1;    
  MaximumIterations = 100;
  AllocatedSpace    = 24000000;
  SolveMethod       = 5;
  KrylovVectors     = 20;
  PrecondMeth       = 1;
  FillLevels        = 0;
  rows              = 1;
  BlockSize         = 3;  
  OutputLevel       = 5;   
  RecomputePrecond  = 0;
  Scaling           = 0;
  Tolerance         = 1.0E-6;
  SORRelaxParm      = 1.0;
  DropTol           = 1.0E-2;
  SparsityTarget    = 0;
  MILUParam         = 1.0;
}

SplibParms::SplibParms(int *iunit,
		       int UnitsOpen,
             	       int relative,    
             	       int MaximumIterations,
             	       int AllocatedSpace,
		       int SolveMethod,
             	       int KrylovVectors,
             	       int PrecondMeth,
             	       int FillLevels,
             	       int rows,
             	       int BlockSize,  
             	       int OutputLevel,   
             	       int RecomputePrecond,
             	       int Scaling,
             	       double Tolerance,
             	       double SORRelaxParm,
             	       double DropTol,
             	       double SparsityTarget,
             	       double MILUParam) {
  int k; 
  for (k = 0; k < 20; k++) this->iunit[k] = iunit[k];
  this->UnitsOpen         = UnitsOpen;
  this->relative          = relative    ;
  this->MaximumIterations = MaximumIterations;
  this->AllocatedSpace    = AllocatedSpace;
  this->SolveMethod       = SolveMethod;
  this->KrylovVectors     = KrylovVectors;
  this->PrecondMeth       = PrecondMeth;
  this->FillLevels        = FillLevels;
  this->rows              = rows;
  this->BlockSize         = BlockSize  ;
  this->OutputLevel       = OutputLevel   ;
  this->RecomputePrecond  = RecomputePrecond;
  this->Scaling           = Scaling;
  this->Tolerance         = Tolerance;
  this->SORRelaxParm      = SORRelaxParm;
  this->DropTol           = DropTol;
  this->SparsityTarget    = SparsityTarget;
  this->MILUParam         = MILUParam;
};

SplibParms::~SplibParms(){
  // Commented out by kec.
  //delete[] iunit;
};

// Return a control parameter object to driver
// This version uses the splib driver's routines
// that read in a file of methods.  No error
// checking is done, so beware.

SplibParms getParameters(){
        int k ;
	int ipcprm;
	double rpcprm;
         
        SplibParms* p = new SplibParms;
        for (k = 0; k < 20; k++) (p->iunit)[k] = k+16;
	getparms_( &rpcprm,
		   &(p->relative),
		   &(p->PrecondMeth),
		   &ipcprm,
		   &(p->SolveMethod),
		   &(p->KrylovVectors),
		   &(p->Tolerance),
		   &(p->MaximumIterations),
		   &(p->RecomputePrecond),
		   p->iunit,
		   &(p->OutputLevel),
		   &(p->Scaling));

//  rpcprm  : real preconditioner parameter --
//             - tolerance for numerical dropping with ILUT;
//             - omega parameter for SSOR(omega);
//             - relaxation constant for MILU.
//             - sparsity pattern for ECIMGS
//  Fortunately, only one of those roles can hold on a given run.
	p->DropTol        = rpcprm;
	p->SORRelaxParm   = rpcprm;
	p->MILUParam      = rpcprm;
	p->SparsityTarget = rpcprm;
//  ipcprm  : integer preconditioning parameter, usually levels of fill.
//            For tridiag preconditioning, it means the block size.
	p->FillLevels = ipcprm;
	p->BlockSize  = ipcprm;
//  God only knows how much memory will be needed by Splib.
//  It actually returns an estimate of what is needed if it
//  fails initially.  Following is typically an overestimate.
        p->AllocatedSpace = 48000000;

	return *p;
}

void hpcxx_pack(HPCxx_Buffer& buffer, SplibParms* p, int count){
  for(int i=0; i<count; i++) {

    //NexusRMI packs byte 20 to indicate that it is an object
    hpcxx_byte k20(20);
    hpcxx_pack(buffer, &k20,1);
    //NexusRMI packs the fully qualified class name of the object
    HPCxx_String FQN("idl.splibParms.SplibParms");
    hpcxx_pack(buffer, &FQN, 1);

    int iunitsLength = 20;
    hpcxx_pack(buffer, p->iunit, iunitsLength);
    hpcxx_pack(buffer, &(p->UnitsOpen), 1);
    hpcxx_pack(buffer, &(p->relative), 1);
    hpcxx_pack(buffer, &(p->MaximumIterations), 1);
    hpcxx_pack(buffer, &(p->AllocatedSpace), 1);
    hpcxx_pack(buffer, &(p->SolveMethod), 1);
    hpcxx_pack(buffer, &(p->KrylovVectors), 1);
    hpcxx_pack(buffer, &(p->PrecondMeth), 1);
    hpcxx_pack(buffer, &(p->FillLevels), 1);
    hpcxx_pack(buffer, &(p->rows), 1);
    hpcxx_pack(buffer, &(p->BlockSize), 1);
    hpcxx_pack(buffer, &(p->OutputLevel), 1);
    hpcxx_pack(buffer, &(p->RecomputePrecond), 1);
    hpcxx_pack(buffer, &(p->Scaling), 1);
    hpcxx_pack(buffer, &(p->Tolerance), 1);
    hpcxx_pack(buffer, &(p->SORRelaxParm), 1);
    hpcxx_pack(buffer, &(p->DropTol), 1);
    hpcxx_pack(buffer, &(p->SparsityTarget), 1);
    hpcxx_pack(buffer, &(p->MILUParam), 1);
  }
}

void hpcxx_unpack(HPCxx_Buffer& buffer, SplibParms* p, int count){
  for(int i=0; i<count; i++) {

    //NexusRMI packs byte 20 to indicate that it is an object
    hpcxx_byte k20(20);
    hpcxx_unpack(buffer, &k20,1);
    //NexusRMI packs the fully qualified class name of the object
    HPCxx_String FQN("idl.splibParms.SpliParms");
    hpcxx_unpack(buffer, &FQN, 1);   

    int iunitsLength = 20;
    hpcxx_unpack(buffer, p->iunit, iunitsLength);
    hpcxx_unpack(buffer, &(p->UnitsOpen), 1);
    hpcxx_unpack(buffer, &(p->relative), 1);
    hpcxx_unpack(buffer, &(p->MaximumIterations), 1);
    hpcxx_unpack(buffer, &(p->AllocatedSpace), 1);
    hpcxx_unpack(buffer, &(p->SolveMethod), 1);
    hpcxx_unpack(buffer, &(p->KrylovVectors), 1);
    hpcxx_unpack(buffer, &(p->PrecondMeth), 1);
    hpcxx_unpack(buffer, &(p->FillLevels), 1);
    hpcxx_unpack(buffer, &(p->rows), 1);
    hpcxx_unpack(buffer, &(p->BlockSize), 1);
    hpcxx_unpack(buffer, &(p->OutputLevel), 1);
    hpcxx_unpack(buffer, &(p->RecomputePrecond), 1);
    hpcxx_unpack(buffer, &(p->Scaling), 1);
    hpcxx_unpack(buffer, &(p->Tolerance), 1);
    hpcxx_unpack(buffer, &(p->SORRelaxParm), 1);
    hpcxx_unpack(buffer, &(p->DropTol), 1);
    hpcxx_unpack(buffer, &(p->SparsityTarget), 1);
    hpcxx_unpack(buffer, &(p->MILUParam), 1);
  }
}
