#ifndef SPLIBPARMS_H
#define SPLIBPARMS_H

// Control parameters for SPLIB

#include <iomanip.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <ctype.h>

#include <hpcxx_rts.h>

#ifdef REMOVE_
#define getparms_ getparms
#endif

// ------------------
// Fortran functions:
// ------------------
extern "C" {
  void getparms_( double* rpcprm,
                  int* ireltv,
                  int* pcmeth,
                  int* ipcprm,
                  int* solmeth,
                  int* krylov,
                  double* tol,
                  int* maxits,
                  int* iredo,
                  int* iunit,
                  int* instlvl,
                  int* scale);
           };

class SplibParms {
 public:
  int iunit[20];   // Fortran I/O unit numbers for SPLIB instrumentation
  int UnitsOpen;
  int relative;    // Whether tolerance is absolute or relative
  int MaximumIterations;
  int AllocatedSpace;
  int SolveMethod;
  int KrylovVectors;
  int PrecondMeth;
  int FillLevels;
  int rows;
  int BlockSize;  // block size for TRID preconditioner
  int OutputLevel;   // called instlvl in SPLIB: level of instrumentation
  int RecomputePrecond;
  int Scaling;
  double Tolerance;
  double SORRelaxParm;
  double DropTol;
  double SparsityTarget;
  double MILUParam;
 public:
  SplibParms();
  SplibParms(int *iunit,
             int UnitsOpen,
             int relative,    
             int MaximumIterations,
             int AllocatedSpace,
             int SolveMethod,
             int KrylovVectors,
             int PrecondMeth,
             int FillLevels,
             int rows,
             int BlockSize,  
             int OutputLevel,   
             int RecomputePrecond,
             int Scaling,
             double Tolerance,
             double SORRelaxParm,
             double DropTol,
             double SparsityTarget,
             double MILUParam);
  ~SplibParms();
  friend void hpcxx_pack(HPCxx_Buffer& b, SplibParms *sp, int count);
  friend void hpcxx_unpack(HPCxx_Buffer& b, SplibParms *sp, int count);
};

SplibParms getParameters();

#endif
