c======================================================================== 
      subroutine denser( n     ,c_ind , r_point,  omega,
     1                   ilu   ,ilup  ,  iwork ,  space,
     2                   ierr  , mneed )

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

      implicit none
      include '../include/intdbl.inc'
c======================================================================== 
c
c on entry:
c========== 
c  n       = The order of the matrix A.
c  c_ind, r_point = Integer arrays. The structure of matrix A stored in 
c		    CSR format.
c  levfill = Integer. Level of fill-in allowed.
c
c on return:
c=========== 
c
c ilu   = The integer pointer. iwork(ilu) is the array of ILU preconditioners.
c ilup   = Integer array of length n containing the pointers to        
c           the beginning of each row of u in the matrix ldu,ijlu. 
c ierr   = Error message with the following meaning.
c           ierr  =  0   --> successful return.
c           ierr  >  0   --> zero pivot encountered at step number ierr.
c           ierr  = -1   --> not enough storage, mneed contains the
c	                     approximate additional storage needed for ldu
c
c mneed   = contains the actual number of elements in ldu, or the amount
c		of additional storage needed for ldu
c
c work arrays:
c=============
c iwork      = whole working space. All the working arrays are subarray
c	       of this array.
c rowll      = index to the integer array of length n containing pointers 
c	       to implement a linked list for the fill-in elements.
c
c calling routines:
c==================
c
c external functions:
c====================
c
c note:
c======
c the columns of matrix A shall be sorted in increase order.
c======================================================================== 
c
c----------------------------------------------------------------------- 
      integer n,c_ind(*), r_point(*)
      integer ilu,ilup,levfill
      integer lastcol, rowll, levels, remain, nzlu
      integer iwork(*), space
      integer ierr,   mneed
      integer i, nnz_ilu
      real*8  omega
c
c
c Compute pointers into work array.  This version uses two arrays
c dependent on the (unknown in advance) size nzlu, ilu(1:nzlu) and
c levels(1:nzlu).  To handle this, first the data space for the
c work arrays in symbfac is allocated, then half the remainder is
c assigned to ilu, the rest to levels.  The partitioning of the 
c integer work array is as:
c
c         [ilup   ilu      ...      levels    rowll lastcol],
c         [-n-----nzlu-----...-------nzlu-------n------n---]
c
c  The index of the last entry in the work array is space, and
c  rowll and lastcol start at the far end of the work array.
c  Note that these are not necessarily aligned on double word 
c  boundaries.
c
      lastcol = space - n + 1
      rowll = lastcol - n
c
c Set ilup to the far left of the avaiable work array space.
c
      ilup = 1
      ilu  = ilup + n + mod(n,intdbl)
c
c Of space remaining, can allocate half to ilu and half to levels.
c Compute the halfway dividing mark "remain".  Note that
c    growth available = total - lastcol  - rowll  - ilup 
c
      remain = (space - 3*n) / 2
c
c Now levels = storage for ilup + half remaining words.
c
      levels = n                + remain
c
c If remain is nonpositive, estimate 4n as needed storage, 2n for lu
c and 2n for ilu: this will handle a diagonal preconditioner.
c
      if (remain .le. 0) then
         ierr = -1
         mneed = 4*n + 1
         return
      end if
c
c Perform symbolic factorization:
c
      levfill = int(omega)
      call symbfac(n     , c_ind ,r_point,levfill,remain ,nzlu   ,
     1     iwork(ilu)   ,iwork(ilup), iwork(lastcol),iwork(levels),
     2     iwork(rowll), ierr  ,mneed)

	if (ierr .ne. 0) then
           return
	endif
c
c  Merge the pattern with the pattern of A and set index of the pattern
c  to negative. The elements in A only (not in the pattern) are kept positive.
c  Because the pattern is denser than A, only need to set index to be negative.
c  Note that nnz_ilu shown here is actually the number of nonzeros from the
c  off-diagonal.
c
	nnz_ilu = iwork(ilu+n)-n-2
	do i = ilu+n+1, ilu+n+nnz_ilu
	    iwork(i) = -iwork(i)
	enddo
1000	continue
        return
        end
