c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
      subroutine ecimgs(n,a,c_ind,r_point,relax,ilu,alu,ilup,iwork,
     *                maxspace, ierr, mneed)

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

      implicit none
      include '../include/intdbl.inc'
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
c                                                                       *
c	 Extended CIMGS for general sparse matrix.                      *
c   									*
c	This routine performs ECIMGS with different pattern of target   *
c	sparsity patterns. It allows the sparsity pattern be sparser	*
c	than the sparsity pattern of the given matrix A or be denser	*
c	than the sparsity pattenr of A. 				*
c									*
c	ECIMGS requires more storage than the space for storing the	*
c	non zeros of  the factors. Amount of intermediate storage	*
c	is needed.  							*
c									*
c	For More information of ECIMGS, see             		*
c									*
c     Xiaoge Wang                                			*
c     Department of Computer Science				        *
c     Indiana University					        *
c     email: xwang@cs.indiana.edu				        *
c     Fri Aug  5 15:20:35 EST 1994                                      *
c                                                                       *
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
c
c parameters
c-----------
c
c on entry:
c========== 
c n       : integer. the dimension of the matrix A.
c
c a,c_ind,r_point : matrix A stored in compressed sparse row format.
c
c relax   : pattern selection relaxation parameter.
c	    relax = 0.0	    use non zero pattern of matrix A
c	    0.0 < relax < 1.0 use sparser pattern. 
c	    relax = 1.0     use diagonal pattern
c	    relax >1.0      use denser pattern (int(relax) level of fillin)
c
c on return:
c=========== 
c
c ilu, ilup, alu : pointers to the position of preconditioners L and U
c	 in MSR format in the workarray iwork.
c                                                                        
c ierr is an error flag:
c        ierr  = -i --> near zero pivot in step i.
c        ierr  = 0  --> no error is detected.
c        ierr  = 1  --> not enough storage; check mneed.
c
c mneed   = the number of elements in iwork that have been used
c           for the factorization if it is successful, or the number
c	    of additional storage locations needed for lu otherwise.
c
c Work arrays:
c=============
c
c iwork     = integer work array of the whole available space.
c
c maxspace      = size of iwork
c
c Storage requirements:
c======================
c
c  Warnings:
c===========
c
c 1) A must have nonzero diagonal entries, at least.  This does not assure
c    completion of the factorization, however.
c
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
      integer  n, r_point(*), c_ind(*)
      real*8   a(*)
      real*8   relax
      integer  iwork(*), maxspace
      integer  ierr, used, mneed
      integer  ilu, alu, ilup
      integer  nzlu, lstrow, alu_new
c
c Initialize the parameters
c
      ierr  = 0
      mneed = 0
      used  = 0
c
c If parameter of fills is negative, set to 0
c 
      relax = max( relax, 0.0d0)
c
c Perform symbolic factorization:
c on return, iwork will be taken by ilup, ilu. mneed gives the
c size of used space. Total available space after symbolic analysis
c is then (maxspace - mneed).
c
      call symbgs(n, a, c_ind , r_point , relax,
     1             ilu   , ilup  ,  iwork(1)  , maxspace,
     2             ierr  , mneed)
      if ( ierr .ne. 0 ) then
c         print*, 'symbfac fail due to the space shortage', maxspace
	  ierr = 1
          return
      endif
c          
c Perform numerical factorization: more space are needed for numerical
c factorization. n integer for working space and nzlu real*8 for the
c values of the elements in LU. 
c
      nzlu = iwork(ilu+n)-1
      used = ilu + nzlu - 1
      if (maxspace-used .lt. intdbl*nzlu+n) then
         ierr =  1
         mneed = intdbl*nzlu+n - (maxspace-used)
         return
      endif
      alu =  used + 1
      alu =  alu + mod(alu-1,intdbl)
      lstrow = alu + intdbl*nzlu
c      mneed = max(lstrow + n - 1, mneed)
      mneed = n + (1 + intdbl)*nzlu
c
c use numfac without extra real type working space
c
      call  numgs(n, r_point, c_ind, a, iwork(ilu),iwork(ilup),
     1        iwork(alu) , iwork(lstrow), ierr) 
      nzlu = iwork(ilu+n)-1
      if (ierr .ne. 0) then
	  print*, 'error in numerical phase of ECIMGS'
	  ierr = -ierr
	  return
      endif
c
c Collect space
c
      alu_new = ilu+nzlu
      alu_new = alu_new + mod(alu_new-1,intdbl)
      call collect( iwork(alu), iwork(alu_new), nzlu, 1)
      alu = alu_new
c
      return
      end
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
	subroutine collect(a1, a2, n, flag)
c
c	mv real array a1 to array a2. the length of the arrays
c	is n. flag indicate the direction of moving. when flag=0
c	move right. when flag=1, move to left
c	this subroutine is usd to collect the space in a global array.
c
	implicit none
	integer n, flag, i
	real*8 a1(n), a2(n)
	
	if (flag.eq.0) then
c	move right: start from tail of the array
	    do i = n, 1, -1
		a2(i) = a1(i)
	    enddo
	elseif(flag.eq.1) then
c	move left: start form head of the array
	    do i = 1, n
		a2(i) = a1(i)
	    enddo
	else
	    print*, 'direction flag error in subroutin COLLECT'
	endif
	return
	end
