c======================================================================== 
      subroutine  numgs (n, r_point, c_ind, a, ilu, ilup, lu, 
     &        lstrow, ierr)

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

c======================================================================== 
c
c Numerical factorization with given sparsity pattern identified in
c the MSR data structure.  There are few safeguards in this, and
c in particular there is no tolerance test for when to reject a pivot.
c Because of the MSR storage format, the method used is a deferred update
c version.  At the k-th step, the updates from the previous rows m,
c m < k and m corresponding to a nonzero in row k of L/U, are applied
c to row k.  
c
c======================================================================== 
c
      	implicit none
      	integer  n, c_ind(*), r_point(*), ilu(*), ilup(*)
      	real*8   a(*), lu(*)
      	integer  lstrow(n), ierr, istart, lp

      	real*8   li, dk
      	integer  jcol, jli, i, j, k, k1
c Divide by zero tolerance:
	real*8 SMALL, dlamch
	SMALL = sqrt(dlamch('E'))

c
c Initialization
c
      	do k = 1, n
           lstrow(k) = 0
        enddo
c
c Proceed row by row:
c
        do k = 1, n
c 
c Set up the array lstrow that will contains the address that 
c link row k in sparse form.
c zero out row entries which correspond to allowed nonzeros in LU
c
            do j =  ilu(k),  ilu(k+1)-1
	        lstrow( abs(ilu(j)))  = j
                lu(j) = 0.0d0
	    enddo
c
c Set the diagonal entry:
c
            lstrow(k) =  k
            lu(k) =  0.0d0
C
c Set entries of row corresponding to original nonzeros of A to
c those values from A:
c
            do j = r_point(k),r_point(k+1)-1
		k1 = c_ind(j)
		if (lstrow(k1).ne. 0) then
		    lu(lstrow(k1)) = a(j)
		endif
	    enddo
c
c At this point, row(:) contains the k-th row of LU, both upper and 
c lower triangular parts.  All but possibly the first (k,1) entry
c will need to have previous updates applied.
c
c Process the k-th row of LU, by updating it with the multiplier
c in (k,i) times U(i,i+1:n):
c
	    do i =  ilu(k), ilup(k)-1
c
c jli is the column index of an entry in L in row k, and
c li is the corresponding value in that (k,jli) position.
c This becomes the multiplier lu(i) = L(k,jli) on this stage of
c processing row k:
c
                jli =  ilu(i)
	        if (jli .gt. 0) then
c               only element in R could be fill in
                    li =  lu(i)*lu(jli)
                    lu(i) =  li
                    do j = ilup(jli),  ilu(jli+1)-1
                        jcol      =  ilu(j)
		        if (jcol.lt.0) then
			    jcol = -jcol
			    if (lstrow(jcol) .ne. 0) then
                                lu(lstrow(jcol))=lu(lstrow(jcol))-
     &                                        li*lu(j)
			    endif
                        endif
		    enddo
	        else
c	 	all elements can be fillin
		    jli = -jli
		    li =  lu(i)*lu(jli)
                    lu(i) =  li
                    do j = ilup(jli),  ilu(jli+1)-1
                        jcol      =  abs(ilu(j))
		        if (lstrow(jcol) .ne. 0) then
                            lu(lstrow(jcol)) = 
     &                            lu(lstrow(jcol))-li*lu(j)
                        endif
		    enddo
	        endif
	    enddo
c
c  At this point, all the updates to row k are complete.  The
c  diagonal entry is the corresponding one of L, and will be
c  stored in inverted form to avoid divisions in further processing
c  and in the triangular solves with L.  Note that the test is
c  against exact zero, and should be replaced with a tolerance.
c
            if (abs(lu(k)) .le. SMALL) then
c                ierr  =  k
c                return
                 lu(k) = sign(SMALL,lu(k))
            endif
            dk = 1.0d0/lu(k)
            lu(k) = dk
c
c Now scale the upper triangular entries in row k using the
c inverted diagonal entry:
c
c         do 100 j = ilup(k), ilu(k+1)-1
c            lu(j) = row( ilu(j) )
c            lu(j) = row( ilu(j) ) * dk
c 100     continue
c
c Clear the vector lstrow before starting the next row
c
          do j = ilu(k), ilu(k+1)-1
              lstrow(abs(ilu(j))) = 0
          enddo
          lstrow(k) = 0
c Go to next row.
c
	enddo
c
c Extract R out,
c
	lp = n+2
	istart = ilu(1)
	do i = 1, n
	    do k = istart, ilup(i)-1
		if (ilu(k).lt.0) then
		    ilu(lp) = -ilu(k)
		    lu(lp) = lu(k)
		    lp = lp + 1
		endif
	    enddo
	    istart = ilup(i)
	    ilup(i) = lp
	    do k = istart, ilu(i+1)-1
                if (ilu(k).lt.0) then
                    ilu(lp) = -ilu(k)
		    lu(lp) = lu(k)
                    lp = lp + 1
                endif
	    enddo
	    istart = ilu(i+1)
	    ilu(i+1) = lp
	enddo
c
	ierr  = 0
        return
        end
