c======================================================================== 
      	subroutine ssor (n,a,colind,rwptr,omega,lu,jlu,uptr,work,
     &			 storg,ierr)

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

c======================================================================== 
c                      *** SSOR preconditioner ***			*
c									*
c  Set up implicitly the SSOR preconditioning matrix.  Let		*
c  A = L + D + U be the usual splitting, with D diagonal, L strictly	*
c  lower triangular, and U strictly upper triangular.  Because an 	*
c  additional matrix for L and U need not be created, this routine	*
c  simply sets up pointers to the diagonal entries of A for use		*
c  in the triangular solves needed.  It also orders the row entries 	*
c  of A.  								*
c									*
c  The preconditioner is given by 					*
c      M = (I+omega*L*D^-1) * (D/omega+U)				*
c  where omega in (0,2) is the SSOR acceleration parameter.		*
c  Note that the usual way of writing this is as			*
c     M = (1/(2-omega)) * (D/omega+L)*(omega*D^-1) * (D/omega+U)	*
c  and so I have left off the scaling factor 1/(2-omega) and have	*
c  combined the last two factors.  The second form shows that when	*
c  A is symmetric, so is the SSOR preconditioner.			*
c									*
c  The calling sequence of this routine has lotsa garbage that is 	*
c  not needed, to keep it consistent with the explicit setup of   	*
c  SSOR in an earlier version.                                    	*
c									*
c  Note: the diagonal of A must be larger than the parameter		*
c        SMALL, set in this routine as a parameter.			*
c									*
c     Randall Bramley							*
c     Department of Computer Science					*
c     Indiana University						*
c     email: bramley@cs.indiana.edu					*
c     Wed Jun 22 14:57:13 EST 1994					*
c									*
c======================================================================== 
c									*
c PARAMETERS
c-----------
c
c on entry:
c========== 
c n       = integer. The dimension of the matrix A.
c
c a,colind,rwptr = matrix stored in Compressed Sparse Row format.
c
c work    = dummy work array 
c
c omega   = real*8 relaxation parameter omega for ssor.
c           Theory requires  0 < omega < 2.
c
c On return:
c=========== 
c
c lu   = parameter omega
c jlu  = dummy array
c uptr = integer array of length n containing pointers to        
c           the diagonal entry of each row of A.
c                                                                       
c storg   = integer. Amount of real*8 storage needed for precond (always 1)
c
c  ierr is an error flag:
c        ierr  = -i --> near zero pivot in step i
c        ierr  = 0  --> all's OK
c        ierr  = 1  --> not enough storage; check mneed.
c        ierr  = 2  --> illegal parameter
c
c======================================================================== 

	implicit none

c       Declaration of arguments
c       ========================
	integer rwptr(*), colind(*), jlu(*), uptr(*), n, ierr
	real*8 work(*), a(*), lu, omega
	integer storg

c       Local variables
c       ===============
	integer i,k,is,ie
	real*8 SMALL, dlamch


c=======================================================================
c     Start of executable statements
c=======================================================================

c       -------------------------------
c       Set tolerance for safe division
c       -------------------------------
	SMALL = sqrt(dlamch('S'))

	storg = 1
	ierr = 0

c       ------------------------------
c       Check for omega in right range
c       ------------------------------
	if (omega .le. 0.0d0 .or. omega .gt. 2.0d0) then
           ierr = 2
           return
        end if

	lu = omega

c       ----------------------------
c       Proceed row by row through A
c       ----------------------------

        do 60 k = 1, n

c          -----------------------------------------
c          Start indices of rows k and k+1 in matrix
c          -----------------------------------------
           is = rwptr(k)
           ie = rwptr(k+1)

c          ------------------------------------------------
c          Sort k-th row into increasing column index order.
c          ------------------------------------------------
           call rowinc( ie - is, colind(is), a(is) )

c          ------------------------------
c          Find pointer to diagonal entry
c          ------------------------------
           uptr(k) = 0

           do 20 i = is, ie-1
              if (colind(i) .eq. k) then
                 uptr(k) = i
                 go to 40
              end if
 20        continue

c          --------------------------------------------
c          Error condition if diagonal entry is missing
c          --------------------------------------------
           ierr = -k
           return

 40        continue

c          ----------------------------------------------
c          Error condition if diagonal entry is near-zero
c          ----------------------------------------------
           if (abs(a(i)) .le. SMALL) then
              ierr = -k
              return
           end if

 60     continue

	return
	end			
