c=========================================================================
      subroutine  bicgs (A,colind,rwptr,n,
     1                   x,r,b,
     2                   tolrnc,reltv,maxits,ierr,
     3                   work,wksz,
     4                   lu,jlu,uptr,pcmeth,iunit,instlvl)

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

c-----------------------------------------------------------------------*
c                           *** Bi-CGSTAB ***				*
c									*
c  Van der Vorst's biconjugate gradients stabilized algorithm.		*
c  This version is a direct translation of the Templates version to	*
c  the SPLIB paradigm.  Because it uses a different change of variables	*
c  to handle the preconditioning, its performance and convergence	*
c  can differ greatly from the CGSTAB routine of SPLIB, which in	*
c  theory implements the same algorithm.  Double (v = v + a*w + b*u)	*
c  vector updates are done in a single loop.  For machines like the	*
c  IBM RS/6000, this gives a more accurate update than using a sequence	*
c  of scalings and vector updates because the combined multiply/add	*
c  unit retains the full result of a multiply (106 bit mantissa).	*
c									*
c-----------------------------------------------------------------------*
c
c  Arguments:
c  ----------
c
c     (Data structure for A)
c  n       : Order of matrix A.
c  A       : Real*8 array for nonzero entries of matrix A; of
c            length nnz = rwptr(n+1)-1.
c  colind  : Integer array of column indices for entries in A; of
c            length nnz = rwptr(n+1)-1.
c  rwptr   : Integer array of pointers to beginning position of rows
c            in arrays A and colind.  Must be of length n+1.
c
c     (Required vectors)
c  x       : Vector of unknowns to be solved for.  Contains initial
c            estimate of x on entry, and contains final approximate
c            on return.
c  r       : Residual vector r = b - A*x.
c  b       : Right hand side vector of system to be solved: Ax = b
c  work    : Double precision array of additional storage.  Bicg
c            requires 6 vectors of length n, and if instrumentation
c            routine instr.f is called, an additional 2 vectors of
c            length n are required.  If not enough storage is given,
c            nothing is done and an error code is returned specifying
c            the additional storage needed (in double precision words).
c  wksz    : Length of array work, in double precision words.  
c
c     (Preconditioner data structure)
c  lu,jlu  : Preconditioner in Modified Sparse Row format containing
c            the L and U factors together. The diagonal of U (stored in
c            lu(1:n) ) is inverted.  The i-th row of the lu,jlu matrix 
c            contains the i-th row of L (excluding the diagonal entry) 
c            followed by the i-th row of U.  The diagonal of L has
c            all ones, and so is not stored explicitly.
c  uptr    : Integer array of length n containing the pointers to        
c            the beginning of each row of U in the matrix lu,jlu. 
c  pcmeth  : Preconditioning method to use.  See splib.f documentation
c            for the interpretation of this integer.
c
c     (Iteration control)
c  tolrnc  : Tolerances on norm(s) used in stopping test.  This is
c            passed in as a vector in case future versions
c            set pdr and SMALL, rather than using
c            the dlamch() function.  Since Krylov methods generally
c            use the dotproduct r'*r, a local variable tolsq is used
c            in order to avoid square roots.  Note: because the
c            preconditioned residual norm || inv(M)*r || is the natural
c            one tracked by the iterative method, but users generally
c            want the true residual norm || r || as the stopping test,
c            this routine will exit the iterations when the precond
c            residual is small enough and then test the true residual.
c            If it is not sufficiently reduced, then the local tolerance
c            tolsq is reduced and iterations resume.  This is done
c            a maximum of eight times before failure is declared and
c            the routine returns.
c  reltv   : Logical variable indicating whether or not to use a
c            relative residual as the stopping test.  If this is
c            false, the stopping test is || r_k || < tolrnc(1).  If it
c            is true, the test is || r_k || < || r_0 || * tolrnc(1).
c            Note that it is possible to modify SPLIB to use other
c            stopping tests (see routine instr.f).
c  maxits  : Maximum number of iterations allowed.
c
c      (Other stuff)
c  ierr    : Error flag; see below.
c  iunit   : Integer vector of unit numbers for the instrumentation
c            routine instr.f to use.  Not referenced by bicgs, just
c            passed along to instr.f
c  instlvl : Integer giving instrumentation level.  If instlvl = 0,
c            then instr.f is not called and array work() need not
c            provide the 2n double precision words used by instr.f
c
c  Some internal variables:
c  ------------------------
c
c  early   : Logical parameter that indicates if a tentative exit
c            was made because of early termination.  This is needed
c            because of the way SPLIB uses unpreconditioned residuals
c            as the stopping test.
c  first   : Logical parameter that specifies to instr.f if this is
c            the first iteration, and so if new headers need to be
c            written to the Xgraph output files.
c  SMALL   : Parameter that determines when it is not safe to divide
c            by a scalar quantity.  Error condition is set if a divisor
c            is smaller than SMALL.  Set by dlamch() function.
c  pdr     : Parameter to determine if preconditioner is no good.
c            Experience shows that if the preconditioned residual norm
c            is more than 1/eps, where eps is the machine epsilon, then
c            even if the preconditioned residual converges, the original
c            system residual does not.  If you are not comfortable with
c            this try setting pdr = 1.0d200.  
c  urnorm  : Unpreconditioned initial residual.  Used for tests to see
c            if initial residual is good enough, and if preconditioner
c            is no good (see pdr above).
c  Mrnorm  : preconditioned initial residual.
c
c  rtld,t,
c  p,ptld,
c  stld,v  : Integer pointers in the array work(*) that give the
c            starting position of the corresponding vector used by BICGS.
c            Note that the array s in Templates bicgs is identified
c            with r in this routine.
c  wins1,
c  wins2   : Integer pointers in the array work(*) that give starting
c            positions of two vectors used by instrumentation routines.
c
c======================================================================
c
c Return values:                                                           
c --------------
c
c  x      : Contains an approximate solution (upon successful return).  
c
c  ierr   : Error flag with the following meanings:          
c           ierr = -m --> Not enough workspace provided; m more double
c                         precision words needed in array work().
c           ierr = 0  --> Successful return; tolerance achieved.
c           ierr = 1  --> The initial guess satisfies the stopping test.
c                         This is often a clue that rhs not set properly.
c           ierr = 2  --> Tolerance reset 8 times in attempt to
c                         get true residual small enough.
c           ierr = 3  --> convergence not achieved in maxits iterations.  
c           ierr = 4  --> Preconditioned residual is 1/eps times
c                         larger than unpreconditioned residual.
c           ierr = 5  --> error condition of abs(omega) <= SMALL.
c           ierr = 6  --> error condition of abs(rho) <= SMALL.
c
c  maxits : Contains the number of actual iterations performed.
c  tolrnc : May be modified if reltv = .true.
c                                                                       
c======================================================================
c  Routines
c  --------
c
c  lusl    : Solves M z = r.
c  bmux    : Computes matrix-vector product with A.
c  instr   : Instrumentation routine for timing, etc.
c  dlamch  : Function from LAPACK project, for determining machine
c            constants.  This provides machine epsilon and the safe
c            invertible floating point minimum. 
c  BLAS routines: ddot, daxpy, dcopy
c
c======================================================================
c
c     Randall Bramley
c     Department of Computer Science
c     Indiana University
c     email: bramley@cs.indiana.edu
c     Mon Jul 25 12:39:12 EST 1994
c     Documented Tue Jul  4 15:19:35 EST 1995
c     Approved by Radha, Tue Jul  4 16:20:57 EST 1995
c
c----------------------------------------------------------------------
       implicit none

c     -------------------------
c     Declaration of arguments:
c     -------------------------
       integer n
       real*8 A(*)
       real*8 x(*), b(*), r(*)
       real*8 work(n,*)
       real*8 tolrnc(*)
       real*8 lu(*)

       integer colind(*), rwptr(*)
       integer wksz
       integer maxits, ierr
       integer jlu(*), uptr(*)
       integer pcmeth, iunit(*), instlvl
       logical reltv

c     ------------------
c     External functions
c     ------------------
      real*8 ddot, dnrm2 , dlamch
      external dlamch, ddot, dnrm2

c     ------------- 
c     Local storage 
c     ------------- 
      logical first, early
      integer i, its, resets
      real*8 alpha, beta, rho, rho1, omega, tolsq,temp, res
      real*8 SMALL, pdr, Mrnorm, rnorm

c     -------------------------------
c     Pointers for work array vectors
c     -------------------------------
      integer rtld, p, v, t, ptld, stld, wins1, wins2

c=======================================================================
c     Start of executable statements
c=======================================================================

c     ------------------------------
c     Set up square root of safe min
c     ------------------------------
      SMALL = sqrt(dlamch('S'))

c     ---------------------------------
c     Set up inverse of machine epsilon
c     ---------------------------------
      pdr = 1.0d0/dlamch('E')

c----------------------------------------------------------------------
c
c  Key to vectors in array work
c
c       s    -> r          (not used here; r is just substituted)
c       rtld -> work(:,1)  (arbitrary fixed vector)
c       p    -> work(:,2)
c       v    -> work(:,3)
c       t    -> work(:,4)
c       ptld -> work(:,5)
c       stld -> work(:,6)
c
c----------------------------------------------------------------------

c     -------------------------------------
c     Set up pointers to vectors in work(*)
c     -------------------------------------
      if (instlvl .gt. 0) then
         wins1 = 1
         wins2 = wins1 + 1
      else
         wins1 = 0
         wins2 = 0
      end if
      rtld = wins2 + 1
      p    = rtld + 1
      v    = p + 1
      t    = v + 1
      ptld = t + 1
      stld = ptld + 1

c     ==========================
c     Initialize for iterations:
c     ==========================

      first = .true.
      resets = 0
      tolsq = tolrnc(1)*tolrnc(1)

c     ----------------------------------------
c     Return if not enough workspace provided:
c     ----------------------------------------
      if (wksz .lt. (wins2+6)*n) then
         ierr = wksz - (wins2+6)*n
         maxits = 0
         return
      end if 

c     --------------------------------------------
c     Indication of early termination in middle of
c     computing two vectors for updating x:
c     --------------------------------------------
      early = .false.
      its = 0

c     ---------------------------
c     Initial residual r = b - Ax
c     ---------------------------
      call bmux(n,n,x,r,A,colind,rwptr)
      do i = 1,n
         r(i) = b(i) - r(i)
      end do
      rnorm = ddot(n,r,1,r,1)

c     -------------------------------------------------
c     Adjust tolerance if relative residual to be used:
c     -------------------------------------------------
      if (reltv) then
          tolsq = tolsq*rnorm
          tolrnc(1) = tolrnc(1)*sqrt(rnorm)
      end if

c     -----------------------------------------
c     Stop if initial residual is small enough:
c     -----------------------------------------
      if (rnorm .le. tolsq) go to 900

c     ------------------------------------------
c     Note: r-tilde can be set arbitrarily here.
c     ------------------------------------------
      call dcopy( n, r, 1, work(1,rtld), 1 )


c=======================================================================
c     Begin main loop of iterations.  Loop is equivalent to
c      do while (rnorm .gt. tolsq) .and. (its .le. maxits)
c=======================================================================

 20      continue

         call instr(A,colind,rwptr,n, x,b,work(1,wins1), lu,jlu,uptr,
     &              pcmeth,first,its, iunit,instlvl)
         its = its + 1
         rho = ddot(n,work(1,rtld),1,r,1)
         if (abs(rho) .le. SMALL) go to 930

c        ----------------------------
c        p = beta*(p - omega*v) + r
c        (on first iteration, p = r).
c        ----------------------------
         if (its .gt. 1) then
            beta = (rho/rho1)* (alpha/omega)
            do i = 1, n
               work(i,p) = r(i) +
     &                     beta*(work(i,p) - omega*work(i,v))
            end do
         else
            call dcopy( n, r, 1, work(1,p), 1 )
         end if

c        ----------------
c        Solve M*ptld = p
c        ----------------
         call lusl(n,work(1,p),work(1,ptld),lu,jlu,uptr,pcmeth,
     &             A,colind,rwptr)

c        ------------------------------------------
c        Check initial preconditioned residual size 
c        ------------------------------------------
         if (its .eq. 1) then
            Mrnorm = ddot(n, work(1,ptld), 1, work(1,ptld), 1)
            if (Mrnorm .gt. pdr*pdr*rnorm) then
               ierr = 4
               maxits = 0
               return
            end if 
         end if 

c        --------------
c        Set v = A*ptld
c        --------------
         call bmux(n, n,work(1,ptld),work(1,v),A,colind,rwptr)

c        ------------------
c        alpha = rho/v'rtld
c        ------------------
         alpha = rho/ddot( n, work(1,rtld), 1, work(1,v), 1 )

c        ------------------------------------------------
c        s = r - alpha*v.  Note s and r are aliased here.
c        ------------------------------------------------
         do i = 1, n
            r(i) = r(i) - alpha*work(i,v)
         end do

c        ---------------------------------------
c        Early termination if s is small enough:
c        ---------------------------------------
         if ( ddot(n,r,1,r,1) .le. tolsq) then

c           -------------------------------------------------------------
c           Indicate early termination in case true res not small enough.
c           -------------------------------------------------------------
            early = .true.

c           -------- 
c           Update x 
c           -------- 
            call daxpy( n, alpha, work(1,ptld), 1, x, 1 )
            go to 910
         end if 
 30      continue

c        -----------------------------------------
c        Compute stabilizer vector stld = inv(M)*s
c        -----------------------------------------
         call lusl(n,r,work(1,stld),lu,jlu,uptr,pcmeth,A,colind,rwptr)

c        ------------------
c        Compute t = A*stld
c        ------------------
         call bmux(n, n,work(1,stld),work(1,t),A,colind,rwptr)
         omega = ddot( n, work(1,t), 1, r, 1 ) /
     &           ddot( n, work(1,t), 1, work(1,t), 1 )

c        --------
c        Update x
c        --------
         do i = 1, n
            x(i) = x(i) + alpha*work(i,ptld) + omega*work(i,stld)
         end do

c        --------------------------------------
c        Update residual vector r = s - omega*t
c        --------------------------------------
         call daxpy( n, -omega, work(1,t), 1, r, 1 )
         res = ddot(n,r,1,r,1)
         if (res .le. tolsq) go to 910
         if (its .ge. maxits) go to 920
         if (abs(omega) .le. SMALL) go to 940
         rho1 = rho
         go to 20

c=======================================================================
c                  Stopping conditions
c=======================================================================

c     ====================
c     (Almost) exact guess
c     ====================
 900  continue
      ierr = 1
      maxits = 0
      return

c     ========================================= 
c     Possible convergence; check true residual 
c     ========================================= 
 910  continue
      ierr = 0

c     Stop if residual tolerance reset 8 times.
c     -----------------------------------------
      resets = resets + 1
      if (resets .gt. 8) then
         ierr = 2
         go to 915
      end if
      call bmux(n,n,x,work(1,ptld),A,colind,rwptr)
      do i = 1, n
         work(i,ptld) = b(i) - work(i,ptld)
      end do
      res = ddot(n,work(1,ptld),1,work(1,ptld),1)
      temp = tolrnc(1)*tolrnc(1)
      if (res .gt. temp) then
          tolsq = tolsq*temp/res
c          write(6,*)'>>>> tolerance reset to ', sqrt(tolsq),'  <<<<<<'
          if (early) then
             early = .false.
             alpha = 0.0d0
             go to 30
          end if
          go to 20
      end if
 915  continue
      maxits = its
      call instr(A,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its, iunit,instlvl)
      return

c     ========================================
c     No convergence in max allowed iterations
c     ========================================
 920  continue
      ierr = 3
      maxits = its
      call instr(A,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its, iunit,instlvl)
      return

c     =============
c     rho too small
c     =============
 930  continue
      ierr = 6
      maxits = its
      call instr(A,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its, iunit,instlvl)
      return

c     ===============
c     omega too small
c     ===============
 940  continue
      ierr = 5
      maxits = its
      call instr(A,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its, iunit,instlvl)
      return

c================= End of BICGS =======================================

      end
