c=======================================================================*
      subroutine cscgstab(A,colind,rwptr,n,
     1                    x,r,b,
     2                    tolrnc,reltv,maxits,ierr,force,
     3                    work,wksz,
     4                    lu,jlu,uptr,pcmeth, iunit,instlvl)

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

c-----------------------------------------------------------------------*
c                       *** CSCGSTAB ***				*
c  The composite step stabilized biconjugate gradient algorithm of Chan *
c  and Szego.  This is from the article in the November 1996 issue of   *
c  SIAM J. Sci. Comp., and may not be completely correct.  Furthermore, *
c  the preconditioning it performs is a very simple-minded left pc	*
c  without any
c									*
c-----------------------------------------------------------------------*
c
c  This routine is just a wrapper for the subroutine cscgs.f, which is
c  in the same file below.  See it for a description of all the variables.
c  This wrapper carves up the workspace vector work() and then calls
c  cscgs, so the only error code relevant is ierr < 0, indicating not
c  enough memory.
c
c======================================================================
c  Routines
c  --------
c
c  cscgs   : This is the actual work routine.  See it for explanation
c            of the variables used.
c
c======================================================================
c
c     Randall Bramley
c     Department of Computer Science
c     Indiana University
c     email: bramley@cs.indiana.edu
c     Sun Nov 10 13:13:45 EST 1996
c
c======================================================================

c     -------------------------
c     Declaration of arguments:
c     -------------------------
      implicit none

      integer n
      real*8 A(*)
      real*8 x(*), b(*), r(*)
      real*8 work(*)
      real*8 tolrnc(*)
      real*8 lu(*)

      integer colind(*), rwptr(*)
      integer wksz
      integer maxits, ierr, need
      integer jlu(*), uptr(*)
      integer pcmeth, iunit(*), instlvl
      logical reltv, force

c     ------------------------------
c     Pointers to work array vectors
c     ------------------------------
      integer rtilde, p, e, q, c, u, y, d, rhat, ehat
      integer s, t, w, v, rhat2, ehat2, zt, wt, wins1

c=======================================================================
c     Start of executable statements
c=======================================================================

c     ---------------------------
c     Check the available storage
c     ---------------------------

      need = 18
      if (instlvl .ge. 0) need = need + 2

      if (n*need .gt. wksz) then
         ierr = wksz - n*need
         maxits = 0
         return
      end if

c     -----------------------------------------
c     Set up pointers for vectors in work array
c     -----------------------------------------
      rtilde = 1
      p = rtilde + n
      e = p + n
      q = e + n
      c = q + n
      u = c + n
      y = u + n
      d = y + n
      rhat = d + n
      ehat = rhat + n
      s = ehat + n
      t = s + n
      w = t + n
      v = w + n
      rhat2 = v + n
      ehat2 = rhat2 + n
      zt = ehat2 + n
      wt = zt + n
      wins1 = wt + n

      call cscgs(A,colind,rwptr,n,
     1          x,r,b,
     2          work(rtilde),work(p),work(e),work(q),work(c),
     3          work(u),work(y),work(d),work(rhat),work(ehat),work(s),
     4          work(t),work(w),work(v),work(rhat2),work(ehat2),
     5          work(zt),work(wt),work(wins1),
     6          tolrnc,reltv,maxits,ierr,force,
     7          lu,jlu,uptr,pcmeth, iunit,instlvl)

      return

c================= End of CSCGSTAB =====================================

      end


c=======================================================================*
      subroutine  cscgs(A,colind,rwptr,n,
     1                  x,r,b,
     2                  rtilde,p,e,q,c,u,y,d,rhat,ehat,s,t,w,v,
     3                  rhat2,ehat2,
     4                  zt, wt, wins1,
     5                  tolrnc,reltv,maxits,ierr,force,
     6                  lu,jlu,uptr,pcmeth, iunit,instlvl)

c**********************************************************************
c                   SPLIB Copyright (C) 1995                         **
c                     Indiana University                             **
c**********************************************************************

c-----------------------------------------------------------------------*
c                         *** CSCGSTAB ***				*
c  The composite step stabilized biconjugate gradient algorithm.       	*
c									*
c-----------------------------------------------------------------------*
c
c  Arguments:
c  ----------
c
c     (Data structure for A)
c  n       : Order of matrix A.
c  A       : Real*8 array for nonzero entries of matrix A; of
c            length nnz = rwptr(n+1)-1.
c  colind  : Integer array of column indices for entries in A; of
c            length nnz = rwptr(n+1)-1.
c  rwptr   : Integer array of pointers to beginning position of rows
c            in arrays A and colind.  Must be of length n+1.
c
c     (Required vectors)
c  x       : Vector of unknowns to be solved for.  Contains initial
c            estimate of x on entry, and contains final approximate
c            on return.
c  r       : Residual vector r = b - A*x.
c  b       : Right hand side vector of system to be solved: Ax = b
c  rtilde,p,e,q,c,u,y,d,rhat,ehat,s,t,w,v,rhat2,ehat2 :
c                                      Cast of thousands of vectors.
c
c     (Preconditioner data structure)
c  lu,jlu  : Preconditioner in Modified Sparse Row format containing
c            the L and U factors together. The diagonal of U (stored in
c            lu(1:n) ) is inverted.  The i-th row of the lu,jlu matrix 
c            contains the i-th row of L (excluding the diagonal entry) 
c            followed by the i-th row of U.  The diagonal of L has
c            all ones, and so is not stored explicitly.
c  uptr    : Integer array of length n containing the pointers to        
c            the beginning of each row of U in the matrix lu,jlu. 
c  pcmeth  : Preconditioning method to use.  See splib.f documentation
c            for the interpretation of this integer.
c
c     (Iteration control)
c  tolrnc  : Tolerances on norm(s) used in stopping test.  This is
c            passed in as a vector in case future versions
c            set pdr and SMALL, rather than using
c            the dlamch() function.  Since Krylov methods generally
c            use the dotproduct r'*r, a local variable tol is used
c            in order to avoid square roots.  Note: because the
c            preconditioned residual norm || inv(M)*r || is the natural
c            one tracked by the iterative method, but users generally
c            want the true residual norm || r || as the stopping test,
c            this routine will exit the iterations when the precond
c            residual is small enough and then test the true residual.
c            If it is not sufficiently reduced, then the local tolerance
c            tol is reduced and iterations resume.  This is done
c            a maximum of eight times before failure is declared and
c            the routine returns.
c  reltv   : Logical variable indicating whether or not to use a
c            relative residual as the stopping test.  If this is
c            false, the stopping test is || r_k || < tolrnc(1).  If it
c            is true, the test is || r_k || < || r_0 || * tolrnc(1).
c            Note that it is possible to modify SPLIB to use other
c            stopping tests (see routine instr.f).
c  maxits  : Maximum number of iterations allowed.
c  force   : Logical parameter to force the method to always use one-step.
c                 force = .true.  ---> use one step always
c                 force = .false. ---> use composite step 
c
c      (Other stuff)
c  ierr    : Error flag; see below.
c  iunit   : Integer vector of unit numbers for the instrumentation
c            routine instr.f to use.  Not referenced by cscgstab, just
c            passed along to instr.f
c  instlvl : Integer giving instrumentation level.  If instlvl = 0,
c            then instr.f is not called and array work() need not
c            provide the 2n double precision words used by instr.f
c
c  Some internal variables:
c  ------------------------
c
c  first   : Logical parameter that specifies to instr.f if this is
c            the first iteration, and so if new headers need to be
c            written to the Xgraph output files.
c  SMALL   : Parameter that determines when it is not safe to divide
c            by a scalar quantity.  Error condition is set if a divisor
c            is smaller than SMALL.  Set by dlamch() function.
c  pdr     : Parameter to determine if preconditioner is no good.
c            Experience shows that if the preconditioned residual norm
c            is more than 1/eps, where eps is the machine epsilon, then
c            even if the preconditioned residual converges, the original
c            system residual does not.  If you are not comfortable with
c            this try setting pdr = 1.0d200.  
c  urnorm  : Unpreconditioned initial residual.  Used for tests to see
c            if initial residual is good enough, and if preconditioner
c            is no good (see pdr above).
c
c  wins1   : vector of length 2n used by instrumentation routines.
c            Can be null if instlvl < 0.
c
c======================================================================
c
c Return values:                                                           
c --------------
c
c  x      : Contains an approximate solution (upon successful return).  
c
c  ierr   : Error flag with the following meanings:          
c           ierr = -m --> Not enough workspace provided; m more double
c                         precision words needed in array work().
c           ierr = 0  --> Successful return; tolerance achieved.
c           ierr = 1  --> The initial guess satisfies the stopping test.
c                         This is often a clue that rhs not set properly.
c           ierr = 2  --> Tolerance reset 8 times in attempt to
c                         get true residual small enough.
c           ierr = 3  --> convergence not achieved in maxits iterations.  
c           ierr = 4  --> Preconditioned residual is 1/eps times
c                         larger than unpreconditioned residual.
c           ierr = 5  --> omega1 small divisor 
c           ierr = 6  --> mu small divisor
c           ierr = 7  --> omegah small divisor
c           ierr = 8  --> omega2 small divisor
c           ierr = 9  --> sigma small divisor
c           ierr = 10 --> mu (second location) small divisor
c           ierr = 11 --> rho small divisor
c           ierr = 12 --> delta small divisor
c           ierr = 13 --> gamma small divisor
c
c  maxits : Contains the number of actual iterations performed.
c  tolrnc : May be modified if reltv = .true.
c                                                                       
c======================================================================
c  Routines
c  --------
c
c  lusl    : Solves M z = r.
c  bmux    : Computes matrix-vector product with A.
c  instr   : Instrumentation routine for timing, etc.
c  dlamch  : Function from LAPACK project, for determining machine
c            constants.  This provides machine epsilon and the safe
c            invertible floating point minimum. 
c  BLAS routines: ddot, daxpy, dcopy, dnrm2
c
c======================================================================
c
c     Randall Bramley
c     Department of Computer Science
c     Indiana University
c     email: bramley@cs.indiana.edu
c     Sun Nov 10 09:34:41 EST 1996
c
c======================================================================

c     -------------------------
c     Declaration of arguments:
c     -------------------------
      implicit none

      integer n
      real*8 A(*)
      real*8 x(n), b(n), r(n)
      real*8 rtilde(n),p(n),e(n),q(n),c(n),u(n),y(n)
      real*8 d(n),rhat(n),ehat(n),s(n),t(n),w(n),v(n),z
      real*8 rhat2(n),ehat2(n), zt(n), wt(n)
      real*8 wins1(*)
      real*8 tolrnc(*)
      real*8 lu(*)

      integer colind(*), rwptr(*)
      integer maxits, ierr
      integer jlu(*), uptr(*)
      integer pcmeth, iunit(*), instlvl
      logical reltv, force

c     ------------------
c     External functions
c     ------------------
      real*8 ddot, dlamch, dnrm2
      external dlamch, ddot, dnrm2

c     -------------
c     Local storage 
c     -------------
      logical first
      logical onestep
      integer i, its, resets, k
      integer steps(2)
      real*8 res, tol
      real*8 rho, phi, mu, sigma, omega1, omega2, psi, omegah
      real*8 a11, a12, a21, a22, b1, b2, alpha1, alpha2
      real*8 delta, gamma1, gamma2, nu2, beta1, beta2, nutilde
      real*8 SMALL, urnorm, pdr
      real*8 temp1, temp2

c=======================================================================
c     Start of executable statements
c=======================================================================

c     ------------------------------
c     Set up square root of safe min
c     ------------------------------
      SMALL = sqrt(dlamch('S'))

c     ---------------------------------
c     Set up inverse of machine epsilon
c     ---------------------------------
      pdr = 1.0d0/dlamch('E')

c----------------------------------------------------------------------

c     ---------------------
c     Initialize iterations
c     ---------------------
      steps(1) = 0
      steps(2) = 0
      resets = 0
      first = .true.
      tol = tolrnc(1)
      its = 0
      ierr = 0

c     ------------------
c     r = A*x; r = b - r
c     ------------------
      call bmux(n, n,x,p,A,colind,rwptr)
      do i = 1,n
         p(i) = b(i) - p(i)
      end do
      urnorm = ddot(n,p,1,p,1)
      call lusl(n, p, r, lu, jlu, uptr, pcmeth, A,colind,rwptr)
      do i = 1,n
         p(i) = r(i)
         rtilde(i) = r(i)
      end do

c     -------------------------------------------------------------
c     Check to see if preconditioned residual is too large relative
c     to unpreconditioned residual:
c     -------------------------------------------------------------
      if (ddot(n,r,1,r,1) .gt. pdr*pdr*urnorm) then
         ierr = 4
         maxits = 0
         return
      end if 

      rho = ddot(n, rtilde, 1, r, 1)
      phi = dnrm2(n, r, 1)

c     --------------
c     e = A*r; q = e
c     --------------
      call bmux(n, n,r,e,A,colind,rwptr)
      call lusl(n, e, q, lu, jlu, uptr, pcmeth, A,colind,rwptr)
      call dcopy(n,q,1,e,1)
      mu = 1

c     -------------------------------------------------
c     Adjust tolerance if relative residual to be used:
c     -------------------------------------------------
      if (reltv) then
          tol = tol*phi
          tolrnc(1) = tolrnc(1)*phi
      end if

      if (phi .lt. tol) go to 900


c=======================================================================
c     Begin main loop of iterations.  Loop is equivalent to
c      do while (phi .gt. tol) .and. (its .le. maxits)
c=======================================================================

 2       continue
         call instr(A,colind,rwptr,n, x,b, wins1, lu,jlu,uptr,
     &               pcmeth,first,its, iunit,instlvl)

         a11 = ddot(n,rtilde,1,q,1) 
         sigma = a11*mu

c        ------
c        c = Aq
c        ------
         call bmux(n, n,q,zt,A,colind,rwptr)
         call lusl(n, zt, c, lu, jlu, uptr, pcmeth, A,colind,rwptr)

         do k = 1, n
            u(k) = sigma*r(k) - rho*q(k)
            y(k) = sigma*e(k) - rho*c(k)
         end do 

c        ------
c        d = Ay
c        ------
         call bmux(n, n,y,zt,A,colind,rwptr)
         call lusl(n, zt, d, lu, jlu, uptr, pcmeth, A,colind,rwptr)

c        ----------------
c        omega1 = y'u/y'y
c        ----------------
         temp1 = 0.0d0
         temp2 = 0.0d0
         do k = 1, n
            temp1 = temp1 + y(k)*u(k)
            temp2 = temp2 + y(k)*y(k)
         end do 
         if (temp2  .le. SMALL) go to 930
         omega1 = temp1/temp2

c        ----------------------------------------------------------
c        rhat = u - omega1*y; ehat = y - omega1*d; psi = || rhat ||
c        ----------------------------------------------------------
         psi = 0.0d0
         do k = 1, n
            rhat(k) = u(k) - omega1*y(k)
            ehat(k) = y(k) - omega1*d(k)
            psi = psi + rhat(k)*rhat(k)
         end do 
         psi = sqrt(psi)

c        ---------------------------------------------
c        Shortcircuit the two step stuff if specified.
c        ---------------------------------------------
         if (force) then
            onestep = .true.
            go to 40
         end if 

         onestep = ( psi .lt. abs(sigma)*phi )
         if (.not. onestep) then
            a12 = 0.0d0
            a22 = 0.0d0
            a21 = 0.0d0
            b2  = 0.0d0
            do k = 1, n
               a12 = a12 + rtilde(k)*y(k)
               a21 = a21 + rtilde(k)*c(k)
               a22 = a22 + rtilde(k)*d(k)
               b2  = b2  + rtilde(k)*e(k)
            end do 
            delta = a11*a22 - a12*a21
            if (abs(mu)  .le. SMALL) go to 940 
            b1 = rho/mu
            alpha1 = a22*b1 - a12*b2
            alpha2 = -a21*b1 + a11*b2
            do k = 1, n
              s(k) = delta*r(k) - alpha1*q(k) - alpha2*y(k)
              t(k) = delta*e(k) - alpha1*c(k) - alpha2*d(k)
            end do 

            temp1 = 0.0d0
            temp2 = 0.0d0
            do k = 1, n
               temp1 = temp1 + t(k)*s(k)
               temp2 = temp2 + t(k)*t(k)
            end do 
            if (abs(temp2)  .le. SMALL) go to 950
            omegah = temp1/temp2

            nutilde = 0.0d0
            do k = 1, n
               wt(k)  = s(k) - omega1*t(k)
               nutilde = nutilde + (s(k) - omegah*t(k))**2
            end do 

            nutilde = sqrt(nutilde) 
            if (abs(delta)*psi  .lt.  abs(sigma)*nutilde) then
               onestep = .true.
            else
               call bmux(n, n,t,zt,A,colind,rwptr)
               call lusl(n,zt,v,lu,jlu,uptr,pcmeth,A,colind,rwptr)
               call bmux(n, n,v,zt,A,colind,rwptr)
               call lusl(n,zt,w,lu,jlu,uptr,pcmeth,A,colind,rwptr)
               temp1 = 0.0d0
               temp2 = 0.0d0
               do k = 1, n
                  z = t(k) - omega1*v(k)
c                  temp1 = temp1 + z*u(k)
                  temp1 = temp1 + z*wt(k)
                  temp2 = temp2 + z*z
               end do 
               if (abs(temp2)  .le. SMALL) go to 960
               omega2 = temp1/temp2

               gamma1 = -(omega1 + omega2)
               gamma2 = omega1 * omega2
               nu2 = 0.0d0
               do k = 1, n
                  rhat2(k) = s(k) + gamma1*t(k) + gamma2*v(k)
                  ehat2(k) = t(k) + gamma1*v(k) + gamma2*w(k)
                  nu2 = nu2 + rhat2(k)*rhat2(k)
               end do 
               nu2 = sqrt(nu2)
               onestep = ( abs(delta)*psi .lt. abs(sigma)*nu2)
            end if 
         end if 

 40      continue

         if (onestep) then
c        =======================
c        Perform one-step CGSTAB
c        =======================
            if (abs(sigma)  .le. SMALL) go to 970
            steps(1) = steps(1) + 1
            temp1 = 1.0d0/sigma
            temp2 = 0.0d0
            do k = 1, n
               r(k) = temp1 * rhat(k)
               e(k) = temp1 * ehat(k)
               x(k) = x(k) + temp1 * (rho*p(k) + omega1*u(k))
               temp2 = temp2 + r(k)*rtilde(k)
            end do
            phi = psi*temp1
            if (abs(omega1)  .le. SMALL) go to 980
            mu = mu*rho*temp1/omega1
            temp2 = mu*temp2

            if (abs(rho)  .le. SMALL) go to 990
            beta1 = temp2/rho
            rho = temp2
            do k = 1, n
               p(k) = r(k) + beta1*(p(k) - omega1*q(k))
               q(k) = e(k) + beta1*(q(k) - omega1*c(k))
            end do 
            its = its + 1
         else
c        =======================
c        Perform two-step CGSTAB
c        =======================
            if (abs(delta)  .le. SMALL) go to 995
            steps(2) = steps(2) + 1
c            Write(*,*) 'Two step iteration: ', its
            temp1 = 1.0d0/delta
            temp2 = 0.0d0
            b1 = 0.0d0
            b2 = 0.0d0
            phi = nu2*temp1
            do k = 1, n
               r(k) = temp1 * rhat2(k)
               e(k) = temp1 * ehat2(k)
               x(k) = x(k) + temp1 * 
     &          (alpha1*p(k) + alpha2*u(k) - gamma1*s(k) - gamma2*t(k))
               temp2 = temp2 + rtilde(k)*r(k)
               b1 = b1 + rtilde(k)*t(k)
               b2 = b2 + rtilde(k)*v(k)
            end do 
            if (abs(gamma2)  .le. SMALL) go to 997
            mu = -mu*temp1*alpha2*rho/gamma2
            rho = temp2*mu
            beta1 = temp1*(a22*b1 - a12*b2)
            beta2 = temp1*(-a21*b1 + a11*b2)
            do k = 1, n
               p(k) = r(k) - beta1*(p(k) + gamma1*q(k) + gamma2*c(k))
     &                     - beta2*(u(k) + gamma1*y(k) + gamma2*d(k))
            end do 
            call bmux(n, n,p,zt,A,colind,rwptr)
            call lusl(n,zt,q,lu,jlu,uptr,pcmeth,A,colind,rwptr)
            its = its + 2
         end if

         if (its .ge. maxits) go to 920

         if (dnrm2(n,r,1) .lt. tol) go to 910

        go to 2

c=======================================================================
c           Stopping conditions
c=======================================================================


c     ===================== 
c     (Almost) exact guess 
c     ===================== 
 900  continue
      ierr = 1
      maxits = 0
      return

c     ========================================= 
c     Possible convergence; check true residual 
c     ========================================= 
 910  continue
      ierr = 0
      resets = resets + 1
      if (resets .gt. 8) then
         ierr = 2
         go to 999
      end if
      call bmux(n,n,x,rhat,A,colind,rwptr)
      do i = 1, n
         rhat(i) = b(i) - rhat(i)
      end do
      res = dnrm2(n,rhat,1)
      if (res .gt. tolrnc(1)) then
          tol = tol*tolrnc(1)/res
c          write(6,*)'>>>>> tolerance reset to ', tol,'  <<<<<<'
          go to 2
      end if
      go to 999

c     ======================================== 
c     No convergence in max allowed iterations 
c     ======================================== 
 920  continue
      ierr = 3
      go to 999

c     ================================ 
c     omega1 computation small divisor 
c     ================================ 
 930  continue
      ierr = 5
      go to 999

c     ====== 
c     mu = 0 
c     ====== 
 940  continue
      ierr = 6
      go to 999

c     ================================ 
c     omegah computation small divisor
c     ================================ 
 950  continue
      ierr = 7
      go to 999

c     ================================ 
c     omega2 computation small divisor
c     ================================ 
 960  continue
      ierr = 8
      go to 999

c     =============================== 
c     sigma computation small divisor
c     =============================== 
 970  continue
      ierr = 9
      go to 999

c     =================================== 
c     second mu computation small divisor
c     =================================== 
 980  continue
      ierr = 10
      go to 999

c     ============================= 
c     rho computation small divisor
c     ============================= 
 990  continue
      ierr = 11
      go to 999

c     =============================== 
c     delta computation small divisor
c     =============================== 
 995  continue
      ierr = 12
      go to 999

c     =============================== 
c     gamma computation small divisor
c     =============================== 
 997  continue
      ierr = 13
      go to 999

c     =================================== 
c     Final instrumentation call and exit
c     =================================== 
 999  continue
      maxits = its
      Write(6,*) 'One step iterations: ', steps(1)
      Write(6,*) 'Two step iterations: ', steps(2)
      call instr(A,colind,rwptr,n, x,b, wins1, lu,jlu,uptr,
     &               pcmeth,first,its, iunit,instlvl)
      return

c================= End of CSCGS ========================================

      end
