c=======================================================================
      subroutine gmrest(A,colind,rwptr,n,
     1                  x,r,b,
     2                  tolrnc,reltv,maxits,ierr,
     3                  work,wksz,
     4                  lu,jlu,uptr,pcmeth,iunit,instlvl)

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

c-----------------------------------------------------------------------*
c                                                                       *
c                 *** GMRES (Templates version) ***			*
c                                                                       *
c  This is the version of GMRES(m) from Templates, changed to fit in	*
c  the paradigm used for SPLIB.  It is provided to allow comparisons	*
c  with the native SPLIB version of GMRES(m), and differs in its	*
c  implementation primarily in how the orthogonal rotations are 	*
c  computed and applied, and in using left preconditioning instead	*
c  of right preconditioning.  The reverse communication model 		*
c  has been replaced with SPLIB's forward communications.  This		*
c  routine is just a wrapper, to divide up the work() array and then	*
c  call the slightly modified Templates routine, renamed gmres1 here.	*
c                                                                       *
c-----------------------------------------------------------------------*
c                                                                       *
c     Randall Bramley							*
c     Department of Computer Science					*
c     Indiana University						*
c     email: bramley@cs.indiana.edu					*
c     Tue Aug  2 16:53:53 EST 1994					*
c     Documented Mon Jul 10 10:15:28 EST 1995				*
c                                                                       *
c-----------------------------------------------------------------------*
c
c  Arguments:
c  ----------
c
c     (Data structure for A)
c  n       : Order of matrix A.
c  A       : Real*8 array for nonzero entries of matrix A; of
c            length nnz = rwptr(n+1)-1.
c  colind  : Integer array of column indices for entries in A; of
c            length nnz = rwptr(n+1)-1.
c  rwptr   : Integer array of pointers to beginning position of rows
c            in arrays A and colind.  Must be of length n+1.
c
c     (Required vectors)
c  x       : vector of unknowns to be solved for.
c  b       : right hand side vector of system to be solved: Ax = b
c  r       : residual vector.
c  work    : work array 
c  wksz    : size of work array 
c
c     (Preconditioner data structure)
c  lu,jlu  : matrix stored in Modified Sparse Row (MSR) format containing
c            the L and U factors together. The diagonal (stored in
c            lu(1:n) ) is inverted. Each i-th row of the lu,jlu matrix
c            contains the i-th row of L (excluding the diagonal entry=1)
c            followed by the i-th row of U.
c  uptr    : integer array of length n containing the pointers to
c            the beginning of each row of U in the matrix lu,jlu.
c  pcmeth  : Preconditioning method to use; see splib.f.
c
c     (Iteration control)
c  tolrnc  : tolerances on norm(s) used in stopping test.
c  maxits  : Maximum number of iterations allowed; also maximal size
c            of Krylov subspace to use.
c            maxits(1) ---> maximum iterations to allow.
c            maxits(2) ---> maximal Krylov subspace size.
c
c      (Other stuff)
c  ierr    : Error flag; see return values below.
c  iunit   : Integer vector of unit numbers for the instrumentation
c            routine instr.f to use.  Not referenced by gmrest, just
c            passed along to instr.f
c  instlvl : Integer giving instrumentation level.  If instlvl = 0,
c            then instr.f is not called and array work() need not
c            provide the 2n double precision words used by instr.f
c
c   Local variables
c   ---------------
c  h       : integer pointer for begin position of the Hessenberg
c            matrix H in work array
c  w       : integer pointer for begin position of the Krylov subspace 
c            basis vectors in work
c  wkins   : integer pointer for begin position of the work array for 
c            instrumentation routine
c  pdr     : parameter to determine if preconditioner is unstable.
c  urnorm  : unpreconditioned initial residual.
c  im      : size of Krylov subspace.
c
c----------------------------------------------------------------------
c
c
c Return values:
c --------------
c
c  x      : Contains an approximate solution (upon successful return).
c
c  ierr   : Error flag with the following meanings:
c           ierr = -m --> Not enough workspace provided; m more double
c                         precision words needed in array work().
c           ierr = 0  --> Successful return; tolerance achieved.
c           ierr = 1  --> The initial guess satisfies the stopping test.
c                         This is often a clue that rhs not set properly.
c           ierr = 2  --> Tolerance reset 8 times in attempt to
c                         get true residual small enough.
c           ierr = 3  --> convergence not achieved in maxits iterations.
c           ierr = 4  --> Preconditioned residual is 1/eps times
c                         larger than unpreconditioned residual.
c
c  maxits : Contains the number of actual iterations performed.
c  tolrnc : May be modified if reltv = .true.
c
c----------------------------------------------------------------------
c  Routines
c  --------
c  gmres1  : Modified Templates version of gmres
c  bmux    : Multiplies y = A*x
c  lusl    : Solves M z = r.
c  instr   : Instrumentation routine for timing, etc.
c  BLAS1 routines: ddot, daxpy, dnrm2, dscal 
c  BLAS2 routine: dgemv
c----------------------------------------------------------------------* 
        implicit none
        integer n, im, maxits(2), ierr
        real*8 work(*), b(n), x(n), A(*), lu(*), r(*)
        real*8 tolrnc(*)
        integer colind(*), rwptr(n+1), jlu(*), uptr(n)
        integer pcmeth, instlvl, iunit(*)
        integer wksz
        integer kwork
        integer h, wkins, w
        logical reltv

c       ------------------------------------
c       Unpack maximal Krylov subspace size.
c       ------------------------------------
        im = maxits(2)

c       ----------------------------------------------------
c       Set pointers and check if enough workspace provided.
c       ----------------------------------------------------
        wkins =  1
        if (instlvl. gt. 0) then
           w = wkins + 2*n
        else
           w = wkins
        end if
        h = w + n*(im+4)
        kwork = h + (im+1)*(im+2)

        if (wksz .lt. kwork) then
           ierr = wksz - kwork
           maxits(1) = 0
           return
        end if
c
c       ---------------------------------------------
c       Call the gmres1.f routine (Templates version)          
c       ---------------------------------------------
        call gmres1(A,colind,rwptr,n,
     1              x,r,b,
     2              tolrnc(1),reltv,maxits(1),im,ierr,
     3              work(w),n,work(h),im+1,
     4              lu,jlu,uptr,pcmeth,work(wkins),iunit, instlvl)
c
        return
        end
c
c======================================================================
c
      subroutine gmres1(A,colind,rwptr,n,
     1                  x,res,b,
     2                  tolrnc,reltv,maxits,restrt,ierr,
     3                  work,ldw,h,ldh,
     4                  lu,jlu,uptr,pcmeth,wkins,iunit, instlvl)
c
       implicit none
       integer n, ldw, ldh
       real*8 work(ldw,*), b(n), x(n), A(*), lu(*), res(*)
       real*8 wkins(*)
       real*8 tolrnc
       integer restrt, maxits, ierr
       integer colind(*), rwptr(n+1), jlu(*), uptr(n)
       integer pcmeth, iter, resets, instlvl, iunit(*)
       logical first, reltv
       real*8 h( ldh,* )
       real*8 dlamch, pdr, urnorm
c=======================================================================
c  
c WARNING: the below documentation is from the original TEMPLATES
c version of this, and has been changed.  In particular, the matrix
c and its preconditioner are explicitly passed in, and the "reverse
c communication" model of Templates has been changed to the "forward
c communication" model of SPLIB.  This has required changes in the
c calls to the matrix-vector and preconditioning routines.  Other
c changes include the error flag modifications, and calls to the
c instrumentation routines.
c
c Although credit is given below to the Templates authors, any errors
c in this version probably should be attributed to the modifications
c made; follow the instructions below to get the Templates documentation
c and information for getting an unmodified version from netlib.
c
*
*  -- Iterative template routine --
*     Univ. of Tennessee and Oak Ridge National Laboratory
*     October 1, 1993
*     Details of this algorithm are described in "Templates for the
*     Solution of Linear Systems: Building Blocks for Iterative
*     Methods", Barrett, Berry, Chan, Demmel, Donato, Dongarra,
*     Eijkhout, Pozo, Romine, and van der Vorst, SIAM Publications,
*     1993. (ftp netlib2.cs.utk.edu; cd linalg; get templates.ps).
c
c  purpose
c  =======
c
c  gmres solves the linear system ax = b using the
c  generalized minimal residual iterative method with preconditioning.
c
c  convergence test: ( norm( b - a*x ) / norm( b ) ) < tolrnc.
c  for other measures, see the above reference.
c  (Note: convergence test is norm( b - a*x ) < tolrnc if reltv = .false.)
c
c  arguments
c  =========
c
c  n       (input) integer. 
c  on entry, the dimension of the matrix.
c  unchanged on exit.
c 
c  b       (input) double precision array, dimension n.
c  on entry, right hand side vector b.
c  unchanged on exit.
c
c  x       (input/output) double precision array, dimension n.
c  on input, the initial guess; on exit, the iterated solution.
c
c  restrt  (input) integer
c  restart parameter, <= n. this parameter controls the amount
c  of memory required for matrix h (see work and h).
c
c  work    (workspace) double precision array, dimension (ldw,restrt+4).
c
c  ldw     (input) integer
c  the leading dimension of the array work. ldw >= max(1,n).
c
c  h       (workspace) double precision array, dimension (ldh,restrt+2).
c  this workspace is used for constructing and storing the
c  upper hessenberg matrix. the two extra columns are used to
c  store the givens rotation matrices.
c
c  ldh    (input) integer
c  the leading dimension of the array h. ldh >= max(1,restrt+1).
c
c  iter    (input/output) integer
c  on input, the maximum iterations to be performed.
c  on output, actual number of iterations performed.
c
c  resid   (input/output) double precision
c  on input, the allowable convergence measure for
c  norm( b - a*x ) / norm( b ).
c  on output, the final value of this measure.
c
c  matvec  (external subroutine)
c  lusol (external subroutine)
c (Note: matvec, lusol have been replaced with bmux, lusl)
c
c  info    (output) integer
c  =  0: successful exit. iterated approximate solution returned.
c  >  0: convergence to tolerance not achieved. this will be
c        set to the number of iterations performed.
c  <  0: illegal input parameter.
c           -1: matrix dimension n < 0
c           -2: ldw < n
c           -3: maximum number of iterations iter <= 0.
c           -4: ldh < restrt
c (Note: info has been replaced with ierr, with different 
c  return values.  See gmrest.f above).
c
c  blas calls:   daxpy, dcopy, ddot, dnrm2, drot, drotg, dscal
c  ============================================================
c
      integer   i, k, av, cs, sn, r, s, v, w, y
      real*8  aa, bb, rnorm, tol, ddot, dnrm2, temp
      external  daxpy, dcopy, ddot, dnrm2, drot, drotg, dscal
      external dlamch
c
c Tolerance for size of preconditioned residual:
c
      pdr = 1.0d0/dlamch('E')
c
      tol   = tolrnc
      resets = 0
      first = .true.
c
c   alias workspace columns. 
c
      r  = 0
      s  = r + 1
      w  = s + 1
      y  = w
      av = y
      v  = av + 1
c
c   store the Givens parameters in matrix h.
c
      cs = restrt + 1
      sn = cs + 1
c
c   set initial residual (av is temporary workspace here).
c
      call bmux (n,n, x, work(1,av), A, colind, rwptr)
      do k = 1, n
         work(k,av) = b(k) - work(k,av)
      end do
c
c Return if initial guess works ok.
c
      urnorm = dnrm2(n,work(1,av),1)
      if (urnorm .le. tol) go to 999
c     -------------------------------------------------
c     Adjust tolrnc if relative residual to be used:
c     -------------------------------------------------
      if (reltv) then
          tol = tol*urnorm
          tolrnc = tolrnc*urnorm
      end if
c
c
c Initial preconditioned residual
c
      call lusl (n, work(1,av), res, lu, jlu, uptr, pcmeth, A,
     &           colind,rwptr) 
c
c Check to see if preconditioned residual is too large relative
c to unpreconditioned residual:
c
      rnorm = dnrm2( n, res, 1 )
      if (rnorm .gt. pdr*urnorm) then
         ierr = 4
         maxits = 0
         return
      end if 

      iter = 0
c
   10 continue
c
         i = 0
         call instr(A,colind,rwptr,n, x,b, wkins, lu,jlu,uptr,pcmeth,
     &               first,iter, iunit,instlvl)
c
c  construct the first column of v.
c
         if (iter .gt. 0) rnorm = dnrm2( n, res, 1 )
         temp = 1.0d0/rnorm
         do k = 1, n
            work(k,v) = temp * res(k)
         end do
c
c  initialize s to the elementary vector e1 scaled by rnorm.
c
         work( 1,s ) = rnorm
         do 20 k = 2, n
            work( k,s ) = 0.0d0
   20    continue
c
   30    continue
c
            i = i + 1
            iter = iter + 1
c
            call bmux (n,n, work(1,v+i-1), work(1,av), A, colind, rwptr)
            call lusl (n, work(1,av), work(1,w), lu, jlu, uptr, pcmeth,
     &                 A,colind,rwptr) 
c
c   construct i-th column of h orthnormal to the previous 
c   i-1 columns.
c
            call basis( i, n, h( 1,i ), work( 1,v ), ldw, work( 1,w ) )
c
c   apply Givens rotations to the i-th column of h. this
c   "updating" of the qr factorization effectively reduces
c   the hessenberg matrix to upper triangular form during
c   the restrt iterations.
c
            do 40 k = 1, i-1
               call drot( 1, h( k,i ), ldh, h( k+1,i ), ldh,
     $            h( k,cs ), h( k,sn ) )
   40       continue
c
c   construct the i-th rotation matrix, and apply it to h so that
c   h(i+1,i) = 0.
c
            aa = h( i,i )
            bb = h( i+1,i )
            call drotg( aa, bb, h( i,cs ), h( i,sn ) )
            call drot( 1, h( i,i ), ldh, h( i+1,i ), ldh,
     $            h( i,cs ), h( i,sn ) )
c
c   apply the i-th rotation matrix to [ s(i), s(i+1) ]'. this
c   gives an approximation of the residual norm. if less than
c   tolrnc, update the approximation vector x and quit.
c
            call drot( 1, work( i,s ), ldw, work( i+1,s ), ldw,
     $         h( i,cs ), h( i,sn ) )
            rnorm = abs( work( i+1,s ) )
            if ( rnorm .le. tol ) then
               call update( i, n, x, h, ldh, work( 1,y ), work( 1,s ),
     $            work( 1,v ), ldw )
               go to 990
            endif
            if ( iter .ge. maxits ) go to 50
            if ( i .lt. restrt )   go to 30
c
   50    continue
c
c  compute current solution vector x.
c
c         call update( restrt, n, x, h, ldh, work( 1,y ), work( 1,s ),
c     $                work( 1,v ), ldw )
         call update( i, n, x, h, ldh, work( 1,y ), work( 1,s ),
     $                work( 1,v ), ldw )
c
c  compute residual vector r, find norm, then check for tolerance.
c  (av is temporary workspace here.)
c
         call bmux (n,n, x, work(1,av), a, colind, rwptr)
         do k = 1, n
            work(k,av) = b(k) - work(k,av)
         end do 
         call lusl (n, work(1,av), res, lu, jlu, uptr, pcmeth, 
     &              a,colind,rwptr) 
         work( i+1,s ) = dnrm2( n, res , 1 )
         rnorm = work( i+1,s )
         if ( rnorm .le. tol  ) go to 990
  60     continue
         if ( iter .ge. maxits ) go to 991
c
c  restart.
c
         go to 10
c
c-----------------------------------------------------------------------
c  Stopping conditions.
c-----------------------------------------------------------------------
c

c
c Possible convergence; check true residual 
c
 990   continue
       ierr = 0
       resets = resets + 1
       if (resets .gt. 8) then
         ierr = 2
         go to 995
       end if
       call bmux(n,n,x,work(1,v),A,colind,rwptr)
       do k = 1, n
          work(k,v) = work(k,v) - b(k)
       end do
       temp = dnrm2(n,work(1,v),1)
       if (temp .gt. tolrnc) then
          tol = tol*tolrnc/temp
c          write(6,*)'>>>>> tolrnc reset to ', tol,'  <<<<<<'
          go to 60
       end if
 995   continue
       maxits = iter
      call instr(A,colind,rwptr,n, x,b, wkins, lu,jlu,uptr,
     &               pcmeth,first,iter, iunit,instlvl)
       return
c
c  No convergence within allowed iterations 
c
 991   continue
       ierr = 3
       maxits = iter
       call instr(A,colind,rwptr,n, x,b, wkins, lu,jlu,uptr,
     &               pcmeth,first,iter, iunit,instlvl)
       return
c
c  Almost exact guess on entry to routine 
c
 999   continue
       ierr = 1
       maxits = iter
       return
c----------------- End of GMRES1 ------------------------------------- 
           end
c
c   ===============================================================
      subroutine update( i, n, x, h, ldh, y, s, v, ldv )
c
      implicit none
      integer            n, i, ldh, ldv
      double precision   x( * ), y( * ), s( * ), h( ldh,* ), v( ldv,* )
      external           daxpy, dcopy, dtrsv
c
c   this routine updates the gmres iterated solution approximation.      
c
      double precision   one
      parameter        ( one = 1.0d+0 )
c
c   .. executable statements ..
c
c   solve h*y = s for upper triangualar h.
c
      call dcopy( i, s, 1, y, 1 )
      call dtrsv( 'upper', 'notrans', 'nonunit', i, h, ldh, y, 1 )
c
c   compute current solution vector x = x + v*y.
c
      call dgemv('notrans', n, i, one, v, ldv, y, 1, one, x, 1 )
c
      return
c
      end
c
c   =========================================================
      subroutine basis( i, n, h, v, ldv, w )
c
      implicit none
      integer            i, n, ldv
      double precision   h( * ), w( * ), v( ldv,* )
c
c   construct the i-th column of the upper hessenberg matrix h
c   using the gram-schmidt process on v and w.
c
      integer            k
      double precision   ddot, dnrm2, one
      parameter        ( one = 1.0d+0 )
      external           daxpy, dcopy, ddot, dnrm2, dscal
c
      do 10 k = 1, i
         h( k ) = ddot( n, w, 1, v( 1,k ), 1 )
         call daxpy( n, -h( k ), v( 1,k ), 1, w, 1 )
   10 continue
      h( i+1 ) = dnrm2( n, w, 1 )
      call dcopy( n, w, 1, v( 1,i+1 ), 1 )
      call dscal( n, one / h( i+1 ), v( 1,i+1 ), 1 )
c
      return
c
      end
