c=========================================================================
      subroutine qmrtf(A,colind,rwptr,n,
     1                 x,r,b,
     2                 tolrnc,reltv,maxits,ierr,
     3                 work,wksz,
     4                 lu,jlu,uptr,pcmeth,iunit,instlvl)

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

c-----------------------------------------------------------------------*
c                     *** Transpose Free QMR ***			*
c									*
c   Transpose-free QMR method of Freund and Nachtigal.  This is an	*
c   implementation for small scale testing, and should not be used	*
c   as a measure of the quality of all QMR methods.  In particular,	*
c   users interested in QMR methods should obtain QMRPACK from netlib	*
c   and experiment with the "look-ahead Lanczos" versions before	*
c   rejecting QMR as a solution strategy.				*   
c									*
c-----------------------------------------------------------------------*
c
c  Arguments:
c  ----------
c
c     (Data structure for A)
c  n       : Order of matrix A.
c  A       : Real*8 array for nonzero entries of matrix A; of
c            length nnz = rwptr(n+1)-1.
c  colind  : Integer array of column indices for entries in A; of
c            length nnz = rwptr(n+1)-1.
c  rwptr   : Integer array of pointers to beginning position of rows
c            in arrays A and colind.  Must be of length n+1.
c
c     (Required vectors)
c  x       : Vector of unknowns to be solved for.  Contains initial
c            estimate of x on entry, and contains final approximate
c            on return.
c  r       : Residual vector r = b - A*x.
c  b       : Right hand side vector of system to be solved: Ax = b
c  work    : Double precision array of additional storage.  QMR
c            requires 10 vectors of length n, and if instrumentation
c            routine instr.f is called, an additional 2 vectors of
c            length n are required.  If not enough storage is given,
c            nothing is done and an error code is returned specifying
c            the additional storage needed (in double precision words).
c  wksz    : Length of array work, in double precision words.  
c
c     (Preconditioner data structure)
c  lu,jlu  : Preconditioner in Modified Sparse Row format containing
c            the L and U factors together. The diagonal of U (stored in
c            lu(1:n) ) is inverted.  The i-th row of the lu,jlu matrix 
c            contains the i-th row of L (excluding the diagonal entry) 
c            followed by the i-th row of U.  The diagonal of L has
c            all ones, and so is not stored explicitly.
c  uptr    : Integer array of length n containing the pointers to        
c            the beginning of each row of U in the matrix lu,jlu. 
c  pcmeth  : Preconditioning method to use.  See splib.f documentation
c            for the interpretation of this integer.
c
c     (Iteration control)
c  tolrnc  : Tolerances on norm(s) used in stopping test.  This is
c            passed in as a vector in case future versions
c            set pdr and SMALL, rather than using
c            the dlamch() function.  Since Krylov methods generally
c            use the dotproduct r'*r, a local variable tolsq is used
c            in order to avoid square roots.  Note: because the
c            preconditioned residual norm || inv(M)*r || is the natural
c            one tracked by the iterative method, but users generally
c            want the true residual norm || r || as the stopping test,
c            this routine will exit the iterations when the precond
c            residual is small enough and then test the true residual.
c            If it is not sufficiently reduced, then the local tolerance
c            tolsq is reduced and iterations resume.  This is done
c            a maximum of eight times before failure is declared and
c            the routine returns.
c  reltv   : Logical variable indicating whether or not to use a
c            relative residual as the stopping test.  If this is
c            false, the stopping test is || r_k || < tolrnc(1).  If it
c            is true, the test is || r_k || < || r_0 || * tolrnc(1).
c            Note that it is possible to modify SPLIB to use other
c            stopping tests (see routine instr.f).
c  maxits  : Maximum number of iterations allowed.
c
c      (Other stuff)
c  ierr    : Error flag; see below.
c  iunit   : Integer vector of unit numbers for the instrumentation
c            routine instr.f to use.  Not referenced by qmrtf, just
c            passed along to instr.f
c  instlvl : Integer giving instrumentation level.  If instlvl = 0,
c            then instr.f is not called and array work() need not
c            provide the 2n double precision words used by instr.f
c
c  Some internal variables:
c  ------------------------
c
c  early   : Logical parameter that indicates if a tentative exit
c            was made because of early termination.  This is needed
c            because of the way SPLIB uses unpreconditioned residuals
c            as the stopping test.
c  first   : Logical parameter that specifies to instr.f if this is
c            the first iteration, and so if new headers need to be
c            written to the Xgraph output files.
c  SMALL   : Parameter that determines when it is not safe to divide
c            by a scalar quantity.  Error condition is set if a divisor
c            is smaller than SMALL.  Set by dlamch() function.
c  pdr     : Parameter to determine if preconditioner is no good.
c            Experience shows that if the preconditioned residual norm
c            is more than 1/eps, where eps is the machine epsilon, then
c            even if the preconditioned residual converges, the original
c            system residual does not.  If you are not comfortable with
c            this try setting pdr = 1.0d200.  
c  urnorm  : Unpreconditioned initial residual.  Used for tests to see
c            if initial residual is good enough, and if preconditioner
c            is no good (see pdr above).
c
c  h,w,y,
c  rh,d,t,
c  v,y2,t2,
c  w2      : Integer pointers in the array work(*) that give the
c            starting position of the corresponding vectors.
c  wins1,
c  wins2   : Integer pointers in the array work(*) that give starting
c            positions of two vectors used by instrumentation routines.
c
c======================================================================
c
c Return values:                                                           
c --------------
c
c  x      : Contains an approximate solution (upon successful return).  
c
c  ierr   : Error flag with the following meanings:          
c           ierr = -m --> Not enough workspace provided; m more double
c                         precision words needed in array work().
c           ierr = 0  --> Successful return; tolerance achieved.
c           ierr = 1  --> The initial guess satisfies the stopping test.
c                         This is often a clue that rhs not set properly.
c           ierr = 2  --> Tolerance reset 8 times in attempt to
c                         get true residual small enough.
c           ierr = 3  --> convergence not achieved in maxits iterations.  
c           ierr = 4  --> Preconditioned residual is 1/eps times
c                         larger than unpreconditioned residual.
c           ierr = 5  --> convergence not achieved due to rho = 0
c           ierr = 6  --> convergence not achieved due to tau = 0
c           ierr = 7  --> convergence not achieved due to sigma = 0
c
c  maxits : Contains the number of actual iterations performed.
c  tolrnc : May be modified if reltv = .true.
c                                                                       
c======================================================================
c
c  Routines
c  --------
c
c  lusl    : Solves M z = r.
c  bmux    : Computes matrix-vector product with A.
c  instr   : Instrumentation routine for timing, etc.
c  dlamch  : Function from LAPACK project, for determining machine
c            constants.  This provides machine epsilon and the safe
c            invertible floating point minimum. 
c  BLAS routines: ddot, dcopy
c
c======================================================================
c
c     Randall Bramley
c     Department of Computer Science
c     Indiana University
c     email: bramley@cs.indiana.edu
c     Sun Jul 31 11:54:53 EST 1994
c
c----------------------------------------------------------------------

       implicit none

c     -------------------------
c     Declaration of arguments:
c     -------------------------
      integer n
      real*8 a(*)
      real*8 x(*), b(*), r(*)
      real*8 work(n,*)
      real*8 tolrnc(*)
      real*8 lu(*)

      integer colind(*), rwptr(*)
      integer wksz
      integer maxits, ierr
      integer jlu(*), uptr(*)
      integer pcmeth, iunit(*), instlvl
      logical reltv

c     ------------------
c     External functions
c     ------------------
      real*8 ddot,dnrm2
      real*8 dlamch
      external dlamch, ddot, dnrm2

c     ------------- 
c     Local storage 
c     ------------- 
      logical first
      integer i, its, resets
      real*8 SMALL, pdr, urnorm
      real*8 alfa, beta, rho, rhonew, tau, res, tolsq
     *      ,theta, eta, sigma, omega, thetanew, etanew, cm

c     ------------------------------
c     Pointers to work array vectors
c     ------------------------------
      integer h, w, y, rh, d, t, v, y2, t2, w2
      integer wins1, wins2

c=======================================================================
c     Start of executable statements
c=======================================================================

c     ------------------------------
c     Set up square root of safe min
c     ------------------------------
      SMALL = sqrt(dlamch('S'))

c     ---------------------------------
c     Set up inverse of machine epsilon
c     ---------------------------------
      pdr = 1.0d0/dlamch('E')

c     ----------------------------------------
c     Set up meaningful names for work vectors
c     ----------------------------------------
      if (instlvl .gt. 0) then
         wins1 = 1
         wins2 = wins1 + 1
      else
         wins1 = 0
         wins2 = 0
      end if
       h = wins2 + 1
       w = h + 1
       y = w + 1
      rh = y + 1
       d = rh + 1
       t = d + 1
       v = t + 1
      y2 = v + 1
      t2 = y2 + 1
      w2 = t2 + 1

c----------------------------------------------------------------
c
c     Key to work array columns if no instrumentation:
c     h    ---> work(:,1)
c     work ---> work(:,2)
c     y    ---> work(:,3)
c     rh   ---> work(:,4)
c     d    ---> work(:,5)
c     t    ---> work(:,6)
c     v    ---> work(:,7)
c     y2   ---> work(:,8)  
c     t2   ---> work(:,9)
c     work2 --> work(:,10)
c
c----------------------------------------------------------------


c     -------------------------------------
c     Stop if not enough workspace provided
c     -------------------------------------
      if (wksz .lt. (wins2+10)*n) then
         ierr = wksz - (wins2+10)*n
         maxits = 0
         return
      end if


c     -------------------------
c     Initialize for iterations
c     -------------------------
      first = .true.
      tolsq = tolrnc(1)*tolrnc(1)

c     ----------
c     r = b - Ax
c     ----------
      call bmux(n,n,x,r,a,colind,rwptr)
      do 10 i = 1,n
         work(i,h) = b(i) - r(i)
10    continue

c     -------------
c     h = inv(LU)*r
c     -------------
      call lusl(n, work(1,h), r, lu, jlu, uptr, pcmeth,a,colind,rwptr)

c     -------------------------------------------------------------
c     Check to see if preconditioned residual is too large relative
c     to unpreconditioned residual:
c     -------------------------------------------------------------
      urnorm = ddot(n, work(1,h), 1, work(1,h), 1)
      if (ddot(n,r,1,r,1) .gt. pdr*pdr*urnorm) then
         ierr = 4
         maxits = 0
         return
      end if 

c     -------------------------------------------------
c     Adjust tolerance if relative residual to be used:
c     -------------------------------------------------
      if (reltv) then
          tolsq = tolsq*urnorm
          tolrnc(1) = tolrnc(1)*sqrt(urnorm)
      end if

c     ---------------------
c     w = y = rh = r; d = 0
c     ---------------------
      do i = 1, n
         work(i,w) = r(i)
         work(i,y) = r(i)
         work(i,rh) = r(i)
         work(i,d) = 0.0d0
      end do

      call bmux(n,n,work(1,y),work(1,t),a,colind,rwptr)
      call lusl(n, work(1,t), work(1,v), lu, jlu, uptr, pcmeth,a,
     &          colind,rwptr)
      call dcopy(n, work(1,v),1,work(1,t),1)

      rho = ddot(n,work(1,rh),1,r,1)
      tau = ddot(n,r,1,r,1)

      res = tau
      tau = dsqrt(tau)
      theta = 0.0d0
      eta = 0.0d0

      resets = 0
      its = 0
      if (res.lt.tolsq) go to 900

c=======================================================================
c     Begin main loop of iterations.  Loop is equivalent to
c      do while (rnorm .gt. tolsq) .and. (its .le. maxits)
c=======================================================================

  2      continue
c
         call instr(a,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its,iunit,instlvl)
         its = its + 1

         sigma = ddot(n,work(1,rh),1,work(1,v),1)
  	 if (abs(sigma) .le. SMALL) go to 950

         alfa = rho/sigma
         do i = 1,n
            work(i,y2) = work(i,y) - alfa*work(i,v)
         end do

         call bmux(n,n,work(1,y2),work(1,h),a,colind,rwptr)
         call lusl(n, work(1,h), work(1,t2), lu, jlu, uptr, pcmeth,
     1             a,colind,rwptr)

         do i = 1,n
            work(i,w2) = work(i,w) - alfa*work(i,t)
         end do

         omega = dnrm2(n,work(1,w2),1)
  	 if (abs(tau) .le. SMALL) go to 940

         thetanew = omega/tau
         cm = 1/dsqrt(1+thetanew*thetanew)
         tau = omega*cm
         etanew = cm*cm*alfa
         do i = 1,n
            work(i,d) = work(i,y)+(theta*theta*eta/alfa)*work(i,d)
            x(i) = x(i) + etanew*work(i,d)
         end do
         do i = 1,n
            work(i,w) = work(i,w2)-alfa*work(i,t2)
         end do
         omega = dnrm2(n,work(1,w),1)

  	 if (abs(tau) .le. SMALL) go to 940

         theta = omega/tau
         cm = 1.0d0/dsqrt(1+theta*theta)
         tau = omega*cm
         eta = cm*cm*alfa
         do i = 1,n
            work(i,d) = work(i,y2)+
     1             (thetanew*thetanew*etanew/alfa)*work(i,d)
            x(i) = x(i)+eta*work(i,d)
         end do
c
         call bmux(n,n,x,r,a,colind,rwptr)
         do i = 1,n
            work(i,h) = b(i) - r(i)
         end do
         call lusl(n, work(1,h), r, lu, jlu, uptr, pcmeth,a,
     &             colind,rwptr)

         rhonew = ddot(n,work(1,rh),1,work(1,w),1)
         res = ddot(n,r,1,r,1)
c
  	 if (abs(rho) .le. SMALL) go to 930
c
         beta = rhonew/rho
         rho = rhonew
         do i = 1,n
            work(i,y) = work(i,w)+beta*work(i,y2)
         end do
         call bmux(n,n,work(1,y),work(1,h),a,colind,rwptr)
         call lusl(n, work(1,h), work(1,t), lu, jlu, uptr, pcmeth,
     1             a,colind,rwptr)
         do i = 1,n
            work(i,v) = work(i,t) + beta*(work(i,t2)+beta*work(i,v))
         end do
c
         if (res.lt.tolsq) go to 910
  60     continue
         if (its .ge. maxits) go to 920
c	
      go to 2

c=======================================================================
c           Stopping conditions
c=======================================================================

c     =====================
c     (Almost) exact guess
c     =====================
 900  continue
      ierr = 1
      maxits = 0
      return

c     =========================================
c     Possible convergence, check true residual
c     =========================================
 910  continue
      ierr = 0
      resets = resets + 1
      if (resets .gt. 8) then
         ierr = 2
         go to 915
      end if
      call bmux(n,n,x,work(1,h),a,colind,rwptr)
      do i = 1, n
         work(i,h) = b(i) - work(i,h)
      end do
      res = ddot(n,work(1,h),1,work(1,h),1)
      if (res .gt. tolrnc(1)*tolrnc(1)) then
          tolsq = tolsq*tolrnc(1)*tolrnc(1)/res
c          write(6,*)'>>>>> tolerance reset to ', sqrt(tolsq),'  <<<<<<'
          go to 60
      end if
 915  continue
      maxits = its
      call instr(a,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its,iunit,instlvl)
      return

c     =========================================
c     No convergence in max allowed iterations
c     =========================================
 920  continue
      ierr = 3
      maxits = its
      call instr(a,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its,iunit,instlvl)
      return

c     =======
c     rho = 0
c     =======
 930  continue
      ierr = 5
      maxits = its
      call instr(a,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its,iunit,instlvl)
      return

c     =======
c     tau = 0
c     =======
 940  continue
      ierr = 6
      maxits = its
      call instr(a,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its,iunit,instlvl)
      return

c     =========
c     sigma = 0
c     =========
 950  continue
      ierr = 7
      maxits = its
      call instr(a,colind,rwptr,n, x,b, work(1,wins1), lu,jlu,uptr,
     &               pcmeth,first,its,iunit,instlvl)
      return

c================= End of QMRTF ==========================================

      end
