      integer function addtbl (name  , length, type , from )

c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c									*
c  The following Fortran memory management package was written and	*
c  provided by Dominique Pelletier of Ecole Polytechnique, Montreal.	*
c  This allows memory managment with limited garbage collection.	*
c  The code has been slightly modified by R. Bramley, and since it	*
c  worked perfectly before the modifications, all bugs are probably	*
c  due to his changes.							*
c									*
c  The memory manager is initialized by a call to initbl, which will	*
c  set the number of arrays to 0 and the first pointer to 1.  In	*
c  addition, a logical unit number should be provided to initbl, and	*
c  all output (such as from listbl) will go to that lun.  After		*
c  initialization, when an array is needed a call to addtbl will	*
c  allocate the space and perform double word alignment if necessary.	*
c									*
c  All the memory is set up in the main program via the include file 	*
c  memory.inc; a single integer array memblk() is set up in a common	*
c  block called "memory", which needs to be included in the calling	*
c  routine.								*
c									*
c     Randall Bramley							*
c     Department of Computer Science					*
c     Indiana University						*
c     email: bramley@cs.indiana.edu					*
c     Wed May  4 11:12:23 EST 1994					*
c									*
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

c
c=======================================================================
c
cs    addtbl
c
c     function : pseudo dynamic allocation of an array
c     ---------  returns address of first element of array in memory
c                common
c     usage :    integer addtbl
c     -----      .....
c                nx = addtbl (name  , length, type , from  )
c     arguments :
c     ---------
c     input :
c     ------
c       length .............. length of the array
c       type   .............. type of array to allocate
c                               = 'integer' integer array
c                               = 'real'    real    array
c       name   .............. name of the table to create
c       from   .............. name of routine requesting allocation
c                             of array 'name'
c
c     output:
c     ------
c       addtbl .............. address of the first element
c                             of table.
c
c     internal :
c     ---------
c       table(i)............. list of all arrays allocated
c       type (i)............. type of array 'table(i)'
c       origin(i)............ name of routine resting creation of
c                             array 'table(i)'
c       ibeg(i) ............. address of first integer word
c                             of table table(i)
c       iend(i) ............. address of last integer word
c                             of table table(i)
c       maxtbl  ............. max number of tables that can be allocated
c       ntable  ............. current number of tables
c       nxttbl  ............. address of first integer word
c                             for the next table to be allocated
c
c i/o         : to printout file number iprint
c portability : full fortran-77
c-----------------------------------------------------------------------
c notes    : the user should be very careful in using the routines
c            so as to ensure that tables that will eventually
c            be removed be the last ones to be allocated.
c            this is necessary because the pointers returned
c            to the user by 'addtbl' are not updated by rmvtbl.
c
c----------+------------------------------------------------+----------*
c status   : production
c level    : 1.0.0
c created  : 14 nov 1986   dominique pelletier
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date
c----------+------------------------------------------------+----------*
c ssmm0001 | none so far                        d pelletier | 14nov86
c ssmm0002 | mise a jour version 4.0            d pelletier |  7mar91
cf=====================================================================
c
      implicit none
c
c     memory manager commons
c     ----------------------
      include '../include/memory.inc'
      include '../include/intdbl.inc'
c     -----------------
c     a r g u m e n t s
c     -----------------
      integer       length
      character*(*) name
      character*(*) type
      character*(*) from
      character*6 lowname
      character*7 lowtype
      character*6 lowfrom
c     -----------------
c     f u n c t i o n s
c     -----------------
      integer len
c     ------------------------------
c     l o c a l    v a r i a b l e s
c     ------------------------------
c        align ......... = .true. perform double word memory alignment
c
      logical     align
      parameter ( align = .true. )
      logical     debug
      parameter ( debug  = .false.)
      character*6 caller
      parameter ( caller ='addtbl' )
      character*7 reel , entier
      parameter ( reel = 'real' , entier = 'integer')
      integer i
      integer nwords
      integer nsize
      character*6 tabnam
      integer     namlen
c
      lowname = name
      lowtype = type
      lowfrom = from
      call lowcas ( lowname )
      call lowcas ( lowtype )
      call lowcas ( lowfrom )

      namlen = len (lowname)
      tabnam = '      '
      tabnam = lowname(1:namlen)
c     ------------------------------------------
c     check if table has already been allocated.
c     ------------------------------------------
      do  i = 1,ntable
         if (tabnam .eq. table(i)) then
            write (iprint,1000) tabnam  , lowfrom
            call listbl (caller)
            stop
         endif
      end do
c
c     ------------------------------------------------------------
c     length of the array in integer words of the target computer.
c     ------------------------------------------------------------
      if ( lowtype .eq. entier ) then
         nwords = length
      else if (lowtype .eq. reel ) then
         nwords = length * intdbl
      else
         write(iprint,1050) tabnam, lowtype , entier, reel
         stop
      endif
c
      nsize = nxttbl + nwords
      if ( nsize .gt. given) then
         write ( iprint,1100) tabnam , nsize, given
         call listbl('addtbl')
         stop
      else
         ntable = ntable + 1
         if ( ntable .gt. maxtbl ) then
            write(iprint,1150) tabnam, maxtbl
            stop
         endif
         table  (ntable) = tabnam
         tblsiz (ntable) = length
         tabtyp (ntable) = lowtype
         ibeg   (ntable) = nxttbl
         iend   (ntable) = nxttbl + nwords - 1
         origin (ntable) = lowfrom
         addtbl          = nxttbl
         nxttbl          = iend(ntable)  + 1
c***     ------------------------------------------------------------
c***     Force double word alignment 
c***     ------------------------------------------------------------
         if (align) then
            if(tabtyp(ntable).eq.entier .and. mod(nxttbl,2).eq.0) then
               nxttbl = nxttbl + 1
               iend(ntable) = iend(ntable) + 1
            end if
         endif
c***     -------------------------------------------------------------
c***     End of if clause for double word alignment 
c***     -------------------------------------------------------------
      end if
      if ( debug ) then
         write(iprint,1200) ntable, table(ntable), tblsiz(ntable),
     &                    tabtyp(ntable),nwords, ibeg (ntable),
     &                    iend  (ntable),origin(ntable)
      endif
      return

c-----------------------------------------------------------------------
c                       f o r m a t s
c-----------------------------------------------------------------------
1000  format(1h0,'*** addtbl *** execution stops - table "',
     &           a6,'" has already been allocated.',/
     &             ,' "addtbl" called from routine : ',a6,/
     &       1h ,'    dump of tables allocated follows')
1050  format(1h0,'*** addtbl *** execution stops - table "',
     &           a6,'" has unknown type =',a10/
     &       1h ,'                permitted types = ',a10,/
     &       1h ,'                                = ',a10,/)
1100  format (1h0,'*** addtbl *** ',
     &            'error in memory allocation for table "',a6,'"',/
     &        1h ,'    total space requested = ',i10,' words,',/
     &        1h ,'    total space available = ',i10,' words.',/
     &        1h ,'    increase storage in blank common'/)
1150  format(1h0,'*** addtbl *** ',
     &           'execution stops - not enough space in bookeeping ',
     &           ' arrays to allocate table "',a6,'".',/
     &       1h ,'    increase sixe of array table(i), ibeg(i), ',
     &           'iend(i), type(i).',/,
     &       1h ,'    current size is = maxtbl =',i5 )
1200  format(1h ,'*** addtbl *** table',i3,' - "',a6,
     &           '" length = ',i5,' ',a7,', size = ',i6,' words',
     &   ' from memblk (',i6,' )',' to memblk (',i6,' ) origin = ',a6)
      end

      subroutine initbl (mempr,space)
c
c=======================================================================
c
cs    initbl
c
c function : initialises arrays for pseudo-dynamic memory alloation
c --------
c usage    : call  initbl ()
c -----
c
c arguments : 
c ---------
c     mempr   ............. logical unit number to print on.
c     space   ............. integer words allocated in array work
c
c internal variables :
c ------------------
c     ntable  ............. current number of tables
c     nxttbl  ............. address of first integer word
c
c portability : full fortran-77
c-----------------------------------------------------------------------
c notes    : 
c----------+------------------------------------------------+----------*
c status   : production
c level    : 1.0.0
c created  : 14 nov 1986   dominique pelletier
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date
c----------+------------------------------------------------+----------*
c ssmm0001 | version 4.0                        d pelletier | 12mar91
cf======================================================================
c
      implicit none
      integer already_done
      save already_done
      integer mempr, space
c
c     memory manager commons
c     ----------------------
      include '../include/memory.inc'
      data already_done /0/
c
      if (already_done .eq. 0) then
        given = space
        nxttbl = 1
        ntable = 0
        iprint = mempr
        already_done = 1
      end if
c
      return
      end
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c     
c     end of initbl    
c     
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
  
  
  
      logical function istbl (name  , from )
c
c=======================================================================
c system   : cadyf
c program  : cadyf
c
cs    istbl
c
c     function : determine si le tableau 'name' a deja ete cree
c     ---------
c
c     usage :    logical istbl , ok
c     -----      .....
c                ok = istbl (name  , from )
c     arguments :
c     ---------
c     input :
c     ------
c       name   .............. name of the table to create
c
c     output:
c     ------
c       istbl  .............. = .true . tableau 'name' existe
c                             = .false. tableau 'name' n'existe pas
c
c portability : full fortran-77
c-----------------------------------------------------------------------
c notes    : none
c reference: n/a
c----------+------------------------------------------------+----------*
c status   : development
c level    : 1.0.0
c created  : 05 avr 1989   dominique pelletier
c verified : 05 avr 1989   jean francois hetu
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date
c----------+------------------------------------------------+----------*
c mod00001 | version 4.0                 d. pelletier       | 12mar91
cf=====================================================================
c
      implicit none
c
c     memory manager commons
c     ----------------------
      include '../include/memory.inc'
c     -----------------
c     a r g u m e n t s
c     -----------------
      character*(*) name
      character*(*) from
c     ------------------------------
c     l o c a l    v a r i a b l e s
c     ------------------------------
      integer level
      parameter ( level=1 )
      integer i
      character*6 tabnam
      integer     namlen
      character*6 lowname
      character*6 lowfrom
c
c     functions :
c     ----------
      integer len
c
      lowname = name
      lowfrom = from
      call lowcas ( lowname )
      call lowcas ( lowfrom )

      namlen = len (lowname)
      tabnam = '      '
      tabnam = lowname(1:namlen)
c     ------------------------------------------
c     check if table has already been allocated.
c     ------------------------------------------
      istbl = .false.
      i = 1
      do while ( i .le. ntable .and. .not.istbl )
         if (tabnam .eq. table(i)) then
            istbl  = .true.
         end if
         i = i + 1
      end do
      return
c
      end
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c     
c     end of istbl    
c     
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
  
  
  
      subroutine listbl (from)                                          
c                                                                       
c=======================================================================
c system   : cadyf                                                      
c program  : cadyf                                                      
c                                                                       
cs    listbl                                                            
c                                                                       
c usage :    call listbl (from)                                         
c -----                                                                 
c function : displays information on arrays that are                    
c --------   dynamically allocated.                                     
c                                                                       
c arguments :                                                           
c ---------                                                             
c    from............. name of calling routine                          
c                                                                       
c i/o         : to printout file                                        
c portability : full fortran-77                                         
c-----------------------------------------------------------------------
c notes :  n/a                                                          
c----------+------------------------------------------------+----------*
c status   : production                                                 
c level    : 1.0.0                                                      
c created  : 14 nov 1986   dominique pelletier                          
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date     
c----------+------------------------------------------------+----------*
c ssmm0001 | none so far                        d pelletier | 14nov86   
c ssmm0001 | version 1.0                        d pelletier | 25jan89   
cf===================================================================== 
c                                                                       
      implicit none                                                     
c
c     memory manager commons
c     ----------------------
      include '../include/memory.inc'
c                                                                       
c     arguments                                                         
c     ---------                                                         
      character*(*) from                                                
      character*6 lowfrom
c     ------------------------------                                    
c     l o c a l    v a r i a b l e s                                    
c     ------------------------------                                    
      integer i                                                         
c                                                                       
c     functions:                                                        
c     ----------                                                        
      integer len                                                       
c                                                                       
      lowfrom = from
      lowfrom = lowfrom(1:len(lowfrom))                                          
      write(iprint,1000)
      do  i = 1,ntable                                                  
         write(iprint,1100) i,table(i),tblsiz(i),tabtyp(i),             
     &                    ibeg(i),iend(i),origin(i)                     
      end do                                                            
      write(iprint,1200)                                                
      return                                                            
                                                                        
c-----------------------------------------------------------------------
c                       f o r m a t s                                   
c-----------------------------------------------------------------------
                                                                        
1000  format('<TABLE BORDER=8 WIDTH=80%>',/,'<CAPTION><B>',/,
     1  'Listing of major arrays used in run',/,'</B></CAPTION>',/,
     2  '<TR ALIGN="center">',/,'  <TH>No.</TH> <TH>Array</TH>',/,
     3  '  <TH>Int words</TH> <TH>Type</TH>',/,
     4  '  <TH>Begins at</TH> <TH>Ends at</TH> <TH>Caller</TH>',/,
     5  '</TR>')

1100  format('<TR ALIGN="right">',/,
     1           '<TD>',i3,'</TD>',
     2           '<TD>',a6,'</TD>',
     3           '<TD>',i7,'</TD>',
     4           '<TD>',a7,'</TD>',
     5           '<TD>',i10,'</TD>',
     6           '<TD>',i10,'</TD>',
     7           '<TD>',a6,'</TD>',
     8           '</TR>')                                
1200  format(/,'</TABLE>',/,/)
      end                                                               
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c     
c     end of listbl    
c     
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
  
  
  
      integer function loctbl (name  , from )
c
c=======================================================================
c system   : cadyf
c program  : cadyf
c
cs    locate
c
c     function : finds the address of the first element of table 'name'
c     ---------  returns adrress of first element of array in memory
c                common
c     usage :    integer loctbl
c     -----      .....
c                nx = loctbl (name  , from )
c     arguments :
c     ---------
c     input :
c     ------
c       name   .............. name of the table to create
c       from   .............. name of the routine requesting information
c
c     output:
c     ------
c       loctbl .............. address of the first element of table.
c
c     internal :
c     ---------
c       table(i)............. list of all arrays allocated
c       type (i)............. type of array 'table(i)'
c       origin(i)............ name of routine requesting creation of
c                             array 'table(i)'
c       ibeg(i) ............. address of first integer word
c                             of table table(i)
c       iend(i) ............. address of last integer word
c                             of table table(i)
c       maxtbl  ............. max number of tables that can be allocated
c       ntable  ............. current number of tables
c       nxttbl  ............. address of first integer word
c                             for the next table to be allocated
c
c portability : full fortran-77
c-----------------------------------------------------------------------
c notes    : none
c reference: n/a
c----------+------------------------------------------------+----------*
c status   : development
c level    : 1.0.0
c created  : 03 feb 1987   dominique pelletier
c verified : xx mmm 19yy   auditor
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date
c----------+------------------------------------------------+----------*
c mod00001 | if table not found a message is printed        |
c            and the starting address of the array is set   |
c            to 1.                         d. pelletier     | 31aug87
c mod00002 | version 1.0                   d. pelletier     | 25jan89
cf=====================================================================
c
      implicit none
c
c     memory manager commons
c     ----------------------
      include '../include/memory.inc'
c     -----------------
c     a r g u m e n t s
c     -----------------
      character*(*) name
      character*(*) from
c     ------------------------------
c     l o c a l    v a r i a b l e s
c     ------------------------------
      character*6 lowname
      character*6 lowfrom
      integer level
      parameter ( level=1 )
      integer i
      character*6 tabnam
      integer     namlen
      logical     found
c
c     functions :
c     ----------
      integer len
c
      lowname = name
      lowfrom = from
      call lowcas ( lowname )
      call lowcas ( lowfrom )

      namlen = len (lowname)
      tabnam = '      '
      tabnam = lowname(1:namlen)
c     ------------------------------------------
c     check if table has already been allocated.
c     ------------------------------------------
      found = .false.
      do  i = 1,ntable
         if (tabnam .eq. table(i)) then
            found = .true.
            loctbl = ibeg(i)
            return
         end if
      end do
      if ( .not. found ) then
         write ( iprint,1000) lowname, lowfrom
         loctbl = 0
         return
      end if
c=======================================================================
c
c                       f o r m a t s
c
c=======================================================================
c
1000  format(1h0,'*** loctbl *** table "',
     &           a6,'" does not exist.',
     &           ' "loctbl" called from routine : ',a6,/ )
      end
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c     
c     end of loctbl    
c     
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
  
  
  
      subroutine rmvtbl (memblk, name ,from  )
c
c=======================================================================
c system   : cadyf
c program  : cadyf
c
cs    rmvtbl
c
c function : remove table from list and storage.
c ---------  perform garbage collection too
c
c usage :    call rmvtbl(name,from  )
c -----
c
c arguments :
c ----------
c input :
c ------
c   name   .............. name of the table to remove
c   from   .............. name of calling routine
c
c internal variables :
c ------------------
c     table(i)............. list of all arrays allocated
c     ibeg(i) ............. address of first integer word
c                           of table table(i)
c     iend(i) ............. address of last integer word
c                           of table table(i)
c     ntable  ............. current number of tables
c     nxttbl  ............. address of first integer word
c                           for the next table to be allocated
c
c     forced  ............. option flag : (logical)
c                           = .true. , table to be removed must be
c                                      be the last in the list
c                           = .false., table to be removed may  be
c                                      anywhere in the list.
c                           for the next table to be allocated
c
c i/o         : to printout file
c portability : full fortran-77
c-----------------------------------------------------------------------
c note     : the user should be very careful in using the routines
c            so as to ensure that tables that will eventually
c            be removed be the last ones to be allocated.
c            this is necessary becauset the pointers returned
c            to the user by 'addtbl' are not updated by rmvtbl.
c
c             although rmvtbl was written to do garbage collection
c             anywhere in the blank common. it will remove a table
c             only if it is located at the end of the list.
c----------+------------------------------------------------+----------*
c status   : production
c level    : 1.0.0
c created  : 14 nov 1986   dominique pelletier
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date
c----------+------------------------------------------------+----------*
c ssmm0001 | none so far                        d pelletier | 14nov86
c ssmm0001 | version 1.0                        d pelletier | 26jan89
cf=====================================================================
c
      implicit none
c
c     memory manager commons
c     ----------------------
      include '../include/memory.inc'
c     -----------------
c     a r g u m e n t s
c     -----------------
      integer memblk(*)
      character*(*) name
      character*(*) from
c     -----------------------------
c     l o c a l   v a r i a b l e s
c     -----------------------------
      character*6 lowname
      character*6 lowfrom
      character*6 tabnam
      logical forced
      integer  end   , start, tabsiz
      integer  i     , j    , namlen
c
      logical debug
      parameter ( debug = .false. )
      data forced / .true. /
c
c     ------------------------------------------------
c     search list of arrays and remove table if found.
c     ------------------------------------------------
      lowname = name
      lowfrom = from
      call lowcas ( lowname )
      call lowcas ( lowfrom )
      namlen = len(lowname)
      tabnam = '      '
      tabnam = lowname(1:namlen)
      do  i = 1,ntable
         if ( tabnam .eq. table(i)) then
            if ( forced .and . i .ne. ntable ) then
               write(iprint,1000) tabnam , i , ntable, lowfrom
               call listbl (lowfrom)
               stop
            end if
            tabsiz = iend(i) - ibeg(i) + 1
            start  = ibeg(i)
            end    = iend(ntable)
            nxttbl = nxttbl - tabsiz
            if ( debug ) then
               write(iprint,1100) tabnam, tblsiz(i),tabtyp(i),ibeg(i),
     &                          iend(i),tabsiz
               write(iprint,1200) nxttbl
            end if
c           --------------------------------------------------
c           garbage collection : shift remaining tables upward
c           --------------------------------------------------
            if ( i .lt. ntable ) then
               do  j = start,end
                  memblk(j) = memblk(j + tabsiz)
               end do
c              ---------------------------------------------------------
c              adjust table(i), ibeg(i), iend(i) to reflect table remova
c              ---------------------------------------------------------
               do  j = i,ntable
                  table(j) = table(j+1)
                  ibeg(j)  = ibeg(j+1) - tabsiz
                  iend(j)  = iend(j+1) - tabsiz
               end do
            end if
            ntable = ntable - 1
            return
         endif
      end do
      write(iprint,1300) tabnam, lowfrom
      call listbl(from)
      stop
c
c-----------------------------------------------------------------------
c                       f o r m a t s
c-----------------------------------------------------------------------

1000  format(1h0,'*** rmvtbl *** ',
     &           'execution stops - table "',a6,'" is no. ',i3,
     &           ' in the list of ',i3,' tables; is not last table',/
     &           ' rmvtbl was called from : ', a6/
     &       1h ,'    dump of tables allocated follows')
1100  format (1h ,'*** rmvtbl *** ',
     &            'removal of table "',a6,' ',i6,' ',a7,
     &            ' from memblk(',i6,') , to memblk(',i6,'),',
     &            ' garbage collection of ',i6,' words.')
1200  format (1h ,'               ',
     &            'next table to be allocated will start at',
     &            ' nxttbl = ',i6)
1300  format(1h0,'*** rmvtbl *** ',
     &           'execution stops - table "',a6,'" not found ',
     &           'during table removal attempt,'/
     &           '              rmvtbl was called from : ',a6,/
     &       1h ,'              dump of tables allocated follows')
c
      end
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c     
c     end of rmvtbl    
c     
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
  
  
  
      integer function sizeof (name  , from )
c
c=======================================================================
c system   : cadyf
c program  : cadyf
c
cs    sizeof
c
c     function : finds the length in words of table 'name'
c     ---------
c
c     usage :    integer sizeof
c     -----      .....
c                nx = sizeof (name  , from )
c     arguments :
c     ---------
c     input :
c     ------
c       name   .............. name of the table to query
c       from   .............. name of the routine requesting information
c
c     output:
c     ------
c       sizeof .............. length of table 'name' in integer words
c
c     internal :
c     ---------
c       table(i)............. list of all arrays allocated
c       type (i)............. type of array 'table(i)'
c       origin(i)............ name of routine requesting creation of
c                             array 'table(i)'
c       ibeg(i) ............. address of first integer word
c                             of table table(i)
c       iend(i) ............. address of last integer word
c                             of table table(i)
c       maxtbl  ............. max number of tables that can be allocated
c       ntable  ............. current number of tables
c       nxttbl  ............. address of first integer word
c                             for the next table to be allocated
c subprograms called :
c ------------------
c       - listbl............. lists all tables allocated so far
c
c i/o         : to printout file
c portability : full fortran-77
c-----------------------------------------------------------------------
c notes    : none
c reference: n/a
c----------+------------------------------------------------+----------*
c status   : development
c level    : 1.0.0
c created  : 03 feb 1987   dominique pelletier
c verified : xx mmm 19yy   auditor
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date
c----------+------------------------------------------------+----------*
c mod00001 | if table not found a message is printed        |
c mod00001 | version 1.0                     d. pelletier   | 26jan89
cf=====================================================================
c
      implicit none
c
c     memory manager commons
c     ----------------------
      include '../include/memory.inc'
      include '../include/intdbl.inc'
c     -----------------
c     a r g u m e n t s
c     -----------------
      character*(*) name
      character*(*) from
c     ------------------------------
c     l o c a l    v a r i a b l e s
c     ------------------------------
      character*6 lowname
      character*6 lowfrom
      integer level
      parameter ( level=1 )
      integer i
      character*6 tabnam
      integer     namlen 
      logical     found
c
      lowname = name
      lowfrom = from
      call lowcas (lowname)
      call lowcas (lowfrom)

      namlen = len (lowname)
      tabnam = '      '
      tabnam = lowname(1:namlen)
c     ------------------------------------------
c     check if table has already been allocated.
c     ------------------------------------------
      found = .false.
      do  i = 1,ntable
         if (tabnam .eq. table(i)) then
            found = .true.
            if ( tabtyp(i) .eq. 'integer') then
               sizeof = tblsiz(i)
            else if ( tabtyp(i) .eq. 'real') then
               sizeof = tblsiz(i)*intdbl
            end if
         end if
      end do
      if ( .not. found ) then
         write ( iprint,1000) lowname, lowfrom
c-----------------
c begin : mod00001
c-----------------
ctempo   call listbl()
ctempo   stop
         sizeof = 0
c-----------------
c end   : mod00001
c-----------------
      end if

c
c=======================================================================
c
c                       f o r m a t s
c
c=======================================================================
c
c*1000  format(1h0,'*** sizeof *** execution stops - table "',
c*     &           a6,'" does not exist.',/
c*     &             ,' "sizeof" called from routine : ',a6,/
c*     &       1h ,'    dump of tables allocated follows')
1000  format(1h0,'*** sizeof *** table "',
     &           a6,'" does not exist.',
     &           ' "sizeof" called from routine : ',a6,/ )
      end
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c     
c     end of sizeof    
c     
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
  
  
  
      integer function storof (name  , from )
c
c=======================================================================
c system   : cadyf
c program  : cadyf
c
cs    storof
c
c     function : returns number of storage locations occupied
c     ---------  by table 'name', includes possible gaps
c                inserted to ensure double word boundary alignment
c                for ibm-3090-vf
c
c
c     usage :    integer storof
c     -----      .....
c                nx = storof (name  , from )
c     arguments :
c     ---------
c     input :
c     ------
c       name   .............. name of the table to create
c       from   .............. name of the routine requesting information
c
c     output:
c     ------
c       storof .............. length of table 'name' in integer words
c                             includes gaps for double word alignment
c                             on ibm-3090-vf
c     internal :
c     ---------
c       table(i)............. list of all arrays allocated
c       type (i)............. type of array 'table(i)'
c       origin(i)............ name of routine requesting creation of
c                             array 'table(i)'
c       ibeg(i) ............. address of first integer word
c                             of table table(i)
c       iend(i) ............. address of last integer word
c                             of table table(i)
c       maxtbl  ............. max number of tables that can be allocated
c       ntable  ............. current number of tables
c       nxttbl  ............. address of first integer word
c                             for the next table to be allocated
c i/o         : to printout file
c portability : full fortran-77
c-----------------------------------------------------------------------
c notes    : none
c reference: n/a
c----------+------------------------------------------------+----------*
c status   : development
c level    : 1.0.0
c created  : 03 feb 1987   dominique pelletier
c verified : xx mmm 19yy   auditor
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date
c----------+------------------------------------------------+----------*
c mod00001 | if table not found a message is printed        |
c mod00001 | version 1.0                                    | 26jan89
cf=====================================================================
c
      implicit none
c
c     memory manager commons
c     ----------------------
      include '../include/memory.inc'
c     -----------------
c     a r g u m e n t s
c     -----------------
      character*(*) name
      character*(*) from
c     ------------------------------
c     l o c a l    v a r i a b l e s
c     ------------------------------
      character*6 lowname
      character*6 lowfrom
      integer i
      character*6 tabnam
      integer     namlen 
      logical     found
c
      lowname = name
      lowfrom = from
      call lowcas ( lowname )
      call lowcas ( lowfrom )

      namlen = len (lowname)
      tabnam = '      '
      tabnam = lowname(1:namlen)
c     ------------------------------------------
c     check if table has already been allocated.
c     ------------------------------------------
      found = .false.
      do  i = 1,ntable
         if (tabnam .eq. table(i)) then
            found = .true.
            storof = iend(i) - ibeg(i) + 1
         end if
      end do
      if ( .not. found ) then
         write ( iprint,1000) lowname, lowfrom
c-----------------
c begin : mod00001
c-----------------
ctempo   call listbl()
ctempo   stop
         storof = 0
c-----------------
c end   : mod00001
c-----------------
      end if

c
c=======================================================================
c
c                       f o r m a t s
c
c=======================================================================
c
c*1000  format(1h0,'*** storof *** execution stops - table "',
c*     &           a6,'" does not exist.',/
c*     &             ,' "storof" called from routine : ',a6,/
c*     &       1h ,'    dump of tables allocated follows')
1000  format(1h0,'*** storof *** table "',
     &           a6,'" does not exist.',
     &           ' "storof" called from routine : ',a6,/ )
      end
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c     
c     end of storof    
c     
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
  
      subroutine lowcas( chaine )
c
c=======================================================================
c
cs    lowcas
c
c     function : Convert character string chaine to lower case.
c     ---------  
c
c i/o         : none
c portability : full fortran-77
c-----------------------------------------------------------------------
c notes    : none
c reference: n/a
c----------+------------------------------------------------+----------*
c----------+------------------------------------------------+----------*
c  ident.  |          m o d i f i c a t i o n s             |  date
c----------+------------------------------------------------+----------*
c mod00001 | version 4.0                                    | 28fev91
cf=====================================================================

      implicit none

c     arguments:
c     ---------
      character*(*)  chaine

c     variables locales
c     -----------------
      integer        i  , n
      character*26 alpha
      integer        indexc
      data  alpha /'abcdefghijklmnopqrstuvwxyz' /

c     ================================================================

      do i = 1, len(chaine)
         n = index( 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', chaine(i:i) )
c         n = indexc(chaine(i:i) )
         if( n .ne. 0 ) then
            chaine(i:i) = alpha(n:n)
         end if
      end do
      return
      end

c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
c     
c     end of lowcas    
c     
c-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
      integer function indexc(letter)
      character letter
      integer k
      k = 0
      if (letter .eq. 'A') k =  1
      if (letter .eq. 'B') k =  2
      if (letter .eq. 'C') k =  3
      if (letter .eq. 'D') k =  4
      if (letter .eq. 'E') k =  5
      if (letter .eq. 'F') k =  6
      if (letter .eq. 'G') k =  7
      if (letter .eq. 'H') k =  8
      if (letter .eq. 'I') k =  9
      if (letter .eq. 'J') k = 10
      if (letter .eq. 'K') k = 11
      if (letter .eq. 'L') k = 12
      if (letter .eq. 'M') k = 13
      if (letter .eq. 'N') k = 14
      if (letter .eq. 'O') k = 15
      if (letter .eq. 'P') k = 16
      if (letter .eq. 'Q') k = 17
      if (letter .eq. 'R') k = 18
      if (letter .eq. 'S') k = 19
      if (letter .eq. 'T') k = 20
      if (letter .eq. 'U') k = 21
      if (letter .eq. 'V') k = 22
      if (letter .eq. 'W') k = 23
      if (letter .eq. 'X') k = 24
      if (letter .eq. 'Y') k = 25
      if (letter .eq. 'Z') k = 26
      indexc = k
      return
      end
      
c
      integer function getnext()
c
c=======================================================================
c
cs    getnext
c
c     function : return address of first element of array.
c                This differs from addtbl in that it does not
c                create a new array.
c                
c     usage :    integer getnext
c     -----      .....
c                next = getnext()
c
c     arguments :
c     ---------
c
c     output:
c     ------
c       getnext.............. address of the first element of table.
c
c     internal :
c     ---------
c       table(i)............. list of all arrays allocated
c       type (i)............. type of array 'table(i)'
c       origin(i)............ name of routine resting creation of
c                             array 'table(i)'
c       ibeg(i) ............. address of first integer word
c                             of table table(i)
c       iend(i) ............. address of last integer word
c                             of table table(i)
c       maxtbl  ............. max number of tables that can be allocated
c       ntable  ............. current number of tables
c       nxttbl  ............. address of first integer word
c                             for the next table to be allocated
c
c
      implicit none
      include '../include/memory.inc'
      getnext = nxttbl
      return
      end
c
      integer function memleft()
      implicit none
c
c=======================================================================
c
c
c     function : return number of integer words left in memory array
c                
c     usage :    integer memleft
c     -----      .....
c                remains = memleft()
c     arguments :
c     ---------
c     output:
c     ------
c       memleft.............. words left in array.
c
c     internal :
c     ---------
c       table(i)............. list of all arrays allocated
c       type (i)............. type of array 'table(i)'
c       origin(i)............ name of routine resting creation of
c                             array 'table(i)'
c       ibeg(i) ............. address of first integer word
c                             of table table(i)
c       iend(i) ............. address of last integer word
c                             of table table(i)
c       maxtbl  ............. max number of tables that can be allocated
c       ntable  ............. current number of tables
c       nxttbl  ............. address of first integer word
c                             for the next table to be allocated
c       given   ............. total space allocated in work array
c
c
      include '../include/memory.inc'
      memleft = given - nxttbl + 1
      return
      end
