c========================================================================        
c                         **** utilvals.f ****
c========================================================================        
c
c  utilvals.f contains routines for setting values in vectors.
c  Currently contains the routines:
c      setconst: Set an array to a constant value (int or dbl)
c      setb    : Set up a nontrivial rhs vector b.
c      getdia  : Sparskit routine to extract diagonal from a matrix.
c      setlog  : Set a vector of logical values to a constant value.
c
c
c     Randall Bramley
c     Department of Computer Science
c     Indiana University
c     Phone: (812) 855-7790
c     email: bramley@cs.indiana.edu
c     Mon Dec 16 11:35:21 EST 1996
c
c
c========================================================================        

c**********************************************************************
c                   SPLIB Copyright (C) 1995-1999                    **
c                     Indiana University                             **
c**********************************************************************

c----------------------------------------------------------------------
      subroutine setconst(type,n,iarray,rarray,iconst,rconst)
c----------------------------------------------------------------------
c
c  Initialize the array of specified type to be the given constant.
c
c----------------------------------------------------------------------
      implicit none
      integer n, iarray(n), i, iconst
      real*8 rarray(n), rconst
      character*3 type
      if (type .eq. 'int') then
         do i = 1, n
            iarray(i) = iconst
         end do
      end if
      if (type .eq. 'dbl') then
         do i = 1, n
            rarray(i) = rconst
         end do
      end if
 1000 format(8(i6,3x))
 1010 format(4(d12.6,3x))
      return
c----------------------------------------------------------------------
       end
c----------------------------------------------------------------------
c----------------------------------------------------------------------
      subroutine setb(n,b)
c----------------------------------------------------------------------
c
c  Initialize the right hand side vector b
c
c----------------------------------------------------------------------
      implicit none
      integer n, i
      real*8 b(n)
      do i = 1, n
c         b(i) = 1.0d0 - 2.0d0/dfloat(i)
         b(i) = 1.0d0 
      end do 
      return
c----------------------------------------------------------------------
       end
c
c
c-------------------------------------------------------------------------
      subroutine getdia (nrow,ncol,job,a,ja,ia,len,diag,idiag,ioff)
      implicit real*8 (a-h,o-z)
      real*8 diag(*),a(*)
      integer nrow, ncol, job, len, ia(*), ja(*), idiag(*)
c-----------------------------------------------------------------------
c this subroutine extracts a given diagonal from a matrix stored in csr 
c format. the output matrix may be transformed with the diagonal removed
c from it if desired (as indicated by job.) 
c----------------------------------------------------------------------- 
c our definition of a diagonal of matrix is a vector of length nrow
c (always) which contains the elements in rows 1 to nrow of
c the matrix that are contained in the diagonal offset by ioff
c with respect to the main diagonal. if the diagonal element
c falls outside the matrix then it is defined as a zero entry.
c thus the proper definition of diag(*) with offset ioff is 
c
c     diag(k) = a(k,ioff+k) k=1,2,...,nrow
c     with elements falling outside the matrix being defined as zero.
c 
c----------------------------------------------------------------------- 
c 
c on entry:
c---------- 
c
c nrow	= integer. the row dimension of the matrix a.
c ncol	= integer. the column dimension of the matrix a.
c job   = integer. job indicator.  if job = 0 then
c         the matrix a, ja, ia, is not altered on return.
c         if job.ne.1  then getdia will remove the entries
c         collected in diag from the original matrix.
c         this is done in place.
c
c a,ja,
c    ia = matrix stored in compressed sparse row a,ja,ia,format
c ioff  = integer,containing the offset of the wanted diagonal
c	  the diagonal extracted is the one corresponding to the
c	  entries a(i,j) with j-i = ioff.
c	  thus ioff = 0 means the main diagonal
c
c on return:
c----------- 
c len   = number of nonzero elements found in diag.
c         (len .le. min(nrow,ncol-ioff)-max(1,1-ioff) + 1 )
c
c diag  = real*8 array of length nrow containing the wanted diagonal.
c	  diag contains the diagonal (a(i,j),j-i = ioff ) as defined 
c         above. 
c
c idiag = integer array of  length len, containing the poisitions 
c         in the original arrays a and ja of the diagonal elements
c         collected in diag. a zero entry in idiag(i) means that 
c         there was no entry found in row i belonging to the diagonal.
c         
c a, ja,
c    ia = if job .ne. 0 the matrix is unchanged. otherwise the nonzero
c         diagonal entries collected in diag are removed from the 
c         matrix. the structure is modified since the diagonal elements
c	  are removed from a,ja,ia. thus, the  returned matrix will 
c         have len fewer elements if the diagonal is full.
c----------------------------------------------------------------------c
c           y. saad, sep. 21 1989 - modified and tested may 9, 1990.   c
c----------------------------------------------------------------------c
c local variables
      integer istart, max, iend, i, kold, k, kdiag, ko
c
      istart = max(0,-ioff)
      iend = min0(nrow,ncol-ioff)
      len = 0
      do 1 i=1,nrow
         idiag(i) = 0
	 diag(i) = 0.0d0 
 1    continue
c     
c     extract  diagonal elements
c
      do 6 i=istart+1, iend
         do 51 k= ia(i),ia(i+1) -1
            if (ja(k)-i .eq. ioff) then
               diag(i)= a(k)
               idiag(i) = k
               len = len+1
               goto 6
            endif
 51      continue
 6    continue
      if (job .eq. 0 .or. len .eq.0) return
c------------- rewind the structure ------------------------------------ 
      ko = 0
      do  7 i=istart+1,iend
         kold = ko
         kdiag = idiag(i) 
         if (kdiag .eq. 0) goto 7
         do 71 k= ia(i), ia(i+1)-1 
            if (ja(k) .eq. kdiag) goto 71
            ko = ko+1
            a(ko) = a(k)
            ja(ko) = ja(k)
 71      continue
         ia(i) = kold+1
 7    continue
c
c redefine ia(nrow+1)
c
      ia(nrow+1) = ko+1
      return
c------------end-of-getdia----------------------------------------------
c-----------------------------------------------------------------------
      end

       subroutine setlog( n, lvec, lval)
       integer n, k
       logical lvec(n), lval
       do k = 1, n
          lvec(k) = lval
       end do
       return
       end

