#include "SparseLinearSystem_impl.h"
#include "SuperLUComponent.h"
#include "SuperLUParms.h"

SparseLinearSystem_impl::SparseLinearSystem_impl(SuperLUComponent *c)
 : m_component(c) {}

SparseLinearSystem_impl::~SparseLinearSystem_impl() {}

int
SparseLinearSystem_impl::sendSparseLinearSystem(SparseLinearSystem *sls) {
const char *xfn = "SparseLinearSystem_impl::sendSparseLinearSystem";
    
    cout << xfn << " called" << endl << flush;

    // entering critical section. So grab a lock.
    pthread_mutex_lock(&m_component->m_lock);

    // wait till 
    cout << xfn << "m_computing: " <<m_component->m_computing << endl << flush;
    cout << xfn << "m_stopped: " << m_component->m_stopped << endl << flush;
    
    while (m_component->m_computing || m_component->m_stopped) {
        pthread_cond_wait(&m_component->m_cv, &m_component->m_lock);
    }
    assert(!m_component->m_computing);
    assert(!m_component->m_stopped);

    // If we made it past here, then must be ready to compute.
    m_component->m_computing = true;
    m_component->startComputation(sls,
     new SuperLUParms(*m_component->m_parms_cache));

    // entering critical section. So get rid of the lock.
    pthread_mutex_unlock(&m_component->m_lock);

    return 0;
}
