//-*-c++-*-
#ifndef CCAT_SUPER_LU_COMPONENT_H
#define CCAT_SUPER_LU_COMPONENT_H



#include <pthread.h>
#include "../../../../framework/ccacore/Component.h"

class Services;
class SparseLinearSystem;
class SparseLinearSystem_impl;
class SuperLUParms;
class SuperLUParms_impl;

class SuperLUComponent : public Component {
  
  // SuperLU has two input ports
  // One of type sls and one parms port
  friend SparseLinearSystem_impl;
  friend SuperLUParms_impl;

public:

  SuperLUComponent();
  virtual ~SuperLUComponent();
  void setServices(Services *cc);

private:
        
  void startComputation(SparseLinearSystem *sls, SuperLUParms *);
  void compute(SparseLinearSystem *sls, SuperLUParms *);
  static void *compute_wrapper(void *);

private:

  Services *m_core;
  SparseLinearSystem_impl *m_sparseLinearSystemPort;
  SuperLUParms_impl *m_superLUParmsPort;

  // only the parms port object is cached
  // Whenever a new sls comes, it overwrites
  // the exisiting one. However the latest
  // parms needs to be used. If it is not set
  // the default one is used. Computation does
  // not block on the parms object to be set.
  SuperLUParms *m_parms_cache;

  pthread_mutex_t m_lock;
  pthread_cond_t m_cv;
  pthread_attr_t m_threadAttr;

  bool m_computing;
  bool m_stopped;
};

#endif
