#include "SuperLUParms_impl.h"
#include "SuperLUComponent.h"

const int SuperLUParms_impl::maxDirectoryNameLength = 128;

SuperLUParms_impl::SuperLUParms_impl(SuperLUComponent *c) : m_component(c) {
  int ec = -1;
  ec = pthread_attr_init(&m_threadAttr);
  assert(ec == 0);
  ec = pthread_attr_setdetachstate(&m_threadAttr, PTHREAD_CREATE_DETACHED);
  assert(ec == 0);
  m_directoryName = "superlu/";
}

SuperLUParms_impl::~SuperLUParms_impl() {
}

// these methods will be invoked remotely.
// all these methods are part of the
// SuperLUParms_idl interface.

// if you receive a new parms object
// then ditch the old one and replace it
// with this new one. 
int
SuperLUParms_impl::sendSuperLUParms(SuperLUParms *sp) {

  // lock before entering the critical section
    pthread_mutex_lock(&m_component->m_lock);

    assert(m_component->m_parms_cache != NULL);
    delete m_component->m_parms_cache;
    m_component->m_parms_cache = sp;

    // unlock before leaving the critical section
    pthread_mutex_unlock(&m_component->m_lock);

    return 0;
}

// the port has received a signal to start executing.
// set the condition variable m_stopped = false.
// The method sendSparseLinearSystem() would be 
// unblocked by this condition variable.
int
SuperLUParms_impl::start() {

    pthread_mutex_lock(&m_component->m_lock);

    m_component->m_stopped = false;
    pthread_cond_broadcast(&m_component->m_cv);

    pthread_mutex_unlock(&m_component->m_lock);

    return 0;
}
// the port has received a signal to stop executing.
// set the condition variable m_stopped = true.
// The method sendSparseLinearSystem() would be 
// blocked by this condition variable. No new
// compute() can take place as long as the condition
// variable is not changed. 

int
SuperLUParms_impl::stop() {

    pthread_mutex_lock(&m_component->m_lock);

    m_component->m_stopped = true;

    pthread_mutex_unlock(&m_component->m_lock);

    return 0;
}

// The easiest cleanup that can be done !!
//  int
//  SuperLUParms_impl::kill() {
    
//      exit(1);

//      return 0;
//  }

int
SuperLUParms_impl::kill() {
  pthread_t tid;
  cout << "SuperLUParmsParms_impl::kill() has been called !! " << endl;
  int afterHowManyIterations = 30;
  pthread_create(&tid, &m_threadAttr, killInANewThread, &afterHowManyIterations);
  cout << "SuperLUParmsParms_impl::kill() returning !! " << endl;
  return 0;
}

char*
SuperLUParms_impl::getOutputURL() {
  cout << "SuperLUParmsParms_impl::getOutputURL() has been called !! " << endl;
  char *buffer = new char[maxDirectoryNameLength];
  sprintf(buffer,"/%s",m_directoryName);
  return buffer;
}

// wait for a few seconds before dying. This looks like
// a hack. However, I believe this is just a good design
// practice. The user of a the CCAT composition engine needs
// to have some time to disconnect other components before this
// component dies. Note, that other components that have a 
// startpoint to this component will get error signals if they aren't
// disconnected before this component dies. 

void *SuperLUParms_impl::killInANewThread(void *howManyIterations) {
  int secondsToSleep = *(static_cast<int *>(howManyIterations));
  // cout << "SuperLUParmsParms_impl::killInANewThread() after " << secondsToSleep << " seconds " << endl;
  // Need to figure out why howManyIterations is recieved as 0.
  // For now the sleeping time has been fixed to 30 seconds.
  secondsToSleep = 30;
  unsigned int didntSleepFor = sleep(secondsToSleep);
  cout << "SuperLUParmsParms_impl::killInANewThread() done waiting , didn't sleep for " << didntSleepFor << " seconds !! " << endl;
  exit(1);
  return NULL;
}
