/* 
 * Purpose
 * ======= 
 *	Returns the time in seconds used by the process.
 *
 * Note: the timer function call is machine dependent. Use conditional
 *       compilation to choose the appropriate function.
 *
 */

#ifdef SUN 
/*
 * 	It uses the system call gethrtime(3C), which is accurate to 
 *	nanoseconds. 
*/
#include <sys/time.h>
 
double timer_() {
    return ( (double)gethrtime() / 1e9 );
}

#else

#include <sys/types.h>
#include <sys/times.h>
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>

#ifndef CLK_TCK
#define CLK_TCK 60
#endif

double timer_()
{

/*  Original versions of timer routine: 

    struct tms rusage;
    times(&rusage);
    return (double)rusage.tms_utime / CLK_TCK;

    End original version of timer routine */
    struct rusage resources;
    double utime, stime;
    int who = RUSAGE_SELF;
    int ierr;
    ierr = getrusage (who, &resources);
/*  Return garbage value if error occured */
    if (ierr != 0) return (-1.0);

    utime = (double) resources.ru_utime.tv_sec + 1.0e-6 * (double) resources.ru_utime.tv_usec;
    stime = (double) resources.ru_stime.tv_sec + 1.0e-6 * (double) resources.ru_stime.tv_usec;
    return (double) (utime + stime);

}

#endif

