#ifndef __SUPERLU_UTIL /* allow multiple inclusions */
#define __SUPERLU_UTIL

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <assert.h>

/* Macros */
#ifndef USER_ABORT
#define USER_ABORT(msg) superlu_abort_and_exit(msg)
#endif
void superlu_abort_and_exit(char* msg);

#define ABORT(err_msg) \
 { char msg[256];\
   sprintf(msg,"%s at line %d in file %s\n",err_msg,__LINE__, __FILE__);\
   USER_ABORT(msg); }

#ifndef USER_MALLOC
#define USER_MALLOC(size) superlu_malloc(size)
#endif

#define MALLOC(size) \
{ USER_MALLOC(size);}


#define MAX(x, y) 	( (x) > (y) ? (x) : (y) )
#define MIN(x, y) 	( (x) < (y) ? (x) : (y) )

/* 
 * Constants 
 */
#define EMPTY	(-1)
#define NO	(-1)
#define FALSE	0
#define TRUE	1

/*
 * Type definitions
 */
typedef float    flops_t;
typedef unsigned char Logical;
typedef enum {
    RELAX, ETREE, EQUIL, FACT, RCOND, SOLVE, REFINE,
    FLOAT, TRSV, GEMV, FERR, NPHASES
} PhaseType;

typedef struct {
    int     *panel_histo; /* histogram of panel size distribution */
    double  *utime;       /* running time at various phases */
    flops_t *ops;         /* operation count at various phases */
} LUStat_t;

/* Macros */
#define FIRSTCOL_OF_SNODE(i)	(xsup[i])

#ifdef __cplusplus
extern "C" {
#endif

extern void	PrintStat (LUStat_t *);

#ifdef __cplusplus
  }
#endif

#endif /* __SUPERLU_UTIL */
