#include "TimeGenComponent.h"
#include "RunItAgain_impl.h"
#include "TimeGenParms_impl.h"
#include "../Common/TimeStep.h"
#include "../../../../framework/ccacore/Port.h"
#include "../../../../framework/ccacore/CCATServicesProxy.h"
#include "../../../../framework/ccacore/RegisterType.h"
#include "../../../../framework/ccacore/FRAMEWORK_REGISTRATION_ID.h"

#include "../../../../framework/idl/timeStep/UsesTimeStep.h"

TimeGenComponent::TimeGenComponent() {

    int ec = -1;

    m_core = NULL;
    m_runItAgainPort = new RunItAgain_impl(this);
    m_timeGenParmsPort = new TimeGenParms_impl(this);
    m_computing = false;
    m_stopped = false;

    m_currentTime = 0;
    m_timeIncr = 0.1;
    m_m = 5;
    m_n = 5;

    /*
     * Init mutex
     */

    pthread_mutexattr_t attr;

    pthread_mutexattr_init(&attr);

    #if defined(DEBUG) && !defined(__linux__)
        ec = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
        assert(ec == 0);
    #endif

    pthread_mutex_init(&m_lock, &attr);

    pthread_mutexattr_destroy(&attr);

    /*
     * Init condition variable.
     */

    pthread_cond_init(&m_cv, NULL);

    /*
     * Init pthread attr
     */

//     ec = pthread_attr_init(&m_threadAttr);
//     assert(ec == 0);
//     ec = pthread_attr_setdetachstate(&m_threadAttr, PTHREAD_CREATE_DETACHED);
//     assert(ec == 0);
    //ec = pthread_attr_setscope(&m_threadAttr, PTHREAD_SCOPE_SYSTEM);
    //assert(ec == 0);
}

TimeGenComponent::~TimeGenComponent() {

    int ec = -1;

    if (m_core != NULL) {
        delete m_core;
    }

    delete m_runItAgainPort;
    delete m_timeGenParmsPort;

//     ec = pthread_attr_destroy(&m_threadAttr);
//     assert(ec == 0);

    ec = pthread_cond_destroy(&m_cv);
    assert(ec == 0);

    ec = pthread_mutex_destroy(&m_lock);
    assert(ec == 0);
}

void
TimeGenComponent::setServices(Services *cc) {

    if (cc == NULL) {

        assert(m_core != NULL);
        delete m_core;
        m_core = NULL;
        return;

    } else {

        m_core = cc;

        m_core->addProvidesPort(m_runItAgainPort,
         new DefaultPortInfo("inputRunItAgain", "RunItAgain_idl"));
        m_core->addProvidesPort(m_timeGenParmsPort,
         new DefaultPortInfo("inputTimeGenParms", "TimeStepParms_idl"));
        m_core->registerUsesPort(new DefaultPortInfo("outputTimeStep",
         "TimeStep_idl"));
    }
}

void
TimeGenComponent::execute(bool start) {

  static int iterationNo = 0;
    cout << "TimeGenParms_impl::execute() has been called !! " << endl;
    cout << "TimeGenParms_impl::execute() has been called with start " << start << " and m_stopped as " << m_stopped  << endl;

    pthread_mutex_lock(&m_lock);

    if ((start && m_stopped) || (!start && !m_stopped))   {
    //  if ((start && m_stopped) || (start && !m_stopped))   {
      cout << "TimeGenParms_impl::execute() conditions have been met !! " << endl;
            m_stopped = false;

            // If we made it past here, then must be ready to compute.
            TimeStep ts(m_m, m_n, m_currentTime);
            m_currentTime += m_timeIncr;

            UsesTimeStep *up = dynamic_cast<UsesTimeStep *>((m_core)->getPort("outputTimeStep"));
            up->sendTimeStep(&ts);
            m_core->releasePort("outputTimeStep");
	    cout << "TimeGenParms_impl::execute() sendTimeStep() done " << endl << flush;
    }  

    pthread_mutex_unlock(&m_lock);
    cout << "*******************************" << endl;
    cout << "TimeGenComponent: iterationNo: " << iterationNo << endl << flush; 
    cout << "*******************************" << endl;
    iterationNo++;
}
