#include "TimeGenParms_impl.h"
#include "TimeGenComponent.h"
#include "../Common/TimeStep.h"
#include "../../../../framework/idl/timeStep/UsesTimeStep.h"
#include "TimeGenParms.h"
#include <unistd.h>

const int TimeGenParms_impl::maxDirectoryNameLength = 128;

TimeGenParms_impl::TimeGenParms_impl(TimeGenComponent *c) : m_component(c) {
  int ec = -1;
  ec = pthread_attr_init(&m_threadAttr);
  assert(ec == 0);
  ec = pthread_attr_setdetachstate(&m_threadAttr, PTHREAD_CREATE_DETACHED);
  assert(ec == 0);
  m_directoryName = "timegen/";
}

TimeGenParms_impl::~TimeGenParms_impl() {
}

// these methods will be invoked remotely.
// all these methods are part of the
// TimeGenParms_idl interface.

// if you receive a new parms object
// then ditch the old one and replace it
// with this new one. 
int
TimeGenParms_impl::sendTimeStepParms(TimeGenParms *tsp) {
 
	cout << "TimeGenParms_impl::sendTimeGenParms() has been called !! " << endl;
	cout << "TimeGenParms_impl::sendTimeGenParms() has been called !! initialTime:" <<tsp->initialTime <<  endl;
	cout << "TimeGenParms_impl::sendTimeGenParms() has been called !! timeIncr:" << tsp->timeIncr << endl;
	cout << "TimeGenParms_impl::sendTimeGenParms() has been called !! M:" << tsp->m << endl;
	cout << "TimeGenParms_impl::sendTimeGenParms() has been called !! N:" << tsp->n << endl;

  // lock before entering the critical section
    pthread_mutex_lock(&m_component->m_lock);

    //    assert(m_component->m_parms_cache != NULL);

    m_component->m_currentTime = tsp->initialTime;
    m_component->m_timeIncr = tsp->timeIncr;
    m_component->m_m = tsp->m;
    m_component->m_n = tsp->n;

    
    // unlock before leaving the critical section
    pthread_mutex_unlock(&m_component->m_lock);

    cout << "TimeGenParms_impl::setParms returning !!!! " << endl;
    return 0;
}

// the port has received a signal to start executing.
// set the condition variable m_stopped = false.
// The method sendRunItAgain() would be 
// unblocked by this condition variable.
int
TimeGenParms_impl::start() {
  cout << "TimeGenParms_impl::start() has been called !! " << endl;
  // added by Madhu 
  // don't smirk. I am trying to fix a deadlock here.
  m_component->m_stopped = false;
  m_component->execute(false);
  // end of changes  
  //  m_component->execute(true);
  return 0;
}
// the port has received a signal to stop executing.
// set the condition variable m_stopped = true.
// The method sendRunItAgain() would be 
// blocked by this condition variable. No new
// compute() can take place as long as the condition
// variable is not changed. 

int
TimeGenParms_impl::stop() {

    cout << "TimeGenParms_impl::stop() has been called !! " << endl;

    pthread_mutex_lock(&m_component->m_lock);

    m_component->m_stopped = true;

    pthread_mutex_unlock(&m_component->m_lock);

    return 0;
}

// The easiest cleanup that can be done !!
// int
// TimeGenParms_impl::kill() {
//     cout << "TimeGenParms_impl::kill() has been called !! " << endl;
    
//     exit(1);

//     return 0;
// }

int
TimeGenParms_impl::kill() {
  pthread_t tid;
  cout << "TimeGenParms_impl::kill() has been called !! " << endl;
  int afterHowManyIterations = 10;
  pthread_create(&tid, &m_threadAttr, killInANewThread, &afterHowManyIterations);
  cout << "TimeGenParms_impl::kill() returning !! " << endl;
  return 0;
}

char*
TimeGenParms_impl::getOutputURL() {
  cout << "TimeGenParmsParms_impl::getOutputURL() has been called !! " << endl;
  char *buffer = new char[maxDirectoryNameLength];
  sprintf(buffer,"/%s",m_directoryName);
  return buffer;
}

// wait for a few seconds before dying. This looks like
// a hack. However, I believe this is just a good design
// practice. The user of a the CCAT composition engine needs
// to have some time to disconnect other components before this
// component dies. Note, that other components that have a 
// startpoint to this component will get error signals if they aren't
// disconnected before this component dies. 

void *TimeGenParms_impl::killInANewThread(void *howManyIterations) {
  int secondsToSleep = *(static_cast<int *>(howManyIterations));
  // cout << "TimeGenParms_impl::killInANewThread() after " << secondsToSleep << " seconds " << endl;
  // Need to figure out why howManyIterations is recieved as 0
  // For now the sleeping time has been fixed to 10 seconds.
  secondsToSleep = 10;
  unsigned int didntSleepFor = sleep(secondsToSleep);
  cout << "TimeGenParms_impl::killInANewThread() done waiting , didn't sleep for " << didntSleepFor << " !! " << endl;
  exit(1);
  return NULL;
}

 
