
#include <iostream>
#include <string>
#include "../../framework/ccacore/CCATServicesProxy.h"
#include "../../framework/creation/CreationService.h"
#include "../../framework/connection/ConnectionService.h"
#include "../../framework/ccacore/ComponentID.h"
#include "../../framework/ccacore/DefaultPortInfo.h"
#include "../../framework/ccacore/Port.h"
#include "../../framework/idl/eventChannel/ProvidesFilteringEventChannelSkel.h"
#include "../../framework/idl/eventChannel/UsesFilteringEventChannelSkel.h"
#include "../system/event-channel/EventChannelComponentHome.h"
#include "../../framework/ccacore/RegisterType.h"
#include <stdio.h>


#ifdef sgi
extern "C" char *strdup(const char *);
#endif

void TestEvents(char *host1, char *host2) {
  cout << "TestEvents() executing !! " << endl;

  EventChannelComponentHome::init();

  // hardcoded for now 
  // these will be obtained from 
  // the InformationDirectory service
  char *execPath1 = strdup("/tmp/madhu/c++/eventGenerator");
  //  char *execHost1 = strdup("baldy.extreme.indiana.edu");
  char *execHost1 = strdup(host1);

  char *execPath2 = strdup("/tmp/madhu/c++/eventChannel");
  // char *execHost2 = strdup("baldy.extreme.indiana.edu");
  char *execHost2 = strdup(host2);
   
  // CCAT specific core needs to be created for
  // this composition engine
  CCATServicesSkel *core = new CCATServicesSkel();
  core->registerUsesPort(new DefaultPortInfo("filteringEventChannel", 
									"FilteringEventChannel_idl"));
  ComponentID* myCompID = core->getComponentID();


  // get the CreationService Port
  cout << "TestEvents:: attempting to get creationServicePort " << endl << endl;
  cout.flush();
  CreationService *creationService = (CreationService *)(core->getPort("CreationService"));
  cout << "TestEvents:: creationServicePort recieved " << endl << endl;
  cout.flush();
  
  // get the ConnectionService Port
  cout << "TestEvents:: attempting to get connectionServicePort " << endl << endl;
  cout.flush();
  ConnectionService *connectionService = 
    (ConnectionService *)(core->getPort("ConnectionService"));
  cout << "TestEvents:: connectionServicePort received " << endl << endl;
  cout.flush();

  // The creationService API should take the envObject as
  // an argument. However for testing purposes just two 
  // strings needed for gram instantiation are passed on
  // to it. 
  // instantiate the first component
  cout << "TestEvents:: attempting to get componentID for: " << execPath1 << endl << endl;
  cout.flush();

  char url1[200];
  url1[0] = '\0';
  strcat(url1, "gram:/");
  strcat(url1, execHost1);
  strcat(url1, "/MyEventGenerator"); // component's dummy full name

  EnvObj e1;
  e1.put ("executable", execPath1);
  
  ComponentID* eventGenCompID = creationService->createInstance(url1, &e1);

  cout << "TestEvents:: eventGenCompID for: " << execPath1 << " recieved " << endl << endl;
  cout << "TestEvents:: eventGenCompID name is : " << eventGenCompID->getName() << endl << endl;
  cout.flush();


  // instantiate the second component
  cout << "TestEvents:: attempting to get componentID for: " << execPath2 << endl << endl;
  cout.flush();
  
  char url2[200];
  url2[0] = '\0';
  strcat(url2, "gram:/");
  strcat(url2, execHost2);
  strcat(url2, "/MyEventChannelComponent"); // component's dummy full name

  EnvObj e2;
  e2.put ("executable", execPath2);
  
  ComponentID* eventChannelCompID = creationService->createInstance(url2, &e2);

  cout << "TestEvents:: eventChannelCompID for: " << execPath2 << " recieved " << endl << endl;
  cout.flush();
  

  // First connect the uses filteringEventChannel port of this composition
  // engine with the register Provides port of the event Channel
  cout << "TestEvents:: attempting to connect eventChannel registerProvides Port::"
	  << endl << flush;
  connectionService->connect(myCompID, "filteringEventChannel", eventChannelCompID, "register");
  cout << "TestEvents:: connect succeeded ::"<< endl;

  cout << "TestEvents:: attempting to get UsesFilteringEventChannel port for this engine::"
	  << endl;
  UsesFilteringEventChannel *outputPort = 
    dynamic_cast<UsesFilteringEventChannel
    *>(core->getPort("filteringEventChannel"));

  cout << "TestEvents:: UsesFilteringEventChannel port for this engine received ::"<< endl;
  cout << "TestEvents:: invoking outputPort->registerPublisher() "<< endl;

  outputPort->registerPublisher(eventGenCompID, "publish");
  cout << "TestEvents:: invoking outputPort->registerPublisher() done "<< endl;

  // FIXME FIXME, need the type of event to app here 
  cout << "TestEvents:: invoking outputPort->registerListener() "<< endl;
  cout << "TestEvents:: Testing the eventGenCompID " << endl;
  cout << "TestEvents:: eventGenCompID->getName() : " << eventGenCompID->getName() << endl;
  outputPort->registerListener(eventGenCompID, "listen", "debugger");
  cout << "TestEvents:: invoking outputPort->registerListener() done "
	  << endl << flush;
  
  core->releasePort("filteringEventChannel");
  double timeout = 300;
  cout << "TestEvents:: invoking disconnect "<< endl;
  connectionService->disconnect(myCompID, "filteringEventChannel",
						  eventChannelCompID, "register", timeout);
  cout << "TestEvents:: disconnect done "<< endl;
  cout.flush();

  cout << "TestEvents:: calling run on eventGenerator "<< endl;
  cout.flush();
  eventGenCompID->run();
  cout << "TestEvents:: run on eventGenerator done ...  "<< endl;
  cout.flush();
  

  HPCxx_Sync<int> justWait;
  int waitForKill = justWait;

  cout  <<"Compose:: End of test " << endl << endl;
  cout.flush();

  // do your own cleanup
  delete execPath1;
  delete execPath2;
  delete execHost1;
  delete execHost2;
  delete creationService;
  delete connectionService;
  
  
  // FIXME 
  // Am not sure if these componentIDs 
  // should be deleted by the composition
  // engine. Other components may have
  // these componentID in their addressSpace.
  // It is better to actually send a kill
  // signal to the component so that 
  // it can do setServices(NULL);

  //delete remoteComponentID1;
  // delete remoteComponentID2;

  // Inform HPCxx that you want to exit
  hpcxx_exit();
}

