#include <iostream>
#include <string>
#include <stdlib.h>
#include "../../framework/ccacore/CCATServicesProxy.h"
#include "../../framework/creation/CreationService.h"
#include "../../framework/connection/ConnectionService.h"
#include "../../framework/ccacore/ComponentID.h"
#include "../../framework/ccacore/DefaultPortInfo.h"
#include "../../framework/ccacore/Port.h"
#include "../../framework/idl/splibParms/UsesSplibParmsSkel.h"
#include "../../framework/idl/sparseLinearSystem/SparseLinearSystem_idl.h"
#include "../system/event-channel/EventChannelComponentHome.h"
#include "../application/lsa/Splib/SplibComponentHome.h"
#include "../application/lsa/GetSystemFromFile/GetSystemFromFileComponentHome.h"
#include "../../framework/ccacore/RegisterType.h"
#include <stdio.h>


#ifdef sgi
extern "C" char *strdup(const char *);
#endif

void TestLSA(char *host1, char *host2) {

  cout << "TestLSA() executing !! " << endl;

  EventChannelComponentHome::init();
  // hardcoded for now 
  // these will be obtained from 
  // the InformationDirectory service

  char
	execPath1[1024],
	execPath2[1024],
	execPath3[1024],
	execPath4[1024],
	execPath5[1024],
	execPath6[1024],
	execPath7[1024],
	execPath8[1024],
	execPath9[1024],
	execPath10[1024];

  char *logname = getenv("LOGNAME");
  assert(logname != NULL);

  if (strcmp(logname, "mgovinda") == 0) {
	logname = "madhu";
  }

  sprintf(execPath1, "/tmp/%s/getSystemFromFile", logname);
  char *execHost1 = strdup(host1);

  sprintf(execPath2, "/tmp/%s/reorder", logname);
  char *execHost2 = strdup(host2);

  // CCAT specific core needs to be created for
  // this composition engine
  CCATServicesSkel *core = new CCATServicesSkel();
  ComponentID* myCompID = core->getComponentID();

  // get the CreationService Port
  cout << "TestLSA:: attempting to get creationServicePort " << endl << endl;
  cout.flush();
  CreationService *creationService = (CreationService *)(core->getPort("CreationService"));
  cout << "TestLSA:: creationServicePort recieved " << endl << endl;
  cout.flush();
  
  // get the ConnectionService Port
  cout << "TestLSA:: attempting to get connectionServicePort " << endl << endl;
  cout.flush();
  ConnectionService *connectionService = 
    (ConnectionService *)(core->getPort("ConnectionService"));
  cout << "TestLSA:: connectionServicePort received " << endl << endl;
  cout.flush();

  // The creationService API should take the envObject as
  // an argument. However for testing purposes just two 
  // strings needed for gram instantiation are passed on
  // to it. 
  // instantiate the first component
  cout << "TestEvents:: attempting to get componentID for: " << execPath1 << endl << endl;
  cout.flush();

  char url1[200];
  url1[0] = '\0';
  strcat(url1, "gram:/");
  strcat(url1, execHost1);
  strcat(url1, "/MyGetSystemFromFile"); // component's dummy full name

  EnvObj e1;
  e1.put ("executable", execPath1);
  
  ComponentID* getSystemFromFileCompID = creationService->createInstance(url1, &e1);

  cout << "LSAEvents:: getSFFCompID for: " << execPath1 << " recieved " << endl << endl;
  cout << "LSAEvents:: getSFFCompID name is : " << getSystemFromFileCompID->getName() << endl << endl;
  cout.flush();


  // instantiate the second component
  cout << "LSAEvents:: attempting to get componentID for: " << execPath2 << endl << endl;
  cout.flush();
  
  char url2[200];
  url2[0] = '\0';
  strcat(url2, "gram:/");
  strcat(url2, execHost2);
  strcat(url2, "/MySplibComponent"); // component's dummy full name

  EnvObj e2;
  e2.put ("executable", execPath2);
  
  ComponentID* splibCompID = creationService->createInstance(url2, &e2);

  cout << "LSAEvents:: splibCompID for: " << execPath2 << " recieved " << endl << endl;
  cout.flush();

  // connect the getSystemFromFile component's linear system uses port with the
  // splib component's linear system provides port.
  cout << "LSAEvents:: attempting to connect eventChannel getSFF and splib ::"
	  << endl << flush;
  connectionService->connect(getSystemFromFileCompID, "outputSLS", splibCompID, "inputSLS");
  cout << "LSAEvents:: connect succeeded ::"<< endl;

//   core->registerUsesPort(new DefaultPortInfo("outputSplibParms", "SplibParms_idl"));
  
//   UsesSplibParms *output = dynamic_cast<UsesSplibParms *>(core->getPort("outputSplibParms"));
//   SplibParms splibParms;
//   int returnValue = output->sendSplibParms(&splibParms);

  HPCxx_Sync<int> justWait;
  int waitForKill = justWait;

  cout  <<"Compose:: End of test " << endl << endl;
  cout.flush();

  // do your own cleanup
  delete execHost1;
  delete execHost2;
  delete creationService;
  delete connectionService;
  
  
  // FIXME 
  // Am not sure if these componentIDs 
  // should be deleted by the composition
  // engine. Other components may have
  // these componentID in their addressSpace.
  // It is better to actually send a kill
  // signal to the component so that 
  // it can do setServices(NULL);

  //delete remoteComponentID1;
  // delete remoteComponentID2;

  // Inform HPCxx that you want to exit
  hpcxx_exit();
}

