#include "EventChannelComponent.h"
#include "FilteringEventChannel_impl.h"
#include "EventQueueManager.h"
#include "EventPort_impl.h"

#include "../../../framework/ccacore/Port.h"
#include "../../../framework/ccacore/CCATServicesProxy.h"
#include "../../../framework/ccacore/RegisterType.h"
#include "../../../framework/ccacore/FRAMEWORK_REGISTRATION_ID.h"

EventChannelComponent::EventChannelComponent() {
	
	m_core = NULL;
	m_eventQueueManager = new EventQueueManager;
	m_filteringEventChannel = new
	 FilteringEventChannel_impl(m_eventQueueManager);
	m_eventPort = new EventPort_impl(m_eventQueueManager);
}

EventChannelComponent::~EventChannelComponent() {

	if (m_core != NULL) {
		delete m_core;
	}

	delete m_eventQueueManager;
	delete m_filteringEventChannel;
	delete m_eventPort;
}

void
EventChannelComponent::setServices(Services *cc) {

	if (cc == NULL) {

		// XXX Do we need to release the connectionService port here?
		assert(m_core != NULL);
		m_filteringEventChannel->m_core = NULL;
		m_eventQueueManager->m_core = NULL;
		delete m_core;
		m_core = NULL;
		return;

	} else {

		m_core = cc;
		m_filteringEventChannel->m_core = cc;
		m_eventQueueManager->m_core = cc;

		m_connectionService = dynamic_cast<ConnectionService *>
			(cc->getPort("ConnectionService"));
		assert(m_connectionService != NULL);
		m_filteringEventChannel->m_connectionService = m_connectionService;

		m_core->addProvidesPort(m_filteringEventChannel,
		 new DefaultPortInfo("register", "FilteringEventChannel_idl"));
		m_core->addProvidesPort(m_eventPort,
		 new DefaultPortInfo("notify", "EventPort_idl"));
	}
}
