#include "EventQueue.h"
#include <iostream>
#include <assert.h>
#include "Listener.h"
#include "Ptr.h"
#include "../../../framework/idl/eventPort/Event.h"
#include "../../../framework/idl/eventPort/UsesEventPort.h"
#include "../../../framework/ccacore/Services.h"
#include "EventWrapper.h"


//#include <malloc.h>

/*
static int
mu() {
	struct mallinfo mi = mallinfo();
	return mi.usmblks + mi.uordblks;
}
*/


/*----------------------------------------------------------------------------\
|                                                                             |
|                                                                             |
|  EventQueue                                                                |
|                                                                             |
|                                                                             |
\----------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 *
 *  Types-Constants
 */

/*------------------------------------------------------------------------------
 *
 *  Constructors-Initters-Assignment-Conversions-Destructors
 */

EventQueue::EventQueue(std::string type, Services *core) : m_type(type),
 m_core(core) {

	int ec = -1;

	m_nEvents = 0;

	/*
	 * Init mutexes
	 */

    pthread_mutexattr_t attr;

    pthread_mutexattr_init(&attr);

    #if defined(DEBUG) && !defined(__linux__)
        ec = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
        assert(ec == 0);
    #endif

    pthread_mutex_init(&m_setLock, &attr);
    pthread_mutex_init(&m_queueLock, &attr);

    pthread_mutexattr_destroy(&attr);

	/*
	 * Init condition variable.
	 */

	pthread_cond_init(&m_queueCV, NULL);

	std::cerr << "EventQueue created " << std::endl;

	/*
	 * Init pthread attr
	 */

	ec = pthread_attr_init(&m_threadAttr);
	assert(ec == 0);
	//ec = pthread_attr_setscope(&m_threadAttr, PTHREAD_SCOPE_SYSTEM);
	//assert(ec == 0);
}

EventQueue::~EventQueue() {

    assert(m_core != NULL);

    int ec = -1;

	ec = pthread_attr_destroy(&m_threadAttr);
	assert(ec == 0);

	ec = pthread_cond_destroy(&m_queueCV);
	assert(ec == 0);

    ec = pthread_mutex_destroy(&m_setLock);
    assert(ec == 0);

    ec = pthread_mutex_destroy(&m_queueLock);
    assert(ec == 0);

	std::cerr << "EventQueue destroyed " << std::endl;
}

/*------------------------------------------------------------------------------
 *
 *  Access Operators-Methods
 */

/*------------------------------------------------------------------------------
 *
 *  Operators
 */

/*------------------------------------------------------------------------------
 *
 *  Methods
 */

EventQueue::ListenerSet::iterator
EventQueue::addListener(const Ptr<Listener> &l) {

	pthread_mutex_lock(&m_setLock);
	std::pair<ListenerSet::iterator, bool> res = m_set.insert(l);
	pthread_mutex_unlock(&m_setLock);
	assert(res.second);

	return res.first;
}

void
EventQueue::removeListener(ListenerSet::iterator it) {

	assert(it != m_set.end());

	pthread_mutex_lock(&m_setLock);
	m_set.erase(it);
	pthread_mutex_unlock(&m_setLock);

	#ifdef DEBUG
		it = m_set.end();
	#endif
}

void
EventQueue::send(const Ptr<EventWrapper> &ev) {


/*
	pthread_mutex_lock(&m_setLock);
	for (ListenerSet::iterator it = m_set.begin(); it != m_set.end(); it++) {
		
		portm_core->getPort(port);
		std::cerr << "sending to " << (*it)->id().stringify() << std::endl;
	}
	pthread_mutex_unlock(&m_setLock);
*/

	std::cerr << "queuing event of type " << ev->type() << std::endl;

	pthread_mutex_lock(&m_queueLock);

	m_queue.push_back(ev);

	if (m_nEvents++ == 0) {
		pthread_cond_signal(&m_queueCV);
	}

	pthread_mutex_unlock(&m_queueLock);

}

void
EventQueue::start() {

	pthread_create(&m_tid, &m_threadAttr, runMe, this);
}

static void
unlock(void *vp) {
	pthread_mutex_unlock((pthread_mutex_t *) vp);
}

/*
static void
clean(void *vp) {
	Ptr<EventWrapper> *ev = static_cast<Ptr<EventWrapper> *>(vp);
	ev->~Ptr<EventWrapper>();
}
*/

void
EventQueue::run() {

	std::string port_name = m_type + "_uses";

	/*
	while (1) {
		pthread_testcancel();
	}
	*/

	pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);

	while (1) {

		pthread_mutex_lock(&m_queueLock);

		pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
		pthread_cleanup_push(unlock, &m_queueLock);

		while (m_nEvents == 0) {
			pthread_cond_wait(&m_queueCV, &m_queueLock);
		}

		pthread_cleanup_pop(0);
		pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);

		Ptr<EventWrapper> ev = m_queue.front();
		m_queue.pop_front();
		m_nEvents--;

		pthread_mutex_unlock(&m_queueLock);

		std::cerr << "sending event of type " << ev->type()
		 << std::endl;

		// XXX fix the cast away const later.
		UsesEventPort *port
		 = dynamic_cast<UsesEventPort *>(m_core->getPort((char *) port_name.c_str()));
		port->notify(&ev->m_ccaevent);
		m_core->releasePort((char *) port_name.c_str());
	}
}

void
EventQueue::stop() {

	int ec = -1;
	void *vp;

	ec = pthread_cancel(m_tid);
	assert(ec == 0);

	ec = pthread_join(m_tid, &vp);
	assert(ec == 0);
}

/*------------------------------------------------------------------------------
 *
 *  Static Methods and Helper Functions
 */

void *
EventQueue::runMe(void *me) {

	EventQueue *eq = static_cast<EventQueue *>(me);

	eq->run();

	return NULL;
}

/*------------------------------------------------------------------------------
 *
 *  Static Data
 */
