#ifndef CCAT_LISTENER_SET_H
#define CCAT_LISTENER_SET_H



#include <string>
#include <set>
#include <map>
#include <deque>
#include <pthread.h>
#include "Ptr.h"
#include "PtrObject.h"
#include "EventWrapper.h"
#include "Listener.h"





/*----------------------------------------------------------------------------\
|                                                                             |
|                                                                             |
|  EventQueue                                                                 |
|                                                                             |
|                                                                             |
\----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
|  Forwards                                                                   |
-----------------------------------------------------------------------------*/

class Event;
class Services;

/*-----------------------------------------------------------------------------
|  Inline Helper Functions                                                    |
-----------------------------------------------------------------------------*/

class EventQueue : public PtrObject {

    /*--------------------------------------------------------------------------
     *  Access Specifiers
     */

    /*--------------------------------------------------------------------------
     *  Forwards
     */

    /*--------------------------------------------------------------------------
     *  Friends
     */

    /*--------------------------------------------------------------------------
     *  Types-Constants
     */

	public:
		
		typedef std::set<Ptr<Listener>, Listener::Cmp> ListenerSet;

    /*--------------------------------------------------------------------------
     *  Constructors-Initters-Assignment-Conversions-Destructors
     */

	public:
		
		EventQueue(std::string type, Services *core);
		~EventQueue();
    
    /*--------------------------------------------------------------------------
     *  Access Operators-Methods
     */

	public:

		const std::string type() const { return m_type; }
		int nListeners() { return m_set.size(); }

    /*--------------------------------------------------------------------------
     *  Operators
     */

    /*--------------------------------------------------------------------------
     *  Methods
     */

	public:

		ListenerSet::iterator addListener(const Ptr<Listener> &l);
		void removeListener(ListenerSet::iterator);
		void send(const Ptr<EventWrapper> &ev);
		void setLink(std::pair<std::map<std::string, Ptr<EventQueue> >::iterator, bool>);

		void start();
		void stop();

	private:

		void run();

    /*--------------------------------------------------------------------------
     *  Data
     */

	private:

		const std::string m_type;
		ListenerSet m_set;
		pthread_mutex_t m_setLock;
		pthread_mutex_t m_queueLock;
		pthread_cond_t m_queueCV;
		std::deque<Ptr<EventWrapper> > m_queue;
		int m_nEvents;

		pthread_attr_t m_threadAttr;

		pthread_t m_tid;

		Services *const m_core;

	public:
		
		std::map<std::string, Ptr<EventQueue> >::iterator typeMapPos;

    /*--------------------------------------------------------------------------
     *  Static Methods
     */

	private:

		static void *runMe(void *);

    /*--------------------------------------------------------------------------
     *  Static Data
     */
};



#endif
