#include "FilteringEventChannel_impl.h"
#include <iostream>
#include <string>
#include <assert.h>
#include "Listener.h"
#include "EventQueue.h"
#include "EventQueueManager.h"
#include "../../../framework/ccacore/Services.h"
#include "../../../framework/ccacore/DefaultPortInfo.h"
#include "../../../framework/connection/ConnectionService.h"



/*----------------------------------------------------------------------------\
|                                                                             |
|                                                                             |
|  FilteringEventChannel_impl                                                 |
|                                                                             |
|                                                                             |
\----------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 *
 *  Types-Constants
 */

/*------------------------------------------------------------------------------
 *
 *  Constructors-Initters-Assignment-Conversions-Destructors
 */

FilteringEventChannel_impl::FilteringEventChannel_impl(EventQueueManager *eqm)
 : m_eventQueueManager(eqm), m_core(NULL), m_connectionService(NULL) {

	pthread_mutexattr_t attr;

	pthread_mutexattr_init(&attr);

	#if defined(DEBUG) && !defined(__linux__)
		int ec = -1;
		ec = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
		assert(ec == 0);
	#endif

	pthread_mutex_init(&m_registrationLock, &attr);

	pthread_mutexattr_destroy(&attr);

}

FilteringEventChannel_impl::~FilteringEventChannel_impl() {

	int ec = -1;

	ec = pthread_mutex_destroy(&m_registrationLock);
	assert(ec == 0);

}

/*------------------------------------------------------------------------------
 *
 *  Access Operators-Methods
 */

/*------------------------------------------------------------------------------
 *
 *  Operators
 */

/*------------------------------------------------------------------------------
 *
 *  Methods
 */

int
FilteringEventChannel_impl::registerListener(ComponentID *listener,
 char *lport, char *type) throw (AlreadyRegisteredException) {

	assert(m_core != NULL);
	assert(m_connectionService != NULL);

	bool new_port = false;
	int ec = -1;
   
   std::cout << "FilteringEventChannel_impl::registerListener got strings: " << lport << " " << type << std::endl;
   std::cout << "FilteringEventChannel_impl::registerListener says I LOVE C+++++++++++++"<< std::endl;
   std::cout << "FilteringEventChannel_impl::registerListener recd compID for "<<  listener->getName()  << std::endl;
   std::cout.flush();

   std::cerr << "FilteringEventChannel_impl::registerListener got strings: " << lport << " " << type << std::endl;
   std::cerr << "FilteringEventChannel_impl::registerListener says I LOVE C+++++++++++++"<< std::endl;
   std::cout.flush();


	ec = pthread_mutex_lock(&m_registrationLock);
	assert(ec == 0);

	// toMyString leaks memory here.
	std::string my_port
	 = m_eventQueueManager->addListener(listener->toMyString(), lport, type,
	 &new_port);

	if (new_port) {
		m_core->registerUsesPort(new DefaultPortInfo(my_port, "EventPort_idl"));
	}

	m_connectionService->connect(m_core->getComponentID(), my_port.c_str(),
	 listener, lport);

	ec = pthread_mutex_unlock(&m_registrationLock);
	assert(ec == 0);

	return 0;
}

int
FilteringEventChannel_impl::registerPublisher(ComponentID *publisher,
 char *publisher_uses_port) throw (AlreadyRegisteredException) {

	assert(m_core != NULL);
	assert(m_connectionService != NULL);

	std::cout << "FilteringEventChannel_impl::registerPublisher got string: " << publisher_uses_port << std::endl;
	std::cout << "FilteringEventChannel_impl::registerPublisher says Yahooooooooooooooooooooooo" << publisher_uses_port << std::endl;
   std::cout << "FilteringEventChannel_impl::registerPublisher recd compID for "<<  publisher->getName()  << std::endl;
	std::cout.flush();

	std::cerr << "FilteringEventChannel_impl::registerPublisher got string: " << publisher_uses_port << std::endl;
	std::cerr << "FilteringEventChannel_impl::registerPublisher says Yahooooooooooooooooooooooo" << publisher_uses_port << std::endl;
   std::cout << "FilteringEventChannel_impl::registerPublisher recd compID for "<<  publisher->getName()  << std::endl;
	std::cout.flush();

	ComponentID *my_id = m_core->getComponentID();
	assert(my_id != NULL);
	m_connectionService->connect(publisher, publisher_uses_port, my_id,
	 "notify");
	// XXX Should I delete my_id here?  -kec

	return 0;
}

int
FilteringEventChannel_impl::unregister(ComponentID *component,
 char *port) throw (NotRegisteredException) {

	assert(m_core != NULL);
	assert(m_connectionService != NULL);

	std::string port_name;
	int ec = -1;

	ec = pthread_mutex_lock(&m_registrationLock);
	assert(ec == 0);

	// First, try to remove it as a listener.  If that fails, it must
	// be a publisher.
	if (m_eventQueueManager->removeListener(component->toMyString(), port,
	 &port_name)) {

		m_connectionService->disconnect(m_core->getComponentID(),
		 port_name.c_str(), component, port, 999999);

	} else {

		// Must be a publisher.
		m_connectionService->disconnect(component, port,
		 m_core->getComponentID(), "notify", 999999);
	}

	ec = pthread_mutex_unlock(&m_registrationLock);
	assert(ec == 0);

	return 0;
}

/*------------------------------------------------------------------------------
 *
 *  Static Methods and Helper Functions
 */

/*------------------------------------------------------------------------------
 *
 *  Static Data
 */


