#include <iostream>
#include <stdlib.h>
#include <unistd.h>
#include <malloc.h>
#include "FilteringEventChannel_impl.h"
#include "EventQueueManager.h"
#include "EventPort_impl.h"
#include "EventChannelComponent.h"
#include "EventChannelComponentHome.h"
#include "../../../framework/idl/eventPort/Event.h"
#include "../../../framework/idl/eventPort/UsesEventPortSkel.h"
#include "../../../framework/idl/eventPort/ProvidesEventPortSkel.h"
#include "../../../framework/idl/eventChannel/ProvidesFilteringEventChannelSkel.h"

#include "../../../framework/ccacore/Port.h"
#include "../../../framework/ccacore/Component.h"
#include "../../../framework/ccacore/CCATServicesProxy.h"
#include "../../../framework/ccacore/RegisterType.h"
#include "../../../framework/ccacore/FRAMEWORK_REGISTRATION_ID.h"
#include <hpcxx.h>


HPCxx_Mutex myLock;

EventChannelComponent *channel = NULL;
CCATServicesSkel *channelCore = NULL;

// Flag used to make sure that the core is initialized
// before the function  returnComponentID tries to access it.
// We cannot use sync variables here, as hpcxx_initAgent
// may not have gone  through (i.e. Nexus may not have been
// initialized) when this function will get called.
static bool coreDone = false;

// This global method is invoked by the creation
// service (gram) to get a handle to the component.
// It is okay to busy wait (and hence waste cpu cycles)
// as the component should not be allowed to attempt to
// do anything useful before coreDone gets set to true
CCATServicesProxy returnComponentID(){
  while (!coreDone) ;
  return *((CCATServicesProxy *)(channelCore->getComponentID()));
}

int main(int argc, char **argv) {
  hpcxx_initAgent(argc, argv);

  // returnComponentID(...) is specific to
  // a user component. As a result it needs to
  // be registered here
  hpcxx_register(returnComponentID, RETURN_COMPONENT_ID);
  cout.flush();

  // register all the remote methods
  // that will be remotely invoked
  // once the component is initialized
  // Needs to be done only once per
  // instantiation, hence it is a static
  // method call
  CCATServicesSkel::registerAll();

  // Avoid race condtions
  // Let only one thread be able to
  // setServices(...) on the component
  myLock.lock();


  // Create an actual user component instance
  channel = EventChannelComponentHome::instantiate();

  // Instantiate a CCAT specific implementation
  // of the services object
  channelCore = new CCATServicesSkel();

  // This call is required as part of Component
  // Interface. The component now has a handle
  // to the framework.
  channel->setServices(channelCore);

  myLock.unlock();

  // It is now okay to return the componentID
  // to the creationService
  coreDone = true;

  std::cout << "EventChnnaelkjl:Up and waiting for requests\n" << std::endl;
  cout.flush();

  // The component needs to get beyond this
  // blocking call to call the component's execute()
  // method. This call will unblock when
  // he composition engine calls run() on the
  // the componentID of this component.
  channelCore->startRunning();
  std::cout << "EventChannel: Main:: Received signal to execute \n" << std::endl;
  cout.flush();


  // Will fix this later
  // This needs to be used, but am not
  // sure if this is the righ place for it
  // FIXME FIXME
  channelCore->waitForKill();
  cout << "Component has been killed! \n ";
  cout << "EventChannel exiting ... \n";
  cout.flush();

  // This sleep() call is useful if the
  // above kill statement is used. Just ensures
  // that the component dies before HPCxx
  // starts its cleanup
  sleep(5);
  hpcxx_exit();
}



#if 0
int
main(int argc, char *argv[]) {

	long count = 0;

	EventQueueManager *lsm = new EventQueueManager;
	FilteringEventChannel_impl *ec = new FilteringEventChannel_impl(lsm);
	EventPort_impl *ep = new EventPort_impl(lsm);

	Event ev1("event1", "ccaevent.connection");
	Event ev2("event2", "ccaevent.connection");

	ComponentID id1("listener1");
	ComponentID id2("listener2");

	ec->registerListener(&id1, "port1", "ccaevent.connection");

	ep->notify(&ev1);
	ep->notify(&ev2);

	int allocated = 0;
	ec->registerListener(&id2, "port2", "ccaevent.connection");

	for (int i = 0; i < 5; i++) {

		int t;

		count++;

		ep->notify(&ev1);
//		ep->notify(&ev);
		sleep(1);


		/*
		struct mallinfo mi = mallinfo();
		t = mi.usmblks + mi.uordblks;
		if (t != allocated) {
			allocated = t;
			cout << "allocated: " << allocated << "  count: " << count << std::endl;
		}
		*/
	}

	ec->unregister(&id1, "port1");
	ec->unregister(&id2, "port2");
	std::cerr << "done" << std::endl;
}
#endif
