// -*-c++-*-

// this class provides the interface
// needed for class that implements ComponentID.
// In our implementation it is a proxy
// for the services object with a restricted
// view to the composition engine. 
// The pseudo components being part of the
// framework can view the private interaface 
// (private methods) of the ComponentID. 
// They get that view as they have been
// delcared "friends" of ComponentID
// --Madhu

#ifndef _CCAT_SERVICES_PROXY_H__
#define _CCAT_SERVICES_PROXY_H__

#include "ComponentID.h"
#include "CCATServicesSkel.h"
#include "Port.h"
#include "ProvidesPort.h"
#include "PortInfo.h"
#include "../util/MyHPCxx_GlobalPtr.h"

class CCATServicesProxy: public ComponentID {

public: 
  MyHPCxx_GlobalPtr<CCATServicesSkel> *skp;

public:
  CCATServicesProxy();
  CCATServicesProxy(CCATServicesSkel *);
  CCATServicesProxy(HPCxx_ContextID* contextID);
  CCATServicesProxy(CCATServicesProxy&);
  CCATServicesProxy(CCATServicesProxy *);

  virtual ~CCATServicesProxy();

  // remote methods used by users of the
  // componentID. Part of the ComponentID
  // interface.
  PortInfo** getProvidesPorts(int &length);
  PortInfo** getUsesPorts(int &length);

  int operator==(const ComponentID& cid);
  int equals(ComponentID *cid);
  int ccatEquals(CCATServicesProxy cid);
  ComponentID* create(char* stringifiedReference);

  // remote methods used by the frameork
  int ccatConnect(const char *usesPortName, Port* providesPort) 
    throw (ConnectionException);

  int ccatDisconnect(const char *usesPortName, Port* providesPort, 
				 double timeout) throw (DisconnectionException);

  int ccatExportAs(ComponentID* idMe, const char *portName, const char *newportname) 
    throw (ExportAsException);

  char* ccatGetPortTypeForPortName(const char *portName);
  int ccatProvideTo(const char *usesPortName, Port *port) throw (ProvideToException);

  Port* ccatGetProvidesPort(const char *providesPortName);
  
  // this method has been added just for testing
  // purposes 
  // int run();
  char *getName();
  int setName(char *);
  char* toMyString() const;
  char* getRef();
  
  // you must use "delete" to delete the ContextID that you get here.
  HPCxx_ContextID* getContextID();

  friend void hpcxx_pack(HPCxx_Buffer& b, CCATServicesProxy* p, int count);
  friend void hpcxx_unpack(HPCxx_Buffer& b, CCATServicesProxy* p, int count);

private:
  CCATServicesProxy* proxyPointer;
  char *ref;
  // will be used when connecting
  // to java components.
  char *getIfaceName (char *fqn);

  // used internally by toMyString
  void charToString (unsigned char c, char* mystr) const;
  // need to figure out the best
  // way to use this.
  int kill();
};

#endif // _CCAT_SERVICES_PROXY_H__







