// CCATServicesSkel.C: Implementation of the ServicesObject.
// This class has private methods that the ComponentID makes
// use of, to provide service to the remote components. Local
// processes make extensive use of this class for port management
// and data flow logic.
// --Madhu

#include "CCATServicesSkel.h"
#include "../connection/ConnectionServiceImpl.h"
#include "../creation/CreationServiceImpl.h"
#include "CCATServicesProxy.h"
#include "../util/HPCxx_String.h"
#include "PortFactoryProvBase.h"
#include "PortFactoryUsesBase.h"
#include "UsesPort.h"
#include "PortSkelUses.h"
#include "ProvidesPort.h"
#include "FrameworkServicesSingletonFactory.h"
#include <hpcxx.h>
#include <iostream>
#include <assert.h>
#include <stdio.h>

#ifdef sgi
extern "C" char *strdup(const char *);
#endif


// these vectors have been declared static in the
// .h file. So a redeclaration is necessary in the
// .C file. The vectors are filled in by calls to
// methods registerUsesPortType(...) and 
// registerProvPortType(...) 
vector<PortFactoryUsesBase* > CCATServicesSkel::usesFactoryList;
vector<PortFactoryProvBase* > CCATServicesSkel::providesFactoryList;


// needed according to the specs, I think.
// Need to verify this. In any case a
// null constructor is useful for the 
// composition engine.
CCATServicesSkel::CCATServicesSkel(CCATServicesProxy* eventCompID_) {
  // As of now this is used so that
  // the component stays alive 
  // till it is explicitly killed 
  // via the killMe() method call
  mywait = new HPCxx_Sync<int>;    
  waitForRunSignal = new HPCxx_Sync<int>;

  // setup the standard event channel
  eventCompID = eventCompID_;

  // The factory maintains single instances of the pesudo components. So, if two
  // cores ask for the creation services, they get the same pointer.
  creationServiceImpl = 
	 FrameworkServicesSingletonFactory::getCreationService ();
  connectionServiceImpl = 
	 FrameworkServicesSingletonFactory::getConnectionService ();
  // connect the various framework services to the event ports.
  if (eventCompID)
	 FrameworkServicesSingletonFactory::makeEventPortConnections (eventCompID);

  debugName = HPCxx_String("DEFAULT");
}

// register only those methods that will be
// invoked remotely.These methods are specific
// to our framework implementation
void CCATServicesSkel::registerAll() {
  hpcxx_registerClass((CCATServicesSkel *)NULL);
  hpcxx_register(&CCATServicesSkel::ccatGetProvidesPort, CCAT_GET_PROVIDES_PORT_ID);
  hpcxx_register(&CCATServicesSkel::ccatGetProvidesPorts, CCAT_GET_PROVIDES_PORTS_ID);
  hpcxx_register(&CCATServicesSkel::ccatExportAs, CCAT_EXPORT_AS_ID);
  hpcxx_register(&CCATServicesSkel::getPortTypeForPortName, CCAT_GET_PORT_TYPE_FOR_PORT_NAME_ID);
  //  hpcxx_register(&CCATServicesSkel::equals, CCAT_EQUALS_ID);
  hpcxx_register(&CCATServicesSkel::ccatProvideTo, CCAT_PROVIDE_TO_ID);
  hpcxx_register(&CCATServicesSkel::ccatGetUsesPorts, CCAT_GET_USES_PORTS_ID);
  hpcxx_register(&CCATServicesSkel::ccatConnect, CCAT_CONNECT_ID);
  hpcxx_register(&CCATServicesSkel::ccatDisconnect, CCAT_DISCONNECT_ID);
  //  hpcxx_register(&CCATServicesSkel::run, CCAT_RUN_ID);
  hpcxx_register(&CCATServicesSkel::setName, CCAT_SET_NAME_ID);
  hpcxx_register(&CCATServicesSkel::getName, CCAT_GET_NAME_ID);
  hpcxx_register(&CCATServicesSkel::getRef, CCAT_GET_REF_ID);
}

// A mapping exists between the UsesPortTypes and
// skel implementations for the type. 
// This mapping is registered by the user by 
// invoking "registerUsesPortType" method.
// The mapping table is a vector named
// usesFactoryList.
// The component developer needs to invoke this method
// in her implementation of setServices(...) method.
// Search the usesPortFactory to check if this type
// has already been registered before. If it has been
// registered before then add it to skelUsesPortsList.
// Note that usesPorts are never sent to other processes
// so no proxies are formed for them.
void CCATServicesSkel::registerUsesPort(PortInfo *portInfo){
  myString typeNameOfUsesPort = portInfo->getType();
  std::cout << "CCATServicesSkel::registerUsesPort: " << portInfo->getName() << ":" << portInfo->getType() << endl;
  printf("\n\n");
  
  for(vector<PortFactoryUsesBase*>::iterator iter=
        CCATServicesSkel::usesFactoryList.begin();
      iter != CCATServicesSkel::usesFactoryList.end(); iter++) {

    std::cout << "CCATServicesSkel:: registerUsesPort iter element --> " << (*iter)->getType() << "  " << endl;
    printf("\n\n");
    // check if the type matches the type of 
    // the registered type
    if ( (*iter)->getType() == typeNameOfUsesPort) {
     
      std::cout << "CCATServicesSkel:: registerUsesPort: Found templated instance for " << typeNameOfUsesPort << "   " << endl << endl;
      printf("\n\n");
      
      // get an instance of the registered type
      // for this uses port
      PortSkelUses *portSkelUses = (*iter)->getPortSkelUses(typeNameOfUsesPort);      
      assert(portSkelUses != NULL);

      // store it in a list so that getPort(...)
      // can find it.
      skelUsesPortsList.push_back(new PortSkelUsesRecord(portSkelUses, new DefaultPortInfo(portInfo->getName(), portInfo->getType())));
      break;
    }
  }
}

// A mapping exists between the ProvidesPortTypes and skel implementations for
// the type.  This mapping is registered by the user by invoking
// "registerProvidesPortType" method.  The mapping table is a vector named
// providesFactoryList.

// the framework maintains the port pointer that implements the methods declared
// in the specific port idl. This executingPort pointer is the one that is
// passed in by the user in addProvidesPort as "inPorvidesPort". For each
// element in the vector check if a proxy Port object can be formed. If it can
// be done then add an elment (PortRecord) to the provPortsList. This list will
// be used in when ccatGetProvidesPort() is called by the ComponenID.  Store the
// portSkel pointer in PortSkelProvRecord and it to the skelProvPortsList to be
// used when getPort() is called by a local process.

void CCATServicesSkel::addProvidesPort(Port* inProvidesPort, PortInfo *portInfo){
  assert(inProvidesPort != NULL);
  myString typeNameOfPort = portInfo->getType();
  std::cout << "CCATServicesSkel::addProvidesPort name: " << portInfo->getName() << " type " << portInfo->getType() << std::endl; 
  cout.flush();
  Port *g;
  for(vector<PortFactoryProvBase*>::iterator iter =
        CCATServicesSkel::providesFactoryList.begin();
      iter != CCATServicesSkel::providesFactoryList.end(); 
      iter++) {
    if ((*iter)->getTypeName() == typeNameOfPort) {
      std::cout << "CCATServicesSkel:: addProvidesPort: Found templated instance for " << typeNameOfPort << "   " << endl << endl;
      printf("\n\n");
      g = (*iter)->makeGPofProvSkel(typeNameOfPort, inProvidesPort);
      provPortsList.push_back(new PortRecord(g, 
					     new DefaultPortInfo(portInfo->getName(),
								 portInfo->getType())));

      PortSkel *skelPort = (*iter)->getPortSkelProv(typeNameOfPort, inProvidesPort);
      skelProvPortsList.push_back(new PortSkelProvRecord(skelPort, 
						      new DefaultPortInfo(portInfo->getName(),
									  portInfo->getType())));
      break;
    }
  }
}

// Each element of the vector of uses port is
// PortUsesSkelRecord and for provides port
// it is PortProvSkelRecord.
// This records are added in registerUsesPort(...) and
// addProvidesPort(...) methods respectively.
// The port pointer being returned for uses port
// is of type PortSkelUses and for provides port
// it is of type PortSkel.
// framework specific services like CreationService
// and ConnectionService are not registered
// FIXME FIXME. getPort() needs to have blocking
// semantics for uses ports. As of now it will always return a
// valid Port pointer back. Blocking semantics 
// can be added later.

Port* CCATServicesSkel::getPort(char *portName){
  myString name((char *) portName);
  myString creationServiceString ("CreationService");
  myString connectionServiceString ("ConnectionService");

  if (name == creationServiceString) {
    return this->creationServiceImpl;
  } else   if (name == connectionServiceString) {
    return this->connectionServiceImpl;
  }

  cout << "CCATServicesSkel getPort - searching for:" << portName << endl << flush;

  vector<PortSkelProvRecord*>::iterator iter;
  for(iter=skelProvPortsList.begin(); iter != skelProvPortsList.end();iter++) {
    std::cout << "CCATServicesSkel::getPort ProvList: iter has name " 
		    << (*iter)->getInfo()->getName() << " " << endl;
    if(iter &&(*iter) && ((*iter)->getInfo()) &&
       ((*iter)->getInfo()->getName() == name)){
      cout << "CCATServicesSkel:: getPort() : found " << portName 
		 << " in providesPorts list \n"; 
      return (Port *)((*iter)->getSkelPort());
    }
  }
  // it's not a provides port, so let's try our luck with uses ports.

  // gets a locked port if non-NULL
  PortSkelUses* myP = getUsesPortNonBlocking (name);
  // implement blocking semantics
  if (myP) {
    // if the port has no non-void methods, return immediately, after
    // incrementing the reference count
    if (!myP->hasAllVoidMethods ())
	 // block until there is a connection established with this port. When you
	 // are awaken, you already have a lock.
	 myP->unlockAndWaitForConnection();
    myP->incrRefCount ();
    myP->unlock();
  }
  else {
    cout << "CCATServicesSkel:: getPort() : Failed to find: " << portName << endl
	    << flush;
    cout << "CCATServicesSkel:: getPort() : I never thought this will be printed !! " << endl;
    // need to figure out later
    // what is a good default value
    // to return 
    // return (Port *)((*iter)->getSkelPort());
  }
  return myP;
}

// It needs to take a reference to an int.
// This way the int can be filled up with the
// the length of the array. Or else the user
// has no way of knowing how to iterate through
// the returned list.

PortInfo** CCATServicesSkel::getProvidesPorts(int &length){ 
  // length is passed in as a reference,
  // so fill it up.
  length  = provPortsList.size();
  PortInfo** ar = new PortInfo*[length+1];
  for(int j = 0; j < length+1; j++) ar[j] = NULL;
  int i = 0;
  for(vector<PortRecord*>::iterator iter=provPortsList.begin();
      iter != provPortsList.end();iter++, i++)
    ar[i] =(PortInfo *)((*iter)->getInfo());
  PortInfoArray a(ar, length);
  return a.getArray();
}

// It needs to take a reference to an int.
// This way the int can be filled up with the
// the length of the array. Or else the user
// has no way of knowing how to iterate through
// the returned list.
PortInfo** CCATServicesSkel::getUsesPorts(int &length){ 
  // length is passed in as a reference,
  // so fill it up.
  length = usesPortsList.size();
  PortInfo** ar = new PortInfo*[length+1];
  for(int j = 0; j < length+1; j++) ar[j] = NULL;
  int i = 0;
  for(vector<PortRecord*>::iterator iter=usesPortsList.begin();
      iter != usesPortsList.end();iter++, i++)
    ar[i] = (PortInfo *)((*iter)->getInfo());
  PortInfoArray a(ar, length);
  return a.getArray();
}


// decrements the reference count for the port if it's a uses port. Doesn't do
// anything currently if it's a provides port. Does anything need to be done if
// it's a provides port?? Do we need to maintain a reference count for provides
// ports??
void CCATServicesSkel::releasePort(char *name) {
  // See if it's a uses port. If yes then this call will get us a locked uses
  // port.
  PortSkelUses* myP = getUsesPortNonBlocking (myString(name));
  if (myP) {
    myP->decrRefCount();
    myP->unlock();
  }
}


// I don't think we will be using this 
// method for conections. This functionality
// seems useless as we are into a more 
// sophisticated events model !!!
void CCATServicesSkel::addConnectionListener(ConnectionListener *l) {
}

// ComponentID is a proxy object. It moves
// around in different processes. However,
// it needs to point to this implementation
// class of ther services interface. So,
// the constructor for CCATServicesProxy
// stores a global pointer to this class.
// As a result we pass "this" to its 
// constructor here.
ComponentID* CCATServicesSkel::getComponentID() {
  return new CCATServicesProxy(this);
//   proxyPointer = new CCATServicesProxy(this);
//   proxyStartpoint = proxyPointer.getProxyStartpoint();
//   return proxyPointer;
}


// When this method is called on the ComponentID,
// The proxy (CCATServicesProxy) minvokes to this call.
// This method is called on the ComponentID by the
// ConnectionServices.
PortInfoArray CCATServicesSkel::ccatGetProvidesPorts(){ 
  // length is passed in as a reference,
  // so fill it up.
  cout << "CCATServicesSkel::ccatGetProvidesPorts() called " << endl << endl;
  int length  = provPortsList.size();
  PortInfo** ar = new PortInfo*[length+1];
  for(int j = 0; j < length+1; j++) ar[j] = NULL;
  int i = 0;
  for(vector<PortRecord*>::iterator iter=provPortsList.begin();
      iter != provPortsList.end();iter++, i++)
    ar[i] =(PortInfo *)((*iter)->getInfo());
  PortInfoArray a(ar, length);
  cout << "CCATServicesSkel::ccatGetProvidesPorts() returning  " << endl << endl;
  return a;
}

// Method to be called remotely for adding a provides port in an "exportAs"
// mode. This method will add an entry in the provPortsList so that any remote
// calls on ccatGetProvidesPort will return this Port as a return value. But
// local getPort calls will not return anything, as no entry is made in the
// skelProvPortsList. This is because the exported port lives only "virtually"
// in the core object, and there is no real port object associated with this
// provides port in this core. Also, in order to export a port, you must have
// registered its port type with the providesPortFactory.
int CCATServicesSkel::ccatExportAs (Port p, DefaultPortInfo info) {
  cout << "CCATServicesSkel::ccatExportAs: name " << info.getName().c_str() 
	  << " type: " << info.getType().c_str() << endl << flush;

  myString typeNameOfPort = info.getType();

  for(vector<PortFactoryProvBase*>::iterator iter = 
	   CCATServicesSkel::providesFactoryList.begin();
      iter != CCATServicesSkel::providesFactoryList.end(); 
	 iter++) {

    if ((*iter)->getTypeName() == typeNameOfPort) {
      std::cout << "CCATServicesSkel:: ccatExportAs: Found templated instance for " 
			 << typeNameOfPort << "   " << endl << flush;

	 Port *g = new Port (p);
      provPortsList.push_back(new PortRecord(g, new DefaultPortInfo(info)));

	 return 0;
    }
  }
  
  std::cout << "CCATServicesSkel:: ccatExportAs: Error, found no matching type\n" 
		  << flush;
  return -1;
}

// needed by ccatExportAs method in CCATServiceProxy.
// we can avoid this call if ccatExportAs above just
// took a portName not portInfo. I think it is crazy
// that we make an extra remote call just becuase we 
// need to get the type associated with the name.
// If you find a sane reason to justify this, let me know.

// search just the list of provides port
// as only provides ports are exported
HPCxx_String CCATServicesSkel::getPortTypeForPortName(HPCxx_String portName) {

  // convert portName to type std::string 
  myString portNameString((char *)portName);
  cout << "CCATServicesSkel:: getPortTypeForPortName() : received portName " << portName << endl;
  const char *typeOfPort = NULL;
  for(vector<PortRecord*>::iterator iter = 
	CCATServicesSkel::provPortsList.begin();
      iter != CCATServicesSkel::provPortsList.end(); 
      iter++) {
    if (((*iter)->getInfo())->getName() == portNameString) {
      typeOfPort = strdup((((*iter)->getInfo())->getType()).c_str());
      std::cerr << "CCATServicesSkel:: getPortTypeForPortName: returning type " << typeOfPort << endl;
      break;
    }
  }
  assert(typeOfPort != NULL);
  return HPCxx_String(const_cast<char *>(typeOfPort));
}

int CCATServicesSkel::ccatProvideTo (HPCxx_String usesPortName, Port p) {
  cout << "CCATServicesSkel::provideTo: name " << usesPortName  
	  << endl << flush;

  // gets a locked port if non-NULL
  PortSkelUses* usesPort = 
    implementConnectionBlockingSemantics( myString((char *)usesPortName) );
  if (usesPort == NULL) {
    // should really throw an exception here
    cout << "CCATServicesSkel::ccatProvideTo failed\n" << flush;
    return -1;
  }
  // all well, add the provides port.
  // change this call to addListener 
  // so that it is compatible with the
  // java version and it doesn't cause
  // mixup with a method in this class
  usesPort->addProvidesPort (p);
  usesPort->unlock();
    
  cout << "CCATServicesSkel::provideTo: done with adding provides port\n" 
	  << flush;
  return 0;
}

// When this method is called on the ComponentID,
// The proxy (CCATServicesProxy) minvokes this call
PortInfoArray CCATServicesSkel::ccatGetUsesPorts(){ 
  // length is passed in as a reference,
  // so fill it up.
  int length  = usesPortsList.size();
  cout << "CCATServicesSkel::ccatGetUsesPorts() called, length: " << length << endl;    
  PortInfo** ar = new PortInfo*[length+1];
  for(int j = 0; j < length+1; j++) ar[j] = NULL;
  int i = 0;
  for(vector<PortRecord*>::iterator iter=usesPortsList.begin();
      iter != usesPortsList.end();iter++, i++)
    ar[i] = (PortInfo *)((*iter)->getInfo());
  PortInfoArray a(ar, length);
  cout << "CCATServicesSkel::ccatGetUsesPorts() returning  " << endl << endl;
  return a;
}



// This method will be invoked remotely on this
// skeleton by the ComponentID. 
// Note that any method that is invoked remotely
// on this skeleton will return an object and not a
// pointer. This is because HPC++ allows only pass
// by value for remote method invocations and secondly
// these objects really are objectPointers. They are
// objects that wrap a globalPointer.
Port CCATServicesSkel::ccatGetProvidesPort(HPCxx_String portName){
  myString name((char *) portName);
  cout << "in CCATServicesSkel ccaGetProvidesPort - searching for:" << portName << endl;
  cout.flush();
  vector<PortRecord*>::iterator iter;
  for(iter=provPortsList.begin(); iter != provPortsList.end();iter++)
    if(iter &&(*iter) && ((*iter)->getInfo()) &&
       ((*iter)->getInfo()->getName() == name)){
      cout << "CCATServicesSkel:: ccaGetProvidesPort() : found " << portName << " in providesPorts list \n"; 
      return *((*iter)->getPort());
    }
  // Later on this method may be modified to 
  // ccatGetPort(...) that allows both provides
  // and uses ports to be sent remotely. As of now
  // our design doesn't require this. The specs anyway
  // don't talk about this kind of framework specific
  // implementation details.
  //   for(iter=usesPortsList.begin(); iter != usesPortsList.end();iter++)
  //     if(iter &&(*iter) && ((*iter)->getInfo()) &&
  //        ((*iter)->getInfo()->getName() == name)){
  //       cout << "CCATServicesSkel:: ccaGetPort() : found " << portName << "in UsesPorts list \n"; 
  //       cout.flush();
  //       return *((*iter)->getPort());
  //     }
  cout << "CCATServicesSkel::ccatGetProvidesPort(): Failed to find: " << portName << endl;
  cout << "CCATServicesSkel:: ccatGetProvidesPort(): I never thought this will be printed !! " << endl;
  cout.flush();
  // should never reach here
  // need to figure out later
  // what is a good default value
  // to return 
  return *((*iter)->getPort());
}

// Called by the ConnectionService on the
// componentID.
int CCATServicesSkel::ccatConnect(HPCxx_String usesPortName, Port providesPortIn) {
  cout << "CCATServicesSkel::ccatConnect " << usesPortName << "  with  a ProvidesPort" 
	  << endl << flush;
  cout << "CCATServicesSkel::ccatConnect calling getPort() " << endl << flush;

  // gets a locked port if non-NULL
  PortSkelUses* portSkelUses = 
    implementConnectionBlockingSemantics (myString((char *)usesPortName));
  if (portSkelUses == NULL) {
    cout << "CCATServicesSkel::ccatConnect failed\n" << flush;
    return -1;
  }
  cout << "CCATServicesSkel::ccatConnect:getPort() succeeded " << endl << flush;
  cout << "CCATServicesSkel::ccatConnect calling addProvidesPort() " << endl << flush;
  portSkelUses->addProvidesPort(providesPortIn);
  portSkelUses->unlock();
  
  cout << "CCATServicesSkel::ccatConnect calling addProvidesPort() done " 
	  << endl << flush;
  return 0;
}

// Called by the ConnectionService on the
// componentID.
int CCATServicesSkel::ccatDisconnect(HPCxx_String usesPortName, Port providesPortIn, 
							  double timeout) {
  cout << "CCATServicesSkel::ccatDisconnect calling removeProvidesPort() " 
	  << endl << flush;

  // gets a locked port if non-NULL
  PortSkelUses *portSkelUses = 
    implementConnectionBlockingSemantics (myString((char *)usesPortName));
  if (portSkelUses == NULL) {
    cout << "CCATServicesSkel::ccatDisconnect failed\n" << flush;
    return -1;
  }
  portSkelUses->removeProvidesPort(providesPortIn);
  portSkelUses->unlock();
  
  cout << "CCATServicesSkel::ccatDisconnect calling removeProvidesPort() done " 
	  << endl << flush;
  return 0;
}

// int CCATServicesSkel::equals(ComponentID cid) {
//   // if any of the ComponentID's are uninitialized, the result is an immediate
//   // false
//   if (!proxyStartpoint || !((CCATServicesProxy *)cid)->skp) {
//     return 0;  
//   }
//   return (*proxyStartpoint == *(((CCATServicesProxy *)cid)->skp));
// }

int CCATServicesSkel::setName(HPCxx_String name_) {
  cout << "CCATServicesSkel:: setName() called: with " << name_ << endl;
  cout.flush();
  debugName = name_;
  return 0;
}

HPCxx_String CCATServicesSkel::getName() {
  cout << "CCATServicesSkel:: getName() called: returning " << debugName << endl;
  cout.flush();
  return debugName;
}

HPCxx_String CCATServicesSkel::getRef() {
  cout << "CCATServicesSkel::getRef called\n" << flush;
  CCATServicesProxy *myProxy = new CCATServicesProxy(this);
  char *ref = myProxy->getRef();
  cout << "CCATServicesSkel::getRef: ref: " << ref << endl << flush;
  return HPCxx_String (ref);
}

void CCATServicesSkel::startRunning() {
  int shouldIStartRunning = *waitForRunSignal;
}
int CCATServicesSkel::run() {
  int startRunning = 1;
  *waitForRunSignal = startRunning;
  return startRunning;
}

void CCATServicesSkel::waitForKill(){
  int i = *mywait;
}

// may have to do more
// possibly cleanup stuff
// FIXME FIXME
int CCATServicesSkel::kill(){
  int dummy = 1;
  cout << "doing  kill " << endl;
  *mywait = dummy;
  return dummy;
}

// returns a locked uses port if possible, else NULL
PortSkelUses* 
CCATServicesSkel::implementConnectionBlockingSemantics (myString name) {
  // gets a locked port
  PortSkelUses* usesPort = getUsesPortNonBlocking(name);
  if (usesPort == NULL) {
    // no matching uses port find, cannot do the operation.
    return NULL;
  }
  // implement the blocking semantics
  if ( !usesPort->hasAllVoidMethods() && (usesPort->getRefCount() > 0) ) {
    // This means that the port has some non-void methods requiring blocking
    // semantics and that the port is blocked because of a getPort method and
    // any modifications to the port (such as connections or disconnections are
    // not allowed.
    cout << "CCATServicesSkel::implementConnectionBlocking: port is blocked!!!" 
	    << ", cannot do operation\n" 
	    << flush;
    usesPort->unlock();
    return NULL;
  }
  return usesPort;
}

// returns a locked uses port if found, else returns NULL
PortSkelUses* CCATServicesSkel::getUsesPortNonBlocking (myString name) {
  // find the uses portin the uses port vector.
  vector<PortSkelUsesRecord*>::iterator iter;
  for(iter=skelUsesPortsList.begin(); iter != skelUsesPortsList.end(); iter++) {
    std::cout << "CCATServicesSkel::getUsesPortNonBlocking : UsesList iter has name " 
		    << (*iter)->getInfo()->getName() << " " << endl << flush;

    if (iter &&(*iter) && ((*iter)->getInfo()) &&
	   ((*iter)->getInfo()->getName() == name)) {

      assert((*iter)->getPortSkelUses() != NULL);

	 (*iter)->getPortSkelUses()->lock();  // lock before returning

      cout << "CCATServicesSkel:: getUsesPortNonBlocking: uses port found " 
		 << name.c_str() << " in UsesPorts list \n" << flush;
	 return (*iter)->getPortSkelUses();
    }
  }
  return NULL;
}

// this is the skeleton class.
// It never moves from the imlementing process.
// So we provide only empty packs and unpacks.
// These methods will never be called.
void hpcxx_pack(HPCxx_Buffer& b, CCATServicesSkel* p, int count){}
void hpcxx_unpack(HPCxx_Buffer& b, CCATServicesSkel* p, int count){}









