// -*-c++-*-
// This class implements the services interface.
// This is a skeleton class. So it has the implementation
// for all the methods. It doesn't move around, so 
// should have empty pack and unpack methods.
// --Madhu

#ifndef _CCAT_SERVICES_SKELETON_H__
#define _CCAT_SERVICES_SKELETON_H__

#include <iostream>
#include <string>
#include "../util/MyHPCxx_GlobalPtr.h"
#include "Port.h"
#include "ProvidesPort.h"
#include "UsesPort.h"
#include "PortInfo.h"
#include "DefaultPortInfo.h"
#include "PortFactoryProvBase.h"
#include "PortFactoryUsesBase.h"
#include "PortRecord.h"
#include "PortSkelProvRecord.h"
#include "PortSkelUsesRecord.h"	
#include "PortInfoArray.h"
#include "Services.h"
#include "../creation/CreationServiceImpl.h"
#include "../connection/ConnectionServiceImpl.h"
#include "STL.h"
#include "FRAMEWORK_REGISTRATION_ID.h"
#include "../connection/ConnectionServiceImpl.h"

class CCATServicesProxy;

class CCATServicesSkel : public Services {

  // provPortsList, usesPortsList
  // skelProvPortsList, skelUsesPortsList
  // Elements are added to these vectors after checking
  // that the types mentioned by the component user
  // have been previously registerd. The mapping
  // is verified by iterating through 
  // providesFactoryList and usesFactoryList.
private:
  // to be used with remote calls
  // elements of these lists are proxies
  // to the actual ports. In HPC++ this can
  // be achived by wrapping the globalPointer
  // to the implementation class in a PortObject
  // and returning it to the user.
  vector<PortRecord*> provPortsList;
  vector<PortRecord*> usesPortsList;

  // to be used with local calls
  vector<PortSkelProvRecord*> skelProvPortsList;
  vector<PortSkelUsesRecord*> skelUsesPortsList;


  // services that the framework provides
  // to the users. InfoDirectory and Registry 
  // Service will be added soon.
  CreationServiceImpl *creationServiceImpl;
  ConnectionServiceImpl *connectionServiceImpl;

  // Although the standard event channel component is NOT a pseudo component,
  // its ComponentID is given as a standard framework service for each
  // component.
  CCATServicesProxy* eventCompID;

  // Needed for synchronization in the 
  // execution model
  HPCxx_Sync<int> *mywait;
  HPCxx_Sync<int> *waitForRunSignal;
  HPCxx_String debugName;
public:
  // list of reigstered types for ports.
  // Formed by the component writer using
  // registerUsesPortType(...) and
  // registerProvPortType(...) method calls.
  // These vectors have to be declared public
  // as they need to be accessed by above 
  // mentioned method calls in RegisterType.h
  static vector<PortFactoryUsesBase* > usesFactoryList; 
  static vector<PortFactoryProvBase* > providesFactoryList; 


  static int registrationID() { return CCAT_SERVICES_SKEL_ID; }

  // Register all of the server related functions.
  // that will be invoked remotely through the 
  // ComponentID.
  static void registerAll(); 

  // CCA1.0 spec requires that a component writer
  // have access to an implementation of the services
  // object with an empty constructor. Well, I think
  // I read that somewhere. Will verify this.
  // FIXME FIXME
  CCATServicesSkel(CCATServicesProxy* eventCompID_ = NULL);

  // methods that are part of Services interface
  PortInfo** getProvidesPorts(int &length);
  PortInfo** getUsesPorts(int &length);
  Port* getPort(char *name);
  void registerUsesPort(PortInfo* nameAndType);
  void addProvidesPort(Port *inPort, PortInfo *name);
  void addConnectionListener(ConnectionListener *l); 
  void releasePort(char *name);
  ComponentID* getComponentID(); 

  // methods that are called on ComponentID
  // by the other services. This ensures that
  // the ComponentID has a wider view of the 
  // the ServicesObject implementation as 
  // compared to the component writers and users.
  PortInfoArray ccatGetProvidesPorts();
  PortInfoArray ccatGetUsesPorts();
  Port ccatGetProvidesPort(HPCxx_String portName);
  int ccatExportAs (Port p, DefaultPortInfo info);
  HPCxx_String getPortTypeForPortName(HPCxx_String portName);
  int ccatProvideTo (HPCxx_String usesPortName, Port p);
  int ccatConnect(HPCxx_String, Port providesPort);
  int ccatDisconnect(HPCxx_String, Port providesPort, double);

  int setName(HPCxx_String);
  HPCxx_String getName();
  HPCxx_String getRef();
    
  // internal methods used for synchronization
  // as part of the execution model
  int run();
  int kill();
  void waitForKill(); 
  void startRunning();

private:
  // returns a locked uses port if possible, else NULL. Implements the checks
  // required by connect and disconnect calls
  PortSkelUses* implementConnectionBlockingSemantics (myString name);
  
  // non bockign call to get a uses port, used internally. returns a locked port 
  // if returning non-NULL.
  PortSkelUses* getUsesPortNonBlocking (myString name);
  
  // methods needed for marshalling and 
  // unmarshalling this object into the stream
  // However, since skeleton implementations don't
  // move across processes, these methods can/should
  // have empty implmentation methods.
  friend void hpcxx_pack(HPCxx_Buffer& b, CCATServicesSkel* p, int count);
  friend void hpcxx_unpack(HPCxx_Buffer& b, CCATServicesSkel* p, int count);
};

#endif // _CCAT_SERVICES_SKEL_H__














