// -*-c++-*-

// ComponentID.h: This interface is part
// of the specs. The private methods will
// be used by the pseudo components to
// implement some of the ConnectionService
// methods.
// --Madhu

#ifndef _COMPONENT_ID_H__
#define _COMPONENT_ID_H__

#include "../exceptions/ConnectionException.h"
#include "../exceptions/DisconnectionException.h"
#include "../exceptions/ExportAsException.h"
#include "../exceptions/ProvideToException.h"
#include "PortInfo.h"
#include "ProvidesPort.h"
#include "UsesPort.h"
#include "STL.h"

class ComponentID {
  friend class ConnectionServiceImpl;

public:

  // virtual destructor so that a user can just call "delete ComponentID"
  // without about the actual type of the ComponentID.
  virtual ~ComponentID() = 0;

  virtual PortInfo** getProvidesPorts(int& length) = 0;
  virtual PortInfo** getUsesPorts(int& length)= 0;
    
  virtual int operator==(const ComponentID& cid) = 0;

  virtual ComponentID* create(char* stringifiedReference) = 0;

  // Returns a string representing a given ComponentID. This string can again be
  // passed to the create call to get a ComponentID back. The string is
  // guaranteed to be unique for different ComponentIDs that point to different
  // components. If two componentIDs point to the same component, they will be
  // the same.
  virtual char* toMyString() const = 0;
  
  // Basically same as operator ==, however this method needs
  // to be called remotely, from java and as a result I have
  // introduced this method here.
  virtual int equals(ComponentID *cid) = 0;

  // I know this method is non-CCA.
  // This is just to make life easy
  // for testing purposes.
  // virtual int run() = 0;
  virtual char* getName() = 0;
  virtual char* getRef() = 0;
  virtual int setName(char *) = 0;
  // you must use "delete" to delete the ContextID that you get here.
  virtual HPCxx_ContextID* getContextID() = 0;

private:
  virtual int ccatConnect(const char *usesPortName, Port* providesPort) 
    throw (ConnectionException) = 0;
  
  virtual int ccatDisconnect(const char *usesPortName, Port* providesPort, 
					    double timeout) throw (DisconnectionException) = 0;
  
  virtual int ccatExportAs(ComponentID *idSub, const char *portName, 
					  const char *newportname) throw (ExportAsException) = 0;
  
  virtual int ccatProvideTo(const char *usesPortName, Port *port) 
    throw (ProvideToException) = 0;

  virtual Port* ccatGetProvidesPort(const char *providesPortName) = 0;
};

inline
ComponentID::~ComponentID() {}


#endif // _COMPONENT_ID_H__
