// -*-c++-*-

// PortFactoryProv.h: Used to create port proxy
// objects for provides ports. It even creates
// PortSkel wrappers around user supplied pointers
// to provides port implementations.
// --Madhu

#ifndef _PORT_FACTORY_H__
#define _PORT_FACTORY_H__

#include "Port.h"
#include "STL.h"
#include "PortFactoryProvBase.h"

template < class Tskel >
class PortFactoryProv: public PortFactoryProvBase{
public:
  // with this constructor call, each
  // templated instance of the provides port 
  // skel has a specific typeName in string fromat
  // associated with it:
  // called by registerProvidesPortType(...)
  PortFactoryProv(myString _typeName): PortFactoryProvBase(_typeName) {}
  // since this is a templated class
  // the constructor must have been called
  // with a specific typeName in registerUsesPortType(...).
  // The typeName is stored in the base class.
  // makeGPofProvSkel(...) is called by 
  // CCATServicesSkel::addProvidesPort(...)
  Port* makeGPofProvSkel(myString typeName_, Port* p){
    if(typeName_ == typeName){
      std::cout << "PortFactoryProv********************************************************************::makeGPofProvSkel:: " << typeName_ << " found!" << endl;
      PortSkel * sk = new Tskel(p);
      // Make a GlobalPointer to the Port skeleton
      // and and save it in the Port object 
      // return a pointer to the Port object.
      Port *g = new Port(sk);

      // eg: converting typeName_ = SparseLinearSystem_idl
      // to idl.sparseLinearSystem.ProvidesSparseLinearSystemSkel
      // potential memory leaks in the code here.
      // FIXME FIXME 
      char *s;
      char *buf = new char[100];

	  // Dup string.
	  s = (char *) malloc(strlen(typeName_.c_str()) + 1);
	  strcpy(s, typeName_.c_str());

      // find the last occurrence of _ (_idl really)
      *strrchr(s, '_') = '\0';
      sprintf(buf, "idl.%s.Provides%sSkel",s, s);
      buf[4] += 32;
      //g->skp->setFullyQualifiedFileName (HPCxx_String (_typeName.c_str()));
      cout << "PortFactoryProv::setting FQN : " << buf << endl << flush;
      cerr << "PortFactoryProv::setting FQN : " << buf << endl << flush;
      cerr << "PortFactoryProv::setting FQN : " << HPCxx_String(buf) << endl << flush;
      g->skp->setFullyQualifiedFileName (HPCxx_String (buf));
      return g;
    } else {
      return NULL;
    }
  }

  // used in a getPort(...) call. Return the skel pointer
  // passed on by the user with RegisterUsesPortType(...) call
  // When the userdoes a getPort(...), then the call is definitely
  // local and so return the skel pointer back to her. We don't use a proxy
  // concept in local cases to avoid inefficiency caused due to 
  // an extra level of indirection when invoked through a proxy.
  PortSkel* getPortSkelProv(myString _typeName, Port* p) {
    if (_typeName == typeName) {
      PortSkel *sk = new Tskel(p);
      return sk;
    } else {
      return NULL;
    }
  }

};

#endif // _PORT_FACTORY_PROV_H__

