
// PortInfoArray.C : When the user invokes
// getProvidesPorts() and getUsesPorts() on 
// a componentID, then the skeleton returns
// an object of type PortInfoArray to the 
// componentID class
// --Madhu
#include "PortInfoArray.h"
#include "DefaultPortInfo.h"
#include "assert.h"

PortInfoArray::PortInfoArray(PortInfo **a, int len_){
  len = len_;
  info = new DefaultPortInfo[len];
  for(int i = 0; i < len; i++)
    info[i] = *(a[i]);
}

// create on the heap and allow the
// user to manage memory associated with
// it. The specs clearly mention that the
// caller is expected to clean up that memory
PortInfo ** PortInfoArray::getArray(){
  PortInfo ** a = new PortInfo*[len];
  for(int i = 0; i < len; i++){
    a[i] = (PortInfo *)(new DefaultPortInfo(info[i].getName(), info[i].getType()));
  }
  return a;
}

int PortInfoArray::getLength(){ return len; }

void hpcxx_pack(HPCxx_Buffer& b, PortInfoArray* p, int count){
  for(int i = 0; i < count; i++){
    // marshal length first
    int length = p[i].len;
    b.pack (&length, 1);
    
    // now marshal the array elements
    for (int j = 0; j < length; j++)
	 hpcxx_pack (b, (DefaultPortInfo *)&(p[i].info[j]), 1);
  }
}
void hpcxx_unpack(HPCxx_Buffer& b, PortInfoArray* p, int count){
  for(int i = 0; i < count; i++){
    // unmarshal length first
    int length;
    b.unpack (&length, 1);
    p[i].len = length;
    if (p[i].info) delete [] p[i].info;
    p[i].info = new DefaultPortInfo[length];
    // now unmarshal the array elements
    for (int j = 0; j < length; j++)
	 hpcxx_unpack (b, (DefaultPortInfo *)&(p[i].info[j]), 1);
  }
}
