// -*-c++-*-
#ifndef __COMPONENT_FACTORY_H
#define __COMPONENT_FACTORY_H

#include "../ccacore/ComponentID.h"
#include "../ccacore/CCATServicesSkel.h"

class ComponentRecordBase {
protected:
  myString name;
public:
  ComponentRecordBase (myString name_) { name = name_; }
  virtual ComponentID* create (myString name_) { return NULL; }
};

template <class ComponentType, class ComponentHomeType>
class ComponentRecord: public ComponentRecordBase {
public:
  ComponentRecord (myString name_): ComponentRecordBase (name_) {}

  // Create a component, a core and set the core, and then return the
  // ComponentID for this core.
  ComponentID* create(myString name_) {
    cout << "looking for: " << name_.c_str() << endl << flush;
    cout << "A name in the factory: " << name.c_str() << endl << flush;
    if (name_ == name) {
      cout << "Component " << name.c_str() << " found!" << endl << flush;
	 ComponentType* comp = ComponentHomeType::instantiate();
	 CCATServicesSkel* core = new CCATServicesSkel();
	 comp->setServices (core);
	 
	 return core->getComponentID();
    }
    else
      return NULL;
  }
};

class ComponentFactory {
public:
  static vector <ComponentRecordBase *> components;

  static ComponentID* create (myString& componentName) {

    ComponentID *cid = NULL;
    vector<ComponentRecordBase*>::iterator iterStart = components.begin();
    vector<ComponentRecordBase*>::iterator iterEnd = components.end();
    
    for(vector <ComponentRecordBase*>::iterator iter = iterStart; 
	   iter != iterEnd; 
	   iter++) {
	 
	 if((cid = (*iter)->create(componentName)) != NULL)
	   break;
    }
    return cid;
  }
};

template <class ComponentType, class ComponentHomeType>
void registerComponentType(myString& name, ComponentType* cType, 
					  ComponentHomeType* cHomeType);

#include "ComponentFactory.cc"

#endif // __COMPONENT_FACTORY_H
