
// CreationServiceImpl.C: imlements the functionality
// mentioned in the CreationServices. 
// --Madhu

#include "CreationServiceImpl.h"
#include "ComponentFactory.h"
#include <globus_toolkit.h>
#include "../exceptions/InstantiateException.h"
#include "../ccacore/CCATServicesProxy.h"
#include "../ccacore/FRAMEWORK_REGISTRATION_ID.h"
#include "../util/EnvObj.h"
#include <string>
#include <iostream>
#include <assert.h>

extern "C" char *strdup(const char *);



#pragma instantiate void hpcxx_pack(HPCxx_Buffer&,HPCxx_GlobalPtr<HPCxx_Sync<CCATServicesProxy> >*,int)

// dummy global method for return ComponentID(...)
// This method will never be called
// Needed so that we can make the coresponding
// call on the components
// XXX
static CCATServicesProxy returnComponentID(){
  assert(false);
 //  CCATServicesProxy notUsedCompID;
  CCATServicesProxy notUsedCompID((CCATServicesSkel *)NULL);
  return notUsedCompID;
}


CreationServiceImpl::CreationServiceImpl() {
  // empty constructor as it has no state
  hpcxx_register(returnComponentID, RETURN_COMPONENT_ID);
}

#ifdef ORIG
ComponentID* CreationServiceImpl::createInstance(char* execHost, char *execPath) throw (InstantiateException) {
  // need to recieve an env object
  // so that depending on the protocol
  // we can call a specific method
  return createGramInstance(execHost, execPath);
}
#else  // ORIG
ComponentID* CreationServiceImpl::createInstance(char* url, EnvObj* env) 
throw (InstantiateException) {
  
  std::string myUrl(url);
  // get the different sections of the url
  myString mechanism, targetHost, componentName;
  parseUrl (myUrl, mechanism, targetHost, componentName);
  cout << "url: " << myUrl.c_str() << endl
       << "mechanism: " << mechanism.c_str() << endl
       << "targetHost: " << targetHost.c_str() << endl 
       << "componentName: " << componentName.c_str() << endl;
  
  if (mechanism == "gram") {
    // get executable path from the env object
    char *path = env->get ("executable");
    if (!path) {
      cout << "Error: full path name for the executable on the target host is required\n"
	   << "for globus gram based instantiation\n" << flush;
      return NULL;
    }
    return createGramInstance(targetHost.c_str(), path);    
  }
  else if (mechanism == "rsh") {
    cout << "Error: unknown creation mechanism: " << mechanism.c_str() << endl << flush;
    return NULL;
  }
  else if (mechanism == "ssh") {
    cout << "Error: unknown creation mechanism: " << mechanism.c_str() << endl << flush;
    return NULL;
  }
  else if (mechanism == "same-host") {
    cout << "Error: unknown creation mechanism: " << mechanism.c_str() << endl << flush;
    return NULL;
  }
  else if (mechanism == "in-process") {
    return createInProcessInstance(componentName);
  }
  else {
    cout << "Error: unknown creation mechanism: " << mechanism.c_str() << endl << flush;
    return NULL;
  }
}
#endif // ORIG

void CreationServiceImpl::parseUrl (myString& url, myString& mechanism, 
				    myString& targetHost, myString& componentName) {
  // get the instantiation mechanism
  int offset = url.find (':');
  mechanism = url.substr (0, offset);

  // now the target host  name
  int offset1 = url.find ('/');             // first '/'
  int offset2 = url.find ('/', offset1+1);  // second '/'
  targetHost = url.substr(offset1+1, offset2-offset1-1);

  componentName = url.substr(offset2+1);
}

// not implemented FIXME FIXME
void CreationServiceImpl::deleteInstance(ComponentID* id) throw (InstantiateException) {
}

ComponentID* CreationServiceImpl::createInProcessInstance(myString &componentName) {
  // Just testing
  Event *ev = new Event("Ken&Madhu", "debugger", "MasterKey", 0, "CreationEvent");
  int result = eventUser->notify(ev);
  cout << "Event published to the event channel.\n" << flush;
  return ComponentFactory::create (componentName);
}

ComponentID* CreationServiceImpl::createGramInstance(const char *host, const char *path) {
  char *gateKeeperMachine = strdup(host);
  char *executableMachine = strdup(host);
  
  // Initialize GlobusToolkit.
  // This must be done only once in the context.
  
  int errorval = gs_init();
  if(errorval) {
    cout << "CreationServiceImpl<gram> :: gs_init() failed with return value " << errorval << endl;
    cout << ".. Exiting ...  \n";
    exit(EXIT_FAILURE);
  }

  // make contact with the gatekeeper
  // what about checking the return value ??
  // FIXME FIXME
  hpcxx_connectContext(gateKeeperMachine);
  
  // Create a context on the target machine,
  // and get its contextID back.

  HPCxx_ContextID *myremoteCID = 
    hpcxx_createContext((char *)executableMachine, (char *)path);
  if (!myremoteCID)  {
    cout << "NULL remoteCID  received by CreationServiceImpl<gram>\n";
    cout << "Please take a look at the following file on bread to find a description for the error # " << endl;
    cout << "File Name: /l/extreme/globus/development/mips-sgi-irix6.5-64_pthreads_standard_debbug/include/globus_gram_client.h " << endl;
    cout << "... Exiting ... \n";
    exit(EXIT_FAILURE);
  }

  // Is this remoteComponentID a memory leak ?
  // FIXME FIXME 
  //CCATServicesProxy *remoteComponentID = new CCATServicesProxy();
  CCATServicesProxy *remoteComponentID = new CCATServicesProxy((CCATServicesSkel *)NULL);
  cout << "calling getComonentID on client " << executableMachine << endl;
  cout.flush();
  hpcxx_invoke(*myremoteCID, *remoteComponentID, RETURN_COMPONENT_ID);
  return remoteComponentID;
}


void CreationServiceImpl::addEventUser(UsesEventPortSkel* eventUser_) {
  eventUser = eventUser_;
}

// FIXME FIXME
// these instantiation mechanisms need to be 
// implemented 
// ComponentID* CreationServiceImpl::createRSHInstance(char *host, char *path) {}
// ComponentID* CreationServiceImpl::createSSHInstance(char *host, char *path) {}
// ComponentID* CreationServiceImpl::createExecInstance(char *host, char *path) {}








