
#include "Event.h"
#include "../../util/HPCxx_String.h"

// I think HPC++ requires an empty
// constructor for every objec that
// will be marshalled into the stream.
// template/hpcxx_register.cc has a statement
// A1 a1; This calls the empty constructor.
// Need to verify this.

Event::Event() {
  source = myString("DEFAULT");
  type = myString("DEFAULT");
  key = myString("DEFAULT");
  details = myString("DEFAULT");
  seqNum = 0;
}
Event::Event(myString _source, myString _type, myString _key, long _seqNum, myString _details) {
  source = _source;
  type = _type;
  key = _key;
  seqNum = _seqNum;
  details = _details;
}

// This copy constructor should
// use pass by reference.
// This will be called from 
// ProvidesEventPortSkel.C
Event::Event(Event &ev) {
  source = ev.source;
  type = ev.type;
  key = ev.key;
  seqNum = ev.seqNum;
  details = ev.details;
}

myString Event::getSource() const {
  return source;
}

void Event::setSource(myString _source) {
  source = _source;
}

myString Event::getType() const {
  return type;
}

void Event:: setType(myString _type) {
  type = _type;
}

myString Event::getKey() const {
  return key;
}

void Event::setKey(myString _key) {
  key = _key;
  
}

long Event:: getSequenceNum() const {
  return seqNum;
}

void Event:: setSequenceNum(long _seqNum) {
  seqNum = _seqNum;
}

myString Event:: getDetails() const {
  return details;
}

void Event:: setDetails(myString _details) {
  _details = details;
}

std::ostream& operator <<(std::ostream &out, Event &e) {
  out << "<EventPort> <source " << e.getSource() << "> <type " << e.getType() << " > <key " <<e.getKey() << " seqNum " << e.getSequenceNum()  << " > < details " << e.getDetails() << " > " << std::endl;
  return out;
}

void hpcxx_pack(HPCxx_Buffer& b, Event* p, int count){
  for(int i = 0; i < count; i++){
    // pack the string sourceName
    HPCxx_String sourceName((char *) p[i].getSource().c_str());
    hpcxx_pack(b, &sourceName, 1);

    // pack the string typeName
    HPCxx_String typeName((char *) p[i].getType().c_str());
    hpcxx_pack(b, &typeName, 1);

    // pack the key (type long)
    HPCxx_String keyName((char *) p[i].getType().c_str());
    hpcxx_pack(b, &keyName, 1);

    // pack the sequenceNum (type long)
    long seqNum = p[i].getSequenceNum();
    hpcxx_pack(b, &seqNum, 1);

    //pack the string details
    HPCxx_String detailsName((char *) p[i].getDetails().c_str());
    hpcxx_pack(b, &detailsName, 1);
  }
}

void hpcxx_unpack(HPCxx_Buffer& b, Event* p, int count){
  for(int i = 0; i < count; i++){
 
    HPCxx_String sourceName;
    HPCxx_String typeName;
    HPCxx_String keyName;
    HPCxx_String detailsName;
    long seqNum;

    hpcxx_unpack(b, &sourceName, 1);
    hpcxx_unpack(b, &typeName, 1);
    hpcxx_unpack(b, &keyName, 1);
    hpcxx_unpack(b, &seqNum, 1);
    hpcxx_unpack(b, detailsName);

    myString name((char *) sourceName);
    myString type((char *) typeName);
    myString key((char *) keyName);
    myString details((char *) detailsName);
    p->setSource(name);
    p->setType(type);
    p->setKey(key);
    p->setSequenceNum(seqNum);
    p->setDetails(details);
  }
}

