package eventChannel;
import com.objectspace.jgl.*;
import idl.eventPort.*;
import ccacore.*;

public class EventQueue extends Thread {

	EventQueue(String type_, Services core_) {

		type = type_;
		nEvents = 0;
		queue = new Deque();
		listenerSet = new OrderedSet();
		core = core_;
	}

	public void send(CCATEvent ev) {

		synchronized (queue) {
			queue.pushBack(ev);
			if (nEvents++ == 0) {
				queue.notify();
			}
		}
	}

	public void run() {

		while (true) {

			CCATEvent ev = null;

			synchronized (queue) {

				while (nEvents == 0) {
					try {
						queue.wait();
					} catch (InterruptedException e) {
						System.err.println("thread terminating");
						return;
					}
				}

				ev = (CCATEvent) queue.popFront();
				nEvents--;
			}

			UsesEventPort port = null;
			try {
				port = (UsesEventPort) core.getPort(type + "_uses");
			} catch (exceptions.CCAException e) {
				System.err.println("CCA Exception.");
				System.exit(1);
			}
			try {
				port.notify(ev);
			} catch (java.lang.Exception e) {
				System.err.println("notify Exception.");
				System.exit(1);
			}

			System.err.println("sent event " + ev.getKey());
		}
	}

	public void addListener(Listener l) {

		synchronized (listenerSet) {
			listenerSet.add(l);
		}
	}

	public void removeListener(Listener l) {

		synchronized (listenerSet) {
			listenerSet.remove(l);
		}
	}

	synchronized public int nListeners() {
		
		return listenerSet.size();
	}

	String type;
	Deque queue;
	int nEvents;
	OrderedSet listenerSet;
	Boolean term;

	Services core;
}
