package compositiontool.view;

import javax.swing.JInternalFrame;
import javax.swing.ImageIcon;
import java.awt.Dimension;

/**
 * <tt>ComponentIcon</tt> is a minimised version of a <tt>ComponentView</tt>.
 * @see ComponentView
 */
public class ComponentIcon extends JInternalFrame {
    private ComponentView fullView;

    /**
     * Create a component icon for the specified component with the specified name.
     * @param componentName the component's name.
     * @param fullView the maximized component view.
     */
    public ComponentIcon(String componentName, ComponentView fullView) {
	   super(componentName);
	   this.setMaximizable(true);
	   this.pack();
	   this.setVisible(true);
	   this.fullView = fullView;
    }

    /**
     * Get the maximized view of this icon.
     *
     * @return the full component view.
     *
     */
    public ComponentView getFullView() {
	   return fullView;
    }

    /**
     * Get the preferred size of this icon.
     *
     * @return the component icon's preferred size.
     *
     */
    public Dimension getPreferredSize() {
	return new Dimension(175,45);
    }

    /**
     * Overrides the default <tt>isIcon</tt> method to always return true.
     * Note that the base class implementation checks if the frame's state is 
     * iconified or not before returning true or false accordingly. Here we 
     * have a custom icon so we always return true instead of checking the 
     * frame's state.
     * @return true
     */
    public boolean isIcon() {
	   return true;
    }
}
