package compositiontool.view;


import javax.swing.JFrame;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JTextArea;
import javax.swing.ImageIcon;
import javax.swing.DefaultListCellRenderer;
import java.awt.Component;
import java.awt.Color;
import javax.swing.Icon;
import java.awt.Dimension;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.util.Date;
import java.util.Hashtable;
import idl.eventPort.CCATEvent;


public class EventListenerGUI extends JFrame implements MouseListener {
  private DefaultListModel model;
  private JList listDisplay;
  private static int id = 0;
  private Hashtable buttonsForItems;
  private static Icon CONNECT_ICON = new ImageIcon(MainCompositionToolWindow.CONFIG_DIR+"icons/NewPlug.gif");
  private static Icon DISCONNECT_ICON = new ImageIcon(MainCompositionToolWindow.CONFIG_DIR+"icons/UnPlug.gif");
  private static Icon CREATE_ICON = new ImageIcon(MainCompositionToolWindow.CONFIG_DIR+"icons/instancealive.gif");

  public EventListenerGUI() {
    super("Event Listener");
    // set up the model
    model = new DefaultListModel();
    // create a list for displaying events
    listDisplay = new JList();
    listDisplay.addMouseListener(this);
    listDisplay.setModel(model);
    buttonsForItems = new Hashtable();
    // set the renderer for this list
    listDisplay.setCellRenderer(new DefaultListCellRenderer() {
      public Component getListCellRendererComponent(JList list, Object value,
						    int index, boolean hasSelected,
						    boolean hasFocus) {
	final ListElement currentElement = (ListElement) value;
	JButton buttonForItem = null;
	if (currentElement!=null)
	  buttonForItem = (JButton) buttonsForItems.get(currentElement);
	if (buttonForItem == null) {
	  // create a button
	  if (currentElement.getType().equals("creation"))
	    buttonForItem = new JButton(CREATE_ICON);
	  else if (currentElement.getType().equals("connection"))
	    buttonForItem = new JButton(CONNECT_ICON);
	  else
	    buttonForItem = new JButton(DISCONNECT_ICON);
	  buttonForItem.setText(currentElement.toString());
	  System.out.println("Created button with text "+buttonForItem.getText());
	  buttonForItem.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	  buttonsForItems.put(currentElement,buttonForItem);
	} 
	// if the item has not been read, set the background to cyan
	if (!currentElement.hasBeenRead()) {
	  buttonForItem.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
	  buttonForItem.setBackground(Color.cyan);
	} else {
	  buttonForItem.setBackground(Color.yellow);
	}
	return buttonForItem;
      }
    });
    // add the list to a scrollable panel
    JScrollPane panel = new JScrollPane(listDisplay);
    panel.setPreferredSize(new Dimension(200,300));
    this.setContentPane(panel);
    this.pack();
    this.setVisible(true);
  }

  public void mouseEntered(MouseEvent event) { }
  public void mouseExited(MouseEvent event) { }
  public void mousePressed(MouseEvent event) { }
  public void mouseReleased(MouseEvent event) { }
  public void mouseClicked(MouseEvent event) { 
    // aha, display the details for the current element
    JFrame currentDetails = new JFrame();
    JTextArea eventDetails = new JTextArea(10,20);
    eventDetails.setPreferredSize(new Dimension(250,200));
    ListElement currentElement = (ListElement) listDisplay.getSelectedValue();
    eventDetails.setText(currentElement.getType()+"\n\nEvent received at: "+currentElement.getTimeStamp()+"\n"+currentElement.getDetails());
    currentDetails.getContentPane().add(eventDetails);
    currentDetails.pack();
    currentDetails.setVisible(true);
    System.out.println("Displayed frame for event details for event "+currentElement);
  }

  public void addEvent(CCATEvent event) {
    // create an element to add
      System.out.println("Adding event to event GUI");
    ListElement newElement = new ListElement(event,id++);
    // add it in front
    model.add(0,newElement);
    System.out.println("Added item to list");
  }
  
  public class ListElement {
    private String eventType;
    private int eventId;
    private String eventDetails;
    private String timeStamp;
    private boolean hasBeenRead;

    public ListElement(CCATEvent event,int id) {
      // this has not been read
      hasBeenRead = false;
      // set the event ID
      eventId = id;
      // set the time stamp
      timeStamp = new Date().toString();
      // get the event type
      eventType = event.getType();
      // create the details based on the event type
      // everything after the last . character describes the type of CCA event
      int dotPosition = eventType.lastIndexOf(".");
      eventType = eventType.substring(dotPosition+1);
      // ugh, ugly code
      if (eventType.equals("connection")) {
	String connectionDetails = event.getDetails();
	int colonPosition = connectionDetails.indexOf(":");
	String firstPortName = connectionDetails.substring(0,colonPosition);
	String secondPortName = connectionDetails.substring(colonPosition+1);
	eventDetails = "Connection made from port "+firstPortName+" to port "+
	  secondPortName;
      } else if (eventType.equals("disconnection")) {
	String disconnectionDetails = event.getDetails();
	int colonPosition = disconnectionDetails.indexOf(":");
	String firstPortName = disconnectionDetails.substring(0,colonPosition);
	String secondPortName = disconnectionDetails.substring(colonPosition+1);
	System.out.println("Received disconnection event for connection between "+
			   firstPortName+" and "+secondPortName);
      } else if (eventType.equals("creation")) {
	eventDetails = "No details available";
      } else {
	eventId = -1;
      }
    }

    public int getId() {
      return eventId;
    }

    public String getDetails() {
      hasBeenRead = true;
      return eventDetails;
    }

    public String getTimeStamp() {
      return timeStamp;
    }

    public String getType() {
      return eventType;
    }

    public boolean hasBeenRead() {
      return hasBeenRead;
    }

    public String toString() {
      return "Event #"+eventId;
    }
  }
}
