package compositiontool.view;

import java.awt.Point;
import java.awt.geom.Line2D;

public class Line {
    private Point start;
    private Point end;
    private boolean dummy;
    private boolean visible;
    private PortView output;
    private PortView input;
    private double slope;
    
    public Line() {
	start = new Point();
	end = new Point();
	dummy = true;
    }
    
    public Line(Point start, Point end, PortView output, PortView input, boolean dummy) {
	this.start = new Point(start);
	this.end = new Point(end);
	this.dummy = dummy;
	this.output = output;
	this.input = input;
	this.visible = true;
	this.slope = computeSlope(start,end);
    }

    private double computeSlope(Point p1, Point p2) {
	if (p2.x == p1.x) {
	    // infinite slope
	    return 25;
	}
	double slope = ((p2.y-p1.y)*1.0)/(p2.x-p1.x);
	if (slope > 25)
	    slope = 25;
	if (slope < -25)
	    slope = -25;
	return slope;
    }

    public PortView getOutput() {
	return output;
    }

    public PortView getInput() {
	return input;
    }

    public Point getStart() {
	return start;
    }
    
    public Point getEnd() {
	return end;
    }

    public boolean isDummy() {
	return dummy;
    }

    public void setVisible(boolean state) {
	this.visible = state;
    }

    public boolean isVisible() {
	return visible;
    }

    public String toString() {
	if (dummy) {
	    if (visible)
		return ("dummy line from "+start+" to "+end);
	    else
		return ("invisible dummy line from "+start+" to "+end);
	} else {
	    if (visible)
		return ("real line from "+start+" to "+end);
	    else
		return ("invisible real line from "+start+" to "+end);
	}
    }
    
    public boolean containsPoint(Point point) {
	// compute slope of the line formed by this point and the start
	double slope = computeSlope(point,start);
	double difference = slope - this.slope;
	double absDifference = (difference > 0)? difference : -difference;
	double absSlope = (this.slope > 0)? this.slope : -this.slope;
	if (absDifference < absSlope*0.01 || absDifference < 0.01)
	    return true;
	return false;
    }
}
