package compositiontool.view;

import ids.InformationDirectoryService;
import ids.InformationDirectoryContext;
import registry.RegistryService;
import registry.RegistryContext;
import javax.swing.JToolBar;
import javax.swing.JButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.ImageIcon;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.lang.reflect.Method;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;

/**
 * This holds buttons for commonly used CCAT functions. 
 */
public class ToolbarView extends JToolBar {
    private RegistryService registryService;
    private InformationDirectoryService idsService;
    public static String DEFAULT_REGISTRY_CONTEXT = "http://lookout.extreme.indiana.edu:4242/registry_test";
    public static String DEFAULT_IDS_CONTEXT = "http://lookout.extreme.indiana.edu:4242/ccat/XML";
    private String initialRegistryContext;
    private String initialIDSContext;
    // the ids and registry browser windows
    private idsbrowser.MainBrowserWindow idsBrowserWindow;
    private registrybrowser.MainBrowserWindow registryBrowserWindow;
    // working area view
    private WorkingAreaController workingAreaController;

    /**
     * Create a tool bar given handles to the ids and registry services
     */
    public ToolbarView(RegistryService registryService,
		       InformationDirectoryService idsService,
		       WorkingAreaController controller) {
	// set the services
	this.registryService = registryService;
	this.idsService = idsService;
	// set the working area view
	this.workingAreaController = controller;
	// no ids and registry windows initially
	idsBrowserWindow = null;
	registryBrowserWindow = null;
	// set the default initial contexts
	initialIDSContext = DEFAULT_IDS_CONTEXT;
	initialRegistryContext = DEFAULT_REGISTRY_CONTEXT;
	// set the border
	setBorder(javax.swing.border.LineBorder.createBlackLineBorder());
	// create the individual tool bar items and add them to the tool bar
	// browse information directory
	ToolbarItem currentItem = new ToolbarItem(this, "instancedead.gif","Information Directory Browser",
						  "startIDSBrowser");
	add(currentItem);
	// browse registry
	currentItem = new ToolbarItem(this, "instancealive.gif","Registry Browser",
				      "startRegistryBrowser");
	add(currentItem);
	// set preferences
	currentItem = new ToolbarItem(this, "preferences.gif","Set Preferences",
				      "setPreferences");
	add(currentItem);
	// quit CCAT
	currentItem = new ToolbarItem(this, "Delete.gif","Quit CCAT","quit");
	add(currentItem);
    }

    /**
     * Start the IDS browser 
     */
    public void startIDSBrowser() {
	// check if we already started one before
	if (idsBrowserWindow != null) {
	    idsBrowserWindow.setVisible(true);
	    if (idsBrowserWindow.getState() == Frame.ICONIFIED) {
		idsBrowserWindow.setState(Frame.NORMAL);
	    }
	    return;
	}
	try {
	    InformationDirectoryContext context = idsService.getContext(initialIDSContext);
	    workingAreaController.setStatusText("Starting IDS browser...");
	    idsBrowserWindow = 
		new idsbrowser.MainBrowserWindow(idsService,context,initialIDSContext);
	    workingAreaController.setStatusText("IDS browser started");
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    /**
     * Start the Registry browser 
     */
    public void startRegistryBrowser() {
	// if we started one before, reuse the window
	if (registryBrowserWindow != null) {
	    registryBrowserWindow.setVisible(true);
	    if (registryBrowserWindow.getState() == Frame.ICONIFIED) {
		registryBrowserWindow.setState(Frame.NORMAL);
	    }
	    return;
	}
	try {
	    RegistryContext context = registryService.getContext(initialRegistryContext);
	    workingAreaController.setStatusText("Starting registry browser...");
	    registryBrowserWindow = 
		new registrybrowser.MainBrowserWindow(registryService,context,
						      initialRegistryContext);
	    workingAreaController.setStatusText("Registry browser started");
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    /**
     * Set preferences
     */
    public void setPreferences() {
	// create a viewer for changing preferences
	JTabbedPane preferenceWindow = new JTabbedPane();
	// create a panel for changing the IDS/registry initial contexts
	JPanel changePreferences = new JPanel();
	JPanel buttonPanel = new JPanel(new GridLayout(2,2));
	buttonPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	final JFrame top = new JFrame("Set preferences");
	JLabel regLabel = new JLabel("Initial Registry Context");
	final JTextField initialRegistry = new JTextField(initialRegistryContext);
	JLabel idsLabel = new JLabel("Initial IDS Context");
	final JTextField initialIDS = new JTextField(initialIDSContext);
	JButton apply = new JButton("Apply");
	apply.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent event) {
		initialIDSContext = initialIDS.getText();
		initialRegistryContext = initialRegistry.getText();
		top.setVisible(false);
		// if there is any current invisible IDS or registry browser
		// set it to null 
		if (idsBrowserWindow != null && !idsBrowserWindow.isVisible())
		    idsBrowserWindow = null;
		if (registryBrowserWindow != null && !registryBrowserWindow.isVisible())
		    registryBrowserWindow = null;
	    }
	});
	changePreferences.setLayout(new BoxLayout(changePreferences,BoxLayout.Y_AXIS));
	buttonPanel.add(regLabel);
	buttonPanel.add(initialRegistry);
	buttonPanel.add(idsLabel);
	buttonPanel.add(initialIDS);
	changePreferences.add(buttonPanel);
	changePreferences.add(apply);
	preferenceWindow.addTab("Contexts",null,changePreferences,"Change browser contexts");
	// create a panel for setting the background image to be used
	JPanel changeBackground = new JPanel();
	// create an array of background names by peeking at the skins directory
	File backgroundDirectory = new File(MainCompositionToolWindow.CONFIG_DIR+"backgrounds");
	File [] backgrounds = backgroundDirectory.listFiles();
	String [] backgroundNames = new String [backgrounds.length];
	for (int i=0; i<backgroundNames.length; i++) {
	    backgroundNames[i] = new String(backgrounds[i].getName());
	}
	// create a list box with possible backgrounds
	JList listBox = new JList(backgroundNames);
	// put this in a scrollable pane
	JScrollPane backgroundPane = new JScrollPane(listBox);
	listBox.addListSelectionListener(new ListSelectionListener() {
	    public void valueChanged(ListSelectionEvent event) {
		JList backgroundList = (JList) event.getSource();
		String background = (String) backgroundList.getSelectedValue();
		// change the background!
		workingAreaController.changeBackground(background);
	    }
	});
	// add the scroll pane to our main panel
	changeBackground.add(backgroundPane);
	preferenceWindow.addTab("Background",null,changeBackground,"Change interface background");
	preferenceWindow.setVisible(true);
	top.getContentPane().add(preferenceWindow);
	top.pack();
	top.setVisible(true);
    }
	
    /**
     * Quit CCAT
     */
    public void quit() {
	workingAreaController.setStatusText("Quitting CCAT, kiling all components...");
	workingAreaController.quit();
	System.out.println("Goodbye!");
	System.exit(0);
    }

    /**
     * <tt>ToolbarItem</tt> is a button for which the name of a method in the 
     * toolbar view can be specified as the event handler.
     * @see ToolbarView
     */
    public class ToolbarItem extends JButton implements ActionListener {
	private Method eventHandler;
	private ToolbarView view;

	/** 
	 * Creates a toolbar item with the specified button text, tool tip and event handler.
	 * @param text the text to be displayed on the button
	 * @param toolTip the string to be displayed as tooltip text.
	 * @param eventHandlerName the name of a method in the <tt>ToolbarView</tt>
	 * class which should be called when the uses clicks on this toolbar item.
	 */
	public ToolbarItem(ToolbarView view, String imageFileName, 
			   String toolTip, String eventHandlerName) {
	    super();
	    String fullFileName = MainCompositionToolWindow.CONFIG_DIR+"icons/"+imageFileName;
	    this.setIcon(new ImageIcon(fullFileName));
	    this.view = view;
	    try {
		eventHandler = view.getClass().getDeclaredMethod(eventHandlerName,new Class[0]);
	    } catch (Exception exception) {
		System.err.println("Unable to look up method named "+eventHandlerName+" in toolbar controller\n");
	    }
	    this.addActionListener(this);
	    this.setToolTipText(toolTip);
	}
	
	/** 
	 * Called when the user clicks on this toolbar item; results in the 
	 * appropriate event handler in the controller being invoked.
	 * 
	 */
	public void actionPerformed(ActionEvent event) {
	    // invoke the handler
	    try {
		eventHandler.invoke(view,new Object[0]);
	    } catch (Exception exception) {
		System.err.println("Unable to invoke method "+eventHandler.getName()+" due to exception "+exception);
		exception.printStackTrace();
	    }
	}
    }
}
