/**
 * This class allows users to browse the specified information directory.
 * This will start a GUI which allows the user to browse the directory
 * hierarchy; the interface is similar to a file system browser.
 * Users can drag component icons into and out of the browser. When a 
 * component icon is dropped into the IDS browser, it looks for a 
 * transferable file associated with that icon and attempts to interpret 
 * it as a component XML file. If it fails, the drop is not successful, 
 * otherwise the dropped icon is added into the current context as a new
 * component. Components dragged from the IDS browser will similarly have
 * a transferable file associated. These can be dropped over applications
 * which accept icon drops, but the behaviour is application dependent.
 * @author Nirmal (nmukhi)
 */

package idsbrowser;

import ccacore.Component;
import ccacore.Services;
import ccacore.ServicesImpl;
import ccacore.Port;
import ids.InformationDirectoryService;
import ids.InformationDirectoryContext;

public class InformationDirectoryBrowserComponent implements Component  {
    private MainBrowserWindow browserWindow;
    private Services myServices;
    
    public InformationDirectoryBrowserComponent() { 
    }
        
    public void setServices(Services s){
	// no ports to be registered
	myServices = s;
    }
    
    public void execute(String initialContext) {
	try {
	    System.out.println("\n IDS Browser Component:: execute  ");
	    // get the IDS port
	    InformationDirectoryService idsPort = 
		(InformationDirectoryService) myServices.getPort("InformationDirectoryService");
	    System.out.println("ids port is "+idsPort);
	    // create the initial context
	    InformationDirectoryContext context = idsPort.getContext(initialContext);
	    // start up the window
	    new MainBrowserWindow(idsPort,context,initialContext);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    // main program
    // this is the "framework main" which creates a services object
    // and starts up the component
    public static void main(String [] args) {
	// create the component object
	InformationDirectoryBrowserComponent component = 
	    new InformationDirectoryBrowserComponent();
	// create services object and set the component services
	try {
	    Services services = new ServicesImpl();
	    component.setServices(services);
	} catch (Exception exception) {
	    System.err.println("Could not set services");
	    exception.printStackTrace();
	}
	// execute the component
	// args 0 is the initial context
	component.execute(args[0]);
    }
}
