/**
 * This class is used to choose an icon for an information 
 * object in the information directory tree. 
 * @author Nirmal (nmukhi)
 * @see InformationDirectoryIconChooserFactory
 */

package idsbrowser;

import javax.swing.Icon;
import ids.InformationDirectoryNode;

public interface InformationDirectoryIconChooser {

    /**
     * Choose an icon for an info object 
     * @param infoNode the IDS tree node object for which an icon is desired
     * @expanded whether this info object has been expanded or not
     * in the tree
     * @return the icon to be used for this info object
     */
    public Icon getMatchingIcon(InformationDirectoryNode infoNode, 
				     boolean expanded);

    /**
     * Open a window which allows browsing of the icon filters being used
     * by this icon chooser
     */
    public void browseIconFilters();
}

