/**
 * This class creates icon chooser based on parameters which dictate
 * the way an icon chooser selects the icon to be rendered for a 
 * file. 
 * @see InformationDirectoryIconChooser
 */

package idsbrowser;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.tree.DefaultTreeCellRenderer;
import java.util.Vector;

public class InformationDirectoryIconChooserFactory {
    public static Icon DEFAULT_CONTEXT_ICON = 
	new DefaultTreeCellRenderer().getDefaultClosedIcon();
    public static Icon DEFAULT_EXPANDED_CONTEXT_ICON = 
	new DefaultTreeCellRenderer().getDefaultOpenIcon();
    public static Icon DEFAULT_INFO_ICON = 
	new DefaultTreeCellRenderer().getDefaultLeafIcon();

    /**
     * Create a default icon chooser
     * @return a default icon chooser
     */
    public static InformationDirectoryIconChooser createDefaultIconChooser() {
	InfoIconSelector selector = new InfoIconSelector(InfoIconSelector.WILD_CARD,
							 InfoIconSelector.WILD_CARD,
							 DEFAULT_INFO_ICON);
	Vector iconSelectors = new Vector();
	iconSelectors.add(selector);
	return createIconChooser(DEFAULT_CONTEXT_ICON,
				 DEFAULT_EXPANDED_CONTEXT_ICON,
				 iconSelectors);
    }

    /**
     * Create an icon chooser with the specified parameters.
     * @param contextIcon the icon to be used for a context when it
     * is not expanded
     * @param expandedContextIcon the icon to be used for a context 
     * when it is expanded
     * @param infoIconSelectorList a vector of info icon selectors
     * which are used to determine the icon for an info object in
     * the hierarchy
     * @return an icon chooser which chooses icons based on the given
     * information
     */
    public static InformationDirectoryIconChooser createIconChooser(Icon contextIcon,
								    Icon expandedContextIcon,
								    Vector infoIconSelectors) {
	// create an icon chooser with the specified parameters
	return new InformationDirectoryIconChooserImplementation(contextIcon, expandedContextIcon,
								 infoIconSelectors);
    }
}
