/**
 * This class is used to update the information directory tree 
 * when a node is expanded. It works by using an information
 * directory context to get the children of the expanded node 
 * and adding those to the tree if this was not done previously.
 * @see InformationDirectoryTreeView
 * @author Nirmal (nmukhi)
 */

package idsbrowser;

import javax.swing.event.TreeExpansionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeNode;
import java.util.Enumeration;
import javax.swing.event.TreeExpansionEvent;

public class InformationDirectoryTreeExpansionListener implements TreeExpansionListener {
    private DefaultTreeModel model;

    /** Create a tree expansion listener 
     * @param model the tree model for the information directory tree
     */
    public InformationDirectoryTreeExpansionListener(DefaultTreeModel model) {
	// set the model 
	this.model = model;
    }

    /**
     * Called when a tree expansion event occurs. This will get the node
     * which got expanded and get its children if they weren't already 
     * a part of the tree. These children are then added to the tree model 
     * @param event the tree expansion event that triggered this method 
     * call
     * @see InformationNode#readTree()
     */
    public void treeExpanded(TreeExpansionEvent event) {
	// get the node corresponding to the expansion event
	TreePath path = event.getPath();
	InformationDirectoryTreeNode node = 
	    (InformationDirectoryTreeNode)path.getLastPathComponent();
	// read the node's children
	if (node.readTree()) {
	    // get the children
	    int childrenIdx[] = new int[node.getChildCount()];
	    int i=0;
	    for(Enumeration e = node.children(); e.hasMoreElements();) {
		Object obj = e.nextElement();
		childrenIdx[i] = node.getIndex((TreeNode)obj);
		i++;
	    }
	    model.nodesWereInserted(node,childrenIdx);
	}
    }

    // required for interface implementation
    public void treeCollapsed(TreeExpansionEvent event) { }
}

