/**
 * This class creates icon chooser based on parameters which dictate
 * the way an icon chooser selects the icon to be rendered for some
 * information. 
 * @see RegistryIconChooser
 */

package registrybrowser;

import javax.swing.Icon;
import javax.swing.tree.DefaultTreeCellRenderer;
import java.util.Vector;
import idsbrowser.InfoIconSelector;

public class RegistryIconChooserFactory {
    private static Icon DEFAULT_CONTEXT_ICON = 
	new DefaultTreeCellRenderer().getDefaultClosedIcon();
    private static Icon DEFAULT_EXPANDED_CONTEXT_ICON = 
	new DefaultTreeCellRenderer().getDefaultOpenIcon();
    private static Icon DEFAULT_INFO_ICON = 
	new DefaultTreeCellRenderer().getDefaultLeafIcon();

    /**
     * Create a default icon chooser
     * @return a default icon chooser
     */
    public static RegistryIconChooser createDefaultIconChooser() {
	InfoIconSelector selector = new InfoIconSelector(InfoIconSelector.WILD_CARD,
							 InfoIconSelector.WILD_CARD,
							 DEFAULT_INFO_ICON);
	Vector iconSelectors = new Vector();
	iconSelectors.add(selector);
	return createIconChooser(DEFAULT_CONTEXT_ICON,
				 DEFAULT_EXPANDED_CONTEXT_ICON,
				 iconSelectors);
    }

    /**
     * Create an icon chooser with the specified parameters.
     * @param contextIcon the icon to be used for a context when it
     * is not expanded
     * @param expandedContextIcon the icon to be used for a context 
     * when it is expanded
     * @param infoIconSelectorList a vector of info icon selectors
     * which are used to determine the icon for an info object in
     * the hierarchy
     * @return an icon chooser which chooses icons based on the given
     * information
     */
    public static RegistryIconChooser createIconChooser(Icon contextIcon,
							Icon expandedContextIcon,
							Vector infoIconSelectors) {
	// create an icon chooser with the specified parameters
	return new RegistryIconChooserImplementation(contextIcon, expandedContextIcon,
						     infoIconSelectors);
    }
}
