
package matlab;

import ccacore.Component;
import ccacore.Services;
import ccacore.ServicesImpl;
import ccacore.Port;
import ccacore.ComponentID;
import ccacore.DefaultPortInfo;
import eventChannel.FilteringEventChannelComponent;
import ids.InformationDirectoryService;
import idl.filteringEventChannel.UsesFilteringEventChannel;
import creation.CreationService;
import connection.ConnectionService;
import registry.RegistryService;
import nexusrmi.RemoteException;

import java.io.FileOutputStream;
import java.io.PrintStream;


public class CompositionToolComponent implements Component  {
    private Services myServices;
    private EventListener eventListener;
    private ComponentCommunicator componentCommunicator;
    private WorkingAreaController controller;
    
    // public CompositionToolComponent() { }

    public CompositionToolComponent() {




        System.out.println ("Initializing CompositionToolComponent");
	// create services object and set the component services
	try {
	    FilteringEventChannelComponent eventChannel = new FilteringEventChannelComponent();
	    ServicesImpl eventChannelCore = new ServicesImpl();
	    eventChannel.setServices(eventChannelCore);
	    ComponentID eventChannelComponentID = eventChannelCore.getComponentID();
	    Services services = new ServicesImpl(eventChannelComponentID);
	    setServices(services);
	    ComponentID myComponentID = services.getComponentID();
	    
	    // get the connection service to connect yourself to the
	    // event Channel
	    ConnectionService connectionService = (ConnectionService)services.getPort("ConnectionService");
	    
	    System.out.println("\nGUI component::calling connection Service connect ");
	    connectionService.connect(myComponentID, "filteringEventChannelPort", eventChannelComponentID, "register");
	    System.out.println("\nGUI component::connection Service connect succeeded ");
	    UsesFilteringEventChannel outputPort = (UsesFilteringEventChannel)(services.getPort("filteringEventChannelPort"));
	    // now send this information to the event Channel that
	    // that the GUI wants to listen to all "cca*" type events
	    
	    outputPort.registerListener(myComponentID, "guiEventListener", "cca");
	    services.releasePort("filteringEventChannelPort");
	    execute();
	} catch (Exception exception) {
	    System.err.println("Could not set services");
	    exception.printStackTrace();
	}
    }
        

    public void setServices(Services s){
	// set the services
	myServices = s;
	// register the event listener port
	try {
	    //eventListener = new EventListener(new EventListenerGUI());
	    eventListener = new EventListener(null);
	    myServices.addProvidesPort(eventListener,
				       new DefaultPortInfo("guiEventListener", 
							   "EventPort_idl"));
	    // create an output port that will be used to
	    // register with the evenChannel
	    s.registerUsesPort(new DefaultPortInfo("filteringEventChannelPort", "FilteringEventChannel_idl"));
	} catch (Exception exception) {
	    System.err.println("Could not set up event listener, "+
			       "got exception "+exception);
	    exception.printStackTrace();
	}
	// create the component communicator component
	ComponentCommunicatorImplementation componentCommunicatorImpl = 
	    new ComponentCommunicatorImplementation();
	componentCommunicatorImpl.setServices(myServices);
	componentCommunicator = (ComponentCommunicator) componentCommunicatorImpl;
    }
    
    public void execute() {
	    System.out.println("\n Composition Tool Component:: execute  ");

	try {
	    // get the IDS port
	    InformationDirectoryService idsPort = 
		(InformationDirectoryService) myServices.getPort("InformationDirectoryService");
	    // get the registry port
	    RegistryService registryPort = 
		(RegistryService) myServices.getPort("RegistryService");
	    // get the creation port
	    CreationService creationPort = 
		(CreationService) myServices.getPort("CreationService");
	    // get the connection port
	    ConnectionService connectionPort = 
		(ConnectionService) myServices.getPort("ConnectionService");
            controller = new WorkingAreaController(creationPort, connectionPort, registryPort, idsPort, eventListener, componentCommunicator);
            System.out.println ("Set working area controller");

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }



    /* Return the Working Area Controller
    */
    public WorkingAreaController  getController() {
        
        return controller;

    }



}
