package matlab;

import idl.eventPort.CCATEvent;
import idl.eventPort.ProvidesEventPortImpl;
import nexusrmi.RemoteException;

public class EventListener extends ProvidesEventPortImpl {
    private WorkingAreaController guiController;
  private EventListenerGUI gui;
    public EventListener() throws RemoteException {
	super();
	guiController = null;
	gui = null;
    }

  public EventListener(EventListenerGUI gui) throws RemoteException {
      this();
      this.gui = gui;
  }

    public int notify(CCATEvent ccatEvent){
	try {
	    System.out.println("EventListener got event "+ ccatEvent.toString());
	    if (gui != null)
	      gui.addEvent(ccatEvent);
	    // if there is not GUI yet, ignore the event
	    if (guiController == null)
		return 0;
	    // parse the event type and call the appropriate method on 
	    // the GUI controller
	    String type = ccatEvent.getType();
	    // everything after the last . character describes the type of CCA event
	    int dotPosition = type.lastIndexOf(".");
	    String subType = type.substring(dotPosition+1);
	    // switch on the subtype
	    // ugh, ugly code
	    if (subType.equals("connection")) {
		guiController.connectionEventReceived(ccatEvent);
	    } else if (subType.equals("disconnection")) {
		guiController.disconnectionEventReceived(ccatEvent);
	    } else if (subType.equals("creation")) {
		guiController.creationEventReceived(ccatEvent);
	    } else if (subType.equals("ids_addition")) {
		guiController.idsAdditionEventReceived(ccatEvent);
	    } else if (subType.equals("ids_modification")) {
		guiController.idsModificationEventReceived(ccatEvent);
	    } else if (subType.equals("registry_bind")) {
		guiController.registryBindEventReceived(ccatEvent);
	    } else if (subType.equals("registry_unbind")) {
		guiController.registryUnbindEventReceived(ccatEvent);
	    } else {
		System.out.println("Warning: recevied event of unknown type "+ccatEvent);
	    }
	    return 0;
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return 0;
    }

    public void setGUIController(WorkingAreaController controller) {
	this.guiController = controller;
    }
}
