package matlab;

import ids.InformationDirectoryContext;
import ids.InformationDirectoryNode;
import ids.InformationInfo;
import registry.InstanceXMLUtils;
import registry.InstanceInfo;
import util.EnvObj;
import util.Info;
import ids.InformationXMLUtils;
import creation.CreationService;
import connection.ConnectionService;
import ccacore.ComponentID;
import matlab.ComponentModel;

public class IDSRequestHandler {
    
    public static void handleIDSLookup( InformationDirectoryContext idsContext) {
	// make a list of component names under this context
	// allow user to view or create

	try {
	    // lookup the context to get all the children
	    InformationDirectoryNode [] children = idsContext.getChildren();
	    // create an option list of component information here
	    // and a list of subcontexts
	    StringBuffer componentList = new StringBuffer("componentchoice: \n");
	    StringBuffer contextList = new StringBuffer("subcontext: \n");
	    String contextName = idsContext.getName();
	    for (int i=0; i<children.length; i++) {
		InformationDirectoryNode currentNode = children[i];
		// get the stripped name (i.e. name relative to this context)
		String fullName = currentNode.getName();
		String strippedName = fullName.substring(contextName.length());
		// check if this is component information
		if (currentNode.isInformation()==1) {
		    // this corresponds to component information
		    componentList.append(strippedName+"    ");
		} else if(currentNode.isContext()==1) {
		    // this corresponds to a subcontext
		    contextList.append(strippedName+"    ");
		}
	    }
	    componentList.append("\0");
	    contextList.append("\0");
	    // now show these two lists along with possible actions
            System.out.println ( componentList + " \n" + contextList) ;

	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    

    public static ComponentModel handleIDSCreate(String componentName,
				       InformationDirectoryContext idsContext,
				       CreationService creationPort,
                                       ConnectionService connectionPort,
                                       WorkingAreaController controller,
                                       ComponentCommunicator communicator) {
	// create the selected component information and add to workspace


        ComponentModel componentModel = null;

	try {
	    // lookup this information in the current context
	    InformationInfo componentInfo = (InformationInfo) idsContext.lookup(componentName);
	    // get the machine name and creration mechanism of the first installation
	    Info [] installations = 
		componentInfo.getInfo("component-info")[0].getInfo("installation");
	    if (installations == null) {
		// no installations! can't create
		System.out.println("Did not find any installations");
	    } else {
		// get the host and creation mechanism of the first installation
		String hostName = installations[0].getValue("host-name")[0];
		String creationMech = installations[0].getInfo("creation-info")[0].
		    getValue("creation-method")[0];
		System.out.println("Will create component on "+hostName+" using "+
				   creationMech);
		EnvObj creationEnv = 
		    componentInfo.getCreationEnvironment(hostName,creationMech);
		// add a dummy id to the creation env to be used so we can identify 
		// the creation event 
		String instanceId = creationEnv.get("user-name")+creationEnv.get("creation-time")+Math.random();
		creationEnv.put("instance-name",instanceId);
		// create the instance
		ComponentID id = creationPort.createInstance(creationEnv);
		// create instance information for this component
		Info [] paramInfoArray = componentInfo.getInfo("component-info")[0].getInfo("parameter-block");
		Info paramInfo = null;
		if (paramInfoArray.length != 0)
		    paramInfo = paramInfoArray[0];
		String xmlDescription = 
		    InstanceXMLUtils.createXMLFromInfo(id,componentInfo.getName(),
						       creationEnv,
						       componentInfo.getInfo("component-info")[0].getInfo("port-type-list")[0],
						       componentInfo.getInfo("component-info")[0].getInfo("port-list")[0],
						       paramInfo,
						       componentInfo.getInfo("component-info")[0].getValue("executable")[0]);
		InstanceInfo instanceInfo = InstanceXMLUtils.createInstanceInfoFromXML(xmlDescription);
		// create a component model for this instance
		componentModel = new ComponentModel (creationPort,
                                                     connectionPort,
                                                     communicator,
                                                     controller,
                                                     instanceInfo);
		// add this to the workspace
		// compositionModel.addComponent(componentModel);
	    }
	} catch (Exception exception) {
	    System.out.println("Could not create "+componentName+
			       ", got exception "+exception);
	    exception.printStackTrace();
	}


        return componentModel;
    }


}
