
% Sample Matlab script file that runs the CCAT Demo.
% Please setenv JAVA_HOME to jdk1.1 before starting
% Matlab. Also, the classpath.txt file for Matlab 
% should contain the paths for the Java classes
% that we want to use for CCAT. 
% -- Last modified by Madhuri on 29 Feb, 2000.

% Turn on the java option so that 
% we can use Java classes in Matlab.
java on


% Create an environment in which we can use CCAT.
compositionTool


% Create component models for the components 
% TimeGen, Discretizer, BasicInfo, Splib, and Assemble.
timegen = createComponent ('TimeGen')
discretizer = createComponent ('Discretizer')
% basicinfo = createComponent ('BasicInfo')
splib = createComponent ('Splib')
assemble = createComponent ('Assemble')


% specify the machines where we want to create 
% instances of the components.
setMachineName (timegen, 'baldy.extreme.indiana.edu')
setMachineName (discretizer, 'baldy.extreme.indiana.edu')
% setMachineName (basicinfo, 'baldy.extreme.indiana.edu')
setMachineName (assemble, 'baldy.extreme.indiana.edu')
setMachineName (splib, 'baldy.extreme.indiana.edu')


% specify the creation mechanisms that are
% to be used while creating the components.
setCreationMechanism (timegen, 'gram')
setCreationMechanism (discretizer, 'gram')
% setCreationMechanism (basicinfo, 'gram')
setCreationMechanism (assemble, 'gram')
setCreationMechanism (splib, 'gram')


% Create live instances of the components.
createInstance (timegen)
createInstance (discretizer)
%createInstance (basicinfo)
createInstance (splib)
createInstance (assemble)


% Pause for a few seconds. This is done so
% that the instance creation is completed before
% we attempt to make and connections.
pause(10)


% Connects input and output ports in the following 
% manner:
%         TimeGen -> Discretizer
%
%         Discretizer -> splib
%
%         Splib -> Assemble
%
%         Assemble -> TimeGen (feedback)
connectPorts (timegen, 'outputTimeStep', discretizer, 'inputTimeStep')
% connectPorts (discretizer, 'outputSLS', basicinfo, 'inputSLS')
connectPorts (discretizer, 'outputGeometry', assemble, 'inputGeometry')
connectPorts (discretizer, 'outputSLS', splib, 'inputSLS')
connectPorts (splib, 'outputSV', assemble, 'inputSV')
connectPorts (assemble, 'outputRunItAgain', timegen, 'inputRunItAgain')


% Set the parameters of the TimeGen component to
% their default values.
setParams (timegen)


% Execute the TimeGen component. This will automotically
% execute all the other components.
execute (timegen)
