package python;

import ccacore.Services;
import ccacore.ComponentID;
import ccacore.DefaultPortInfo;
import ccacore.Port;
import connection.ConnectionService;
import java.lang.reflect.Method;
import java.util.Hashtable;

public class ComponentCommunicatorImplementation implements ComponentCommunicator {
    private Services services;
    private Hashtable connections;
    private ComponentID componentID;

    // create a component communicator
    public ComponentCommunicatorImplementation() {
	connections = new Hashtable();
    }

    // set the component communicator's services
    public void setServices(Services services) {
	this.services = services;
	this.componentID = services.getComponentID();
    }

    // set the params of the component with the specified
    // component id and param port name
    // paramValue is what the params should be set to
    public synchronized void setComponentParams(ComponentID componentID,
						String paramPortClassName,
						String parameterPortName,
						String parameterMethodName,
						String instanceID,
						Object paramValue) {
	Object [] methodParams = new Object[1];
	methodParams[0] = paramValue;
	invokeMethodOnComponent(componentID,paramPortClassName,
				parameterPortName,instanceID,parameterMethodName,
				methodParams);
    }
    
    // make the component with the specifed component id stop
    public synchronized void stopComponent(ComponentID componentID, 
					   String controlPortClassName, 
					   String controlPortName,
					   String instanceID) {
	invokeMethodOnComponent(componentID,controlPortClassName,
				controlPortName,instanceID,"stop",new Object[0]);
    }
    
    // make the component with the specifed component id start
    public synchronized void startComponent(ComponentID componentID, 
					    String controlPortClassName, 
					    String controlPortName,
					    String instanceID) {
	invokeMethodOnComponent(componentID,controlPortClassName,
				controlPortName,instanceID,"start",new Object[0]);
    }
    
    // kill the component
    public synchronized void killComponent(ComponentID componentID, 
					   String controlPortClassName, 
					   String controlPortName,
					   String instanceID) {
	invokeMethodOnComponent(componentID,controlPortClassName,
				controlPortName,instanceID,"kill",new Object[0]);
    }

    // invoke a method on a component
    private void invokeMethodOnComponent(ComponentID componentID,
					 String portClassName,
					 String portName,
					 String instanceID,
					 String methodName,
					 Object [] methodParams) {
	System.out.println("Attempting to invoke method "+methodName+
			   " on class "+portClassName);
	
	// check if we have a connection to this instance
	try {
	    String usesPortName = (String) connections.get(instanceID+portName);
	    if (usesPortName == null) {
		// registration of a port of this type not done yet, do it
		usesPortName = instanceID+portName;
		System.out.println("Registering port of type "+portClassName+" under name "+usesPortName);
		services.registerUsesPort(new DefaultPortInfo(usesPortName,portClassName));
		// connect to this component on the parameter port
		ConnectionService connectionService = 
		    (ConnectionService) services.getPort("ConnectionService");
		connectionService.connect(this.componentID,usesPortName,
					  componentID,portName);
		// add this to our table of ports
		connections.put(instanceID+portName,usesPortName);
	    }
	    // get the uses port so I can send the parameters
	    Port port = services.getPort(usesPortName);
	    Class portClass = port.getClass();
	    System.out.println("Got port class, loking for method "+methodName);
	    // get the method we want to invoke - use the method name and the types 
	    // of the parameter values for the methods to identify the method
	    Class [] methodParamTypes = new Class[methodParams.length];
	    for (int i=0; i<methodParamTypes.length; i++)
		methodParamTypes[i] = methodParams[i].getClass();
	    Method method = portClass.getMethod(methodName,methodParamTypes);
	    System.out.println("Got method, invoking...");
	    // invoke the method
	    method.invoke(port,methodParams);
	} catch (Exception exception) {
	    System.out.println("Exception while sending params: "+exception);
	    exception.printStackTrace();
	}
    }
}
