package python;

import registry.RegistryContext;
import registry.InstanceInfo;
import registry.RegistryNode;
import registry.InstanceXMLUtils;
import ccacore.ComponentID;
import python.ComponentModel;
import creation.CreationService;
import connection.ConnectionService;


public class RegistryRequestHandler {
    

    public static void bindInstance (ComponentModel componentModel,
                                     String bindingName,
				     RegistryContext registryContext) {
	// bind the specified instance into the context
	// under the specified name, display main screen to user
	
	// get the instance info for this instance
	InstanceInfo info = (InstanceInfo) componentModel.getInfo();
	// bind it!
	try {
	    registryContext.bind(info,bindingName);
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    public static void displayBoundInstances (RegistryContext registryContext) {
	// display a list of instances whose binding can be removed
	// from the current context

	// lookup the current context and get a list of instances bound here
	// lookup the context to get all the children
	try {
	    RegistryNode [] children = registryContext.getChildren();
	    // create an option list of instance information here
	    StringBuffer instanceList = new StringBuffer ("instancechoice: ");
	    String contextName = registryContext.getName();
	    for (int i=0; i<children.length; i++) {
		RegistryNode currentNode = children[i];
		// get the stripped name, i.e. the name relative to this context
		String fullName = currentNode.getName();
		String strippedName = fullName;
		if (fullName.indexOf("/")!=-1)
		    strippedName = fullName.substring(contextName.length());
		// check if this is instance information
		if (currentNode.isInstance()==1) {
		    // this corresponds to instance information
		    instanceList.append(strippedName+"    ");
		} else if (currentNode.isContext()==1) {
		    RegistryContext context = (RegistryContext) currentNode;
		    RegistryNode [] contextChildren = context.getChildren();
		    int j=0;
		    for (j=0; j<contextChildren.length; j++) {
			if (contextChildren[j].isContext()==1)
			    break;
		    }
		    if (j==contextChildren.length) { // yucky code
			// aha, this context has just instance information
			instanceList.append(strippedName+"    ");
		    }
		}
	    }
	    instanceList.append("\0");
	    // now show this list along with an unbind action button
            System.out.println (instanceList);

	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    public static void unBind (String instanceName,
			       RegistryContext registryContext) {
	// remove the binding of the specified instance from the context
	// get the instance which was to be unbound
	// look up this instance
	try {
	    InstanceInfo [] instances = registryContext.lookup(instanceName);
	    // unbind the first one if there are multiple instances bound
	    // under this name
	    ComponentID id = instances[0].getComponentID();
	    // unbind
	    registryContext.unbind(id);
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }



    
    public static void handleRegistryLookup(RegistryContext registryContext) {
	// lookup the specified context and generate a list of instances for
	// the user, along with a list of subcontexts
	// allow the user to view or copy instances as well as 
	// enter subcontexts
	
	try {
	    // lookup the context to get all the children
	    RegistryNode [] children = registryContext.getChildren();
	    // create an option list of instance information here
	    // and a list of subcontexts
	    StringBuffer instanceList = new StringBuffer ("instancechoice: \n");
	    StringBuffer contextList = new StringBuffer ("subcontext: \n");
	    String contextName = registryContext.getName();
	    for (int i=0; i<children.length; i++) {
		RegistryNode currentNode = children[i];
		// get the stripped name, i.e. the name relative to this context
		String fullName = currentNode.getName();
		String strippedName = fullName;
		if (fullName.indexOf("/")!=-1)
		    strippedName = fullName.substring(contextName.length());
		// check if this is instance information
		if (currentNode.isInstance()==1) {
		    // this corresponds to instance information
		    instanceList.append(strippedName+"    ");
		} else if(currentNode.isContext()==1) {
		    // this corresponds to a subcontext
		    // check if this subcontext has only instance information
		    // if so, just treat this as an instance
		    // this is temporary
		    // TODO: provide a general, intuitive way of looking at multiple 
		    // instances bound under the same name
		    RegistryContext context = (RegistryContext) currentNode;
		    RegistryNode [] contextChildren = context.getChildren();
		    int j=0;
		    for (j=0; j<contextChildren.length; j++) {
			if (contextChildren[j].isContext()==1)
			    break;
		    }
		    if (j==contextChildren.length) { // yucky code
			// aha, this context has just instance information
			instanceList.append(strippedName+"    ");
		    } else {
			contextList.append(strippedName+"    ");
		    }
		}
	    }
	    instanceList.append("\0");
	    contextList.append("\0");
	    // now show these two lists along with possible actions
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    public static ComponentModel handleRegistryCopy (String instanceName,
                                           RegistryContext registryContext, 
                                           CreationService creationPort,
                                       ConnectionService connectionPort,
                                       WorkingAreaController controller,
                                       ComponentCommunicator communicator) {
	// copy the specified instance in this context and create a component
	// model from it
	// add this to the workspace
	// give user the main screen

        ComponentModel componentModel = null;

	// look up to get the info about the instance
	// assume there is only one instance bound under this name
	try {
	    InstanceInfo info = registryContext.lookup(instanceName)[0];
	    // create a component model for this instance
	    componentModel = new ComponentModel  (creationPort,
                                                     connectionPort,
                                                     communicator,
                                                     controller,
                                                     info);
	} catch (Exception exception) {
	    exception.printStackTrace();
	}

        return componentModel;
    }
}
