# Import class to create composition tool component.
import compositiontool.view.CompositionToolComponent


# Create a composition tool. Declare it as global
# so that all the functions can access the component.
global CompositionTool
CompositionTool = compositiontool.view.CompositionToolComponent()



# createComponent (componentName) creates a dummy
# component model from the file 'componentName'.
# 'componentName' is the name of the XML file that
# describes the component. A live instance of this
# component can be created using the createInstance()
# function.
def createComponent (componentName):

	componentModel = CompositionTool.getController().create (componentName)
        return componentModel

# setMachineName (componentModel, machineName) specifies
# the machine on which we would like to instantiate
# the component.
def setMachineName (componentModel, machineName):

	componentModel.setMachineName (machineName)
        return 


# setCreationMechanism (componentModel, creationMechanism)
# specifies the creation mechanism to be used while creating
# the component instance.
def setCreationMechanism (componentModel, creationMechanism):

	componentModel.setCreationMechanism (creationMechanism)
        return


# createInstance takes in a handle to the component
# model that we want to create and then creates a
# live instance of this component.
def createInstance (componentModel):

	componentModel.create()
        return


# connectPorts connects the output port 'outputPort' of
# the component 'outputPortComponent' to the input port
# 'inputPort' of the component 'inputPortComponent'.
def connectPorts (outputPortComponent, outputPortName, inputPortComponent, inputPortName):

	outputPortComponent.getOutputPortByName (outputPortName).connectPort (inputPortComponent.getInputPortByName (inputPortName))
        return

# setParams (componentModel) helps us in setting 
# the parameters for the component given by the 
# componentModel. Currently, it sets the paramters 
# to their default values.

def setParams (componentModel):

	componentModel.setParams()
        return 


# execute (componentModel) starts executing the 
# component specified by the componentModel. 

def execute (componentModel):

	componentModel.execute()
        return 



# function that takes in a handle to a live component
# handle and puts it in the current context in the
# registry with the name 'bindingName'. A bound
# instance can be unbound using the unbindInstance()
# function. A bound instace can be copied by using
# the copyInstanceFromRegistry() function.

def bindInstance (componentModel, bindingName):

	CompositionTool.getController().bindInstance (componentModel, bindingName)
	return




# function that creates a live
# component model from the file 'componentName'.
# 'componentName' is the name of the XML file that
# describes the component.

def create (componentName):

	CompositionTool.getController().createInstanceFromIDS (componentName)
	return



# function that copies a live instance from the registry.
# copyInstance takes in an instance name and
# looks up the registry to find an instance bound
# with that name. It returns a component handle
# to the instance if it is found.

def copyInstance (instanceName):

	CompositionTool.getController().copyInstanceFromRegistry (instanceName)
	return




# function that shows the bound instances in the registry.
# displayBoundInstances gives a list of the bound
# instances in the current context in the registry.

def displayBoundInstances():

	CompositionTool.getController().displayBoundInstances()
	return



# function that diplays the default paramter values.
# displayDefaultParams (componentModel) shows
# the default values of the parameters for the component
# given by 'componentModel'. These vaules can be
# changed using the function setParms().

def displayDefaultParams (componentModel):

	CompositionTool.getController().displayDefaultParams (componentModel)
	return



# function that displays the contents of the registry.
# lookupRegistry shows the list of component instances
# and sub-contexts present in the current context at
# registry. The default context at the registry
# can be changed using the setRegistryLocation()
# function.

def lookupRegistry():

	CompositionTool.getController().lookupRegistry()
	return



# function that changes the default location of the IDS.
# setIDSLocation (idsLocation) sets the
# location of the information directory service to
# to the one specified in the string 'idsLocation'.

def setIDSLocation (idsLocation):

	CompositionTool.getController().setIDSLocation (idsLocation)
	return




# function that sets the value of a parameter for a component.
# setParam assigns the value of parameter given by
# string 'paramName' of component 'componentModel'
# to the value given in the string 'paramValue'.

def setParam (componentModel, paramName, paramValue):

	CompositionTool.getController().setParams (componentModel, paramName, paramValue)
	return




# function that changes the default location of the registry.
# setRegistryLocation (registryLocation) sets the
# location of the registry to the one specified
# in the string 'registryLocation'.

def setRegistryLocation (registryLocation):

	CompositionTool.getController().setRegistryLocation (registryLocation)
	return




# function that unbinds instance bound with the name 'bindingName'.
# unbindInstance takes in a string which is the
# name by which a live component is bound in the
# registry and unbinds the instance.

# NOTE: This function is currently not working.

def unbindInstance (bindingName):

	CompositionTool.getController().unbindInstance (bindingName)
	return




# function that displays the contents of the IDS.
# lookupIDS shows the list of components and
# sub-contexts present in the current context at
# Information Directory Server. The default context
# at IDS can be changed using the setIDSLocaiton()
# function.

def lookupIDS():

	CompositionTool.getController().lookupIDS()
	return


