package servlet;

import java.util.Hashtable;
import java.util.Enumeration;
import java.awt.Point;
import java.awt.Dimension;
import util.Info;
import ccacore.ComponentID;
import registry.InstanceInfo;

public class ComponentModel {
    private Hashtable providesPorts; // mapping from port names to port information
    private Hashtable usesPorts; // mapping from port names to port information
    private ComponentID id;
    private InstanceInfo instanceInfo;
    private String uniqueName;
    private Point topLeftCoordinates;
    private Dimension dimensions;
    private String outputURL;

    public ComponentModel(InstanceInfo instanceInfo) {
	this.instanceInfo = instanceInfo;
	this.id = instanceInfo.getComponentID();
	providesPorts = new Hashtable();
	usesPorts = new Hashtable();
	outputURL = null;
	// look up information on ports and add that
	// get the list of ports 
	Info portInformation = instanceInfo.getInfo("instance-info")[0].
	    getInfo("port-list")[0];
	// go through all the ports and create port models for them
	Info [] ports = portInformation.getInfo("port");
	for (int i=0; i<ports.length; i++) {
	    Info currentPort = ports[i];
	    // get the name, direction and type of this port
	    String portName = currentPort.getValue("port-name")[0];
	    String portDirection = currentPort.getValue("port-dir")[0];
	    String portType = currentPort.getValue("my-port-type")[0];
	    if (portDirection.equals("input"))
		providesPorts.put(portName,new PortDetails(portType));
	    else
		usesPorts.put(portName,new PortDetails(portType));
	}
    }

    public String getParameterPortName() {
	if (instanceInfo.getInfo("instance-info")[0].getInfo("parameter-block").length!=0) {
	    String parameterPortName = instanceInfo.getInfo("instance-info")[0].getInfo("parameter-block")[0].getValue("parameter-port-name")[0];
	    return parameterPortName;
	}
	return null;
    }

    public String getParameterPortClassName() {
	if (instanceInfo.getInfo("instance-info")[0].getInfo("parameter-block").length!=0) {
	    String parameterPortClass = instanceInfo.getInfo("instance-info")[0].getInfo("parameter-block")[0].getValue("parameter-port-class")[0];
	    return parameterPortClass;
	}
	return null;
    }

    public String getParameterClassName() {
	if (instanceInfo.getInfo("instance-info")[0].getInfo("parameter-block").length!=0) {
	    String parameterClass = instanceInfo.getInfo("instance-info")[0].getInfo("parameter-block")[0].getValue("parameter-class")[0];
	    return parameterClass;
	}
	return null;
    }
	
    public void setName(String uniqueName) {
	this.uniqueName = uniqueName;
    }

    public String getName() {
	return uniqueName;
    }

    public void setOutputURL(String outputURL) {
	this.outputURL = outputURL;
    }

    public String getOutputURL() {
	return outputURL;
    }

    public InstanceInfo getInfo() {
	return instanceInfo;
    }

    public ComponentID getComponentID() {
	return id;
    }

    public Enumeration getInputPortNames() {
	return providesPorts.keys();
    }

    public Enumeration getOutputPortNames() {
	return usesPorts.keys();
    }

    public int getNumInputs() {
	return providesPorts.size();
    }

    public int getNumOutputs() {
	return usesPorts.size();
    }

    public boolean hasCoordinates() {
	if(topLeftCoordinates==null)
	    return false;
	return true;
    }

    public void setDimensions(Dimension dimensions) {
	this.dimensions = dimensions;
    }

    public Dimension getDimensions() {
	return dimensions;
    }

    public boolean containsPoint(Point point) {
	if (point.x >= topLeftCoordinates.x && 
	    point.x <= (topLeftCoordinates.x+dimensions.width) && 
	    point.y >= topLeftCoordinates.y && 
	    point.y <= (topLeftCoordinates.y+dimensions.height)) {
	    return true;
	}
	return false;
    }

    public void setCoordinates(Point topLeftCoordinates) {
	this.topLeftCoordinates = new Point(topLeftCoordinates);
    }

    public Point getCoordinates() {
	return topLeftCoordinates;
    }

    public Point getUsesPortCoordinates(String usesPortName) {
	return ((PortDetails)usesPorts.get(usesPortName)).getCoordinates();
    }

    public Point getProvidesPortCoordinates(String providesPortName) {
	return ((PortDetails)providesPorts.get(providesPortName)).getCoordinates();
    }

    public void setUsesPortCoordinates(String usesPortName, Point coordinates) {
	PortDetails portDetails = (PortDetails)(usesPorts.get(usesPortName));
	portDetails.setCoordinates(coordinates);
    }

    public void setProvidesPortCoordinates(String providesPortName, Point coordinates) {
	PortDetails portDetails = (PortDetails)(providesPorts.get(providesPortName));
	portDetails.setCoordinates(coordinates);
    }

    public static boolean isCompatible(ComponentModel usesInstance, String usesPortName,
				       ComponentModel providesInstance, 
				       String providesPortName) {
	String usesPortType = ((PortDetails)usesInstance.usesPorts.get(usesPortName)).getType();
	String providesPortType = ((PortDetails)providesInstance.providesPorts.get(providesPortName)).getType();
	if (usesPortType.equals(providesPortType))
	    return true;
	return false;
    }

    public class PortDetails {
	private String type;
	private Point coordinates;
	
	public PortDetails(String type) {
	    this.type = type;
	}

	public void setCoordinates(Point coordinates) {
	    this.coordinates = coordinates;
	}

	public Point getCoordinates() {
	    return coordinates;
	}

	public String getType() {
	    return type;
	}
    }
}
