package servlet;

import connection.ConnectionService;
import ccacore.ComponentID;
import javax.servlet.http.HttpServletRequest;
import java.io.PrintWriter;
import java.io.IOException;
import java.util.Enumeration;

public class ConnectionRequestHandler {

    public static void handleConnectInitiation(HttpServletRequest request,
					       PrintWriter out,
					       WorkingAreaModel compositionModel,
					       ConnectionService connectionService) {
	// create a list of instances in the workspace, ask user
	// to select one from where the connection begins and one form where
	// the connection ends
	out.println("<html><head><title>Connect ports</title></head><body bgcolor=\"#FFFFFF\"><h3>Connect port</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\"METHOD=\"POST\">Name of instance the desired connection begins from:"+ServletComponent.getInstanceOptionList(compositionModel,"usesinstance")+"Name of instance the desired connection ends at:"+ServletComponent.getInstanceOptionList(compositionModel,"providesinstance")+"<INPUT TYPE=\"HIDDEN\" NAME=\"command\" VALUE=\"firstconnect\"><INPUT TYPE=\"SUBMIT\" NAME=\"submit\" VALUE=\"Connect / Browse valid connections\"></FORM><br><img src=\"/servlets/FreshImage\"><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
    }
    
    public static void handleConnectProcess(HttpServletRequest request,
					    PrintWriter out,
					    WorkingAreaModel compositionModel,
					    ConnectionService connectionService) {
	// get the instances involved in the connection and
	// get a list of possible connections between them
	// if there is only one, make the connection
	// and add it to the workspace; display main screen
	// otherwise display a list of possible connections and
	// ask the user to choose between them

	// generate a list of valid connections from the supplied provides instance
	// to the supplied uses instance
	String providesInstanceName = request.getParameter("providesinstance");
	ComponentModel providesInstance = 
	    compositionModel.getComponent(providesInstanceName);
	String usesInstanceName = request.getParameter("usesinstance");
	ComponentModel usesInstance = compositionModel.getComponent(usesInstanceName);
	
	// go through the list of ports on the provides instance and check 
	// compatibility with each port on the uses instance
	Enumeration usesPorts = usesInstance.getOutputPortNames();
	StringBuffer connectionList = new StringBuffer("<select name=\"connectionchoice\">");
	int numConnections = 0;
	String validUsesPortName = null;
	String validProvidesPortName = null;
	while(usesPorts.hasMoreElements()) {
	    // go through the provides ports
	    Enumeration providesPorts = providesInstance.getInputPortNames();
	    String usesPortName = (String)(usesPorts.nextElement());
	    while(providesPorts.hasMoreElements()) {
		String providesPortName = (String)(providesPorts.nextElement());
		// check for compatibility
		if(ComponentModel.isCompatible(usesInstance,usesPortName,
					       providesInstance,providesPortName)) {
		    // add to buffer for display in list
		    connectionList.append("<option name=\""+usesPortName+"-"+providesPortName+"\">"+usesPortName+" to "+providesPortName+"</option>");
		    // increment number of valid connections
		    numConnections++;
		    validUsesPortName = usesPortName;
		    validProvidesPortName = providesPortName;
		}
	    }
	    connectionList.append("</select><br>");
	}
	if (numConnections==1) {
	    // just make the connection between the valid uses port name
	    // and valid provides port name
	    makeConnection(usesInstanceName,validUsesPortName,
			   providesInstanceName,validProvidesPortName,
			   compositionModel,connectionService);
	    // update workspace and go back
	    try {
		compositionModel.drawSession();
		out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
	    } catch (Exception exception) {
		exception.printStackTrace();
	    }
	} else if (numConnections == 0) {
	    out.println("<html><head><title>Error</title></head><body bgcolor=\"#FFFFFF\"><h3>No compatible ports</h3><p>Sorry, no valid connection can be made from "+usesInstanceName+" to "+providesInstanceName+"</p><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
	} else {
	    // we have multiple valid connections, display the option list
	    out.println("<html><head><title>Choose between valid connections</title></head><body bgcolor=\"#FFFFFF\"><h3>Choose between valid connections</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\"METHOD=\"POST\">Valid connections:"+connectionList+"<BR><INPUT TYPE=\"HIDDEN\" NAME=\"command\" VALUE=\"midconnect\"><INPUT TYPE=\"HIDDEN\" NAME=\"usesinstance\" VALUE=\""+usesInstanceName+"\"><INPUT TYPE=\"HIDDEN\" NAME=\"providesinstance\" VALUE=\""+providesInstanceName+"\"><INPUT TYPE=\"SUBMIT\" NAME=\"submit\" VALUE=\"Connect\"></FORM><br><img src=\"/servlets/FreshImage\"><b><br><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
	}
    }

    public static void handleConnectCompletion(HttpServletRequest request,
						  PrintWriter out,
						  WorkingAreaModel compositionModel,
						  ConnectionService connectionService) 
	throws IOException {
	// get the connection to be made, make it, add it to the workspace
	// display main screen

	// grab the information needed to make the desired connection
	String usesInstanceName = request.getParameter("usesinstance");
	String providesInstanceName = request.getParameter("providesinstance");
	String chosenConnection = request.getParameter("connectionchoice");
	// extract uses and provides port names from the connection choice
	int separator = chosenConnection.indexOf("-");
	String usesPortName = chosenConnection.substring(0,separator);
	String providesPortName = chosenConnection.substring(separator+1);
	makeConnection(usesInstanceName,usesPortName,
		       providesInstanceName,providesPortName,
		       compositionModel,connectionService);
	compositionModel.drawSession();
	out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
    }

    public static void handleDisconnectInitiation(HttpServletRequest request,
						  PrintWriter out,
						  WorkingAreaModel compositionModel,
						  ConnectionService connectionService) {
	// create a list of connections in the workspace, ask user
	// to select the one to remove

	// make a list of connections
	StringBuffer sb = new StringBuffer("<select name=\"connectionchoice\">");
	// get all connections
	Enumeration connections = compositionModel.getConnections();
	// go through all connections and add to option list
	while(connections.hasMoreElements()) {
	    ConnectionModel currentConnection = (ConnectionModel) connections.nextElement();
	    String connectionId = currentConnection.getId();
	    ComponentModel usesInstance = currentConnection.getUsesInstance();
	    ComponentModel providesInstance = currentConnection.getProvidesInstance();
	    String usesInstanceName = usesInstance.getName();
	    String providesInstanceName = providesInstance.getName();
	    String usesPortName = currentConnection.getUsesPortName();
	    String providesPortName = currentConnection.getProvidesPortName();
	    sb.append("<option value=\""+connectionId+"\">"+usesInstanceName+" port "+usesPortName+" to "+providesInstanceName+" port "+providesPortName+"</option>");
	}
	sb.append("</select><br>");
	// generate the form for the user
	out.println("<html><head><title>Choose connection to remove</title></head><body bgcolor=\"#FFFFFF\"><h3>Choose connection to remove</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\"METHOD=\"POST\">Valid connections:"+sb+"<BR><INPUT TYPE=\"HIDDEN\" NAME=\"command\" VALUE=\"middisconnect\"><INPUT TYPE=\"SUBMIT\" NAME=\"submit\" VALUE=\"Disconnect\"></FORM><br><img src=\"/servlets/FreshImage\"><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
    }
    
    public static void handleDisconnectCompletion(HttpServletRequest request,
						  PrintWriter out,
						  WorkingAreaModel compositionModel,
						  ConnectionService connectionService) 
	throws IOException {
	// get the connection ot be removed, use the connection service
	// to disconnect
	// remove connection from the workspace  
	// display main screen

	// get the connection id
	String connectionID = request.getParameter("connectionchoice");
	// get the connection
	ConnectionModel connection = compositionModel.getConnection(connectionID);
	// remove the connection
	removeConnection(connection,compositionModel,connectionService);
	compositionModel.drawSession();
	out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
    }

    private static String makeConnection(String usesInstanceName, String usesPortName,
					 String providesInstanceName, String providesPortName,
					 WorkingAreaModel workingAreaModel, 
					 ConnectionService connectionService) {
	// look up the working area model and search for the instances
	// involved in the connection
	ComponentID usesID = workingAreaModel.getComponentID(usesInstanceName);
	if (usesID==null) {
	    return "Could not find instance named "+usesInstanceName;
	}
	ComponentID providesID = workingAreaModel.getComponentID(providesInstanceName);
	if (providesID==null) {
	    return "Could not find instance named "+providesInstanceName;
	}
	try {
	    // use the connection service to make the connection
	    connectionService.connect(usesID,usesPortName,providesID,providesPortName);
	    // if exception thrown, return failure string
	    // add a connection model to the list of connections
	    ConnectionModel newConnection  = new ConnectionModel(workingAreaModel.getComponent(usesInstanceName),
								 usesPortName,
								 workingAreaModel.getComponent(providesInstanceName),
								 providesPortName);
	    workingAreaModel.addConnection(newConnection);
	} catch (Exception exception) {
	    System.out.println("Got connection exception while connecting instance "+
			       usesInstanceName+" port "+usesPortName+" to instance "+
			       providesInstanceName+" port "+providesPortName);
	    exception.printStackTrace();
	    return "Could not connect, perhaps your port names are wrong?";
	}
	// return success 
	return null;
    }

    private static String removeConnection(ConnectionModel connection,
					   WorkingAreaModel workingAreaModel, 
					   ConnectionService connectionService) {
	ComponentModel usesInstance = connection.getUsesInstance();
	ComponentModel providesInstance = connection.getProvidesInstance();
	String usesPortName = connection.getUsesPortName();
	String providesPortName = connection.getProvidesPortName();
	ComponentID usesID = usesInstance.getComponentID();
	ComponentID providesID = providesInstance.getComponentID();
	try {
	    // use the connection service to remove the connection
	    connectionService.disconnect(usesID,usesPortName,providesID,providesPortName,5);
	    // remove the connection form the workspace model
	    workingAreaModel.removeConnection(connection);
	} catch (Exception exception) {
	    System.out.println("Got connection exception while disconnecting instance "+
			       usesInstance.getName()+" port "+usesPortName+" from instance "+
			       providesInstance.getName()+" port "+providesPortName);
	    exception.printStackTrace();
	    return "Could not disconnect";
	}
	// return success 
	return null;
    }
}
