package servlet;

import compositiontool.view.ComponentCommunicator;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.IOException;
import java.lang.reflect.Field;
import util.Info;
import ccacore.ComponentID;

public class ControlRequestHandler {

    public static void handleControlInitiation(HttpServletRequest request,
					       PrintWriter out,
					       WorkingAreaModel compositionModel,
					       ComponentCommunicator componentCommunicator,
					       String instanceName) {
	out.println("<html><head><title>Instance Controls</title></head><body bgcolor=\"#FFFFFF\"><h3>Instance Controls for "+instanceName+"</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\" METHOD=\"POST\"><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"midstart\"><a href=\"../options.html#start\"> Start up instance</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"midkill\"><a href=\"../options.html#kill\"> Kill</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"firstsetparams\"><a href=\"../options.html#setparams\"> Set parameters</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"getoutput\"><a href=\"../options.html#getoutput\"> Get output</a><br><INPUT TYPE=\"HIDDEN\" NAME=\"instance\" VALUE=\""+instanceName+"\"><INPUT TYPE=\"SUBMIT\" NAME=\"submit\" VALUE=\"Invoke control method\"></FORM><br><img src=\"/servlets/FreshImage\"><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
    }

    public static void handleSetParameterForInstance(HttpServletRequest request,
						     PrintWriter out,
						     WorkingAreaModel compositionModel,
						     ComponentCommunicator 
						     componentCommunicator) {
	// get a list of default parameters for the instance specified
	// display default parameters and allow editing

	// get the instance whose params are to be set
	String instanceName = request.getParameter("instance");
	// get the component model
	ComponentModel component = compositionModel.getComponent(instanceName);
	// get the fields and defaults from the info
	Info instanceInfo = component.getInfo();
	Info [] paramList = instanceInfo.getInfo("instance-info")[0].getInfo("parameter-block")[0].getInfo("parameters");
	// display a set of text fields with default values in them
	out.println("<html><head><title>Update default parameters</title></head><body bgcolor=\"#FFFFFF\"><h3>Update default parameter values for "+instanceName+"</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\" METHOD=\"POST\">");
	for (int i=0; i<paramList.length; i++) {
	    out.println(paramList[i].getValue("name")[0]+": <INPUT TYPE=\"TEXT\" NAME=\""+paramList[i].getValue("name")[0]+"\" VALUE=\""+paramList[i].getValue("default-value")[0]+"\"><BR>");
	}
	out.println("<INPUT TYPE=\"HIDDEN\" NAME=\"instance\" VALUE=\""+instanceName+"\"><INPUT TYPE=\"HIDDEN\" NAME=\"command\" VALUE=\"finalsetparams\"><INPUT TYPE=\"SUBMIT\" NAME=\"submit\" VALUE=\"Set parameters\"></FORM><br><img src=\"/servlets/FreshImage\"><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
    }

    public static void handleSetParameterCompletion(HttpServletRequest request,
						    PrintWriter out,
						    WorkingAreaModel compositionModel,
						    ComponentCommunicator 
						    componentCommunicator) {
	// get the name of the instance and create an object of
	// its parameter class
	// populate fields with edited values
	// invoke method to set parameter on the instance
	// display main screen

	// get the name of the instance
	String instanceName = request.getParameter("instance");
	// get the component model
	ComponentModel component = compositionModel.getComponent(instanceName);
	System.out.println("Setting params for "+instanceName);
	ComponentID id = component.getComponentID();
	// get the name of the parameter class
	String paramClassName = component.getParameterClassName();
	// get the name of the parameter port class
	String paramPortClassName = component.getParameterPortClassName();
	// get the name of the parameter port
	String paramPortName = component.getParameterPortName();
	// get the name of the method to invoke
	String paramMethodName = component.getInfo().getInfo("instance-info")[0].getInfo("parameter-block")[0].getValue("method-name")[0];
	// load the param class
	try {
	    Class paramClass = Class.forName(paramClassName);
	    // create an instance of this class
	    Object paramObject = paramClass.newInstance();
	    // get its fields
	    Field [] classFields = paramClass.getDeclaredFields();
	    // look up the values of the fields on the form and populate this
	    // object
	    for (int i=0; i<classFields.length; i++) {
		String className = classFields[i].getType().getName();
		String fieldName = classFields[i].getName();
		String enteredValue = request.getParameter(fieldName);
		Object value = null;
		if (className.equals("boolean")) {
		    // create a Boolean object
		    value = Boolean.valueOf(enteredValue);
		} else if (className.equals("double")) {
		    try {
			// create a Double object
			value = Double.valueOf(enteredValue);
		    } catch (NumberFormatException exception) {
			System.out.println("Could not convert param value "+enteredValue+" into a Double");
			exception.printStackTrace();
			value = new Double(0.0d);
		    }
		} else if (className.equals("float")) {
		    // create a Float object
		    try {
			value = Float.valueOf(enteredValue);
		    } catch (NumberFormatException exception) {
			System.out.println("Could not convert param value "+enteredValue+" into a Float");
			exception.printStackTrace();
			value = new Float(0.0f);
		    }
		} else if (className.equals("int")) {
		    // create an Integer object
		    try {
			value = Integer.valueOf(enteredValue);
		    } catch (NumberFormatException exception) {
			System.out.println("Could not convert param value "+enteredValue+" into an Integer");
			exception.printStackTrace();
			value = new Integer(0);
		    }
		} else if (className.equals("long")) {
		    // create a Long object
		    try {
			value = Long.valueOf(enteredValue);
		    } catch (NumberFormatException exception) {
			System.out.println("Could not convert param value "+enteredValue+" into a Long");
			exception.printStackTrace();
			value = new Long(0);
		    }
		} else if (className.equals("java.lang.String")) {
		    value = enteredValue;
		} else {
		    System.out.println("WARNING >>> Could not recognize parameter type "+className+" for parameter field "+fieldName);
		}
		// now put the created object into the param object, in 
		// the appropriate field
		Field thisParamField = paramClass.getDeclaredField(fieldName);
		thisParamField.set(paramObject,value);
		System.out.println("Set "+fieldName+" to "+enteredValue);
	    }
	    System.out.println("Prepared param object, calling set on instance");
	    // param object now has all the required fields, send it over!
	    componentCommunicator.setComponentParams(id,paramPortClassName,
						     paramPortName,
						     paramMethodName,
						     instanceName,
						     paramObject);
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
	// go back to main page
	out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
    }

    public static void handleKillCompletion(HttpServletRequest request,
					    PrintWriter out,
					    WorkingAreaModel compositionModel,
					    ComponentCommunicator componentCommunicator) 
	throws IOException {
	// kill the specified instance
	// display main screen

	// get the name of the instance to kill
	String instanceName = request.getParameter("instance");
	// kill it!
	ComponentModel model = compositionModel.getComponent(instanceName);
	final ComponentCommunicator communicator = componentCommunicator;
	if (model!=null) {
	    final ComponentModel current = model;
	    new Thread() {
		public void run() {
		    String paramPortName = current.getParameterPortName();
		    String paramClassName = current.getParameterPortClassName();
		    communicator.killComponent(current.getComponentID(),
					       paramClassName,
					       paramPortName,
					       current.getName());
		}
	    }.start();
	    compositionModel.removeComponent(model);
	    compositionModel.drawSession();
	}
	out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
    }
    
    public static void handleStartCompletion(HttpServletRequest request,
					     PrintWriter out,
					     WorkingAreaModel compositionModel,
					     ComponentCommunicator componentCommunicator) {
	// start the specified instance
	// display main screen

	// get the name of the instance to start
	String instanceName = request.getParameter("instance");
	// start 'er up!
	ComponentModel model = compositionModel.getComponent(instanceName);
	if (model!=null) {
	    String paramPortName = model.getParameterPortName();
	    String paramClassName = model.getParameterPortClassName();
	    componentCommunicator.startComponent(model.getComponentID(),
						 paramClassName,
						 paramPortName,
						 model.getName());
	}
	out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
    }

    public static void handleGetOutputCompletion(HttpServletRequest request,
						 HttpServletResponse response,
						 WorkingAreaModel compositionModel,
						 ComponentCommunicator componentCommunicator) 
    throws IOException {
	// get the output URL from the instance
	// display that URL

	// get the name of the instance whose output is required
	String instanceName = request.getParameter("instance");
	// get the instance
	ComponentModel model = compositionModel.getComponent(instanceName);
	if (model!=null) {
	    // check if we already know the URL
	    String urlString = model.getOutputURL();
	    if (urlString == null) {
		// invoke the method and get the return value
		String paramPortName = model.getParameterPortName();
		String paramClassName = model.getParameterPortClassName();
		urlString = componentCommunicator.getComponentOutputURL
		    (model.getComponentID(),
		     paramClassName,
		     paramPortName,
		     model.getName());
		model.setOutputURL(urlString);
	    }
	    response.sendRedirect(urlString);
	}
    }
}
