package servlet;

import javax.servlet.http.HttpServletRequest;
import java.io.PrintWriter;
import ids.InformationDirectoryContext;
import ids.InformationDirectoryNode;
import ids.InformationInfo;
import registry.InstanceXMLUtils;
import registry.InstanceInfo;
import util.EnvObj;
import util.Info;
import ids.InformationXMLUtils;
import creation.CreationService;
import ccacore.ComponentID;

public class IDSRequestHandler {
    
    public static void handleIDSLookup(HttpServletRequest request,
				       PrintWriter out,
				       WorkingAreaModel compositionModel,
				       InformationDirectoryContext idsContext) {
	// make a list of component names under this context
	// allow user to view or create

	try {
	    // lookup the context to get all the children
	    InformationDirectoryNode [] children = idsContext.getChildren();
	    // create an option list of component information here
	    // and a list of subcontexts
	    StringBuffer componentList = new StringBuffer("<select name=\"componentchoice\" size=\"3\">");
	    StringBuffer contextList = new StringBuffer("<select name=\"subcontext\"> size=\"3\"");
	    String contextName = idsContext.getName();
	    for (int i=0; i<children.length; i++) {
		InformationDirectoryNode currentNode = children[i];
		// get the stripped name (i.e. name relative to this context)
		String fullName = currentNode.getName();
		String strippedName = fullName.substring(contextName.length());
		// check if this is component information
		if (currentNode.isInformation()==1) {
		    // this corresponds to component information
		    componentList.append("<option name=\""+strippedName+"\">"+strippedName+"</option>");
		} else if(currentNode.isContext()==1) {
		    // this corresponds to a subcontext
		    contextList.append("<option name=\""+strippedName+"\">"+strippedName+"</option>");
		}
	    }
	    componentList.append("</select><br>");
	    contextList.append("</select><br>");
	    // now show these two lists along with possible actions
	    out.println("<html><head><title>Information Directory Lookup</title></head><body bgcolor=\"#FFFFFF\"><h3>Information Directory lookup</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\"METHOD=\"POST\">"+componentList+"<INPUT TYPE=\"SUBMIT\" NAME=\"command\" VALUE=\"View component information\"><INPUT TYPE=\"SUBMIT\" NAME=\"command\" VALUE=\"Create component\"><br>"+contextList+"<INPUT TYPE=\"SUBMIT\" NAME=\"command\" VALUE=\"Enter IDS subcontext\"></FORM><br><img src=\"/servlets/FreshImage\"><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }


    public static void handleIDSCreate(HttpServletRequest request,
				       PrintWriter out,
				       WorkingAreaModel compositionModel,
				       InformationDirectoryContext idsContext,
				       CreationService creationPort) {
	// create the selected component information and add to workspace
	// display main screen to user

	// get the name of the component to be created
	String componentName = request.getParameter("componentchoice");
	try {
	    // lookup this information in the current context
	    InformationInfo componentInfo = (InformationInfo) idsContext.lookup(componentName);
	    // get the machine name and creration mechanism of the first installation
	    Info [] installations = 
		componentInfo.getInfo("component-info")[0].getInfo("installation");
	    if (installations == null) {
		// no installations! can't create
		System.out.println("Did not find any installations");
	    } else {
		// get the host and creation mechanism of the first installation
		String hostName = installations[0].getValue("host-name")[0];
		String creationMech = installations[0].getInfo("creation-info")[0].
		    getValue("creation-method")[0];
		System.out.println("Will create component on "+hostName+" using "+
				   creationMech);
		EnvObj creationEnv = 
		    componentInfo.getCreationEnvironment(hostName,creationMech);
		// add a dummy id to the creation env to be used so we can identify 
		// the creation event 
		String instanceId = creationEnv.get("user-name")+creationEnv.get("creation-time")+Math.random();
		creationEnv.put("instance-name",instanceId);
		// create the instance
		ComponentID id = creationPort.createInstance(creationEnv);
		// create instance information for this component
		Info [] paramInfoArray = componentInfo.getInfo("component-info")[0].getInfo("parameter-block");
		Info paramInfo = null;
		if (paramInfoArray.length != 0)
		    paramInfo = paramInfoArray[0];
		String xmlDescription = 
		    InstanceXMLUtils.createXMLFromInfo(id,componentInfo.getName(),
						       creationEnv,
						       componentInfo);
		InstanceInfo instanceInfo = InstanceXMLUtils.createInstanceInfoFromXML(xmlDescription);
		// create a component model for this instance
		ComponentModel componentModel = new ComponentModel(instanceInfo);
		// add this to the workspace
		compositionModel.addComponent(componentModel);
	    }
	    compositionModel.drawSession();
	    out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
	} catch (Exception exception) {
	    System.out.println("Could not create "+componentName+
			       ", got exception "+exception);
	    exception.printStackTrace();
	}
    }


    public static void handleIDSView(HttpServletRequest request,
				     PrintWriter out,
				     WorkingAreaModel compositionModel,
				     InformationDirectoryContext idsContext) {
	// display selected component information to user
	// get the name of the component to be created
	String componentName = request.getParameter("componentchoice");
	try {
	    // lookup this information in the current context
	    InformationInfo componentInfo = (InformationInfo) idsContext.lookup(componentName);
	    // get the pretty string describing this information
	    String xmlDescription = InformationXMLUtils.createXMLFromInfo(componentInfo,
									  componentInfo.getName());
	    StringBuffer newBuffer = new StringBuffer();
	    // replace < with &lt; and > with &gt; in the XML
	    // replace carriage returns with <br> tags
	    for (int i=0; i<xmlDescription.length(); i++) {
		char currentChar = xmlDescription.charAt(i);
		// check if this is the < or >
		if (currentChar == '<') {
		    // replace it with &lt;
		    newBuffer.append("&lt;");
		} else if (currentChar == '>') {
		    // replace it with $gt;
		    newBuffer.append("&gt;");
		} else if (currentChar == '\n') {
		    newBuffer.append("<br>\n");
		} else {
		    newBuffer.append(currentChar);
		}
	    }
	    // display this
	    out.println("<html><head><title>Component description for "+componentName+"</title></head><body bgcolor=\"#FFFFFF\"><h3>Component description for "+componentName+"</h3><p>If you need to copy this information to email it or otherwise use it for creating a component, please copy only the portion <em>between</em> the horizontal lines since the component information is required to be in that format (extraneous text will confuse programs interpreting it).</p><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b><br><hr><br>"+newBuffer+"<br><hr></body></html>");
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }
}
