package servlet;

import javax.servlet.http.HttpServletRequest;
import java.io.PrintWriter;
import registry.RegistryContext;
import registry.InstanceInfo;
import registry.RegistryNode;
import registry.InstanceXMLUtils;
import ccacore.ComponentID;

public class RegistryRequestHandler {
    
    public static void startBind(HttpServletRequest request, PrintWriter out,
				 WorkingAreaModel compositionModel, 
				 RegistryContext registryContext) {
	// display a list of instances which can be bound
	// and a text field for the binding name
	out.println("<html><head><title>Bind instance</title></head><body bgcolor=\"#FFFFFF\"><h3>Set instance parameters</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\"METHOD=\"POST\">Instance to bind:"+ServletComponent.getInstanceOptionList(compositionModel,"instance")+"<BR><INPUT TYPE=\"HIDDEN\" NAME=\"command\" VALUE=\"registrymidbind\">Binding name:<INPUT TYPE=\"TEXT\" NAME=\"bindingname\"><br><INPUT TYPE=\"SUBMIT\" NAME=\"submit\" VALUE=\"Bind instance\"></FORM><br><img src=\"/servlets/FreshImage\"><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
    }

    public static void completeBind(HttpServletRequest request, PrintWriter out,
				    WorkingAreaModel compositionModel, 
				    RegistryContext registryContext) {
	// bind the specified instance into the context
	// under the specified name, display main screen to user
	
	// get the instance to be bound
	String instanceName = request.getParameter("instance");
	// get the binding name
	String bindingName = request.getParameter("bindingname");
	// get the instance info for this instance
	InstanceInfo info = compositionModel.getComponent(instanceName).getInfo();
	// bind it!
	try {
	    registryContext.bind(info,bindingName);
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
	// display main screen
	out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
    }

    public static void startUnbind(HttpServletRequest request, PrintWriter out,
				   WorkingAreaModel compositionModel, 
				   RegistryContext registryContext) {
	// display a list of instances whose binding can be removed
	// from the current context

	// lookup the current context and get a list of instances bound here
	// lookup the context to get all the children
	try {
	    RegistryNode [] children = registryContext.getChildren();
	    // create an option list of instance information here
	    StringBuffer instanceList = new StringBuffer("<select name=\"instancechoice\" size=\"3\">");
	    String contextName = registryContext.getName();
	    for (int i=0; i<children.length; i++) {
		RegistryNode currentNode = children[i];
		// get the stripped name, i.e. the name relative to this context
		String fullName = currentNode.getName();
		String strippedName = fullName;
		if (fullName.indexOf("/")!=-1)
		    strippedName = fullName.substring(contextName.length());
		// check if this is instance information
		if (currentNode.isInstance()==1) {
		    // this corresponds to instance information
		    instanceList.append("<option name=\""+strippedName+"\">"+strippedName+"</option>");
		} else if (currentNode.isContext()==1) {
		    RegistryContext context = (RegistryContext) currentNode;
		    RegistryNode [] contextChildren = context.getChildren();
		    int j=0;
		    for (j=0; j<contextChildren.length; j++) {
			if (contextChildren[j].isContext()==1)
			    break;
		    }
		    if (j==contextChildren.length) { // yucky code
			// aha, this context has just instance information
			instanceList.append("<option name=\""+strippedName+"\">"+strippedName+"</option>");
		    }
		}
	    }
	    instanceList.append("</select><br>");
	    // now show this list along with an unbind action button
	    out.println("<html><head><title>Unbind from registry</title></head><body bgcolor=\"#FFFFFF\"><h3>Unbind from registry</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\"METHOD=\"POST\">"+instanceList+"<INPUT TYPE=\"HIDDEN\" NAME=\"command\" VALUE=\"registrymidunbind\"><br><INPUT TYPE=\"SUBMIT\" NAME=\"submit\" VALUE=\"Unbind\"></FORM><br><img src=\"/servlets/FreshImage\"><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    public static void completeUnbind(HttpServletRequest request, PrintWriter out,
				      WorkingAreaModel compositionModel, 
				      RegistryContext registryContext) {
	// remove the binding of the specified instance from the context
	// get the instance which was to be unbound
	String instanceName = request.getParameter("instancechoice");
	// look up this instance
	try {
	    InstanceInfo [] instances = registryContext.lookup(instanceName);
	    // unbind the first one if there are multiple instances bound
	    // under this name
	    ComponentID id = instances[0].getComponentID();
	    // unbind
	    registryContext.unbind(id);
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
	// display main screen
	out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
    }

    public static void handleRegistryView(HttpServletRequest request, PrintWriter out,
					  WorkingAreaModel compositionModel, 
					  RegistryContext registryContext) {
	// display the instance information for the specified instance
	// in this context
	String instanceName = request.getParameter("instancechoice");
	try {
	    // lookup this information in the current context
	    InstanceInfo instanceInfo = registryContext.lookup(instanceName)[0];
	    // get the pretty string describing this information
	    String xmlDescription = InstanceXMLUtils.createXMLFromInfo(instanceInfo,
								       instanceInfo.getName());
	    StringBuffer newBuffer = new StringBuffer();
	    // replace < with &lt; and > with &gt; in the XML
	    // replace carriage returns with <br> tags
	    for (int i=0; i<xmlDescription.length(); i++) {
		char currentChar = xmlDescription.charAt(i);
		// check if this is the < or >
		if (currentChar == '<') {
		    // replace it with &lt;
		    newBuffer.append("&lt;");
		} else if (currentChar == '>') {
		    // replace it with $gt;
		    newBuffer.append("&gt;");
		} else if (currentChar == '\n') {
		    newBuffer.append("<br>\n");
		} else {
		    newBuffer.append(currentChar);
		}
	    }
	    // display this
	    out.println("<html><head><title>Instance description for "+instanceName+"</title></head><body bgcolor=\"#FFFFFF\"><h3>Instance description for "+instanceName+"</h3><p>If you need to copy this information to email it or otherwise use it for copying an instance, please copy only the portion <em>between</em> the horizontal lines since the instance information is required to be in that format (extraneous text will confuse programs interpreting it).</p><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b><br><hr><br>"+newBuffer+"<br><hr></body></html>");
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }
    
    public static void handleRegistryLookup(HttpServletRequest request, PrintWriter out,
					    WorkingAreaModel compositionModel, 
					    RegistryContext registryContext) {
	// lookup the specified context and generate a list of instances for
	// the user, along with a list of subcontexts
	// allow the user to view or copy instances as well as 
	// enter subcontexts
	
	try {
	    // lookup the context to get all the children
	    RegistryNode [] children = registryContext.getChildren();
	    // create an option list of instance information here
	    // and a list of subcontexts
	    StringBuffer instanceList = new StringBuffer("<select name=\"instancechoice\" size=\"3\">");
	    StringBuffer contextList = new StringBuffer("<select name=\"subcontext\"> size=\"3\"");
	    String contextName = registryContext.getName();
	    for (int i=0; i<children.length; i++) {
		RegistryNode currentNode = children[i];
		// get the stripped name, i.e. the name relative to this context
		String fullName = currentNode.getName();
		String strippedName = fullName;
		if (fullName.indexOf("/")!=-1)
		    strippedName = fullName.substring(contextName.length());
		// check if this is instance information
		if (currentNode.isInstance()==1) {
		    // this corresponds to instance information
		    instanceList.append("<option name=\""+strippedName+"\">"+strippedName+"</option>");
		} else if(currentNode.isContext()==1) {
		    // this corresponds to a subcontext
		    // check if this subcontext has only instance information
		    // if so, just treat this as an instance
		    // this is temporary
		    // TODO: provide a general, intuitive way of looking at multiple 
		    // instances bound under the same name
		    RegistryContext context = (RegistryContext) currentNode;
		    RegistryNode [] contextChildren = context.getChildren();
		    int j=0;
		    for (j=0; j<contextChildren.length; j++) {
			if (contextChildren[j].isContext()==1)
			    break;
		    }
		    if (j==contextChildren.length) { // yucky code
			// aha, this context has just instance information
			instanceList.append("<option name=\""+strippedName+"\">"+strippedName+"</option>");
		    } else {
			contextList.append("<option name=\""+strippedName+"\">"+strippedName+"</option>");
		    }
		}
	    }
	    instanceList.append("</select><br>");
	    contextList.append("</select><br>");
	    // now show these two lists along with possible actions
	    out.println("<html><head><title>Registry Lookup</title></head><body bgcolor=\"#FFFFFF\"><h3>Registry lookup</h3><FORM ACTION=\"/servlets/servlet.ServletComponent\"METHOD=\"POST\">"+instanceList+"<INPUT TYPE=\"SUBMIT\" NAME=\"command\" VALUE=\"View instance information\"><INPUT TYPE=\"SUBMIT\" NAME=\"command\" VALUE=\"Copy instance\"><br>"+contextList+"<INPUT TYPE=\"SUBMIT\" NAME=\"command\" VALUE=\"Enter registry subcontext\"></FORM><br><img src=\"/servlets/FreshImage\"><br><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    public static void handleRegistryCopy(HttpServletRequest request, PrintWriter out,
					  WorkingAreaModel compositionModel, 
					  RegistryContext registryContext) {
	// copy the specified instance in this context and create a component
	// model from it
	// add this to the workspace
	// give user the main screen

	// get the name of the instance to be copied
	String instanceName = request.getParameter("instancechoice");
	// look it up to get the info
	// assume there is only one instance bound under this name
	try {
	    InstanceInfo info = registryContext.lookup(instanceName)[0];
	    // create a component model for this instance
	    ComponentModel componentModel = new ComponentModel(info);
	    // add this to the workspace
	    compositionModel.addComponent(componentModel);
	    compositionModel.drawSession();
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
	out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
    }
}
