
package servlet;

import ccacore.Component;
import ccacore.Services;
import ccacore.ServicesImpl;
import ccacore.ComponentID;
import ids.InformationDirectoryService;
import ids.InformationDirectoryContext;
import ids.InformationInfo;
import util.EnvObj;
import util.Info;
import creation.CreationService;
import connection.ConnectionService;
import registry.RegistryService;
import registry.RegistryContext;
import registry.InstanceXMLUtils;
import registry.InstanceInfo;
import compositiontool.view.ComponentCommunicator;
import compositiontool.view.ComponentCommunicatorImplementation;
import nexusrmi.RemoteException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.File;
import java.awt.Point;
import javax.servlet.*;
import javax.servlet.http.*;
import java.lang.reflect.Field;
import java.util.Enumeration;

public class ServletComponent extends HttpServlet implements Component  {
    private Services myServices;
    private CreationService creationPort;
    private ConnectionService connectionPort;
    private RegistryService registryPort;
    private InformationDirectoryService idsPort;
    private ComponentCommunicator componentCommunicator;

    // things required by the component interface
    public void setServices(Services s) {
	myServices = s;
    }

    public void init() {
	try {
	    super.init();
	    Services services = new ServicesImpl();
	    // set the component services
	    this.setServices(services);
	    // get the creation port
	    creationPort = (CreationService) myServices.getPort("CreationService");
	    // get the connection port
	    connectionPort = (ConnectionService) myServices.getPort("ConnectionService");
	    // get the IDS port
	    idsPort = (InformationDirectoryService) myServices.getPort("InformationDirectoryService");
	    // get the registry port
	    registryPort = (RegistryService) myServices.getPort("RegistryService");
	    // start up the component communicator
	    componentCommunicator = new ComponentCommunicatorImplementation();
	    ((ComponentCommunicatorImplementation)componentCommunicator).setServices(services);
	    // ready for requests
	    // set the output stream to a log file
	    PrintStream logStream = new PrintStream(new FileOutputStream("/tmp/servletlog"));
	    System.setOut(logStream);
	    System.setErr(logStream);
	    System.out.println("Servlet component ready for requests");
	} catch (Exception exception) {
	    System.out.println("Got exception "+exception);
	}
    }

    public void sessionInit(HttpServletRequest request, HttpSession session) {
	try {
	    // create a composition model
	    System.out.println("Initializing session "+session.getId());
	    System.out.println("Creating workspace model...");
	    WorkingAreaModel compositionModel = new WorkingAreaModel(session.getId());
	    session.putValue("workingArea",compositionModel);
	    compositionModel.drawSession();
	    // create an information directory context
	    String idsContextString = request.getParameter("idslocation");
	    System.out.println("Creating default IDS context "+idsContextString);
	    InformationDirectoryContext idsContext = idsPort.getContext(idsContextString);
	    System.out.println("Created IDS context");
	    session.putValue("idsContext",idsContext);
	    session.putValue("idsLocation",idsContextString);
	    // create a registry context
	    String registryContextString = request.getParameter("registrylocation");
	    System.out.println("Creating default registry context "+registryContextString);
	    RegistryContext registryContext = registryPort.getContext(registryContextString);
	    System.out.println("Created registry context");
	    session.putValue("registryContext",registryContext);
	    session.putValue("registryLocation",registryContextString);
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }


    public static String getInstanceOptionList(WorkingAreaModel workSpace, 
					       String selectionName) {
	// buffer to hold stuff
	StringBuffer sb = new StringBuffer();
	sb.append("<select name=\""+selectionName+"\">");
	// go through the workspace and get all instance names
	Enumeration components = workSpace.getComponents();
	// go through components and get their names
	while(components.hasMoreElements()) {
	    String componentName = ((ComponentModel)components.nextElement()).getName();
	    sb.append("<option>"+componentName+"</option>");
	}
	sb.append("</select><br>");
	return sb.toString();
    }

    // handle an incoming request
    // this will check the command and call a function which will service
    // the client as requested
    private void handleRequest(HttpServletRequest request, HttpServletResponse response,
			       WorkingAreaModel compositionModel) 
	throws IOException {
	PrintWriter out = response.getWriter();
	String command = request.getParameter("command");
	System.out.println("Handling command "+command+" from session "+
			   compositionModel.getSessionID());
	// connection commands
	// three screens:
	// 1. connect selected on main page, action taken is that user
	//    is asked for instances involved in connection
	// 2. user provides instances name, action taken is that possible
	//    connections are checked, if none user gets error, if one
	//    the connection is made, if multiple possibilities user is 
	//    asked to select
	// 3. user selects one among many possible connections, action taken
	//    is that the connection is made, user gets main page
	if (command.equals("connect")) {
	    // call method to begin connection process
	    ConnectionRequestHandler.handleConnectInitiation(request,out,
							     compositionModel,
							     connectionPort);
	} else if (command.equals("firstconnect")) {
	    // call method to get instances involved in connection
	    // and connect if only one possibility
	    ConnectionRequestHandler.handleConnectProcess(request,out,
							  compositionModel,
							  connectionPort);
	} else if (command.equals("midconnect")) {
	    // call method to complete the connection
	    ConnectionRequestHandler.handleConnectCompletion(request,out,
							     compositionModel,
							     connectionPort);
	} 
	// note that if-else is continued below
	// set parameter commands
	// two screens:
	// 1. user specifies name of instance, action taken is that
	//    instance information is looked up for parameter fields 
	//    and default values and these are presented to the user 
	//    for editing
	// 2. user edits parameters and sets them, action taken is
	//    that parameter object is constructed using these fields
	//    and method is invoked on instance, user gets main page
	else if (command.equals("firstsetparams")) {
	    // call method to get default parameters for instance
	    // and display them
	    ControlRequestHandler.handleSetParameterForInstance(request,
								out,compositionModel,
								componentCommunicator);
	} else if (command.equals("finalsetparams")) {
	    // set the parameters 
	    ControlRequestHandler.handleSetParameterCompletion(request,out,
							       compositionModel,
							       componentCommunicator);
	}
	// note that if-else is continued below
	// kill commands
	// one screen:
	// 1. user specifies name, action taken is that kill method is invoked
	//    on the instance and user gets main page
	else if (command.equals("midkill")) {
	    // call method to invoke kill on instance
	    ControlRequestHandler.handleKillCompletion(request,out,
						       compositionModel,
						       componentCommunicator);
	}
	// note that if-else is continued below
	// start commands
	// one screen:
	// 1. user specifies name, action taken is that start method is invoked
	//    on the instance and user gets main page
	else if (command.equals("midstart")) {
	    // call method to invoke start on instance
	    ControlRequestHandler.handleStartCompletion(request,out,
							compositionModel,
							componentCommunicator);
	}
	// note that if-else is continued below
	// get output command
	// one screen
	// 1. user has specified the name of the instance whose output is
	// required; action taken is that getOutputURL is invoked and that URL
	// is displayed
	else if (command.equals("getoutput")) {
	    // call method to invoke getOutputURL on instance
	    ControlRequestHandler.handleGetOutputCompletion(request,response,
							    compositionModel,
							    componentCommunicator);
	}
	// note that if-else is continued below
	// disconnection commands
	// two screens:
	// 1. user selects disconnect on main page, action taken is
	//    that user is asked to select from a list of connections
	// 2. user selects a connection, action taken is that connection
	//    is removed and user gets main page
	else if (command.equals("disconnect")) {
	    // call method to get connection to be removed
	    ConnectionRequestHandler.handleDisconnectInitiation(request,out,
								compositionModel,
								connectionPort);
	} else if (command.equals("middisconnect")) {
	    // call method to remove the connection
	    ConnectionRequestHandler.handleDisconnectCompletion(request,out,
								compositionModel,
								connectionPort);
	}
	// note that if-else is continued below
	// ids commands
	// three screens:
	// 1. user selects ids lookup on main page, action taken is 
	//    that user is given a list of possible components to
	//    view or create as well as a list of subcontexts into
	//    which he can enter
	// 2. user opts to view a component, action taken is that 
	//    component information is displayed 
	// 3. user opts to create component, action taken is that
        //    component is created and user gets main screen
	// if user enters a subcontext, it is the same as doing a lookup
	// with this new context
	else if (command.equals("idslookup")) {
	    // get the current IDS context
	    HttpSession session = request.getSession(false);
	    InformationDirectoryContext idsContext = 
		(InformationDirectoryContext) (session.getValue("idsContext"));
	    // call method to lookup current IDS context
	    IDSRequestHandler.handleIDSLookup(request,out,compositionModel,idsContext);
	} else if (command.equals("Create component")) {
	    // get the current IDS context
	    HttpSession session = request.getSession(false);
	    InformationDirectoryContext idsContext = 
		(InformationDirectoryContext) (session.getValue("idsContext"));
	    // call method to create the selected component
	    IDSRequestHandler.handleIDSCreate(request,out,compositionModel,
					      idsContext,creationPort);
	} else if (command.equals("View component information")) {
	    // get the current IDS context
	    HttpSession session = request.getSession(false);
	    InformationDirectoryContext idsContext = 
		(InformationDirectoryContext) (session.getValue("idsContext"));
	    // call method to display the component information
	    // make sure we change the context type before that
	    IDSRequestHandler.handleIDSView(request,out,compositionModel,idsContext);
	} else if (command.equals("Enter IDS subcontext")) {
	    try {
		// get the name of the subcontext
		String subContextName = request.getParameter("subcontext");
		// get the current IDS context
		HttpSession session = request.getSession(false);
		InformationDirectoryContext idsContext = 
		    (InformationDirectoryContext) (session.getValue("idsContext"));
		// use this to get the subcontext
		InformationDirectoryContext idsSubContext = 
		    idsContext.getSubContext(subContextName);
		// make this the current context
		// also make the stored context string refer to the new context
		String contextString = (String) session.getValue("idsLocation");
		contextString+=subContextName;
		session.putValue("idsLocation",contextString);
		session.putValue("idsContext",idsSubContext);
		// lookup using this new context
		IDSRequestHandler.handleIDSLookup(request,out,compositionModel,idsSubContext);
	    } catch (Exception exception) {
		exception.printStackTrace();
	    }
	}

	// note that it if-else is continued below
	// registry commands
	// registry bind
	// two screens
	// 1. user selects registry bind on main page, action taken is
	//    that user is asked for name of instance to bind and binding name
	// 2. user supplies name of instance, action taken is that the instance
	//    gets bound under the given binding name into the current registry
	//    context and user gets main screen
	else if (command.equals("registrybind")) {
	    // get the current registry context
	    HttpSession session = request.getSession(false);
	    RegistryContext registryContext = (RegistryContext) (session.getValue("registryContext"));
	    // call method to handle registry bind initiation
	    RegistryRequestHandler.startBind(request,out,compositionModel,registryContext);
	} else if (command.equals("registrymidbind")) {
	    // get the current registry context
	    HttpSession session = request.getSession(false);
	    RegistryContext registryContext = (RegistryContext) (session.getValue("registryContext"));
	    // call method to complete bind process
	    RegistryRequestHandler.completeBind(request,out,compositionModel,
						registryContext);
	}
	// note that it if-else is continued below
	// registry unbind
	// two screens
	// 1. user selects registry unbind on main page, action taken is
	//    that user is asked for name of instance to unbind
	// 2. user supplies name of instance, action taken is that the instance
	//    binding form the current context gets removed and user gets main 
	//    screen
	else if (command.equals("registryunbind")) {
	    // get the current registry context
	    HttpSession session = request.getSession(false);
	    RegistryContext registryContext = (RegistryContext) (session.getValue("registryContext"));
	    // call method to handle registry unbind initiation
	    RegistryRequestHandler.startUnbind(request,out,compositionModel,registryContext);
	} else if (command.equals("registrymidunbind")) {
	    // get the current registry context
	    HttpSession session = request.getSession(false);
	    RegistryContext registryContext = (RegistryContext) (session.getValue("registryContext"));
	    // call method to complete unbind process
	    RegistryRequestHandler.completeUnbind(request,out,compositionModel,
						  registryContext);
	}
	// note that if-else is continued below
	// registry lookup
	// three screens
	// 1. user selects registry lookup on main page, action taken is that
	//    user gets a list of instances bound in the current context
	//    with the option of viewing the information of copying the instance
	//    into the workspace as well as a list of subcontexts he can enter
	// 2. user opts to view instance information, action taken is that
	//    the information gets displayed
	// 3. user opts to copy instance, action taken is that the instance
	//    is copied and added to the workspace, user gets main screen
	//    if user enters subcontext, the new context is looked up
	else if (command.equals("registrylookup")) {
	    // get the current registry context
	    HttpSession session = request.getSession(false);
	    RegistryContext registryContext = (RegistryContext) (session.getValue("registryContext"));
	    // call method to lookup curent registry context
	    RegistryRequestHandler.handleRegistryLookup(request,out,
							compositionModel,registryContext);
	} else if (command.equals("View instance information")) {
	    // get the current registry context
	    HttpSession session = request.getSession(false);
	    RegistryContext registryContext = (RegistryContext) (session.getValue("registryContext"));
	    // call method to display selected instance information
	    RegistryRequestHandler.handleRegistryView(request,out,
						      compositionModel,registryContext);
	} else if (command.equals("Copy instance")) {
	    // get the current registry context
	    HttpSession session = request.getSession(false);
	    RegistryContext registryContext = (RegistryContext) (session.getValue("registryContext"));
	    // call method to make a copy of the instance and add to workspace
	    RegistryRequestHandler.handleRegistryCopy(request,out,
						      compositionModel,registryContext);
	} else if (command.equals("Enter registry subcontext")) {
	    try {
		// get the name of the subcontext
		String subContextName = request.getParameter("subcontext");
		// get the current registry context
		HttpSession session = request.getSession(false);
		RegistryContext registryContext = (RegistryContext) (session.getValue("registryContext"));
		// use this to get the subcontext
		RegistryContext registrySubContext = 
		    registryContext.getSubContext(subContextName);
		// also make the stored context string refer to the new context
		String contextString = (String) session.getValue("registryLocation");
		contextString+=subContextName;
		session.putValue("registryLocation",contextString);
		// make this the current context
		session.putValue("registryContext",registrySubContext);
		// lookup using this new context
		RegistryRequestHandler.handleRegistryLookup(request,out,compositionModel,registrySubContext);
	    } catch (Exception exception) {
		exception.printStackTrace();
	    }
	}
	// note that if-else is continued below
	// create given instance information
	// user is allowed to copy an instance into the workspace
	// given the instance information 
	// two screens
	// 1. user selects instance copy from main screen, action taken is that
	//    user is asked to paste instance information into a text area
	// 2. user pastes information and asks for copy to be made, action
	//    taken is that instance is copied and addedf to workspace, user
	//    gets main screen
	else if (command.equals("instancecopy")) {
	    // call method to get the instance information
	    InstanceCopyRequestHandler.startCopy(request,out,compositionModel);
	} else if (command.equals("midinstancecopy")) {
	    // call method to create a copy if the instance and add
	    // to workspace
	    InstanceCopyRequestHandler.completeCopy(request,out,compositionModel);
	} else if (command.equals("quit")) {
	    // thank you message
	    out.println("<html><head><title>Thanks!</title></head><body bgcolor=\"#FFFFFF\"><h3>Session complete</h3><p>Thanks for using the CCAT web composition interface!</p></body></html>");
	    // remove session image
	    File file = new File("/u/drseuss/session/images/"+
				 compositionModel.getSessionID()+".jpg");
	    file.delete();
	    request.getSession(false).invalidate();
	} else if (command.equals("quitclean")) {
	    // kill all components
	    Enumeration components = compositionModel.getComponents();
	    while(components.hasMoreElements()) {
		ComponentModel current = (ComponentModel) components.nextElement();
		// kill it
		String paramPortName = current.getParameterPortName();
		String paramClassName = current.getParameterPortClassName();
		componentCommunicator.killComponent(current.getComponentID(),
						    paramClassName,
						    paramPortName,
						    current.getName());
	    }
	    // thank you message
	    out.println("<html><head><title>Thanks!</title></head><body bgcolor=\"#FFFFFF\"><h3>Session complete</h3><p>Thanks for using the CCAT web composition interface!</p></body></html>");
	    // remove session image
	    File file = new File("/u/drseuss/session/images/"+
				 compositionModel.getSessionID()+".jpg");
	    file.delete();
	    request.getSession(false).invalidate();
	} else {
	    out.println("<html<head><title>Error</title></head><body bgcolor=\"#FFFFFF\"><h2>Error</h2><p>This service is not currently supported through this interface.</p></body></html>");
	}
    }

    public static String getDefaultMenu(HttpSession session) {
	// get the default menu corresponding ot this session
	// get the ids context name
	String idsLocation = (String) session.getValue("idsLocation");
	String registryLocation = (String) session.getValue("registryLocation");
	// contruct the default menu
	// the first two items are the ids and registry context fields
	StringBuffer sb = new StringBuffer();
	sb.append("<html><head><title>Compose a CCAT application</title></head><body bgcolor=\"#FFFFFF\"><h4>Compose a CCAT application</h4><FORM ACTION=\"/servlets/servlet.ServletComponent\" METHOD=\"POST\"><p>Current information directory: <INPUT TYPE=\"TEXT\" SIZE=\"55\" NAME=\"idslocation\" VALUE=\""+idsLocation+"\"></p><p>Current registry: <INPUT TYPE=\"TEXT\" SIZE=\"55\" NAME=\"registrylocation\" VALUE=\""+registryLocation+"\"></p>Available services:<br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"idslookup\"><a href=\"../options.html#ids\">Lookup information directory and create instance</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"registrybind\"><a href=\"../options.html#registrybind\">Bind instance into registry</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"registryunbind\"><a href=\"../options.html#registryunbind\">Unbind an instance from registry</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"registrylookup\"><a href=\"../options.html#registrylookup\">Lookup registry</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"connect\"><a href=\"../options.html#connection\">Make a connection</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"disconnect\"><a href=\"../options.html#disconnection\">Remove an existing connection</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"instancecopy\"><a href=\"../options.html#copy\">Copy instance</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"quitclean\"><a href=\"../options.html#quitclean\">Quit, killing all instances</a><br><INPUT TYPE=\"RADIO\" NAME=\"command\" VALUE=\"quit\"><a href=\"../options.html#quit\">Quit, leaving instances alive</a><br><INPUT TYPE=\"SUBMIT\" NAME=\"submit\" VALUE=\"Access service\"></FORM><br>Click on an instance below to access instance controls<br><a href=\"/servlets/servlet.ServletComponent\"><img ismap src=\"/servlets/FreshImage\"></a></body></html>");
	return sb.toString();
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response) 
	throws ServletException, IOException {
	// if there is no query string, just bring up the default menu for this session
	String query = request.getQueryString();
	response.setContentType("text/html");
	PrintWriter out = response.getWriter();
	if (query==null || query.equals("")) {
	    // get the default menu
	    out.println(ServletComponent.getDefaultMenu(request.getSession(false)));
	} else {
	    // this is called when the user clicks on the session image
	    HttpSession session = request.getSession(false);
	    // get the composition model
	    WorkingAreaModel compositionModel = (WorkingAreaModel) session.getValue("workingArea");
	    // get the coordinates the user clicked on
	    String coordinates = request.getQueryString();
	    // translate into integer coordinates
	    int separator = coordinates.indexOf(",");
	    int xcoord = new Integer(coordinates.substring(0,separator)).intValue();
	    int ycoord = new Integer(coordinates.substring(separator+1)).intValue();
	    Point userClickPoint = new Point(xcoord,ycoord);
	    // check which instance has this point, if any
	    ComponentModel instance = compositionModel.getComponent(userClickPoint);
	    // if no match, complain
	    if (instance == null) {
		out.println("<html><head><title>Error</title></head><body bgcolor=\"#FFFFFF\"><h3>Error</h3><p>The mouse was not clicked over any instace, instance controls could not be accessed</p><b><a href=\"/servlets/servlet.ServletComponent\">Back to main menu</a></b></body></html>");
	    } else {
		ControlRequestHandler.handleControlInitiation(request,out,compositionModel,
							      componentCommunicator,
							      instance.getName());
	    }
	}
    }

	
    protected void doPost(HttpServletRequest request, HttpServletResponse response) 
	throws ServletException, IOException {
	// this will translate the HttpServletRequest depending on the
	// form parameters
	// then it will call one of the above methods to use a 
	// framework service in order to satisfy the request
	// get the session this request is associated with
	HttpSession session = request.getSession(true);
	response.setContentType("text/html");
	// get the composition model
	WorkingAreaModel compositionModel = (WorkingAreaModel) session.getValue("workingArea");
	if (compositionModel == null) {
	    // session has not been initialised
	    sessionInit(request,session);
	}
	try {
	    // get the composition model of the initialised session
	    compositionModel = (WorkingAreaModel) session.getValue("workingArea");
	    // get the ids context
	    // check if the current IDS location is the same as the old one
	    String currentIDSLocation = request.getParameter("idslocation");
	    if (currentIDSLocation!=null) {
		String oldIDSLocation = (String) session.getValue("idsLocation");
		if (!currentIDSLocation.equals(oldIDSLocation)) {
		    // we have to replace the IDS context since it has changed
		    session.putValue("idsContext",idsPort.getContext(currentIDSLocation));
		}
	    }
	    InformationDirectoryContext idsContext = (InformationDirectoryContext) session.getValue("idsContext");
	    // get the registry context
	    // check if the current REGISTRY location is the same as the old one
	    String currentRegistryLocation = request.getParameter("registrylocation");
	    if (currentRegistryLocation!=null) {
		String oldRegistryLocation = (String) session.getValue("registryLocation");
		if (!currentRegistryLocation.equals(oldRegistryLocation)) {
		    // we have to replace the Registry context since it has changed
		    session.putValue("registryContext",registryPort.getContext(currentRegistryLocation));
		}
	    }
	} catch (Exception exception) {
	    exception.printStackTrace();
	}
	RegistryContext registryContext = (RegistryContext) session.getValue("registryContext");
	handleRequest(request,response,compositionModel);
    }
}


