package servlet;

import ccacore.ComponentID;
import java.util.Vector;
import java.util.Enumeration;
import java.io.*;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Point;
import java.awt.Dimension;
import javax.servlet.http.HttpSession;
import com.sun.image.codec.jpeg.*;

public class WorkingAreaModel {
    private Vector componentModels;
    private Vector connectionModels;
    private int sessionCounter;
    private String sessionID;

    public WorkingAreaModel(String sessionID) {
	componentModels = new Vector();
	connectionModels = new Vector();
	sessionCounter = 0;
	this.sessionID = sessionID;
    }

    public void addComponent(ComponentModel model) {
	componentModels.addElement(model);
	// get the component name from the info
	String componentName = model.getInfo().getInfo("instance-info")[0].
	    getValue("instance-name")[0];
	// get the last part of the component name
	int lastSlashPos = componentName.lastIndexOf('/');
	String basicName = componentName;
	if (lastSlashPos != -1)
	    basicName = componentName.substring(lastSlashPos+1);
	model.setName(basicName+sessionCounter);
	sessionCounter++;
    }

    public void removeComponent(ComponentModel model) {
	componentModels.removeElement(model);
    }

    public void addConnection(ConnectionModel connection) {
	connectionModels.addElement(connection);
    }

    public void removeConnection(ConnectionModel connection) {
	connectionModels.removeElement(connection);
    }

    public int getSessionCounter() {
	return sessionCounter;
    }

    public String getSessionID() {
	return sessionID;
    }

    public ComponentModel getComponent(String componentName) {
	// go through the components and find the one with this name
	Enumeration componentModelList = componentModels.elements();
	while(componentModelList.hasMoreElements()) {
	    ComponentModel current = (ComponentModel) componentModelList.nextElement();
	    // check if the names match
	    if (current.getName().equals(componentName))
		// return the component model
		return current;
	}
	// did not find a component with this name
	return null;
    }

    public ComponentModel getComponent(Point point) {
	// go through the components and find the one which contains this point
	Enumeration componentModelList = componentModels.elements();
	while(componentModelList.hasMoreElements()) {
	    ComponentModel current = (ComponentModel) componentModelList.nextElement();
	    // check if the point is onctained in this component
	    if (current.containsPoint(point))
		// return the component model
		return current;
	}
	// did not find a component that contains this point
	return null;
    }

    public ConnectionModel getConnection(String connectionId) {
	// go through the connections and find the one with this id
	Enumeration connectionModelList = connectionModels.elements();
	while(connectionModelList.hasMoreElements()) {
	    ConnectionModel current = (ConnectionModel) connectionModelList.nextElement();
	    // check if the names match
	    if (current.getId().equals(connectionId))
		// return the connection model
		return current;
	}
	// did not find a component with this name
	return null;
    }

    public Enumeration getComponents() {
	return componentModels.elements();
    }
	
    public Enumeration getConnections() {
	return connectionModels.elements();
    }

    public ComponentID getComponentID(String componentName) {
	// go through the components and find the one with this name
	Enumeration componentModelList = componentModels.elements();
	while(componentModelList.hasMoreElements()) {
	    ComponentModel current = (ComponentModel) componentModelList.nextElement();
	    // check if the names match
	    if (current.getName().equals(componentName))
		// return the id
		return current.getComponentID();
	}
	// did not find a component with this name
	return null;
    }

    private Dimension drawComponent(ComponentModel model, Graphics2D graphics,
				    Point topLeft) {
	// width of component depends on number of characters in component name
	// height of component depends on max(number of input ports, number of output ports)
	// horizontal space between adjacent components depends on length of longest 
	// output port name of left component and length of longest input port name 
	// of right component (ugh, make it fixed for now)
	// vertical space betwen component rows is fixed
	// cursorX and cursorY are the coordinates of the pen for the next draw
	System.out.println("Top left of component is "+topLeft);
	int cursorX; int cursorY;

	cursorX = topLeft.x; cursorY = topLeft.y;
	System.out.println("Starting component draw at "+cursorX+","+cursorY);
	String name = model.getName();
	// figure out component width
	// do this very approximately, just directly porportional to num chars
	int width = name.length()*7+15;
	int numInputs = model.getNumInputs();
	int numOutputs = model.getNumOutputs();
	int maxPorts = numOutputs;
	if (numInputs > numOutputs)
	    maxPorts = numInputs;
	int height = maxPorts*20+40;
	// draw box for component
	graphics.setColor(new Color(240,200,200));
	graphics.fill3DRect(cursorX,cursorY,width,height,true);
	// draw component name
	graphics.setColor(Color.black);
	cursorX=topLeft.x+width/2-25; cursorY=topLeft.y+10;
	graphics.drawString(name,cursorX,cursorY);
	// draw ports
	// input ports
	Enumeration inputPortStrings = model.getInputPortNames();
	cursorX=topLeft.x;
	cursorY=topLeft.y+21;
	while(inputPortStrings.hasMoreElements()) {
	    String currentPortName = (String)inputPortStrings.nextElement();
	    graphics.setColor(Color.blue);
	    graphics.fillRoundRect(cursorX,cursorY,8,8,2,2);
	    graphics.setColor(Color.black);
	    graphics.drawString(currentPortName,
				cursorX-5,cursorY+15);
	    model.setProvidesPortCoordinates(currentPortName,new Point(cursorX+4,cursorY+4));
	    cursorY+=20;
	}
	// output ports
	cursorY=topLeft.y+21;
	cursorX=topLeft.x+width-8;
	Enumeration outputPortStrings = model.getOutputPortNames();
	while(outputPortStrings.hasMoreElements()) {
	    String currentPortName = (String)outputPortStrings.nextElement();
	    graphics.setColor(Color.red);
	    graphics.fillRoundRect(cursorX,cursorY,8,8,2,2);
	    graphics.setColor(Color.black);
	    graphics.drawString(currentPortName,
				cursorX-20,cursorY+22);
	    model.setUsesPortCoordinates(currentPortName,new Point(cursorX+4,cursorY+4));
	    cursorY+=27;
	}
	System.out.println("Drew component "+name);
	// return width and height of this component
	return new Dimension(width,height);
    }
	
    public void drawSession() throws IOException {
	// draw a rectangular box for each component and a small circle
	// for each of the component's ports
	// label the component with a unique ID and each of its ports
	// with their names
	// draw connections between ports which are connected
	// create a graphics context to use
	BufferedImage image = new BufferedImage(450,350,BufferedImage.TYPE_INT_RGB);
	Graphics2D graphics = image.createGraphics();
	// fill a white rectangle as the canvas
	graphics.setColor(Color.white);
	graphics.fill3DRect(0,0,450,350,false);
	// go through the components
	int topLeftX = 10; 
	int topLeftY=10;
	// do 2 components per row
	Enumeration components = componentModels.elements();
	int numDrawn = 0;
	int oldHeight = 0;
	while(components.hasMoreElements()) {
	    System.out.println("Beginning draw for component "+(numDrawn+1));
	    ComponentModel componentModel = (ComponentModel) components.nextElement();
	    // check if this component has been drawn before
	    if (!componentModel.hasCoordinates()) {
		// use the computed top left for the next component, and set 
		// this value in the model
		componentModel.setCoordinates(new Point(topLeftX,topLeftY));
	    }
	    // draw the component 
	    Dimension dimension = drawComponent(componentModel,graphics,
						componentModel.getCoordinates());
	    componentModel.setDimensions(dimension);
	    int height = (int)(dimension.getHeight());
	    int width = (int)(dimension.getWidth());
	    int bottomRightX = topLeftX+width;
	    int bottomRightY = topLeftY+height;
	    numDrawn++;
	    if (numDrawn % 2 == 0) {
		// end of row
		topLeftX = 10;
		topLeftY+=height;
		if (oldHeight>height)
		    topLeftY+=oldHeight;
		topLeftY+=10;
		oldHeight = 0;
	    } else {
		topLeftX=topLeftX+width+80;
		oldHeight = height;
	    }
	}
	// draw all connections
	Enumeration connections = connectionModels.elements();
	while(connections.hasMoreElements()) {
	    ConnectionModel currentConnection = (ConnectionModel) connections.nextElement();
	    // get the uses instance
	    String usesInstanceName = currentConnection.getUsesInstance().getName();
	    // get the provides instance
	    String providesInstanceName = currentConnection.getProvidesInstance().getName();
	    ComponentModel usesInstance = null;
	    ComponentModel providesInstance = null;
	    Enumeration componentList = componentModels.elements();
	    while(componentList.hasMoreElements()) {
		ComponentModel currentComponent = 
		    (ComponentModel) componentList.nextElement();
		if (currentComponent.getName().equals(usesInstanceName)) {
		    usesInstance = currentComponent;
		    if (providesInstance!=null)
			break;
		} else if (currentComponent.getName().equals(providesInstanceName)) {
		    providesInstance = currentComponent;
		    if (usesInstance!=null)
			break;
		}
	    }
	    // get the uses port name
	    String usesPortName = currentConnection.getUsesPortName();
	    // get the provides port name
	    String providesPortName = currentConnection.getProvidesPortName();
	    // figure out the coordinates for the provides port
	    Point providesPortCoordinates = 
		providesInstance.getProvidesPortCoordinates(providesPortName);
	    // figure out coordinates for the uses port
	    Point usesPortCoordinates = usesInstance.getUsesPortCoordinates(usesPortName);
	    // draw a line for the connection
	    graphics.setColor(Color.black);
	    graphics.drawLine(usesPortCoordinates.x,usesPortCoordinates.y,
			      providesPortCoordinates.x,providesPortCoordinates.y);
	}
	try {
	    File file = new File("/u/drseuss/session/images/"+sessionID+".jpg");
	    System.out.println("Saving image to file "+file.getAbsolutePath());
	    OutputStream out = new FileOutputStream(file);
	    JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);    
	    JPEGEncodeParam param = JPEGCodec.getDefaultJPEGEncodeParam(image);    
	    encoder.encode(image, param);
	    out.flush();    
	    out.close();
	} catch (Exception exception) {
	    System.out.println("Could not save image to jpeg file, got exception");
	    exception.printStackTrace();
	}
    }
}
