//  This class implements the all the methods
//  that will be invoked on ComponentID.
//  Note that some methods are invoked by
//  the user as part of ComponentID interface
//  and some by the framework as part of the
//  ComponentIDprivate interface.
//  --Madhu
 
package ccacore;

import java.io.*;
import java.util.*;
import nexusrmi.*;
import nexusrmi.server.*;
import exceptions.*;
import nexus.*;

public class ComponentIDImpl extends UnicastRemoteObject implements ComponentID, ComponentIDconnection, ComponentIDregistry {

  private ServicesPrivate servicesImpl;
  private String name = "DEFAULT";
  private String ref;

  // this constructor is called by ServicesImpl.getComponentID()
  // This ensures that a ComponentId always points to an
  // implementation of the Services interface. The componentID
  // in our implementation is really a proxy for the Services
  // implementation that moves to remote processes.
  public ComponentIDImpl(ServicesPrivate _servicesImpl) throws nexusrmi.RemoteException {
    servicesImpl = _servicesImpl;
    try {
    ref = this.toMyString();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  // FIXME FIXME
  // called by create(stringifiedRef) in this class. 
  // XXX Exception added to get it to compile.  -kec 10/21/99
    //  public ComponentIDImpl(String _ref) throws nexusrmi.RemoteException {
    //ref = _ref;
    //}
  
  // Invoke the corresponding method on the
  // ServicesImpl reference. ComponentID after
  // all is just a proxy and does not store these
  // tables.
  public PortInfo[] getProvidesPorts()  throws Exception {
    try {
      PortInfo[] returnProvidesPorts;
      returnProvidesPorts = ((Services)servicesImpl).getProvidesPorts();
      return returnProvidesPorts;
    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }

  // FIXME FIXME needs to be CCAException
  // Invoke the corresponding method on the
  // ServicesImpl reference. ComponentID after
  // all is just a proxy and does not store these
  // tables.
  public PortInfo[] getUsesPorts() throws Exception {

    try {
      PortInfo[] returnUsesPorts;
      returnUsesPorts = ((Services)servicesImpl).getUsesPorts();
      return returnUsesPorts;
    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }

  // This method is part of the ComponentIDprivate interface.
  // This method is called by ConnectionServiceImpl::connect(...)
  // The Connection service calls this method when it wants to
  // connect a remote provides port to the userPort of the this
  // component. ProvidesPort reference that has been passed in
  // here is a remote reference. 
  public int ccatConnect(String usesPortName, Port providesPort) throws Exception {
    int result = 0;
//     System.out.println("\nComponentIDImpl::ccatConnect: calling servicesImpl.ccatConnect()");
    try {
      result = servicesImpl.ccatConnect(usesPortName, providesPort);
    } catch (Exception e) {
      e.printStackTrace();
    }
//     System.out.println("\nComponentIDImpl::ccatConnect: servicesImpl.ccatConnect() done");
    return result;
  }

  // This method is part of the ComponentIDprivate interface.
  // This method is called by ConnectionServiceImpl::disconnect(...)
  // The Connection service calls this method when it wants to
  // disconnect a remote provides port to the userPort of the this
  // component. ProvidesPort reference that has been passed in
  // here is a remote reference. 
  public int ccatDisconnect(String usesPortName, Port providesPort) throws Exception {
    int result = 0;
    try {
      result = servicesImpl.ccatDisconnect(usesPortName, providesPort);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return result;
  }

  // This method is part of the ComponentIDprivate interface.
  // This method is called by ConnectionServiceImpl::connect(...)
  // and ConnectionServiceImpl::disconnect(...). The Connection
  // service obtains a remote reference to a provides port via
  // this call and passes it on to another ComponentID to connect
  // to its uses port.
  public Port ccatGetProvidesPort(String providesPortName) throws Exception {
    Port p = null;
//     System.out.println("\nComponentIDImpl::ccatConnect: calling servicesImpl.ccatGetProvidesPort()");
    try {
      p = servicesImpl.ccatGetProvidesPort(providesPortName);
//       System.out.println("\nComponentIDImpl::ccatConnect: calling servicesImpl.ccatGetProvidesPort()");
      return p;
    } catch (nexusrmi.RemoteException e) {
      e.printStackTrace();
      return null;
    } catch (Exception ex) {
      ex.printStackTrace();
      return null;
    }
  }

  // export the given provides port as my newPortName. 
  public int ccatExportAs(DefaultPortInfo newPortInfo, Port providesPort) 
    throws Exception {
    int result = -1;
    try {
      result = servicesImpl.ccatExportAs(newPortInfo, providesPort);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return result;
  }

  public int ccatProvideTo(String usesPortName, Port providesPort) throws Exception {
    int result = -1;
    try {
      result = servicesImpl.ccatProvideTo(usesPortName, providesPort);
    } catch (Exception e) {
      e.printStackTrace();
    }    
    return result;
  }
  
  // get the port type string corresponding to a port name string.
  public String ccatGetPortTypeForPortName(String portName) throws Exception {
    String result = null;
    try {
	 result = servicesImpl.ccatGetPortTypeForPortName (portName);
    } catch (Exception e) {
      e.printStackTrace();
    }
    return result;
  }

  // These methods were added as we
  // need to be able to store and
  // retrieve ComponentIDs from the
  // instance registry

  public boolean equals(ComponentIDregistry cid) throws nexusrmi.RemoteException {
    nexus.Startpoint mysp = UnicastRemoteObject.getStartpoint((nexusrmi.Remote)this);	 
    nexus.Startpoint othersp = UnicastRemoteObject.getStartpoint((nexusrmi.Remote)(this)); 
    if (mysp == null) {
      return false;
    } else {
      return mysp.equals(othersp);
    }
  }
  
  public String toMyString() throws nexusrmi.RemoteException {

      System.out.println("In toMyString");
    // first get the startpoint for this remote object
    nexus.Startpoint sp = UnicastRemoteObject.getStartpoint((nexusrmi.Remote)this);

    // marshal the data format into the byte stream. This will be required
    // for unmarshalling the sp from the string.
    ByteArrayOutputStream bstream = new ByteArrayOutputStream();
    //    bstream.write(nexus.DataConversion.NEXUS_DC_FORMAT_JAVA);
    bstream.write((byte)nexus.DataConversion.NEXUS_DC_FORMAT_JAVA); //SMD : TEST

    // marshal the startpoint into a byte array output stream
    sp.write (bstream);
    // instead of converting a byte array to a string directly, we will
    // convert it into a string containing a sequence of integers, each integer
    // representing a character
    
    // read the byte array one byte at a time
    byte [] bytes = bstream.toByteArray();
    System.out.println("ComponentIDImpl::toMyString(): Converting id to string...");
    StringBuffer stringifiedByteArray = new StringBuffer();
    for (int i=0; i<bytes.length; i++) {
	byte current = bytes[i];
	// convert to an integer and add to buffer
	stringifiedByteArray.append((int)current);
	stringifiedByteArray.append(":");
    }
    // remove the colon at the end
    int lastCharPos = stringifiedByteArray.length()-1;
    stringifiedByteArray.deleteCharAt(lastCharPos);
    System.out.println("ComponentIDImpl::toMyString(): Stringified id is "+stringifiedByteArray);
    return stringifiedByteArray.toString();
  }

  public void setName(String _name) throws nexusrmi.RemoteException {
//     System.out.println("\nComponentIDImpl::setName() called with: " + _name);
    name = _name;
  }
  
  public String getName() throws nexusrmi.RemoteException {
    return name;
  }
  
  public String getRef() throws nexusrmi.RemoteException {
    return ref;
  }
}



