
//   This class represents the crux of our framework
//   implementation. It implements the Services interface
//   for the component developer and the ServicesPrivate
//   interface so the the proxy (ComponentID) in a remote
//   process can invoke methods from ConnectionServiceImpl.
//   --Madhu

package ccacore;

import java.io.*;
import java.util.*;
import nexusrmi.*;
import nexusrmi.server.*;
import exceptions.*;
import connection.*;
import creation.*;
import registry.*;
import ids.*; 
import idl.*;

// ServicesImpl  is a local object. The servicesImpl should
// not ever move to a different process. The ComponentID is the only
// object that moves to a different process. 
public class ServicesImpl implements Services, ServicesPrivate  {

  Vector ProvidesPortlist = new Vector();
  Vector UsesPortlist = new Vector();
  CreationService creationService;
  ConnectionService connectionService;
  RegistryService registryService;
  InformationDirectoryService idsService;
  private boolean runCalled = false;

  // The framework main would first call the null constructor
  // so that a core can be formed for the
  // event Channel. Then the other constructor would be called again
  // with the eventChannelComponentID as an argument. The idea
  // is that the eventChannel and the component should both
  // have their own componentIDs, but they should share the
  // jvm, creation, registry, connetion and directory service.

  public ServicesImpl() throws nexusrmi.RemoteException {
    this(null);
  }

  // If an eventCompID is being passed, it means that the
  // services need to connect to the eventChannel. This way
  // the services will be able to publish events to the event
  // channel. The GUI component will register itself as a listener
  // to the event channel and receive the events. 
  public ServicesImpl(ComponentID eventCompID) throws nexusrmi.RemoteException {
    init();
    if (eventCompID != null) {
      System.out.println("\nServicesImpl:calling makeEventPorts ... ");
      FrameworkServicesSingletonFactory.makeEventPortConnections (eventCompID);
    }
  }
  
  private void init() {
    creationService = FrameworkServicesSingletonFactory.getCreationService();
    connectionService = FrameworkServicesSingletonFactory.getConnectionService();
    registryService = FrameworkServicesSingletonFactory.getRegistryService();
    idsService = FrameworkServicesSingletonFactory.getInformationDirectoryService();
  }
  
  public PortInfo[] getProvidesPorts()  throws Exception {
    int n = ProvidesPortlist.size();
    if(n == 0) {
      return null;
    } else {
      PortInfo[] localProvidesList = new PortInfo[n];
      int i;
      for(i = 0; i < n; i++)
	localProvidesList[i] = ((PortRecord) ProvidesPortlist.elementAt(i)).pinfo;
      return localProvidesList;
    }
  }

  // FIXME FIXME, needs to be CCA exception 
  public PortInfo[] getUsesPorts() throws Exception {
    int n = UsesPortlist.size();
    if(n == 0) return null;
    else {
      PortInfo[] localUsesList = new PortInfo[n];
      int i;
      for(i = 0; i < n; i++)
	localUsesList[i] = ((PortRecord) UsesPortlist.elementAt(i)).pinfo;
      return localUsesList;
    }
  }

  // FIXME FIXME needs to be CCAException
  public Port getPort(String nameOfPort) throws CCAException {
    int n = UsesPortlist.size();
    int i;
    System.out.println("\n ServicesImpl::getPort() called");
    if (nameOfPort.equals("CreationService")) {
      return creationService;
    } else if (nameOfPort.equals("ConnectionService")) {
      return connectionService;
    } else if (nameOfPort.equals("RegistryService")) {
	return registryService;
    } else if (nameOfPort.equals("InformationDirectoryService")) {
      return idsService;
    }
    
    UsesPortImpl up = getUsesPortImpl (nameOfPort);
    if (up != null)
      synchronized (up) {
      if (!up.hasAllVoidMethods ())
	// block until there is a connection established with this port. When
	// you are awaken, you already have a lock.
	while (!up.isConnected())
	  try {
	System.out.println ("ServerciesImpl.getPort: waiting for a connection");
	up.wait();
	System.out.println ("ServerciesImpl.getPort: connection wait ended!!!");
      } catch (InterruptedException exp) {
	System.out.println ("ServerciesImpl.getPort: wait interrupted");
	exp.printStackTrace();
	return null;
      }
      up.incrRefCount ();
      return up;
    }

    n = ProvidesPortlist.size();
    for(i = 0; i < n; i++){
      PortRecord pRecord = (PortRecord) ProvidesPortlist.elementAt(i);
      if(pRecord.pinfo.getName().equals(nameOfPort))
	return pRecord.port;
    }
    return null;
  }
  
  private UsesPortImpl getUsesPortImpl (String nameOfPort) {
    int n = UsesPortlist.size();
    int i;
    System.out.println("\nServicesImpl::getUsesPortImpl() inside !!!");
    for (i = 0; i < n; i++) {
      PortRecord pRecord = (PortRecord) UsesPortlist.elementAt(i);
      if(pRecord.pinfo.getName().equals(nameOfPort)) {
	System.out.println("\n ServicesImpl::getUsesPortImpl found it !!");
	//	System.out.println("\n ServicesImpl::getUsesPortImpl found it !!" + ((UsesPortImpl)pRecord.port)).getClass());
	return (UsesPortImpl)pRecord.port;
      }
    }
    System.out.println("\nServicesImpl::getUsesPortImpl() is returning null !!!");
    return null;
  }
    
  // eg: myCore.addProvidesPort(PortPointer, new PortInfo("input", "String_idl"));
  // fqn  =  idl.string.ProvidesStringPortSkel
  // User defines type as String_idl
  // The package name for this will be idl.string
  // The fqn of the implementation will be idl.string.ProvidesStringPortSkel
  public void addProvidesPort(Port inPort, PortInfo portInfo) throws CCAException {
    Class c;
    String fqn = null;
    try{
      String idlType = portInfo.getType();
      int indexOfidlPart = idlType.indexOf("_idl");
      String strippedType = idlType.substring(0, indexOfidlPart);
      String packageName = strippedType.substring(0,1).toLowerCase() + strippedType.substring(1, strippedType.length());			  
      fqn  = "idl." + packageName + ".Provides" + strippedType+ "Skel";
      System.out.println("\n ServicesImpl: addProvidesPort : fqn = " + fqn);
      c = Class.forName(fqn);
    } catch(Exception e) {
      System.out.println("ServicesImpl: addProvidesPort: CCAT framework is unaware of the type you are trying to register " + fqn);
      e.printStackTrace();
      System.out.println("class bummer:"+e);
      c = null;
    }
    if (c == null) {
      return;
    } else {
      try{
	ProvidesPort stub = (ProvidesPort) c.newInstance();
	if(stub == null) {
	  System.out.println("ServicesImpl::addProvidesPort(): null providesPort !");
	  return;
	}
	System.out.println("ServicesImpl::addProvidesPort(): inPort: " + inPort.getClass());
	stub.setProxy((ProvidesPort)inPort);
	ProvidesPortlist.addElement(new PortRecord((Port)stub, portInfo));
      }  catch(Exception e){
	System.out.println("ServicesImpl::addProvidesPort(): can't add providesPort !");
	e.printStackTrace();
      }
    }
    return;
  }
  
  // eg: myCore.registerUsesPort(new PortInfo("output", "String_idl"));
  //  fqn  =  idl.String.UsesStringPortSkel
  public void registerUsesPort(PortInfo portInfo)  {
    Class c;
    String fqn = null;
    try{
      String idlType = portInfo.getType();
      int indexOfidlPart = idlType.indexOf("_idl");
      String strippedType = idlType.substring(0, indexOfidlPart);
      String packageName = strippedType.substring(0,1).toLowerCase() + strippedType.substring(1, strippedType.length());			  
      fqn  = "idl." + packageName + ".Uses" + strippedType+ "Skel";
      System.out.println("\n ServicesImpl::registerUsesPort : " + portInfo.getName() + " :" + " fqn = " + fqn);
      c = Class.forName(fqn);
    } catch(Exception e) {
      System.out.println("ServicesImpl::registerUsesPort: CCAT framework is unaware of the type you are trying to register " + fqn);
      e.printStackTrace();
      c = null;
    }
    if(c == null) {
      System.out.println("ServicesImpl:: registerUsesPort bad port type ! ");
      return;
    } else {
      try {
	UsesPort stub = (UsesPort) c.newInstance();
	UsesPortlist.addElement(new PortRecord((Port)stub, portInfo));
      } catch(Exception e){
	System.out.println("ServicesImpl:: registerUsesPort: bummer !"+e);
      }
    }
    return;
  }

  // Takes part in the blocking semantics. Decrements the reference count.
  public void releasePort(String nameOfPort) {
    UsesPortImpl up = getUsesPortImpl (nameOfPort);
    if (up != null) {
      synchronized (up) {	 
	up.decrRefCount();
      }
    }
  }

  // Called by the ComponentID in a remote process. This method is used in the
  // ConnectionServiceImplementation. It gets a uses port, adds the provides
  // port as the listener and then wakes up any threads waiting for a connection 
  // to be made.
  public int ccatConnect(String usesPortName, Port providesPort) throws Exception {
    synchronized(this) {
      try {
	UsesPortImpl usesPort = getUsesPortImpl (usesPortName);
	if (usesPort != null) {
	  synchronized (usesPort) {
	    int result = -1;
	    if (usesPort.hasAllVoidMethods () ||
		(!usesPort.hasAllVoidMethods () && (usesPort.getRefCount() ==
						    0))) {
	      result = usesPort.addListener((ProvidesPort)providesPort);
	      usesPort.notifyAll();
	    }
	    else
	      System.out.println ("ServerciesImpl.ccatConnect: uses port" +
				  usesPortName + " is locked, cannot connect");
	    return result;
	  }
	}
	else {
	  System.out.println ("ServerciesImpl.ccatConnect: failed to find uses port: " 
+ usesPortName);
	  return -1;
	}
      } catch (Exception e) {
	e.printStackTrace();
	return 0;
      }

      //	UsesPort usesPort = (UsesPort)getPort(usesPortName);
      //	int result = usesPort.addListener((ProvidesPort)providesPort);
      //	return result;
      //      } catch (Exception e) {
      //	e.printStackTrace();
      //	return 0;
      //      }
    }
  }

  // Called by the ComponentID in a remote process.
  // This method is used in the ConnectionServiceImplementation.
  public int ccatDisconnect(String usesPortName, Port providesPort) throws Exception {
    synchronized(this) {
      try {
	UsesPortImpl usesPort = getUsesPortImpl (usesPortName);
	if (usesPort != null) {
	  synchronized (usesPort) {
	    int result = -1;
	    if (usesPort.hasAllVoidMethods () ||
		(!usesPort.hasAllVoidMethods () && (usesPort.getRefCount() == 0)))
	      result = usesPort.removeListener((ProvidesPort)providesPort);
	    else
	      System.out.println ("ServerciesImpl.ccatDisconnect: uses port" +
				  usesPortName + " is locked, cannot disconncted");
	    return result;
	  }
	}
	else {
	  System.out.println ("ServerciesImpl.ccatDisconnect: failed to find uses port: " 
+ usesPortName);
	  return -1;
	}
      } catch (Exception e) {
	e.printStackTrace();
	return 0;
      }
	 


      //      try {
      //	UsesPort usesPort = (UsesPort)getPort(usesPortName);
      //	int result = usesPort.removeListener((ProvidesPort)providesPort);
      //	return result;
      //      } catch (Exception e) {
      //	e.printStackTrace();
      //	return 0;
      //      }
    }
  }

  // Called by the ComponentID in a remote process.
  // This method is used in the ConnectionServiceImplementation.
  public Port ccatGetProvidesPort(String providesPortName) throws Exception {
    synchronized(this) {
      try {
	Port providesPort = getPort(providesPortName);
	return providesPort;
      } catch (CCAException e) {
	e.printStackTrace();
	return null;
      } catch (Exception e) {
	e.printStackTrace();
	return null;
      }
    }
  }

  // Export the given port with the given portInfo. Note that this is not an
  // actual port implementation object but a proxy for some provides port
  // implementation in another context.
  public int ccatExportAs(DefaultPortInfo newPortInfo, Port providesPort) 
       throws Exception {
	 // the providesPort you get will be actually of a NexusRMI stub type. For
	 // example, if the provides port is of type string_idl, then it will be of
	 // type ProvidesStringSkel_Stub.java. This class does satisfy the
	 // ProvidesString interface, which in turn implements the string_idl
	 // interface. So, it's safe to add this object itself in the
	 // ProvidesPortlist.
	 ProvidesPortlist.addElement(new PortRecord(providesPort, newPortInfo));
	 return 0;
  }

  // This is the same as ccatConnect, so just call ccatConnect. We should do the 
  // same optimization for the C++ implementation. Also, there is almost no need 
  // for a ccatProvideTo in the first place, just ccatConnect will do. FIXME
  public int ccatProvideTo(String usesPortName, Port providesPort) 
       throws Exception {
	 return ccatConnect (usesPortName, providesPort);
  }

  public String ccatGetPortTypeForPortName(String portName) throws Exception {
    System.out.println ("ServerciesImpl::ccatGetPortTypeForPortName: " +
			" Port Name: " + portName);

    int n = ProvidesPortlist.size();
    for(int i = 0; i < n; i++) {
      DefaultPortInfo info = 
	(DefaultPortInfo) ((PortRecord) ProvidesPortlist.elementAt(i)).pinfo;
      if (info.getName().equals (portName)) {
	System.out.println ("ServerciesImpl::ccatGetPortTypeForPortName: " +
			    " Type found: " + info.getType());
	return info.getType();
      }
    }
    System.out.println ("ServerciesImpl::ccatGetPortTypeForPortName: " +
			" No matching port name found");
    return new String("");
  }

  // Will not be used as we will be using a more
  // sophisticated events model instead
  public void addCCAConnectionListener(CCAConnectionListener l) {
  }

  // Used usually by the creation service
  // to return a handle to the instantiated
  // process.
  public ComponentID getComponentID() {
    try {
      return new ComponentIDImpl(this);
    } catch (nexusrmi.RemoteException e) {
      e.printStackTrace();
      return null;
    } catch (Exception ex) {
      ex.printStackTrace();
      return null;
    }
  }

//   // **TEMPORARY** measure
//   // This is not a part of the CCAspec.
//   // Used by the eventChannelServer
//   // Will be removed shortly- Madhu
  public void startRunning() throws Exception {
    synchronized(this) {
      while (!runCalled) {
	try {
	  wait();
	} catch (Exception e) {
	  e.printStackTrace();
	}
      }
      // depending on how you want to execute it
      // reset the bool flag
      // runCalled = false;
    }
  }

}





    
