
package compositionEngine;

import ccacore.*;
import creation.*;
import connection.*;
import registry.*;
import util.*;
import idl.control.UsesControl;

import java.io.*;
import java.net.*;
import nexusrmi.*;
import nexusrmi.server.*;

public class Compose {
  private String contextName = "http";
  public static final String RELEASE = "$Revision: 1.6 $ by $Author: mgovinda $ on $Date: 1999/11/09 18:18:52 $ (GMT)";
  
  public void runit() {
    try {
      long t1, t2;
      // Yeah ! right, timing this program is important 
      t1 = System.currentTimeMillis();

      // The Gui would pick the env object from the IDS
      // and use that. For now create it by hand.
      // initialize the creation env for the
      // stringGenerator instance
      EnvObj envStringGen = new EnvObj();

      // Some of the keys that I am adding here
      // are not not used to create the Info Object.
      // However that's okay. The extra elements
      // are used only by the creation service.
      envStringGen.put("username","mgovinda");
      envStringGen.put("LD_LIBRARY_PATH","/foo");
      envStringGen.put("PATH",".:/bin:/usr/bin:/usr/local/bin");
      envStringGen.put("WD","/l/extreme/components/workingdir");
      envStringGen.put("TMPDIR","/tmp");
      envStringGen.put("SHELL","/bin/sh");
      envStringGen.put("registry-host","lookout.extreme.indiana.edu");
      envStringGen.put("registry-port","4242");
      String creationTime = new java.util.Date().toString().replace(' ','_');
      envStringGen.put("creation-time",creationTime);
      envStringGen.put("creation-mech","rsh");
      envStringGen.put("creation-mech-path","/usr/bin/rsh");
      envStringGen.put("host","school.cs.indiana.edu");
      envStringGen.put("exec-path", "/tmp/madhu/java/JavaStringGenComponent.sh");
      envStringGen.put("context-name", contextName);
      envStringGen.put("instance-name", "stringGeneratorInstance1st");
      envStringGen.put("port-for-component-callback", "3456");
      System.out.println("Creation env is "+envStringGen);
    

      // Some of the keys that I am adding here
      // are not not used to create the Info Object.
      // However that's okay. The extra elements
      // are used only by the creation service.
      // Initialize the creation environment for
      // the printer component.
      EnvObj envPrinter = new EnvObj();
      envPrinter.put("username","mgovinda");
      envPrinter.put("LD_LIBRARY_PATH","/foo");
      envPrinter.put("PATH",".:/bin:/usr/bin:/usr/local/bin");
      envPrinter.put("WD","/l/extreme/components/workingdir");
      envPrinter.put("TMPDIR","/tmp");
      envPrinter.put("SHELL","/bin/sh");
      envPrinter.put("registry-host","lookout.extreme.indiana.edu");
      envPrinter.put("registry-port","4242");
      String creationTime1 = new java.util.Date().toString().replace(' ','_');
      envPrinter.put("creation-time",creationTime1);
      envPrinter.put("creation-mech","rsh");
      envPrinter.put("creation-mech-path","/usr/bin/rsh");
      envPrinter.put("host","school.cs.indiana.edu");
      envPrinter.put("exec-path", "/tmp/madhu/java/JavaPrinterComponent.sh");
      envPrinter.put("context-name", contextName);
      envPrinter.put("instance-name", "printerInstance1st");
      envPrinter.put("port-for-component-callback", "4567");
      System.out.println("Creation env is "+envPrinter);
    
      Services core = new ServicesImpl();
      ComponentID compositionEngineCompID = core.getComponentID();
      CreationService creationService = (CreationService)core.getPort("CreationService");
      System.out.println("\nCompose::calling creation Service for stringGenCompID ");
      ComponentID stringGenCompID = creationService.createInstance(envStringGen);
      System.out.println("\nCompose::calling creation Service for printerCompID ");
      ComponentID printerCompID = creationService.createInstance(envPrinter);
      

//       // Binding is optional. The composition engine needs to decide whether
//       // or not to bind these references to the registry. Right now these are
//       // placed here to make sure that the bind and lookup calls work.

      // Bind the printer instance
      RegistryService registryService = (RegistryService)core.getPort("RegistryService");
      String printerRegistryContextName = envPrinter.get("context-name") + "://" + envPrinter.get("registry-host") + "/registry_test";
      RegistryContext printerContext = registryService.getContext(printerRegistryContextName);
      printerContext.bind(printerCompID, envPrinter.get("instance-name"), envPrinter);
      
      
      // Bind the stringGenerator instance
      String stringGenRegistryContextName = envStringGen.get("context-name") + "://" + envStringGen.get("registry-host") + "/registry_test";
      RegistryContext stringGenContext = registryService.getContext(stringGenRegistryContextName);
      stringGenContext.bind(stringGenCompID, envStringGen.get("instance-name"), envStringGen);
      
      int fooledYou = 1;
      while(fooledYou != 0) {
	try {
	Thread.currentThread().sleep(100000);
	} catch (Exception e) {
	  e.printStackTrace();
	}
      }
      
//       // Lookup the printer instance 
//       printerCompID lookupComponentInstance(printerRegistryContext, envPrinter.get("instance-name"), envPrinter.get("username"));

//       // Lookup the string Generatorinstance 
//       stringGenCompID lookupComponentInstance(stringGenRegistryContext, envStringGen.get("instance-name"), envStringGen.get("username"));
      
 
      // Connect the two components 
      ConnectionService connectionService = (ConnectionService)core.getPort("ConnectionService");
      System.out.println("\nCompose::calling connection Service connect");
      connectionService.connect(stringGenCompID, "stringOutPortName", printerCompID, "stringInPortName");
      System.out.println("\nCompose::connection Service connect succeeded ");


      // how else do I start the upstream component ? The old CAT controlled
      // every component's execution. Now the control is not so centralized.
      // The composition engine just connects the components and asks the top
      // component (most upstream) to start executing. Its a data flow model
      // from that point on. 
      core.registerUsesPort(new DefaultPortInfo("control", "Control_idl"));
      
      System.out.println("\nCompose::Connection service connecting control ports");
      connectionService.connect(compositionEngineCompID, "control", stringGenCompID, "control");
      System.out.println("\nCompose::Connection service connecting control ports ... succeeded");
      
      UsesControl controlPort = (UsesControl) (core.getPort("control"));
      System.out.println("\nCompose::controlPort.startExecuting()");
      controlPort.startExecuting();
      System.out.println("\nCompose::controlPort.startExecuting() done");
      core.releasePort("control");

//       int fooledYou = 1;
//       while (fooledYou != 0) {
// 	int x  = fooledYou;
//       }

      System.out.println("\nCompose::calling connection Service disconnect ");
      double timeout = 300;
      connectionService.disconnect(stringGenCompID, "stringOutPortName", printerCompID, "stringInPortName", timeout);

      connectionService.disconnect(compositionEngineCompID, "control", stringGenCompID, "control", timeout);
      System.out.println("\nCompose::calling connection Service disconnect done ");
      
      t2 = System.currentTimeMillis();
      System.out.println("Compose::Elapsed time: " + (t2 - t1) + " msec.");
    }  catch(Exception e) {
      System.out.println("run exception"+e);
    }
  }
  

  ComponentID lookupComponentInstance(RegistryContext context, String instanceName, String currentUser) {
//     boolean foundInstance = false;
//     ComponentID componentInstance = null;
//     try {
// 	InstanceInfo [] instances = context.lookup(instanceName);
// 	System.out.println("There are "+instances.length+" instances with that
// name in this context, trying to get the one created by " + currentUser);
// 	for (int i=0; i<instances.length; i++) {
// 	  InstanceInfo currentInstance = instances[i];
// 	  String userName = currentInstance.getInfo("instance-info")[0].getValue("creator")[0];
// 	  if (userName.equals(currentUser)) {
// 	    System.out.println("Instance #"+(i+1)+" created by " + currentUser + ", details on instance are\n"+currentInstance);
// 	    foundInstance = true;
// 	    componentInstance =  currentInstance.getComponentID();
// 	    break;
// 	  }
// 	}
//       } catch (Exception e) {
// 	System.out.print("CreationServiceImpl::lookupComponentInstance Exception received searching for instanceName : " + instanceName);
//       }
//       if (foundInstance) {
// 	System.out.print("CreationServiceImpl::lookupComponentInstance: Failed in search for instanceName : " + instanceName);
//       }
//      return componentInstance;
    return null;
  }
  
  public static void main(String[] args){
    System.out.println(RELEASE);
    Compose compose = new Compose();
    compose.runit();
    System.out.println("\n Compose: End of test \n");
    System.runFinalizersOnExit(true);
    System.runFinalization();
  }
}









