
package compositionEngine;

import ccacore.*;
import creation.*;
import connection.*;
import registry.*;
import eventChannel.FilteringEventChannelComponent;
import idl.filteringEventChannel.UsesFilteringEventChannel;
import util.*;


import java.io.*;
import java.net.*;
import nexusrmi.*;
import nexusrmi.server.*;

public class TestLSA {

  private String contextName = "http";
  public static final String RELEASE = "$Revision: 1.2 $ by $Author: mgovinda $ on $Date: 1999/11/11 07:40:56 $ (GMT)";
  
  public void runit() {
    try {
      long t1, t2;
    
      t1 = System.currentTimeMillis();

      FilteringEventChannelComponent eventChannel = new FilteringEventChannelComponent();
      ServicesImpl eventChannelCore = new ServicesImpl();
      eventChannel.setServices(eventChannelCore);
      ComponentID eventChannelComponentID = eventChannelCore.getComponentID();

      Services myCore = new ServicesImpl(eventChannelComponentID);
      ComponentID myComponentID = myCore.getComponentID();
      
      // create an output port that will be used to
      // register with the evenChannel
      myCore.registerUsesPort(new DefaultPortInfo("filteringEventChannelPort", "FilteringEventChannel_idl"));

      // now add a provides port that would listen to all the events
      // from the event channel. EventListener is a class in this package.
      EventListener eventListener = new EventListener();
      myCore.addProvidesPort(eventListener, new DefaultPortInfo("guiEventListener", "EventPort_idl"));

      // get the connection service to connect yourself to the
      // event Channel
      ConnectionService connectionService = (ConnectionService)myCore.getPort("ConnectionService");
      
      System.out.println("\nTestLSA::calling connection Service connect ");
      connectionService.connect(myComponentID, "filteringEventChannelPort", eventChannelComponentID, "register");
      System.out.println("\nTestLSA::connection Service connect succeeded ");
      
      UsesFilteringEventChannel outputPort = (UsesFilteringEventChannel)(myCore.getPort("filteringEventChannelPort"));

      // now send this information to the event Channel that
      // that the GUI wants to listen to all "cca*" type events
      
      outputPort.registerListener(myComponentID, "guiEventListener", "cca");
      // initialize the creation env for an instance

      myCore.releasePort("filteringEventChannelPort");

      EnvObj envGSFF = new EnvObj();

      // Some of the keys that I am adding here
      // are not not used to create the Info Object.
      // However that's okay. The extra elements
      // are used only by the creation service 
      envGSFF.put("username","mgovinda");
      envGSFF.put("LD_LIBRARY_PATH","/foo");
      envGSFF.put("PATH",".:/bin:/usr/bin:/usr/local/bin");
      envGSFF.put("WD","/l/extreme/components/workingdir");
      envGSFF.put("TMPDIR","/tmp");
      envGSFF.put("SHELL","/bin/sh");
      envGSFF.put("registry-host","lookout.extreme.indiana.edu");
      envGSFF.put("registry-port","4242");
      String creationTime = new java.util.Date().toString().replace(' ','_');
      envGSFF.put("creation-time",creationTime);
      envGSFF.put("creation-proto","gram");
      envGSFF.put("creation-mech-path","/usr/local/bin/ssh");
      envGSFF.put("exec-host","gromit.cs.indiana.edu");
      envGSFF.put("exec-path", "/tmp/madhu/execs/");
      envGSFF.put("exec-name", "getSystemFromFile");
      envGSFF.put("globus-run-script-path", "/tmp/madhu/scripts/gromitGlobusRun.sh");
      envGSFF.put("context-name", contextName);
      envGSFF.put("instance-name", "gsffInstance");
      envGSFF.put("port-for-component-callback", "1234");

      System.out.println("TestLSA:: Creation env is "+envGSFF);
    

      // Some of the keys that I am adding here
      // are not not used to create the Info Object.
      // However that's okay. The extra elements
      // are used only by the creation service 
      EnvObj envSplib = new EnvObj();
      envSplib.put("username","mgovinda");
      envSplib.put("LD_LIBRARY_PATH","/foo");
      envSplib.put("PATH",".:/bin:/usr/bin:/usr/local/bin");
      envSplib.put("WD","/l/extreme/components/workingdir");
      envSplib.put("TMPDIR","/tmp");
      envSplib.put("SHELL","/bin/sh");
      envSplib.put("registry-host","lookout.extreme.indiana.edu");
      envSplib.put("registry-port","4242");
      String creationTime1 = new java.util.Date().toString().replace(' ','_');
      envSplib.put("creation-time",creationTime1);
      envSplib.put("creation-proto","gram");
      envSplib.put("creation-mech-path","/usr/local/bin/ssh");
      envSplib.put("exec-host","gromit.cs.indiana.edu");
      envSplib.put("exec-path", "/tmp/madhu/execs/");
      envSplib.put("exec-name", "splib");
      envSplib.put("globus-run-script-path", "/tmp/madhu/scripts/gromitGlobusRun.sh");
      envSplib.put("context-name", contextName);
      envSplib.put("instance-name", "splibInstance");
      envSplib.put("port-for-component-callback", "2345");

      System.out.println("TestLSA:: Creation env is "+envSplib);
    

      CreationService creationService = (CreationService)myCore.getPort("CreationService");
      System.out.println("\nTestLSA::calling creation Service for gsff ");
      ComponentID gsffCompID = creationService.createInstance(envGSFF);
      System.out.println("\nTestLSA::testing gsff compID");
      System.out.println(" gsffCompID name is " + ((ComponentIDregistry) gsffCompID).getName());

      
      System.out.println("\nTestLSA::calling creation Service for splib ");
      ComponentID splibCompID = creationService.createInstance(envSplib);

      ComponentID testLSAComponentID = myCore.getComponentID();
      

      
      System.out.println("\nTestLSA::calling connection Service connect ");
      connectionService.connect(gsffCompID, "outputSLS", splibCompID, "inputSLS");
      System.out.println("\nTestLSA::connection Service connect succeeded ");


      t2 = System.currentTimeMillis();
      System.out.println("TestLSA::Elapsed time: " + (t2 - t1) + " msec.");
    }  catch(Exception e) {
      e.printStackTrace();
      System.out.println("TestLSA::run exception"+e);
    }
  }


  ComponentID lookupComponentInstance(RegistryContext context, String instanceName, String currentUser) {
    //     boolean foundInstance = false;
    //     ComponentID componentInstance = null;
    //     try {
    // 	InstanceInfo [] instances = context.lookup(instanceName);
    // 	System.out.println("There are "+instances.length+" instances with that
    // name in this context, trying to get the one created by " + currentUser);
    // 	for (int i=0; i<instances.length; i++) {
    // 	  InstanceInfo currentInstance = instances[i];
    // 	  String userName = currentInstance.getInfo("instance-info")[0].getValue("creator")[0];
    // 	  if (userName.equals(currentUser)) {
    // 	    System.out.println("Instance #"+(i+1)+" created by " + currentUser + ", details on instance are\n"+currentInstance);
    // 	    foundInstance = true;
    // 	    componentInstance =  currentInstance.getComponentID();
    // 	    break;
    // 	  }
    // 	}
    //       } catch (Exception e) {
    // 	System.out.print("CreationServiceImpl::lookupComponentInstance Exception received searching for instanceName : " + instanceName);
    //       }
    //       if (foundInstance) {
    // 	System.out.print("CreationServiceImpl::lookupComponentInstance: Failed in search for instanceName : " + instanceName);
    //       }
    //      return componentInstance;
    return null;
  }
  
  public static void main(String[] args){
    System.out.println(RELEASE);
    TestLSA testLSA= new TestLSA();
    testLSA.runit();
    System.out.println("\n TestLSA: End of test \n");
    System.runFinalizersOnExit(true);
    System.runFinalization();
  }
}









