
//  ConnectionServiceImpl implements the ConnectionServices
//  interface. It invokes methods from the ComponentIDprivate
//  interface to achieve its functionality. The User is not
//  exposed to the implementation details of the framework or
//  the ConnectionService.
//  --Madhu
 
package connection;

import idl.eventPort.UsesEventPortSkel;
import idl.eventPort.CCATEvent;
import ccacore.ComponentID;
import ccacore.Port;
import ccacore.UsesPort;
import ccacore.ComponentIDconnection;
import ccacore.DefaultPortInfo;
import exceptions.ConnectionException;
import exceptions.DisconnectionException;
import exceptions.ExportAsException;
import exceptions.ProvideToException;

public class ConnectionServiceImpl implements ConnectionService, ConnectionServicePrivate {

  private UsesEventPortSkel usesEventPort = null;

  public void setEventUsesPort(UsesEventPortSkel usesEventPort_) {
    //    usesEventPort = (UsesEventPort)usesEventPort_;
    usesEventPort = usesEventPort_;
  }

  public void connect(ComponentID idUses, String usesPortName, ComponentID idProv, String provPortName) throws ConnectionException {
    // instead of wrapping this entire
    // block with this exception, catch
    // specific exceptions in each place
    // FIXM FIXME
    try {
      System.out.println("\n ConnectionService::connect: attempting ccatGetProvidesPort");
      Port providesPort = ((ComponentIDconnection)idProv).ccatGetProvidesPort(provPortName);
      System.out.println("\n ConnectionService::connect: ccatGetProvidesPort done");
      System.out.println("\n ConnectionService::connect: attempting ccatConnect done");
      int result = ((ComponentIDconnection)idUses).ccatConnect(usesPortName, providesPort);
      System.out.println("\n ConnectionService::connect: ccatConnect done");

      // send an event to the eventChannel that
      // the component was successfully created.
      // If the usesEventPort is not set then a
      // null pointer exception will be thrown.
      // I don't want to catch this run time exception.
      String details = usesPortName+":"+provPortName;
      String source = "CONNECTION_SERVICE";
      String type = "cca.connection";
      String key = "GET A LOCK FIRST"; // not used now
      long seqNum = 007; // not used now
      CCATEvent ccatEvent = new CCATEvent(source, type, key, seqNum, details);
      if (usesEventPort == null) {
	  //System.out.println("\nExiting as eventChannel connection is not right ");
	  //System.exit(-1);
	  // don't notify
	  return;
      }
      usesEventPort.notify(ccatEvent);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void disconnect(ComponentID idUses, String usesPortName, ComponentID idProv, String provPortName, double timeout) throws DisconnectionException {
    try {
      Port providesPort = ((ComponentIDconnection)idProv).ccatGetProvidesPort(provPortName);
      int result = ((ComponentIDconnection)idUses).ccatDisconnect(usesPortName, providesPort);
      // send an event to the eventChannel that
      // the component was successfully created.
      // If the usesEventPort is not set then a
      // null pointer exception will be thrown.
      // I don't want to catch this run time exception.
      String details = usesPortName+":"+provPortName;
      String source = "CONNECTION_SERVICE";
      String type = "cca.disconnection";
      String key = "GET A LOCK FIRST"; // not used now
      long seqNum = 007; // not used now
      CCATEvent ccatEvent = new CCATEvent(source, type, key, seqNum, details);    
      usesEventPort.notify(ccatEvent);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  // export idSub's provides port with name "portName" as idMe's provides port
  // with name "newPortName". Currently it's not implemented for uses
  // port. FIXME
  public void exportAs(ComponentID idMe, ComponentID idSub, String portName, 
				   String newPortName) 
	 throws ExportAsException {

    System.out.println("CCATServicesImpl:exportAs Port:" + portName + " as Port:" + 
				   newPortName);
    Port providesPort = null;
    String portType = null;
    try {
	 providesPort =
	   ((ComponentIDconnection)idSub).ccatGetProvidesPort(portName);
	 
	 // get the portType for this port
	 portType = 
	   ((ComponentIDconnection)idSub).ccatGetPortTypeForPortName(portName);
	 System.out.println ("CCATServicesImpl: exportAs: the portType found is: " + 
					 portType);
    
	 // get the port info for this port
	 DefaultPortInfo newPortInfo = new DefaultPortInfo (newPortName, portType);
	 
	 ((ComponentIDconnection)idMe).ccatExportAs (newPortInfo, providesPort);
    }
    catch (Exception e) {
	 e.printStackTrace ();
    }
  }

  // Provide the provides port "providesPort" to the componentID idSub's uses
  // port named "usesPortName".
  public void provideTo(ComponentID idSub, String usesPortName, Port providesPort) 
    throws ProvideToException {
    System.out.println("CCATServicesImpl:provideTo usesPortName:" +
				   usesPortName);
    try {
	 ((ComponentIDconnection)idSub).ccatProvideTo (usesPortName, providesPort);
    }
    catch (Exception e) {
	 e.printStackTrace ();
    }	 
  }
}




