/**
 * This is the internal implementation of a context node.
 * It stores the name of the context and implements 
 * the InformationNode interface. It is the base class
 * for any kind of information context.
 * @author Nirmal (nmukhi)
 *
 */

package ids;

public abstract class Context implements InformationDirectoryNode {
    private String name;

    /**
     * Create a context node with the specified name
     *
     */
    public Context(String name) {
	if (!name.endsWith("/"))
	    name+="/";
	this.name = name;
    }


    // implementation of the InformationDirectoryNode interface

    /**
     * Gets the name of this node
     * @returns the name of the node.
     */
    public String getName() {
	return name;
    }

    /**
     * Checks if this node corresponds to information
     * @returns 0 since this node is a context
     */
    public int isInformation() {
	return 0;
    }

    /**
     * Checks if this node corresponds to a context
     * @returns 1 since this node is a context
     */
    public int isContext() {
	return 1;
    }
}
              
    
