/**
 * This is a system-provided port available to all CCAT components
 * that can be used to access a a directory containing information
 * about components and other resources.
 * Components can use this interface to get a handle to any information 
 * directory context, irrespective of the location of the registry, the 
 * mechanism it uses to store instances and the protocol used to transfer 
 * information from that registry.
 * Details on the API are available from
 * <a href="http://www.extreme.indiana.edu/ccat/directory_service.html">here</a>.
 * @author Nirmal (nmukhi)
 * @see InformationDirectoryContext
 */

package ids;

import ccacore.Port;
import registry.NotFoundException;

public interface InformationDirectoryService extends Port {

    /**
     * Gets a context for a directory. 
     * @param context the address of the context, for example
     * http://lookout.extreme.indiana.edu:4242/ccat/components
     * @return a context object for the named context
     * @exception NotFoundException thrown if the specified context
     * did not correspond to an actual information directory
     * @exception ServerNotFoundException thrown if the server could
     * not be located
     */    
    public InformationDirectoryContext getContext(String context) 
	throws NotFoundException, ServerNotFoundException;
}
             
