/**
 * This is an interface into an item of information stored in the
 * information directory.
 * Information can be accessed using the appropriate key.
 * Some of the information is just in the form of name-value
 * pairs, other information is accessed by providing a name which
 * returns a list of <var>Info</var> objects.
 * @author Nirmal (nmukhi)
 */

package ids;

import util.Info;
import util.EnvObj;

public interface InformationInfo extends InformationDirectoryNode, Info { 

    /**
     * This method uses the information describing this component
     * to get a creation environment suitable for instantiation of
     * that component on the specified machine using the specified
     * creation mechanism.<br>
     * Note that this will not be a complete creation environment 
     * and some things may have to be added before it can be used 
     * for creation.
     * @return a partial creation environment suitable for instantiation
     * of the component described in this information
     */
    public EnvObj getCreationEnvironment(String machine, String creationMechanism);
}
