/**
 * This class is an implementation of the 
 * <var>InstanceInfo</var> interface. It allows access to
 * and instance's <var>ComponentID</var> and other
 * instance information stored in name/value pairs.
 * @author Nirmal (nmukhi)
 */

package registry;

import ccacore.ComponentID;
import util.InfoImplementation;
import nexusrmi.Remote;

public class InstanceInfoImplementation extends InfoImplementation implements InstanceInfo {
    private ComponentID id;
    private String name;

    /** 
     * Creates information for an instance given the instance's
     * component ID and name.
     * @param id the instance's <var>ComponentID</var>
     * @param name the instance's name
     */
    public InstanceInfoImplementation(ComponentID id, String name) {
	// default InfoImplementation constructor
	super();
	// set the name
	this.name = name;
	// remember the component ID
	this.id = id;
    }

    /**
     * Creates information for an instance given the instance's
     * component ID, name and an <var>InfoImplementation</var>
     * object containing the associated hierarchical information.
     */
    public InstanceInfoImplementation(ComponentID id, String name, 
				      InfoImplementation info) {
	// store the given info
	super(info);
	// set the name
	this.name = name;
	// set the id
	this.id = id;
    }

    /**
     * Gets a handle to the component instance this object 
     * refers to.
     * @return the <var>ComponentID</var> for the instance
     */
    public ComponentID getComponentID() {
	return id;
    }

    // implementation of RegistryNode interface

    /**
     * Gets the name of this node
     * @return the name of the node.
     */
    public String getName() {
	return name;
    }

    /**
     * @return 1 since this refers to an instance
     */
    public int isInstance() {
	return 1;
    }
    
    /**
     * @return 0 since this does not refer to a context
     */
    public int isContext() {
	return 0;
    }

    /**
     * Convert this information to a formatted string
     */
    public String toString() {
	StringBuffer sb = new StringBuffer();
	sb.append("Instance name: ");
	sb.append(name);
	sb.append("\nComponent ID: ");
	sb.append(id);
	sb.append("\n");
	sb.append(super.toString());
	return sb.toString();
    }
}
