/**
 * This is a system-provided port accessible to all CCAT components
 * that can be used to access a registry.
 * Components can use this interface to get a handle to any registry
 * context, irrespective of the location of the registry, the mechanism 
 * it uses to store instances and the protocol used to transfer 
 * information from that registry.
 * Details on the API are available from
 * <a href="http://www.extreme.indiana.edu/ccat/registry_service.html">here</a>.
 * @author Nirmal (nmukhi)
 * @see RegistryContext
 */

package registry;

import ccacore.Port;

public interface RegistryService extends Port {

    /**
     * Gets a context for a registry
     * @param context the address of the context, for example
     * http://lookout.extreme.indiana.edu:4242/ccat/instances
     * @return a context object for the named context
     * @exception NotFoundException thrown if the specified context
     * did not correspond to an actual registry
     * @exception BadContextNameException if the specified context
     * corresponded to a node in the registry which was not a valid
     * context
     */    
    public RegistryContext getContext(String context) 
	throws NotFoundException, BadContextNameException;
}
             
