/**
 * This represents a set of name-value pairs.
 * Names are just strings, values can be strings or
 * a list of Info objects, thus allowing a tree of information
 * to be represented using this class.
 * @author Nirmal (nmukhi)
 *
 */

package util;

public interface Info {
    /**
     * Gets the list of values mapped to the supplied key.
     * @param key the key
     * @return the values corresponding to <var>key</var>.
     */
    public String [] getValue(String key);

    /**
     * Gets the <var>Info</var> objects corresponding to
     * the given key. 
     * @param key the key
     * @return the list of <var>Info</var> objects 
     * that <var>key</var> maps to
     */
    public Info [] getInfo(String key);

    /**
     * Gets the keys used to access information 
     * stored in this <var>Info</var> object
     * @return a list of all the keys mapped to
     * anything in this object
     */
    public String [] getKeys();
}
