
package util;

import nexus.*;
import java.lang.reflect.*;
import nexusrmi.Communication;
import ccacore.ComponentID;
import ccacore.ComponentIDregistry;

public class NexusConnectionServer implements HandlerInterface, AttachApprovalInterface {

  private Nexus nexus;
  private EndpointAttr epattr = null;
  private Endpoint ep = null;
  private Startpoint thisServerStartpoint = null;
  private Startpoint componentIDprocessStartpoint = null;
  private Startpoint componentIDstartpoint = null;
  private boolean RecdFromClient = false;
  private boolean recdComponentID = false;
  private boolean gotShutdown = false;
  private PutBuffer replyBuffer;
  private ComponentID componentIDofInstantiatedComponent;
  private int portForCallBack = 0;
  private int RETURN_COMPONENT_ID = 4000;
  private int HPCXX_OFFSET = 0;
  private int HPCXX_GF_HANDLER = 0;
  
  public NexusConnectionServer() {
    portForCallBack = this.start();
    (new Thread() {
      public void run() {
	waitForShutdown();
      }
    }).start();
  }
  
  public int getPortForComponentCallBack() {
    return portForCallBack;
  }
  
  public ComponentID getInstantiatedComponentID() {
    waitForClient();
    waitForComponentID();
    createStubFromStartpoint(componentIDstartpoint);
    return componentIDofInstantiatedComponent;
  }
  
  public int start() {
    
    String hostname;
    String[] notUsedArray = null;
    nexus = nexusrmi.Communication.getNexus();
    Integer notUsedInt = new Integer(12);
    int port = nexus.allow_attach(0, this, notUsedInt);
    hostname = nexus.attach_hostname();
    System.out.println("NexusConnectionServer::allow_attach() returned, listening on "+hostname+"/"+port);
    return port;
  } /* start() */

  
  // for apporoval interface implementation 
  public AttachReturn attach_approval (Object userArg, String url ) {
    // This endpoint is what the startpoint
    // binds to before returning it to the client
    epattr = nexus.endpointattr_init();
    epattr.set_handler_object(this);
    ep = nexus.endpoint_init(epattr);
    thisServerStartpoint = nexus.startpoint_bind(ep);
//     System.out.println("\nNexusConnectionServer:url sent by client is: " + url);
//     System.out.println("\nNexusConnectionServer:creationServiceStartpoint is: " + thisServerStartpoint);
    return (new AttachReturn(0, thisServerStartpoint));
  }

  // called when a client issues a send_request 
  public void invoke_handler(Endpoint ep, GetBuffer buffer , int handler_id ) {
//     System.out.println("\n NexusConnectionServer:: invoke_handler called !!! ");
    switch (handler_id) {
    case 0: {
      try {
//  	System.out.println("\nNexusConnectionServer::invoke_handler(): received message from Client");
	System.out.flush();
	retrieveClientBuffer(buffer);
	stopWaitingForClient(true);
	break;
      } catch ( Exception e ) {
	e.printStackTrace();
      }
    }
    case 5: {
//       System.out.println("\nNexusConnectionServer::invoke_handler(): received message from Client on 5");
      System.out.flush();
      setComponentIDstartpoint(buffer);
      stopWaitingForComponentID(true);
      break;
    }
    default:
      System.out.println("NexusConnectionServer:invoke_handler() : Error : handler not recognized within the object");
      break;
    }
  }

  
  private void retrieveClientBuffer(GetBuffer buffer ) {
    try {
      int dummyForBen = buffer.get_int();
      componentIDprocessStartpoint = buffer.get_startpoint();
      sendArequestForComponentID(componentIDprocessStartpoint);
    } catch ( Exception e ) {
      System.out.println("NexusConnectionServer: Error in retrieving buffer from client ");
    } 
  }

  // componentID startpoint is a class data member
  // so once this method sets it, we can allow
  // createStubFromStarptoint(...) to be called.
  private void setComponentIDstartpoint(GetBuffer buffer) {
    try {
      int methodID = buffer.get_int();
//       System.out.println("\n methodID is  :" + methodID);
      byte five = buffer.get_byte();
//       System.out.println("\n byte five :" + five);
      byte two = buffer.get_byte();
//       System.out.println("\n byte two :" + two);
      int lengthOfFQN = buffer.get_int();
//       System.out.println("\n length :" + lengthOfFQN);
      char[] FQN = new char[lengthOfFQN];
      buffer.get_char(FQN, 0, lengthOfFQN);
      two = buffer.get_byte();
//       System.out.println("\n two :" + two);
      componentIDstartpoint = buffer.get_startpoint();
      // just ignore the rest of the buffer
      // Am interested just in the startpoint
    } catch (Exception e) {
      System.out.println("NexusConnectionServer: Error in retrieving componenIDstartpoin");
    }
  }
  
  private void sendArequestForComponentID(Startpoint processStartpoint) {
    PutBuffer buffer;    
    int size = thisServerStartpoint.sizeof() + nexus.sizeof_int(2);
    buffer = nexus.buffer_init(size,0);
    try {
	// FIXME
	// damn it, do what you say you should do
      // could just use thisServerStartpoint
      epattr = nexus.endpointattr_init();
      epattr.set_handler_object(this);
      ep = nexus.endpoint_init(epattr);
      Startpoint newStartpoint = nexus.startpoint_bind(ep);
      newStartpoint.put(buffer);
      buffer.put_int(HPCXX_OFFSET);
      buffer.put_int(RETURN_COMPONENT_ID);
    } catch ( Exception e ) {
      System.out.println ("NexusConnectionClient::Could not append int to the buffer");
      e.printStackTrace();
    }
    // FIXME should be false, or should it ?
    // Find out. Later. 
    boolean destroy_context = true ; 
    try {
//       System.out.println("\nNexusConnetionServer:: sending request for ComponentID ");
      nexus.send_rsr(buffer, processStartpoint, HPCXX_GF_HANDLER, destroy_context);
      nexus.destroy_current_context(false);
    } catch ( Exception e ) {
      e.printStackTrace();
      System.out.println( "NexusConnectionClient:: failed in the process of invoking returnComponentID() ! " );
    }
  }
  

  private void createStubFromStartpoint(Startpoint componentIDstartpoint) {
    if (componentIDstartpoint == null)
      throw new NullPointerException("NexusConnectionServer::createStubFromStarpoint: Startpoint is null!! ");
    
    try {
      String fullStubName = "ccacore.ComponentIDImpl_Stub";
      String serprot = "nexusrmi.Nexusize";
      String codebase = "";
      System.out.flush();
      nexusrmi.Remote stub = (nexusrmi.Remote)(nexusrmi.server.UnicastRemoteObject.createStub(fullStubName, componentIDstartpoint, serprot, codebase));
      componentIDofInstantiatedComponent = (ComponentID)stub;
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private synchronized void waitForClient() {
    while (!RecdFromClient) { 
      try {
	wait();
       } catch ( Exception e ) {
	 e.printStackTrace();
       } 
    }
  }
  
  private synchronized void stopWaitingForClient (boolean MesgFromHandler) {
    RecdFromClient = MesgFromHandler;
    try {
      notifyAll();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }


  private synchronized void waitForComponentID() {
    while (!recdComponentID) { 
      try {
	wait();
       } catch ( Exception e ) {
	 e.printStackTrace();
       } 
    }
  }
  
  private synchronized void stopWaitingForComponentID (boolean MesgFromHandler) {
    recdComponentID = MesgFromHandler;
    try {
      notifyAll();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  
  private synchronized void waitForShutdown() {
    while (!gotShutdown) {
//       System.out.println( "NexusConnectionServer:: Waiting for server shutdown" );
      try {
	wait();	
      } catch ( Exception e ) {
	e.printStackTrace();
      }
    }
//     System.out.println( "NexusConnectionServer:: Received message for server shutdown");
    // FIXME FIXME
    // this should be deleted (destroy should take place)
    // nexus.disallow_attach(portForCallBack);
    // ep.destroy();
    // nexus.destroy_current_context(false);
  }

  public synchronized void shutDown (boolean MesgFromCreationService) {
    gotShutdown = MesgFromCreationService;
    try {
      notifyAll();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}




