/*
 * @(#)EntityReader.java 1.0 6/3/97
 * 
 * Copyright (c) 1997 Microsoft, Corp. All Rights Reserved.
 * 
 */
 
package com.ms.xml.parser;

import java.io.*;

/**
 * This is a wrapper class on entities to provide transparent
 * inclusion of entity text
 *
 * @version 1.0, 6/3/97
 */
class EntityReader 
{
    EntityReader(InputStream in, int line, int column, EntityReader prev, Object owner)
    {        
        this.line = line;
        this.column = column;
        this.prev = prev;
        this.owner = owner;
        this.pos = -1;
        this.input = in;
    }

    public int read() throws ParseException
    {
        int c;

        if( pos >= 0 )
        {
            c = next[pos--];
        }
        else
        {
            try
            {
                c = input.read();
            }
            catch( IOException e )
            {
                String s = owner.toString();
                c = -1;
                throw new ParseException( "Error reading " + s + " at (" + line + "," + column + ") : " + e,
                    line, column, owner);
            }
        }

        // check only line-feed for now...
        if (c == '\n')
        {
            line++;
            column = 1;
        }
        else
        {
            column++;
        }
	// quick fix
	// doesn't seem to return -1 at EOF, returns 0
	// so just change this to -1 for now (done by nmukhi, 08/17)
	if (c == 0){
	    c = -1;
	}
        return c;
    }

    void push(char next) throws ParseException
    {
        if (pos == 1) {
            throw new ParseException("Error unreading '" + next + "' at (" + line + "," + column + ") : ",
                    line, column, owner);
        }
        this.next[++pos] = next;
    }

    /**
     * line number
     */    
    int line;
    
    /**
     * character pos
     */    
    int column;

    /**
     * previous reader to go back to when this is done
     */
    EntityReader prev;
    
    /**
     * Entity reading from
     */    
    Object owner;

    /**
     * Character pushed back into the stream.
     * Need to be able push back two characters for auto-unicode
     * detection support.
     */
    int next[] = new int[2];
    int pos;

    /**
     * The input stream for reading characters
     */
    InputStream input;
}
