/**
 * This is used to create connections to DAV servers.
 * @author Nirmal (nmukhi)
 * @see DAVConnection
 * @see DAVConnectionImplementation
 *
 */

package util.webdav.client;

public class DAVConnectionFactory {
    /**
     * The default port on which a DAV server listens.
     *
     */
    public static int DEFAULT_PORT = 8080;

    /**
     * Creates a connection to the specified host at the port DEFAULT_PORT.
     * @param host the name of the host to which the connection should 
     * be made e.g. school.cs.indiana.edu
     * @return the connection object that can be used to make requests.
     *
     */
    public static DAVConnection createConnection(String host) {
	return new DAVConnectionImplementation(host,DEFAULT_PORT);
    }

    /**
     * Create a connection to the specified host at the specified port.
     * @param host the name of the host to which the connection should be 
     * made e.g. bread.extreme.indiana.edu
     * @param port the port number on which the DAV server will be listening 
     * for requests
     * @return the connection object used ot make requests to the server.
     *
     */
    public static DAVConnection createConnection(String host, int port) {
	return new DAVConnectionImplementation(host,port);
    }
}
