/**
 * This presents a uniform way of looking at any response form the 
 * DAV server. It hides all server response codes and instead just
 * indicates success or failure, along with any data returned from
 * the server.
 *
 * @see DAVConnection
 * @author Nirmal (nmukhi)
 *
 */

package util.webdav.client;

public interface DAVServerResponse {
    /**
     * @return true if this response refers to a successful request, 
     * false otherwise
     *
     */
    public boolean isSuccess();

    /**
     * @return a string describing the action that the server took
     * for the request.
     *
     */
    public String getActionDescription();

    /**
     * @return the data form the server. Requests that cause
     * the server to return data include GET.
     *
     */
    public byte [] getData();
}
