/**
 * This encapsulates an information node on the DAV server. It refers
 * to a DAV resource which may be a collection or a file. The name of 
 * the resource and its type (collection/file) is contained here.
 * @author Nirmal (nmukhi)
 * @see DAVUtilityCalls
 */

package util.webdav.client;

public class DirectoryNode {
    private String name;
    private boolean isCollection;

    /**
     * Create a directory node, default ifnormation.
     *
     */
    public DirectoryNode() {
	name = "";
	isCollection = false;
    }

    /**
     * Create a directory node with the specified name 
     * and type.
     * @param name the node name
     * @param isCollection true if this is node refers to
     * a collection, false otherwise
     *
     */
    public DirectoryNode(String name, boolean isCollection) {
	this.name = name;
	this.isCollection = isCollection;
    }

    /** 
     * Get the name of the resource this node refers to.
     * @return the name of the node
     */
    public String getName() {
	return name;
    }

    /**
     * Check if this is a file 
     * @return true if this is a file, false otherwise
     */
    public boolean isFile() {
	return !isCollection;
    }

    /**
     * Check if this is a collection
     * @return true if this is a collection, false otherwise
     */
    public boolean isCollection() {
	return isCollection;
    }
}
