package nexus;

public class DataConversion
{
  public static final String INVALID_FORMAT= 
    "DataConversion.java:  ERROR - invalid machine data format\n";

  /*
   * little endian, with unsigned
   *  8 bit byte
   *  8 bit char
   * 16 bit short
   * 32 bit integer
   * 32 bit long
   * 32 bit pointer
   * 32 bit IEEE float
   * 64 bit IEEE double
   */
  public static final int NEXUS_DC_FORMAT_32BIT_LE = 0;

  /*
   * big endian, with unsigned
   *  8 bit byte
   *  8 bit char
   * 16 bit short
   * 32 bit integer
   * 32 bit long
   * 32 bit pointer
   * 32 bit IEEE float
   * 64 bit IEEE double
   */
  public static final int NEXUS_DC_FORMAT_32BIT_BE = 1;

  /*
   * little endian, with unsigned
   *  8 bit byte
   *  8 bit char
   * 16 bit short
   * 32 bit integer
   * 64 bit long
   * 64 bit pointer
   * 32 bit IEEE float
   * 64 bit IEEE double
   */
  public static final int NEXUS_DC_FORMAT_64BIT_LE = 2;

  /*
   * big endian, with unsigned
   *  8 bit byte
   *  8 bit char
   * 16 bit short
   * 32 bit integer
   * 64 bit long
   * 64 bit pointer
   * 32 bit IEEE float
   * 64 bit IEEE double
   */
  public static final int NEXUS_DC_FORMAT_64BIT_BE = 3;

  /*
   * big endian, with unsigned?
   *  8 bit byte
   *  8 bit char
   * 64 bit short
   * 64 bit integer
   * 64 bit long
   * 64 bit pointer
   * 32 bit Cray float
   * 64 bit Cray double
   */
  public static final int NEXUS_DC_FORMAT_CRAYC90 = 4;

  /*
   * big endian, without unsigned
   *  8 bit byte
   * 16 bit char
   * 16 bit short
   * 32 bit integer
   * 64 bit long
   * no pointer
   * 32 bit IEEE float
   * 64 bit IEEE double
   */
  public static final int NEXUS_DC_FORMAT_JAVA = 5;

  /*
   * little endian, with unsigned
   *  8 bit byte
   *  8 bit char
   * 16 bit short
   * 32 bit int
   * 32 bit long
   * 32 bit pointer
   * 32 bit VAX float
   * 64 bit VAX double
   */
  public static final int NEXUS_DC_FORMAT_LAST = 5;

  public static final int NEXUS_DC_FORMAT_LOCAL = NEXUS_DC_FORMAT_JAVA;


  /*
   * native sizeof_TYPE()
   */
  public static final int SIZEOF_BYTE = 1;
  public static final int SIZEOF_CHAR = 2;
  public static final int SIZEOF_SHORT = 2;
  public static final int SIZEOF_INT = 4;
  public static final int SIZEOF_LONG = 8;
  public static final int SIZEOF_FLOAT = 4;
  public static final int SIZEOF_DOUBLE = 8;
  public static int sizeof_byte(int count)
  { return SIZEOF_BYTE * count; }
  public static int sizeof_char(int count)
  { return SIZEOF_CHAR * count; }
  public static int sizeof_short(int count)
  { return SIZEOF_SHORT * count; }
  public static int sizeof_int(int count)
  { return SIZEOF_INT * count; }
  public static int sizeof_long(int count)
  { return SIZEOF_LONG * count; }
  public static int sizeof_float(int count)
  { return SIZEOF_FLOAT * count; }
  public static int sizeof_double(int count)
  { return SIZEOF_DOUBLE * count; }

  /*
   * machine-specific sizeof_TYPE()
   */
  public static int sizeof_byte(int count, int format)
  { return 1 * count; } 
  public static int sizeof_char(int count, int format) 
  { 
    if (format == NEXUS_DC_FORMAT_JAVA)
      return SIZEOF_CHAR * count;
    else 
      return 1 * count;
  }
  public static int sizeof_u_char(int count, int format) 
  { 
    /* Java does not support unsigned dataypes */
    if (format == NEXUS_DC_FORMAT_JAVA)
      return SIZEOF_CHAR * count;
    else 
      return 1 * count;
  }
  public static int sizeof_short(int count, int format)   
  {
    if (format == NEXUS_DC_FORMAT_CRAYC90)
      return 8 * count;
    else
      return 2 * count; 
  }
  public static int sizeof_u_short(int count, int format)   
  {
    /* Java does not support unsigned dataypes */
    if (format == NEXUS_DC_FORMAT_CRAYC90)
      return 8 * count;
    else
      return 2 * count; 
  }
  public static int sizeof_int(int count, int format)   
  {
    if (format == NEXUS_DC_FORMAT_CRAYC90)
      return 8 * count;
    else
      return 4 * count; 
  }
  public static int sizeof_u_int(int count, int format)   
  {
    /* Java does not support unsigned dataypes */
    if (format == NEXUS_DC_FORMAT_CRAYC90)
      return 8 * count;
    else
      return 4 * count; 
  }
  public static int sizeof_long(int count, int format)
  {
    if (format == NEXUS_DC_FORMAT_32BIT_LE
        || format == NEXUS_DC_FORMAT_32BIT_BE)
      return 4 * count;
    else
      return 8 * count; 
  }
  public static int sizeof_u_long(int count, int format)
  {
    /* Java does not support unsigned dataypes */
    if (format == NEXUS_DC_FORMAT_32BIT_LE
        || format == NEXUS_DC_FORMAT_32BIT_BE )
      return 4 * count;
    else
      return 8 * count; 
  }
  public static int sizeof_float(int count, int format)
  { return 4 * count; }
  public static int sizeof_double(int count, int format)
  { return 8 * count; }


  /*
   * put_TYPE() by individual primitive, and by array
   */

  /*
   * put_byte() 
   */
  public static void put_byte(byte[] put_buf, 
			      int put_start, 
			      int put_count, 
			      byte[] buf, 
			      int location)
  {
    System.arraycopy(put_buf, put_start, buf, location, put_count);
/*
      for (int i = put_start; i < put_start + put_count; i++)
      {
          buf[location++] = put_buf[i];
      }
*/
  } /* put_byte() */
  public static void put_byte(byte put_buf, 
			      byte[] buf, 
			      int location)
  {
    buf[location] = put_buf;
  } /* put_byte() */


  /*
   * put_char() 
   */
  public static void put_char(char[] put_buf, 
			      int put_start, 
			      int put_count, 
			      byte[] buf, 
			      int location)
  {
    for (int i = put_start; i < put_start + put_count; i++)
    { 
      buf[location++] = (byte)(put_buf[i] >> 8);
      buf[location++] = (byte)(put_buf[i]);
    }
  } /* put_char() */
  public static void put_char(char put_buf, 
			      byte[] buf, 
			      int location)
  {
    buf[location++] = (byte)(put_buf >> 8);
    buf[location++] = (byte)(put_buf);
  } /* put_char() */


  /*
   * put_short()
   */
  public static void put_short(short[] put_buf, 
			       int put_start, 
			       int put_count, 
			       byte[] buf, 
			       int location)
  {
    for (int i = put_start; i < put_start + put_count; i++)
    {
      buf[location++] = (byte)(put_buf[i] >> 8);
      buf[location++] = (byte)(put_buf[i]);
    }
  } /* put_short() */
  public static void put_short(short put_buf, 
			       byte[] buf, 
			       int location)
  {
    buf[location++] = (byte)(put_buf >> 8);
    buf[location++] = (byte)(put_buf);
  } /* put_short() */


  /*
   * put_int()
   */ 
  public static void put_int(int[] put_buf, 
			     int put_start, 
			     int put_count, 
			     byte[] buf, 
			     int location)
  {
    for (int i = put_start; i < put_start + put_count; i++)
    {
      buf[location++] = (byte)(put_buf[i] >> 24);
      buf[location++] = (byte)(put_buf[i] >> 16);
      buf[location++] = (byte)(put_buf[i] >> 8);
      buf[location++] = (byte)(put_buf[i]);
    }
  } /* put_int() */
  public static void put_int(int put_buf, 
			     byte[] buf, 
			     int location)
  {
    buf[location++] = (byte)(put_buf >> 24);
    buf[location++] = (byte)(put_buf >> 16);
    buf[location++] = (byte)(put_buf >> 8);
    buf[location++] = (byte)(put_buf);
  } /* put_int() */


  /*
   * put_long()
   */
  public static void put_long(long[] put_buf, 
			      int put_start, 
			      int put_count, 
			      byte[] buf, 
			      int location)
  {
    for (int i = put_start; i < put_start + put_count; i++)
    {
      buf[location++] = (byte)(put_buf[i] >> 56);
      buf[location++] = (byte)(put_buf[i] >> 48);
      buf[location++] = (byte)(put_buf[i] >> 40);
      buf[location++] = (byte)(put_buf[i] >> 32);
      buf[location++] = (byte)(put_buf[i] >> 24);
      buf[location++] = (byte)(put_buf[i] >> 16);
      buf[location++] = (byte)(put_buf[i] >> 8);
      buf[location++] = (byte)(put_buf[i]);
    }
  } /* put_long() */
  public static void put_long(long put_buf, 
			      byte[] buf, 
			      int location)
  {
    buf[location++] = (byte)(put_buf >> 56);
    buf[location++] = (byte)(put_buf >> 48);
    buf[location++] = (byte)(put_buf >> 40);
    buf[location++] = (byte)(put_buf >> 32);
    buf[location++] = (byte)(put_buf >> 24);
    buf[location++] = (byte)(put_buf >> 16);
    buf[location++] = (byte)(put_buf >> 8);
    buf[location++] = (byte)(put_buf);
  } /* put_long() */


  /*
   * put_float()
   */
  public static void put_float(float[] put_buf, 
			       int put_start, 
			       int put_count, 
			       byte[] buf, 
			       int location)
  {
    for (int i = put_start; i < put_start + put_count; i++)
    {
      int tmp = Float.floatToIntBits(put_buf[i]);

      buf[location++] = (byte)(tmp >> 24);
      buf[location++] = (byte)(tmp >> 16);
      buf[location++] = (byte)(tmp >> 8);
      buf[location++] = (byte)(tmp);
    }
  } /* put_float() */
  public static void put_float(float put_buf, 
			       byte[] buf, 
			       int location)
  {
    int tmp = Float.floatToIntBits(put_buf);

    buf[location++] = (byte)(tmp >> 24);
    buf[location++] = (byte)(tmp >> 16);
    buf[location++] = (byte)(tmp >> 8);
    buf[location++] = (byte)(tmp);
  } /* put_float() */


  /*
   * put_double()
   */
  public static void put_double(double[] put_buf, 
				int put_start, 
				int put_count, 
				byte[] buf, 
				int location)
  {
    for (int i = put_start; i < put_start + put_count; i++)
    {
      long tmp = Double.doubleToLongBits(put_buf[i]);

      buf[location++] = (byte)(tmp >> 56);
      buf[location++] = (byte)(tmp >> 48);
      buf[location++] = (byte)(tmp >> 40);
      buf[location++] = (byte)(tmp >> 32);
      buf[location++] = (byte)(tmp >> 24);
      buf[location++] = (byte)(tmp >> 16);
      buf[location++] = (byte)(tmp >> 8);
      buf[location++] = (byte)(tmp);
    }
  } /* put_double() */
  public static void put_double(double put_buf, 
				byte[] buf, 
				int location)
  {
    long tmp = Double.doubleToLongBits(put_buf);

    buf[location++] = (byte)(tmp >> 56);
    buf[location++] = (byte)(tmp >> 48);
    buf[location++] = (byte)(tmp >> 40);
    buf[location++] = (byte)(tmp >> 32);
    buf[location++] = (byte)(tmp >> 24);
    buf[location++] = (byte)(tmp >> 16);
    buf[location++] = (byte)(tmp >> 8);
    buf[location++] = (byte)(tmp);
  } /* put_double() */


  /*
   * get_TYPE() - by individual primitive, and by array
   */

  /*
   * get_byte()
   */
  public static void get_byte(byte[] get_buf,
			      int get_start,
			      int get_count,
			      byte buf[],
			      int location,
			      int format) throws LostPrecisionException
  {
    if (format < 0 || format > NEXUS_DC_FORMAT_LAST)
      System.out.println(INVALID_FORMAT);
    else   
      System.arraycopy(buf, location, get_buf, get_start, get_count);
/*
      for (int i = get_start; i < get_start + get_count; i++)
      {
        get_buf[i] = buf[location++];
      }
*/
  } /* get_byte() */
  public static byte get_byte(byte buf[],
			      int location,
			      int format) throws LostPrecisionException
  {
    if (format < 0 || format > NEXUS_DC_FORMAT_LAST)
    {        
      System.out.println(INVALID_FORMAT);
      return (byte)0;
    }	
    else
      return(buf[location]);
  } /* get_byte() */


  /*
   * get_char()
   */
  public static void get_char(char[] get_buf,
			      int get_start,
			      int get_count,
			      byte buf[],
			      int location,
			      int format) throws LostPrecisionException
  {
    if (format < 0 || format > NEXUS_DC_FORMAT_LAST)
      System.out.println(INVALID_FORMAT);
    else if (format == NEXUS_DC_FORMAT_JAVA)
    {
      for (int i = get_start; i < get_start + get_count; i++)
      {
        get_buf[i] = (char)(
                    ((char)(char)buf[location++] << 8) 
                  | ((char)(char)buf[location++] & 0x00ff) );
      }
    }
    else 
    {
      for (int i = get_start; i < get_start + get_count; i++)
      {
        get_buf[i] = (char)(buf[location++]);
      }
    }
  } /* get_char() */
  public static char get_char(byte buf[],
			      int location,
			      int format) throws LostPrecisionException
  {
    if (format < 0 || format > NEXUS_DC_FORMAT_LAST)
    {
      System.out.println(INVALID_FORMAT);
      return (char)0;
    }
    else if (format == NEXUS_DC_FORMAT_JAVA)
    {
      return (char)(
            ((char)(char)buf[location++] << 8) 
          | ((char)(char)buf[location++] & 0x00ff) );
    }
    else 
    {
      return (char)buf[location++];
    }
  } /* get_char() */


  /*
   * get_short()
   */
  public static void get_short(short[] get_buf,
			       int get_start,
			       int get_count,
			       byte buf[],
			       int location,
			       int format) throws LostPrecisionException
  {
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
      case NEXUS_DC_FORMAT_64BIT_LE:
        for (int i = get_start; i < get_start + get_count; i++)
        {
          get_buf[i] = (short)(
                      ((short)(short)buf[location++] & 0x00ff) 
                    | ((short)(short)buf[location++] << 8) );
        }    
      break;
    
      case NEXUS_DC_FORMAT_32BIT_BE:    
      case NEXUS_DC_FORMAT_64BIT_BE:    
      case NEXUS_DC_FORMAT_JAVA:
        for (int i = get_start; i < get_start + get_count; i++)
        {
          get_buf[i] = (short)(
                      ((short)(short)buf[location++] << 8) 
                    | ((short)(short)buf[location++] & 0x00ff) );
        }    
      break;

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      break;

      default:    
        System.out.println(INVALID_FORMAT);
      break;
    }
  } /* get_short() */
  public static short get_short(byte buf[],
			        int location,
			        int format) throws LostPrecisionException
  {
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
      case NEXUS_DC_FORMAT_64BIT_LE:
        return (short)(
              ((short)(short)buf[location++] & 0x00ff) 
            | ((short)(short)buf[location++] << 8) );

      case NEXUS_DC_FORMAT_32BIT_BE:    
      case NEXUS_DC_FORMAT_64BIT_BE:    
      case NEXUS_DC_FORMAT_JAVA:
        return (short)(
              ((short)(short)buf[location++] << 8) 
            | ((short)(short)buf[location++] & 0x00ff) );

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      return((short)0);

      default:    
        System.out.println(INVALID_FORMAT);
      return((short)0);
    }
  } /* get_short() */


  /*
   * get_int()
   */
  public static void get_int(int[] get_buf,
			     int get_start,
			     int get_count,
			     byte buf[],
			     int location,
			     int format) throws LostPrecisionException
  {
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
      case NEXUS_DC_FORMAT_64BIT_LE:
        for (int i = get_start; i < get_start + get_count; i++)
        {
          get_buf[i] = (int)(
                     ( (int)(int)buf[location++] & 0x000000ff)
              | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
              | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
                   | ( (int)(int)buf[location++] << 24 ) );
        }    
      break;
    
      case NEXUS_DC_FORMAT_32BIT_BE:    
      case NEXUS_DC_FORMAT_64BIT_BE:    
      case NEXUS_DC_FORMAT_JAVA:
        for (int i = get_start; i < get_start + get_count; i++)
        {
          get_buf[i] = (int)(
                     ( (int)(int)buf[location++] << 24 )
              | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
              | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
                   | ( (int)(int)buf[location++] & 0x000000ff ) );
        }    
      break;

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      break;

      default:    
        System.out.println(INVALID_FORMAT);
      break;
    }
  } /* get_int() */
  public static int get_int(byte buf[],
			    int location,
			    int format) throws LostPrecisionException
  {
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
      case NEXUS_DC_FORMAT_64BIT_LE:
        return (int)(
             ( (int)(int)buf[location++] & 0x000000ff)
           | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
           | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
           | ( (int)(int)buf[location++] << 24 ) );

      case NEXUS_DC_FORMAT_32BIT_BE:    
      case NEXUS_DC_FORMAT_64BIT_BE:    
      case NEXUS_DC_FORMAT_JAVA:
        return (int)(
             ( (int)(int)buf[location++] << 24 )
           | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
           | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
           | ( (int)(int)buf[location++] & 0x000000ff ) );

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      return((int)0);

      default:    
        System.out.println(INVALID_FORMAT);
      return((int)0);
    }
  } /* get_int() */


  /*
   * get_long()
   */
  public static void get_long(long[] get_buf,
			      int get_start,
			      int get_count,
			      byte buf[],
			      int location,
			      int format) throws LostPrecisionException
  {
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
        for (int i = get_start; i < get_start + get_count; i++)
        {
          get_buf[i] = (long)(
                     ( (int)(int)buf[location++] & 0x000000ff)
              | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
              | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
              | ( (int)(int)buf[location++] << 24 ) );
       }
     break;

     case NEXUS_DC_FORMAT_64BIT_LE:    
       for (int i = get_start; i < get_start + get_count; i++)
       {
         get_buf[i] = (long)(
              ( (long)(long)(long)buf[location++] << 56 >>> 56 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 48 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 40 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 32 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 24 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 16 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 8 )
            |       ( (long)(long)buf[location++] << 56 ) );	
       }    
     break;

     case NEXUS_DC_FORMAT_32BIT_BE:    
       for (int i = get_start; i < get_start + get_count; i++)
       {
         get_buf[i] = (long)(
                    ( (int)(int)buf[location++] << 24 )
             | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
             | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
                  | ( (int)(int)buf[location++] & 0x000000ff ) );
       }    
     break;

     case NEXUS_DC_FORMAT_64BIT_BE:    
     case NEXUS_DC_FORMAT_JAVA:
       for (int i = get_start; i < get_start + get_count; i++)
       {
         get_buf[i] = (long)(
                    ( (long)(long)buf[location++] << 56 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 8 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 16 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 24 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 32 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 40 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 48 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 56 ) );	
       }    
     break;

     case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
     break;

     default:
        System.out.println(INVALID_FORMAT);
     break;
    }
  } /* get_long() */
  public static long get_long(byte buf[],
			      int location,
			      int format) throws LostPrecisionException
  {
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
        return (long)(
             ( (int)(int)buf[location++] & 0x000000ff)
           | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
           | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
           | ( (int)(int)buf[location++] << 24 ) );

      case NEXUS_DC_FORMAT_64BIT_LE:    
        return (long)(
             ( (long)(long)(long)buf[location++] << 56 >>> 56 )
           | ( (long)(long)(long)buf[location++] << 56 >>> 48 )
           | ( (long)(long)(long)buf[location++] << 56 >>> 40 )
           | ( (long)(long)(long)buf[location++] << 56 >>> 32 )
           | ( (long)(long)(long)buf[location++] << 56 >>> 24 )
           | ( (long)(long)(long)buf[location++] << 56 >>> 16 )
           | ( (long)(long)(long)buf[location++] << 56 >>> 8 )
           |       ( (long)(long)buf[location++] << 56 ) );
	
      case NEXUS_DC_FORMAT_32BIT_BE:    
        return (long)(
             ( (int)(int)buf[location++] << 24 )
      | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
      | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
           | ( (int)(int)buf[location++] & 0x000000ff ) );

       case NEXUS_DC_FORMAT_64BIT_BE:    
       case NEXUS_DC_FORMAT_JAVA:
         return (long)(
              ( (long)(long)buf[location++] << 56 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 8 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 16 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 24 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 32 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 40 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 48 )
            | ( (long)(long)(long)buf[location++] << 56 >>> 56 ) );	

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      return((long)0);    

      default:
        System.out.println(INVALID_FORMAT);
      return((long)0);    
    }
  } /* get_long() */


  /*
   * get_float()
   */
  public static void get_float(float[] get_buf,
			       int get_start,
			       int get_count,
			       byte buf[],
			       int location,
			       int format) throws LostPrecisionException
  {
    int tmp[] = new int[get_buf.length]; 
 
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
      case NEXUS_DC_FORMAT_64BIT_LE:
        for (int i = 0; i < get_buf.length; i++)
        {
          tmp[i] = (int)(
                 ( (int)(int)buf[location++] & 0x000000ff)
          | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
          | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
               | ( (int)(int)buf[location++] << 24 ) );
          get_buf[get_start++] = Float.intBitsToFloat(tmp[i]);
        }    
      break;
    
      case NEXUS_DC_FORMAT_32BIT_BE:    
      case NEXUS_DC_FORMAT_64BIT_BE:    
      case NEXUS_DC_FORMAT_JAVA:
        for (int i = 0; i < get_buf.length; i++)
        {
          tmp[i] = (int)(
                 ( (int)(int)buf[location++] << 24 )
          | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
          | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
               | ( (int)(int)buf[location++] & 0x000000ff ) );
          get_buf[get_start++] = Float.intBitsToFloat(tmp[i]);
        }    
      break;

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      break;

      default:    
        System.out.println(INVALID_FORMAT);
      break;
    }
  } /* get_float() */
  public static float get_float(byte buf[],
		 	        int location,
			        int format) throws LostPrecisionException
  {
    int tmp; 

    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
      case NEXUS_DC_FORMAT_64BIT_LE:
        tmp = (int)(
            ( (int)(int)buf[location++] & 0x000000ff)
          | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
          | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
          | ( (int)(int)buf[location++] << 24 ) );
        return(Float.intBitsToFloat(tmp));

      case NEXUS_DC_FORMAT_32BIT_BE:    
      case NEXUS_DC_FORMAT_64BIT_BE:    
      case NEXUS_DC_FORMAT_JAVA:
        tmp = (int)(
            ( (int)(int)buf[location++] << 24 )
     | ( (int)(int)(int)buf[location++] << 16 & 0x00ff0000 )
     | ( (int)(int)(int)buf[location++] << 8 & 0x0000ff00 )
          | ( (int)(int)buf[location++] & 0x000000ff ) );
      return(Float.intBitsToFloat(tmp));

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      return((float)0);    

      default:    
        System.out.println(INVALID_FORMAT);
      return((float)0);    
    }
  } /* get_float() */


  /*
   * get_double()
   */
  public static void get_double(double[] get_buf,
				int get_start,
				int get_count,
				byte buf[],
				int location,
				int format) throws LostPrecisionException
  {
    long tmp[] = new long[get_buf.length]; 
 
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
      case NEXUS_DC_FORMAT_64BIT_LE:
        for (int i = 0; i < get_buf.length; i++)
        {
          tmp[i] = (long)(
                 ( (long)(long)(long)buf[location++] << 56 >>> 56 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 48 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 40 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 32 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 24 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 16 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 8 )
               |       ( (long)(long)buf[location++] << 56 ) );
          get_buf[get_start++] = Double.longBitsToDouble(tmp[i]);
        }    
      break;
  
      case NEXUS_DC_FORMAT_32BIT_BE:    
      case NEXUS_DC_FORMAT_64BIT_BE:    
      case NEXUS_DC_FORMAT_JAVA:
        for (int i = 0; i < get_buf.length; i++)
        {
          tmp[i] = (long)(
                 ( (long)(long)buf[location++] << 56 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 8 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 16 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 24 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 32 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 40 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 48 )
               | ( (long)(long)(long)buf[location++] << 56 >>> 56 ) );
          get_buf[get_start++] = Double.longBitsToDouble(tmp[i]);
        }    
      break;

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      break;

      default:
        System.out.println(INVALID_FORMAT);
      break;
    }
  } /* get_double() */
  public static double get_double(byte buf[],
		  		  int location,
				  int format) throws LostPrecisionException
  {
    long tmp; 
 
    switch(format)
    {
      case NEXUS_DC_FORMAT_32BIT_LE:    
      case NEXUS_DC_FORMAT_64BIT_LE:
        tmp = (long)(
            ( (long)(long)(long)buf[location++] << 56 >>> 56 )
          | ( (long)(long)(long)buf[location++] << 56 >>> 48 )
          | ( (long)(long)(long)buf[location++] << 56 >>> 40 )
          | ( (long)(long)(long)buf[location++] << 56 >>> 32 )
          | ( (long)(long)(long)buf[location++] << 56 >>> 24 )
          | ( (long)(long)(long)buf[location++] << 56 >>> 16 )
          | ( (long)(long)(long)buf[location++] << 56 >>> 8 )
          |       ( (long)(long)buf[location++] << 56 ) );
        return(Double.longBitsToDouble(tmp));

      case NEXUS_DC_FORMAT_32BIT_BE:    
      case NEXUS_DC_FORMAT_64BIT_BE:    
      case NEXUS_DC_FORMAT_JAVA:
        tmp = (long)(
            ( (long)(long)buf[location++] << 56 )
    | ( (long)(long)(long)buf[location++] << 56 >>> 8 )
    | ( (long)(long)(long)buf[location++] << 56 >>> 16 )
    | ( (long)(long)(long)buf[location++] << 56 >>> 24 )
    | ( (long)(long)(long)buf[location++] << 56 >>> 32 )
    | ( (long)(long)(long)buf[location++] << 56 >>> 40 )
    | ( (long)(long)(long)buf[location++] << 56 >>> 48 )
    | ( (long)(long)(long)buf[location++] << 56 >>> 56 ) );
        return(Double.longBitsToDouble(tmp));

      case NEXUS_DC_FORMAT_CRAYC90:    
        System.out.println("DataConversion.java:  NexusJava does not yet support Cray C90 datatypes\n");		
      return((double)0);

      default:    
        System.out.println(INVALID_FORMAT);
      return((double)0);
    }
  } /* get_double() */


  /* 
   * get_u_TYPE() - by individual primitive, and by array
   */

  /*
   * get_u_char() 
   */
  public static void get_u_char(char[] get_buf,
  				int get_start,
  				int get_count,
  				byte buf[],
  				int location,
  				int format) throws LostPrecisionException
  {
    get_char(get_buf, get_start, get_count, buf, location, format);
  } /* get_u_char() */
  public static char get_u_char(byte buf[],
  				int location,
  				int format) throws LostPrecisionException
  {
    return get_char(buf, location, format);
  } /* get_u_char() */
 

  /*
   * get_u_short()
   */
  public static void get_u_short(short[] get_buf,
  				 int get_start,
  				 int get_count,
  				 byte buf[],
  				 int location,
  				 int format) throws LostPrecisionException
  {
    get_short(get_buf, get_start, get_count, buf, location, format);
  } /* get_u_short() */
  public static short get_u_short(byte buf[],
  				  int location,
  				  int format) throws LostPrecisionException
  {
    return get_short(buf, location, format);
  } /* get_u_short() */


  /*
   * get_u_int()
   */
  public static void get_u_int(int[] get_buf,
  			       int get_start,
  			       int get_count,
  			       byte buf[],
  			       int location,
  			       int format) throws LostPrecisionException
  {
    get_int(get_buf, get_start, get_count, buf, location, format);
  } /* get_u_int() */
  public static int get_u_int(byte buf[],
  			      int location,
  			      int format) throws LostPrecisionException
  {
    return get_int(buf, location, format);
  } /* get_u_int() */


  /*
   * get_u_long() 
   */
  public static void get_u_long(long[] get_buf,
				int get_start,
				int get_count,
				byte buf[],
				int location,
				int format) throws LostPrecisionException
  {
    get_long(get_buf, get_start, get_count, buf, location, format);
  }
  public static long get_u_long(byte buf[],
				int location,
				int format) throws LostPrecisionException
  {
    return get_long(buf, location, format);
  } /* get_u_long() */

}
