package nexus;

public class Endpoint
{
    private Nexus				nexus;
    private Object				user_object;
    private HandlerInterface			handler_object;
    private long				id;
    private ProtoInfo				protoInfo;
    private byte[]				comm_descriptor_table;
    // private TransformState			transform_state;
    // private int				transform_id;

    /*
     * Endpoint()
     *
     * To instantiate a null endpoint.
     */
    public
    Endpoint(
	Nexus					nexus,
	EndpointAttr				epattr)
    {
	this.nexus = nexus;
	id = nexus.register_endpoint(this);
	this.handler_object = epattr.get_handler_object();
	this.protoInfo = epattr.get_protocol();
	this.comm_descriptor_table =
	    nexus.create_comm_descriptor_table(protoInfo, id);
    }
    /* Endpoint() */

    /*
     * set_user_object()
     *
     * Set the object to which this endpoint points.
     */
    public
    void
    set_user_object(
	Object					user_object)
    {
	this.user_object = user_object;
    }
    /* set_user_object() */

    /*
     * get_user_object()
     *
     * Return the object to which this endpoint points.
     */
    public
    Object
    get_user_object()
    {
	return user_object;
    }
    /* get_user_object() */

    /*
     * get_handler_object()
     *
     * Return the handler object to which this endpoint points.
     */
    public
    HandlerInterface
    get_handler_object()
    {
	return handler_object;
    }
    /* get_handler_object() */

    /*
     * get_protocol()
     */
    public
    ProtoInfo
    get_protocol()
    {
	return protoInfo;
    }
    /* get_protocol() */

    /*
     * get_comm_descriptor_table()
     */
    public
    byte[]
    get_comm_descriptor_table()
    {
	return this.comm_descriptor_table;
    }
    /* get_comm_descriptor_table() */

    /*
     * id()
     *
     * Return the endpointID under which this endpoint's
     * object is registered with nexus.
     */
    public
    long
    id()
    {
	return id;
    }
    /* id() */

    /*
     * destroy()
     *
     * Unregister this endpoint with nexus.
     */
    public
    void
    destroy()
    {
	nexus.unregister_endpoint(id);

	/*
	 * TODO: should it be possible to shutdown persistent connections from
	 * here???
	 */

	/*
	 * TODO: need to thread call to TotemProto through Nexus object to
	 * unregister ep from ProcessGroup
	 */
    }
    /* destroy() */
} /* class Endpoint */

