/*
 * GetBuffer
 */

package nexus;

import java.io.InvalidClassException;

public class GetBuffer extends Object
{
    private Nexus       nexus;
    private byte[]      buffer;
    private int         size = 0;
    private int         current_location;
    private int		data_format;
    private int		header_size;
    
    /*
     * GetBuffer()
     */
    public GetBuffer(Nexus nexus,
                     byte[] buffer,
		     int header_size,
		     int data_format)
    {
	this.nexus = nexus;
        this.buffer = buffer;
        this.size = buffer.length - header_size;
        this.header_size = this.current_location = header_size;
	this.data_format = data_format;
    } /* GetBuffer() */
   

    /*
     * get_buffer()
     */
    public byte[] get_buffer()
    {
	return (buffer);
    }/* get_buffer() */


    /*
     * get_message_length()    was get_buffer_length()
     */
    public int get_message_length()
    {
        return(buffer.length - header_size);
    } /* get_message_length() */

        
    /*
     * get_buffer_length()		was get_size()
     */
    public int get_buffer_length()
    {
        return(buffer.length);
    } /* get_buffer_length() */


    /*
     * check_for_overrun()
     */
    private void check_for_overrun(int get_size)
        throws BufferOverrunException
    {
        if (current_location + get_size > buffer.length)
        {
            throw(new BufferOverrunException()); 
        }
    } /* check_for_overrun() */


    /*
     * get_TYPE()
     */
    public float get_float()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_float(1, data_format);
	check_for_overrun(size);
	float value = DataConversion.get_float(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public double get_double()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_double(1, data_format);
	check_for_overrun(size);
	double value = DataConversion.get_double(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public short get_short()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_short(1, data_format);
	check_for_overrun(size);
	short value = DataConversion.get_short(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public short get_u_short()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_u_short(1, data_format);
	check_for_overrun(size);
	short value = DataConversion.get_u_short(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public int get_int()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_int(1, data_format);
	check_for_overrun(size);
	int value = DataConversion.get_int(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public int get_u_int()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_u_int(1, data_format);
	check_for_overrun(size);
	int value = DataConversion.get_u_int(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public long get_long()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_long(1, data_format);
	check_for_overrun(size);
	long value = DataConversion.get_long(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public long get_u_long()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_u_long(1, data_format);
	check_for_overrun(size);
	long value = DataConversion.get_u_long(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public char get_char()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_char(1, data_format);
	check_for_overrun(size);
	char value = DataConversion.get_char(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public char get_u_char()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_u_char(1, data_format);
	check_for_overrun(size);
	char value = DataConversion.get_u_char(buffer, current_location, data_format);
	current_location += size;
	return value;
    }
    public byte get_byte()
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_byte(1, data_format);
	check_for_overrun(size);
	byte value = DataConversion.get_byte(buffer, current_location, data_format);
	current_location += size;
	return value;
    }


    /*
     * get_TYPE(value[], start, count)
     */
    public void get_float(float value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_float(count, data_format);
	check_for_overrun(size);
	DataConversion.get_float(value, start, count, buffer,
				 current_location, data_format);
	current_location += size;
    }
    public void get_double(double value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_double(count, data_format);
	check_for_overrun(size);
	DataConversion.get_double(value, start, count, buffer,
				  current_location, data_format);
	current_location += size;
    }
    public void get_short(short value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_short(count, data_format);
	check_for_overrun(size);
	DataConversion.get_short(value, start, count, buffer,
				 current_location, data_format);
	current_location += size;
    }
    public void get_u_short(short value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_u_short(count, data_format);
	check_for_overrun(size);
	DataConversion.get_u_short(value, start, count, buffer,
				   current_location, data_format);
	current_location += size;
    }
    public void get_int(int value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_int(count, data_format);
	check_for_overrun(size);
	DataConversion.get_int(value, start, count, buffer,
			       current_location, data_format);
	current_location += size;
    }
    public void get_u_int(int value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_u_int(count, data_format);
	check_for_overrun(size);
	DataConversion.get_u_int(value, start, count, buffer,
				 current_location, data_format);
	current_location += size;
    }
    public void get_long(long value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_long(count, data_format);
	check_for_overrun(size);
	DataConversion.get_long(value, start, count, buffer,
				current_location, data_format);
	current_location += size;
    }
    public void get_u_long(long value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_u_long(count, data_format);
	check_for_overrun(size);
	DataConversion.get_u_long(value, start, count, buffer,
				  current_location, data_format);
	current_location += size;
    }
    public void get_char(char value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_char(count, data_format);
	check_for_overrun(size);
	DataConversion.get_char(value, start, count, buffer,
				current_location, data_format);
	current_location += size;
    }
    public void get_u_char(char value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_u_char(count, data_format);
	check_for_overrun(size);
	DataConversion.get_u_char(value, start, count, buffer,
				  current_location, data_format);
	current_location += size;
    }
    public void get_byte(byte value[], int start, int count)
	throws BufferOverrunException, LostPrecisionException
    {
    	int size = DataConversion.sizeof_byte(count, data_format);
	check_for_overrun(size);
	DataConversion.get_byte(value, start, count, buffer,
				current_location, data_format);
	current_location += size;
    }


    /*   
     * sizeof_startpoint()
     */  
    public static int sizeof_startpoint(Startpoint sp)
    {
        return (sp.sizeof());
    }   
    public static int sizeof_startpoint(Startpoint[] sp,
                                        int start,
                                        int count)
    {
        int size = 0;
        for (int i = start; i < start + count; i++)
        {
            size += sp[i].sizeof();
        }
        return (size);
    }


    /*
     * get_startpoint()
     */
    public Startpoint get_startpoint()
	throws BufferOverrunException,
	       LostPrecisionException,
	       InvalidClassException
    {
	return (new Startpoint(nexus, this));
    }

    public void get_startpoint(Startpoint[] sp,
				   int start,
				   int count)
	throws BufferOverrunException,
	       LostPrecisionException,
	       InvalidClassException
    {
	for (int i = start; i < start + count; i++)
	{
	    sp[i] = new Startpoint(nexus, this);
	}
    }
    
} /* class GetBuffer */

