/*
 * LocalProto
 *
 * extends Object
 */

package nexus;

public class LocalProto
  implements ProtoInterface, OutgoingInterface, Runnable
{
    private Nexus	nexus;
    private Queue	message_queue;
    private Thread	handler_thread;

    /*
     * LocalProto()
     */
    public LocalProto(Nexus nexus)
    {
	this.nexus = nexus;
	message_queue = new Queue();

	/* Start a handler thread */
	handler_thread = new Thread(this, "LocalProto_handler");
	nexus.add_thread(handler_thread);
	handler_thread.setDaemon(true);
	handler_thread.start();
    } /* LocalProto() */


    /*
     * run()
     *
     * This is the entry point for the handler thread.
     */
    public void run()
    {
	handle_messages();
	nexus.remove_thread(handler_thread);
	Nexus.debug_println(2,"LocalProto.run(): handler thread terminating");
    } /* run() */


    /*
     * shutdown()
     */
    public void shutdown()
    {
	message_queue.awaken_all_waiting_threads();
    } /* shutdown() */


    /*
     * LocalProto.get_Protocol_Version()
     */
    public
    int
    get_Protocol_Version()
    {
	return 0;
    }
    /* LocalProto.get_Protocol_Version() */

    /*
     * send_rsr()
     *
     * Local sends need to be serialized to avoid infinite
     * recursive decent.  An easy solution to this is to just
     * create a separate threads that runs all of the local
     * handlers serially from a queue.
     *
     * However, this is suboptimal in the multithreaded case, since
     * sends from different threads could do local sends without
     * worry of recursive decent.  But that would require keeping track
     * of whether the calling thread is already in a local handler,
     * and that's not really worth the effort at this point.
     */
    public void send_rsr(byte buffer[])
    {
	message_queue.add(buffer);
    } /* send_rsr() */

    public void send_rsr(PutBuffer buffer)
    {
	Chunk c;
	int curpos = 0;

	//	System.out.println("Local: send message (" + buffer.getSize() + " bytes)");
	byte[] buf = new byte[buffer.getSize()];

	c = buffer.getHeader();
	//	System.out.println("Local: sending header (" + c.getStart() + ", " + c.getSize() + " bytes)");
	System.arraycopy(c.getData(), c.getStart(), buf, curpos, c.getSize());
	curpos += c.getSize();
	c = buffer.getFirstChunk();
	while(c != null)
	{
	    //	    System.out.println("Local: sending chunk (" + c.getStart() + ", " + c.getSize() + " bytes)");
	    System.arraycopy(c.getData(), c.getStart(), buf, curpos, c.getSize());
	    curpos += c.getSize();
	    c = buffer.getNextChunk();
	}
	send_rsr(buf);
    }

    /*
     * handle_messages()
     */
    private void handle_messages()
    {
	byte	buffer[];

	while ((buffer = (byte[]) message_queue.remove_with_wait()) != null)
	{
	    nexus.buffer_dispatch(buffer, ProtoInterface.USE_BUFFER_EP_ID);
	}
    } /* handle_messages() */


    /*
     * get_my_mi_proto()
     */
    public byte[] get_my_mi_proto(ProtoInfo protoInfo, long endpointID)
    {
	/* This never gets used, so just dummy it up */
	return (new byte[1]);
    } /* get_my_mi_proto() */


    /*
     * construct_from_mi_proto()
     */
    public OutgoingInterface construct_from_mi_proto(byte[] mi_proto,
						     int start,
						     int size,
						     String node_name,
					     /*int node_number,*/
						     boolean[] is_valid)
    {
	/* This never gets used, so just dummy it up */
	is_valid[0] = false;
	return (null);
    } /* construct_from_mi_proto() */
    
} /* class LocalProto */


