/*
 * PutBuffer
 *
 * TODO: Change this to use a linked list of byte[] when you do check_buffer_size()
 */

package nexus;

public class PutBuffer extends Object
{
    private Nexus	nexus;
    private Chunk       header;
    private Chunk       data_list;
    private Chunk       currentChunk;
    private int 	num_direct_puts;
    private int         size;
    
    private static final int NEXUS_MAX_LIBA_SIZE = 64;
    private static final int NEXUS_MAX_TRANSFORM_INFO_SIZE = 64;
    private static final int FIXED_HEADER_SIZE = 64;
    private static final int RESERVED_HEADER_SIZE = (NEXUS_MAX_LIBA_SIZE
                                                   + NEXUS_MAX_TRANSFORM_INFO_SIZE
                                                   + FIXED_HEADER_SIZE);

//     private static int count = 0;
//     private static synchronized void inc()
//     {
// 	count++;
//     }
//     private static synchronized void dec()
//     {
// 	count--;
//     }
//     public static int getCount()
//     {
// 	return count;
//     }
//     protected void finalize()
//     {
// 	dec();
//     }
    
    /*
     * PutBuffer()
     */
    public PutBuffer(Nexus nexus,
		     int buffer_size,
		     int num_direct_puts)
    {
	this.nexus = nexus;
	this.num_direct_puts = num_direct_puts;
	header = null;
	currentChunk = null;
	size = 0;
	addChunk();
// 	inc();
    } /* PutBuffer() */

    public Chunk newHeader()
    {
	header = new Chunk(RESERVED_HEADER_SIZE);
	return header;
    }

    public void setHeaderSize(int hsize)
    {
	size += hsize;
	header.setSize(hsize);
	header.setStart(RESERVED_HEADER_SIZE - hsize);
    }
    
    public void addChunk()
    {
	Chunk c = new Chunk();
	if(currentChunk == null)
	    data_list = c;
	else
	    currentChunk.setNext(c);

	currentChunk = c;
// 	System.out.println("Added new Chunk");
    }

    public int getSize()
    {
	return size;
    }

    public void setHeader(Chunk c)
    {
	header = c;
    }
    
    public Chunk getHeader()
    {
	return header;
    }

    public Chunk getFirstChunk()
    {
	currentChunk = data_list;
	return currentChunk;
    }

    public Chunk getNextChunk()
    {
	currentChunk = currentChunk.getNext();
	return currentChunk;
    }
    
    /*	
     * get_num_direct_puts()
     */
    public int get_num_direct_puts()
    {
	return(num_direct_puts);
    }    

    /*
     * get_header_size()
     */
    public int get_header_size()
    {
	return(RESERVED_HEADER_SIZE);
    } /* get_header_buffer() */
    


    
    /*
     * put_TYPE(value)
     */
    public void put_float(float value)
	throws BufferOverrunException
    {
// 	int ds = DataConversion.sizeof_float(1);
// 	if(ds > currentChunk.available())
// 	    addChunk();
	try
	{
	    size += currentChunk.put_float(value);
	}
	catch(ArrayIndexOutOfBoundsException e)
	{
	    addChunk();
	    size += currentChunk.put_float(value);
	}
    }
    public void put_double(double value)
	throws BufferOverrunException
    {
// 	int ds = DataConversion.sizeof_double(1);
// 	if(ds > currentChunk.available())
// 	    addChunk();
	try
	{
	    size += currentChunk.put_double(value);
	}
	catch(ArrayIndexOutOfBoundsException e)
	{
	    addChunk();
	    size += currentChunk.put_double(value);
	}
    }
    public void put_short(short value)
	throws BufferOverrunException
    {
// 	int ds = DataConversion.sizeof_short(1);
// 	if(ds > currentChunk.available())
// 	    addChunk();
	try
	{
	    size += currentChunk.put_short(value);
	}
	catch(ArrayIndexOutOfBoundsException e)
	{
	    addChunk();
	    size += currentChunk.put_short(value);
	}
    }
    public void put_int(int value)
	throws BufferOverrunException
    {
// 	int ds = DataConversion.sizeof_int(1);
// 	if(ds > currentChunk.available())
// 	    addChunk();
	try
	{
	    size += currentChunk.put_int(value);
	}
	catch(ArrayIndexOutOfBoundsException e)
	{
	    addChunk();
	    size += currentChunk.put_int(value);
	}
    }
    public void put_long(long value)
	throws BufferOverrunException
    {
// 	int ds = DataConversion.sizeof_long(1);
// 	if(ds > currentChunk.available())
// 	    addChunk();
	try
	{
	    size += currentChunk.put_long(value);
	}
	catch(ArrayIndexOutOfBoundsException e)
	{
	    addChunk();
	    size += currentChunk.put_long(value);
	}
    }
    public void put_char(char value)
	throws BufferOverrunException
    {
// 	int ds = DataConversion.sizeof_char(1);
// 	if(ds > currentChunk.available())
// 	    addChunk();
	try
	{
	    size += currentChunk.put_char(value);
	}
	catch(ArrayIndexOutOfBoundsException e)
	{
	    addChunk();
	    size += currentChunk.put_char(value);
	}
    }
    public void put_byte(byte value)
	throws BufferOverrunException
    {
// 	int ds = DataConversion.sizeof_byte(1);
// 	if(ds > currentChunk.available())
// 	    addChunk();
	try
	{
	    size += currentChunk.put_byte(value);
	}
	catch(ArrayIndexOutOfBoundsException e)
	{
	    addChunk();
	    size += currentChunk.put_byte(value);
	}
    }


    /*
     * put_TYPE(value[], start, count)
     */
    public void put_float(float value[], int start, int count)
	throws BufferOverrunException
    {
	int es = DataConversion.sizeof_float(1);
	while(count > 0)
	{
	    int n = currentChunk.available() / es;
	    size += currentChunk.put_float(value, start, Math.min(count, n));
	    count -= n;
	    start += n;
	    if(count > 0)
		addChunk();
	}
    }
    public void put_double(double value[], int start, int count)
	throws BufferOverrunException
    {
	int es = DataConversion.sizeof_double(1);
	while(count > 0)
	{
	    int n = currentChunk.available() / es;
	    size += currentChunk.put_double(value, start, Math.min(count, n));
	    count -= n;
	    start += n;
	    if(count > 0)
		addChunk();
	}
    }
    public void put_short(short value[], int start, int count)
	throws BufferOverrunException
    {
	int es = DataConversion.sizeof_short(1);
	while(count > 0)
	{
	    int n = currentChunk.available() / es;
	    size += currentChunk.put_short(value, start, Math.min(count, n));
	    count -= n;
	    start += n;
	    if(count > 0)
		addChunk();
	}
    }
    public void put_int(int value[], int start, int count)
	throws BufferOverrunException
    {
	int es = DataConversion.sizeof_int(1);
	while(count > 0)
	{
	    int n = currentChunk.available() / es;
	    size += currentChunk.put_int(value, start, Math.min(count, n));
	    count -= n;
	    start += n;
	    if(count > 0)
		addChunk();
	}
    }
    public void put_long(long value[], int start, int count)
	throws BufferOverrunException
    {
	int es = DataConversion.sizeof_long(1);
	while(count > 0)
	{
	    int n = currentChunk.available() / es;
	    size += currentChunk.put_long(value, start, Math.min(count, n));
	    count -= n;
	    start += n;
	    if(count > 0)
		addChunk();
	}
    }
    public void put_char(char value[], int start, int count)
	throws BufferOverrunException
    {
	int es = DataConversion.sizeof_char(1);
	while(count > 0)
	{
	    int n = currentChunk.available() / es;
	    size += currentChunk.put_char(value, start, Math.min(count, n));
	    count -= n;
	    start += n;
	    if(count > 0)
		addChunk();
	}
    }
    public void put_byte(byte value[], int start, int count)
	throws BufferOverrunException
    {
	int es = DataConversion.sizeof_byte(1);
	while(count > 0)
	{
	    int n = currentChunk.available() / es;
	    size += currentChunk.put_byte(value, start, Math.min(count, n));
	    count -= n;
	    start += n;
	    if(count > 0)
		addChunk();
	}
    }


    /*
     * sizeof_startpoint()
     */
    public static int sizeof_startpoint(Startpoint sp)
    {
	return (sp.sizeof());
    }
    public static int sizeof_startpoint(Startpoint[] sp,
					int start,
					int count)
    {
	int spsize = 0;
	for (int i = start; i < start + count; i++)
	{
	    spsize += sp[i].sizeof();
	}
	return (spsize);
    }


    /*
     * put_startpoint()
     */
    public void put_startpoint(Startpoint sp)
	throws BufferOverrunException
    {
	sp.put(this);
    }
    public void put_startpoint(Startpoint[] sp,
				   int start,
				   int count)
	throws BufferOverrunException
    {
	for (int i = start; i < start + count; i++)
	{
	    sp[i].put(this);
	}
    }
} /* class PutBuffer */
