/*
 * Queue
 *
 * A general FIFO queue class.
 */

package nexus;

public class Queue
{
    private QueueEntry	head = null;
    private QueueEntry	tail = null;
    private int		threads_waiting = 0;
    private boolean	awakening_threads = false;

    /*
     * add()
     *
     * Add an object to the end of the queue.
     */
    public synchronized void add(Object obj)
    {
	QueueEntry qe = new QueueEntry(obj);
	if (head != null)
	{
	    tail.next = qe;
	    tail = qe;
	}
	else
	{
	    head = qe;
	    tail = qe;
	    if (threads_waiting > 0)
	    {
		try
		{
		    notify();
		}
                catch (Exception e) { e.printStackTrace(); }
	    }
	}
    } /* add() */


    /*
     * remove()
     *
     * Remove and return the first entry in the queue.
     * Return null if the queue is empty.
     */
    public synchronized Object remove()
    {
	QueueEntry qe;
	Object obj;
	if (head != null)
	{
	    qe = head;
	    head = qe.next;
	    obj = qe.contents;
	}
	else
	{
	    obj = null;
	}
	return (obj);
    } /* remove() */

    
    /*
     * remove_with_wait()
     *
     * Remove and return the first entry in the queue.
     * If the queue is empty, wait until something is put in.
     */
    public synchronized Object remove_with_wait()
    {
	QueueEntry qe;
	Object obj = null;

	while (obj == null && (!awakening_threads || head != null))
	{
	    if (head != null)
	    {
		qe = head;
		head = qe.next;
		obj = qe.contents;
	    }
	    else
	    {
		try
		{
		    threads_waiting++;
		    wait();
		    threads_waiting--;
		}
		catch (Exception e) { e.printStackTrace(); }
	    }
	}
	return (obj);
    } /* remove_with_wait() */


    /*
     * awaken_all_waiting_threads()
     */
    public synchronized void awaken_all_waiting_threads()
    {
	try
	{
	    awakening_threads = true;
	    notifyAll();
	}
	catch (Exception e) { e.printStackTrace(); }
    } /* awaken_all_waiting threads() */
    
} /* class Queue */


class QueueEntry
{
    public QueueEntry	next;
    public Object	contents;

    public QueueEntry(Object obj)
    {
	next = null;
	contents = obj;
    } /* QueueEntry() */
    
} /* class QueueEntry */
