/*
 * TcpOutgoing
 *
 * extends Object
 */

package nexus;

import java.util.Vector;
import java.net.Socket;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;

public class TcpOutgoing implements OutgoingInterface
{
    private Nexus				nexus;
    private TcpProto				tcp_proto;
    private String				hostname;
    private int					port;

    private boolean				is_open;
    private boolean				is_shutdown;
    private Socket				socket;
    private OutputStream			output_stream;
    private InputStream				input_stream;

    /*
     * TcpOutgoing.TcpOutgoing()
     */
    public
    TcpOutgoing(
	Nexus					nexus,
	TcpProto				tcp_proto,
	String					hostname,
	int					port)
    {
	this.nexus = nexus;
	this.tcp_proto = tcp_proto;
	this.hostname = hostname;
	this.port = port;

	this.is_open = false;
	is_shutdown = false;
	
	tcp_proto.add_outgoing(this);
    }
    /* TcpOutgoing.TcpOutgoing() */

    /*
     * TcpOutgoing.get_Protocol_Version()
     */
    public
    int
    get_Protocol_Version()
    {
	return TcpProto.PROTOCOL_VERSION;
    }
    /* TcpOutgoing.get_Protocol_Version() */

    /*
     * TcpOutgoing.send_rsr()
     */
    public synchronized
    void
    send_rsr(
	byte[]					buffer)
    {
	if (is_shutdown)
	{
	    Nexus.debug_println(1,
				"TcpOutgoing.send_rsr(): " +
				"outgoing already shutdown, exiting");
	}
	
	/*
	 * open the socket if necessary
	 */
	if (!is_open)
	{
	    Nexus.debug_println(2,
				"TcpOutgoing.send_rsr(): " +
				"opening connection to " +
				hostname + ":" + port);

	    if (open() == false)
	    {
		Nexus.debug_println(1,
				    "TcpOutgoing.send_rsr(): " +
				    "unable to open " + hostname + ":" + port);
		return;
	    }

	    Nexus.debug_println(2,
				"TcpOutgoing.send_rsr(): " +
				"connection established with " +
				hostname + ":" + port);
	}

	/*
	 * write buffer data
	 */
	Nexus.debug_println(2,
			    "TcpOutgoing.send_rsr(): " +
			    "writing " + buffer.length + " bytes");

	if (write(buffer, 0, buffer.length) == false)
	{
	    if (read_close_msg() == true)
	    {
		Nexus.debug_println(1,
				    "TcpOutgoing.send_rsr(): " +
				    "connection closed remotely");
	    }
	    else
	    {
		Nexus.debug_println(1,
				    "TcpOutgoing.send_rsr(): " +
				    "connection failed");
	    }

	    close();

	    return;
	}

	Nexus.debug_println(2,
			    "TcpOutgoing.send_rsr(): " +
			    buffer.length + " bytes written");
    }
    /* TcpOutgoing.send_rsr() */

    public synchronized
    void
    send_rsr(
	PutBuffer				buffer)
    {
	if (is_shutdown)
	{
	    Nexus.debug_println(1,
				"TcpOutgoing.send_rsr(): " +
				"outgoing already shutdown, exiting");
	}
	
	/*
	 * open the socket if necessary
	 */
	if (!is_open)
	{
	    Nexus.debug_println(2,
				"TcpOutgoing.send_rsr(): " +
				"opening connection to " +
				hostname + ":" + port);

	    if (open() == false)
	    {
		Nexus.debug_println(1,
				    "TcpOutgoing.send_rsr(): " +
				    "unable to open " + hostname + ":" + port);
		return;
	    }

	    Nexus.debug_println(2,
				"TcpOutgoing.send_rsr(): " +
				"connection established with " +
				hostname + ":" + port);
	}

	/*
	 * write buffer data
	 */
	boolean success;
        Nexus.debug_println(2, "TcpOutgoing.send_rsr(): writing "+buffer.getSize()+" bytes");
	Chunk c;
	c = buffer.getHeader();
	success = write(c.getData(), c.getStart(), c.getSize());
	c = buffer.getFirstChunk();
	while(success && (c != null))
	{
	    success = write(c.getData(), c.getStart(), c.getSize());
	    c = buffer.getNextChunk();
	}
	if(!success)
	{
	    if (read_close_msg() == true)
	    {
		Nexus.debug_println(1,
				    "TcpOutgoing.send_rsr(): " +
				    "connection closed remotely");
	    }
	    else
	    {
		Nexus.debug_println(1,
				    "TcpOutgoing.send_rsr(): " +
				    "connection failed");
	    }

	    close();

	    return;
	}
	Nexus.debug_println(2,
			    "TcpOutgoing.send_rsr(): " +
			    buffer.getSize() + " bytes written");
    }
    /* TcpOutgoing.send_rsr() */

    
    /*
     * TcpOutgoing.shutdown()
     */
    public synchronized
    void
    shutdown()
    {
	Nexus.debug_println(
	    2,
	    "TcpOutgoing.shutdown(): " +
	    "shutting down connection to " + hostname + ":" + port);

	if (is_shutdown)
	{
	    Nexus.debug_println(1,
				"TcpOutgoing.shutdown(): " +
				"outgoing already shutdown, exiting");
	}
	
	is_shutdown = true;
	
	if (is_open)
	{
	    /*
	     * Send close message before closing connection
	     */
	    if (write_close_msg() == true)
	    {
		Nexus.debug_println(2,
				    "TcpOutgoing.shutdown(): " +
				    "close message sent");
	    }
	    else
	    {
		if (read_close_msg() == true)
		{
		    Nexus.debug_println(2,
					"TcpOutgoing.shutdown(): " +
					"connection already closed remotely");
		}
		else
		{
		    Nexus.debug_println(
			1,
			"TcpOutgoing.shutdown(): " +
			"connection failed, proper shutdown failed");
		}
	    }
	    
	    if (close() == true)
	    {
		Nexus.debug_println(2,
				    "TcpOutgoing.shutdown(): " +
				    "connection successfully closed");
	    }
	    else
	    {
		Nexus.debug_println(1,
				    "TcpOutgoing.shutdown(): " +
				    "connection close failed");
	    }
	}

	tcp_proto.remove_outgoing(this);
	
	Nexus.debug_println(
	    2,
	    "TcpOutgoing.shutdown(): " +
	    "connection to " + hostname + ":" + port + " shut down");
    }
    /* TcpOutgoing.shutdown() */

    /*
     * TcpOutgoing.open()
     */
    private
    boolean
    open()
    {
	socket = null;
	output_stream = null;
	input_stream = null;
	is_open = false;
	
	try
	{
	    socket = new Socket(hostname, port);
	    output_stream = socket.getOutputStream();
	    input_stream = socket.getInputStream();

	    is_open = true;
	    
	    return true;
	}
	catch (Exception e)
	{
	    e.printStackTrace();
	    close();
	    
	    return false;
	}
    }
    /* TcpOutgoing.open() */

    /*
     * TcpOutgoing.write()
     */
    private
    boolean
    write(
	byte []					buffer,
	int                                     offset,
	int                                     length)
    {
    	try
	{
	    output_stream.write(buffer, offset, length);
	    
	    return true;
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	    return false;
	}
    }
    /* TcpOutgoing.write() */

    /*
     * TcpOutgoing.read_close_msg()
     */
    private
    boolean
    read_close_msg()
    {
	int bytes_read, bytes_left;
	
	try
	{
	    if (input_stream.available() < 2)
	    {
		return false;
	    }
	
	    byte []				buffer;

	    buffer = new byte[2];
	    bytes_read = 0;
	    bytes_left = buffer.length;
	    while(bytes_left > 0)
	    {
		int nbytes = input_stream.read(buffer, bytes_read, bytes_left);
		bytes_read += nbytes;
		bytes_left -= nbytes;
	    }
		
	    if (buffer[0] == (byte) get_Protocol_Version() &&
		buffer[1] == (byte) TcpProto.CLOSE_NORMAL_FLAG)
	    {
		return true;
	    }
	    else
	    {
		return false;
	    }
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	    return false;
	}
    }
    /* TcpOutgoing.read_close_msg() */


    /*
     * TcpOutgoing.write_close_msg()
     */
    private
    boolean
    write_close_msg()
    {
	try
	{
	    byte []				buffer;

	    buffer = new byte[2];
	    buffer[0] = (byte) get_Protocol_Version();
	    buffer[1] = (byte) TcpProto.CLOSE_NORMAL_FLAG;

	    output_stream.write(buffer, 0, buffer.length);

	    return true;
	}
	catch (IOException e)
	{
	    e.printStackTrace();
	    return false;
	}
    }
    /* TcpOutgoing.read_close_msg() */


    /*
     * TcpOutgoing.close()
     */
    private
    boolean
    close()
    {
	boolean					rc;

	rc = true;
	
	is_open = false;
		
	if (input_stream != null)
	{
	    try
	    {
		input_stream.close();
	    }
	    catch(Exception e2)
	    {
		e2.printStackTrace();
		rc = false;
	    }

	    input_stream = null;
	}
	    
	if (output_stream != null)
	{
	    try
	    {
		output_stream.close();
	    }
	    catch(Exception e2)
	    {
		e2.printStackTrace();
		rc = false;
	    }

	    output_stream = null;
	}
	    
	if (socket != null)
	{
	    try
	    {
		socket.close();
	    }
	    catch(Exception e2)
	    {
		e2.printStackTrace();
		rc = false;
	    }

	    socket = null;
	}
	    
	return rc;
    }
    /* TcpOutgoing.close() */
	
}
/* class TcpOutgoing */
