
package nexus;

import nexus.com.totem.*;
import java.util.*;
import java.net.*;
 
public class TotemProto
  implements ProtoInterface
{
    public static final int PROTOCOL_VERSION = Nexus.BUFFER_VERSION + 0;
    public static final int MI_PROTO_VERSION = 0;

  private Nexus        nexus;
  private Hashtable    pgHash; // maps process group id's to vectors of endpoints
  private PG_Inf       pg;     // process group layer
  private boolean      connected = false;  
  private Queue        msgQueue;
  private TotemReader  reader;
  private NexusThread  rThread;
  private TotemHandler handler;
  private NexusThread  hThread;

  /*
   * TotemProto()
   */
  public TotemProto(Nexus nexus) {

Nexus.debug_println(0, "TotemProto.TotemProto():");

    this.nexus = nexus;

    // create process group layer
    pg = new PG_Inf();

    // create hashtable to map from pgid to epid's
    pgHash = new Hashtable();

    // as messages are received by the reader
    // they are placed on the queue to be handled
    // by the handler
    msgQueue = new Queue();

    // instantiate reader
    reader = new TotemReader(pg, this, msgQueue, nexus);
    rThread = new NexusThread(nexus, reader, "totem_reader");
    rThread.setDaemon(true);
    rThread.start();

    // instantiate handler
    handler = new TotemHandler(this, msgQueue, pgHash, nexus);
    hThread = new NexusThread(nexus, handler, "totem_handler");
    hThread.setDaemon(true);
    hThread.start();

  } /* TotemProto() */

  /*
   * get_my_mi_proto()
   */
  public byte[] get_my_mi_proto(ProtoInfo protoInfo,
                                long endpointID) {

    /* TODO: XXX: fix mi_proto to include version number */
    TotemInfo info; 
    byte[] miProto;

Nexus.debug_println(0, "TotemProto.get_my_mi_proto():");

    // get user-specified connection information
    info = (TotemInfo)protoInfo;

    // connect to Totem server
    this.connect(info, endpointID);
 
    // return Machine Independent Proto for connection
    miProto = new byte[DataConversion.sizeof_long(1) + 1];
    miProto[0] = (byte)(DataConversion.NEXUS_DC_FORMAT_JAVA);
    DataConversion.put_long((long)(info.pgid), miProto, 1);
System.out.println("TotemProto.get_my_mi_proto(): pgid = " + info.pgid);
for (int i = 0; i < miProto.length; i++)
  System.out.print(" " + miProto[i]);
System.out.flush();
System.out.println("");
    return miProto;
  
  } /* get_my_mi_proto() */

  /*
   * construct_from_mi_proto()
   */
  public OutgoingInterface construct_from_mi_proto(byte[] mi_proto,
                                                    int start,
                                                    int size,
                                                    String node_name,
                                                    boolean[] is_valid) {
    TotemOutgoing outgoing;
    int pgid;

Nexus.debug_println(0, "TotemProto.construct_from_mi_proto():");

    // get pgid - process group to send outgoing message to
    try {
      pgid = (int)(DataConversion.get_long(mi_proto, start + 1,
                                           (int)(mi_proto[start])));
System.out.println("TotemProto.construct_from_mi_proto(): pgid = " + pgid);
for (int i = 0; i < mi_proto.length; i++)
  System.out.print(" " + mi_proto[i]);
System.out.flush();
System.out.println("");
    }
    catch (LostPrecisionException lpX) {
      pgid = 0;
      System.err.println("TotemProto.construct_from_mi_proto(): "
                         + "Lost precision on pgid conversion.  "
                         + "Using pgid = " + pgid);
    }

/*
    // if not already connected, attempt local connection.
    if (!connected) {
      String host;
      try {
        host = InetAddress.getLocalHost().getHostName();
        this.connect(new TotemInfo(host, pgid), (long)(-1));
      }
      catch (Exception x) {
        System.err.println("TotemProto.construct_from_mi_proto(): "
                           + "Unable to determine local machine's name. ");
      }
    }   
*/

    return new TotemOutgoing(this, pgid);

  } /* construct_from_mi_proto() */ 

  /*
   * connect()
   */
  private void connect(TotemInfo info,
                       long endpointID) {

    Vector  epVector;

Nexus.debug_println(0, "TotemProto.connect():");

    // attempt to connect, if not already connected
    if (!connected) {
      if (pg.ProcessConnect(info.host) == -1) {
System.err.println("TotemProto.connect(): failed connection");
        System.err.println("TotemProto.connect(): "
                           + "Failed to connect to Totem server on "
                           + info.host + ":" + PG_Inf.PORT);
      }
      else {
System.err.println("TotemProto.connect(): successful connection");
        connected = true;
      }
    }

    if (connected) {

      Integer key = new Integer(info.pgid);

      // if member of process group, add endpointID to process
      // group's endpoint vector 
      if (pgHash.containsKey(key)) {

/*
System.err.println("TotemProto.connect(): connection already a member of group "
                   + info.pgid + " adding endpointID " + endpointID);
*/
        if (endpointID != -1) {
          epVector = (Vector)(pgHash.get(key));
          epVector.addElement(new Long(endpointID));
        }
      }
      // if not member of process group, join group, create endpoint
      // vector for group, add endpointID to it, and add pgid to
      // process group hashtable
      else {
/*
System.err.println("TotemProto.connect(): connection not a member of group, "
                   + info.pgid + " joining...");
*/
        if (pg.ProcessJoin(info.pgid) == -1) {
          System.err.println("TotemProto.connect(): "
                             + "Failed to join process group "
                             + info.pgid);
        }
        else {
/*
System.err.println("TotemProto.connect(): connection joined group "
                   + info.pgid + " adding endpointID " + endpointID);
*/
          epVector = new Vector();
          if (endpointID != -1) {
            epVector.addElement(new Long(endpointID));
          }
          pgHash.put(key, epVector);
        }
      }
    }
//System.err.println("TotemProto.connect(): exiting");
  } /* connect() */ 

  /*
   * shutdown()
   */ 
  public void shutdown()
  {
Nexus.debug_println(0, "TotemProto.shutdown(): ");
    reader.shutdown();
    handler.shutdown();
    pg.close();
  } /* shutdown() */

  /*
   * send_rsr()
   *
   * sends a message to a Process Group
   *  int     length of complete message
   *  int     message type can be AGREED, SAFE
   *  InfId[] an array defining process groups to send to
   *  byte[]  data to be sent
   *  returns an integer that is used to flag any error
   */
  public void send_rsr(byte[] buffer,
                       int pgid) {

Nexus.debug_println(0, "TotemProto.send_rsr(): ");

    InfId[] infId = new InfId[1];
    infId[0] = new InfId(pgid);

/*
System.out.println("\nTotemProto.send_rsr(): sending buffer of length = "
                   + buffer.length + " to  process group " + pgid + "\n");
    for (int i = 0; i < buffer.length; i++)
      System.out.print(" " + buffer[i]);
    System.out.flush(); 
*/

    // send message
    if (pg.ProcessSend(buffer.length, PG_Inf.AGREED, infId, buffer) == -1) {
      System.err.println("TotemProto.send_rsr(): "
                         + "Error occurred while sending message");
    }

  } /* send_rsr() */

  /*
   * connected()
   */
  public boolean connected() {
    return connected;
  } /* connected() */

} /* TotemProto */


/*
 * TotemReader
 *   receives messages from process group layer
 */
class TotemReader
  implements Runnable {

  private PG_Inf     pg;
  private TotemProto totem;
  private Queue      msgQueue;
  private Nexus      nexus;
  private boolean    done = false;

  /*
   * TotemReader()
   */
  public TotemReader(PG_Inf pg,
                     TotemProto totem,
                     Queue msgQueue,
                     Nexus nexus) {
    this.pg = pg;
    this.totem = totem;
    this.msgQueue = msgQueue;
    this.nexus = nexus;
  } /* TotemReader() */

  /*
   * run() 
   */
  public void run() {
   
    nexus.add_thread(Thread.currentThread()); 

    while (!nexus.is_destroying() && !done)  {

      // don't try to read until Totem is connected
      if (totem.connected()) {
//System.err.println("TotemReader.run(): before ProcessReceive().");
        if (pg.ProcessReceive() != -1) {
          // successfully received message
          // enqueue header and message
//System.err.println("TotemReader.run(): after ProcessReceive().");
          this.receive();
        }
        else {
          System.err.println("TotemReader.run(): Error receiving message.");
        }
      }
    }

    nexus.remove_thread(Thread.currentThread());

  } /* run() */

  /*
   * receive()
   */
  private synchronized void receive() {

    if (pg.infh.type == PG_Inf.AGREED || pg.infh.type == PG_Inf.SAFE) {
      //System.out.println("TotemReader.receive(): non-control message.");

      byte[] b = new byte[pg.infh.msgLength()/*pg.data.length*/];
      System.arraycopy(pg.data, 0, b, 0, pg.data.length);

      InfHeader infHdr = new InfHeader(PG_Inf.AGREED, 1,
                                       pg.infh.pgid, 96, b.length);
  
/*
      System.out.println("\nTotemReader.run(): read buffer of length = "
                         + pg.data.length + "\n");
      for (int i = 0; i < pg.data.length; i++)
        System.out.print(" " + pg.data[i]);
      System.out.flush(); 

      System.out.println("\nTotemReader.run(): read buffer of length = "
                         + b.length + "\n");
      for (int i = 0; i < b.length; i++)
        System.out.print(" " + b[i]);
      System.out.flush(); 
*/
  
      msgQueue.add(infHdr); 
      msgQueue.add(b); 
    }
    else {
//      System.out.println("TotemReader.receive(): control message.");
    }
  } /* receive() */

  /*
   * shutdown()
   */
  public synchronized void shutdown() {

    // pg is closed by TotemProto.shutdown()
    done = true;

  } /* shutdown() */

} /* TotemReader */
     

/*
 * TotemHandler
 *   dispatches received meassages to appropriate endpoints
 */
class TotemHandler
  implements Runnable {

  private TotemProto totem;
  private Queue      msgQueue;
  private Hashtable  pgHash;
  private Nexus      nexus;
  private boolean    done = false;

  /*
   * TotemHandler()
   */
  public TotemHandler(TotemProto totem,
                      Queue msgQueue,
                      Hashtable pgHash, 
                      Nexus nexus) {
    this.totem = totem;
    this.msgQueue = msgQueue;
    this.pgHash = pgHash;
    this.nexus = nexus;
  }  /* TotemHandler() */

  /*
   * run()
   */
  public void run() {

    InfHeader header;
    byte[]    msg;
    Vector    epVector;
    Long      epID;
     
    nexus.add_thread(Thread.currentThread());

    while (!nexus.is_destroying() && !done)  {

      try {
        // get header and message
        header = (InfHeader)(msgQueue.remove_with_wait());
        msg = (byte[])(msgQueue.remove_with_wait());

        // for each pgid in header
        for (int i = 0; i < header.num_sets; i++) {

          // get process group's associated endpoint vector
          epVector = (Vector)(pgHash.get(new Integer(header.pgid[i].pid))); 

          if (epVector != null) {
            // for each endpoint in pg
            for (Enumeration enum = epVector.elements();
                 enum.hasMoreElements(); )
            {
              epID = (Long)(enum.nextElement());

              // dispatch message
              nexus.buffer_dispatch(msg, epID.longValue());
            }
          }
        }
      }
      catch (Exception x) {
        x.printStackTrace();
      }
    }

    nexus.remove_thread(Thread.currentThread());

  } /* run() */

  /*
   * shutdown()
   */
  public synchronized void shutdown() {
    done = true;
  } /* shutdown() */

} /* TotemHandler */
