
package nexus;

import java.util.*;
import java.net.*;
import java.io.*;

class UdpMessage
{
  public int         messageNum;
  public int         fromPort;
  public InetAddress fromAddr;
  public long        endpointID;

  private int        totalSize;
  private int        expectedSeqNum;
  private int        offset;
  private byte[]     message;
  private Nexus      nexus;


  public UdpMessage(int inSize, int msg_num, UdpPacket packet, Nexus inNexus)
  {
    totalSize = inSize;
    messageNum = msg_num;
    fromAddr = packet.getAddress();
    fromPort = packet.getPort();
    endpointID = packet.getEndpointID();
    message = new byte[inSize];
    offset = 0;
    expectedSeqNum = 0;
    nexus = inNexus;
  }

  public static int unpackSeqNum(byte b[])
  {
   int seq_num;

   seq_num = (int)b[1] << 16;
   seq_num += ((int)b[2] << 8);
   seq_num += ((int)b[3]);

   return seq_num;
  }

  public static int unpackMsgNum(byte b[])
  {
   int msg_num;

   msg_num = (int)b[0];

   return msg_num;
  }

  public static byte [] packHeader(int msg_num, int seq_num, int size)
  {
    byte b[];

    b = new byte[size + UdpProto.UDP_HDR_SIZE];

    b[0] = (byte)msg_num;
    b[1] = (byte)((seq_num >> 16) & 0xff);
    b[2] = (byte)((seq_num >> 8) & 0xff);
    b[3] = (byte)((seq_num) & 0xff);

    return b;
  }


  /*  return true if object is to be destroyed.
             false if it is still working. */
  public boolean addBytes(byte b[], int size, int seqNum)
  {
    if(seqNum != expectedSeqNum)
    {
      return true;
    }
    try
    {
      System.arraycopy(b, UdpProto.UDP_HDR_SIZE, message, offset,
                       size - UdpProto.UDP_HDR_SIZE);
      offset += size;
    }
    catch(Exception e)
    {
      return true;
    }

    if(offset >= totalSize)
    {
      nexus.buffer_dispatch(message, endpointID);
      return true;
    }

    expectedSeqNum++;
    return false;
  }
}

