package nexus.com.totem;

import java.net.*;
import java.io.*;
import java.lang.Math.*;

/** defines process group identifieror the user process identifier depending
    on usage.
    contains integer <i>pid</i> and byte[] <i>proc</i> fields <br>
    specifying <i>pid</i> alone (<i>proc</i> initialized) specifies a Process
    Group <br>
    specifying both <i>pid</i> and <i>proc</i> specifies a user process.
  */
public class InfId {
  /** pid field defines process identifier / Process group based on usage. */
  /* doug */
  public
  /* doug */
  int pid;   
  /** processor identifier proc (in IP address format) defines host runing
      Totem that the user has connected to. */
  byte[] proc;     
  
 
  /** Creates an empty identifier for IPV4. */
  InfId() { 
    pid = 0;
    proc = new byte[4];
  }
  
  /** creates a <i> Process Group </i> identifier for IPV4 by specifying
      the pid field and initializing the proc field.
      @param i an integer defining the process group identifier*/
  public InfId(int i) { 
    pid = i;
    proc = new byte[4]; // initialised processor id
  }

  /** creates a process identifier (= both pid + proc specified) */
  InfId(int i, byte[] b) {  
    pid = i;
    proc = b;
  }
  

  //-------------------------------------------------------------------------
  /**
    * Print this string for debugging - Can be easily commented out, if no
    * debugging is desired <br>
    * If the output call inside this method is commented out, the compiler
    * should eliminate the call of debug().
    */
  public static final void debug( String str )
  {
    System.out.println( "InfId: "+ str );
    System.out.flush();
  }

  //-------------------------------------------------------------------------  
  /** returns length (in bytes) of complete identifer (pid + proc) */
  int length() 
  {           
    return (4 + proc.length); 
  }
  
  /** sets the process identifer (pid) to specifed value */
  void setPid(int i) 
  {    
    pid = i;
  }
  
  /** sets processor identifier(proc) to specified value */
  void setProc(byte[] b) { // set processor to specified value
    proc = b; 
  }
  
  /** writes processor id to output data stream <br>
    * throws exception to be caught by higher level
    */
  void write(DataOutputStream out) throws IOException {
    out.write(proc, 0, proc.length);
    out.writeInt(pid);  // writes int high byte first
  }
  
  /** reads complete id (proc then pid) from input data stream <br>
    * accepts 4bytes as input - not sufficient for IPv6 
    */
  void read(DataInputStream in) throws IOException {
    int i;
    proc = new byte[4];
    i = in.read(proc, 0, 4);  // will not work with IPV6
    debug("Expected to read 4 bytes, got "+i);
    pid = in.readInt();
  }

  /** prints complete identifier (first pid then proc)*/
  void print() throws IOException {

    System.out.print("InfId: Process Id : ");
    System.out.print(pid);

    System.out.print("\tProcessor = ");
    for (int i = 0; i < proc.length; i++) 
    {
      // convert signed byte to unsigned byte and print it
      System.out.print( ((proc[i]<0) ? (256+proc[i]) : (proc[i])) + "." );
    }
    System.out.print( "\b \n" );
  }
}
