package nexus.com.totem;

import java.net.*;
import java.io.*;

/** Implements the process group layer above the Totem Protocol */
public class PG_Inf {

  /** socket to connect the user to totem */
  Socket my_mbox;  

  /** Header Last Received By App */
  public InfHeader infh = new InfHeader();

  /** identifier of last sender */
  InfId sender   = new InfId();      // 

  /**data part of message last received */
  public byte[] data;   

  /** the identifier assigned to this user process */
  InfId my_id;

  /** port setup for connecting to Totem */
  public static int PORT = 4589;         

  /** mask for agreed delivery message type*/
  public static int AGREED = 0x0030; 

  /** mask for safe delivery message type */
  public static int SAFE   = 0x0040;

  /** Message type for a Join request */   
  public static int JOIN_SET  = 1002;
 
  /** Message type for a Join confirm */
  public static int JOIN_ANS  = 1102;

  /** Message type for a Leave request */
  public static int LEAVE_SET = 1004;

  /** Message type for a Leave confirm */
  public static int LEAVE_ANS = 1104;

  /** Message type to comfirm a Disconnect Ok */
  public static int DISCONNECT_OK = 1105;

  /** Message type for a Membership message */
  public static int MEMBERSHIP = 1108;

  // boolean isMembershipMsg()  ????

  //-------------------------------------------------------------------------
  /**
    * Print this string for debugging. Can be easily commented out, if no
    * debugging is desired.
    * If the output call inside this method is commented out, the compiler
    * should eliminate the call of debug().
    */
  public static final void debug( String str )
    {
      System.err.println( "PG_Inf: "+ str );   
    }
  //-------------------------------------------------------------------------  
  
  /** connects the application to host running TOTEM
    @return currently returns an integer that flags the occurence of errors */
  public int ProcessConnect(String host) { return ProcessConnect(host, PORT); }
  public int ProcessConnect(String host, int port) {
    
    int my_logical_name;
    
    try {
      my_mbox = new Socket(InetAddress.getByName(host), port);
      
      // sets up a data stream to Totem 
      DataOutputStream out = new DataOutputStream(my_mbox.getOutputStream());
      
      // sets up a data stream to receive from Totem
      DataInputStream   in = new DataInputStream(my_mbox.getInputStream());

      debug("Connect >> Sending data");

      // let's TOTEM know we're here and to assign us a unique id
      out.writeInt(0);  // = 4bytes
	    
      //            System.out.println("Wrote" + out.size() + "bytes");
      //            System.out.flush();

      /*            while (in.available() == 0);

		    System.out.print("Receiving data:  ");
		    System.out.println(in.available());
		    System.out.flush();
      */

      // Totem creates a process id by which to refer to this user
      byte[] tmp = new byte[4];
      in.read(tmp,0,4);   // receive our process id

      int i = Fun.BytetoInteger(tmp, 0);  // convert the process id to an integer
      //      	    System.out.println(i);
      debug("Connect >> Received unique id from Totem");

      // my_id consists of process id assigned + processor id (host) connecting to
      // this creates a unique process identifier for this user
      my_id = new InfId(i, (InetAddress.getByName(host)).getAddress()); 

      //Inf.insert(my_id);

    }
    catch (UnknownHostException e) {
      // check spelling of hostname
      debug(" an exception occurred " + e.getMessage());
      e.printStackTrace();
      return -1;
    } 
    catch (IOException e) {
      // some sort of error
      debug(" an exception occurred " + e.getMessage());
      e.printStackTrace();
      return -1;
    } 
    catch (NullPointerException e) {
      debug(" an exception occurred " + e.getMessage());
      e.printStackTrace();
      return -1;
    }
    return 0;
  }

  /** sends a message to a Process Group
      @param len length of complete message
      @param type message type can be AGREED, SAFE
      @param pgid an array defining process groups to send to
      @param buf a byte array that contains the data to be sent
      @return returns an integer that is used to flag any error
  */
  public int ProcessSend(int len, int type, InfId[] pgid, byte[] buf) {

    // by default creates InfHeader to accomodate max. destination process group
    // but however restricts the number of dstinations to 1??
    InfHeader ih = new InfHeader(type, 1, pgid, InfHeader.MAX_LENGTH, len);
    
    try {
      byte[] tmp = new byte[(InfHeader.MAX_PG-ih.num_sets)*8];

      DataOutputStream out = new DataOutputStream(my_mbox.getOutputStream());

      debug("\nPSend>> writing total length of msg: "+ih.len);
      out.writeInt(ih.len);  // Totem expects the length of the message first

      debug("PSend>> writing data part of header: ");
      ih.print();
      ih.write(out);         // write the header to Totem

      debug("PSend>> padding:    "+tmp.length);
      out.write(tmp, 0, tmp.length);  // padding (blank pids)

      //debug("Sent data:" + buf.toString());
      //    for (int i = 0; i < len; i++)
      // 	System.out.print(" "+(int)buf[i]);
      //       System.out.println();

      debug("PSend>> writing data array:");
      debug("PSend>> len:        "+len);
      debug("PSend>> buf.length: "+buf.length);
      out.write(buf, 0, len);  // write the data
    } 
    catch (UnknownHostException e) {
      // check spelling of hostname
      debug("an exception occurred: " + e.getMessage());
      e.printStackTrace();
      return -1;
    } 
    catch (IOException e) {
      // some sort of error
      debug("an exception occurred: " + e.getMessage());
      e.printStackTrace();
      return -1;
    } 
    catch (NullPointerException e) {
      debug("an exception occurred: " + e.getMessage());
      e.printStackTrace();
      return -1;
    }
    return 0;
  }

  // receive a message; place header into infh, sender id into sender, message into data
  public int ProcessReceive()
    {
      int amnt_rd = 0; // amount read so far
      int membershipCount, i;
      int data_length = -1;
      //InfNode inf_p;

      //inf_p = InfSearchNode();
      try {
	DataInputStream in = new DataInputStream(my_mbox.getInputStream());
System.out.println("PG_Inf.ProcessReceive(): before first read");
	// add inf_p for msg_size
	infh.read(in);            // read the header
System.out.println("PG_Inf.ProcessReceive(): after first read");
	amnt_rd = infh.length();  // how much was read
	debug("\nPReceive>> amnt_rd = infh.length()       = "+amnt_rd);

	debug("Header read = ");
	infh.print();
	debug("end of header");

	int tmp = infh.header_len - amnt_rd;  
	// amount of blank space we need to read before getting to valid data (TOTEM)

	//            System.out.println("Skipping " + tmp + " bytes");

	//            byte[] btmp = new byte[tmp];
	//            in.read(btmp, 0, btmp.length);
	/*            for (i = 0; i < btmp.length; i++)
		      System.out.print((int)btmp[i]);
		      System.out.println();
		      System.out.write(btmp, 0, btmp.length);
		      System.out.println();
	*/
	in.skipBytes(infh.header_len - amnt_rd);
	debug("PReceive>> skipped infh.header_len - amnt_rd = "+tmp);
	sender.read(in);  // sender id is always first info in message

	//            System.out.println("Sender read = " + sender.length());

	amnt_rd += sender.length();
	debug("PReceive>> amnt_rd += sender.length()    = "+amnt_rd);
	sender.print();

	if (infh.type == DISCONNECT_OK) {
	  // if (sender.pid == inf_p.inf_name.inf_pid) ProcessClose();
	  // else
	  infh.type = LEAVE_ANS; 
	  // since only one PG, DISCONNECT equivalent to leaving this PG
	}


	// Process  message content (i.e after header separation)
	//- place message content in data

	if (infh.type != MEMBERSHIP) {  
	  // not a membership message, so read in as a normal message
	  i = infh.len - infh.header_len - sender.length(); 
	  // i = length of actual data content

	  //                System.out.println("Expecting to read " + i + " bytes");

	  this.data = new byte[i];
	  debug("PReceive>> init new byte array with length i = "+i);

	  // that's the total no. of bytes to read
	  tmp = 0;

	  while( tmp < data.length )
	  {
	    tmp +=  in.read(data, tmp , data.length - tmp );
	    debug("PReceive>> tmp: "+tmp);
	  }
	  debug("PReceive>> data.length [should be i]         = "+data.length);
	  debug("PReceive>> Data read tmp                     = " + tmp);
	
	  amnt_rd += data.length;
	  debug("PReceive>> amnt_rd                  = "+amnt_rd);

	  //                for (i = 0; i < this.data.length; i++)
	  //                    System.out.print((int)this.data[i]);
	  //                System.out.println();
	}
	else {  // this is a membership message 

	  membershipCount = in.readInt();  // first the number of members

	  debug("Membership Count: " + membershipCount);

	  i = membershipCount*sender.length();

	  //                System.out.println("Expecting to read " + i + " bytes");

	  this.data = new byte[i];
	  in.read(data, 0, data.length);

	  //                System.out.println("Data read = " + data.length);

	  amnt_rd += data.length;

	  //                System.out.print(data);
	  //                 System.out.println();
	  // 		System.out.println("Membership: ");
	  // 		for (i = 0; i < membershipCount; i++)
	  // 		  Fun.PrintMember(data, i*sender.length());
	}
      } 
      catch (UnknownHostException e) {
	// check spelling of hostname
	debug("an exception occurred: " + e.getMessage());
	System.out.println(e.getMessage());
	e.printStackTrace();
	return -1;
      }
      catch (IOException e) {
	// some sort of error
	debug("an exception occurred: " + e.getMessage());
	System.out.println(e.getMessage());
	e.printStackTrace();
	return -1;
      } 
      catch (NullPointerException e) {
	debug("an exception occurred: " + e.getMessage());
	System.out.println(e.getMessage());
	e.printStackTrace();
	return -1;
      }
	
      //        System.out.println("Total amount read = " + amnt_rd);
      return amnt_rd;
    }



  // Join a process group
  public int ProcessJoin(int pgid) {
    return ProcessJoinOrLeave(pgid, JOIN_SET);
  }

  // leaving a process group
  public int ProcessLeave(int pgid) {
    return ProcessJoinOrLeave(pgid, LEAVE_SET);
  }

  // implementation of joining and leaving of process groups
  // what this achieves is to just pass on the header alone with appropriate type.
  // Join and Leave request messages have no data content
  private int ProcessJoinOrLeave(int pgid, int type)
    {
      int sig_mask, msg_len;

      InfId[] id = new InfId[1];
      id[0] = new InfId(pgid);
      InfHeader ih = new InfHeader(type, 1, id, 0);

      try {
	DataOutputStream out = new DataOutputStream(my_mbox.getOutputStream());  // throws IO exception

	out.writeInt(ih.len);
	ih.write(out);
	//            System.out.println("ProcessJoinOrLeave: msg type = " + type + ", pgid = " + pgid);
	//            System.out.println("Length: " + ih.len);
      } 
      catch (UnknownHostException e) {
	// check spelling of hostname
	debug("an exception occurred: ");
	System.out.println(e.getMessage());
	e.printStackTrace();
	return -1;
      }
      catch (IOException e) {
	// some sort of error
	debug("an exception occurred: ");
	System.out.println(e.getMessage());
	e.printStackTrace();
	return -1;
      }
      catch (NullPointerException e) {
	debug("an exception occurred: ");
	System.out.println(e.getMessage());
	e.printStackTrace();
	return -1;
      }
      return 0;
    }
  
  // closes the connection (closes the socket to Totem)
  public void close() {

    try {
      my_mbox.close();
    } catch (IOException e) {
      debug("Couldn't close");
    }
  }
}




