/* 
 * NexusRMI
 *
 * Copyright (c) 1999
 * Fabian Breg
 * Indiana University/Leiden University
 * All Rights Reserved
 *
 * Please refer to the LICENSE file provided in this
 * distribution for full licensing terms and
 * conditions.
 *
 * Please report all bugs/problems to
 * breg@cs.indiana.edu
 *
 ******************************************************
 *
 * Communication.java
 *
 */


package nexusrmi;

import nexus.Nexus;
import nexus.Startpoint;
import nexus.AttachReturn;
import java.net.*;

// This class contains the nexus object for a host together with methods to
// create startpoints, endpoints and links between them.
public class Communication
{
    private static Nexus nexus;

    // Return the nexus for this host.
    public static Nexus getNexus()
    {
	return nexus;
    }

    // Return a startpoint to communicate with the host in the URL.
    // This performs a nexus attach to the nexus server running on that host.
    public static Startpoint getStartpoint(URL url) throws nexusrmi.UnknownHostException
    {
	AttachReturn ar = null;

	try
	{
	    ar = nexus.attach(url);
	}
	catch(Throwable e)
	{
	    e.printStackTrace();
	    return null;
	}
	
	Startpoint sp = null;

	switch(ar.status)
	{
	case 0:
	    sp = ar.sp;
	    break;
	case 1:
	    throw new nexusrmi.UnknownHostException(url.toString());
	default:
	    return null;
	}
	return sp;
    }

    // Create a local endpoint and bind a startpoint to it.
    // The startpoint can be passed to other (remote) objects
    // to communicate with this object.
//     public static Startpoint getStartpoint(HandlerInterface h, Object o)
//     {
// 	Nexus nexus = Communication.getNexus();
// 	EndpointAttr endpointAttr = nexus.endpointattr_init(h);
// 	Endpoint endpoint = nexus.endpoint_init(endpointAttr);
// 	Startpoint sp = nexus.startpoint_bind(endpoint);
// 	endpoint.set_user_object(o);
// 	return sp;
//     }

    public static void end()
    {
	nexus.destroy_current_context(true);
    }
    
    // Initialize Nexus here, so that it can be accessed from everywhere
    static
    {
	String[] Args = {""};
	nexus = new Nexus();
	nexus.activate();
    }
}
