/* 
 * NexusRMI
 *
 * Copyright (c) 1999
 * Fabian Breg
 * Indiana University/Leiden University
 * All Rights Reserved
 *
 * Please refer to the LICENSE file provided in this
 * distribution for full licensing terms and
 * conditions.
 *
 * Please report all bugs/problems to
 * breg@cs.indiana.edu
 *
 ******************************************************
 *
 * HPCxxNexusize.java
 *
 */


package nexusrmi;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import nexus.Startpoint;
import nexusrmi.server.*;

public class HPCxxNexusize extends Serialize
{
    protected void writeByteArray(byte[] ba)
    {
	int len = ba.length;
	try
	{
	    outbuf.put_int(len);
	    outbuf.put_byte(ba, 0, len);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    protected byte[] readByteArray()
    {
	byte[] ba = null;
	try
	{
	    int l = inbuf.get_int();
	    ba = new byte[l];
	    inbuf.get_byte(ba, 0, l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return ba;
    }

        
    protected void writeBooleanArray(boolean[] ba)
    {
	int len = ba.length;
	byte[] tmp = new byte[len];
	for(int i = 0; i < len; i++)
	    tmp[i] = ba[i] ? (byte)1 : (byte)0;

	try
	{
	    outbuf.put_int(len);
	    outbuf.put_byte(tmp, 0, len);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    protected boolean[] readBooleanArray()
    {
	boolean[] ba = null;
	try
	{
	    int l = inbuf.get_int();
	    byte[] tmp = new byte[l];
	    inbuf.get_byte(tmp, 0, l);
	    ba = new boolean[l];
	    for(int i = 0; i < l; i++)
		ba[i] = (tmp[i] == 1);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return ba;
    }


    protected void writeCharArray(char[] ca)
    {
	int len = ca.length;
	try
	{
	    outbuf.put_int(len);
	    outbuf.put_char(ca, 0, len);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    protected char[] readCharArray()
    {
	char[] ca = null;
	try
	{
	    int l = inbuf.get_int();
	    ca = new char[l];
	    inbuf.get_char(ca, 0, l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return ca;
    }


    protected void writeShortArray(short[] sa)
    {
	int len = sa.length;
	try
	{
	    outbuf.put_int(len);
	    outbuf.put_short(sa, 0, len);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    protected short[] readShortArray()
    {
	short[] sa = null;
	try
	{
	    int l = inbuf.get_int();
	    sa = new short[l];
	    inbuf.get_short(sa, 0, l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return sa;
    }


    protected void writeIntArray(int[] ia)
    {
	int len = ia.length;
	try
	{
	    outbuf.put_int(len);
	    outbuf.put_int(ia, 0, len);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    protected int[] readIntArray()
    {
	int[] ia = null;
	try
	{
	    int l = inbuf.get_int();
	    ia = new int[l];
	    inbuf.get_int(ia, 0, l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return ia;
    }


    protected void writeLongArray(long[] la)
    {
	int len = la.length;
	try
	{
	    outbuf.put_int(len);
	    outbuf.put_long(la, 0, len);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    protected long[] readLongArray()
    {
	long[] la = null;
	try
	{
	    int l = inbuf.get_int();
	    la = new long[l];
	    inbuf.get_long(la, 0, l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return la;
    }


    protected void writeFloatArray(float[] fa)
    {
	int len = fa.length;
	try
	{
	    outbuf.put_int(len);
	    outbuf.put_float(fa, 0, len);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    protected float[] readFloatArray()
    {
	float[] fa = null;
	try
	{
	    int l = inbuf.get_int();
	    fa = new float[l];
	    inbuf.get_float(fa, 0, l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return fa;
    }


    protected void writeDoubleArray(double[] da)
    {
	int len = da.length;
	try
	{
	    outbuf.put_int(len);
	    outbuf.put_double(da, 0, len);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    protected double[] readDoubleArray()
    {
	double[] da = null;
	//System.out.println("reading: double array");
	try
	{
	    int l = inbuf.get_int();
	    da = new double[l];
	    inbuf.get_double(da, 0, l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return da;
    }



    public void writebyte(byte b)
    {
	try
	{
	    outbuf.put_byte(b);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    public byte readbyte()
    {
	try
	{
	    return inbuf.get_byte();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return (byte)0;
    }

    
    public void writeboolean(boolean b)
    {
	try
	{
	    outbuf.put_byte((byte)(b ? 1 : 0));
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }
    
    public boolean readboolean()
    {
	try
	{
	    return (inbuf.get_byte() == 1);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return false;
    }


    public void writechar(char c)
    {
	try
	{
	    outbuf.put_char(c);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }
    
    public char readchar()
    {
	try
	{
	    return inbuf.get_char();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return (char)0;
    }


    public void writeshort(short s)
    {
	try
	{
	    outbuf.put_short(s);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    public short readshort()
    {
	try
	{
	    return inbuf.get_short();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return 0;
    }

    
    public void writeint(int i)
    {
	try
	{
	    outbuf.put_int(i);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }
    
    public int readint()
    {
	try
	{
	    return inbuf.get_int();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return 0;
    }


    public void writelong(long l)
    {
	try
	{
	    outbuf.put_long(l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    public long readlong()
    {
	try
	{
	    return inbuf.get_long();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return 0L;
    }

    
    public void writefloat(float f)
    {
	try
	{
	    outbuf.put_float(f);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }
    
    public float readfloat()
    {
	try
	{
	    return inbuf.get_float();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return 0.0F;
    }


    public void writedouble(double d)
    {
	try
	{
	    outbuf.put_double(d);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    public double readdouble()
    {
	try
	{
	    return inbuf.get_double();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return 0.0;
    }

    

    public void writeString(String s)
    {
	writeCharArray(s.toCharArray());
    }

    public String readString()
    {
	return new String(readCharArray());
    }
    
    public void writeObject(Object o)
    {
	try
	{
	    if(o == null)
	    {
		writebyte((byte)0);
	    }
	    else
	    {
		if(o instanceof RemoteStub)
		{
		    writebyte((byte)6);
		    writeFinalObject(java.lang.String.class, o.getClass().getName());
		    writeFinalObject(nexus.Startpoint.class, UnicastRemoteObject.getStartpoint((Remote)o));
		    writeFinalObject(java.lang.String.class, UnicastRemoteObject.getSerClass((Remote)o));
		    writeFinalObject(java.lang.String.class, UnicastRemoteObject.getCodebase((Remote)o));
		}
		else if(o instanceof Remote)
		{
		    writebyte((byte)5);
		    writeFinalObject(java.lang.String.class, o.getClass().getName());
		    writeFinalObject(nexus.Startpoint.class, UnicastRemoteObject.getStartpoint((Remote)o));
		    writeFinalObject(java.lang.String.class, UnicastRemoteObject.getSerClass((Remote)o));
		    writeFinalObject(java.lang.String.class, UnicastRemoteObject.getCodebase((Remote)o));
		}
		else if(o instanceof String)
		{
		    writebyte((byte)31);
		    writeString((String)o);
		}
		else if(o instanceof Class)
		{
		    writebyte((byte)32);
		    writeFinalObject(java.lang.String.class, ((Class)o).getName());
		}
		else if(o instanceof Vector)
		{
		    writebyte((byte)33);
		    writeint(((Vector)o).size());
		    Enumeration e = ((Vector)o).elements();
		    while(e.hasMoreElements())
		    {
			writeObject(e.nextElement());
		    }
		}
		else
		{
		    Class oclass = o.getClass();
		    if(oclass.isArray())
		    {
			Class ctype = oclass.getComponentType();
			
			if(ctype == boolean.class)
			{
			    writebyte((byte)10);
			    writeBooleanArray((boolean[])o);
			}
			else if(ctype == byte.class)
			{
			    writebyte((byte)11);
			    writeByteArray((byte[])o);
			}
			else if(ctype == short.class)
			{
			    writebyte((byte)12);
			    writeShortArray((short[])o);
			}
			else if(ctype == int.class)
			{
			    writebyte((byte)13);
			    writeIntArray((int[])o);
			}
			else if(ctype == long.class)
			{
			    writebyte((byte)14);
			    writeLongArray((long[])o);
			}
			else if(ctype == float.class)
			{
			    writebyte((byte)15);
			    writeFloatArray((float[])o);
			}
			else if(ctype == double.class)
			{
			    writebyte((byte)16);
			    writeDoubleArray((double[])o);
			}
			else if(ctype == char.class)
			{
			    writebyte((byte)17);
			    writeCharArray((char[])o);
			}
			else
			{
			    int length = Array.getLength(o);
			    String cname = ctype.getName();

			    writebyte((byte)18);
			    writeint(length);
			    writeFinalObject(java.lang.String.class, cname);

			    if(!ctype.isArray() && Modifier.isFinal(ctype.getModifiers()))
			    {
				for(int i = 0; i < length; i++)
				    writeFinalObject(ctype, Array.get(o, i));
			    }
			    else
			    {
				for(int i = 0; i < length; i++)
				    writeObject(Array.get(o, i));
			    }
			}
		    }
		    else if(o instanceof SerialObject)
		    {
			writebyte((byte)20);
			writeFinalObject(java.lang.String.class, oclass.getName());
			((SerialObject)o).serializeWrite(this);
		    }
		    else if(o instanceof nexusrmi.Externalizable)
		    {
			writebyte((byte)21);
			writeFinalObject(java.lang.String.class, oclass.getName());
			((nexusrmi.Externalizable)o).writeExternal(this);
		    }
		}
	    }
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }

    public Object readObject(Object caller, Object container)
    {
	byte kind = readbyte();
	String stubsuf = "";
	try
	{
	    switch(kind)
	    {
	    case 0:
		return null;
	    case 5:
		stubsuf = "_Stub";
	    case 6:
		{
		    Remote res = null;

		    String stubName = (String)readFinalObject(java.lang.String.class, null, null);
		    Startpoint stubsp = (Startpoint)readFinalObject(nexus.Startpoint.class, null, null);
		    String serprot = (String)readFinalObject(java.lang.String.class, null, null);
		    String codebase = (String)readFinalObject(java.lang.String.class, null, null);
		    
		    res = (Remote)UnicastRemoteObject.createStub(stubName + stubsuf, stubsp, serprot, codebase);
		    return res;
		}
	    case 20:
		{
		    String cname = (String)readFinalObject(java.lang.String.class, null, null);
		    Class cclass = RMIClassLoader.loadClass(codebase, cname);
		    //		    System.out.println("readObject(): Creating instance of: " + cclass);
		    //		    System.out.println("readObject(): container = " + container);
		    Object res = null;
		    Class[] fparams = null;
		    Object[] aparams = null;
		    if(container != null)
		    {
			String oname = cname.substring(0, cname.lastIndexOf('$'));
			Class oclass = RMIClassLoader.loadClass(codebase, oname);
			fparams = new Class[] {oclass, nexusrmi.Serialize.class};
			aparams = new Object[] {container, this};
		    }
		    else
		    {
			fparams = new Class[] {nexusrmi.Serialize.class};
			aparams = new Object[] {this};
		    }
		    Constructor m = cclass.getConstructor(fparams);
		    res = m.newInstance(aparams);
		    return ((SerialObject)res).serializeRead(this);
		}
	    case 21:
		{
		    String cname = (String)readFinalObject(java.lang.String.class, null, null);
		    Class oclass = RMIClassLoader.loadClass(codebase, cname);
		    Object res = oclass.newInstance();
		    ((nexusrmi.Externalizable)res).readExternal(this);
		    return res;
		}
	    case 10:
		{
		    boolean[] res = readBooleanArray();
		    return res;
		}
	    case 11:
		{
		    byte[] res = readByteArray();
		    return res;
		}
	    case 12:
		{
		    short[] res = readShortArray();
		    return res;
		}
	    case 13:
		{
		    int[] res = readIntArray();
		    return res;
		}
	    case 14:
		{
		    long[] res = readLongArray();
		    return res;
		}
	    case 15:
		{
		    float[] res = readFloatArray();
		    return res;
		}
	    case 16:
		{
		    double[] res = readDoubleArray();
		    return res;
		}
	    case 17:
		{
		    char[] res = readCharArray();
		    return res;
		}
	    case 18:
		{
		    Object res = null;
		    int length = readint();
		    String cname = (String)readFinalObject(java.lang.String.class, null, null);

		    Class ctype = RMIClassLoader.loadClass(codebase, cname);
		    res = Array.newInstance(ctype, length);

		    if(!ctype.isArray() && Modifier.isFinal(ctype.getModifiers()))
		    {
			for(int i = 0; i < length; i++)
			    Array.set(res, i, readFinalObject(ctype, caller, null));
		    }
		    else
		    {
			for(int i = 0; i < length; i++)
			    Array.set(res, i, readObject(caller, null));
		    }
		    return res;
		}
	    case 31:
		{
		    String res = readString();
		    return res;
		}
	    case 32:
		{
		    String cname = (String)readFinalObject(java.lang.String.class, null, null);
		    Class res = RMIClassLoader.loadClass(codebase, cname);
		    return res;
		}
	    case 33:
		{
		  //		    System.out.println("readObject(): reading Vector...");
		    Vector res = new Vector();
		    int i, n = readint();
		    for(i = 0; i < n; i++)
			res.addElement(readObject(null, null));
		    //		    System.out.println("Done.");
		    return res;
		}
	    default:
		throw new SerializationException("kind = " + kind);
	    }
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return null;
    }
    
     public void writeFinalObject(Class type, Object o)
     {
	 int oindex = 0;
	 Class oclass = null;
	 
	 try
	 {
	     if(o == null)
		 writebyte((byte)0);
	     else
	     {
		 writebyte((byte)2);
		 if(type == java.lang.String.class)
		 {
		     writeCharArray(((String)o).toCharArray());
		 }
		 else if(type == java.lang.Class.class)
		 {
		     writeString(((Class)o).getName());
		 }
		 else if(type == nexus.Startpoint.class)
		 {
		     outbuf.put_startpoint((Startpoint)o);
		 }
		 else
		 {
		     if(o instanceof nexusrmi.Externalizable)
			 ((nexusrmi.Externalizable)o).writeExternal(this);
		     else
		     {
			 ((SerialObject)o).serializeWrite(this);
		     }
		 }
	     }
	 }
	 catch(Exception e)
	 {
	     e.printStackTrace();
	     //System.exit(1);
	 }
     }

    public Object readFinalObject(Class type, Object caller, Object container)
    {
	byte kind = readbyte();
	Class oclass = null;
	
 	try
 	{
	    switch(kind)
	    {
	    case 0:
		return null;
	    case 2:
		if(type == java.lang.String.class)
		{
		    char[] ca = readCharArray();
		    String res = new String(ca);
		    return res;
		}
		else if(type == java.lang.Class.class)
		{
		    Class res = Class.forName(readString());
		    return res;
		}
		else if(type == nexus.Startpoint.class)
		{
		    Startpoint res = null;
		    try
		    {
			res = inbuf.get_startpoint();
		    }
		    catch(Exception e)
		    {
			e.printStackTrace();
		    }
		    return res;
		}			    
		else
		{
		    Object res = null;
		    Class[] fparams = null;
		    Object[] aparams = null;
		    int index;

		    if(container != null)
		    {
			String cname = type.getName();
			String oname = cname.substring(0, cname.lastIndexOf('$'));
			fparams = new Class[] {nexusrmi.Serialize.class};
			aparams = new Object[] {this, container};
		    }
		    else
		    {
			fparams = new Class[] {nexusrmi.Serialize.class};
			aparams = new Object[] {this};
		    }
		    Constructor m = type.getConstructor(fparams);
		    res = m.newInstance(aparams);
		    if(res instanceof nexusrmi.Externalizable)
		    {
			((nexusrmi.Externalizable)res).readExternal(this);
			return res;
		    }
		    else
		    {
			return ((SerialObject)res).serializeRead(this);
		    }
		}
	    }
 	}
 	catch(Exception e)
 	{
 	    e.printStackTrace();
 	}
 	return null;
    }
}
