/* 
 * NexusRMI
 *
 * Copyright (c) 1999
 * Fabian Breg
 * Indiana University/Leiden University
 * All Rights Reserved
 *
 * Please refer to the LICENSE file provided in this
 * distribution for full licensing terms and
 * conditions.
 *
 * Please report all bugs/problems to
 * breg@cs.indiana.edu
 *
 ******************************************************
 *
 * Javaize.java
 *
 */


package nexusrmi;

import java.io.*;

public class Javaize extends Serialize
{
    private ByteArrayOutputStream baos;
    private ObjectOutputStream oos;
    private ByteArrayInputStream bais;
    private ObjectInputStream ois;
    
    public Javaize()
    {
	super();
	baos = null;
	oos = null;
	bais = null;
	ois = null;
    }
    
    public void initWrite()
    {
	super.initWrite();
	baos = null;
	oos = null;
	bais = null;
	ois = null;
    }
    
    public void startWriteData()
    {
	try
	{
	    baos = new ByteArrayOutputStream();
	    oos = new ObjectOutputStream(baos);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    
    public void initRead(Object buffer, String codebase)
    {
	super.initRead(buffer, codebase);
	bais = null;
	ois = null;
	baos = null;
	oos = null;
    }

    public void startReadData()
    {
	try
	{
	    int datalen = inbuf.get_int();
	    byte[] data = new byte[datalen];
	    inbuf.get_byte(data, 0, datalen);
	    bais = new ByteArrayInputStream(data);
	    ois = new ObjectInputStream(bais);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }

    public Object getBuffer()
    {
	try
	{
	    oos.close();
	    byte[] data = baos.toByteArray();
	    outbuf.put_int(data.length);
	    outbuf.put_byte(data, 0, data.length);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return outbuf;
    }
    
    public void writeboolean(boolean b)
    {
	try
	{
	    oos.writeBoolean(b);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public boolean readboolean()
    {
	boolean b = false;
	try
	{
	    b = ois.readBoolean();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return b;
    }

    public void writebyte(byte b)
    {
	try
	{
	    oos.writeByte(b);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public byte readbyte()
    {
	byte b = (byte)0;
	try
	{
	    b = ois.readByte();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return b;
    }
    	   
    public void writechar(char c)
    {
	try
	{
	    oos.writeChar(c);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public char readchar()
    {
	char c = '\0';
	try
	{
	    c = ois.readChar();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return c;
    }
    	   
    public void writeshort(short s)
    {
	try
	{
	    oos.writeShort(s);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public short readshort()
    {
	short s = 0;
	try
	{
	    s = ois.readShort();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return s;
    }
    	   
    public void writeint(int i)
    {
	try
	{
	    oos.writeInt(i);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public int readint()
    {
	int i = 0;
	try
	{
	    i = ois.readInt();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return i;
    }
    	   
    public void writelong(long l)
    {
	try
	{
	    oos.writeLong(l);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public long readlong()
    {
	long l = 0;
	try
	{
	    l = ois.readLong();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return l;
    }
    	   
    public void writefloat(float f)
    {
	try
	{
	    oos.writeFloat(f);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public float readfloat()
    {
	float f = 0.0f;
	try
	{
	    f = ois.readFloat();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return f;
    }
    	   
    public void writedouble(double d)
    {
	try
	{
	    oos.writeDouble(d);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public double readdouble()
    {
	double d = 0.0;
	try
	{
	    d = ois.readDouble();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return d;
    }
    	   
    public void writeObject(Object o)
    {
	try
	{
	    oos.writeObject(o);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public Object readObject(Object caller, Object container)
    {
	Object o = null;
	try
	{
	    o = ois.readObject();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return o;
    }
	   
    public void writeFinalObject(Class type, Object o)
    {
	try
	{
	    oos.writeObject(o);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
    public Object readFinalObject(Class type, Object caller, Object container)
    {
	Object o = null;
	try
	{
	    o = ois.readObject();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
	return o;
    }
}
