/* 
 * NexusRMI
 *
 * Copyright (c) 1999
 * Fabian Breg
 * Indiana University/Leiden University
 * All Rights Reserved
 *
 * Please refer to the LICENSE file provided in this
 * distribution for full licensing terms and
 * conditions.
 *
 * Please report all bugs/problems to
 * breg@cs.indiana.edu
 *
 ******************************************************
 *
 * Serialize.java
 *
 */


package nexusrmi;

import java.net.*;
import nexus.Nexus;
import nexus.GetBuffer;
import nexus.PutBuffer;
import nexus.Startpoint;

public abstract class Serialize
{
    protected Nexus nxs;
    protected PutBuffer outbuf;
    protected GetBuffer inbuf;
    protected URL codebase;

    public Serialize()
    {
	nxs = Communication.getNexus();
	this.codebase = null;
	outbuf = null;
	inbuf = null;
    }
    
    public void initWrite()
    {
	codebase = null;
	inbuf = null;
	outbuf = nxs.buffer_init(0, 0);
    }
    
    public void initRead(Object buffer, String codebase)
    {
	try
	{
	    this.codebase = new URL(codebase);
	}
	catch(MalformedURLException e)
	{
	    this.codebase = null;
	}
	inbuf = (GetBuffer)buffer;
	outbuf = null;
    }

    public void startWriteHeader() { }
    public void startReadHeader() { }
    public void startWriteData() { }
    public void startReadData() { }
    
    public final void writeHPCxxHeaderStartpoint(Startpoint sp)
    {
	try
	{
	    sp.put(outbuf);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    public final void writeHPCxxHeaderOffset(int off)
    {
	try
	{
	    outbuf.put_int(off);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    public final void writeHPCxxHeaderHandlerid(int id)
    {
	try
	{
	    outbuf.put_int(id);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
    }

    public final Startpoint readHPCxxHeaderStartpoint()
    {
	Startpoint sp = null;
	try
	{
	    sp = new Startpoint(nxs, inbuf);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return sp;
    }

    public final int readHPCxxHeaderOffset()
    {
	int off = 0;
	try
	{
	    off = inbuf.get_int();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return off;
    }

    public final int readHPCxxHeaderHandlerid()
    {
	int id = 0;
	try
	{
	    id = inbuf.get_int();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return id;
    }
    
    public Object getBuffer()
    {
//	System.out.println("\n Nexusrmi.Serialize: getBuffer : " + outbuf.getSize());
	return outbuf;
    }
    
    public void writeboolean(boolean b)
    { }
    public boolean readboolean()
    { return false; }
    	   
    public void writebyte(byte b)
    { }
    public byte readbyte()
    { return (byte)0; }
    	   
    public void writechar(char c)
    { }
    public char readchar()
    { return '\0'; }
    	   
    public void writeshort(short s)
    { }
    public short readshort()
    { return 0; }
    	   
    public void writeint(int i)
    { }
    public int readint()
    { return 0; }
    	   
    public void writelong(long l)
    { }
    public long readlong()
    { return 0; }
    	   
    public void writefloat(float f)
    { }
    public float readfloat()
    { return 0.0f; }
    	   
    public void writedouble(double d)
    { }
    public double readdouble()
    { return 0.0d; }
    	   
    public void writeObject(Object o)
    { }
    public Object readObject(Object caller, Object container)
    { return null; }
	   
    public void writeFinalObject(Class type, Object o)
    { }
    public Object readFinalObject(Class type, Object caller, Object container)
    { return null; }
}
