/* 
 * NexusRMI
 *
 * Copyright (c) 1999
 * Fabian Breg
 * Indiana University/Leiden University
 * All Rights Reserved
 *
 * Please refer to the LICENSE file provided in this
 * distribution for full licensing terms and
 * conditions.
 *
 * Please report all bugs/problems to
 * breg@cs.indiana.edu
 *
 ******************************************************
 *
 * LogStream.java
 *
 */


package nexusrmi.server;

import java.io.*;
import java.util.*;

public class LogStream extends PrintStream
{
    public static final int SILENT  = 0;
    public static final int BRIEF   = 10;
    public static final int VERBOSE = 20;

    private static Hashtable logHash = new Hashtable();
    private static PrintStream defaultStream = null;
    private PrintWriter logWriter = null;
    private ByteArrayOutputStream baos = null;
    private String logName = null;

    private LogStream(OutputStream out, String name)
    {
	super(out);
	logName = name;
	logWriter = new PrintWriter(out);
	baos = new ByteArrayOutputStream();
    }
    
    public static LogStream log(String name)
    {
	LogStream ls = (LogStream)logHash.get(name);
	if(ls == null)
	{
	    ls = new LogStream(System.out, name);
	    logHash.put(name, ls);
	}
	return ls;
    }

    public static synchronized PrintStream getDefaultStream()
    {
	return defaultStream;
    }

    public static synchronized void setDefaultStream(PrintStream newDefault)
    {
	defaultStream = newDefault;
    }

    public synchronized OutputStream getOutputStream()
    {
	return out;
    }
    
    public synchronized void setOutputStream(OutputStream out)
    {
	this.out = out;
	logWriter = new PrintWriter(out);
    }
    
    public void write(int b)
    {
	baos.write(b);
	if(b == '\n')
	{
	    String buf = ((new Date()) + ":" +
			  Thread.currentThread().getName() + ":" +
			  logName + ":" +
			  new String(baos.toByteArray()));
	    super.write(buf.getBytes(), 0, buf.length());
	    baos = new ByteArrayOutputStream();
	}
    }

    public void write(byte b[], int off, int len)
    {
	for(int i = off; i < len; i++)
	    write(b[i]);
    }

    public String toString()
    {
	return logName;
    }

    public static int parseLevel(String s)
    {
	return 0;
    }
}
