/* 
 * NexusRMI
 *
 * Copyright (c) 1999
 * Fabian Breg
 * Indiana University/Leiden University
 * All Rights Reserved
 *
 * Please refer to the LICENSE file provided in this
 * distribution for full licensing terms and
 * conditions.
 *
 * Please report all bugs/problems to
 * breg@cs.indiana.edu
 *
 ******************************************************
 *
 * UnicastRemoteObject.java (1.1 version)
 *
 */


package nexusrmi.server;

import java.net.*;
import java.lang.reflect.*;
import nexus.Nexus;
import nexus.Startpoint;
import nexus.Endpoint;
import nexus.EndpointAttr;
import nexus.HandlerInterface;
import nexusrmi.*;
import java.util.Vector;
import java.util.Hashtable;

// This class is typically the direct superclass of remote objects.
// Most of its functionality is in the static methods however,
// which can be used by any object.
public class UnicastRemoteObject extends RemoteServer
{
    // Store stub information of exported remote objects.
    // This information is sent whenever a remote reference
    // is transferred.
    private static WeakHashMap objTab = new WeakHashMap();
    private static WeakHashMap stubTab = new WeakHashMap();

    private static Vector spTab = new Vector();
    private static Vector rsTab = new Vector();


    protected UnicastRemoteObject() throws RemoteException
    {
	exportObject(this, new Nexusize());
    }
    protected UnicastRemoteObject(Serialize ser) throws RemoteException
    {
	exportObject(this, ser);
    }

    // Obviously not implemented yet.
    public Object clone() throws CloneNotSupportedException
    {
	return null;
    }

    // Export the given remote object. That is create stub and skeleton and
    // return the stub, although I have not seen this return value actually
    // used by anyone...
    public static RemoteStub exportObject(Remote obj) throws RemoteException
    {
	return exportObject(obj, new Nexusize());
    }

    public static RemoteStub exportObject(Remote obj, Serialize ser) throws RemoteException
    {
	new IdleThread().start();
	Startpoint osp = createSkeleton(obj, ser.getClass().getName());
	return (RemoteStub)createStub(obj, osp, ser.getClass().getName(), System.getProperty("java.rmi.server.codebase"));
    }

    private static synchronized Object getStubForStartpoint(Startpoint osp)
    {
	int nsp = spTab.size();
	for(int i = 0; i < nsp; i++)
	    if(((Startpoint)spTab.elementAt(i)).equals(osp))
	    {
		return rsTab.elementAt(i);
	    }
	return null;
    }
    
    // Create a stub instance for the given remote object with the given startpoint and serialization object
    public static synchronized Object createStub(Remote obj, Startpoint osp, String ser, String codebase)
    {
	Object stub = null;
	String oname = obj.getClass().getName();
	String stubName = oname + "_Stub";
// 	System.out.println("\n UnicastRemoteObject::createStub stubName " + stubName);

	stub = getStubForStartpoint(osp);
// 	System.out.println("\n UnicastRemoteObject::createStub stub is " + stub);
// 	System.out.println("\n UnicastRemoteObject::createStub sp is " + osp);
// 	System.out.println("\n UnicastRemoteObject::createStub ser is " + ser);
// 	System.out.println("\n UnicastRemoteObject::createObj codebase is " + codebase);
	if(stub != null)
	    return stub;
	
	try
	{
	    // Create the stub using the reflection classes.
	    Nexus nexus = Communication.getNexus();
	    Class[] type = new Class[4];
	    Object[] param = new Object[4];
	    type[0] = Class.forName("nexus.Nexus");
	    type[1] = Class.forName("nexus.Startpoint");
	    type[2] = Class.forName("java.lang.String");
	    type[3] = Class.forName("java.lang.String");
	    param[0] = nexus;
	    param[1] = osp;
	    param[2] = ser;
	    param[3] = codebase;
	    Class StubClass = Class.forName(stubName);
	    Constructor StubCons = StubClass.getConstructor(type);
	    stub = StubCons.newInstance(param);

	    spTab.addElement(osp);
	    rsTab.addElement(stub);
	    
	    if(!objTab.containsKey(obj))
	    {
		StubEntry se = new StubEntry(osp, ser, codebase);
		objTab.put(obj, se);
	    }
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return stub;
    }

    public static synchronized Object createStub(String stubName, Startpoint osp, String ser, String codebase)
    {
	Object stub = null;

// 	System.out.println("\n UnicastRemoteObject::sp is " + osp);
// 	System.out.println("\n $$$$$$$$$$$$$$$$$$UnicastRemoteObject::createStub stubName is " + stubName);
	stub = getStubForStartpoint(osp);
// 	System.out.println("\n UnicastRemoteObject::stub is " + stub);
// 	System.out.println("\n UnicastRemoteObject::sp is " + osp);
// 	System.out.println("\n UnicastRemoteObject::ser is " + ser);
// 	System.out.println("\n UnicastRemoteObject::codebase is " + codebase);
	
	if(stub != null) {
//  	  System.out.println("\n $$$$$$$$$$$$$$$$$$$$$$$$ UnicastRemoteObject:: stub is not null ! " + stub);
	    return stub;
	}
	
	try
	{
	    // Create the stub using the reflection classes.
	    Nexus nexus = Communication.getNexus();
	    Class[] type = new Class[4];
	    Object[] param = new Object[4];
	    type[0] = Class.forName("nexus.Nexus");
	    type[1] = Class.forName("nexus.Startpoint");
	    type[2] = Class.forName("java.lang.String");
	    type[3] = Class.forName("java.lang.String");
	    param[0] = nexus;
	    param[1] = osp;
	    param[2] = ser;
	    param[3] = codebase;
// 	    Class StubClass = Class.forName(stubName);
// 	    System.out.println("\nUnicastRemoteObject::attemping try");
	    URL cburl = null;
	    try
	    {
		cburl = new URL(codebase);
	    }
	    catch(Exception e)
	    {
		cburl = null;
	    }
//  	    System.out.println("\n$$$$$$$$$$$$$$$$$$$$UnicastRemoteObject::loading stub class  ");
	    Class StubClass = RMIClassLoader.loadClass(cburl, stubName);
//  	    System.out.println("\n$$$$$$$$$$$$$$$$$$$$$UnicastRemoteObject::StubClass is " + StubClass);
	    Constructor StubCons = StubClass.getConstructor(type);
	    stub = StubCons.newInstance(param);
//  	    System.out.println("\n$$$$$$$$$$$$$$$$$$$$$UnicastRemoteObject::stub instance is " + stub);

	    spTab.addElement(osp);
	    rsTab.addElement(stub);
	    
	    StubEntry se = new StubEntry(osp, ser, codebase);
	    stubTab.put(stub, se);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
// 	System.out.println("\nUnicastRemoteObject::returning stub  " + stub);
	return stub;
    }

    // Return the startpoint for the given remote object.
    public static synchronized Startpoint getStartpoint(Remote obj)
    {
	StubEntry se = (StubEntry)objTab.get(obj);
	if(se == null)
	    se = (StubEntry)stubTab.get(obj);
	if(se == null)
	    return null;
	else
	    return se.getStartpoint();
    }

    // Return the serialization object for the given remote object.
    public static synchronized String getSerClass(Remote obj)
    {
	StubEntry se = (StubEntry)objTab.get(obj);
	if(se == null)
	    se = (StubEntry)stubTab.get(obj);
	if(se == null)
	    return null;
	else
	    return se.getSerialization();
    }

    public static synchronized String getCodebase(Remote obj)
    {
	StubEntry se = (StubEntry)objTab.get(obj);
	if(se == null)
	    se = (StubEntry)stubTab.get(obj);
	if(se == null)
	    return null;
	else
	    return se.getCodebase();
    }

    // Create the skeleton object and return a startpoint for it.
    static Startpoint createSkeleton(Remote o, String ser)
    {
	Startpoint sp = null;
	
	try
	{
	    // Create skeleton using the reflection class.
	    Nexus nexus = Communication.getNexus();
	    Class[] type = new Class[2];
	    Object[] param = new Object[2];
	    type[0] = Class.forName("nexus.Nexus");
	    param[0] = nexus;
	    type[1] = Class.forName("java.lang.String");
	    param[1] = ser;
	    String skelName = o.getClass().toString().substring(6) + "_Skel";
	    Class SkelClass = Class.forName(skelName);
	    Constructor SkelCons = SkelClass.getConstructor(type);
	    Object Skel = SkelCons.newInstance(param);
	    // Get startpoint for this skeleton.
	    EndpointAttr endpointAttr = nexus.endpointattr_init();
	    endpointAttr.set_handler_object((HandlerInterface)Skel);
	    Endpoint endpoint = nexus.endpoint_init(endpointAttr);
	    sp = nexus.startpoint_bind(endpoint);
	    endpoint.set_user_object(o);
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	    //System.exit(1);
	}
	return sp;
    }

    public static int nrStubs()
    {
	return stubTab.size();
    }
}

class IdleThread extends Thread
{
    public synchronized void run()
    {
	try
	{
	    wait();
	}
	catch(Exception e)
	{
	    e.printStackTrace();
	}
    }
}

// The next two classes are for 1.1 use only
class WeakHashMap extends Hashtable
{
}
class WeakReference
{
    Object referent;
    WeakReference(Object ref)
    {
	referent = ref;
    }
    Object get()
    {
	return referent;
    }
}

class StubEntry
{
    private Startpoint sp;
    private String ser;
    private String codebase;

    public StubEntry(Startpoint sp, String ser, String codebase)
    {
	this.sp = sp;
	this.ser = ser;
	this.codebase = codebase;
    }

    public Startpoint getStartpoint()
    {
	return sp;
    }

    public String getSerialization()
    {
	return ser;
    }

    public String getCodebase()
    {
	return codebase;
    }
}

