

/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** ast.c : AST routines
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

/* This file has been modified from the original to support
 * Java 1.1 language.
 *
 *     Fabian Breg.
 */

/* ********************************************************
   *** INCLUDE FILES and DEFINITIONS                    ***
   ********************************************************  */

#include <math.h>

#include "global.h"
#include "y.tab.h"

/* ********************************************************
   *** EXTERNAL VARIABLES                               ***
   ******************************************************** */

/* PUBLIC 
   ****** */

stmt_ptr cur_class      = NULL;
stmt_ptr cur_method     = NULL;
stmt_ptr cur_decl       = NULL;
expr_ptr cur_inv        = NULL;

char    *comment_buffer = NULL;

stmt_ptr comp_unit      = NULL;

stmt_ptr *header_stack  = NULL;

/* PRIVATE
   ******* */

static int header_stack_depth = 0;
static int max_header_stack_depth = 0;

static type_ptr type_glink = NULL;
static expr_ptr expr_glink = NULL;
static stmt_ptr stmt_glink = NULL;

static dist_symb_ptr dist_sym_tab = NULL;
static dist_symb_ptr last_dist_sym = NULL;

static short MODFILTER[] = { /* JAVA 1.1 */
 (PUB_B|PRO_B|PRI_B|STA_B|FIN_B|TRA_B|VOL_B),
  0, (PUB_B|ABS_B|FIN_B|PRO_B|PRI_B|STA_B), (PUB_B|ABS_B|PRI_B|PRO_B|STA_B), (PUB_B|PRO_B|PRI_B), (STA_B),
 (PUB_B|PRO_B|PRI_B|STA_B|ABS_B|FIN_B|NAT_B|SYN_B), 0
} ;

static expr_ptr javar_excep;
static long     javar_threads, javar_sync, javar_multi, javar_cut;
static short    javar_sched,   javar_bw,   javar_copy;
static int      javar_hosts,   javar_gsize;

static int      iterat_cnt = 0;  /* for/do/while */
static int      switch_cnt = 0;

/* ********************************************************
   *** PRIVATE FUNCTIONS                                ***
   ******************************************************** */

/* New Type Node
   ************* */

static type_ptr new_type(short kind) {
  type_ptr t        = (type_ptr) make_mem(sizeof(struct type_node));
  t -> next         = type_glink;
  t -> kind         = kind;
  return type_glink = t;
}

/* New Expression Node
   ******************* */

static expr_ptr new_expr(short kind) {
  expr_ptr e        = (expr_ptr) make_mem(sizeof(struct expr_node));
  e -> next         = expr_glink;
  e -> type         = NULL;
  e -> lineno       = lineno;
  e -> kind         = kind;
  return expr_glink = e;
}

/* New Stmt Node
   ************* */

static stmt_ptr new_stmt(short kind) {
  stmt_ptr s        = (stmt_ptr) make_mem(sizeof(struct stmt_node));
  s -> next         = stmt_glink;
  s -> lineno       = lineno;
  s -> comment      = -1;
  s -> kind         = kind;
  return stmt_glink = s;
}

/* Check Modifiers
   *************** */

/* JAVA 1.1 */

static void check_mod(short kind, short mod, int inout) {
  static short acc_filter = (PUB_B | PRO_B | PRI_B);
  static short abs_filter = (PRI_B | STA_B | FIN_B | NAT_B | SYN_B);

  short acctest = (mod & acc_filter);

  if ((mod & MODFILTER[kind-DECL_S]) != mod)
    javar_out(0, "invalid modifier");
  else if ( (kind == CLASS_S) && (inout == 0) && (mod & PRI_B) ) /* outer classes */
    javar_out(0, "outer class cannot be private");
  else if ( (kind == CLASS_S) && (inout == 0) && (mod & PRO_B) ) /* outer classes */
    javar_out(0, "outer class cannot be protected");
  else if ( (kind == INTERF_S) && (inout == 0) && (mod & PRI_B) ) /* outer interfaces */
    javar_out(0, "outer interface cannot be private");
  else if ( (kind == INTERF_S) && (inout == 0) && (mod & PRO_B) ) /* outer interfaces */
    javar_out(0, "outer interface cannot be protected");
  else if ( (acctest) &&
            (acctest != PUB_B) && (acctest != PRO_B) && (acctest != PRI_B) )
    javar_out(0, "invalid combination of public/protected/private");
  else if ( (mod & FIN_B) && (mod & VOL_B) )
    javar_out(0, "invalid combination with of final and volatile modifier");
}

/* Determine Value in Annotation
   ***************************** */

static long get_val(long cur_val, expr_ptr e, short z) {
  long res = cur_val; 

  if ( (e) && (e -> kind == LITERAL_) && (IS_INTEGRAL1(e -> type)) &&
       (e -> u.lit.set) ) {

    long l = e -> u.lit.val.l;

    if ((l > 0) || ((l == 0) && (z)))
      res = l;
    else
      javar_out(0, "invalid %s value in annotation", 
		     (l) ? "negative" : "zero");
  }
  else
    javar_out(0, "appropriate value missing in annotation");

  return res;
}

/* Count Dimensions
   **************** */

static int scan_left(expr_ptr e) {
  if ((e) && (e -> kind == '['))
    return (1 + scan_left(e -> u.op.lhs));
  else
    return 0;
}

/* Determining member/anonymous classes
   ************************************ */

static int equal_id(expr_ptr e1, expr_ptr e2)
{
    assert((e1->kind == ID_) && (e2->kind == ID_));
    return !strcmp(e1->u.name.id->symbol, e2->u.name.id->symbol);
}

static int nlist_lookup_and_add(stmt_ptr cc, expr_ptr id)
{
    int i = 0;
    
    assert((cc->kind == CLASS_S) || (cc->kind == INTERF_S));

    while(i < cc->u.header.nllength)
    {
	if(equal_id(id, cc->u.header.nlist[i].cn))
	    return (cc->u.header.nlist[i].n)++;

	i++;
    }

    /* name not present in class, add it now and initialize counter to 0 */
    if(cc->u.header.nllength >= cc->u.header.maxnllength)
    {
	cc->u.header.maxnllength += 5;
	if(cc->u.header.maxnllength == 5)
	    cc->u.header.nlist =
		(struct _nlist *)make_mem(cc->u.header.maxnllength * sizeof(struct _nlist));
	else
	    cc->u.header.nlist =
		(struct _nlist *)more_mem(cc->u.header.nlist,
						       cc->u.header.maxnllength * sizeof(struct _nlist));
    }
/*     printf("adding: "); */
/*     dump_expr(id, stdout); */
/*     printf(" to "); */
/*     dump_expr(cc->u.header.name, stdout); */
/*     printf(" at %d\n", cc->u.header.nllength); */
    
    cc->u.header.nlist[cc->u.header.nllength].cn = id;
    cc->u.header.nlist[cc->u.header.nllength].n = 2;
    cc->u.header.nllength++;
    return 1;
}

static expr_ptr determine_fqcn(stmt_ptr s)
{
    int unique_id, uid_len;
    char *uid_str;
    stmt_ptr container;
    expr_ptr cont_fqn = NULL;
    expr_ptr res;

    assert((s->kind == INTERF_S) || (s->kind == CLASS_S));
    container = s->u.header.container;

    if(!container)
	return s->u.header.name;
    
    if(((container->kind == CLASS_S) || (container->kind == INTERF_S)) &&
       !(container->u.header.mod & PRI_B))
	return make_op('$', container->u.header.fqcn, s->u.header.name);

    while(((container->kind != CLASS_S) && (container->kind != INTERF_S)) ||
	  (container->u.header.mod & PRI_B))
	container = container->u.header.container;

    unique_id = nlist_lookup_and_add(container, s->u.header.name);
    uid_len = ((int)log10((double)unique_id)) + 1;
    uid_str = (char *)make_mem((uid_len + 1) * sizeof(char));
    sprintf(uid_str, "%d", unique_id);
    res = make_op('$',
		  make_op('$',
			  container->u.header.fqcn,
			  make_name(NULL, insert_table(uid_str), 0)),
		  s->u.header.name);
/*     free(uid_str); */
    return res;
}
    
/* ********************************************************
   *** PUBLIC FUNCTIONS                                 ***
   ******************************************************** */

/* Header stack
   ************ */

void push_header(stmt_ptr header)
{
/*     printf("pushing: %s...\n", header->u.header.name ? header->u.header.name->u.name.id->symbol : "null"); */
    if(header_stack_depth >= max_header_stack_depth)
    {
	max_header_stack_depth += 5;
	if(max_header_stack_depth == 5)
	    header_stack = (stmt_ptr *)make_mem(max_header_stack_depth * sizeof(stmt_ptr));
	else
	    header_stack = (stmt_ptr *)more_mem(header_stack, max_header_stack_depth * sizeof(stmt_ptr));
    }
    header_stack[header_stack_depth] = header;
    header_stack_depth++;
}

stmt_ptr pop_header(void)
{
    stmt_ptr res;
    
    if(header_stack_depth == 0)
	javar_out(-1, "Header stack underflow");
    header_stack_depth--;
    res = header_stack[header_stack_depth];
/*     printf("popped %s...\n", res->u.header.name ? res->u.header.name->u.name.id->symbol : "null"); */
    if(header_stack_depth == 0)
    {
	max_header_stack_depth = 0;
	free(header_stack);
    }
    return res;
}

stmt_ptr top_header(void)
{
    if(header_stack_depth == 0)
	return NULL;
    return header_stack[header_stack_depth - 1];
}

/* *************************
   *** Comment Handling ****
   *********************************************************
   *** add_comment() : add string to comment environment ***
   *** get_comment() : retreive comment from environment ***
   ********************************************************* */

static int comment_len  = 0;
static int comment_pos  = 0;
static int comment_base = 0;

void add_comment(char *s) {
  int i, l;

  if (! comments_flag)
    return;

  l = strlen(s);

  if ((comment_pos + l) >= comment_len) {
    comment_len += (l + 1000);
    if (comment_buffer) 
      comment_buffer = (char *) more_mem(comment_buffer, comment_len * sizeof(char));
    else 
      comment_buffer = (char *) make_mem(comment_len * sizeof(char)); 
  }
  for (i = 0; i < l; i++)
    comment_buffer[comment_pos++] = s[i];
  comment_buffer[comment_pos] = '\0';
}

int get_comment(void) {
  if (comment_pos > comment_base) {
    int off      = comment_base;
    comment_base = ++comment_pos;
    return off;
  }
  return -1;
}

/* *************************
   *** Type Construction ***
   ********************************************************
   *** Primitive Types:          make_primitive_type()  ***
   *** Class or Interface Types: make_c_i_type()        ***
   *** Array Types:              make_array_type()      ***
   *** Null Type:                use NULL               ***
   ******************************************************** */

type_ptr make_primitive_type(short kind) {
  return new_type(kind);
}

type_ptr make_c_i_type(expr_ptr name) {
  type_ptr tp  = new_type(C_OR_I_T);
  tp -> u.name = name;

  if ( (! name) || (name -> kind != ID_) ||
      ( (name -> u.name.info != AMBI_N)  && 
	(name -> u.name.info != TYPE_N)  &&
	(name -> u.name.info != EXPR_N) ) )
    javar_out(0, "invalid type expression");
  else 
    name -> u.name.info = TYPE_N;

  if (! strcmp("String", id_table(name -> u.name.entry)))
    tp -> u.name = MK_STRING;

  return tp;
} 

type_ptr make_array_type(type_ptr aref) {
  type_ptr tp  = new_type(ARRAY_T);
  tp -> u.aref = aref;
  return tp;
} 

type_ptr make_atype(type_ptr aref, expr_ptr dims)
{
    type_ptr res = aref;
    int i = scan_left(dims);
    while(i-- > 0)
	res = make_array_type(res);
    return res;
}

/* *******************************
   *** Expression Construction ***
   *********************************************************
   *** Literals:          make_literal()                 ***
   *** Names:             make_name()                    *** 
   *** Decl. Stmt. EXPR.: make_declexpr()                ***
   *** TE Operators:      make_te()       make_cast()    ***
   *** Operators:         make_thissup()                 ***
   ***                    make_op()                      ***
   ********************************************************* */

/* NULL           Literals (TYPE : null type)          (set type -2)
   String         Literals (TYPE : java.lang.String)   (set type -1)
   Character      Literals (TYPE : CHAR)
   Integral       Literals (TYPE : BOOLEAN/INT/LONG) 
   Floating Point Literals (TYPE : FLOAT/DOUBLE) 
   *******************************************************************
   *** all literals are represented symbolically                   ***
   ***                                                             ***
   *** e -> u.lit.set is set in case an internal representation    ***
   *** of the literal can be constructed using a C data type       ***
   ***                                                             ***
   *** NOTE HOWEVER THAT FOR F.P. LITERALS, EVEN IF e -> u.lit.set ***
   *** IS SET, PRECISION MAY BE LOST                               ***
   ******************************************************************* */

expr_ptr make_literal(short type, char *txt, long l) {
  expr_ptr e     = new_expr(LITERAL_);
  e -> u.lit.s   = (txt) ? (char *)strdup(txt) : NULL;
  e -> u.lit.set = 0;

  if (txt) 
    switch (type) {

    case -2:
      break;

    case -1: 
      e -> type = make_c_i_type(MK_STRING);
      break;

    case CHAR_T:
      e -> type = make_primitive_type(CHAR_T);
      break;

    case BOOL_T:

     /* boolean LITERAL can be stored internally
        **************************************** */

       e -> u.lit.set   = 1;
       e -> u.lit.val.l = (long) (tolower(txt[0]) == 't'); 
       e -> type        = make_primitive_type(BOOL_T);
       break;

    case INT_T:
    case LONG_T:
       { long l = strtol(txt, (char **) NULL, 0);

#ifndef IRIX       
         if ((l != LONG_MAX) && (l != LONG_MIN)) {
#endif
           /* integer LITERAL can be stored internally
	      **************************************** */

           e -> u.lit.set   = 1;
           e -> u.lit.val.l = l;
#ifndef IRIX	   
        }
        else 
          javar_out(2, "integer literal will only be handled symbolically");
#endif	 
       }
       e -> type = make_primitive_type(type);
       break;

    case FLOAT_T:
    case DOUBLE_T:
       { double d = strtod(txt, (char **) NULL);
                
 	 if ((d != HUGE_VAL) && ((-d) != HUGE_VAL)) {

           /* floating point LITERAL can be stored internally
	      *BUT* PRECISION MAY BE LOST IN THE C REPRESENTATION
	      *************************************************** */

           e -> u.lit.set   = 1;
           e -> u.lit.val.d = d;
        }
        else 
          javar_out(2, "f.p. literal will only be handled symbolically");
       }
       e -> type = make_primitive_type(type);
       break;

    default:
      javar_out(-1, "incorrect invocation of make_literal()");
  }
  else {
    
    /* Internally Generated integer LITERAL */

    char val_txt[25];

    if (l >= 0)
      sprintf(val_txt, "%li", l);
    else
      sprintf(val_txt, "(%li)", l);
 
    e -> u.lit.s     = (char *)strdup(val_txt);
    e -> u.lit.set   = 1;
    e -> u.lit.val.l = l;
    e -> type        = make_primitive_type(INT_T);
  }

  return e;
}

/* Names (TYPE : determine later) */

expr_ptr make_name(expr_ptr lcomp, symb_ptr id, short info) {
  expr_ptr e        = new_expr(ID_);
  e -> u.name.info  = info;
  e -> u.name.scope = -1;
  e -> u.name.lcomp = lcomp;
  e -> u.name.id    = id;

  if (! id)
    javar_out(-1, "corrupt identifier in make_name()");
  else if (lcomp) {
  
    /* Construct Qualified Name
       DYNAMIC MEM ALLOCATION -> no restrictions on length */

    char *qname = (char *) make_mem(  sizeof(char) * 
                           ( strlen(id_table(lcomp -> u.name.entry)) +   
                             strlen(id_table(id)) + 2) );
    sprintf(qname, "%s.%s", id_table(lcomp -> u.name.entry),
	  	            id_table(id) );

    e -> u.name.entry = insert_table(qname);

    free(qname);
  }
  else
    e -> u.name.entry = id;
  return e;
}

/* Declaration Statement EXPRESSIONS (TYPE : type of declaration) */    

expr_ptr make_declexpr(stmt_ptr s) {
  expr_ptr e      = new_expr('d');

  if ((! s) || (s -> kind != DECL_S))
    javar_out(-1, "invalid invocation of make_declexpr()");

  e -> type       = s -> u.decl.tp;
  e -> u.declstmt = s;
  return e;
}

expr_ptr grep_decls(stmt_ptr s) {
  expr_ptr decls = NULL;
  if (s) {

    /* This hack for for-init constructs
       requires special treatment during unparsing! */

    if (s -> kind == CONCAT_S) 
      decls = make_op(',', grep_decls(s -> u.concat.s1),
                           grep_decls(s -> u.concat.s2));
    else if (s -> kind == DECL_S) 
      decls = make_declexpr(s);     
    else
      javar_out(-1, "AST corrupt in grep_decls() <%i>", s -> kind); 
  }
  return decls;
}

/* TE Operators (TYPE : determine now) */

expr_ptr make_te(short kind, type_ptr tp, expr_ptr ex, stmt_ptr st) {
  expr_ptr e   = new_expr(kind);
  e -> u.te.tp = tp;
  e -> u.te.ex = ex;
  e -> u.te.st = st;

  /* determine type */

  if (kind == NEW_) {

    if ((ex) && (ex -> kind == ']')) {

      /* Account for DIM-part in Array Creation Expression */

      int i  = scan_left(ex -> u.op.lhs);
	  i += scan_left(ex -> u.op.rhs);

      while (i-- > 0)
        tp = make_array_type(tp); 
    }

    e -> type = tp;         

  }
  else if (kind == INSTOF_)
    e -> type = make_primitive_type(BOOL_T);
  else
    e -> type = NULL;         

  return e;
}

expr_ptr make_cast(type_ptr tp, expr_ptr dims, expr_ptr ex) {
  expr_ptr e = new_expr('c');

  for ( ; dims; dims = dims -> u.op.lhs) { 
    if (dims -> kind != '[')
      javar_out(-1, "AST corrupt in make_cast() <%i>", dims -> kind);
    tp = make_array_type(tp); 
  }

  e -> type    = 
  e -> u.te.tp = tp;
  e -> u.te.ex = ex;
  return e;
}

/* This and Super Constructs (TYPE : determine later) */

expr_ptr make_thissup(short kind) {
  expr_ptr e    = new_expr(kind);
  e -> u.op.lhs = NULL;
  e -> u.op.rhs = NULL;
  return e;
}

/* Ordinary Operators (TYPE : determine later) */

expr_ptr make_op(short kind, expr_ptr lhs, expr_ptr rhs) {
  expr_ptr e    = new_expr(kind);
  e -> u.op.lhs = lhs;
  e -> u.op.rhs = rhs;
  return e;
}

/* ******************************
   *** Statement Manipulation ***  
   ****************************** */

/* Assign Label to Statement */

stmt_ptr give_label(stmt_ptr s, symb_ptr symb) {
  stmt_ptr lab       = new_stmt(LAB_S);
  lab -> u.lab.label = make_name(NULL, symb, LABEL_N);
  lab -> u.lab.lstmt = s;
  return lab;
}

/* Assign Modifiers and Types to Declaration Statements */

stmt_ptr give_m_t_c(short mod, type_ptr tp, stmt_ptr s, short info) {
  if (s) {
     if (s -> kind == CONCAT_S) {
       s -> u.concat.s1 = give_m_t_c(mod, tp, s -> u.concat.s1, info); 
       s -> u.concat.s2 = give_m_t_c(mod, tp, s -> u.concat.s2, info); 
     } 
     else if (s -> kind == DECL_S)  {

      /* Account for C-like Array Declarations */

      while (s -> u.decl.mod-- > 0)
        tp = make_array_type(tp); 

      s -> u.decl.tp  = tp;
      s -> u.decl.mod = mod; 

      /* Check Modifiers */

      check_mod(s -> kind, mod, 0);

      s -> u.decl.name -> u.name.info  = info;
    }
    else
      javar_out(-1, "AST corrupt in give_mods_and_type() <%i>", s -> kind); 
  }
  return s;
}

/* Assign Body to Header */

stmt_ptr give_body(stmt_ptr header, stmt_ptr body, short is_concrete) {
  header -> u.header.body = body;

  /* Test Modifier */

  if ((is_concrete) && (header -> u.header.mod & (ABS_B | NAT_B)))
    javar_out(0, "abstract/native method cannot have a body");
  if ((! is_concrete) && (! (header -> u.header.mod & (ABS_B | NAT_B))))
    javar_out(0, "non-abstract/non-native method requires a body");

  /* Reset Comment Buffer */

  comment_pos = comment_base; 

  return header;
}

/* Set Annotation Values */

void set_annot(symb_ptr name, expr_ptr e, short loop) {
  if (name) {
    short  set = 0;
    char  *nm  = id_table(name);

    if ((loop) && (! strcmp(nm, "threads")))
      javar_threads = get_val(javar_threads, e, 0);
    else if ((loop) && (! strcmp(nm, "multi")) )
      javar_multi = get_val(javar_multi, e, 1);
    else if ((loop) && (! strcmp(nm, "sync")) )
      javar_sync = get_val(javar_sync, e, 1);
    else if ((loop) && (! strcmp(nm, "block")) )
      { javar_sched = 0; set = 1; }
    else if ((loop) && (! strcmp(nm, "cyclic")) )
      { javar_sched = 1; set = 1; }
    else if ((loop) && (! strcmp(nm, "guided")) )
      { javar_sched = 2; set = 1; }
    else if ((loop) && (! strcmp(nm, "susp_wait")) )
      { javar_bw    = 0; set = 1; }
    else if ((loop) && (! strcmp(nm, "busy_wait")) )
      { javar_bw    = 1; set = 1; }
    else if ((! loop) && (! strcmp(nm, "cut_depth")) )
      javar_cut = get_val(javar_cut, e, 1);
    else if ((! loop) && (! strcmp(nm, "no_overhead")) )
      { javar_copy  = 1; set = 1; }
    else if ((! loop) && (! strcmp(nm, "overhead")) )
      { javar_copy  = 0; set = 1; }
    else 
      javar_out(0, "cannot process %s in %s annotation", nm,
		   (loop) ? "loop" : "method");
 
    if ((set) && (e))
      javar_out(0, "redundant value in %s annotation", 
		   (loop) ? "loop" : "method");
  }
  else { 

    /* SET DEFAULT VALUES */

    javar_threads = 4;  /* 4 threads              */ 
    javar_sync    = 0;  /* DOALL-like execution   */ 
    javar_multi   = 0;  /* no multi-version code  */
    javar_sched   = 2;  /* guided self-scheduling */
    javar_bw      = 0;  /* suspended waiting      */

    javar_cut     = 1;
    javar_copy    = 0;  /* make a overhead-free copy for 
			   serial execution */

    javar_excep   = NULL;
  }
}

void set_dist_annot(symb_ptr name, expr_ptr e) {
  if(name) {
    char *nm = id_table(name);

    if(! strcmp(nm, "hosts"))
      javar_hosts = get_val(javar_hosts, e, 0);
    else if(! strcmp(nm, "threads"))
      javar_threads = get_val(javar_threads, e, 0);
    else
      javar_out(0, "cannot process %s in distribute annotation", nm);
  }
  else {
    javar_hosts = 2;
    javar_threads = 1;
  }
}

void add_dist_symb(expr_ptr name, short s, short d, expr_ptr offset, expr_ptr bsize, short a) {
  dist_symb_ptr new = (dist_symb_ptr)make_mem(sizeof(struct dist_symb_node));
  new->name   = name;
  new->shared = s;
  new->dist   = d;
  new->offset = offset;
  new->bsize  = bsize;
  new->access = a;
  new->next   = NULL;
  
  if(dist_sym_tab == NULL)
    dist_sym_tab = new;
  else
    last_dist_sym->next = new;

  last_dist_sym = new;
}

stmt_ptr make_dist_annot(void) {
  stmt_ptr dannot = new_stmt(DIST_ANNOT_S);
  dannot->u.dist_annot.hosts   = javar_hosts;
  dannot->u.dist_annot.threads = javar_threads;
  dannot->u.dist_annot.distsym = dist_sym_tab;
  dannot->u.dist_annot.body    = NULL;
  
  dist_sym_tab = NULL;
  return dannot;
}
       
/* Link Name into List */

void add_annot(expr_ptr nm) {
  if (javar_excep) 
    javar_excep = make_op(',', javar_excep, nm);
  else 
    javar_excep = nm;
}

/* ******************************
   *** Statement Construction *** 
   **********************************************************
   *** Special AST Constructs: make_annot()    concat()   ***
   *** Simple Stmts:           make_simple()              ***
   *** Control Stmts:          make_control()             ***
   *** Declaration Stmts:      make_decl()                ***
   *** Headers:                make_header()              ***
   ********************************************************** */

/* Make an Annotation 
   inv  : NULL NULL val 
   post : val  NULL NULL
   wait : val  val  NULL
   par  : NULL NULL NULL
   ********************* */

stmt_ptr make_annot(expr_ptr kvar, expr_ptr arg, stmt_ptr inv) {
  stmt_ptr annot        = new_stmt(ANNOT_S);
  annot -> u.annot.body = NULL;

  if (inv) {

    /* par_inv */

    annot -> u.annot.body = inv;
    annot -> u.annot.f1   = javar_cut;
    annot -> u.annot.f2   = -1;
    annot -> u.annot.f4   = javar_copy;

    annot -> u.annot.arg  = javar_excep;
  }
  else if (kvar) {

    /* post/wait */

    annot -> u.annot.f1  = get_val(0, kvar, 1);
    annot -> u.annot.f2  = 0;
    annot -> u.annot.arg = arg;
  }
  else {

    /* par */

    annot -> u.annot.f1  = javar_threads;
    annot -> u.annot.f2  = javar_sync; 
    annot -> u.annot.f3  = javar_multi; 
    annot -> u.annot.f4  = javar_sched; 
    annot -> u.annot.f5  = javar_bw; 

    annot -> u.annot.arg = javar_excep;
  }
  return annot;
}

/* Concatenate Statement Lists */             

stmt_ptr concat(stmt_ptr s1, stmt_ptr s2) {
  if (s1 == NULL)
    return s2;
  if (s2 == NULL)
   return s1;
  else {
    stmt_ptr s       = new_stmt(CONCAT_S);
    s -> u.concat.s1 = s1;
    s -> u.concat.s2 = s2;
    return s;
  }
}

stmt_ptr make_simple(short kind, expr_ptr arg) {
  stmt_ptr s = new_stmt(kind);
  s -> u.arg = arg;
  return s;
}

stmt_ptr make_control(short kind, expr_ptr e, stmt_ptr s1, stmt_ptr s2) {
  stmt_ptr s           = new_stmt(kind);
  s -> u.control.e     = e;
  s -> u.control.s1    = s1;
  s -> u.control.s2    = s2;
  s -> u.control.scope = -1; 

  if (kind == FOR_S) {
    s -> u.control.scope = new_scope(); 

    if (s2) {

      /* Handle Loop Annotation */

      if(s2->kind == DIST_ANNOT_S)
        s2 -> u.dist_annot.body = s;
      else if(s2->kind == ANNOT_S)
	s2 -> u.annot.body = s;
      else
	assert(0);
	
      s  -> u.control.s2 = NULL;
      s                  = s2;
    }
  }
  return s;
}

stmt_ptr make_decl(symb_ptr entry, short clike, expr_ptr e) {
  stmt_ptr s       = new_stmt(DECL_S);
  s -> u.decl.init = e;
  s -> u.decl.mod  = clike;                         /* Temporarily store clikes */
  s -> u.decl.name = make_name(NULL, entry, AMBI_N); 
  return s;
}

stmt_ptr make_header(short kind, short mod, symb_ptr id, type_ptr tp, 
		     short clike, expr_ptr e1, expr_ptr e2, stmt_ptr body, int inout) {
  stmt_ptr s              = new_stmt(kind);
  s -> u.header.name      = NULL;
  s -> u.header.e1        = e1;
  s -> u.header.e2        = e2;
  s -> u.header.body      = body;
  s -> u.header.mod       = mod;
  s -> u.header.scope     = new_scope(); 
  s -> u.header.container = top_header();
  
  /* Account for C-like Array Declarations */

  while (clike-- > 0)
    tp = make_array_type(tp); 

  s -> u.header.tp = tp;

  /* Check Modifiers */

  check_mod(kind, mod, inout);

  if (id) { 
    switch(kind) {
      case INTERF_S:
      case CLASS_S:
        s -> u.header.name = make_name(NULL, id, TYPE_N);
        break;
      case CONS_S:
      case METHOD_S:
        s -> u.header.name = make_name(NULL, id, METH_N);
	break;
     }
  }

  /* Reset Comment Buffer */
  comment_pos = comment_base; 

  if((s->kind == CLASS_S) || (s->kind == INTERF_S))
  {
      /* set fully qualified classname */
      s->u.header.fqcn = determine_fqcn(s);
/*       printf("Adding fqcn: "); */
/*       dump_expr(s->u.header.fqcn, stdout); */
/*       printf("\n"); */
  
      /* initialize namelist */
      s->u.header.nlist = NULL;
      s->u.header.nllength = 0;
      s->u.header.maxnllength = 0;
  }
  return s;
}

/* Statement Checks
   **************** */

void check_transfer(short brk) {
  if (brk) {
    if ((! switch_cnt) && (! iterat_cnt))
      javar_out(0, "no enclosing switch/while/do/for for break");
  }
  else {
    if (! iterat_cnt)
      javar_out(0, "no enclosing while/do/for for continue");
  }
}

/* ***********************************
   *** Test if two types are equal ***
   **************************************************************
   *** Null Types and                                         ***
   *** Primitive Types: trivial                               ***
   *** Reference Types: if (1) + both class/both interface    ***
   ***                         + same fully-qualified name    ***
   ***                         + loaded by same class loader  ***
   ***                     (2) both array types               ***
   ***                         same component type            ***
   ************************************************************** */   

short equal_type(type_ptr t1, type_ptr t2) {
  if (t1)
    switch (t1 -> kind) {

      case C_OR_I_T:
	return ( (t2)                             && 
		 (t2 -> kind == C_OR_I_T)         &&
		 (t1 -> u.name -> u.name.entry == 
		  t2 -> u.name -> u.name.entry) );

      case ARRAY_T:
	return ( (t2)                                  && 
		 (t2 -> kind == ARRAY_T)               && 
		 (equal_type(t1 -> u.aref, t2 -> u.aref)) );

      default:
        return ( (t2) && (t2 -> kind == t1 -> kind) );

    }
    else
     return (! t2);
}

/* ********************
   *** AST Copying  ***                                
   ********************************************************
   *** copy_type:                                       ***
   *** copy_expr:                                       ***
   *** copy_stmt:                                       ***
   ***            make a fresh copy of type/expr/stmt   ***
   ******************************************************** */

type_ptr copy_type(type_ptr t) {
  type_ptr new = NULL, next;

  if (t) {

   new  = new_type(t -> kind);
   next = new -> next;

   /* Make an Exact Copy of the Node 
      (PRESERVE NEXT, share table entries, copy all other fields)  */

   *new = *t;
   new -> next = next;

   /* Deal with Sub-trees */

   switch(t -> kind) {

      case C_OR_I_T:
	t -> u.name = copy_expr(t -> u.name);
	break;

      case ARRAY_T:
	t -> u.aref = copy_type(t -> u.aref);
	break;

      default:
	break;
    }
  }
  return new;
}

expr_ptr copy_expr(expr_ptr e) {
 expr_ptr new = NULL, next;

 if (e) {
   
   new  = new_expr(e -> kind);
   next = new -> next;

   /* Make an Exact Copy of the Node 
      (PRESERVE NEXT, share table entries, copy all other fields)  */

   *new = *e;
   new -> next = next;

   /* Deal with Sub-trees */

   switch(e -> kind) {

      /* Literals 
	 ******** */

      case LITERAL_:
	break;

      /* Names 
	 ***** */

      case ID_:

	 new -> u.name.lcomp = copy_expr(e -> u.name.lcomp); 
	 break;

      /* Decl. Stmt. EXPRESSION
	 ********************** */

      case 'd':

	new -> u.declstmt = copy_stmt(e -> u.declstmt);
	break;

      /* TE Operators
	 ************ */

      case NEW_:
      case INSTOF_:
      case 'c':
      case 't':

        new -> u.te.tp = copy_type(e -> u.te.tp);
	new -> u.te.ex = copy_expr(e -> u.te.ex);
	break;

      /* List-like and Ordinary Operators
	 ******************************** */

      default: 

	new -> u.op.lhs = copy_expr(e -> u.op.lhs);
	new -> u.op.rhs = copy_expr(e -> u.op.rhs);
	break;
     }
  }
  return new;
}

stmt_ptr copy_stmt(stmt_ptr s) {
  stmt_ptr new = NULL, next;

  if (s) {

   new  = new_stmt(s -> kind);
   next = new -> next;

   /* Make an Exact Copy of the Node 
      (PRESERVE NEXT, share table entries, 
       copy all other fields, make new scope)  */

   *new = *s;
   new -> next = next;

   /* Deal with Sub-trees */

   switch(s -> kind) {

	/* Special AST Constructs
	   ********************** */

	case ANNOT_S:
          new -> u.annot.arg  = copy_expr(s -> u.annot.arg);
          new -> u.annot.body = copy_stmt(s -> u.annot.body);
	  break;

        case CONCAT_S:

           new -> u.concat.s1 = copy_stmt(s -> u.concat.s1);
           new -> u.concat.s2 = copy_stmt(s -> u.concat.s2);
	   break;

        case LAB_S:
           new -> u.lab.label = copy_expr(s -> u.lab.label);
           new -> u.lab.lstmt = copy_stmt(s -> u.lab.lstmt);
	   break;

	/* Simple Statements 
	   ***************** */

        case STIMP_S: 
	case IMPOD_S:
	case PCK_S:    
	case BRK_S:
	case CONT_S:   
	case RET_S:
	case THROW_S:  
	case CASE_S:
	case DEF_S:   
	case EXP_S:
	case EMPTY_S:

          new -> u.arg = copy_expr(s -> u.arg);
	  break;

	/* Control Statements
	   ****************** */

	case FOR_S:
	  new -> u.control.scope = new_scope();

	case IF_S:      /* FALL THROUGH */
	case SWITCH_S:
	case WHILE_S:
	case SYNC_S:
	case TRY_S:
	case FIN_S:
	case DOWHILE_S:

          new -> u.control.e  = copy_expr(s -> u.control.e);
          new -> u.control.s1 = copy_stmt(s -> u.control.s1);
          new -> u.control.s2 = copy_stmt(s -> u.control.s2);
	  break;

        /* Declaration Statements
	   ********************** */

	case DECL_S:
  
	  new -> u.decl.name  = copy_expr(s -> u.decl.name);
          new -> u.decl.init  = copy_expr(s -> u.decl.init);
	  break;

        /* Headers 
  	   ******* */

	case BLOCK_S: 
	case CLASS_S:
	case INTERF_S:
	case CONS_S:
	case SINIT_S:
	case METHOD_S:
	case CATCH_S:
	  
	  new -> u.header.name  = copy_expr(s -> u.header.name);
          new -> u.header.e1    = copy_expr(s -> u.header.e1);
          new -> u.header.e2    = copy_expr(s -> u.header.e2);
          new -> u.header.body  = copy_stmt(s -> u.header.body);
	  new -> u.header.scope = new_scope();
	  break;

        default:
	  javar_out(-1, "AST corrupt in copy_stmt() <%i>", s -> kind); 
    }
  }
  return new;
}

/* *********************
   *** AST Traversal ***
   **********************************************************
   *** type_traverse:                                     ***
   *** expr_traverse:                                     ***
   *** stmt_traverse:                                     ***
   ***    traverse AST and apply (NULL for noaction)      ***
   ***       + stmt_action to each stmt  (PRE-ORDER)      ***
   ***       + expr_action to each expr  (POST-ORDER)     ***
   ********************************************************** */

void type_traverse( type_ptr t,
		    void (*stmt_action)(stmt_ptr),
		    void (*expr_action)(expr_ptr) ) {
  if (t) {

#ifdef DEBUG_TRAV 
  fprintf(stderr, "traversing type %i\n", t -> kind);
#endif

   /* Traversal */

   switch(t -> kind) {

      case C_OR_I_T:
	expr_traverse(t -> u.name, stmt_action, expr_action);
	break;

      case ARRAY_T:
	type_traverse(t -> u.aref, stmt_action, expr_action);
	break;

      default:
	break;
    }
  }
}

void expr_traverse( expr_ptr e,
		    void (*stmt_action)(stmt_ptr),
		    void (*expr_action)(expr_ptr) ) {
  if (e) {

    lineno = e -> lineno;
    
#ifdef DEBUG_TRAV
  fprintf(stderr, "traversing expression %i %c @ %i\n", e -> kind, e -> kind, lineno);
#endif
    
    /* Traversal */

    switch(e -> kind) {

      /* Literals
	 ******** */

      case LITERAL_:

	break;

      /* Names 
	 ***** */

      case ID_:

  	expr_traverse(e -> u.name.lcomp, stmt_action, expr_action);
	break;

      /* Decl. Stmt. EXPRESSION
	 ********************** */

      case 'd': 

	stmt_traverse(e -> u.declstmt, stmt_action, expr_action);
	break;

      /* TE Operators
	 ************ */

      case NEW_:
      case INSTOF_:
      case 'c':      /* CAST */
      case 't':      /* type-list */

        type_traverse(e -> u.te.tp, stmt_action, expr_action);
	expr_traverse(e -> u.te.ex, stmt_action, expr_action);
	break;

      /* List-like and Ordinary Operators
	 ******************************** */

      case 'i':

	{ expr_ptr old_inv = cur_inv; /* to deal with `nested' invocations */
	 
  	  expr_traverse(e -> u.op.rhs, stmt_action, expr_action); /* args first */
          cur_inv = e;
	  expr_traverse(e -> u.op.lhs, stmt_action, expr_action); 
	  cur_inv = old_inv; 
        }
	break;

      /* Class literals
       **************** */
      case 'l':
	type_traverse(e->u.te.tp, stmt_action, expr_action);
	expr_traverse(e->u.te.ex, stmt_action, expr_action);
	break;
	
      default: 

	expr_traverse(e -> u.op.lhs, stmt_action, expr_action);
	expr_traverse(e -> u.op.rhs, stmt_action, expr_action);
	break;
    }

    type_traverse(e -> type, stmt_action, expr_action);

    /* Apply Action */            

    if (expr_action)
      (*expr_action)(e);
  }
}

void stmt_traverse( stmt_ptr s,
		    void (*stmt_action)(stmt_ptr),
		    void (*expr_action)(expr_ptr) )
{
  stmt_ptr saveclass, savemethod;
  
  if (s) {

    lineno = s -> lineno;

#ifdef DEBUG_TRAV
  fprintf(stderr, "traversing statement %i @ %i\n", s -> kind, lineno);
#endif
 
    /* Apply Action */            

    if (stmt_action)
      (*stmt_action)(s);

    /* Traversal */

    switch(s -> kind) {

	/* Special AST Constructs
	   ********************** */

        case DIST_ANNOT_S:
	{
	  dist_symb_ptr dsp = s->u.dist_annot.distsym;

	  stmt_traverse(s->u.dist_annot.body, stmt_action, expr_action);
	  while(dsp)
	  {
	      expr_traverse(dsp->name, stmt_action, expr_action);
	      dsp = dsp->next;
	  }
	}
	break;
	  
	case ANNOT_S:
          expr_traverse(s -> u.annot.arg,  stmt_action, expr_action);
          stmt_traverse(s -> u.annot.body, stmt_action, expr_action);
	  break;

        case CONCAT_S:

           stmt_traverse(s -> u.concat.s1, stmt_action, expr_action);
           stmt_traverse(s -> u.concat.s2, stmt_action, expr_action);
	   break;

        case LAB_S:
       
           activate_lab(s);    

           expr_traverse(s -> u.lab.label, stmt_action, expr_action);
           stmt_traverse(s -> u.lab.lstmt, stmt_action, expr_action);

           deactivate_lab(s -> u.lab.label);    
           break;

	/* Simple Statements 
	   ***************** */

        case STIMP_S: 
	case IMPOD_S:
	case PCK_S:    
	case BRK_S:
	case CONT_S:   
	case RET_S:
	case THROW_S:  
	case CASE_S:
	case DEF_S:   
	case EXP_S:
	case EMPTY_S:

	  expr_traverse(s -> u.arg, stmt_action, expr_action);
	  break;

	/* Control Statements
	   ****************** */

	case FOR_S:

	  /* Traverse Loop in Appropriate Scope */

	  set_scope(s -> u.control.scope); 

	  expr_traverse(s -> u.control.e,  stmt_action, expr_action);

	  iterat_cnt++;

	  stmt_traverse(s -> u.control.s1, stmt_action, expr_action);

	  iterat_cnt--;

	  old_scope();
	  break;

	case IF_S:
	case SWITCH_S:
	case WHILE_S:
	case SYNC_S:
	case TRY_S:
	case FIN_S:
	case DOWHILE_S:

	  expr_traverse(s -> u.control.e,  stmt_action, expr_action);

	  if (s -> kind == SWITCH_S)
	    switch_cnt++;
          else if ( (s -> kind == WHILE_S) || (s -> kind == DOWHILE_S) ) 
	    iterat_cnt++;

	  stmt_traverse(s -> u.control.s1, stmt_action, expr_action);
	  stmt_traverse(s -> u.control.s2, stmt_action, expr_action);

	  if (s -> kind == SWITCH_S)
	    switch_cnt--;
          else if ( (s -> kind == WHILE_S) || (s -> kind == DOWHILE_S) ) 
	    iterat_cnt--;

	  break;

        /* Declaration Statements
	   ********************** */

	case DECL_S:

	  cur_decl = s;

	  type_traverse(s -> u.decl.tp,   stmt_action, expr_action);
	  expr_traverse(s -> u.decl.name, stmt_action, expr_action);
	  expr_traverse(s -> u.decl.init, stmt_action, expr_action);

	  cur_decl = NULL;

	  break;

        /* Headers 
  	   ******* */

	case BLOCK_S: 
	case CLASS_S:
	case INTERF_S:
	case CONS_S:
	case SINIT_S:
	case METHOD_S:
	case CATCH_S:

	   if ( (s -> kind == CLASS_S) || (s -> kind == INTERF_S) ) {
	     saveclass = cur_class;
             cur_class = s;
	   }
           else if ( (s -> kind == METHOD_S) || (s -> kind == SINIT_S) 
		     || (s -> kind == CONS_S) ) {
	     savemethod = cur_method;
             cur_method = s;
	   }

	  type_traverse(s -> u.header.tp,   stmt_action, expr_action);
	  expr_traverse(s -> u.header.name, stmt_action, expr_action);

	  /* Traverse in Appropriate Scope */

	  set_scope(s -> u.header.scope); 

	  expr_traverse(s -> u.header.e1,   stmt_action, expr_action);
	  expr_traverse(s -> u.header.e2,   stmt_action, expr_action);
	  stmt_traverse(s -> u.header.body, stmt_action, expr_action);

	  old_scope();

          if ( (s -> kind == CLASS_S) || (s -> kind == INTERF_S) )
             cur_class  = saveclass;
          else if ( (s -> kind == METHOD_S) || (s -> kind == SINIT_S) 
                                            || (s -> kind == CONS_S) ) 
             cur_method = savemethod;

	  break;

        default:
	  javar_out(-1, "AST corrupt in stmt_traverse() <%i>", s -> kind); 
     }

     lineno = 0;
  }
}

/* *************************************************
   *** AST Initialization and Garbage Collection ***
   ************************************************* */

void set_ast(void) {

  comp_unit = NULL;
  lineno    = 0;

  set_table();

  comment_buffer = NULL;

  comment_len    = 0;
  comment_pos    = 0;
  comment_base   = 0;
}

void del_ast(void) {
  
  comp_unit = NULL;
  lineno    = 0;

  del_table();

  /* Delete Type, Expr, and Stmt Nodes */

  while (type_glink) {
     type_ptr next = type_glink -> next;
     free(type_glink);
     type_glink = next;
  }
  while (expr_glink) {
     expr_ptr next = expr_glink -> next;
     if ( (expr_glink -> kind == LITERAL_) &&
          (expr_glink -> u.lit.s) )
      free(expr_glink -> u.lit.s); 
     free(expr_glink);
     expr_glink = next;
  }
  while (stmt_glink) {
     stmt_ptr next = stmt_glink -> next;
     if((stmt_glink->kind == CLASS_S) && stmt_glink->u.header.nlist)
	 free(stmt_glink->u.header.nlist);
     free(stmt_glink);
     stmt_glink = next;
  }

  if (comment_buffer) 
    free(comment_buffer); 

  comment_buffer = NULL;

  comment_len    = 0;
  comment_pos    = 0;
  comment_base   = 0;
}
