


/* *************
   *** JAVAS ***
   **********************************************************
   *** Copyright (c) 1997                                 ***
   *** Fabian Breg and Aart J.C. Bik   Indiana University *** 
   *** All Rights Reserved                                ***
   **********************************************************
   *** Please refer to the LICENSE file for
   *** licensing terms and conditions
   *** */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <stdarg.h>

/* *************************************
   *** JVM-specific Type Definitions ***
   *************************************
   *** u1_int == unsigned 8  bits    ***
   *** s1_int ==   signed 8  bits    ***
   *************************************
   *** u2_int == unsigned 16 bits    ***
   *** s2_int ==   signed 16 bits    ***
   *************************************
   *** u4_int == unsigned 32 bits    ***
   *** s4_int ==   signed 32 bits    ***
   ************************************* */

#define U1MAX 255u
#define U2MAX 65535u
#define U4MAX 4294967295u
#define S4MAX 2147483647

#if ((CHAR_BIT == 8u) && (UCHAR_MAX == U1MAX))
#define U1 "char"
typedef unsigned char u1_int;
typedef signed   char s1_int;
#endif

#if (USHRT_MAX == U2MAX)
#define U2 "short int"
typedef unsigned short int u2_int;
typedef signed   short int s2_int;
#elif (UINT_MAX == U2MAX)
#define U2 "int"
typedef unsigned int u2_int;
typedef signed   int s2_int;
#endif

#if (UINT_MAX == U4MAX)
#define U4 "int"
typedef unsigned int u4_int;
typedef signed   int s4_int;
#elif (ULONG_MAX == U4MAX)
#define U4 "long int"
typedef unsigned long int u4_int;
typedef signed   long int s4_int;
#elif (ULONGLONG_MAX == U4MAX)
#define U4 "long long int"
typedef unsigned long long int u4_int;
typedef signed   long long int s4_int;
#endif

/* byte <-> int conversions (JVM uses big-endian order) 
   **************************************************** */

#define B2U2(b1,b2)       ((((u2_int)(b1))<<8)+((u2_int)(b2)))
#define B2U4(b1,b2,b3,b4) ((((u4_int)(B2U2((b1),(b2)))<<16)) + \
			    ((u4_int)(B2U2((b3),(b4)))))
#define B2S2(b1,b2)       ((s2_int)(B2U2((b1),(b2))))
#define B2S4(b1,b2,b3,b4) ((s4_int)(B2U4((b1),(b2),(b3),(b4))))

/* ********************************************
   *** Class-file-specific Type Definitions ***
   ******************************************** */

/* Access and Modifier Flags 
   ************************* */

#define ACC_PUBLIC       0x0001
#define ACC_PRIVATE      0x0002
#define ACC_PROTECTED    0x0004
#define ACC_STATIC       0x0008
#define ACC_FINAL        0x0010
#define ACC_SUPER        0x0020
#define ACC_SYNCHRONIZED 0x0020 /* overloaded */
#define ACC_VOLATILE     0x0040
#define ACC_TRANSIENT    0x0080
#define ACC_NATIVE       0x0100
#define ACC_INTERFACE    0x0200
#define ACC_ABSTRACT     0x0400

/* Constant Pool Tags
   ****************** */

#define CONSTANT_Utf8                1u
#define CONSTANT_Integer             3u
#define CONSTANT_Float               4u
#define CONSTANT_Long                5u
#define CONSTANT_Double              6u
#define CONSTANT_Class               7u
#define CONSTANT_String              8u
#define CONSTANT_Fieldref            9u
#define CONSTANT_Methodref          10u
#define CONSTANT_InterfaceMethodref 11u
#define CONSTANT_NameAndType        12u

/* Constant Pool Entries 
   ********************* */

struct constant_node {
   union {
     struct {
       u4_int val1;
       u4_int val2;
     } data;
     struct {
       u2_int index1;
       u2_int index2;
     } indices;
     struct {
       u1_int *s;
       u2_int  l;
     } utf8;
   } u ;
   u1_int tag;
} ;

typedef struct constant_node *constant_ptr;

/* Attribute Entries 
   ***************** */

struct attribute_node {
 u2_int  attribute_name_index;
 u4_int  attribute_length;
 u1_int *info;
} ;

typedef struct attribute_node *attribute_ptr;

/* Field/Method Entries 
   ******************** */

struct fm_node {
 u2_int         access_flags;
 u2_int         name_index;
 u2_int         descr_index;
 u2_int         attributes_count;
 attribute_ptr *attributes;

 u2_int         noe;
 u2_int        *exc;
} ;

typedef struct fm_node *fm_ptr;

/* Class File Entries 
   ****************** */

struct class_node {

  struct class_node *next;

  u1_int            *classname;

  /* class file specific info */

  u4_int         magic;
  u2_int         minor_version, major_version;

  u2_int         constant_pool_count;
  constant_ptr  *constant_pool;

  u2_int         access_flags, this_class, super_class;

  u2_int         interfaces_count;
  u2_int        *interfaces;

  u2_int         fields_count;
  fm_ptr        *fields;

  u2_int         methods_count;
  fm_ptr        *methods;

  u2_int         attributes_count;
  attribute_ptr *attributes;

} ;

typedef struct class_node *class_ptr;

extern short verbose;

/* Prototypes
   ********** */

void    javas_out(s1_int level, char *fmt, ...);
short   read_classes(char *);
void    delete_classes(void);

char   *get_superclass(char *);
char   *get_typedescriptor(char *, fm_ptr);
char   *get_name(char *, fm_ptr);

short   implements(char *, char *);
short   contains_method(char *, char *, char *);
short   contains_imethod(char *, char *, char *);

void    trav_class_init(char *, short, short, short);
fm_ptr  get_next_field(void);
fm_ptr  get_next_method(void);
char   *get_next_interface(void);
char   *get_next_exception(void);

short   is_final_class(char *);
short   is_static_method(fm_ptr m);

