
/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** driver.c : control program
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

/* This file has been modified from the original to support
 * Java 1.1 language and the NexusRMI module.
 *
 *     Fabian Breg.
 */

/* ********************************************************
   *** INCLUDE FILES and DEFINITIONS                    ***
   ********************************************************  */

#include <stdarg.h>
#include "global.h"

/* ********************************************************
   *** EXTERNAL VARIABLES                               ***
   ******************************************************** */

/* PUBLIC  
   ****** */

int   lineno        = 0;   
short comments_flag = 0;
char *filename;
short verbose       = 0;

/* PRIVATE
   ******* */

static char *version  = "1.3.3BETA";

static short error = 0, tot_err = 0;
static int   files   = 0;

/* flags */

static short names   = 0, output = 0;
static short makenexusable = 0;

/* ********************************************************
   *** PRIVATE FUNCTIONS                                ***
   ******************************************************** */

/* Move original  'file.ext' to 'file.orig'
   and open a new 'file.ext' for output
   ************************************************* */

static FILE *new_file(char *oldname) {
  int   i, last = -1;
  char *newname = NULL, c;
  FILE *newfile = NULL;

  if (! oldname)   /* Safety */
    javar_out(-1, "incorrect invocation of new_file()");

  /* Construct new name file.orig by stripping 
     last extension from original file.ext      */

  for (i = 0; oldname[i]; i++) 
    if (oldname[i] == '.') 
      last = i;

  if (last >= 0) {
    c = oldname[last];
    oldname[last] = '\0';
  }

  /* DYNAMIC MEMORY ALLOCATION -> no restriction on length */

  newname = (char *) make_mem(sizeof(char) * (strlen(oldname) + 6));
  sprintf(newname, "%s.orig", oldname);

  if (last >= 0) 
    oldname[last] = c;

  /* Prevent Overwriting of existing file.orig */

  if (newfile = fopen(newname, "r")) {
    javar_out(0, "cannot apply javar to %s"
		 " (%s exists)", oldname, newname);
    fclose(newfile);
    newfile = NULL;
  }
  else {
  
    /* DYNAMIC MEMORY ALLOCATION -> no restriction on length */

    char *command = (char *) make_mem(sizeof(char) * 
                        (strlen(oldname) + strlen(newname) + 5));

    sprintf(command, "mv %s %s", oldname, newname);

    /* Re-name original file.ext to file.orig */

    javar_out(2, "- executing '%s'", command);

    if (system(command))
      javar_out(0, "command %s failed", command);
    else {

      /* Re-open original file.ext for new output */

      if (! (newfile = fopen(oldname, "w")))
        javar_out(0, "cannot open file %s", oldname); 
      else
       javar_out(2, "- output to file %s", oldname);
    }
    free(command);
  } 
  free(newname);

  return newfile;
}

/* Process a Compilation Unit
   ************************** */

static void process(void) { 

  /* Initialize AST */

  set_ast();

  error = 0;
  files++;

  javar_out(2, "--- processing %s", 
    (filename) ? filename : "stdin");

  /* Parsing */

  javar_out(2, "- syntax analysis");

  lineno = 1;

  yyparse();

  lineno = 0;

  /* Semantic Analysis */

/*   if (! error)  { */

/*     javar_out(2, "- semantic analysis"); */

/*     semantic_validate();   */
/*   } */

  if (makenexusable) {

      javar_out(2, "- adding nexusRead / nexusWrite methods");

      if(error)
	  javar_out(2, "no changes made");
      else
	  nexusize();
  }
		
  /* Unparsing */

  if (output) {

    javar_out(2, "- unparsing (Source Code)");
    
    if (error)
      javar_out(1, "no output for %s", (filename) ? filename : "stdin");
    else {

        if (filename) {

          FILE *outfile = new_file(filename);
  	
	  if (outfile) {
	     fprintf(outfile, "// This file has been generated by JAVAR\n");
	     dump_stmt(comp_unit, outfile);
    	     fclose(outfile);
          }
        }
        else {
	  fprintf(stdout, "// This file has been generated by JAVAR\n");
	  dump_stmt(comp_unit, stdout);
	}
      }

  }

  /* Symbol Table Output */

  if ((! error) && (names)) {

    javar_out(2, "- symbol table output");

    show_names();
  }

  /* Delete AST */

  del_ast(); 
}

/* ********************************************************
   *** PUBLIC FUNCTIONS                                 ***
   ******************************************************** */

/* **********************************
   *** Error and Warning Messages ***
   ***************************************************************
   *** level == -1 : FATAL ERROR,     EXIT PROGRAM             ***
   *** level ==  0 : ERRROR,          SET ERROR FLAG           ***
   *** level ==  1 : STRONG MESSAGE                            ***
   *** level ==  2 : MESSAGE,         PRINT ONLY FOR '-v'      ***
   *************************************************************** */

void javar_out(int level, char *fmt, ...)  {
    va_list argv;

    if (level == 0)
       tot_err = error = 1; 
    else if (level > 1 && ! verbose)
      return;
    
    va_start(argv, fmt);

    if (lineno > 0)
      fprintf(stderr, "%s:%i:", (filename) ? filename : "stdin", lineno);
    else
      fprintf(stderr, "javar :: ");

    vfprintf(stderr, fmt, argv);
    putc('\n', stderr);

    va_end(argv);

    if (level < 0)
      exit(1);
}

/* Memory Allocation Functions
   [for size <= 0, size == 1 is used
    because some systems return NULL for malloc(0);]
   ************************************************* */

void *make_mem(int size) { 
  void *p = malloc( (size > 0) ? size : 1);
  if (! p) 
    javar_out(-1, "out of memory (first allocation)");
  return p; 
}

void *more_mem(void *p, int size) { 
  p = realloc(p, (size > 0) ? size : 1);
  if (! p) 
    javar_out(-1, "out of memory (re-allocation)");
  return p;
}

/* ********************
   *** Main Program ***
   ******************** */

int main(int argc, char *argv[]) { 
  int i, unproc = 1;

  /* Process Optional Flags in Command-line Arguments */

  for (i = 1; i < argc; i++)
    if (argv[i][0] == '-') {  
      int c, j = 1;

      while (c = argv[i][j++])     /* SIDE EFFECT TEST */
	     switch (c) { 

		 case 'c':
		    comments_flag = 1;
		    break;

		 case 'i':
		    javar_out(-1, "*** version %s [internal sizes %i/%i/%i/%i]", 
				  version,
			          sizeof(struct symb_node),
			          sizeof(struct type_node),
			          sizeof(struct expr_node),
			          sizeof(struct stmt_node));
		    break;

                 case 'n':
		    names = 1;
		    break;

                 case 'o':
		    output = 1;
		    break;

	         case 'x':
		     makenexusable = 1;
		     break;
		     
	         case 'v':
		    verbose = 1;
		    break;

                 default:
		   javar_out(-1, "usage: %s [-cinoxv] [file ...]", argv[0]);
              }

       argv[i] = NULL;

     }

  javar_out(2, "*** version %s [written by Aart J.C. Bik (C) 1996/1997]", version);

  /* Process Other Command-line Arguments */

  for (i = 1; i < argc; i++)
    if (argv[i]) { 
      unproc = 0;

      yyin = fopen(filename = argv[i], "r");
	
      if (yyin) {
        process();
        fclose(yyin);
      }
      else
        javar_out(0, "cannot open file %s", filename);
  }
  delete_classes();

  /* Process Standard Input by Default */

  if (unproc) {
    yyin = stdin;
    process();
  }

  exit(tot_err);
}
