

/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** global.h : symbol table and AST definitions
   ***            function prototypes
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

/* This file has been modified from the original to support
 * Java 1.1 language.
 *
 *     Fabian Breg.
 */

#include <assert.h>
#include <ctype.h>
#include <limits.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* ********************
   *** Symbol Table ***
   ***************************************************
   *** ambiguous                                   ***
   *** package                                     ***
   *** class or interface type  SCOPE: 0           ***
   ***                                             ***
   *** member: field or method                     ***
   *** constructor              SCOPE: class scope *** 
   ***                                             ***
   *** parameter                                   ***
   *** local variable                              ***
   *** statement label          SCOPE: block scope ***
   *************************************************** */

/* Symbolic Representations */

enum symbols { 
  AMBI_N, 
  PACK_N,  TYPE_N,  
  EXPR_N,  FIELD_N, LOCPAR_N, 
  METH_N,  
  LABEL_N, MISCF_N, MISCM_N
} ;

/* Data Structure for Symbol Table Nodes */

struct symb_node {
   struct symb_node *next;
   struct info_node *info;
   char             *symbol;
} ;

struct info_node {
  struct info_node *next;
  struct stmt_node *declstmt;
  short             scope, info;
} ;

typedef struct symb_node *symb_ptr;
typedef struct info_node *info_ptr;

/* ***********************
   *** Distribute info ***
   *************************************************
   *** tags:                                     ***
   *** SHARED_B / NON_SHARED_B                   ***
   *** access:                                   ***
   *** READ_ONLY_B / SCRATCH_B / PERSISTENT_B    ***
   ***                                           ***
   *** distribution function to be added         ***
   ************************************************* */

enum dist_tags {
  SHARED_B = 1, NON_SHARED_B = 2, METHOD_B = 4
} ;


enum dist_access {
    UNMODIFIED_B = 1, MOD_IN_B = 2, MOD_INOUT_B = 4, MOD_OUT_B = 8, FIN_METH_B = 16
} ;

enum dist_dist {
    NONE_B = 1, BLOCK_B = 2, CYCLIC_B = 4
} ;

struct dist_symb_node {
  struct dist_symb_node *next;
  struct expr_node      *name;
  short                  shared;
  short                  dist;
  struct expr_node      *offset;
  struct expr_node      *bsize;
  short                  access;
} ;

typedef struct dist_symb_node *dist_symb_ptr;

/* ******************
   *** JAVA Types ***
   ***********************************************
   *** Primitive Types: boolean, numeric       ***
   *** Reference Types: class/interface, array ***
   *** the Null Type  : NULL                   ***
   ***                                         ***
   *** NULL is also used for an unknown type   ***
   *********************************************** */

/* Symbolic Representations */

enum types { 

    VOID_T,

  /* Primitive Types */

    BOOL_T, BYTE_T, SHORT_T, INT_T, 
    LONG_T, CHAR_T, FLOAT_T, DOUBLE_T, 

  /* Reference Types */

    C_OR_I_T, ARRAY_T 
} ;

/* Data Structure for Types */

struct type_node {

   struct type_node *next;

   union {
     struct expr_node *name;
     struct type_node *aref;
   } u ;

   short kind;
} ;

typedef struct type_node *type_ptr;

/* ************************
   *** JAVA Expressions ***
   **************************************************************
   *** Some TOKEN symbols (ASCII characters and "y.tab.h")    ***
   *** are used as symbolic representations of expressions    ***
   **************************************************************
   *** Literals:                LITERAL_                      ***
   *** Names:                   ID_                           ***
   *** Decl. Stmt. EXPRESSIONS: 'd'                           ***
   *** TE Operators:            NEW_, INSTOF_                 ***
   ***                          'c'     (CAST)                ***
   ***                          't'     (type-list)           ***
   *** List-like Operators:     'i'     (method-invocation)   ***
   ***                          '{'     (array-initializer)   ***
   ***                          '[' ']' (array-subscript)     ***
   ***                          '?', ':', ',', '.' ';'        ***
   *** Ordinary Operators:      EQ_ .. RRRIS_, '+', '-' ...   ***
   ***                          this/super-construct          ***
   ************************************************************** */

/* Data Structure for Expressions */

struct expr_node {

   struct expr_node *next;

   union {

     /* Literals */

     struct {

       union {
         long   l;
         double d;
       } val;

       char   *s;
       short   set;
     } lit;

     /* Names */

     struct {
       struct symb_node *id;
       struct symb_node *entry;
       struct expr_node *lcomp;
       short             scope, info;
     } name ;

     /* Declaration Stmt. EXPRESSIONS */

     struct stmt_node *declstmt;

     /* TE Operators */

     struct {
       struct type_node *tp;
       struct expr_node *ex;
       struct stmt_node *st; /* java 1.1 */
     } te ;

     /* List-like and Ordinary Operators */

     struct {
       struct expr_node *lhs;
       struct expr_node *rhs;
     } op ;

   } u ;

   type_ptr type;
   int      lineno;
   short    kind;
} ;

typedef struct expr_node *expr_ptr;

/* ***********************
   *** JAVA Statements ***
   ********************************************************************
   *** Special Constructs:       annotations, concatenations,       ***
   ***                           labeled statements                 ***
   *** Simple Statements:        import, package,                   ***
   ***                           break, continue, return, throw,    ***
   ***                           case/default-label, expr_stmt ;    ***
   *** Control Statements:       if, for, switch, while,            ***
   ***                           synchronized, try, finally, do     ***
   *** Declaration Statements:   field and local declarations       ***
   *** Headers:                  blocks {}, class, interface,       ***
   ***                           constructor, initializer,          ***
   ***                           method, catch                      ***
   ******************************************************************** */

/* Symbolic Representations */

enum stmts { 

  /* Special AST Constructs */

  ANNOT_S, CONCAT_S, LAB_S, DIST_ANNOT_S,
  
  /* Simple Statements */

  STIMP_S, IMPOD_S, PCK_S, 
  BRK_S,   CONT_S,  RET_S,  THROW_S,  
  CASE_S,  DEF_S,   EXP_S,  EMPTY_S,

  /* Control Statements */

  IF_S,   FOR_S, SWITCH_S, WHILE_S, 
  SYNC_S, TRY_S, FIN_S,    DOWHILE_S,

  /* Declaration Statements */
  
  DECL_S, 

  /* Headers */
  
  BLOCK_S, CLASS_S, INTERF_S, CONS_S, SINIT_S, METHOD_S, CATCH_S

} ; 

/* Data Structure for Statements */

struct stmt_node {

   struct stmt_node *next;
   
   union {

     /* Special AST Constructs */

     struct {
       struct expr_node *arg;
       struct stmt_node *body;

       long   f1, f2, f3;
       short  f4, f5;

     } annot;

     struct {
       struct stmt_node *s1;
       struct stmt_node *s2;
     } concat;

     struct {
       int                    hosts;
       int                    threads;
       struct dist_symb_node *distsym;
       struct stmt_node      *body;
     } dist_annot;
       
     struct {
       struct expr_node *label;
       struct stmt_node *lstmt;
     } lab;

     /* Simple Statements */

     expr_ptr arg;
  
     /* Control Statements */

     struct {
       struct expr_node *e;
       struct stmt_node *s1;
       struct stmt_node *s2;
       short             scope;

     } control ;

     /* Declaration Statements */

     struct {
       struct type_node *tp;
       struct expr_node *name;
       struct expr_node *init;
       short             mod;

     } decl ;
  
     /* Headers */

     struct {
       struct type_node *tp;
       struct expr_node *name;
       struct expr_node *e1;
       struct expr_node *e2;
       struct stmt_node *body;
       struct stmt_node *container;
	 
       struct expr_node *fqcn;
       struct _nlist {
	   struct expr_node *cn;
	   int               n;
       }                *nlist;
       int               nllength;
       int               maxnllength;
	 
       short             scope, mod;

     } header ; 

   } u ; 

   int   comment;
   int   lineno;
   short kind;
} ;

typedef struct stmt_node *stmt_ptr;

/* **********************
   *** JAVA Modifiers ***
   ******************************************************
   *** PUBLIC/PRIVATE/PROTECTED/STATIC/ABSTRACT/FINAL ***
   *** NATIVE/SYNCHRONIZED/TRANSIENT/VOLATILE         ***
   ****************************************************** */

/* Symbolic Representations  */

enum mods { 
  PUB_B = 1,   PRI_B = 2,   PRO_B = 4,  STA_B  = 8, 
  ABS_B = 16,  FIN_B = 32,  NAT_B = 64, SYN_B = 128,
  TRA_B = 256, VOL_B = 512
} ;

/* ************************************************
   *** PROTOTYPES AND PUBLIC EXTERNAL VARIABLES ***
   ************************************************ */

/* parser.y
   ******** */

/* externals */

extern FILE *yyin;

/* prototypes */

int    yyparse(void);

/* driver.c
   ******** */

/* externals */

extern char *filename;
extern int   lineno;
extern short comments_flag;

/* prototypes */

void  javar_out(int, char *, ...);
void *make_mem(int);
void *more_mem(void *, int);
int   main(int, char *[]);

/* symbol.c
   ******** */

/* prototypes */

void     add_dummy_decl(expr_ptr);
void     det_meaning(expr_ptr, expr_ptr, short, expr_ptr);

short    activate_lab(stmt_ptr);
void     deactivate_lab(expr_ptr);
void     check_target(expr_ptr);
short    arg_match(expr_ptr, expr_ptr);

void     set_scope(short);
short    new_scope(void);
void     old_scope(void);

short    set_info_table(expr_ptr, stmt_ptr);
void     set_table(void);
void     del_info_table(void);
void     del_table(void);
symb_ptr insert_table(char *);
symb_ptr unique_table(char *);
char    *id_table(symb_ptr);
void     classify(expr_ptr, short);
void     show_names(void);

/* ast.c
   ***** */

/* externals */

extern   stmt_ptr cur_class;
extern   stmt_ptr cur_method;
extern   stmt_ptr cur_decl;
extern   expr_ptr cur_inv;

extern   char    *comment_buffer;

extern   stmt_ptr comp_unit;

/* prototypes */

void     push_header(stmt_ptr);
stmt_ptr pop_header(void);
stmt_ptr top_header(void);

void     add_comment(char *);
int      get_comment(void);

type_ptr make_primitive_type(short);
type_ptr make_c_i_type(expr_ptr);
type_ptr make_array_type(type_ptr);
type_ptr make_atype(type_ptr, expr_ptr);

expr_ptr make_literal(short, char *, long);
expr_ptr make_name(expr_ptr, symb_ptr, short);
expr_ptr make_declexpr(stmt_ptr);
expr_ptr grep_decls(stmt_ptr); 
expr_ptr make_te(short, type_ptr, expr_ptr, stmt_ptr);
expr_ptr make_cast(type_ptr, expr_ptr, expr_ptr);
expr_ptr make_thissup(short);
expr_ptr make_op(short, expr_ptr, expr_ptr);

stmt_ptr give_label(stmt_ptr, symb_ptr);
stmt_ptr give_m_t_c(short, type_ptr, stmt_ptr, short);
stmt_ptr give_body(stmt_ptr, stmt_ptr, short);

void     set_annot(symb_ptr, expr_ptr, short);
void     add_annot(expr_ptr);
stmt_ptr make_annot(expr_ptr, expr_ptr, stmt_ptr);
stmt_ptr concat(stmt_ptr, stmt_ptr);

void     set_dist_annot(symb_ptr, expr_ptr);
void     add_dist_symb(expr_ptr, short, short, expr_ptr, expr_ptr, short);
stmt_ptr make_dist_annot(void);

stmt_ptr make_simple(short,  expr_ptr);
stmt_ptr make_control(short, expr_ptr, stmt_ptr, stmt_ptr);
stmt_ptr make_decl(symb_ptr, short, expr_ptr);
stmt_ptr make_header(short, short, symb_ptr, type_ptr, short, 
                     expr_ptr, expr_ptr, stmt_ptr, int);

void     check_transfer(short);
short    equal_type(type_ptr, type_ptr);

type_ptr copy_type(type_ptr);
expr_ptr copy_expr(expr_ptr);
stmt_ptr copy_stmt(stmt_ptr);

void type_traverse(type_ptr, void (*sact)(stmt_ptr), void (*eact)(expr_ptr));
void expr_traverse(expr_ptr, void (*sact)(stmt_ptr), void (*eact)(expr_ptr));
void stmt_traverse(stmt_ptr, void (*sact)(stmt_ptr), void (*eact)(expr_ptr));

void set_ast(void);
void del_ast(void);

#define make_control1(kind, e, s1) (make_control((kind), (e), (s1), NULL))
#define make_block(s) (make_header(BLOCK_S, 0, NULL, NULL, 0, NULL, NULL, s, 0))

#define IS_INTEGRAL1(t) ((! t) || ((t) -> kind == BYTE_T)  || ((t) -> kind == CHAR_T) \
		               || ((t) -> kind == SHORT_T) || ((t) -> kind == INT_T))

#define IS_INTEGRAL2(t) ((IS_INTEGRAL1(t)) || ((t) -> kind == LONG_T))

#define IS_NUM(t)       ((IS_INTEGRAL2(t)) || ((t) -> kind == FLOAT_T) \
                                           || ((t) -> kind == DOUBLE_T) )

#define IS_BOOL(t)      ((! t) || ((t) -> kind == BOOL_T)) 

#define IS_STR(t)       ((! t) || ( ((t) -> kind == C_OR_I_T)  && \
                         ((! strcmp(id_table((t) -> u.name -> u.name.entry), \
                           "java.lang.String"))) ) )

#define IS_REF(t)       ((! t) || ((t) -> kind == C_OR_I_T)  \
			       || ((t) -> kind == ARRAY_T)) 

#define MK_STRING       make_name( make_name( make_name(NULL, \
                          insert_table("java"),   PACK_N),    \
                          insert_table("lang"),   PACK_N),    \
                          insert_table("String"), TYPE_N) 

/* unparse.c
   ********* */

/* externals */
extern FILE *output_file;

/* prototypes */

void dump_type(type_ptr, FILE *);
void dump_expr(expr_ptr, FILE *);
void dump_stmt(stmt_ptr, FILE *);

/* semantic.c
   **********  */

/* prototypes */

void semantic_validate(void);
