
/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** html.c : driver for HTML output
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

/* ********************************************************
   *** INCLUDE FILES and DEFINITIONS                    ***
   ********************************************************  */

#include <stdarg.h>
#include "global.h"

struct pack_node {
  struct pack_node *next;
  char             *name;
  FILE             *src_file;
  FILE             *toc_file;
  FILE             *tom_file;
} ;

typedef struct pack_node *pack_ptr;

/* ********************************************************
   *** EXTERNAL VARIABLES                               ***
   ******************************************************** */

/* PUBLIC
   ****** */

short html_flag = 0, in_html = 0;

/* PRIVATE
   ******* */

static pack_ptr  packages, cur_package;

static int       label;
static char     *html_env;

/* ********************************************************
   *** PRIVATE FUNCTIONS                                ***
   ******************************************************** */

/* Functions for opening W/R files 
   ******************************* */

static FILE *html_file(char *s) {
  FILE *f = NULL;
  char *b = (char *) make_mem((strlen(s) + strlen(html_env) + 2) * sizeof(char));

  sprintf(b, "%s/%s", html_env, s);

  f = fopen(b, "w");

  if (! f)
    javar_out(-1, "HTML generation TERMINATED (cannot open file %s)", b);
  fprintf(f, "\n<!-- This HTML file has been created by JAVAR -->\n\n");

  free(b);

  return f;
}

/* Show a Signature 
   **************** */

static void html_signature(expr_ptr e) {
  if (e) {
    if (e -> kind == ',') {
      html_signature(e -> u.op.lhs);
      fputc(',', cur_package -> tom_file);
      html_signature(e -> u.op.rhs);
    }
    else
      dump_type(e -> type, cur_package -> tom_file);
  }
}

/* Construct Root HTML Files 
   ************************* */

static void html_index(void) {
  FILE *file = html_file("index.html");

  fprintf(file, "<HTML><HEAD><TITLE>Index</TITLE></HEAD>\n");
  fprintf(file, "<FRAMESET ROWS=\"10%%,90%%\">\n");
  fprintf(file, "<FRAME SRC=\"top.html\" SCROLLING=\"no\" NAME=\"top\">\n");
  fprintf(file, "<FRAME SRC=\"can.html\" SCROLLING=\"no\" NAME=\"can\">\n");
  fprintf(file, "</FRAMESET>\n");
  fprintf(file, "</HTML>\n");

  fclose(file);
}

static void html_can(void) {
  FILE *file = html_file("can.html");

  fprintf(file, "<HTML><HEAD><TITLE>Canvas</TITLE></HEAD>\n");
  fprintf(file, "<FRAMESET COLS=\"70%%,30%%\">\n");
  fprintf(file, "<FRAME SRC=\"src.html\"                 NAME=\"src\">\n");
  fprintf(file, "<FRAMESET ROWS=\"50%%,50%%\">\n");
  fprintf(file, "<FRAME SRC=\"toc.html\" SCROLLING=\"yes\" NAME=\"toc\">\n");
  fprintf(file, "<FRAME SRC=\"tom.html\" SCROLLING=\"yes\" NAME=\"tom\">\n");
  fprintf(file, "</FRAMESET>\n");
  fprintf(file, "</FRAMESET>\n");
  fprintf(file, "</HTML>\n");

  fclose(file);
}

static void html_src(void) {
  FILE *file = html_file("src.html");

  fprintf(file, "<HTML><HEAD><TITLE>Source Code</TITLE></HEAD>\n");
  fprintf(file, "<BODY>Source Code</BODY>\n");
  fprintf(file, "</HTML>\n");

  fclose(file);
}

static void html_toc(void) {
  FILE *file = html_file("toc.html");

  fprintf(file, "<HTML><HEAD><TITLE>Classes</TITLE></HEAD>\n");
  fprintf(file, "<BODY>Classes</BODY>\n");
  fprintf(file, "</HTML>\n");

  fclose(file);
}

static void html_tom(void) {
  FILE *file = html_file("tom.html");

  fprintf(file, "<HTML><HEAD><TITLE>Contructors and Methods</TITLE></HEAD>\n");
  fprintf(file, "<BODY>Constructors and Methods</BODY>\n");
  fprintf(file, "</HTML>\n");

  fclose(file);
}

/* Top Level HTML file (All Packages)
   ********************************** */

static void html_top(char ocx) {
  static FILE *file = NULL;

  if (ocx == 'o') { /* open */
  
    file = html_file("top.html");

    fprintf(file, "<HTML><HEAD><TITLE>Top</TITLE></HEAD>\n");
    fprintf(file, "<BASE TARGET=\"can\">\n");
    fprintf(file, "<BODY>\n");
    fprintf(file, "<STRONG>packages:</STRONG>\n");
  }
  else if (! file) {

    javar_out(-1, "top file not open");

  }
  else if (ocx == 'c') { /* close */

    fprintf(file, "</BODY>\n");
    fprintf(file, "</HTML>\n");

    fclose(file);

    file = NULL;
  }
  else if (ocx == 'x') { /* add package */

    char *buf = (char *) make_mem((strlen(cur_package -> name) + 10) * sizeof(char));

    sprintf(buf, "%s_can.html", cur_package -> name);

    { FILE *locf = html_file(buf);
  
      fprintf(locf, "<HTML><HEAD><TITLE>Canvas</TITLE></HEAD>\n");
      fprintf(locf, "<FRAMESET COLS=\"70%%,30%%\">\n");
      fprintf(locf, "<FRAME SRC=\"%s_src.html\""
		    "SCROLLING=\"yes\" NAME=\"src\">\n", cur_package -> name);
      fprintf(locf, "<FRAMESET ROWS=\"50%%,60%%\">\n");
      fprintf(locf, "<FRAME SRC=\"%s_toc.html\""
		    "SCROLLING=\"yes\" NAME=\"toc\">\n", cur_package -> name);
      fprintf(locf, "<FRAME SRC=\"%s_tom.html\""
		    "SCROLLING=\"yes\" NAME=\"tom\">\n", cur_package -> name);
      fprintf(locf, "</FRAMESET>\n");
      fprintf(locf, "</FRAMESET>\n");
      fprintf(locf, "</HTML>\n");

      fclose(locf);
    }

    fprintf(file, " <A HREF=\"%s_can.html\">%s</A>", 
                    cur_package -> name, cur_package -> name);
    free(buf);
  }
  else 
    javar_out(-1, "incorrect invocation of top");
}

/* Local HTML files 
   **************** */

/* source code - window */

static void html_loc_src(char ocxy, char *s) {

  if (ocxy == 'o') { /* open */
    
    char *buf = (char *) make_mem((strlen(cur_package -> name) + 10) * sizeof(char));

    sprintf(buf, "%s_src.html", cur_package -> name);

    cur_package -> src_file = html_file(buf);

    fprintf(cur_package -> src_file, 
	    "<HTML><HEAD><TITLE>Source Code</TITLE></HEAD>\n");
    fprintf(cur_package -> src_file, "<BODY>\n");
    fprintf(cur_package -> src_file, 
	    "<H1>Package %s</H1>\n", cur_package -> name);
  
    free(buf);
  }
  else if (! cur_package -> src_file) {

    javar_out(-1, "source file not open");

  }
  else if (ocxy == 'c') { /* close */

    fprintf(cur_package -> src_file, "</BODY>\n");
    fprintf(cur_package -> src_file, "</HTML>\n");

    fclose(cur_package -> src_file);

    cur_package -> src_file = NULL;
  }
  else if (ocxy == 'x') { /* add class */

      fprintf(cur_package -> src_file, 
              "<STRONG><A NAME=\"LAB%i\">%s</A></STRONG>", label, s);
  }
  else if (ocxy == 'y') { /* add method */

      fprintf(cur_package -> src_file, 
              "<STRONG><A NAME=\"LAB%i\">%s</A></STRONG>", label, s);
  }
  else 
    javar_out(-1, "incorrect invocation of src");
}

/* table of classes - window */

static void html_loc_toc(char ocx, char *s) {

  if (ocx == 'o') { /* open */
    
    char *buf = (char *) make_mem((strlen(cur_package -> name) + 10) * sizeof(char));
  
    sprintf(buf, "%s_toc.html", cur_package -> name);

    cur_package -> toc_file = html_file(buf);

    fprintf(cur_package -> toc_file, 
	    "<HTML><HEAD><TITLE>Classes</TITLE></HEAD>\n");
    fprintf(cur_package -> toc_file, 
	    "<BASE TARGET=\"tom\">\n");
    fprintf(cur_package -> toc_file, 
	    "<BODY>\n");

    free(buf);
  }
  else if (! cur_package -> toc_file) {

    javar_out(-1, "toc file not open");

  }
  else if (ocx == 'c') { /* close */

    fprintf(cur_package -> toc_file, "</BODY>\n");
    fprintf(cur_package -> toc_file, "</HTML>\n");

    fclose(cur_package -> toc_file);

    cur_package -> toc_file = NULL;
  }
  else if (ocx == 'x') { /* add class */
    
    char *buf = (char *) make_mem((strlen(cur_package -> name) + 10) * sizeof(char));

    sprintf(buf, "%s_tom.html", cur_package -> name);

    fprintf(cur_package -> toc_file, 
            "<TT><A HREF=\"%s#LAB%i\"><STRONG>%s</STRONG></A></TT><BR>\n", buf, label, s);

    free(buf);
  }
  else 
    javar_out(-1, "incorrect invocation of toc");
}

/* table of methods - window */

static void html_loc_tom(char ocxy, char *s, expr_ptr e) {

  if (ocxy == 'o') { /* open */
    
    char *buf = (char *) make_mem((strlen(cur_package -> name) + 10) * sizeof(char));
  
    sprintf(buf, "%s_tom.html", cur_package -> name);

    cur_package -> tom_file = html_file(buf);

    fprintf(cur_package -> tom_file, 
	    "<HTML><HEAD><TITLE>Contructors and Methods</TITLE></HEAD>\n");
    fprintf(cur_package -> tom_file, 
	    "<BASE TARGET=\"src\">\n");
    fprintf(cur_package -> tom_file, 
	    "<BODY>\n");

    free(buf);
  }
  else if (! cur_package -> tom_file) {

    javar_out(-1, "tom file not open");

  }
  else if (ocxy == 'c') { /* close */

    fprintf(cur_package -> tom_file, "<HR>\n</BODY>\n");
    fprintf(cur_package -> tom_file, "</HTML>\n");

    fclose(cur_package -> tom_file);

    cur_package -> tom_file = NULL;
  }
  else if (ocxy == 'x') { /* add class */

    char *buf = (char *) make_mem((strlen(cur_package -> name) + 10) * sizeof(char));

    sprintf(buf, "%s_src.html", cur_package -> name);

    fprintf(cur_package -> tom_file, 
	    "<HR>\n<TT><A NAME=\"LAB%i\"HREF=\"%s#LAB%i\">"
	    "<STRONG>%s</STRONG></A></TT><BR>\n", label, buf, label, s);

    free(buf);
  }
  else if (ocxy == 'y') { /* add method */

    char *buf = (char *) make_mem((strlen(cur_package -> name) + 10) * sizeof(char));

    sprintf(buf, "%s_src.html", cur_package -> name);

    fprintf(cur_package -> tom_file, 
	    "<FONT SIZE=-1><TT><A HREF=\"%s#LAB%i\">%s</A>(", buf, label, s);
    html_signature(e);

    fprintf(cur_package -> tom_file, ")</TT></FONT><BR>\n");

    free(buf);
  }
  else 
    javar_out(-1, "incorrect invocation of tom");
}

/* Set Package
   *********** */

static void html_set_package(char *pack_name) {

  /* Search */

  for (cur_package = packages; cur_package; cur_package = cur_package -> next) 
    if (strcmp(pack_name, cur_package -> name) == 0)
      break;

  if (! cur_package) { /* new */

    cur_package = (pack_ptr) make_mem( sizeof(struct pack_node) );
    cur_package -> next = packages;
    cur_package -> name = (char *)strdup(pack_name);
  
    html_top('x');
  
    html_loc_src('o', NULL);
    html_loc_toc('o', NULL);
    html_loc_tom('o', NULL, NULL);

    /* Hook to List */

    packages = cur_package;
  }
}

/* Detetermine Package (TRAVERSAL ROUTINE)
   *************************************** */

static void find_pack(stmt_ptr s) {
  if (s -> kind == PCK_S)
    html_set_package( id_table(s -> u.arg -> u.name.entry) );
}

/* ********************************************************
   *** PUBLIC FUNCTIONS                                 ***
   ******************************************************** */

/* Project open/close 
   ****************** */

void html_open_project(void) {

  packages = NULL;
  label    = 0;

  if (! (html_env = getenv("JAVAR_HTML")))
    html_env = ".";

  javar_out(1, "HTML project placed in %s/*.html", html_env);

  html_index();
  html_can();
  html_src();
  html_toc();
  html_tom();

  html_top('o');
}

void html_close_project(void) {

  html_top('c');

  /* Close all files and free package nodes */

  cur_package = packages;

  while (cur_package) {
    
    pack_ptr next = cur_package -> next;

    html_loc_src('c', NULL);
    html_loc_toc('c', NULL);
    html_loc_tom('c', NULL, NULL);

    free(cur_package -> name);
    free(cur_package);

    cur_package = next;
  }

  packages = NULL;

  javar_out(1, "HTML project closed");
}

/* Open/Close Source File
   ********************** */

void html_process_file(char *src_file) {
 
  cur_package  = NULL;

  /* Determine Package */

  stmt_traverse(comp_unit, find_pack, NULL);
  
  if (! cur_package) 
    html_set_package("default@");

  /* Append Compilation Unit source file to Package source file */

  fprintf(cur_package -> src_file, 
          "\n<HR>\n<H2>Source File \"%s\"</H2>\n<HR>\n", src_file);

  in_html = 1;

  fprintf(cur_package -> src_file, "<PRE>\n");
  dump_stmt(comp_unit, cur_package -> src_file);
  fprintf(cur_package -> src_file, "</PRE>\n");

  in_html = 0;
}

/* Add class or method to current package
   ************************************** */

void html_add_class(expr_ptr nm) {

  char *class = id_table(nm -> u.name.entry);

  if (! cur_package) 
    javar_out(-1, "class outside package");

  html_loc_src('x', class);
  html_loc_toc('x', class);
  html_loc_tom('x', class, NULL);

  label++;
}

void html_add_method(expr_ptr nm, expr_ptr e) {

  char *method = id_table(nm -> u.name.entry);

  if (! cur_package) 
    javar_out(-1, "method outside package");
 
  html_loc_src('y', method);  
  html_loc_tom('y', method, e);

  label++;
}

