

%{

/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** parser.y : yacc definitions of Java
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

/* This file has been modified from the original to support
 * Java 1.1 language.
 *
 *     Fabian Breg.
 */

    
/* ********************************************************
   *** INCLUDE FILES and DEFINITIONS                    ***
   ********************************************************  */

#include "global.h"

/* ********************************************************
   *** EXTERNAL VARIABLES                               ***
   ******************************************************** */

/* PRIVATE
   ******* */

static symb_ptr formid;
static symb_ptr methid;
static symb_ptr locid;
static expr_ptr locf;
static stmt_ptr locs;
static expr_ptr locd1, locd2; 
static short    in_l;
static short    in_formal = 0;
static int      ptype;
 
%}

%union {
  symb_ptr sym;
  type_ptr type;
  expr_ptr expr;
  stmt_ptr stmt;
  short    val;
  int      str;
}

%start compilation_unit

     /* TOKENS (TERMINALS)
        ****************** */

%token ABSTRACT_     BOOLEAN_    BREAK_    BYTE_    CASE_      CATCH_
%token CHAR_         CLASS_      CONTINUE_ DEFAULT_ DO_        DOUBLE_ 
%token ELSE_         EXTENDS_    FINAL_    FINALLY_ FLOAT_     FOR_  
%token IF_           IMPLEMENTS_ IMPORT_   INSTOF_  INT_       INTERFACE_
%token LONG_         NATIVE_     NEW_      PACKAGE_ PRIVATE_   PROTECTED_
%token PUBLIC_       RETURN_     SHORT_    STATIC_  SUPER_     SWITCH_ 
%token SYNCHRONIZED_ THIS_       THROW_    THROWS_  TRANSIENT_ TRY_ 
%token VOID_         VOLATILE_   WHILE_

%token <val>  EQ_     LE_     GE_     NE_     AND_    OR_  
%token <val>  PP_     MM_     LL_     RR_     RRR_ 
%token <val>  PIS_    MINIS_  MULIS_  DIS_    ANDIS_  ORIS_ 
%token <val>  UIS_    IDIVIS_ LLIS_   RRIS_   RRRIS_  

%token <expr> LITERAL_
%token <sym>  ID_

%token        JAVPAR_ JAVINV_ JAVPOST_ JAVWAIT_ JAVDIST_ JAVEND_ 
%token        NONSHARED_ SHARED_ UNMODIFIED_ MOD_IN_ MOD_OUT_ MOD_INOUT_
%token        BLOCK_ CYCLIC_

       /* NON-TERMINALS
          ************* */

%type <sym>  constructordeclarator 

%type <val>  opt_mod modifiers modifier                
%type <val>  variabledeclaratorid methoddeclarator assignop
%type <val>  data_access method_access data_dist
%type <val>  opt_final
    
%type <type> type primitivetype numerictype integraltype
%type <type> fptype referencetype c_i_type classtype 
%type <type> interfacetype arraytype opt_super

%type <expr> name qualifiedname opt_interfaces interfacetypelist 
%type <expr> variableinitializer variableinitializers arrayinitializer 
%type <expr> stmtexp stmtexplist opt_forinit opt_forupdate 
%type <expr> primary primarynonewarray classinst_exp opt_argl 
%type <expr> argumentlist arrayinst_exp dim_part dimexps 
%type <expr> dims fieldaccess superpart methodinvocation arrayaccess postfixexp
%type <expr> postincrementexp postdecrementexp unaryexp preincrementexp
%type <expr> predecrementexp unaryexpnotplusminus castexp 
%type <expr> additiveexp shiftexp relationalexp equalityexp andexp 
%type <expr> inclusiveorexp conditionalandexp conditionalorexp 
%type <expr> assignmentexp assignment lefthandside exp opt_exp opt_extends 
%type <expr> opt_formal formalparameterlist formalparameter classtypelist 
%type <expr> exclusiveorexp extendsinterfaces conditionalexp multiplicativeexp
    
%type <stmt> variabledeclarators variabledeclarator localvariabledecl
%type <stmt> opt_pack opt_import opt_type importdecls importdecl 
%type <stmt> typedecls typedecl classdecl classbody classbodydecls 
%type <stmt> classbodydecl classmemberdecl staticinitializer constructordecl
%type <stmt> constructorbody fielddecl methoddecl abstractmethoddecl 
%type <stmt> methodheader interfacedecl interfacebody interfacememberdecls 
%type <stmt> interfacememberdecl constantdecl block opt_blockstmt blockstmts 
%type <stmt> localvariablestmt stmt stmtnoshortif stmtwithouttrailingsubstmt
%type <stmt> emptystmt labeledstmt labeledstmtnoshortif expstmt breakstmt
%type <stmt> ifthenstmt ifthenelsestmt ifthenelsestmtnoshortif switchstmt
%type <stmt> switchblock switchblockstmtgroups switchblockstmtgroup 
%type <stmt> switchlabel whilestmt whilestmtnoshortif dostmt forstmt 
%type <stmt> continuestmt returnstmt throwstmt synchronizedstmt trystmt 
%type <stmt> finally_block catches catchclause blockstmt forstmtnoshortif
%type <stmt> switchlabels javar_annot javar_stmt javar_block 

/* JAVA 1.1 */
%type <stmt> classdecl_in interfacedecl_in classdecl_in_c opt_classbody

%type <stmt> classbodydecl_c interfacememberdecl_c 
%type <str>  gc

%%

     /* **********************************
        *** Compilation Unit Structure ***
        ********************************** */

compilation_unit : opt_pack opt_import opt_type  

                   { comp_unit = concat($1, concat($2, $3)); }
                 ;                               

     /* PACKAGE stmt */

opt_pack         : gc PACKAGE_ name ';'      { $3 -> u.name.info = PACK_N;
                                               $$ = make_simple(PCK_S, $3); 
					       $$ -> comment = $1;
                                              } 
                 |                           { $$ = NULL; }
                 ;

    /* IMPORT stmts */

opt_import       : importdecls                 
                 |                           { $$ = NULL; }
                 ;

importdecls      : importdecl
                 | importdecls importdecl    { $$ = concat($1, $2); }
                 ;

importdecl       : gc IMPORT_ name ';'         

                    { $3 -> u.name.info = TYPE_N; 
                      $$ = make_simple(STIMP_S, $3); 
		      $$ -> comment = $1;
		    }

                 | gc IMPORT_ name '.' '*' ';'  
   
                    { $3 -> u.name.info = PACK_N; 
                      $$ = make_simple(IMPOD_S, $3); 
		      $$ -> comment = $1;
		    }
                 ;

     /* Type Declarations */

opt_type         : typedecls
                 |                           { $$ = NULL; }
                 ;

typedecls        : typedecl
                 | typedecls typedecl        { $$ = concat($1, $2); }
                 ;

typedecl         : classdecl                     
                 | interfacedecl                 
		 | ';'                       { $$ = NULL;
					       javar_out(2, "stray semicolon deleted");
                                             }
                 | error err_sep                 
                    
                   { /* Skip Block */

                     yyclearin;  /* Discard lookahead */

                     javar_out(2, "resuming parse at next "
                                  "class or interface");
                     $$ = NULL; 
                   }
                 ;

     /* Types: Primitive and Reference
        ****************************** */

type             : primitivetype      
                 | referencetype     
                 ;

primitivetype    : numerictype
                 | BOOLEAN_           { ptype = BOOLEAN_; $$ = make_primitive_type(BOOL_T); }
                 ;

numerictype      : integraltype
                 | fptype              
                 ;

integraltype     : BYTE_              { ptype = BYTE_; $$ = make_primitive_type(BYTE_T);   } 
                 | SHORT_             { ptype = SHORT_; $$ = make_primitive_type(SHORT_T);  } 
                 | INT_               { ptype = INT_; $$ = make_primitive_type(INT_T);    } 
                 | LONG_              { ptype = LONG_; $$ = make_primitive_type(LONG_T);   } 
                 | CHAR_              { ptype = CHAR_; $$ = make_primitive_type(CHAR_T);   } 
                 ;

fptype           : FLOAT_             { ptype = FLOAT_; $$ = make_primitive_type(FLOAT_T);  } 
                 | DOUBLE_            { ptype = DOUBLE_; $$ = make_primitive_type(DOUBLE_T); } 
                 ;

referencetype    : c_i_type
                 | arraytype
                 ;

classtype        : c_i_type
                 ;

interfacetype    : c_i_type
                 ;

c_i_type         : name           { $$ = make_c_i_type($1); }
                 ;

/* arraytype : primitivetype '[' ']' { $$ = make_array_type($1); } */
/*           | name          '[' ']' { $$ = make_array_type(make_c_i_type($1)); } */
/*           | arraytype     '[' ']' { $$ = make_array_type($1); } */
/*           ; */
arraytype : primitivetype dims { $$ = make_atype($1, $2); }
          | name          dims { $$ = make_atype(make_c_i_type($1), $2); }
          ;

     /* Names 
        ***** */

name             : ID_             { $$ = make_name(NULL, $1, AMBI_N); }
                 | qualifiedname   
                 ;

qualifiedname    : name  '.'  ID_  { $$ = make_name($1, $3, AMBI_N); }
                 ;

     /* Modifiers
        ********* */

opt_mod          : modifiers            
                 |                      { $$ = 0; }
                 ;

modifiers        : modifier
                 | modifiers modifier   

                     { /* Check for Repeated Modifier */

                       if ($1 & $2)
                         javar_out(1, "repeated modifier ignored"); 
                         
                       $$ = ($1 | $2); 
                     }
                 ;

modifier         : PUBLIC_              { $$ = PUB_B; }
                 | PROTECTED_           { $$ = PRO_B; }
                 | PRIVATE_             { $$ = PRI_B; }
                 | STATIC_              { $$ = STA_B; }
                 | ABSTRACT_            { $$ = ABS_B; }
                 | FINAL_               { $$ = FIN_B; }
                 | NATIVE_              { $$ = NAT_B; }
                 | SYNCHRONIZED_        { $$ = SYN_B; }
                 | TRANSIENT_           { $$ = TRA_B; }
                 | VOLATILE_            { $$ = VOL_B; }
                 ;

     /* ************************
        **** Class Structure ***
        ************************ */

classdecl         : opt_mod CLASS_ gc ID_ opt_super opt_interfaces
                    {
			stmt_ptr cd = make_header(CLASS_S, $1, $4, $5, 0, $6, NULL, NULL, 0);
			push_header(cd);
		    }
                    classbody     
                    {
			$$ = pop_header();
			$$ -> u.header.body = $8;
			$$ -> comment = $3;
 		    }
                    ;

    /* JAVA 1.1 */
classdecl_in      : opt_mod CLASS_ ID_ opt_super opt_interfaces
                    {
			push_header(make_header(CLASS_S, $1, $3, $4, 0, $5, NULL, NULL, 1));
                    }
                    classbody     
                    {
			$$ = pop_header();
			$$ -> u.header.body = $7;
		    }
                  ;

classdecl_in_c    : opt_mod CLASS_ gc ID_ opt_super opt_interfaces
                    {
			push_header(make_header(CLASS_S, $1, $4, $5, 0, $6, NULL, NULL, 1));
                    }
                    classbody     
                    {
			$$ = pop_header();
			$$ -> u.header.body = $8;
			$$ -> comment = $3;
			javar_out(1, "local class found");
 		    }
                    ;

opt_super         : EXTENDS_ classtype            { $$ = $2;   }
                  |                               { $$ = NULL; }
                  ;

opt_interfaces    : IMPLEMENTS_ interfacetypelist { $$ = $2;   } 
                  |                               { $$ = NULL; }
                  ;

interfacetypelist : interfacetype                { $$ = make_te('t', $1, NULL, NULL); }
                  | interfacetypelist ',' 
                        interfacetype            { $$ = make_te('t', $3, $1, NULL);   }
                  ;

classbody         : '{' classbodydecls '}'       { $$ = $2;   }
                  | '{'                '}'       { $$ = NULL; }
                  ;

classbodydecls    : classbodydecl_c
                  | classbodydecls classbodydecl_c { $$ = concat($1, $2); }
                  ;

classbodydecl_c   : gc classbodydecl               
			   { $$ = $2; 
			     if ($$) 
			       $$ -> comment = $1; 
			   }
		  ;

classbodydecl     : classmemberdecl
                  | staticinitializer
                  | constructordecl
                  | block
		  | ';'          
		  
		     {  $$ = NULL;
		        javar_out(2, "stray semicolon deleted");
                     }
                  | error err_sep
                    
                    {  /* Skip Block */

                       yyclearin;  /* Discard lookahead */

                       javar_out(2, "resuming parse at next member of class");

                       $$ = NULL; 
                    }
                  ;

    /* JAVA 1.1 */
classmemberdecl   : fielddecl
                  | methoddecl
                  | interfacedecl_in
                  | classdecl_in
                  ;

     /* *************************
        **** Member Structure ***
        ************************* */

     /* Field Declarations 
        ****************** */

fielddecl            : opt_mod type variabledeclarators ';'

                          { $$ = give_m_t_c($1, $2, $3, FIELD_N); }
                     ;

variabledeclarators  : variabledeclarator
                     | variabledeclarators ',' 
                       variabledeclarator         { $$ = concat($1, $3); }
                     ;

variabledeclarator   : variabledeclaratorid
 
                         { $$ = make_decl(locid, $1, NULL); }

                     | variabledeclaratorid '=' variableinitializer 

                         { $$ = make_decl(locid, $1, $3); }
                     ;

variabledeclaratorid : ID_
                       {
			   if(in_formal)
			       formid = $1;
			   else
			       locid = $1;
			   $$ = 0;
		       }
                     | variabledeclaratorid '[' ']'

                        { /* C-like array declaration */

                          $$ = $1 + 1;
                        }
                     ;

variableinitializer  : exp
                     | arrayinitializer  
                     ;

     /* Method Declarations 
        ******************* */

methoddecl    : methodheader '{'
                {
		    push_header($1);
		}
                opt_blockstmt
                {
		    pop_header();
                }
  	        '}'
                {
		    $$ = give_body($1, $4, 1);
		} 
              | methodheader ';'                   { $$ = give_body($1, NULL, 0); }
              ;

methodheader  : opt_mod type methoddeclarator THROWS_ classtypelist 
      
                  { $$ = make_header(METHOD_S, $1, methid, $2, $3, locf, $5, NULL, 0); } 

               | opt_mod type methoddeclarator                   

                  { $$ = make_header(METHOD_S, $1, methid, $2, $3, locf, NULL, NULL, 0); } 

               | opt_mod VOID_ methoddeclarator THROWS_ classtypelist 

                  { $$ = make_header(METHOD_S, $1, methid, 
                            make_primitive_type(VOID_T), $3, locf, $5, NULL, 0); 
                  } 

               | opt_mod VOID_ methoddeclarator  

                  { $$ = make_header(METHOD_S, $1, methid, 
                           make_primitive_type(VOID_T), $3, locf, NULL, NULL, 0); 
                  } 
               ;

methoddeclarator    : ID_ '('                    
                              opt_formal 
                          ')'                    { methid = $1; 
                                                   locf  = $3; 
                                                   $$    = 0; 
                                                 }
                    | methoddeclarator '[' ']'          
                         
                         { /* C-like declaration in a method */
                                 
                           $$ = $1 + 1;         
                         }
                    ;

opt_formal          : formalparameterlist
                    |                           { $$ = NULL; }
                    ;

formalparameterlist : formalparameter
                    | formalparameterlist ',' 
                      formalparameter           { $$ = make_op(',', $1, $3); }
                    ;

formalparameter     : opt_final type { in_formal = 1; }
                      variabledeclaratorid   
                      {
			  in_formal = 0;
			  $$ = make_declexpr(give_m_t_c($1, $2, 
							make_decl(formid, $4, NULL), LOCPAR_N)); 
		      }
                    ;

opt_final           : FINAL_ { $$ = FIN_B; } | { $$ = 0; } ;

classtypelist       : classtype           { $$ = make_te('t', $1, NULL, NULL); }
                    | classtypelist ',' 
                      classtype           { $$ = make_te('t', $3, $1, NULL);   }
                    ;

     /* Static Initializer 
        ****************** */

staticinitializer : STATIC_
                    {
			push_header(make_header(SINIT_S, STA_B, NULL, NULL, 0, NULL, NULL, NULL, 0));
		    }
                    '{'	opt_blockstmt '}'
		    {
			$$ = pop_header();
			$$ -> u.header.body = $4;
		    }
		    
                  ;

     /* Constructor Declarations 
        ************************ */

constructordecl : opt_mod constructordeclarator THROWS_ classtypelist
                  {
		      push_header(make_header(CONS_S, $1, $2, NULL, 0, locf, $4, NULL, 0));
		  }
                  '{' constructorbody '}'
		  {
		      $$ = pop_header();
		      $$ -> u.header.body = $7;
		  }
		| opt_mod constructordeclarator
		  {
		      push_header(make_header(CONS_S, $1, $2, NULL, 0, locf, NULL, NULL, 0));
		  }
		  '{' constructorbody '}'
		  {
		      $$ = pop_header();
		      $$ -> u.header.body = $5;
		  }
                ;

constructordeclarator : ID_ '(' opt_formal ')'             { $$ = $1; locf = $3; }
                      ;

    /* JAVA 1.1 */
constructorbody : THIS_  '(' opt_argl ')' ';'  opt_blockstmt
         
                     { $$ = concat(make_simple(EXP_S, 
                         make_op('i', make_thissup(THIS_), $3)), $6); }

                 | SUPER_ '(' opt_argl ')'  ';' opt_blockstmt

                     { $$ = concat(make_simple(EXP_S,
                        make_op('i', make_thissup(SUPER_), $3)), $6); }

                 | primary '.' SUPER_ '(' opt_argl ')'  ';' opt_blockstmt

                     { $$ = concat(make_simple(EXP_S,
			make_op('i', make_op('.', $1, make_thissup(SUPER_)), $5)), $8); }

                 | opt_blockstmt
                 ;

     /* ****************************
        **** Interface Structure ***
        **************************** */

interfacedecl : opt_mod INTERFACE_ gc ID_ opt_extends
                {
		    push_header(make_header(INTERF_S, ($1 | ABS_B), $4, NULL, 0, $5, NULL, NULL, 0));
		}
                interfacebody
                {
		    $$ = pop_header();
		    $$ -> u.header.body = $7;
                    $$ -> comment = $3;
		}
	      ;

    /* JAVA 1.1 */
interfacedecl_in : opt_mod INTERFACE_ ID_ opt_extends
                   {
		       push_header(make_header(INTERF_S, ($1 | ABS_B), $3, NULL, 0, $4, NULL, NULL, 1));
		   }
                   interfacebody
                   {
		       $$ = pop_header();
		       $$ -> u.header.body = $6;
		   }
		 ;

opt_extends           : extendsinterfaces
                      |                        { $$ = NULL; }
                      ;

extendsinterfaces     : EXTENDS_ interfacetype { $$ = make_te('t', $2, NULL, NULL); }
                      | extendsinterfaces ','
                        interfacetype          { $$ = make_te('t', $3, $1, NULL);   }
                      ;

interfacebody         : '{' interfacememberdecls '}'  { $$ = $2;  }
                      | '{' '}'                       { $$ = NULL; }
                      ;

interfacememberdecls  : interfacememberdecl_c                       
                      | interfacememberdecls 
                        interfacememberdecl_c    { $$ = concat($1, $2); }
                      ;

interfacememberdecl_c : gc interfacememberdecl
       
			  { $$ = $2;
			    if ($$) 
			      $$ -> comment = $1;
                          }
		      ;

    /* JAVA 1.1 */
interfacememberdecl   : constantdecl           
                      | abstractmethoddecl
                      | interfacedecl_in
                      | classdecl_in
		      | ';'          
		       
			 {  $$ = NULL;
		            javar_out(2, "stray semicolon deleted");
                         }

                      | error err_sep
                    
                        {  /* Skip Block */
                          
                           yyclearin;  /* Discard lookahead */
                           javar_out(2, "resuming parse at next "
                                        "member of interface");
                           $$ = NULL;
                        }
                      ;

constantdecl          : fielddecl

                         { $1 -> u.decl.mod |= (PUB_B|STA_B|FIN_B); $$ = $1; }
                      ;

abstractmethoddecl    : methodheader ';'

                         { $1 -> u.header.mod |= (PUB_B|ABS_B); 
                            $$ = give_body($1, NULL, 0); 
                         }
                      ;

     /* **************
        *** Arrays ***
        ************** */

arrayinitializer      : '{'  '}'      { $$ = make_op('{', NULL, NULL); }
                      | '{' ','  '}'  { $$ = make_op('{', NULL, NULL); }

                      | '{' variableinitializers     '}' 
                        
                             { $$ = make_op('{', NULL, $2);   }

                      | '{' variableinitializers ',' '}' 
                        
                             { $$ = make_op('{', NULL, $2);   }
                      ;

variableinitializers  : variableinitializer 
                      | variableinitializers ',' variableinitializer 
                      
                             { $$ = make_op(',', $1, $3); }
                      ;

     /* *****************************
        *** Blocks and Statements ***
        ***************************** */

block                 : '{' opt_blockstmt '}'      { $$ = make_block($2); } 
                      ;

opt_blockstmt         : blockstmts
                      |                            { $$ = NULL; }
                      ;

blockstmts            : blockstmt
                      | blockstmts blockstmt       { $$ = concat($1, $2); }
                      ;
    /* JAVA 1.1 */
blockstmt             : localvariablestmt
                      | classdecl_in_c
                      | stmt
                      ;

localvariablestmt     : localvariabledecl ';'        
                      ;

localvariabledecl     : type variabledeclarators    

                          { $$ = give_m_t_c(0, $1, $2, LOCPAR_N);  }

                      | FINAL_ type variabledeclarators

                          { $$ = give_m_t_c(FIN_B, $2, $3, LOCPAR_N);  }

                      ;

stmt                  : stmtwithouttrailingsubstmt
                      | labeledstmt
                      | ifthenstmt
                      | ifthenelsestmt
                      | whilestmt
                      | forstmt
                      | javar_stmt
                      ;

stmtnoshortif         : stmtwithouttrailingsubstmt
                      | labeledstmtnoshortif
                      | ifthenelsestmtnoshortif
                      | whilestmtnoshortif
                      | forstmtnoshortif
                      ;

stmtwithouttrailingsubstmt : block                        
			   | javar_block
                           | emptystmt
                           | expstmt
                           | switchstmt
                           | dostmt
                           | breakstmt
                           | continuestmt
                           | returnstmt
                           | synchronizedstmt
                           | throwstmt
                           | trystmt
                           ;

emptystmt                  : ';'             { $$ = make_simple(EMPTY_S, NULL); }
                           ;

labeledstmt                : ID_ ':' 
                             stmt            { $$ = give_label($3, $1); }
                           ;

labeledstmtnoshortif       : ID_ ':' 
                             stmtnoshortif   { $$ = give_label($3, $1); }
                           ;

expstmt                    : stmtexp ';'     { $$ = make_simple(EXP_S, $1); }
                           ;

stmtexp                    : assignment
                           | preincrementexp
                           | predecrementexp
                           | postincrementexp
                           | postdecrementexp
                           | methodinvocation
                           | classinst_exp
                           ;

ifthenstmt                 : IF_ '(' exp ')' 
                             stmt           
                                
                              { $$ = make_control1(IF_S, $3, $5); }
                           ;

ifthenelsestmt             : IF_ '(' exp ')' 
                             stmtnoshortif 
                             ELSE_ 
                             stmt           
                                
                              { $$ = make_control(IF_S, $3, $5, $7); }
                           ; 

ifthenelsestmtnoshortif    : IF_ '(' exp ')' 
                             stmtnoshortif 
                             ELSE_ 
                             stmtnoshortif  
                                
                              { $$ = make_control(IF_S, $3, $5, $7); } 
                           ;

switchstmt            : SWITCH_ '(' exp ')' switchblock         
                            
                               { $$ = make_control1(SWITCH_S, $3, make_block($5)); } 
                      ;

switchblock           : '{' '}'                       { $$ = NULL; }
                      | '{' switchblockstmtgroups '}' { $$ = $2;   }
                      | '{' switchlabels '}'          { $$ = $2;   }
                      | '{' switchblockstmtgroups 
                            switchlabels 
                         '}'                          { $$ = concat($2, $3); } 
                      ;

switchblockstmtgroups : switchblockstmtgroup
                      | switchblockstmtgroups 
                        switchblockstmtgroup      { $$ = concat($1, $2); }
                      ;

switchblockstmtgroup  : switchlabels blockstmts   { $$ = concat($1, $2); }
                      ;

switchlabels          : switchlabel
                      | switchlabels switchlabel  { $$ = concat($1, $2); }
                      ;

switchlabel           : CASE_    exp ':'   { $$ = make_simple(CASE_S, $2);  }
                      | DEFAULT_     ':'   { $$ = make_simple(DEF_S, NULL); }
                      ;

whilestmt             : WHILE_ '(' exp ')' 
                        stmt 

                          { $$ = make_control1(WHILE_S, $3, $5); }
                      ;

whilestmtnoshortif    : WHILE_ '(' exp ')' 
                        stmtnoshortif 

                          { $$ = make_control1(WHILE_S, $3, $5); }
                      ;

dostmt                : DO_ stmt 
                        WHILE_ '(' exp ')' ';' 

                          { $$ = make_control1(DOWHILE_S, $5, $2); }
                      ;

forstmt               : javar_annot FOR_  
                            '('
                               opt_forinit ';' 
                               opt_exp     ';' 
                               opt_forupdate
                            ')'           
                        stmt       

                         { $$ = make_control(FOR_S, make_op(';', $4, 
                                 make_op(';', $6, $8)), $10, $1); 
                         } 
                      ;

forstmtnoshortif      : javar_annot FOR_   
                            '(' 
                               opt_forinit ';' 
                               opt_exp     ';' 
                               opt_forupdate
                            ')'      
                        stmtnoshortif 

                         { $$ = make_control(FOR_S, make_op(';', $4, 
                                 make_op(';', $6, $8)), $10, $1);
                         } 
                      ;

opt_forinit           : stmtexplist       
                      | localvariabledecl { $$ = grep_decls($1); /* bug fix */ }
                      |                   { $$ = NULL; }
                      ;

opt_forupdate         : stmtexplist
                      |                 { $$ = NULL; }
                      ;

stmtexplist           : stmtexp                   
                      | stmtexplist ',' 
                        stmtexp         { $$ = make_op(',', $1, $3); } 
                      ;

breakstmt             : BREAK_ ID_ ';'  

                       { $$ = make_simple(BRK_S, make_name(NULL, $2, LABEL_N)); }

                      | BREAK_     ';'         { $$ = make_simple(BRK_S, NULL); }
                      ;

continuestmt          : CONTINUE_ ID_   ';' 

                       { $$ = make_simple(CONT_S, make_name(NULL, $2, LABEL_N)); }

                      | CONTINUE_       ';'    { $$ = make_simple(CONT_S, NULL); }
                      ;

returnstmt            : RETURN_ opt_exp ';'    { $$ = make_simple(RET_S, $2);    }
                      ;

throwstmt             : THROW_ exp ';'         { $$ = make_simple(THROW_S, $2);  }
                      ;

synchronizedstmt      : SYNCHRONIZED_ '(' exp ')' block

                          { $$ = make_control1(SYNC_S, $3, $5); }
                      ;

trystmt               : TRY_ block catches 

                          { $$ = make_control(TRY_S, NULL, $2, $3); }

                      | TRY_ block catches finally_block
              
                          { $$ = make_control(TRY_S, NULL, $2, concat($3, $4)); }

                      | TRY_ block finally_block
 
                          { $$ = make_control(TRY_S, NULL, $2, $3); }  
                      ;

finally_block         : FINALLY_ block   { $$ = make_control1(FIN_S, NULL, $2); }
                      ;

catches               : catchclause
                      | catches catchclause  { $$ = concat($1, $2); }
                      ;

catchclause           : CATCH_ '(' formalparameter ')'
                        {
			    push_header(make_header(CATCH_S, 0, NULL, NULL, 0, $3, NULL, NULL, 0));
			}
                        '{' opt_blockstmt '}'
			{
			    $$ = pop_header();
			    $$ -> u.header.body = $7;
			}
                      ;

 /* *******************
    *** Expressions ***
    ******************* */

primary             : primarynonewarray
                    | arrayinst_exp
                    ;

    /* JAVA 1.1 */
primarynonewarray   : LITERAL_
                    | THIS_            { $$ = make_thissup(THIS_); }
                    | name '.' THIS_   { $$ = make_op('.', $1, make_thissup(THIS_)); }
                    | '(' exp ')'      { $$ = $2; }
                    | VOID_ '.' CLASS_
                      {
			  $$ = make_op('.',
				       make_name(NULL, insert_table("void"), 0),
				       make_name(NULL, insert_table("class"), 0));
		      }
                    | primitivetype '.' CLASS_
                      {
/* 			  if(ptype == BOOLEAN_) */
/* 			      $$ = make_te('l', */
/* 				      make_name(NULL, insert_table("boolean"), 0), */
/* 				      make_name(NULL, insert_table("class"), 0)); */
/* 			  else if(ptype == BYTE_) */
/* 			      $$ = make_op('.', */
/* 				      make_name(NULL, insert_table("byte"), 0), */
/* 				      make_name(NULL, insert_table("class"), 0)); */
/* 			  else if(ptype == CHAR_) */
/* 			      $$ = make_op('.', */
/* 				      make_name(NULL, insert_table("char"), 0), */
/* 				      make_name(NULL, insert_table("class"), 0)); */
/* 			  else if(ptype == SHORT_) */
/* 			      $$ = make_op('.', */
/* 				      make_name(NULL, insert_table("short"), 0), */
/* 				      make_name(NULL, insert_table("class"), 0)); */
/* 			  else if(ptype == INT_) */
/* 			      $$ = make_op('.', */
/* 				      make_name(NULL, insert_table("int"), 0), */
/* 				      make_name(NULL, insert_table("class"), 0)); */
/* 			  else if(ptype == LONG_) */
/* 			      $$ = make_op('.', */
/* 				      make_name(NULL, insert_table("long"), 0), */
/* 				      make_name(NULL, insert_table("class"), 0)); */
/* 			  else if(ptype == FLOAT_) */
/* 			      $$ = make_op('.', */
/* 				      make_name(NULL, insert_table("float"), 0), */
/* 				      make_name(NULL, insert_table("class"), 0)); */
/* 			  else if(ptype == DOUBLE_) */
/* 			      $$ = make_op('.', */
/* 				      make_name(NULL, insert_table("double"), 0), */
/* 				      make_name(NULL, insert_table("class"), 0)); */
			  $$ = make_te('l',
				       $1,
				       make_name(NULL, insert_table("class"), 0),
				       NULL);
		      }
                    | name '.' CLASS_
                      {
			  $$ = make_te('l',
				       make_c_i_type($1),
				       make_name(NULL, insert_table("class"), 0),
				       NULL);
		      }
                    | arraytype '.' CLASS_
                      {
			  $$ = make_te('l',
				       $1,
				       make_name(NULL, insert_table("class"), 0),
				       NULL);
		      }
                    | classinst_exp
                    | fieldaccess
                    | methodinvocation
                    | arrayaccess
                    ;

/* JAVA 1.1 */

classinst_exp  : NEW_ classtype '(' opt_argl ')' opt_classbody
                 {
		     $$ = make_te(NEW_, $2, $4, $6);
		     if($6) javar_out(1, "Anonymous class found");
		 }
               | primary '.' NEW_ ID_ '(' opt_argl ')' opt_classbody
                 {
		     $$ = make_op('.', $1, make_te(NEW_, make_c_i_type(make_name(NULL, $4, 0)), $6, $8));
		     if($8) javar_out(1, "Anonymous class found");
		 }
               | name '.' NEW_ ID_ '(' opt_argl ')' opt_classbody
                 {
		     $$ = make_op('.', $1, make_te(NEW_, make_c_i_type(make_name(NULL, $4, 0)), $6, $8));
		     if($8) javar_out(1, "Anonymous class found");
		 }
               ;

opt_classbody  : classbody
               |                            { $$ = NULL; }
               ;

opt_argl       : argumentlist
               |                             { $$ = NULL; }
               ;

argumentlist   : exp   
               | argumentlist ',' exp        { $$ = make_op(',', $1, $3); }
               ;

arrayinst_exp  : NEW_ primitivetype dim_part  { $$ = make_te(NEW_, $2, $3, NULL); }
               | NEW_ c_i_type      dim_part  { $$ = make_te(NEW_, $2, $3, NULL); }
               | NEW_ primitivetype dims arrayinitializer
                 {
		     $$ = make_te(NEW_, make_atype($2, $3), $4, NULL);
		 }
               | NEW_ c_i_type dims arrayinitializer
                 {
		     $$ = make_te(NEW_, make_atype($2, $3), $4, NULL);
		 }
               ;

dim_part       : dimexps              { $$ = make_op(']', $1, NULL); }
               | dimexps dims         { $$ = make_op(']', $1, $2);   }
               ;

dimexps        : '[' exp ']'          { $$ = make_op('[', NULL, $2);  } 
               | dimexps '[' exp ']'  { $$ = make_op('[', $1,   $3);  }
               ; 

dims           : '[' ']'              { $$ = make_op('[', NULL, NULL); }
               | dims '[' ']'         { $$ = make_op('[', $1,   NULL); }
               ;

/* JAVA 1.1 */
fieldaccess    : primary   '.' ID_     

                 { $$ = make_op('.', $1, make_name(NULL, $3, MISCF_N)); }

               | superpart '.' ID_  

                 { $$ = make_op('.', $1, make_name(NULL, $3, MISCF_N)); }

               ;

superpart      : '(' superpart ')'        { $$ = $2;                   }
               | SUPER_                   { $$ = make_thissup(SUPER_); }
               ;

methodinvocation : name '(' opt_argl ')'  

                   { $1 -> u.name.info = METH_N;
                     $$ = make_op('i', $1, $3);
                   }

                 | primary   '.' ID_ '(' opt_argl ')'  
                   
                  { $$ = make_op('i', make_op('.', $1, make_name(NULL, $3, MISCM_N)), $5); }

                 | superpart '.' ID_ '(' opt_argl ')'  

                  { $$ = make_op('i', make_op('.', $1, make_name(NULL, $3, MISCM_N)), $5); }
                 ;

arrayaccess : name              '[' exp ']'      

                          { $1 -> u.name.info = EXPR_N; $$ = make_op('[', $1, $3); }

            | primarynonewarray '[' exp ']'           { $$ = make_op('[', $1, $3); } 
            ;

postfixexp          : primary
                    | name                  { $1 -> u.name.info = EXPR_N; $$ = $1; }
                    | postincrementexp
                    | postdecrementexp
                    ;

postincrementexp    : postfixexp PP_        { $$ = make_op(PP_, $1, NULL); }
                    ;

postdecrementexp    : postfixexp MM_        { $$ = make_op(MM_, $1, NULL); }
                    ;

unaryexp            : preincrementexp
                    | predecrementexp
                    | '+' unaryexp          { $$ = make_op('+', NULL, $2); }
                    | '-' unaryexp          { $$ = make_op('-', NULL, $2); }
                    | unaryexpnotplusminus
                    ;

preincrementexp     : PP_ unaryexp          { $$ = make_op(PP_, NULL, $2); } 
                    ;

predecrementexp     : MM_ unaryexp          { $$ = make_op(MM_, NULL, $2); } 
                    ;

unaryexpnotplusminus : postfixexp
                     | '~' unaryexp         { $$ = make_op('~', NULL, $2); } 
                     | '!' unaryexp         { $$ = make_op('!', NULL, $2); }
                     | castexp
                     ;

castexp              : '(' primitivetype ')'  
                         unaryexp                   { $$ = make_cast($2, NULL, $4); }
                     | '(' primitivetype dims ')'  
                         unaryexp                   { $$ = make_cast($2, $3, $5); }
                     | '(' exp ')'  
                       unaryexpnotplusminus       
                         
                         { $$ = make_cast( make_c_i_type($2), NULL, $4); }

                     | '(' name dims ')'  
                       unaryexpnotplusminus       
                         
                         { $$ = make_cast( make_c_i_type($2), $3, $5); }
                     ;

multiplicativeexp    : unaryexp
                     | multiplicativeexp '*' unaryexp  { $$ = make_op('*', $1, $3); }
                     | multiplicativeexp '/' unaryexp  { $$ = make_op('/', $1, $3); }
                     | multiplicativeexp '%' unaryexp  { $$ = make_op('%', $1, $3); }
                     ;

additiveexp         : multiplicativeexp
                    | additiveexp '+' multiplicativeexp  { $$ = make_op('+', $1, $3); }
                    | additiveexp '-' multiplicativeexp  { $$ = make_op('-', $1, $3); }
                    ;

shiftexp            : additiveexp
                    | shiftexp LL_  additiveexp   { $$ = make_op(LL_,  $1, $3); }
                    | shiftexp RR_  additiveexp   { $$ = make_op(RR_,  $1, $3); }
                    | shiftexp RRR_ additiveexp   { $$ = make_op(RRR_, $1, $3); }
                    ;

relationalexp       : shiftexp
                    | relationalexp '<'    shiftexp       { $$ = make_op('<',    $1, $3);  }
                    | relationalexp '>'    shiftexp       { $$ = make_op('>',    $1, $3);  }
                    | relationalexp LE_     shiftexp      { $$ = make_op(LE_,     $1, $3); }
                    | relationalexp GE_     shiftexp      { $$ = make_op(GE_,     $1, $3); }
                    | relationalexp INSTOF_ referencetype { $$ = make_te(INSTOF_, $3, $1, NULL); }
                    ;

equalityexp         : relationalexp
                    | equalityexp EQ_ relationalexp { $$ = make_op(EQ_, $1, $3); }
                    | equalityexp NE_ relationalexp { $$ = make_op(NE_, $1, $3); }
                    ;

andexp              : equalityexp
                    | andexp '&' equalityexp       { $$ = make_op('&', $1, $3); }
                    ;

exclusiveorexp      : andexp
                    | exclusiveorexp '^' andexp    { $$ = make_op('^', $1, $3); }
                    ;

inclusiveorexp      : exclusiveorexp
                    | inclusiveorexp '|' exclusiveorexp      { $$ = make_op('|', $1, $3); }
                    ;

conditionalandexp   : inclusiveorexp
                    | conditionalandexp AND_ inclusiveorexp  { $$ = make_op(AND_, $1, $3); }
                    ;

conditionalorexp    : conditionalandexp
                    | conditionalorexp  OR_ conditionalandexp { $$ = make_op(OR_, $1, $3); }
                    ;

conditionalexp      : conditionalorexp
                    | conditionalorexp '?' exp ':' conditionalexp 

                         { $$ = make_op('?', $1, make_op(':', $3, $5)); }
                    ;

assignmentexp       : conditionalexp
                    | assignment
                    ;

assignment          : lefthandside assignop assignmentexp { $$ = make_op($2, $1, $3); }
                    ;

assignop            : '='          { $$ = '=';     }
                    | PIS_         { $$ = PIS_;    } 
                    | MINIS_       { $$ = MINIS_;  }
                    | MULIS_       { $$ = MULIS_;  }
                    | DIS_         { $$ = DIS_;    }
                    | ANDIS_       { $$ = ANDIS_;  }
                    | ORIS_        { $$ = ORIS_;   }
                    | UIS_         { $$ = UIS_;    }
                    | IDIVIS_      { $$ = IDIVIS_; }
                    | LLIS_        { $$ = LLIS_;   }
                    | RRIS_        { $$ = RRIS_;   }
                    | RRRIS_       { $$ = RRRIS_;  }
                    ;

lefthandside        : name         { $1 -> u.name.info = EXPR_N; $$ = $1; } 
                    | fieldaccess            
                    | arrayaccess            
                    ;

opt_exp             : exp
                    |              { $$ = NULL; } 
                    ;

exp                 : assignmentexp
                    ;

/* Auxiliary Production
   ******************** */

err_sep             : '}' 
                    | ';'
                    ;

gc                  :           { $$ = get_comment(); }
                    ;

/* JAVAR ANNOTATATIONS
   ******************* */

javar_annot : JAVPAR_           { in_l = 1; set_annot(NULL, NULL, 1); }
                opt_annotf
              JAVEND_           { $$ = make_annot(NULL, NULL, NULL);  }
            | JAVDIST_          { in_l = 1;  set_dist_annot(NULL, NULL); }
	        opt_distannotf
	      JAVEND_           { $$ = make_dist_annot(); }
            |                   { $$ = NULL; }
            ;

opt_annotf : annotfields
           |
           ;

opt_distannotf : distannotfields
                |
                ;
 
annotfields : annotfield
            | annotfields ',' annotfield
            ;

distannotfields : distannotfield
                | distannotfields ',' distannotfield
                ;
 
annotfield  : ID_  '=' LITERAL_            { set_annot($1,   $3, in_l); }
            | ID_                          { set_annot($1, NULL, in_l); }
            | '(' name ')'                 { add_annot($2);             }
            ;

	    
distannotfield : ID_ '=' LITERAL_              { set_dist_annot($1, $3); }
               | name method_access            { add_dist_symb($1, METHOD_B, NONE_B, NULL, NULL, $2); }
               | name data_access              { add_dist_symb($1, SHARED_B, NONE_B, NULL, NULL, $2); }
               | name data_dist data_access    { add_dist_symb($1, NON_SHARED_B, $2, locd1, locd2, $3); }
               ;

data_dist : '[' exp ',' exp ']' BLOCK_         { locd1 = $2; locd2 = $4; $$ = BLOCK_B; }
          ;

data_access : UNMODIFIED_               { $$ = UNMODIFIED_B;  }
            | MOD_IN_                   { $$ = MOD_IN_B; }
            | MOD_OUT_                  { $$ = MOD_OUT_B; }
            | MOD_INOUT_                { $$ = MOD_INOUT_B; }
            ;

method_access: FINAL_                 { $$ = FIN_METH_B;  }
             ;

javar_stmt  : JAVPOST_ LITERAL_         JAVEND_ { $$ = make_annot($2, NULL, NULL); }
            | JAVWAIT_ LITERAL_ ',' exp JAVEND_ { $$ = make_annot($2,   $4, NULL); }
            ;

javar_block : JAVINV_               { in_l = 0; set_annot(NULL, NULL, 0); }
                opt_annotf
              JAVEND_ 
	      '{' opt_blockstmt '}' { $$ = make_annot(NULL, NULL, make_block($6)); } 
            ;

%%

yyerror(char *mess) { 
  /* IMPORTED */
  extern char *yytext;
  javar_out(0, "%s near '%s'", mess, yytext);
  return 0;
}
