

%{

/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** scanner.l : flex definitions of Java
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

/* ********************************************************
   *** INCLUDE FILES and DEFINITIONS                    ***
   ********************************************************  */

#include "global.h"
#include "y.tab.h"

%}

%x COMMENT 

whitespace [\t\f ]+
identifier {letter}(({digit}|{letter})*)

hexdigit   [0-9a-fA-F]
octdigit   [0-7]
digit      [0-9]
letter     [A-Za-z_$]
 
numsuffix  l|L

dec_num    (0|([1-9]{digit}*)){numsuffix}?
hex_num    0x{hexdigit}*{numsuffix}?
oct_num    0{octdigit}+{numsuffix}?

fpsuffix   f|F|d|D
exponent   ("e"|"E")("+"|"-")?{digit}+

fp_lit1    {digit}+"."{digit}*{exponent}?{fpsuffix}?
fp_lit2     "."{digit}+{exponent}?{fpsuffix}?
fp_lit3    {digit}+{exponent}{fpsuffix}?
fp_lit4    {digit}+{exponent}?{fpsuffix}
 
flp_lit    {fp_lit1}|{fp_lit2}|{fp_lit3}|{fp_lit4}

char_esc   \\b|\\t|\\n|\\f|\\r|\\\"|\\'|\\\\
oct_esc    \\({octdigit}|({octdigit}{octdigit})|([0-3]{octdigit}{octdigit}))

unicodeesc \\u{hexdigit}{4}   

escape     {char_esc}|{oct_esc}|{unicodeesc}

chr_lit    \'([^'\n\\]|{escape})\'
str_lit    \"([^"\n\\]|{escape})*(\"|\n)   

%%

     /* Unicode Escapes
        *************** */

{unicodeesc}       { javar_out(0, "%s unicode escape "
                               "is not supported", yytext);  
                   }
     /* Comments
        ******** */

"//".*\n           { add_comment(yytext); lineno++; }

"/*"               { add_comment("/*");                         BEGIN COMMENT;   }
<COMMENT>"*/"      { add_comment(yytext); add_comment("\n");    BEGIN INITIAL;   }
<COMMENT>.         { add_comment(yytext);                                        }
<COMMENT>\n        { add_comment("\n");   lineno++;                              }
<COMMENT><<EOF>>   { javar_out(0, "unexpected <EOF> in comment"); BEGIN INITIAL; }

     /* Annotations (NON-STANDARD)
        ************************** */

"/*par"            { return JAVPAR_;     }
"/*par_inv"        { return JAVINV_;     }
"/*post"           { return JAVPOST_;    }
"/*wait"           { return JAVWAIT_;    }
"/*dist"           { return JAVDIST_;    }
"__non_shared"     { return NONSHARED_;  }
"__shared"         { return SHARED_;     }
"__unmodified"     { return UNMODIFIED_; }
"__mod_in"         { return MOD_IN_;     }
"__mod_inout"      { return MOD_INOUT_;  }
"__mod_out"        { return MOD_OUT_;    }
"__block"          { return BLOCK_;      }
"__cyclic"         { return CYCLIC_;     }

"*/"               { return JAVEND_;  /* outside COMMENT occurrence! */ }

     /* White Space
        *********** */

{whitespace}       { ; }
\n                 { lineno++; }

     /* Reserved KeyWords
        ***************** */

"abstract"     { return ABSTRACT_; }
"boolean"      { return BOOLEAN_; }
"break"        { return BREAK_; }
"byte"         { return BYTE_; }
"case"         { return CASE_; }
"catch"        { return CATCH_; }
"char"         { return CHAR_; }
"class"        { return CLASS_; }
"continue"     { return CONTINUE_; }
"default"      { return DEFAULT_; }
"do"           { return DO_; }
"double"       { return DOUBLE_; }
"else"         { return ELSE_; }
"extends"      { return EXTENDS_; }
"final"        { return FINAL_; }
"finally"      { return FINALLY_; }
"float"        { return FLOAT_; }
"for"          { return FOR_; }
"if"           { return IF_; }
"implements"   { return IMPLEMENTS_; }
"import"       { return IMPORT_; }
"instanceof"   { return INSTOF_; }
"int"          { return INT_; }
"interface"    { return INTERFACE_; }
"long"         { return LONG_; }
"native"       { return NATIVE_; }
"new"          { return NEW_; }
"package"      { return PACKAGE_; }
"private"      { return PRIVATE_; }
"protected"    { return PROTECTED_; }
"public"       { return PUBLIC_; }
"return"       { return RETURN_; }
"short"        { return SHORT_; }
"static"       { return STATIC_; }
"super"        { return SUPER_; }
"switch"       { return SWITCH_; }
"synchronized" { return SYNCHRONIZED_; }
"this"         { return THIS_; }
"throw"        { return THROW_; }
"throws"       { return THROWS_; }
"transient"    { return TRANSIENT_; }
"try"          { return TRY_; }
"void"         { return VOID_; }
"volatile"     { return VOLATILE_; }
"while"        { return WHILE_; }

     /* Reserved, but Unused KeyWords
        ***************************** */

"const"      |
"goto"       { javar_out(0, "%s : reserved, but unused keyword", yytext); }

     /* Compound Operators
        ****************** */

"=="   return EQ_;
"<="   return LE_;
">="   return GE_;
"!="   return NE_;
"&&"   return AND_;
"||"   return OR_;
"++"   return PP_;
"--"   return MM_;

"<<"   return LL_;
">>"   return RR_;
">>>"  return RRR_;

"+="   return PIS_; 
"-="   return MINIS_;
"*="   return MULIS_;
"/="   return DIS_;
"&="   return ANDIS_; 
"|="   return ORIS_;
"^="   return UIS_;
"%="   return IDIVIS_;
"<<="  return LLIS_;
">>="  return RRIS_;
">>>=" return RRRIS_; 

     /* Literals 
        ******** */

"null"        { yylval.expr = make_literal(-2, yytext, 0);
                return LITERAL_;
              }

"true"        |
"false"       { yylval.expr = make_literal(BOOL_T, yytext, 0);
                return LITERAL_; 
              }

{dec_num}     |
{hex_num}     |
{oct_num}     { short type = (tolower(yytext[yyleng-1]) == 'l') ? LONG_T : INT_T;
                yylval.expr = make_literal(type, yytext, 0);
                return LITERAL_;
              }

{flp_lit}     { short  type = (tolower(yytext[yyleng-1]) == 'f') ? FLOAT_T : DOUBLE_T;
                yylval.expr = make_literal(type, yytext, 0);
                return LITERAL_;
              }

{chr_lit}     { yylval.expr = make_literal(CHAR_T, yytext, 0);
                return LITERAL_; 
              }

{str_lit}     { 
                if (yytext[yyleng-1] == '\n') { 
              
                  /* Handle NEWLINE Terminated String */

                  yytext[yyleng-1] = '\"';
                  javar_out(1, "string not terminated at EOL : %s", yytext);
                  lineno++;
                }

                yylval.expr = make_literal(-1, yytext, 0);
                return LITERAL_;
              }

     /* Identifiers 
        *********** */

{identifier}  { yylval.sym = insert_table(yytext);
                return ID_; 
              }

    /* Default Action
       ************** */

.             { return yytext[0]; }


%%

#ifndef yywrap
yywrap() { return 1; }
#endif
