

/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** stubskel.c: stub / skeleton pair generation
   ***             for nexus rmi.
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

#include <stdarg.h>
#include <assert.h>

#include "global.h"

#define MARSHAL 0
#define GETSIZE 1
#define GETTYPE 2
#define GETCAST 3
#define STUB_UNMARSHAL 4
#define SKEL_UNMARSHAL 5

static char *type_names[] = {"void",
			      "boolean",
			      "byte",
			      "short",
			      "int",
			      "long",
			      "char",
			      "float",
			      "double",
                              "????",
                              "????"};

static char *wrapper_names[] = {"void ????",
				  "Boolean",
				  "Byte",
				  "Short",
				  "Integer",
				  "Long",
				  "Character",
				  "Float",
				  "Double",
				  "????",
				  "????"};

static char *type_inits[] = {"void ????",
			      "false",
			      "0",
			      "0",
			      "0",
			      "0",
			      "0",
			      "0.0f",
			      "0.0",
			      "null",
			      "null"};

static char *mod_names[] = {"public ",
			     "private ",
			     "protected ",
			     "static ",
			     "abstract ",
			     "final ",
			     "native ",
			     "synchronized ",
			     "transient ",
			     "volatile "};

static FILE *fp = NULL;
static stmt_ptr server = NULL;
static stmt_ptr imports = NULL;

static char *servername = NULL;
static char *string = NULL;
static char *stubname = NULL;
static char *skelname = NULL;
static char *package_name = NULL;

static int handler_id;
static int trav_nr;
static int array_level;
static int marshal_ex_pos;
static int unmarshal_ex_pos;

static char *gen_array_alloc(type_ptr tp, char *v)
{
    char *elt = NULL;
    int nest = 0;
    int i, j = 0;
    
    while(tp->u.aref->kind == ARRAY_T)
    {
	nest++;
	tp = tp->u.aref;
    }

    if(tp->u.aref->kind == C_OR_I_T)
	elt = tp->u.aref->u.name->u.name.id->symbol;
    else
	elt = type_names[tp->u.aref->kind];

    if(string)
	string = (char *)realloc(string, (strlen(elt) + ((nest + 1) * 2) + strlen(v) + 1) * sizeof(char));
    else
	string = (char *)malloc((strlen(elt) + ((nest + 1) * 2) + strlen(v) + 1) * sizeof(char));

    for(i = 0; i < strlen(elt); i++, j++)
	string[j] = elt[i];

    string[j++] = '[';
    for(i = 0; i < strlen(v); i++, j++)
	string[j] = v[i];
    
    string[j++] = ']';
    for(i = 0; i < nest; i++, j++)
    {
	string[j++] = '[';
	string[j++] = ']';
    }
    string[j] = '\0';
    return string;
}

static char *space(int n)
{
    int i;

    if(string)
	string = (char *)realloc(string, (n + 1) * sizeof(char));
    else
	string = (char *)malloc((n + 1) * sizeof(char));
    
    for(i = 0; i < n; i++)
	string[i] = ' ';
    string[i] = '\0';
    return string;
}

static char *get_modifiers(int mod)
{
    int i, j, k, len;

    len = 0;
    for(i = 0; i < 10; i++)
	if(mod & (1 << i))
	    len += strlen(mod_names[i]);

    len++;
    if(string)
	string = (char *)realloc(string, len * sizeof(char));
    else
	string = (char *)malloc(len * sizeof(char));

    j = 0;
    for(i = 0; i < 10; i++)
	if(mod & (1 << i))
	    for(k = 0; k < strlen(mod_names[i]); k++, j++)
		string[j] = mod_names[i][k];

    string[j] = '\0';
    return string;
}
	
static char *get_handler_name(char *method_name)
{
    char *suffix = "_HANDLER_ID";
    int i, j, len;

    j = 0;
    len = strlen(method_name) + strlen(suffix) + 1;

    if(string)
	string = (char *)realloc(string, len * sizeof(char));
    else
	string = (char *)malloc(len * sizeof(char));

    for(i = 0; i < strlen(method_name); i++, j++)
	string[j] = toupper(method_name[i]);

    for(i = 0; i < strlen(suffix); i++, j++)
	string[j] = suffix[i];

    string[j] = '\0';
    return string;
}

static char *construct_servername(void)
{
    char *name;
    name = (char *)malloc((strlen(filename) - 4) * sizeof(char));
    strncpy(name, filename, strlen(filename) - 5);
    return name;
}

static char *construct_name(char *basename, char *suffix)
{
    char *name;
    int i, j, len;

    len = strlen(basename) + strlen(suffix) + 1;
    name = (char *)malloc(len * sizeof(char));

    j = 0;
    for(i = 0; i < strlen(basename); i++, j++)
	name[j] = filename[i];

    for(i = 0; i < strlen(suffix); i++, j++)
	name[j] = suffix[i];

    name[j] = '\0';
    return name;
}

static char *construct_filename(char *basename)
{
    char *fn;
    char *suffix = ".java";
    int i, j, len;
    
    len = strlen(basename) + 6;
    fn = (char *)malloc(len * sizeof(char));

    j = 0;
    for(i = 0; i < strlen(basename); i++, j++)
	fn[j] = basename[i];

    for(i = 0; i < strlen(suffix); i++, j++)
	fn[j] = suffix[i];

    fn[j] = '\0';
    return fn;
}

static void handle_type_var(type_ptr tp, char *var_name, char *bufname, int action, int indent)
{
    switch(action)
    {
    case GETCAST: assert(tp); assert(!var_name); assert(!bufname); break;
    case GETTYPE: assert(tp); assert(!var_name); assert(!bufname); break;
    case SKEL_UNMARSHAL: assert(tp); assert(var_name); assert(bufname); break;
    case STUB_UNMARSHAL: assert(tp); assert(var_name); assert(bufname); break;
    case MARSHAL: assert(tp); assert(var_name); assert(bufname); break;
    case GETSIZE:assert(tp); assert(var_name); assert(!bufname); break;
    default: assert(0);
    }
    
    switch(tp->kind)
    {
    case VOID_T:
	switch(action)
	{
	case GETCAST:
	    fprintf(stderr, "type = void, var_name = %s, bufname = %s, action = %d, indent = %d.\n",
		    var_name ? var_name : "NULL", bufname ? bufname : "NULL", action, indent);
	    break;
	case GETTYPE:
	    fprintf(fp, "%s", type_names[tp->kind]);
	    break;
	case SKEL_UNMARSHAL:
	    fprintf(stderr, "type = void, var_name = %s, bufname = %s, action = %d, indent = %d.\n",
		    var_name ? var_name : "NULL", bufname ? bufname : "NULL", action, indent);
	    break;
	case STUB_UNMARSHAL:
	    fprintf(stderr, "type = void, var_name = %s, bufname = %s, action = %d, indent = %d.\n",
		    var_name ? var_name : "NULL", bufname ? bufname : "NULL", action, indent);
	    break;
	case MARSHAL:
	    fprintf(stderr, "type = void, var_name = %s, bufname = %s, action = %d, indent = %d.\n",
		    var_name ? var_name : "NULL", bufname ? bufname : "NULL", action, indent);
	    break;
	case GETSIZE:
	    fprintf(stderr, "type = void, var_name = %s, bufname = %s, action = %d, indent = %d.\n",
		    var_name ? var_name : "NULL", bufname ? bufname : "NULL", action, indent);
	    break;
	}
	break;
    case BOOL_T:
	switch(action)
	{
	case GETCAST:
	    fprintf(fp, "((%s)r).%sValue()", wrapper_names[tp->kind], type_names[tp->kind]);
	    break;
	case GETTYPE:
	    fprintf(fp, "%s", type_names[tp->kind]);
	    break;
	case STUB_UNMARSHAL:
	    fprintf(fp, "%s%s = new %s(Nexusize.int2bool(%s.get_int()));\n", space(indent), var_name, wrapper_names[tp->kind], bufname);
	    break;
	case SKEL_UNMARSHAL:
	    fprintf(fp, "%s%s = Nexusize.int2bool(%s.get_int());\n", space(indent), var_name, bufname);
	    break;
	case MARSHAL:
	    fprintf(fp, "%s%s.put_int(Nexusize.bool2int(%s));\n", space(indent), bufname, var_name);
	    break;
	case GETSIZE:
	    fprintf(fp, "%sbufferSize += nexus.sizeof_int(1);\n", space(indent));
	    break;
	}
	break;
    case BYTE_T:
    case SHORT_T:
    case INT_T:
    case LONG_T:
    case CHAR_T:
    case FLOAT_T:
    case DOUBLE_T:
	switch(action)
	{
	case GETCAST:
	    fprintf(fp, "((%s)r).%sValue()", wrapper_names[tp->kind], type_names[tp->kind]);
	    break;
	case GETTYPE:
	    fprintf(fp, "%s", type_names[tp->kind]);
	    break;
	case STUB_UNMARSHAL:
	    fprintf(fp, "%s%s = new %s(%s.get_%s());\n", space(indent), var_name, wrapper_names[tp->kind], bufname, type_names[tp->kind]);
	    break;
	case SKEL_UNMARSHAL:
	    fprintf(fp, "%s%s = %s.get_%s();\n", space(indent), var_name, bufname, type_names[tp->kind]);
	    break;
	case MARSHAL:
	    fprintf(fp, "%s%s.put_%s(%s);\n", space(indent), bufname, type_names[tp->kind], var_name);
	    break;
	case GETSIZE:
	    fprintf(fp, "%sbufferSize += nexus.sizeof_%s(1);\n", space(indent), type_names[tp->kind]);
	    break;
	}
	break;
    case C_OR_I_T:
	if(strcmp(tp->u.name->u.name.id->symbol, "String") == 0)
	{
	    switch(action)
	    {
	    case GETCAST:
		fprintf(fp, "(String)r");
		break;
	    case GETTYPE:
		fprintf(fp, "%s", tp->u.name->u.name.id->symbol);
		break;
	    case SKEL_UNMARSHAL:
	    case STUB_UNMARSHAL:
		fprintf(fp, "%s%s = Nexusize.nexusReadString(%s);\n", space(indent), var_name, bufname);
		break;
	    case MARSHAL:
		fprintf(fp, "%sNexusize.nexusWriteString(%s, %s);\n", space(indent), var_name, bufname);
		break;
	    case GETSIZE:
		fprintf(fp, "%sbufferSize += Nexusize.nexusSizeofString(%s);\n", space(indent), var_name);
		break;
	    }
	}
	else
	{
	    switch(action)
	    {
	    case GETCAST:
		fprintf(fp, "(%s)r", tp->u.name->u.name.id->symbol);
		break;
	    case GETTYPE:
		fprintf(fp, "%s", tp->u.name->u.name.id->symbol);
		break;
	    case STUB_UNMARSHAL:
	    case SKEL_UNMARSHAL:
		unmarshal_ex_pos = 1;
		fprintf(fp, "%s%s = (new %s()).nexusRead(%s);\n", space(indent), var_name, tp->u.name->u.name.id->symbol, bufname);
		break;
	    case MARSHAL:
		marshal_ex_pos = 1;
		fprintf(fp, "%s%s.nexusWrite(%s);\n", space(indent), var_name, bufname);
		break;
	    case GETSIZE:
		fprintf(fp, "%sbufferSize += %s.nexusSizeof();\n", space(indent), var_name);
		break;
	    }
	}
	break;
    case ARRAY_T:
	if((tp->u.aref->kind != ARRAY_T) && (tp->u.aref->kind != C_OR_I_T))
	{
	    switch(action)
	    {
	    case GETCAST:
		fprintf(fp, "(");
		handle_type_var(tp, NULL, NULL, GETTYPE, 0);
		fprintf(fp, ")r");
		break;
	    case GETTYPE:
		{
		    handle_type_var(tp->u.aref, NULL, NULL, GETTYPE, 0);
		    fprintf(fp, "[]");
		}
		break;
	    case STUB_UNMARSHAL:
	    case SKEL_UNMARSHAL:
		fprintf(fp, "%s%s = Nexusize.nexusRead%sArray(%s);\n", space(indent), var_name, type_names[tp->u.aref->kind], bufname);
		break;
	    case MARSHAL:
		fprintf(fp, "%sNexusize.nexusWrite%sArray(%s, %s);\n", space(indent), type_names[tp->u.aref->kind], var_name, bufname);
		break;
	    case GETSIZE:
		if(tp->u.aref->kind == BOOL_T)
		    fprintf(fp, "%sbufferSize += nexus.sizeof_int(1) + nexus.sizeof_%s(%s.length);\n",
			    space(indent), "int", var_name);
		else
		    fprintf(fp, "%sbufferSize += nexus.sizeof_int(1) + nexus.sizeof_%s(%s.length);\n",
			    space(indent), type_names[tp->u.aref->kind], var_name);
		break;
	    }
	}
	else
	{
	    char *len_var;
	    char *index_var;
	    char *new_name;
	    char *new_array;

	    if((action != GETCAST) && (action != GETTYPE))
	    {
		array_level++;
		index_var = (char *)malloc(12 * sizeof(char));
		sprintf(index_var, "i%d\0", array_level);
		new_name = (char *)malloc((strlen(var_name) + 2 + strlen(index_var)) * sizeof(char));
		sprintf(new_name, "%s[%s]", var_name, index_var);
	    }

	    switch(action)
	    {
	    case GETCAST:
		fprintf(fp, "(");
		handle_type_var(tp, NULL, NULL, GETTYPE, 0);
		fprintf(fp, ")r");
		break;
	    case GETTYPE:
		{
		    handle_type_var(tp->u.aref, NULL, NULL, GETTYPE, 0);
		    fprintf(fp, "[]");
		}
		break;
	    case MARSHAL:
		fprintf(fp, "%s%s.put_int(%s.length);\n", space(indent), bufname, var_name);
		fprintf(fp, "%sfor(int %s = 0; %s < %s.length; %s++)\n", space(indent), index_var, index_var, var_name, index_var);
		fprintf(fp, "%s{\n", space(indent));
		handle_type_var(tp->u.aref, new_name, bufname, MARSHAL, indent + 4);
		fprintf(fp, "%s}\n",space(indent));
		break;
	    case GETSIZE:
		fprintf(fp, "%sbufferSize += nexus.sizeof_int(1);\n", space(indent));
		fprintf(fp, "%sfor(int %s = 0; %s < %s.length; %s++)\n", space(indent), index_var, index_var, var_name, index_var);
		fprintf(fp, "%s{\n", space(indent));
		handle_type_var(tp->u.aref, new_name, NULL, GETSIZE, indent + 4);
		fprintf(fp, "%s}\n", space(indent));
		break;
	    case STUB_UNMARSHAL:
	    case SKEL_UNMARSHAL:
		len_var = (char *)malloc(11 * sizeof(char));
		sprintf(len_var, "l%d\0", array_level);

		fprintf(fp, "%sint %s = %s.get_int();\n", space(indent), len_var, bufname);

		new_array = gen_array_alloc(tp, len_var);
		fprintf(fp, "%s%s = new %s;\n", space(indent), var_name, new_array);
		free(new_array);
		
		fprintf(fp, "%sfor(int %s = 0; %s < %s; %s++)\n", space(indent), index_var, index_var, len_var, index_var);
		fprintf(fp, "%s{\n", space(indent));
		handle_type_var(tp->u.aref, new_name, bufname, action, indent + 4);
		fprintf(fp, "%s}\n", space(indent));

		free(len_var);
		break;
	    }

	    if((action != GETCAST) && (action != GETTYPE))
	    {
		free(new_name);
		free(index_var);
		array_level--;
	    }
	}
	break;
    }
}

static void dump_handler_ids(stmt_ptr s)
{
    if((s->kind == METHOD_S) &&
       (s->u.header.mod & PUB_B) &&
       (strcmp(s->u.header.name->u.name.id->symbol, "run") != 0) &&
       (strcmp(s->u.header.name->u.name.id->symbol, "main") != 0))
    {
	fprintf(fp, "    private final int %s = %d;\n", get_handler_name(s->u.header.name->u.name.id->symbol), handler_id);
	handler_id++;
    }
}

static void dump_formal(expr_ptr e)
{
    if(e->kind == 'd')
    {
	if(trav_nr != 0)
	    fprintf(fp, ", ");

	array_level = -1;
	handle_type_var(e->type, NULL, NULL, GETTYPE, 0);
	fprintf(fp, " %s", e->u.declstmt->u.decl.name->u.name.id->symbol);
	trav_nr = 1;
    }
}

static void dump_arg_size(expr_ptr e)
{
    if(e->kind == 'd')
    {
	array_level = -1;
	handle_type_var(e->type, e->u.declstmt->u.decl.name->u.name.id->symbol, NULL, GETSIZE, 12);
    }
}

static void dump_arg_marsh(expr_ptr e)
{
    if(e->kind == 'd')
    {
	array_level = -1;
	handle_type_var(e->type, e->u.declstmt->u.decl.name->u.name.id->symbol, "buffer", MARSHAL, 12);
    }
}

static void dump_intfs(expr_ptr e)
{
    if(e->kind == 't')
	fprintf(fp, "%s, ", e->u.te.tp->u.name->u.name.id->symbol);
}

static void dump_stub_methods(stmt_ptr s)
{
    if((s->kind == METHOD_S) &&
       (s->u.header.mod & PUB_B) &&
       (strcmp(s->u.header.name->u.name.id->symbol, "run") != 0) &&
       (strcmp(s->u.header.name->u.name.id->symbol, "main") != 0))
    {
	array_level = -1;
	fprintf(fp, "    %s", get_modifiers(s->u.header.mod));
	handle_type_var(s->u.header.tp, NULL, NULL, GETTYPE, 0);
	fprintf(fp, " %s(", s->u.header.name->u.name.id->symbol);
	trav_nr = 0;
	expr_traverse(s->u.header.e1, NULL, dump_formal);
	fprintf(fp, ") throws ");
	trav_nr = 0;
	expr_traverse(s->u.header.e1, NULL, dump_intfs);
	fprintf(fp, "RemoteException\n");
	fprintf(fp, "    {\n");
	if(s->u.header.tp->kind != VOID_T)
	{
	    array_level = -1;
	    fprintf(fp, "        ");
	    handle_type_var(s->u.header.tp, NULL, NULL, GETTYPE, 0);
	    fprintf(fp, " res = %s;\n\n", type_inits[s->u.header.tp->kind]);
	}
	
	fprintf(fp, "        try\n");
	fprintf(fp, "        {\n");
	fprintf(fp, "            int index = getFreeEntry();\n");
	fprintf(fp, "            %sHandler handler = new %sHandler(nexus, this, index);\n", stubname, stubname);
	fprintf(fp, "            Startpoint sp = Communication.getStartpoint(handler, null);\n");
	fprintf(fp, "            int bufferSize = sp.sizeof() + nexus.sizeof_int(2);\n");
	expr_traverse(s->u.header.e1, NULL, dump_arg_size);
	fprintf(fp, "            PutBuffer buffer = nexus.buffer_init(bufferSize, 0);\n");
        fprintf(fp, "            sp.put(buffer);\n");
	fprintf(fp, "            buffer.put_int(0);\n");
	fprintf(fp, "            buffer.put_int(%s);\n", get_handler_name(s->u.header.name->u.name.id->symbol));
	marshal_ex_pos = 0;
	expr_traverse(s->u.header.e1, NULL, dump_arg_marsh);
	fprintf(fp, "            nexus.send_rsr(buffer, stubStartpoint, GFHANDLER_ID, false);\n");
	fprintf(fp, "            waitForReply(index);\n");
	fprintf(fp, "            if(ex[index] != null)\n");
	fprintf(fp, "                throw new RemoteException(\"stub-reply\", ex[index]);\n");
	if(s->u.header.tp->kind != VOID_T)
	{
	    fprintf(fp, "            Object r = result[index];\n");
	    array_level = -1;
	    fprintf(fp, "            res = ");
	    handle_type_var(s->u.header.tp, NULL, NULL, GETCAST, 0);
	    fprintf(fp, ";\n");
	}
	fprintf(fp, "            freeEntry(index);\n");
	fprintf(fp, "        }\n");
	if(marshal_ex_pos)
	{
	    fprintf(fp, "        catch(MarshalException e)\n");
	    fprintf(fp, "        {\n");
	    fprintf(fp, "            throw new RemoteException(\"MarshalException\", e);\n");
	    fprintf(fp, "        }\n");
	}
	fprintf(fp, "        catch(Exception e)\n");
	fprintf(fp, "        {\n");
	fprintf(fp, "            e.printStackTrace();\n");
	fprintf(fp, "            System.exit(1);\n");
	fprintf(fp, "        }\n");
	if(s->u.header.tp->kind != VOID_T)
	    fprintf(fp, "        return res;\n");
	fprintf(fp, "    }\n\n");
    }
}

static void dump_stub_case(stmt_ptr s)
{
    if((s->kind == METHOD_S) &&
       (s->u.header.mod & PUB_B) &&
       (strcmp(s->u.header.name->u.name.id->symbol, "run") != 0) &&
       (strcmp(s->u.header.name->u.name.id->symbol, "main") != 0))
    {
	fprintf(fp, "            case %s:\n", get_handler_name(s->u.header.name->u.name.id->symbol));
	fprintf(fp, "                {\n");
	if(s->u.header.tp->kind != VOID_T)
	{
	    switch(s->u.header.tp->kind)
	    {
	    case BOOL_T:
	    case BYTE_T:
	    case SHORT_T:
	    case INT_T:
	    case LONG_T:
	    case CHAR_T:
	    case FLOAT_T:
	    case DOUBLE_T:
		fprintf(fp, "                    %s res = null;\n", wrapper_names[s->u.header.tp->kind]);
		break;
	    case C_OR_I_T:
		fprintf(fp, "                    %s res = null;\n", s->u.header.tp->u.name->u.name.id->symbol);
		break;
	    case ARRAY_T:
		array_level = -1;
		fprintf(fp, "                    ");
		handle_type_var(s->u.header.tp, NULL, NULL, GETTYPE, 0);
		fprintf(fp, " res = null;\n");
		break;
	    }
	    array_level = -1;
	    handle_type_var(s->u.header.tp, "res", "buffer", STUB_UNMARSHAL, 20);
	    fprintf(fp, "                    stub.setResult(index, res);\n");
	}
	else
	    fprintf(fp, "                    stub.setResult(index, null);\n");
	fprintf(fp, "                }\n");
	fprintf(fp, "                break;\n");
    }
}

static void generate_stub(void)
{
    char *fn;
    int i;
    
    javar_out(2, "Generating stub");

    stubname = construct_name(servername, "Stub");
    fn = construct_filename(stubname);
    fp = fopen(fn, "w");
    output_file = fp;
    
    fprintf(fp, "package %s;\n\n", package_name);
    
    fprintf(fp, "import nexus.*;\n");
    dump_stmt(imports, fp);
    fprintf(fp, "\n");
    
    fprintf(fp, "public final class %s extends GeneralStub implements ", stubname);
    dump_expr(server->u.header.e1, fp);
    fprintf(fp, "\n");
    fprintf(fp, "{\n");
    handler_id = 0;
    stmt_traverse(server, dump_handler_ids, NULL);
    fprintf(fp, "    private final int EXCEPTION_HANDLER_ID = 103;\n");
    fprintf(fp, "\n");
    fprintf(fp, "    public %s(Nexus nexus, Startpoint sp)\n", stubname);
    fprintf(fp, "    {\n");
    fprintf(fp, "        super(nexus, sp);\n");
    fprintf(fp, "    }\n\n");

    stmt_traverse(server, dump_stub_methods, NULL);
    fprintf(fp, "}\n\n");

    fprintf(fp, "class %sHandler extends StubHandler\n", stubname);
    fprintf(fp, "{\n");
    handler_id = 0;
    stmt_traverse(server, dump_handler_ids, NULL);
    fprintf(fp, "    private final int EXCEPTION_HANDLER_ID = 103;\n");
    fprintf(fp, "\n");

    fprintf(fp, "    public %sHandler(Nexus nexus, %s stub, int index)\n", stubname, stubname);
    fprintf(fp, "    {\n");
    fprintf(fp, "        super(nexus, stub, index);\n");
    fprintf(fp, "    }\n\n");

    fprintf(fp, "    public void invoke_handler(Endpoint ep, GetBuffer buffer, int handler_id)\n");
    fprintf(fp, "    {\n");
    fprintf(fp, "        try\n");
    fprintf(fp, "        {\n");
    fprintf(fp, "            int id = buffer.get_int();\n\n");

    fprintf(fp, "            switch(id)\n");
    fprintf(fp, "            {\n");
    unmarshal_ex_pos = 0;
    stmt_traverse(server, dump_stub_case, NULL);
    fprintf(fp, "            case EXCEPTION_HANDLER_ID:\n");
    fprintf(fp, "                {\n");
    fprintf(fp, "                    String exname = Nexusize.nexusReadString(buffer);\n");
    fprintf(fp, "                    stub.setException(index, (Exception)Class.forName(exname).newInstance());\n");
    fprintf(fp, "                    break;\n");
    fprintf(fp, "                }\n");
    fprintf(fp, "            default:\n");
    fprintf(fp, "                {\n");
    fprintf(fp, "                    System.out.println(\"Invalid handler ID.\");\n");
    fprintf(fp, "                }\n");
    fprintf(fp, "                break;\n");
    fprintf(fp, "            }\n");
    fprintf(fp, "        }\n");
    if(unmarshal_ex_pos)
    {
	fprintf(fp, "        catch(UnmarshalException e)\n");
	fprintf(fp, "        {\n");
	fprintf(fp, "            stub.setException(index, new UnmarshalException(\"\"));\n");
	fprintf(fp, "        }\n");
    }
    fprintf(fp, "        catch(Exception e)\n");
    fprintf(fp, "        {\n");
    fprintf(fp, "            e.printStackTrace();\n");
    fprintf(fp, "            System.exit(1);\n");
    fprintf(fp, "        }\n");
    fprintf(fp, "    }\n");
    fprintf(fp, "}\n");    
    fclose(fp);
    free(fn);
    free(stubname);
}

static void dump_arg_unmarsh(expr_ptr e)
{
    if(e->kind == 'd')
    {
	array_level = -1;
	handle_type_var(e->type, e->u.declstmt->u.decl.name->u.name.id->symbol, "inbuf", SKEL_UNMARSHAL, 20);
    }
}

static void dump_actual(expr_ptr e)
{
    if(e->kind == 'd')
    {
	if(trav_nr != 0)
	    fprintf(fp, ", ");
    
	fprintf(fp, "%s", e->u.declstmt->u.decl.name->u.name.id->symbol);
	trav_nr = 1;
    }
}

static void dump_arg_decl(expr_ptr e)
{
    if(e->kind == 'd')
    {
	array_level = -1;
	fprintf(fp, "                    ");
	handle_type_var(e->type, NULL, NULL, GETTYPE, 0);
	fprintf(fp, " %s = %s;\n", e->u.declstmt->u.decl.name->u.name.id->symbol, type_inits[e->type->kind]);
    }
}

static void dump_skel_case(stmt_ptr s)
{
    if((s->kind == METHOD_S) &&
       (s->u.header.mod & PUB_B) &&
       (strcmp(s->u.header.name->u.name.id->symbol, "run") != 0) &&
       (strcmp(s->u.header.name->u.name.id->symbol, "main") != 0))
    {
	fprintf(fp, "            case %s:\n", get_handler_name(s->u.header.name->u.name.id->symbol));
	fprintf(fp, "                {\n");
	expr_traverse(s->u.header.e1, NULL, dump_arg_decl);
	expr_traverse(s->u.header.e1, NULL, dump_arg_unmarsh);

	if(s->u.header.tp->kind != VOID_T)
	{
	    fprintf(fp, "                    ");
	    handle_type_var(s->u.header.tp, NULL, NULL, GETTYPE, 0);
	    fprintf(fp, " result = ");
	}
	else
	    fprintf(fp, "                    ");
	
	fprintf(fp, "((%s)ep.get_user_object()).%s(", servername, s->u.header.name->u.name.id->symbol);
	trav_nr = 0;
	expr_traverse(s->u.header.e1, NULL, dump_actual);
	fprintf(fp, ");\n");

	fprintf(fp, "                    int bufferSize = nexus.sizeof_int(1);\n");
	if(s->u.header.tp->kind != VOID_T)
	{
	    array_level = -1;
	    handle_type_var(s->u.header.tp, "result", NULL, GETSIZE, 20);
	}
	fprintf(fp, "                    PutBuffer outbuf = nexus.buffer_init(bufferSize, 0);\n");
	fprintf(fp, "                    outbuf.put_int(%s);\n", get_handler_name(s->u.header.name->u.name.id->symbol));
	if(s->u.header.tp->kind != VOID_T)
	{
	    array_level = -1;
	    handle_type_var(s->u.header.tp, "result", "outbuf", MARSHAL, 20);
	}
	fprintf(fp, "                    nexus.send_rsr(outbuf, replySP, RETURNHANDLER_ID, false);\n");
	fprintf(fp, "                }\n");
	fprintf(fp, "                break;\n");
    }
}

static void generate_skel(void)
{
    char *fn;
    int i;
    
    javar_out(2, "Generating skeleton");

    skelname = construct_name(servername, "Skel");
    fn = construct_filename(skelname);
    fp = fopen(fn, "w");
    output_file = fp;
    
    fprintf(fp, "package %s;\n\n", package_name);
    
    fprintf(fp, "import nexus.*;\n");
    dump_stmt(imports, fp);

    fprintf(fp, "public final class %s extends GeneralSkel\n", skelname);
    fprintf(fp, "{\n");
    fprintf(fp, "    public %s(Nexus nexus)\n", skelname);
    fprintf(fp, "    {\n");
    fprintf(fp, "        super(nexus);\n");
    fprintf(fp, "        start();\n");
    fprintf(fp, "    }\n\n");

    fprintf(fp, "    public void invoke_handler(Endpoint ep, GetBuffer inbuf, int handler_id)\n");
    fprintf(fp, "    {\n");
    fprintf(fp, "        %sUpcallThread handler = new %sUpcallThread(this, nexus, ep, inbuf, handler_id);\n",
	    skelname, skelname);
    fprintf(fp, "        handler.start();\n");
    fprintf(fp, "    }\n");
    fprintf(fp, "}\n\n");

    fprintf(fp, "class %sUpcallThread extends UpcallThread\n", skelname);
    fprintf(fp, "{\n");
    handler_id = 0;
    stmt_traverse(server, dump_handler_ids, NULL);
    fprintf(fp, "    private final int EXCEPTION_HANDLER_ID = 103;\n");
    fprintf(fp, "\n");
    
    fprintf(fp, "    public %sUpcallThread(%s skel, Nexus nexus, Endpoint ep, GetBuffer inbuf, int handler_id)\n",
	    skelname, skelname);
    fprintf(fp, "    {\n");
    fprintf(fp, "        super(skel, nexus, ep, inbuf, handler_id);\n");
    fprintf(fp, "    }\n\n");

    fprintf(fp, "    public void run()\n");
    fprintf(fp, "    {\n");
    fprintf(fp, "        Startpoint replySP = null;\n");
    fprintf(fp, "        try\n");
    fprintf(fp, "        {\n");
    fprintf(fp, "            replySP = new Startpoint(nexus, inbuf);\n");
    fprintf(fp, "            int offset = inbuf.get_int();\n");
    fprintf(fp, "            int key = inbuf.get_int();\n\n");
    
    fprintf(fp, "            switch(key)\n");
    fprintf(fp, "            {\n");
    marshal_ex_pos = 0;
    unmarshal_ex_pos = 0;
    stmt_traverse(server, dump_skel_case, NULL);
    
    fprintf(fp, "            case SHUTDOWN_ID:\n");
    fprintf(fp, "                {\n");
    fprintf(fp, "                    skel.shutdown();\n");
    fprintf(fp, "                }\n");
    fprintf(fp, "            default:\n");
    fprintf(fp, "                {\n");
    fprintf(fp, "                    System.err.println(\"Invalid handler ID.\");\n");
    fprintf(fp, "                }\n");
    fprintf(fp, "                break;\n");
    fprintf(fp, "            }\n");
    fprintf(fp, "        }\n");
    fprintf(fp, "        catch(Exception e)\n");
    fprintf(fp, "        {\n");
    fprintf(fp, "            try\n");
    fprintf(fp, "            {\n");
    fprintf(fp, "                String exname = e.getClass().getName();\n");
    fprintf(fp, "                int bufsize = nexus.sizeof_int(1);\n");
    fprintf(fp, "                bufsize += Nexusize.nexusSizeofString(exname);\n");
    fprintf(fp, "                PutBuffer buf = nexus.buffer_init(bufsize, 0);\n");
    fprintf(fp, "                buf.put_int(EXCEPTION_HANDLER_ID);\n");
    fprintf(fp, "                Nexusize.nexusWriteString(exname, buf);\n");
    fprintf(fp, "                nexus.send_rsr(buf, replySP, RETURNHANDLER_ID, false);\n");
    fprintf(fp, "            }\n");
    fprintf(fp, "            catch(BufferOverrunException ne)\n");
    fprintf(fp, "            {\n");
    fprintf(fp, "                ne.printStackTrace();\n");
    fprintf(fp, "                System.exit(1);\n");
    fprintf(fp, "            }\n");
    fprintf(fp, "        }\n");
    fprintf(fp, "    }\n");
    fprintf(fp, "}\n");
    fclose(fp);
    free(fn);
    free(skelname);
}

static void ss_trav(stmt_ptr s)
{
    if(s->kind == PCK_S)
    {
	package_name = (char *)malloc(strlen(s->u.arg->u.name.entry->symbol) * sizeof(char));
	strcpy(package_name, s->u.arg->u.name.entry->symbol);
    }

    if((s->kind == CLASS_S) && (s->u.header.mod & PUB_B))
	server = s;
}

void make_stub_skeleton(void)
{
    int i;
    
    server = NULL;
    stmt_traverse(comp_unit, ss_trav, NULL);
    imports = comp_unit->u.concat.s2->u.concat.s1;
    
    servername = construct_servername();

    if(server)
    {
	generate_stub();
	generate_skel();
    }
    else
	javar_out(-1, "No public server implementation found in file %s\n", filename);

    free(servername);
    free(package_name);
    if(string)
	free(string);
}
