

/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** symbol.c : symbol table routines
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

/* ********************************************************
   *** INCLUDE FILES and DEFINITIONS                    ***
   ********************************************************  */

#include "global.h"
#include "y.tab.h"

#define HASHENTRIES 211 

/* Following test uses *only* the simple name of the class */

#define IS_CURCLASS(e) ((cur_class) && (cur_class -> u.header.name) && \
  	               ((e) == cur_class -> u.header.name -> u.name.entry))

/* ********************************************************
   *** EXTERNAL VARIABLES                               ***
   ******************************************************** */

/* PRIVATE
   ******* */

static symb_ptr table[HASHENTRIES];
static int      count[HASHENTRIES], symcount;

static int     *scopes = NULL, nscope, scope_pt;

static info_ptr last_info;

/* ********************************************************
   *** PRIVATE FUNCTIONS                                ***
   ******************************************************** */

/* A simple Hash Function 
   ********************** */

static int hash(char *s) {
  int  len, i, result = 0;
 
  if (! s) /* Safety */
    javar_out(-1, "incorrect invocation of hash()");
  
  len = strlen(s);
  for (i = 0; i < len; i++) 
     result = ((256 * result + s[i]) % HASHENTRIES);

  return result;
}

/* Allocate a New Symbol Node
   (makes copy of ID symbol)
   ************************* */

static symb_ptr new_symbol(char *s, symb_ptr next) {
  symb_ptr new  = (symb_ptr) make_mem(sizeof(struct symb_node));
  new -> next   = next;
  new -> symbol = (char *)strdup(s);
  new -> info   = NULL;
  return new;
}

/* Search Table for Symbol
   *********************** */

static symb_ptr search_table(char *s) {
  symb_ptr entry = table[ hash(s) ];
  for ( ; entry; entry = entry -> next) 
    if (strcmp(s , entry -> symbol) == 0) 
      return entry; 
  return NULL;
}

/* In Scope Test
   ************* */

static short in_scope(short s) {
  if (s > 0) {
    int i;
    for (i = 0; i < scope_pt; i++)
      if (scopes[i] == s)
        return 1;
    return 0;
  }
  else 
    return 1;
}

/* Get Info Node from Table
   ************************ */

static int cnt_info_table(symb_ptr entry, short info, expr_ptr args) {
  info_ptr scan = entry -> info;
  int      cnt  = 0;

  last_info = NULL;

  for ( ; scan; scan = scan -> next) 
    if ( (scan -> info == info)    && 
	 (in_scope(scan -> scope)) &&
	 ((info != METH_N) || 
	  ((scan -> declstmt) && 
	   (arg_match(scan -> declstmt -> u.header.e1, args)))) ) {
      cnt++;
      last_info = scan;
    }
  return cnt;
}

/* Handle a Determined Meaning
   *************************** */

static void handle_meaning(expr_ptr n, short info, expr_ptr args, short stat_env) {

   if (cnt_info_table(n -> u.name.id, info, args) == 1) {

     stmt_ptr ds   = last_info -> declstmt;
     short    is_m = (info == METH_N);

     if (ds) {
       type_ptr tp     = (is_m) ? ds -> u.header.tp  : ds -> u.decl.tp;
       short    my_mod = (is_m) ? ds -> u.header.mod : ds -> u.decl.mod;

       if ((stat_env) && (! (my_mod & STA_B)))
         javar_out(0, "invalid static reference %s%s", 
                       id_table(n -> u.name.entry), (is_m) ? "()" : "");
       else {
         n -> type         = tp;                     /* Lookup Type and Scope */
         n -> u.name.scope = last_info -> scope;
       }
     }
     else
       javar_out(-1, "lost declaration of %s%s", 
  	              id_table(n -> u.name.entry), (is_m) ? "()" : "");
  }
  else if (info == LOCPAR_N)
    javar_out(-1, "ambigious use of %s", id_table(n -> u.name.entry));

}

/* ********************************************************
   *** PUBLIC FUNCTIONS                                 ***
   ******************************************************** */

void add_dummy_decl(expr_ptr loc) {
  if ((! loc) || (loc -> kind != ID_) || (loc -> u.name.info != LOCPAR_N))
    javar_out(-1, "incorrect use of add_dummy_decl()");
  else if (cnt_info_table(loc -> u.name.entry, LOCPAR_N, NULL) == 1) { 
    stmt_ptr d = last_info -> declstmt;

    if (! d -> u.decl.init) {
      type_ptr t = d -> u.decl.tp;

      if (! t)
        javar_out(-1, "lost type in add_dummy_decl()");

      if (t -> kind == BOOL_T)
        d -> u.decl.init = make_literal(BOOL_T,   "false", 0);
      else if (t -> kind == LONG_T)
        d -> u.decl.init = make_literal(LONG_T,   "0L",    0);
      else if ((IS_INTEGRAL1(t)) || (t -> kind == CHAR_T))
        d -> u.decl.init = make_literal(INT_T,    "0",     0);  /* possibly narrowed */
      else if (t -> kind == FLOAT_T)
        d -> u.decl.init = make_literal(FLOAT_T,  "0.0f",  0);
      else if (t -> kind == DOUBLE_T)
        d -> u.decl.init = make_literal(DOUBLE_T, "0.0d",  0);
      else
        d -> u.decl.init = make_literal(-2,       "null",  0);
    }
  }
  else
    javar_out(-1, "cannot find local in add_dummy_decl()");
}

/* Determine Meaning of a Qualified Name Q.id  or  Q.id()   (RULE 1, 2, or 3)
		         or expressions (Q).id or (Q).id()  (RULE 3 only)
   Handles only the simple cases:
   1 if Q is a package:
     generate an error
   2 if Q is a class/interface AND Q is simple name of current class: 
     lookup field/method, test static declaration, lookup type
   3 if Q is an expression name or, likewise, if (Q) is an expression AND
     (type of) Q is simple name of current class: 
     lookup field/method, lookup type
   ***************************************************** */

void det_meaning(expr_ptr left, expr_ptr n, short pot_info, expr_ptr args) {
  short is_m = (pot_info == METH_N);

  if ((left -> kind == '.') ||
      (left -> u.name.info == EXPR_N)   || 
      (left -> u.name.info == LOCPAR_N) ||
      (left -> u.name.info == FIELD_N)) { /* EXPRESSION NAME (RULE 3) */

     if (left -> kind == '.')
       left = left -> u.op.lhs;

     if (! IS_REF(left -> type)) 
       javar_out(0, "invalid %s", (is_m) ? "method-invocation" : "field-access");
     else if (left -> type) {
       expr_ptr c = left -> type -> u.name;

       if ((c) && (IS_CURCLASS(c -> u.name.entry)))
         handle_meaning(n, pot_info, args, 0);

     }
     else if ((left -> kind == ID_) && (left -> u.name.info == TYPE_N) &&
	      (IS_CURCLASS(left -> u.name.entry))) {
       handle_meaning(n, pot_info, args, 1);
    }
  }
  else if (left -> u.name.info == PACK_N) {    /* PACKAGE (RULE 1) */

    javar_out(0, "invalid use of package name in %s%s", 
		 id_table(n -> u.name.entry), (is_m) ? "()" : "");
  }
  else if (left -> u.name.info == TYPE_N) {    /* TYPE    (RULE 2) */

    if (IS_CURCLASS(left -> u.name.entry)) 
      handle_meaning(n, pot_info, args, 1);
  }
}

/* *****************************************
   *** Activation/Deactivation of Labels ***
   ***************************************** */

short activate_lab(stmt_ptr s) {
  short hit = 0;

  if (cnt_info_table(s -> u.lab.label -> u.name.entry, LABEL_N, NULL)) { 
    if (last_info -> declstmt)
      hit = 1;
    else
      last_info -> declstmt = s;
  }
  else
    set_info_table(s -> u.lab.label, s);

  return hit;
}

void deactivate_lab(expr_ptr lab) {
  if (cnt_info_table(lab -> u.name.entry, LABEL_N, NULL))
    last_info -> declstmt = NULL;
}

void check_target(expr_ptr nm) {
  if ((nm) && (cnt_info_table(nm -> u.name.entry, LABEL_N, NULL))) { 
    stmt_ptr s = last_info -> declstmt;

    if ( (s) && (s -> u.lab.lstmt) &&
  	        (s -> u.lab.lstmt -> kind != WHILE_S)   && 
	        (s -> u.lab.lstmt -> kind != DOWHILE_S) && 
	        (s -> u.lab.lstmt -> kind != FOR_S)     &&
	(! ((s -> u.lab.lstmt -> kind == ANNOT_S) &&
            (s -> u.lab.lstmt -> u.annot.body)    &&
	    (s -> u.lab.lstmt -> u.annot.f2 != -1)))     )
      javar_out(0, "target of continue must be while/do/for");
  }
}


/* ******************************************************
   *** Same Types in (formal or actual) argument-list ***
   ****************************************************** */

short arg_match(expr_ptr a1, expr_ptr a2) {
  if (a1) {
    if (a2) {
      if (a1 -> kind == ',')
	return ( (a2 -> kind == ',') &&
		 (arg_match(a1 -> u.op.lhs, a2 -> u.op.lhs)) &&
		 (arg_match(a1 -> u.op.rhs, a2 -> u.op.rhs)) );
      else
	return (equal_type(a1 -> type, a2 -> type));
    }
    else
      return 0;
  }
  else
    return (a2 == NULL);
}

/* **********************
   *** Scope Routines *** 
   ********************** */

void set_scope(short s) {
  static int mscope = 0;

  if (s == -1) /* Safety */
    javar_out(-1, "incorrect scope in set_scope()");

  if (scope_pt >= mscope) {
    mscope += 50;
    if (scopes)  
     scopes = (int *) more_mem(scopes, mscope * sizeof(int));  
    else  
     scopes = (int *) make_mem(mscope * sizeof(int));  
  }
  scopes[scope_pt++] = s;
}

short new_scope(void) {
  return ++nscope;
}

void old_scope(void) {
  if (scope_pt == 0) /* Safety */
    javar_out(-1, "scope underflow in old_scope()");
  --scope_pt;
}

/* Set an Info Node in Table
   ************************* */

short set_info_table(expr_ptr name, stmt_ptr declstmt) {

  if ((! name) || (name -> kind != ID_)) /* Safety */
    javar_out(-1, "no name in set_info_table()"); 
  else {

    symb_ptr entry = name -> u.name.entry;
    short    info  = name -> u.name.info;
    short    scope = 0;

    if (info == LOCPAR_N) {
      if (scope_pt > 1)
        scope = scopes[scope_pt-1];
      else
        javar_out(-1, "local/parameter occurs outside context");
    }
    else if ((info == FIELD_N) || (info == METH_N)) {

      if (scope_pt > 0) {

	javar_out(2, "member/constructor in %s class %s", 
	     (scope_pt == 1)? "the outermost": "an inner", id_table(entry));

        scope = scopes[scope_pt-1];
      }
      else
        javar_out(-1, "member/constructor occurs outside context");

    }
    else if (info == TYPE_N) {

      if (scope_pt > 0) 
        scope = scopes[scope_pt-1];

      javar_out(2, "%s class/interface definition is %s",  id_table(entry),
			(scope_pt == 0) ? "outermost" : "innermost");
    }
    else if (info == -1)
      javar_out(-1, "invalid invocation of set_info_table()");

    if (cnt_info_table(entry, info, ((info == METH_N) && (declstmt))
				    ? declstmt -> u.header.e1 : NULL)) {
#ifdef DEBUG_SCOPE
      int i;
      for (i = 0; i < scope_pt; i++)
        printf("%i ", scopes[i]);
      printf(" %s @ %i\n", id_table(entry), scope);
#endif
      return 1;
    }
    else {
  
      info_ptr new = (info_ptr) make_mem(sizeof(struct info_node));
    
      /* Record in Table */
    
      new -> info     = info;
      new -> scope    = scope;
      new -> next     = entry -> info;
      new -> declstmt = declstmt;
  
      entry -> info = new;
    }
  }
  return 0;
}


/* Initialization of Symbol Table
   ****************************** */

void set_table(void) {
  int i; 

  for (i = 0; i < HASHENTRIES; i++) {
    table[i] = NULL;
    count[i] = 0; 
  }
  symcount = 0;

  scope_pt = nscope = 0;
}

/* Deletion of Symbol Table Contents 
   ********************************* */

void del_info_table(void) {
  int      i;
  symb_ptr sscan;
  info_ptr iscan, inext = NULL;

  for (i = 0; i < HASHENTRIES; i++) 
    for (sscan = table[i]; sscan; sscan = sscan -> next) {
       for (iscan = sscan -> info; iscan; iscan = inext) {
          inext = iscan -> next;
          free(iscan);
       }
      sscan -> info = NULL;  
    }
}

void del_table(void) {
  int      max = 0, unused = 0, i;
  symb_ptr sscan,   snext  = NULL;

  del_info_table(); 

  for (i = 0; i < HASHENTRIES; i++) {

    if (count[i] > max) 
      max = count[i]; 
    else if (count[i] == 0) 
      unused++;

    for (sscan = table[i]; sscan; sscan = snext) {
       snext = sscan -> next;
       free(sscan -> symbol);
       free(sscan);
    }
    table[i] = NULL; 
  }
  javar_out(2, "%i symbols to symbol table [max. items: %i, free cells: %i, scopes %i]", 
		 symcount, max, unused, nscope);
}

/* Insert ID in Symbol Table
   ************************* */

symb_ptr insert_table(char *s) {
  symb_ptr entry = search_table(s);

  if (! entry) {
    /* New Symbol */
    int h = hash(s);
    entry = table[h] = new_symbol(s, table[h]);
    count[h]++;
    symcount++;
  }
  return entry;
}

/* Construct Unique ID (using initial as prefix)
   DYNAMIC MEM ALLOCATION -> no restrictions on length
   *************************************************** */

symb_ptr unique_table(char *initial) {
  static int lab = 0; 

  char *new_id  = (char *) make_mem(sizeof(char) * (strlen(initial) + 10));
  symb_ptr res  = NULL;
  short    busy = 1;

  do {
    sprintf(new_id, "%s_%i", initial, lab);

    if (search_table(new_id))
      lab++;
    else
      busy = 0;

  } while (busy);

  res = insert_table( new_id ); 

  free(new_id);

  return res;
}

/* Return ID of Symbol Table Entry
   ******************************* */

char *id_table(symb_ptr entry) {
  if (! entry) /* Safety */
    javar_out(-1, "null symbol-table entry in id_table()");
  return entry -> symbol;
}

/* *******************************************************
   *** Classification, REClassification and meaning of ***
   *** NAMES following the rules given in chapter 6    ***
   *** of the JAVA Language Specification              ***
   ***                                                 ***
   *** set info and scope of name                      ***
   *************************************************************
   *** NOTE THAT IN THIS IMPLEMENTATION                      ***
   *** ID's in declarations, field/method access constructs, *** 
   *** and statement labels are also handled as NAMES        ***
   ************************************************************* */

void classify(expr_ptr n, short info) {

  expr_ptr lcomp = n -> u.name.lcomp, args = NULL;
  symb_ptr entry = n -> u.name.entry;

  if (lcomp) {

    /* Recursive Rules for Qualified Name
       ********************************** */

    if ((info == PACK_N) || (info == TYPE_N))
      classify(lcomp, PACK_N);
    else 
      classify(lcomp, AMBI_N);

    if (info == AMBI_N) {

      /* Re-classification of Qualified Name 
	 *********************************** */
  
      if (lcomp -> u.name.info == PACK_N) {

         /* Further Information is lacking */

	 ;

      } 
      else if ( (lcomp -> u.name.info == TYPE_N)   ||
                (lcomp -> u.name.info == EXPR_N)   ||
                (lcomp -> u.name.info == LOCPAR_N) ||
                (lcomp -> u.name.info == FIELD_N)   )
        info = EXPR_N;
    }
  } 
  else if ((info == AMBI_N) || (info == EXPR_N)) {

    /* Re-classification of Simple Name 
       *********************************** 
       ***  + check local
       ***  + check field 
       ***  + check type */ 

       if ( (cnt_info_table(entry, LOCPAR_N, NULL)  == 1) ||
           ((cnt_info_table(entry, FIELD_N,  NULL)) == 1) ||
           ((cnt_info_table(entry, TYPE_N,   NULL)) == 1) ) 
         info = last_info -> info;
  }  

  /* Record Info in AST-node */

  n -> u.name.info = info;

  /* Determine Meaning
     ***************** */

  switch (info) {

    case PACK_N: 
    case TYPE_N:     /* Further Information is Lacking */

       n -> u.name.scope = 0;
       set_info_table(n, NULL);
       break;

    case EXPR_N:     /* Q.id or id */

       if (lcomp)  
	 det_meaning(lcomp, n, FIELD_N, NULL);    
       break;

    case METH_N:    /* Q.id() or id() OR id in method declaration  */

	if ((cur_inv) && (cur_inv -> u.op.lhs == n))
	  args = cur_inv -> u.op.rhs;
        else if ((cur_method) && (cur_method -> u.header.name == n))
	  args = cur_method -> u.header.e1;
        else
	  javar_out(-1, "lost method context");

    case FIELD_N:    /* FALL THROUGH */
    case LOCPAR_N:

      if (! lcomp) { /* Simple Name */
	handle_meaning(n, info, args, ((info != LOCPAR_N) && (cur_method) && 
			               (cur_method -> u.header.mod & STA_B)) );
      }
      else if (info == METH_N)                 /* Qualified Name */
	 det_meaning(lcomp, n, METH_N, args);
      break;

   case LABEL_N:  /* ID used as stament label */

      if ((cnt_info_table(entry, LABEL_N, NULL) == 1) &&
	  (last_info -> declstmt))
	n -> u.name.scope = 0;
      else 
        javar_out(0, "no enclosing statement label %s", id_table(entry)); 
      break;
  }
}

void show_names(void) {
  int      i;
  symb_ptr sscan;
  info_ptr iscan;

  fprintf(stderr, "\nMeaning of Names and Identifiers\n");
  fprintf(stderr, "--------------------------------\n");
  fprintf(stderr, "+ this option is mainly intended for debugging javar itself\n");
  fprintf(stderr, "+ the current version of javar may fail to determine"
		   " the meaning of names\n\n");

  for (i = 0; i < HASHENTRIES; i++) 
    for (sscan = table[i]; sscan; sscan = sscan -> next) 
      if (sscan -> info) {

       fprintf(stderr, "%s:\n", sscan -> symbol);

       for (iscan = sscan -> info; iscan; iscan = iscan -> next) 
	  switch(iscan -> info) {
	    case PACK_N:
	      fprintf(stderr, "  PACKAGE            (@scope %i)\n", iscan -> scope);
	      break;
	    case TYPE_N:
	      fprintf(stderr, "  TYPE               (@scope %i)\n", iscan -> scope);
	      break;
	    case FIELD_N:
	      fprintf(stderr, "  FIELD              (@scope %i)\n", iscan -> scope);
	      break;
	    case LOCPAR_N:
	      fprintf(stderr, "  LOCAL/PAR          (@scope %i)\n", iscan -> scope);
	      break;
	    case METH_N:
	      fprintf(stderr, "  METHOD/CONSTRUCTOR (@scope %i)\n", iscan -> scope);
	      break;
	    case LABEL_N:
	      fprintf(stderr, "  STATEMENT LABEL\n");
	      break;
	  }
     }
  fputc('\n', stderr);
}
