

/* *************
   *** JAVAR ***
   ****************************************************
   *** Copyright (c) 1996/1997                      ***
   *** Aart J.C. Bik             Indiana University *** 
   *** All Rights Reserved                          ***
   ****************************************************
   *** Please refer to the LICENSE file distributed ***
   *** with this software for further details on    ***
   *** the licensing terms and conditions.          ***
   ***                                              ***
   *** Please, report all bugs, comments, etc.      ***
   *** to: ajcbik@extreme.indiana.edu               ***
   ****************************************************
   *** unparse.c : output of AST
   ***
   ***
   *** Your courtesy in mentioning the use of this compiler 
   *** in any scientific work that presents results obtained
   *** by using (extensions or modifications of) this compiler
   *** is highly appreciated.
   ***
   *** NOTE! 
   *** This implementation does NOT provide a complete Java 
   *** front-end (unicode escapes are not supported
   *** and the semantic analysis is rather limited).
   ***
   *** */

/* This file has been modified from the original to support
 * Java 1.1 language.
 *
 *     Fabian Breg.
 */

/* ********************************************************
   *** INCLUDE FILES and DEFINITIONS                    ***
   ********************************************************  */

#include "global.h"
#include "y.tab.h"

/* ********************************************************
   *** EXTERNAL VARIABLES                               ***
   ******************************************************** */

FILE *output_file;

/* PRIVATE
   ******* */

static void show_expr(int, expr_ptr);
static void show_stmt(int, stmt_ptr);

static int  for_init = 0;
static int  e_out    = 0;

/* WARNING: the following tables depend on the 
   order in which enum constants are defined in global.h
   ***************************************************** */

#ifdef DEBUG_SCOPE
static char *classif_table[] = { 
  "AMBIGUOUS",  
  "package",    "type",    
  "EXPRESSION", "field", "local/par",
  "method", 
  "label",      "MISCF", "MISCM"
} ;
#endif

static char *type_table[] = { 
 "void", "boolean", "byte",  "short", "int",
 "long", "char",    "float", "double"
} ;

static char *simple_table[] = { 
  "import",   "import", "package", "break", 
  "continue", "return", "throw",
  "case",     "default"
} ;

static char *control_table[] = { 
  "if", "for", "switch", "while", 
  "synchronized", "try", "finally"
} ;

static char *mod_table[] = { 
  "public", "private",      "protected", 
  "static", "abstract",     "final",
  "native", "synchronized", "transient", "volatile" 
} ;

/* ********************************************************
   *** PRIVATE FUNCTIONS                                ***
   ******************************************************** */

/* Modifier Output 
   *************** */

static void show_modifiers(int mod) {
  int i, j;
  if (mod)
    for (i = 0, j = 1; i < 10; i++, j *= 2) 
      if (mod & j) 
        fprintf(output_file, "%s ", mod_table[i]);
}

/* Type Output
   ************************************************** */

static void show_type(type_ptr tp) {
  if (tp)
    switch (tp -> kind) {

      case C_OR_I_T:  
	show_expr(0, tp -> u.name); 
        break;

      case ARRAY_T:  
        show_type(tp -> u.aref); 
        fprintf(output_file, "[]");
        break;

      default:  
	fprintf(output_file, type_table[tp -> kind]);
	break;
    }
  else
    fprintf(output_file, "<null-type>");
}

/* Expression Output
   ************************************************** */

static void show_oper(short c) {
  switch (c) {

     case EQ_:     fprintf(output_file, " == ");   break;

     case NE_:     fprintf(output_file, " != ");   break;
     case AND_:    fprintf(output_file, " && ");   break;
     case OR_:     fprintf(output_file, " || ");   break;

     case PP_:     fprintf(output_file, "++");     break;
     case MM_:     fprintf(output_file, "--");     break;

     case PIS_:    fprintf(output_file, " += ");   break;
     case MINIS_:  fprintf(output_file, " -= ");   break;
     case MULIS_:  fprintf(output_file, " *= ");   break;
     case DIS_:    fprintf(output_file, " /= ");   break;
     case ANDIS_:  fprintf(output_file, " &= ");   break;
     case ORIS_:   fprintf(output_file, " |= ");   break;
     case UIS_:    fprintf(output_file, " ^= ");   break;
     case IDIVIS_: fprintf(output_file, " %%= ");  break; /* bug fix */

     case LE_:     fprintf(output_file, " <= ");   break;
     case GE_:     fprintf(output_file, " >= ");   break;
     case LLIS_:   fprintf(output_file, " <<= ");  break;
     case RRIS_:   fprintf(output_file, " >>= ");  break;
     case RRRIS_:  fprintf(output_file, " >>>= "); break;
     case LL_:     fprintf(output_file, " << ");   break;
     case RR_:     fprintf(output_file, " >> ");   break;
     case RRR_:    fprintf(output_file, " >>> ");  break;
     case '<':     fprintf(output_file, " < ");    break;
     case '>':     fprintf(output_file, " > ");    break;

     default:      fputc(' ', output_file);
		   fputc(c,   output_file);
		   fputc(' ', output_file);
		   break;
  }
}

static void show_expr(int inner, expr_ptr e) {
  
  if (e) {

    if (e_out++ > 80) { /* Prevents `line too long' */
      e_out = 0;
      fputc('\n', output_file);
    }

#ifdef DEBUG_TYPE
    if (e -> type) {
      fprintf(output_file, " < ");
      show_type(e -> type);
      fprintf(output_file, " > ");
    }
#endif

    switch(e -> kind) {

      /* LITERALS
	 ******** */

      case LITERAL_: 
	
	/* Using `%s' leaves conversion characters in literal unprocessed */

	fprintf(output_file, "%s", e -> u.lit.s); 
	break;

      /* Names 
	 ***** */

      case ID_:

#ifdef DEBUG_SCOPE
	fprintf(output_file, "[%s = %s @ %i]", 
	        id_table(e -> u.name.entry),
	        classif_table[e -> u.name.info], e -> u.name.scope);  
#endif
        if (e -> u.name.lcomp) {
          show_expr(inner, e -> u.name.lcomp);	
	  fputc('.', output_file);
        }

	/* Using `%s' leaves conversion characters in ID unprocessed */

	fprintf(output_file, "%s", id_table(e -> u.name.id));

	break;

      /* Declaration Statement EXPRESSIONS
	 ********************************* */

      case 'd': 

	if (e -> u.declstmt) { 

	  /* UGLY Hack to output declarations in
	     a special for-init construct while preserving
	     reasonably clean AST representation...
             ************************************************* */

	  show_modifiers(e->u.declstmt->u.decl.mod);
	  
	  if ((for_init == 0) || (for_init == 1)) {
	    if (for_init == 1)
	      for_init = 2;
  	  
	    show_type(e -> u.declstmt -> u.decl.tp);
	      fputc(' ', output_file);
          }

          show_expr(0, e -> u.declstmt -> u.decl.name);
	  if (e -> u.declstmt -> u.decl.init) {
	    fputc(' ', output_file);
	    fputc('=', output_file);
	    fputc(' ', output_file);
	    show_expr(1, e -> u.declstmt -> u.decl.init);
          }
        }
	break;

      /* TE Operators
	 ************* */

      case NEW_: 

	if (inner)
          fputc('(', output_file);

	fprintf(output_file, "new ");

	show_type(e -> u.te.tp);

	if ((e -> u.te.ex) && 
	    (e -> u.te.ex -> kind == ']')) {
	  show_expr(0, e -> u.te.ex -> u.op.lhs);
	  show_expr(0, e -> u.te.ex -> u.op.rhs);
        }
        else
	{
	  if(!(e->u.te.ex) || (e->u.te.ex->kind != '{'))
	      fputc('(', output_file);
	  show_expr(0, e -> u.te.ex);
	  if(!(e->u.te.ex) || (e->u.te.ex->kind != '{'))
	      fputc(')', output_file);
        }

	if(e->u.te.st)
	{
	    fputc('{', output_file);
	    show_stmt(0, e->u.te.st);
	    fputc('}', output_file);
	}
	
	if (inner)
          fputc(')', output_file);
	break;

      case INSTOF_:

	if (inner)
          fputc('(', output_file);

	show_expr(1, e -> u.te.ex);
	fprintf(output_file, " instanceof ");
	show_type(e -> u.te.tp);

	if (inner)
          fputc(')', output_file);
	break;

      case 'c': /* CAST */

	if (inner)
          fputc('(', output_file);

	fputc('(', output_file);
	show_type(e -> u.te.tp);
	fputc(')', output_file);
	fputc(' ', output_file);
	show_expr(1, e -> u.te.ex);

	if (inner)
          fputc(')', output_file);
	break;

      case 't': /* type-list */

	if (e -> u.te.ex) {
	  show_expr(0, e -> u.te.ex);
	  fputc(',', output_file);
	  fputc(' ', output_file);
        }
	show_type(e -> u.te.tp);
	break;

      case 'l': /* class literals */

	if(inner)
	    fputc('(', output_file);
	show_type(e->u.te.tp);
	fputc('.', output_file);
	show_expr(0, e->u.te.ex);
	if(inner)
	    fputc(')', output_file);
	break;
	
      /* List-like Operators
	******************** */

      case 'i': /* method-invocation */

	show_expr(0, e -> u.op.lhs);
	fputc('(', output_file);
	show_expr(0, e -> u.op.rhs);
	fputc(')', output_file);
	break;

      case '{': /* array-initializer */

	if (e -> u.op.lhs)
	  javar_out(-1, "AST of {} corrupt in show_expr()");

	fputc('{', output_file);
	show_expr(0, e -> u.op.rhs);
	fputc('}', output_file);
	break;

      case '[': /* array subscript */

	show_expr(1, e -> u.op.lhs);
	fputc('[', output_file);
	show_expr(0, e -> u.op.rhs);
	fputc(']', output_file);
	break;

      case ']': 
      case ':':
	javar_out(-1, "construct should never be shown directly");
	break;

      case '?': /* Conditional Expression */

	if ((! e -> u.op.rhs) || (e -> u.op.rhs -> kind != ':'))
	  javar_out(-1, "AST of ?:-operator corrupt in show_expr()");

	if (inner)
          fputc('(', output_file);

	show_expr(1, e -> u.op.lhs);
	fputc(' ', output_file);
	fputc('?', output_file);
	fputc(' ', output_file);
	show_expr(1, e -> u.op.rhs -> u.op.lhs);
	fputc(' ', output_file);
	fputc(':', output_file);
	fputc(' ', output_file);
	show_expr(1, e -> u.op.rhs -> u.op.rhs);

	if (inner)
          fputc(')', output_file);

        break;

      case '.':

	if ((! e -> u.op.rhs) ||
	    ((e -> u.op.rhs -> kind != ID_) &&
	     (e->u.op.rhs->kind != NEW_) &&
	     (e->u.op.rhs->kind != THIS_)))
	  javar_out(-1, "AST of '.'-operator corrupt in show_expr()");

      case ',': /* FALL THROUGH,    Semi-Unbracketed Lists */
      case ';': 

	show_expr((e -> kind == '.'), e -> u.op.lhs);
	fputc(e -> kind, output_file);
	if (e -> kind != '.')
  	  fputc(' ', output_file);
	show_expr(0, e -> u.op.rhs);
	break;

      /* Default Operators 
	 ***************** */

      case THIS_:

	fprintf(output_file, "this");
	break;

      case SUPER_:

	fprintf(output_file, "super");
	break;

      case '$':
	show_expr(1, e->u.op.lhs);
	fputc('$', output_file);
	show_expr(1, e->u.op.rhs);
	break;
	
      default:

	if (inner)
          fputc('(', output_file);

	show_expr(1, e -> u.op.lhs);
	show_oper(e -> kind);
	show_expr(1, e -> u.op.rhs);

        if (inner)
          fputc(')', output_file);
	break;
    } 
  }
}

/* Statement Output
   ************************************************** */

static void indent(int level) {
  if (level > 60) level = 60;
  while (level-- > 0)
     fputc(' ', output_file);
}

static void show_comment(char *c, int level) {

  fputc('\n', output_file);

  indent(level);

  while (*c) {
    if (*c == '\n') {
      fputc('\n', output_file);
      indent(level);
    }
    else
      fputc(*c, output_file);
    c++;
  }

  fputc('\n', output_file);
}

static void show_stmt(int level, stmt_ptr s) {

 if (s) {

  e_out = 0;

  if (s -> kind == CONCAT_S) {
    show_stmt(level, s -> u.concat.s1);
    show_stmt(level, s -> u.concat.s2);
  }
  else {

    if (s -> comment >= 0) 
      show_comment(comment_buffer + s -> comment, level);
    else if ((s -> kind == CLASS_S)  ||
	(s -> kind == INTERF_S) ||
	(s -> kind == CASE_S)   ||
	(s -> kind == DEF_S)    ||
	(s -> kind == CONS_S)   || 
	(s -> kind == METHOD_S)  )
      fputc('\n', output_file);

    indent(level);

    switch(s -> kind) {

      /* Special Constructs
	 ****************** */

      case ANNOT_S:

	if (s -> u.annot.f2 == -1) {
          fprintf(output_file, "// PARALLEL INVOCATIONS\n");
          show_stmt(level, s -> u.annot.body);
        }
	else if (s -> u.annot.body) {
          fprintf(output_file, "// PARALLEL LOOP\n");
          show_stmt(level, s -> u.annot.body);
        }
        else if (s -> u.annot.arg) {
          fprintf(output_file, "// WAIT %li,", s -> u.annot.f1);
	  show_expr(0, s -> u.annot.arg);
          fprintf(output_file, "\n");
        }
        else 
          fprintf(output_file, "// POST %li\n", s -> u.annot.f1);
	break;

      case LAB_S:

        show_expr(0, s -> u.lab.label); 
        fputc(':',  output_file);
        fputc('\n', output_file);
	show_stmt(level, s -> u.lab.lstmt);
	break;

      /* Simple Statements 
	 ***************** */

      case STIMP_S: 
      case IMPOD_S:
      case PCK_S:
      case BRK_S:
      case CONT_S:
      case RET_S:
      case THROW_S:
      case CASE_S:
      case DEF_S:

	fprintf(output_file, "%s", simple_table[s -> kind - STIMP_S]);
	if (s -> u.arg) 
	  fputc(' ', output_file); 

      case EXP_S:           /* FALL THROUGH */

	show_expr(0, s -> u.arg);

	if (s -> kind == IMPOD_S)
	  fprintf(output_file, ".*;\n");
        else if ((s -> kind == CASE_S) || (s -> kind == DEF_S)   )
	  fprintf(output_file, ":\n\n");
        else
          fprintf(output_file, ";\n");
	break;

      case EMPTY_S:
        fprintf(output_file, ";\n");
	break;

      /* Control statements 
	 ******************* */

      case IF_S:
      case FOR_S:
      case SWITCH_S:
      case WHILE_S:
      case SYNC_S:
      case TRY_S:
      case FIN_S:

	fprintf(output_file, "%s ", control_table[s -> kind - IF_S]);

	if (s -> kind == FOR_S) {
	  for_init = 1;

#ifdef DEBUG_SCOPE
          fprintf(output_file, " [scope >> %i] ", s -> u.control.scope);
#endif
        }

	if (s -> u.control.e) {
    	  fputc('(', output_file);
	  show_expr(0, s -> u.control.e);
    	  fputc(')', output_file);
        }
  	fputc('\n', output_file);

	for_init = 0;

	show_stmt(level+2, s -> u.control.s1);

        if (s -> u.control.s2) {
	  if (s -> kind == IF_S) {
            indent(level); 
	    fprintf(output_file, "else\n");
          }
          show_stmt(level+2, s -> u.control.s2);
	}
	break;

      case DOWHILE_S:
	
	fprintf(output_file, "do\n");
        show_stmt(level+2, s -> u.control.s1);

	indent(level);
	fprintf(output_file, "while (");
	show_expr(0, s -> u.control.e);
	fprintf(output_file, ");\n");
	break;

      /* Declaration Statements
	 ********************** */

      case DECL_S:

	show_modifiers(s -> u.decl.mod); 
	show_type(s -> u.decl.tp);
	fputc(' ', output_file);
	show_expr(0, s -> u.decl.name);

	if (s -> u.decl.init) {
	  fputc(' ', output_file);
	  fputc('=', output_file);
	  fputc(' ', output_file);
	  show_expr(1, s -> u.decl.init);
        }
	fprintf(output_file, ";\n");
	break;

      /* Headers
	 ******* */

      case CLASS_S:
      case INTERF_S:

	show_modifiers(s -> u.header.mod);

	fprintf(output_file, (s -> kind == CLASS_S) ? "class " : "interface ");

	show_expr(0, s -> u.header.name); 

	if (s -> u.header.tp) {
	  fprintf(output_file, " extends ");
	  show_type(s -> u.header.tp);
        }
	if (s -> u.header.e1) {
	  fprintf(output_file, (s -> kind == CLASS_S) ? " implements " : " extends ");
	  show_expr(0, s -> u.header.e1);
        }

	fputc(' ', output_file);

      case BLOCK_S:       /* FALL THROUGH */      
      case SINIT_S:    
      case CATCH_S:                 

	if (s -> kind == SINIT_S)
	  fprintf(output_file, "static ");
        else if (s -> kind == CATCH_S) {
	  fprintf(output_file, "catch(");
	  show_expr(0, s -> u.header.e1);
	  fprintf(output_file, ") ");
        }

#ifdef DEBUG_SCOPE
        fprintf(output_file, " [scope >> %i] ", s -> u.header.scope);
#endif

	if (s -> u.header.body) {
	  fprintf(output_file, "{\n");
	  show_stmt(level+2, s -> u.header.body);
	  indent(level);
	  fprintf(output_file, "}\n"); 
        }
        else
	  fprintf(output_file, "{ }\n"); 
	break;

      case CONS_S:
      case METHOD_S:

        show_modifiers(s -> u.header.mod);

        if (s -> u.header.tp) {
          show_type(s -> u.header.tp);
          fputc(' ', output_file);
        }

	show_expr(0, s -> u.header.name);

#ifdef DEBUG_SCOPE
        fprintf(output_file, " [scope >> %i] ", s -> u.header.scope);
#endif
	fputc('(', output_file);
	show_expr(0, s -> u.header.e1);
	fputc(')', output_file);

  	if (s -> u.header.e2) {
	  fprintf(output_file, " throws ");
	  show_expr(0, s-> u.header.e2);
        }

	if (s -> u.header.mod & (ABS_B | NAT_B)) {
          fprintf(output_file, ";\n");
	  if (s -> u.header.body)
	    javar_out(-1, "AST corrupt, body for abstract/native method");
        }
	else {
          fprintf(output_file, " {\n");
          show_stmt(level+2, s -> u.header.body);
	  indent(level);
          fprintf(output_file, "}\n");
        }
        break;

      /* Default Action
	 ************** */

      default:  
	javar_out(-1, "AST corrupt in show_stmt() %i", s -> kind); 
     }
   }
  }
}

/* ********************************************************
   *** PUBLIC FUNCTIONS                                 ***
   ******************************************************** */

/* Output of Types, Expressions and Statements to a file */

void dump_type(type_ptr t, FILE *outf) {
  FILE *old = output_file; 
  output_file = (outf) ? outf : stderr;
  show_type(t);
  output_file = old; /* restore old output file
			(allows dump_type() nested *within* other dumps) */
}

void dump_expr(expr_ptr e, FILE *outf) {
  output_file = (outf) ? outf : stderr;
  show_expr(0, e);
}

void dump_stmt(stmt_ptr s, FILE *outf) {
  output_file = (outf) ? outf : stderr;
  show_stmt(0, s);
}
