/**
 * This is the composition tool's view of the component
 * @author Nirmal (nmukhi)
 */

package compositiontool.model;

import exceptions.InstantiateException;
import util.Info;
import util.EnvObj;
import ids.InformationInfo;
import registry.InstanceInfo;
import registry.InstanceXMLUtils;
import creation.CreationService;
import connection.ConnectionService;
import ccacore.ComponentID;
import ccacore.PortInfo;
import compositiontool.view.ComponentView;
import compositiontool.view.WorkingAreaController;
import compositiontool.view.ComponentCommunicator;
import java.util.Vector;
import java.lang.reflect.Field;
import java.util.Enumeration;

public class ComponentModel {
    private CreationService creationService;
    private ConnectionService connectionService;
    private ComponentView componentView;
    private ComponentID id;
    private String componentName;
    private String machineName;
    private Info info;
    private String creationMechanism;
    private String instanceId; // used to identify events associated with this component
    private Vector inputPorts; // list of PortModel objects
    private Vector outputPorts; // list of PortModel objects
    private String paramPortClassName;
    private Class paramClass;
    private Object paramObject;
    private String parameterPortName;
    private String parameterMethodName;
    private ParamFields [] paramFields;
    private Vector registeredParameterUsesPorts;
    private ComponentCommunicator componentCommunicator;
  private WorkingAreaController workingAreaController;

    /**
     * Create a ComponentModel given the creation and connection service
     * This will also create all the lists required
     */
    public ComponentModel(CreationService creationService,
			  ConnectionService connectionService,
			  ComponentCommunicator communicator,
			  WorkingAreaController workingAreaController) {
	this.creationService = creationService;
	this.connectionService = connectionService;
	this.componentCommunicator = communicator;
	this.workingAreaController = workingAreaController;
	this.id = null;
	inputPorts = new Vector();
	outputPorts = new Vector();
	System.out.println("Created component model, communicator is "+communicator);
    }

    /**
     * Create a component model from the info object
     * which describes the component
     */
    public ComponentModel(CreationService creationService,
			  ConnectionService connectionService,
			  ComponentCommunicator communicator,
			  WorkingAreaController controller,
			  InformationInfo componentInfo) {
	this(creationService,connectionService,communicator,controller);
	this.info = (Info)componentInfo;
	// get the component's name
	componentName = componentInfo.getName();
	// set the machine name and creation mechanism to null
	// they will be manipulated by the component view
	machineName = null;
	creationMechanism = null;
	// get the list of ports 
	Info portInformation = componentInfo.getInfo("component-info")[0].getInfo("port-list")[0];
	// get the port type descriptions
	Info [] portTypes = componentInfo.getInfo("component-info")[0].getInfo("port-type-list")[0].getInfo("port-type");
	// go through all the ports and create port models for them
	Info [] ports = portInformation.getInfo("port");
	for (int i=0; i<ports.length; i++) {
	    Info currentPort = ports[i];
	    // create the port model
	    PortModel currentPortModel = new PortModel(this,currentPort,connectionService);
	    System.out.println("Created port "+currentPortModel);
	    if (currentPortModel.isInput())
		inputPorts.add(currentPortModel);
	    else
		outputPorts.add(currentPortModel);
	}
  }
    
    /** 
     * Create a component model from the component id 
     * which refers to an instance of the component
     */
    public ComponentModel(CreationService creationService,
			  ConnectionService connectionService,
			  ComponentCommunicator communicator,
			  WorkingAreaController controller,
			  InstanceInfo component) {
	this(creationService,connectionService,communicator,controller);
	// set the name, machine name, creation mechanism
	// etc.
	componentName = component.getInfo("instance-info")[0].getValue("instance-name")[0];
	machineName = component.getInfo("instance-info")[0].getValue("target-host")[0];
	creationMechanism = component.getInfo("instance-info")[0].getValue("creation-mechanism")[0];
	instanceId = component.getInfo("instance-info")[0].getValue("uniqueID")[0];
	this.info = (Info)component;
	//Info creationDependantInfo = component.getInfo("creation-dep-info")[0];
	// get the keys here
	//String [] keysInCreationDepInfo = creationDependantInfo.getKeys();
	//for (int i=0; i<keysInCreationDepInfo.length; i++) {
	//    String currentKey = keysInCreationDepInfo[i];
	//  if (!currentKey.equals("exe-path") &&
	//!currentKey.equals("cmd-line-pars")) {
		// this key must refer to the creation mechanism
	//creationMechanism = currentKey;
	//  }
	//}
	// get the component id for the instance that 
	// this information describes
	id = component.getComponentID();
	// port information added to instance XML, see reason below
	// get the list of ports 
	Info portInformation = component.getInfo("instance-info")[0].getInfo("port-list")[0];
	// get the port type descriptions
	Info [] portTypes = component.getInfo("instance-info")[0].getInfo("port-type-list")[0].getInfo("port-type");
	// go through all the ports and create port models for them
	Info [] ports = portInformation.getInfo("port");
	for (int i=0; i<ports.length; i++) {
	    Info currentPort = ports[i];
	    // create the port model
	    PortModel currentPortModel = new PortModel(this,currentPort,connectionService);
	    System.out.println("Created port "+currentPortModel);
	    if (currentPortModel.isInput())
		inputPorts.add(currentPortModel);
	    else
		outputPorts.add(currentPortModel);
	}
	// commented out since getUSesPorts and getProvidesPorts does not work currently
	// this was why port information was added to the instance XML
	// nmukhi 12/17
// 	try {
// 	    // get the uses ports and create port models
// 	    // for them 
// 	    PortInfo [] usesPorts = id.getUsesPorts();
// 	    System.out.println("There are "+usesPorts.length+" uses ports");
// 	    for(int i=0; i<usesPorts.length; i++) {
// 		System.out.println("Adding uses port "+usesPorts[i].getName());
// 		PortModel currentPortModel = new PortModel(this,usesPorts[i],
// 							  connectionService,false);
// 		System.out.println("Port model is "+currentPortModel);
// 		outputPorts.add(currentPortModel);
// 	    }
// 	    // get the provides ports and create port models
// 	    // for them 
// 	    PortInfo [] providesPorts = id.getProvidesPorts();
// 	    for(int i=0; i<providesPorts.length; i++) {
// 		PortModel currentPortModel = new PortModel(this,providesPorts[i],
// 							  connectionService,true);
// 		inputPorts.add(currentPortModel);
// 	    }
// 	} catch (Exception exception) {
// 	    System.err.println("Got exception "+exception+" while getting ports from component id");
// 	    exception.printStackTrace();
// 	    }
	// add the executable and parameter information
	// get the name of the parameter class
	// check if this component does indeedy have parameters
	Info [] paramBlock = info.getInfo("instance-info")[0].getInfo("parameter-block");
	if (paramBlock.length != 0) {
	    String paramClassName = paramBlock[0].getValue("parameter-class")[0];
	    paramPortClassName = paramBlock[0].getValue("parameter-port-class")[0];
	    parameterPortName = info.getInfo("instance-info")[0].getInfo("parameter-block")[0].getValue("parameter-port-name")[0];
	    parameterMethodName = paramBlock[0].getValue("method-name")[0];
	    try {
		// load this class
		paramClass = Class.forName(paramClassName);
		// create an instance of this class
		paramObject = paramClass.newInstance();
		// get its fields
		Field [] classFields = paramClass.getDeclaredFields();
		// get the set of default values for the fields from the XML doc
		// and create the list of ParamField objects
		paramFields = new ParamFields[classFields.length];
		for (int i=0; i<paramFields.length;i++) {
		    paramFields[i] = new ParamFields();
		    paramFields[i].name = classFields[i].getName();
		    paramFields[i].className = classFields[i].getType().getName();
		    // look up the default value of this param
		    Info [] paramList = info.getInfo("instance-info")[0].getInfo("parameter-block")[0].getInfo("parameters");
		    for (int j=0; j<paramList.length; j++) {
			// see if this param's name is the same as the one we are filling in
			if (paramFields[i].name.equals(paramList[j].getValue("name")[0])) {
			    // aha, we got the param we wanted
			    // look up its default value
			    paramFields[i].defaultValue = 
				paramList[j].getValue("default-value")[0];
			    // initially set the real value to be the same
			    paramFields[i].realValue = paramFields[i].defaultValue;
			    System.out.println("Class name for param "+
					       paramFields[i].name+" is "+
					       paramFields[i].className);
			    break;
			}
		    }
		}
	    } catch (Exception exception) {
		System.err.println("Got exception "+exception+" while getting param information");
		exception.printStackTrace();
	    }
	}
    }

    public void updateInstanceView() {
	// if this is a component which can be executed, inform the component view to
	// add that button
	if (info.getInfo("instance-info")[0].getValue("executable")[0].equals("true")) {
	    System.out.println("This instance is executable, adding execute button");
	    componentView.addExecuteButton();
	}
	if (info.getInfo("instance-info")[0].getInfo("parameter-block").length != 0) {
	    // this is a component whose params can be set, inform the view to add
	    // that button
	    System.out.println("adding set params button");
	    componentView.addSetParamsButton();
	}
    }

    /**
     * Create an instance of this component
     */
    public void create() {
	// indicate to the component view that a creation is impending
	componentView.creationImpending();
	// get a creation environment for the provided machine name
	// and creation mechanism
	EnvObj creationEnv = 
	    ((InformationInfo)info).getCreationEnvironment(machineName,creationMechanism);
	// add a dummy id to the creation env to be used so we can identify 
	// the creation event 
	instanceId = creationEnv.get("user-name")+creationEnv.get("creation-time")+Math.random();
	creationEnv.put("instance-name",instanceId);
	try {
	    // create the instance
	    id = creationService.createInstance(creationEnv);
	} catch (InstantiateException exception) {
	    System.err.println("Could not create component, got exception "+exception);
	    exception.printStackTrace();
	    componentView.creationAborted();
	    return;
	}
	// if this is a component which can be executed, inform the component view to
	// add that button
	if (info.getInfo("component-info")[0].getValue("executable")[0].equals("true")) {
	    System.out.println("This component is executable, adding execute button");
	    componentView.addExecuteButton();
	}
	// get the name of the parameter class
	// check if this component does indeedy have parameters
	Info [] paramBlock = info.getInfo("component-info")[0].getInfo("parameter-block");
	if (paramBlock.length != 0) {
	    String paramClassName = paramBlock[0].getValue("parameter-class")[0];
	    paramPortClassName = paramBlock[0].getValue("parameter-port-class")[0];
	    parameterPortName = info.getInfo("component-info")[0].getInfo("parameter-block")[0].getValue("parameter-port-name")[0];
	    parameterMethodName = paramBlock[0].getValue("method-name")[0];
	    try {
		// load this class
		paramClass = Class.forName(paramClassName);
		// create an instance of this class
		paramObject = paramClass.newInstance();
		// get its fields
		Field [] classFields = paramClass.getDeclaredFields();
		// get the set of default values for the fields from the XML doc
		// and create the list of ParamField objects
		paramFields = new ParamFields[classFields.length];
		for (int i=0; i<paramFields.length;i++) {
		    paramFields[i] = new ParamFields();
		    paramFields[i].name = classFields[i].getName();
		    paramFields[i].className = classFields[i].getType().getName();
		    // look up the default value of this param
		    Info [] paramList = info.getInfo("component-info")[0].getInfo("parameter-block")[0].getInfo("parameters");
		    for (int j=0; j<paramList.length; j++) {
			// see if this param's name is the same as the one we are filling in
			if (paramFields[i].name.equals(paramList[j].getValue("name")[0])) {
			    // aha, we got the param we wanted
			    // look up its default value
			    paramFields[i].defaultValue = 
				paramList[j].getValue("default-value")[0];
			    // initially set the real value to be the same
			    paramFields[i].realValue = paramFields[i].defaultValue;
			    System.out.println("Class name for param "+
					       paramFields[i].name+" is "+
					       paramFields[i].className);
			    break;
			}
		    }
		}
	    } catch (Exception exception) {
		System.err.println("Got exception "+exception+" while getting param information");
		exception.printStackTrace();
	    }
	    // this is a component whose params can be set, inform the view to add
	    // that button
	    System.out.println("Finished reading param info, adding set params button");
	    componentView.addSetParamsButton();
	}
	// change the info object to now point to the information
	System.out.println("Updating info to instance info");
	Info [] paramInfoArray = info.getInfo("component-info")[0].getInfo("parameter-block");
	Info paramInfo = null;
	if (paramInfoArray.length != 0)
	    paramInfo = paramInfoArray[0];
	String xmlDescription = InstanceXMLUtils.createXMLFromInfo(id,getName(),
								   creationEnv,
								   info);
	System.out.println("instance info is "+xmlDescription);
	info = InstanceXMLUtils.createInstanceInfoFromXML(xmlDescription);
    }

    public ParamFields [] getParamFields() {
	return paramFields;
    }

    public void setParams(ParamFields [] paramFields) {
	// go through all the param fields
	// and set the values of the corresponding members of the param object
	for (int i=0; i<paramFields.length; i++) {
	    this.paramFields[i].realValue = paramFields[i].realValue;
	    // get the name of the field
	    String fieldName = paramFields[i].name;
	    String enteredValue = paramFields[i].realValue;
	    // create an object for this field in the param class
	    // the type of object to be created depends on the name of the
	    // class of the field
	    Object value=null;
	    String className = paramFields[i].className;
	    if (className.equals("boolean")) {
		// create a Boolean object
		value = Boolean.valueOf(enteredValue);
	    } else if (className.equals("double")) {
		// create a Double object
		try {
		    value = Double.valueOf(enteredValue);
		} catch (NumberFormatException exception) {
		    System.out.println("Could not convert param value "+enteredValue+" into a Double");
		    exception.printStackTrace();
		    value = new Double(0.0d);
		}
	    } else if (className.equals("float")) {
		// create a Float object
		try {
		    value = Float.valueOf(enteredValue);
		} catch (NumberFormatException exception) {
		    System.out.println("Could not convert param value "+enteredValue+" into a Float");
		    exception.printStackTrace();
		    value = new Float(0.0f);
		}
	    } else if (className.equals("int")) {
		// create an Integer object
		try {
		    value = Integer.valueOf(enteredValue);
		} catch (NumberFormatException exception) {
		    System.out.println("Could not convert param value "+enteredValue+" into an Integer");
		    exception.printStackTrace();
		    value = new Integer(0);
		}
	    } else if (className.equals("long")) {
		// create a Long object
		try {
		    value = Long.valueOf(enteredValue);
		} catch (NumberFormatException exception) {
		    System.out.println("Could not convert param value "+enteredValue+" into a Long");
		    exception.printStackTrace();
		    value = new Long(0);
		}
	    } else if (className.equals("java.lang.String")) {
		value = enteredValue;
	    } else {
		System.out.println("WARNING >>> Could not recognize parameter type "+className+" for paramter field "+paramFields[i].name);
	    }
	    try {
		// now put the created object into the param object, in 
		// the appropriate field
		Field thisParamField = paramClass.getDeclaredField(fieldName);
		thisParamField.set(paramObject,value);
	    } catch (Exception exception) {
		System.out.println("Could not set value "+value+" for param with name "+fieldName);
		exception.printStackTrace();
	    }
	}
	// param object now has all the required fields, send it over!
	componentCommunicator.setComponentParams(id,paramPortClassName,
						 parameterPortName,
						 parameterMethodName,instanceId,paramObject);
    }
	
    // called when the user hits execute on the component view
    public void execute() {
	// tell the component to start up
      new Thread() {
	public void run() {
	  componentCommunicator.startComponent(id,paramPortClassName,
					       parameterPortName,instanceId);
	}
      }.start();
    }

    // called internally to make sure a component does not execute with new data
    // while params are being set
    public void preventFurtherExecution() {
	// tell the component ot stop
	componentCommunicator.stopComponent(id,paramPortClassName,
					    parameterPortName,instanceId);
    }
	
    // called when the user kills a component view on the GUI
    public void killComponent() {
	// remove all real and dummy connections from and to here
	Enumeration inputPortList = inputPorts.elements();
	while(inputPortList.hasMoreElements()) {
	    // remove all connections associated with this port
	    PortModel currentPort = (PortModel)inputPortList.nextElement();
	    currentPort.removeAllConnections();
	}
	Enumeration outputPortList = outputPorts.elements();
	while(outputPortList.hasMoreElements()) {
	    // remove all connections associated with this port
	    PortModel currentPort = (PortModel)outputPortList.nextElement();
	    currentPort.removeAllConnections();
	}
	// if this is a real component, tell the component to kill itself
	// do this in a separate thread since it right now kill does not return
	if (this.isReal()) {
		    componentCommunicator.killComponent(id,paramPortClassName,
							parameterPortName,
							instanceId);
		    workingAreaController.componentKilled(this);
		}
	}

    public void creationEventOccurred(String instanceId) {
	if (instanceId.equals(this.instanceId)) {
	    // describing the newly created instance
	    // update dummy connections to and from this component
	    // to convert them to real connections if possible
	    System.out.println("Updating dummy input ports");
	    Enumeration inputPortList = inputPorts.elements();
	    while(inputPortList.hasMoreElements()) {
		PortModel currentPort = (PortModel) inputPortList.nextElement();
		currentPort.setOwnerID(id);
		currentPort.updateDummyConnections();
	    }
	    System.out.println("Updating dummy output ports");
	    Enumeration outputPortList = outputPorts.elements();
	    while(outputPortList.hasMoreElements()) {
		PortModel currentPort = (PortModel) outputPortList.nextElement();
		currentPort.setOwnerID(id);
		currentPort.updateDummyConnections();
	    }
	    // indicate to the component view that the creation is complete
	    componentView.creationCompleted();
	}
    }

    /**
     * Set the machine name the for the current installation
     */
    public void setMachineName(String machineName) {
	this.machineName = machineName;
    }

    /**
     * Get the machine name for the current installation
     */
    public String getMachineName() {
	return machineName;
    }

    /**
     * Set the creation mechanism the for the current installation
     */
    public void setCreationMechanism(String creationMechanism) {
	this.creationMechanism = creationMechanism;
    }

    /**
     * Get the creation mechanism for the current installation
     */
    public String getCreationMechanism() {
	return creationMechanism;
    }

    /**
     * Set the component view associated with this model
     */
    public void setComponentView(ComponentView componentView) {
	this.componentView = componentView;
    }

    /**
     * Get the name of this component
     */
    public String getName() {
	return componentName;
    }

    /**
     * Get the corresponding component view
     */
    public ComponentView getComponentView() {
	return componentView;
    }


    /**
     * Check if this component refers to a real instance or not
     */
    public boolean isReal() {
	// this is a real instance if the component id is non-null
	return (id!=null);
    }

    /**
     * Get the list of input ports associated with this component
     */
    public PortModel [] getInputPorts() {
	PortModel [] inputPortArray = new PortModel[inputPorts.size()];
	int i=0;
	Enumeration inputPortList = inputPorts.elements();
	while(inputPortList.hasMoreElements()) {
	    inputPortArray[i++] = (PortModel)inputPortList.nextElement();
	}
	return inputPortArray;
    }

    /**
     * Get the list of output ports associated with this component
     */
    public PortModel [] getOutputPorts() {
	PortModel [] outputPortArray = new PortModel[outputPorts.size()];
	int i=0;
	Enumeration outputPortList = outputPorts.elements();
	while(outputPortList.hasMoreElements()) {
	    outputPortArray[i++] = (PortModel)outputPortList.nextElement();
	}
	return outputPortArray;
    }

    /**
     * Get the info object describing this component's installations
     */
    public Info [] getInstallationInfo() {
	// parse the info object to get the installation 
	// information and return that
	return info.getInfo("component-info")[0].getInfo("installation");
    }

    /**
     * Get the info object describing this component
     */
    public Info getInfo() {
	return info;
    }

    /**
     * Get the component id for this component
     */
    public ComponentID getComponentID() {
	return id;
    }

    /**
     * Get the event id for this component
     */
    public String getEventID() {
	return instanceId;
    }
}
