package python;

import python.ComponentCommunicator;
import python.ComponentModel;
import java.io.IOException;
import java.lang.reflect.Field;
import util.Info;
import ccacore.ComponentID;

public class SetParms {
    

    public static void displayDefaultParams (ComponentModel component) {
	// get a list of default parameters for the instance specified
	// display default parameters and allow editing

	// get the fields and defaults from the info
	Info instanceInfo = component.getInfo();
	Info [] paramList = instanceInfo.getInfo("instance-info")[0].getInfo("parameter-block")[0].getInfo("parameters");


	for (int i=0; i<paramList.length; i++) {
	    System.out.println (" NAME="+paramList[i].getValue("name")[0]+"    VALUE="+paramList[i].getValue("default-value")[0]);
	}


    }



    public static void setParameters (ComponentModel component,
                                      String paramName,
                                      String enteredValue,
                                      ComponentCommunicator componentCommunicator) {
	// get the name of the instance and create an object of
	// its parameter class
	// populate fields with edited values
	// invoke method to set parameter on the instance
	// display main screen

	ComponentID id = component.getComponentID();
	// get the name of the parameter class
	String paramClassName = component.getParameterClassName();
	// get the name of the parameter port class
	String paramPortClassName = component.getParameterPortClassName();
	// get the name of the parameter port
	String paramPortName = component.getParameterPortName();
	// get the name of the method to invoke
	String paramMethodName = component.getInfo().getInfo("instance-info")[0].getInfo("parameter-block")[0].getValue("method-name")[0];
	// load the param class
	try {
	    Class paramClass = Class.forName(paramClassName);
	    // create an instance of this class
	    Object paramObject = paramClass.newInstance();
	    // get its fields
	    Field [] classFields = paramClass.getDeclaredFields();
	    // look up the values of the fields on the form and populate this
	    // object

            int i = 0;
            String fieldName = classFields[i].getName();
            while ((i<classFields.length) && (fieldName.compareTo (paramName) != 0))
            {
                i++;
                fieldName = classFields[i].getName();
            }


            if (fieldName.compareTo (paramName) == 0)
            {

		String className = classFields[i].getType().getName();
		Object value = null;
		if (className.equals("boolean")) {
		    // create a Boolean object
		    value = Boolean.valueOf(enteredValue);
		} else if (className.equals("double")) {
		    try {
			// create a Double object
			value = Double.valueOf(enteredValue);
		    } catch (NumberFormatException exception) {
			System.out.println("Could not convert param value "+enteredValue+" into a Double");
			exception.printStackTrace();
			value = new Double(0.0d);
		    }
		} else if (className.equals("float")) {
		    // create a Float object
		    try {
			value = Float.valueOf(enteredValue);
		    } catch (NumberFormatException exception) {
			System.out.println("Could not convert param value "+enteredValue+" into a Float");
			exception.printStackTrace();
			value = new Float(0.0f);
		    }
		} else if (className.equals("int")) {
		    // create an Integer object
		    try {
			value = Integer.valueOf(enteredValue);
		    } catch (NumberFormatException exception) {
			System.out.println("Could not convert param value "+enteredValue+" into an Integer");
			exception.printStackTrace();
			value = new Integer(0);
		    }
		} else if (className.equals("long")) {
		    // create a Long object
		    try {
			value = Long.valueOf(enteredValue);
		    } catch (NumberFormatException exception) {
			System.out.println("Could not convert param value "+enteredValue+" into a Long");
			exception.printStackTrace();
			value = new Long(0);
		    }
		} else if (className.equals("java.lang.String")) {
		    value = enteredValue;
		} else {
		    System.out.println("WARNING >>> Could not recognize parameter type "+className+" for parameter field "+fieldName);
		}
		// now put the created object into the param object, in 
		// the appropriate field
		Field thisParamField = paramClass.getDeclaredField(fieldName);
		thisParamField.set(paramObject,value);
		System.out.println("Set "+fieldName+" to "+enteredValue);
	    System.out.println("Prepared param object, calling set on instance");
	    // param object now has all the required fields, send it over!
	    componentCommunicator.setComponentParams(id,paramPortClassName,
						     paramPortName,
						     paramMethodName,
						     component.getName(),
						     paramObject);
	    }

	} catch (Exception exception) {
	    exception.printStackTrace();
	}
    }

    
}
